@@ -82,8 +82,11 @@
 /* Define to enable Zlib compression / decompression support. */
 #define HT_ZLIB 1
 
+/* Define to enable Digest Authenticaiton support */
+#define HT_MD5 1
+
 /* Define to enable POSIX RegEx support. */
-/* #undef HT_POSIX_REGEX */
+/* #under HT_POSIX_REGEX */
 
 /* Define to enable support for enabling a rules file w/o user interaction. */
 /* #undef HT_AUTOMATIC_RULES */
@@ -338,7 +341,7 @@
 /* #undef HAVE_PWD_H */
 
 /* Define if you have the <curses.h> header file.  */
-/* #undef HAVE_REGEX_H */
+#define HAVE_REGEX_H 1
 
 /* Define if you have the <resource.h> header file.  */
 /* #undef HAVE_RESOURCE_H */
@@ -490,3 +493,8 @@
 /* Define this if sys_nerr must be declared (if it exists). */
 /* #undef NEED_SYS_NERR_DECLARED */
 
+/* Define this if you want WebDAV methods from HTDAV module */
+#define HT_DAV 1
+
+/* Define this if you want enable the "extension methods" */
+/*#undef HT_EXT */
