@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -18,14 +18,14 @@
 #define THOT_EXPORT extern
 #include "amaya.h"
 #include "css.h"
-#include "GraphML.h"
+#include "SVG.h"
 
 static Document     BgDocument;
 static int          RepeatValue;
 static char       DirectoryImage[MAX_LENGTH];
 static char       LastURLImage[MAX_LENGTH];
 static char       ImageName[MAX_LENGTH];
-static char       ImgAlt[NAME_LENGTH];
+static char       ImgAlt[MAX_LENGTH];
 
 #include "AHTURLTools_f.h"
 #include "HTMLactions_f.h"
@@ -45,7 +45,7 @@ static char       ImgAlt[NAME_LENGTH];
 /*----------------------------------------------------------------------
    DeleteMap                                              
   ----------------------------------------------------------------------*/
-ThotBool            DeleteMap (NotifyElement * event)
+ThotBool DeleteMap (NotifyElement * event)
 {
    Element             image;
    ElementType	       elType;
@@ -321,8 +321,8 @@ void CallbackImage (int ref, int typedata, char *data)
 	}
       break;
     case ImageAlt:
-      strncpy (ImgAlt, data, NAME_LENGTH-1);
-      ImgAlt[NAME_LENGTH-1] = EOS;
+      strncpy (ImgAlt, data, MAX_LENGTH - 1);
+      ImgAlt[MAX_LENGTH - 1] = EOS;
       break;
     case ImageDir:
       if (!strcmp (data, ".."))
@@ -375,7 +375,7 @@ void InitImage (void)
    BaseImage = TtaSetCallback (CallbackImage, IMAGE_MAX_REF);
    RepeatValue = 0;
    LastURLImage[0] = EOS;
-   strcpy (ImgFilter, ".gif");
+   strcpy (ImgFilter, "*.gif");
    /* set path on current directory */
    getcwd (DirectoryImage, MAX_LENGTH);
 }
@@ -384,7 +384,7 @@ void InitImage (void)
 /*----------------------------------------------------------------------
    GetAlt gets the Alt value for an Area                            
   ----------------------------------------------------------------------*/
-static void         GetAlt (Document document, View view)
+static void GetAlt (Document document, View view)
 {
   ImgAlt[0] = EOS;
 #ifndef _WINDOWS
@@ -424,7 +424,10 @@ static void CreateAreaMap (Document doc, View view, char *shape)
    ElementType         elType;
    AttributeType       attrType;
    Attribute           attr, attrRef, attrShape, attrRefimg;
-   char               *url;
+   char                *url;
+#ifdef _I18N_
+   char                *tmp;
+#endif
    int                 length, w, h;
    int                 firstchar, lastchar;
    int                 docModified;
@@ -612,7 +615,7 @@ static void CreateAreaMap (Document doc, View view, char *shape)
 	if (shape[0] == 'p')
 	  {
 	    TtcInsertGraph (doc, 1, 'p');
-	    if (TtaGetVolume (child) < 3)
+	    if (TtaGetElementVolume (child) < 3)
 	      {
 		/* the polyline doesn't have enough points */
 		if (newMap)
@@ -634,7 +637,13 @@ static void CreateAreaMap (Document doc, View view, char *shape)
 	attr = TtaNewAttribute (attrType);
 	TtaAttachAttribute (el, attr, doc);
 	GetAlt (doc, view);
+#ifdef _I18N_
+	tmp = TtaConvertByteToMbs (ImgAlt, TtaGetDefaultCharset ());
+	TtaSetAttributeText (attr, tmp, el, doc);
+	TtaFreeMemory (tmp);
+#else /* _I18N_ */
 	TtaSetAttributeText (attr, ImgAlt, el, doc);
+#endif /* _I18N_ */
 	ImgAlt[0] = EOS;
 	/* The link element is a new created one */
 	IsNewAnchor = TRUE;
@@ -844,14 +853,14 @@ void ComputeSRCattribute (Element el, Document doc, Document sourceDocument,
   char               imagename[MAX_LENGTH];
 
   elType = TtaGetElementType (el);
-  if (strcmp (TtaGetSSchemaName (elType.ElSSchema), "GraphML"))
+  if (strcmp (TtaGetSSchemaName (elType.ElSSchema), "SVG"))
     /* it's not a SVG element, it's then a HTML img element, which is
        itself a Thot picture element */
     pict = el;
   else
     /* it's a SVG image. The Thot picture element is one of its children */
     {
-      elType.ElTypeNum = GraphML_EL_PICTURE_UNIT;
+      elType.ElTypeNum = SVG_EL_PICTURE_UNIT;
       pict = TtaSearchTypedElement (elType, SearchInTree, el);
       if (!pict)
 	/* no Thot picture element. Create one */
@@ -911,6 +920,7 @@ void ComputeSRCattribute (Element el, Document doc, Document sourceDocument,
 	  TtaFreeMemory (value);
 	  /* set the element content */
 	  TtaSetTextContent (pict, pathimage, SPACE, doc);
+	  DisplayImage (doc, pict, pathimage, NULL);
 	}
       else
 	{
@@ -936,7 +946,7 @@ void UpdateSRCattribute (NotifyElement *event)
   Document           doc;
   char*            text;
   char*            pathimage;
-  char*            imagename;
+  char*            tmp;
 
    el = event->element;
    doc = event->document;
@@ -954,20 +964,11 @@ void UpdateSRCattribute (NotifyElement *event)
 	TtaDeleteTree (el, doc);
 	return;
      }
-   /* search the SRC attribute */
-   attrType.AttrSSchema = TtaGetDocumentSSchema (doc);
-   attrType.AttrTypeNum = HTML_ATTR_SRC;
    elSRC = TtaGetParent (el);
    if (elSRC != NULL)
       elSRC = el;
-   attr = TtaGetAttribute (elSRC, attrType);
-   if (attr == 0)
-     {
-	attr = TtaNewAttribute (attrType);
-	TtaAttachAttribute (elSRC, attr, doc);
-     }
-   ComputeSRCattribute (elSRC, doc, 0, attr, text);
    /* add the ALT attribute */
+   attrType.AttrSSchema = TtaGetDocumentSSchema (doc);
    attrType.AttrTypeNum = HTML_ATTR_ALT;
    attr = TtaGetAttribute (elSRC, attrType);
    if (attr == 0)
@@ -978,20 +979,35 @@ void UpdateSRCattribute (NotifyElement *event)
    /* copy image name in ALT attribute */
    if (ImgAlt[0] == EOS)
      {
-       imagename = TtaGetMemory (MAX_LENGTH);
+       tmp = TtaGetMemory (MAX_LENGTH);
        pathimage = TtaGetMemory (MAX_LENGTH);
-       strcpy (imagename, " ");
-       TtaExtractName (text, pathimage, &imagename[1]);
-       strcat (imagename, " ");
-       TtaSetAttributeText (attr, imagename, elSRC, doc);
+       strcpy (tmp, " ");
+       TtaExtractName (text, pathimage, &tmp[1]);
+       strcat (tmp, " ");
+       TtaSetAttributeText (attr, tmp, elSRC, doc);
        TtaFreeMemory (pathimage);
-       TtaFreeMemory (imagename);
+       TtaFreeMemory (tmp);
      }
    else
      {
+#ifdef _I18N_
+       tmp = TtaConvertByteToMbs (ImgAlt, TtaGetDefaultCharset ());
+       TtaSetAttributeText (attr, tmp, elSRC, doc);
+       TtaFreeMemory (tmp);
+#else /* _I18N_ */
        TtaSetAttributeText (attr, ImgAlt, elSRC, doc);
+#endif /* _I18N_ */
        ImgAlt[0] = EOS;
      }
+   /* search the SRC attribute */
+   attrType.AttrTypeNum = HTML_ATTR_SRC;
+   attr = TtaGetAttribute (elSRC, attrType);
+   if (attr == 0)
+     {
+	attr = TtaNewAttribute (attrType);
+	TtaAttachAttribute (elSRC, attr, doc);
+     }
+   ComputeSRCattribute (elSRC, doc, 0, attr, text);
 }
 
 
@@ -1007,9 +1023,9 @@ void SvgImageCreated (NotifyElement *event)
   ElementType        elType;
   Element            el, desc, leaf;
   Document           doc;
-  char*            text;
-  char*            pathimage;
-  char*            imagename;
+  char              *text;
+  char              *pathimage;
+  char              *imagename;
 
    el = event->element;
    doc = event->document;
@@ -1021,19 +1037,9 @@ void SvgImageCreated (NotifyElement *event)
 	TtaDeleteTree (el, doc);
 	return;
      }
-   /* search the xlink:href attribute */
    elType = TtaGetElementType (el);
-   attrType.AttrSSchema = elType.ElSSchema;
-   attrType.AttrTypeNum = GraphML_ATTR_xlink_href;
-   attr = TtaGetAttribute (el, attrType);
-   if (attr == 0)
-     {
-	attr = TtaNewAttribute (attrType);
-	TtaAttachAttribute (el, attr, doc);
-     }
-   ComputeSRCattribute (el, doc, 0, attr, text);
    /* set the desc child */
-   elType.ElTypeNum = GraphML_EL_desc;
+   elType.ElTypeNum = SVG_EL_desc;
    desc = TtaSearchTypedElement (elType, SearchInTree, el);
    if (!desc)
      {
@@ -1060,6 +1066,16 @@ void SvgImageCreated (NotifyElement *event)
        TtaSetTextContent (leaf, ImgAlt, SPACE, doc);
        ImgAlt[0] = EOS;
      }
+   /* search the xlink:href attribute */
+   attrType.AttrSSchema = elType.ElSSchema;
+   attrType.AttrTypeNum = SVG_ATTR_xlink_href;
+   attr = TtaGetAttribute (el, attrType);
+   if (attr == 0)
+     {
+	attr = TtaNewAttribute (attrType);
+	TtaAttachAttribute (el, attr, doc);
+     }
+   ComputeSRCattribute (el, doc, 0, attr, text);
 }
 
 
@@ -1131,7 +1147,7 @@ void                SRCattrModified (NotifyAttribute *event)
 /*----------------------------------------------------------------------
   CreateImage
   ----------------------------------------------------------------------*/
-void                CreateImage (Document doc, View view)
+void CreateImage (Document doc, View view)
 {
   Element            sibling;
   ElementType        elType;
@@ -1144,8 +1160,8 @@ void                CreateImage (Document doc, View view)
       /* Get the type of the first selected element */
       elType = TtaGetElementType (sibling);
       name = TtaGetSSchemaName (elType.ElSSchema);
-      if (!strcmp (name, "GraphML"))
-	elType.ElTypeNum = GraphML_EL_image;
+      if (!strcmp (name, "SVG"))
+	elType.ElTypeNum = SVG_EL_image;
       else
 	{
 	  elType.ElSSchema = TtaGetSSchema ("HTML", doc);
@@ -1208,6 +1224,8 @@ ThotBool AddLocalImage (char *fullname, char *name, char *url, Document doc,
 	{
 	  /* It is a new loaded image */
 	  pImage = (LoadedImageDesc *) TtaGetMemory (sizeof (LoadedImageDesc));
+	  /* clear the structure */
+	  memset ((void *) pImage, 0, sizeof (LoadedImageDesc));
 	  pImage->originalName = TtaGetMemory (strlen (url) + 1);
 	  strcpy (pImage->originalName, url);
 	  pImage->localName = TtaGetMemory (strlen (localname) + 1);
