@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -68,10 +68,29 @@ static int              PagePerSheet;
 
 static ThotBool GetIncludedDocuments ();
 
+/*----------------------------------------------------------------------
+  RedisplayDocument redisplays a view of a document
+  ----------------------------------------------------------------------*/
+void  RedisplayDocument(Document doc, View view)
+{
+  Element	       el;
+  int		       position;
+  int		       distance;
+
+  /* get the current position in the document */
+  position = RelativePosition (doc, &distance);
+  TtaSetDisplayMode (doc, NoComputedDisplay);
+  TtaSetDisplayMode (doc, DisplayImmediately);
+  /* show the document at the same position */
+  el = ElementAtPosition (doc, position);
+  TtaShowElement (doc, view, el, distance);
+}
+
+
 /*----------------------------------------------------------------------
   RegisterSubDoc adds a new entry in SubDoc table.
   ----------------------------------------------------------------------*/
-static void         RegisterSubDoc (Element el, char *url)
+static void RegisterSubDoc (Element el, char *url)
 {
   struct _SubDoc  *entry, *last;
 
@@ -100,7 +119,7 @@ static void         RegisterSubDoc (Element el, char *url)
   within the SubDoc table.
   Return the DIV element that correspond to the sub-document or NULL.
   ----------------------------------------------------------------------*/
-static Element      SearchSubDoc (char *url)
+static Element SearchSubDoc (char *url)
 {
   Element          el;
   struct _SubDoc  *entry;
@@ -127,7 +146,7 @@ static Element      SearchSubDoc (char *url)
 /*----------------------------------------------------------------------
   FreeSubDocTable frees all entries in SubDoc table.
   ----------------------------------------------------------------------*/
-static void         FreeSubDocTable ()
+static void FreeSubDocTable ()
 {
   struct _SubDoc  *entry, *last;
 
@@ -151,7 +170,7 @@ static void         FreeSubDocTable ()
   InternalLink is a Thot reference attribute that links a source and a
   target anchor and that allows P schemas to display and print cross-references
   ----------------------------------------------------------------------*/
-void             SetInternalLinks (Document document)
+void SetInternalLinks (Document document)
 {
   Element	        el, div, link, target, sibling;
   ElementType		elType, linkType;
@@ -346,7 +365,7 @@ void             SetInternalLinks (Document document)
   CheckPrintingDocument reinitialize printing parameters as soon as
   the printing document changes.
   ----------------------------------------------------------------------*/
-static void         CheckPrintingDocument (Document document)
+static void CheckPrintingDocument (Document document)
 {
   char         docName[MAX_LENGTH];
   char        *ptr; 
@@ -371,7 +390,7 @@ static void         CheckPrintingDocument (Document document)
       if (PSfile[lg - 1] == DIR_SEP)
 	PSfile[--lg] = EOS;
       strcpy (docName, TtaGetDocumentName (document));
-      ExtractSuffix (docName, suffix);
+      TtaExtractSuffix (docName, suffix);
       sprintf (&PSfile[lg], "%c%s.ps", DIR_SEP, docName);
       TtaSetPsFile (PSfile);
     }
@@ -380,7 +399,7 @@ static void         CheckPrintingDocument (Document document)
 /*----------------------------------------------------------------------
    PrintDocument prints the document using predefined parameters.
    ----------------------------------------------------------------------*/  
-static void         PrintDocument (Document doc, View view)
+static void PrintDocument (Document doc, View view)
 {
   AttributeType      attrType;
   ElementType        elType;
@@ -418,7 +437,7 @@ static void         PrintDocument (Document doc, View view)
 	}
     }
   else if (DocumentTypes[doc] == docSVG)
-    TtaSetPrintSchema ("GraphMLP");
+    TtaSetPrintSchema ("SVGP");
   else if (DocumentTypes[doc] == docMath)
     TtaSetPrintSchema ("MathMLP");
   else if (DocumentTypes[doc] == docAnnot)
@@ -511,7 +530,7 @@ static void         PrintDocument (Document doc, View view)
 /*----------------------------------------------------------------------
    PrintAs prints the document using predefined parameters.
    ----------------------------------------------------------------------*/  
-void                PrintAs (Document doc, View view)
+void PrintAs (Document doc, View view)
 {
 #ifdef _WINDOWS
   DocPrint = doc;
@@ -522,7 +541,7 @@ void                PrintAs (Document doc, View view)
 }
 
 /*----------------------------------------------------------------------
-   CallbackImage manage returns of Picture form.                   
+   CallbackImage handle return of Print form.                   
   ----------------------------------------------------------------------*/
 void CallbackPrint (int ref, int typedata, char *data)
 {
@@ -650,7 +669,7 @@ void CallbackPrint (int ref, int typedata, char *data)
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-void                InitPrint (void)
+void InitPrint (void)
 {
   char* ptr;
 
@@ -685,13 +704,13 @@ void                InitPrint (void)
 /*----------------------------------------------------------------------
   SetupAndPrint sets printing parameters and starts the printing process
   ----------------------------------------------------------------------*/
-void                SetupAndPrint (Document doc, View view)
+void SetupAndPrint (Document doc, View view)
 {
 #ifndef _WINDOWS
   char           bufMenu[MAX_LENGTH];
-  int              i;
-#endif /* !_WINDOWS */
-  ThotBool           textFile;
+  int            i;
+#endif /* _WINDOWS */
+  ThotBool       textFile;
 
   textFile = (DocumentTypes[doc] == docText || DocumentTypes[doc] == docCSS);
   /* Print form */
@@ -797,15 +816,6 @@ void                SetupAndPrint (Document doc, View view)
 #endif /* _WINDOWS */
 }
 
-/*----------------------------------------------------------------------
-  SectionNumbering
-  Execute the "Section Numbering" command
-  ----------------------------------------------------------------------*/
-void                SectionNumbering (Document doc, View view)
-{
-   ChangeAttrOnRoot (doc, HTML_ATTR_SectionNumbering);
-}
-
 /*----------------------------------------------------------------------
   UpdateURLsInSubtree
   Update NAMEs and URLs in subtree of el element, to take into account
@@ -815,7 +825,7 @@ void                SectionNumbering (Document doc, View view)
   Transform the HREF and SRC attribute to make them independent from their
   former base.
   ----------------------------------------------------------------------*/
-static void         UpdateURLsInSubtree (NotifyElement *event, Element el)
+static void UpdateURLsInSubtree (NotifyElement *event, Element el)
 {
   Element             nextEl, child;
   ElementType         elType;
@@ -1075,21 +1085,22 @@ void   GetIncludedDocuments_callback (int newdoc, int status,
   that link by the contents of the target document.
   Return TRUE if one inclusion is launched.
   ----------------------------------------------------------------------*/
-static ThotBool  GetIncludedDocuments (Element el, Element link,
-				       Document document, IncludeCtxt *prev)
+static ThotBool GetIncludedDocuments (Element el, Element link,
+				      Document doc, IncludeCtxt *prev)
 {
   ElementType           elType;
   Attribute		attr;
   AttributeType	        attrType;
   Document		newdoc;
   IncludeCtxt          *ctx = NULL;
+  CHARSET               charset;
   char		       *text, *ptr, *url = NULL;
   int			length;
   ThotBool              found = FALSE;
 
   /* look for anchors with the attribute rel within the element  el */
   attr = NULL;
-  attrType.AttrSSchema = TtaGetSSchema ("HTML", document);
+  attrType.AttrSSchema = TtaGetSSchema ("HTML", doc);
   elType.ElSSchema = attrType.AttrSSchema;
   elType.ElTypeNum = HTML_EL_Anchor;
 
@@ -1144,7 +1155,7 @@ static ThotBool  GetIncludedDocuments (Element el, Element link,
 	      IncludedDocument = TtaNewDocument ("HTML", "tmp");
 	      if (IncludedDocument != 0)
 		{
-		  TtaSetStatus (document, 1, TtaGetMessage (AMAYA, AM_FETCHING), url);
+		  TtaSetStatus (doc, 1, TtaGetMessage (AMAYA, AM_FETCHING), url);
 		  ctx = TtaGetMemory (sizeof (IncludeCtxt));
 		  ctx->div =  el;
 		  ctx->link = link;
@@ -1152,11 +1163,16 @@ static ThotBool  GetIncludedDocuments (Element el, Element link,
 		  ctx->name = ptr;
 		  ctx->ctxt = prev; /* previous context */
 		  /* Get the reference of the calling document */
-		  SetStopButton (document);
-		  newdoc = GetHTMLDocument (url, NULL, IncludedDocument,
-					    document, CE_MAKEBOOK, FALSE, 
-					    (void *) GetIncludedDocuments_callback,
-					    (void *) ctx);
+		  SetStopButton (doc);
+#ifdef _I18N_
+		  charset = UTF_8;
+#else /* _I18N_ */
+		  charset = TtaGetDocumentCharset (doc);
+#endif /* _I18N_ */
+		  newdoc = GetAmayaDoc (url, NULL, IncludedDocument,
+					doc, CE_MAKEBOOK, FALSE, 
+					(void *) GetIncludedDocuments_callback,
+					(void *) ctx, charset);
 		  found = TRUE;
 		}
 	    }
@@ -1173,21 +1189,21 @@ static ThotBool  GetIncludedDocuments (Element el, Element link,
   Replace all links in a document which have an attribute REL="chapter"
   or REL="subdocument" by the corresponding target document.
   ----------------------------------------------------------------------*/
-void                MakeBook (Document document, View view)
+void MakeBook (Document doc, View view)
 {
   Element	    root, body;
   ElementType	    elType;
 
   /* stops all current transfers on this document */
-  StopTransfer (document, 1);
+  StopTransfer (doc, 1);
   /* simulate a transfert in the main document */
-  DocBook = document;
+  DocBook = doc;
   IncludedDocument = 0;
-  root = TtaGetMainRoot (document);
+  root = TtaGetMainRoot (doc);
   elType = TtaGetElementType (root);
   elType.ElTypeNum = HTML_EL_BODY;
   body = TtaSearchTypedElement (elType, SearchForward, root);
 
   if (body)
-    GetIncludedDocuments (body, body, document, NULL);
+    GetIncludedDocuments (body, body, doc, NULL);
 }
