@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -42,11 +42,13 @@
 static char        *buffer;    /* temporary buffer used to build the query
 				  string */
 static int          lgbuffer;  /* size of the temporary buffer */
+static int          last_buffer_char; /* gives the index of the last char + 1 added to
+					 the buffer (only used in AddBufferWithEos) */
 static int          documentStatus;
 
 #ifdef _WINDOWS 
 extern HWND         FrMainRef [12];
-extern int          currentFrame;
+extern int          ActiveFrame;
 Document            opDoc;
 Element             opOption [200];
 #endif /* _WINDOWS */
@@ -102,6 +104,42 @@ static void AddToBuffer (char *orig)
       strcat (buffer, orig);
 }
 
+/*----------------------------------------------------------------------
+  AddToBufferWithEos
+  reallocates memory and concatenates a string into buffer. Skips the
+  last EOS char.
+  ----------------------------------------------------------------------*/
+static void AddToBufferWithEOS (char *orig)
+{
+   void               *status;
+   int                 lg;
+   int                 i;
+
+   lg = strlen (orig) + 2;
+   if ((int)(&buffer[last_buffer_char] - buffer) + lg > lgbuffer)
+     {
+	/* it is necessary to extend the buffer */
+	if (lg < PARAM_INCREMENT)
+	   i = PARAM_INCREMENT;
+	else i = lg;
+
+	status = TtaRealloc (buffer, sizeof (char) * (lgbuffer + i));
+
+	if (status != NULL)
+	  {
+	     buffer = status;
+	     lgbuffer += i;
+	     strcpy (&buffer[last_buffer_char], orig);
+	     last_buffer_char = last_buffer_char + lg - 1;
+	  }
+     }
+   else
+     {
+       strcpy (&buffer[last_buffer_char], orig);
+       last_buffer_char = last_buffer_char + lg - 1;
+     }
+}
+
 
 /*----------------------------------------------------------------------
   AddElement
@@ -357,7 +395,7 @@ static void ResetOption (Element option, ThotBool multipleSelects,
   ResetOptionMenu
   
   ----------------------------------------------------------------------*/
-static void        ResetOptionMenu (Element menu, Document doc)
+static void ResetOptionMenu (Element menu, Document doc)
 {
    ElementType	       elType;
    Element             option, firstOption, child;
@@ -689,10 +727,15 @@ static void ParseForm (Document doc, Element ancestor, Element el, int mode)
   ----------------------------------------------------------------------*/
 static void DoSubmit (Document doc, int method, char *action)
 {
+  CHARSET             charset;
   int                 buffer_size;
   int                 i;
   char               *urlName;
 
+  /* clear the selection */
+  if (TtaGetSelectedDocument () == doc)
+    TtaUnselect (doc);
+
   /* remove any trailing & */
   if (buffer)
     buffer_size = strlen (buffer);
@@ -707,6 +750,11 @@ static void DoSubmit (Document doc, int method, char *action)
       buffer_size--;
     }
 
+#ifdef _I18N_
+  charset = UTF_8;
+#else /* _I18N_ */
+  charset = TtaGetDocumentCharset (doc);
+#endif /* _I18N_ */
   switch (method)
     {
     case -9999:	/* index attribute, not yet supported by Amaya */
@@ -726,14 +774,14 @@ static void DoSubmit (Document doc, int method, char *action)
       if (urlName != NULL)
 	{
 	  strcpy (urlName, action);
-	  GetHTMLDocument (urlName, buffer, doc, doc,
-			   CE_FORM_GET, TRUE, NULL, NULL);
+	  GetAmayaDoc (urlName, buffer, doc, doc, CE_FORM_GET, TRUE,
+		       NULL, NULL, charset);
 	  TtaFreeMemory (urlName);
 	}
       break;
     case HTML_ATTR_METHOD_VAL_Post_:
-      GetHTMLDocument (action, buffer, doc, doc,
-		       CE_FORM_POST, TRUE, NULL, NULL);
+      GetAmayaDoc (action, buffer, doc, doc, CE_FORM_POST, TRUE,
+		   NULL, NULL, charset);
       break;
     default:
       break;
@@ -1069,7 +1117,7 @@ void SelectCheckbox (Document doc, Element el)
    SelectOneRadio
    selects one Radio input				
   ----------------------------------------------------------------------*/
-void                SelectOneRadio (Document doc, Element el)
+void SelectOneRadio (Document doc, Element el)
 {
   ElementType         elType;
   Element             elForm;
@@ -1221,7 +1269,7 @@ void SelectOneOption (Document doc, Element el)
 #ifdef _WINDOWS
   int                 nbOldEntries = 20;
 #endif /* _WINDOWS */
-#define MAX_OPTIONS 100
+#define MAX_OPTIONS MAX_SUBMENUS
 #define MAX_SUBOPTIONS 20
 #define MAX_LABEL_LENGTH 50
   ElementType         elType, childType;
@@ -1233,8 +1281,9 @@ void SelectOneOption (Document doc, Element el)
   AttributeType       attrType;
   Attribute	      attr;
   SSchema	      htmlSch;
-  char              text[MAX_LABEL_LENGTH];
-  char              buffmenu[MAX_LENGTH];
+  char                text[MAX_LABEL_LENGTH + 1];
+  char                buffmenu[MAX_LENGTH];
+  char               *tmp;
   Language            lang;
   int                 length, nbitems, lgmenu, i, nbsubmenus, nbsubitems;
   int                 modified;
@@ -1260,7 +1309,6 @@ void SelectOneOption (Document doc, Element el)
    if (elType.ElTypeNum == HTML_EL_Option && elType.ElSSchema == htmlSch)
      {
        /* create the option menu */
-       lgmenu = 0;
        nbitems = 0;
        nbsubmenus = 0;
        elType.ElTypeNum = HTML_EL_Option_Menu;
@@ -1278,7 +1326,14 @@ void SelectOneOption (Document doc, Element el)
 	   
 	   attrType.AttrTypeNum = HTML_ATTR_Selected;
 	   el = TtaGetFirstChild (menuEl);
-	   while (nbitems < MAX_OPTIONS && el)
+
+	   /* use the global allocation buffer to store the entries */
+	   buffer = TtaGetMemory (PARAM_INCREMENT);
+	   lgbuffer = PARAM_INCREMENT;
+	   buffer[0] = EOS;
+	   last_buffer_char = 0;
+
+	   while (el && nbitems < MAX_OPTIONS)
 	     {
 	       elType = TtaGetElementType (el);
 	       if (elType.ElTypeNum == HTML_EL_OptGroup &&
@@ -1310,52 +1365,67 @@ void SelectOneOption (Document doc, Element el)
 		   attr = TtaGetAttribute (el, attrType);
 		   length = MAX_LABEL_LENGTH;
 		   if (attr)
-		     TtaGiveTextAttributeValue (attr, text, &length);
+		     TtaGiveTextAttributeValue (attr, text + 1, &length);
 		   else if (elType.ElTypeNum == HTML_EL_Option)
 		     /* there is no label attribute, but it's an Option
 			Take its content as the item label */
 		     {
 		       elText = TtaGetFirstChild (el);
 		       if (elText)
-			 TtaGiveTextContent (elText, text, &length, &lang);
+			 TtaGiveTextContent (elText, text + 1, &length, &lang);
 		       else
 			 length = 0;
 		     }
 		   else
 		     length = 0;
 		   /* count the EOS character */
-		   text[length] = EOS;
-		   length++;
+		   text[length + 1] = EOS;
+		   /* add an item */
 		   /* we have to add the 'B', 'T' or 'M' character */
-		   length++;
-		   if (lgmenu + length < MAX_LENGTH)
-		     /* add an item */
-		     {
-		       if (elType.ElTypeNum == HTML_EL_OptGroup)
-			 sprintf (&buffmenu[lgmenu], "M%s", text);
-		       else if (multipleOptions)
-			 sprintf (&buffmenu[lgmenu], "T%s", text);
-		       else
-			 sprintf (&buffmenu[lgmenu], "B%s", text);
-		       nbitems++;
-		     }
-		   lgmenu += length;
+		   if (elType.ElTypeNum == HTML_EL_OptGroup)
+		     text[0] = 'M';
+		   else if (multipleOptions)
+		     text[0] = 'T';
+		   else
+		     text[0] = 'B';
+#ifdef _I18N_
+		   /* convert the UTF-8 string */
+		   tmp = TtaConvertMbsToByte (text, TtaGetDefaultCharset ());
+		   AddToBufferWithEOS (tmp);
+		   TtaFreeMemory (tmp);
+#else /* _I18N_ */
+		   AddToBufferWithEOS (text);
+#endif /* _I18N_ */
+		   nbitems++;
 		 }
 	       TtaNextSibling (&el);
 	     }
-	   if (nbitems > 0)
+
+	   if (nbitems == 0)
+	     TtaFreeMemory (buffer);
+	   else
 	     {
 	       /* create the main menu */
-	       TtaNewPopup (BaseDialog + OptionMenu, TtaGetViewFrame (doc, 1),
-			    NULL, nbitems, buffmenu, NULL, 'L');
+#if defined (_WINDOWS) || defined (_GTK)
+	       if (nbsubmenus == 0)
+		 TtaNewScrollPopup (BaseDialog + OptionMenu, TtaGetViewFrame (doc, 1),
+				    NULL, nbitems, buffer, NULL, multipleOptions, 'L');
+	       else
+#endif /* WINDOWS || _GTK */
+		 TtaNewPopup (BaseDialog + OptionMenu, TtaGetViewFrame (doc, 1),
+			      NULL, nbitems, buffer, NULL, 'L');
+
+	       TtaFreeMemory (buffer);
+
 	       if (multipleOptions)
 		 for (i = 0; i < nbitems; i++)
 		   if (selected[i])
 #ifdef _WINDOWS
-		     WIN_TtaSetToggleMenu (BaseDialog + OptionMenu, i, TRUE, FrMainRef [currentFrame]);
-#else  /* !_WINDOWS */
+		     WIN_TtaSetToggleMenu (BaseDialog + OptionMenu, i, TRUE, FrMainRef [ActiveFrame]);
+#else  /* _WINDOWS */
 	             TtaSetToggleMenu (BaseDialog + OptionMenu, i, TRUE);
 #endif /* _WINDOWS */
+		     
 	       if (nbsubmenus > 0)
 		 {
 		   /* There ia at least 1 OPTGROUP. Create submenus corresponding to OPTGROUPs */
@@ -1408,16 +1478,26 @@ void SelectOneOption (Document doc, Element el)
 				   /* count the EOS character */
 				   text[length] = EOS;
 				   length++;
+#ifdef _I18N_
+				   /* convert the UTF-8 string */
+				   tmp = TtaConvertMbsToByte (text, TtaGetDefaultCharset ());
+				   length = strlen (tmp) + 1;
+#else /* _I18N_ */
+				   tmp = text;
+#endif /* _I18N_ */
 				   /* we have to add the 'B'or 'T' character */
 				   length++;
 				   if (lgmenu + length < MAX_LENGTH)
 				     { /* append that item to the buffer */
 				       if (multipleOptions)
-					 sprintf (&buffmenu[lgmenu], "T%s", text);
+					 sprintf (&buffmenu[lgmenu], "T%s", tmp);
 				       else
-					 sprintf (&buffmenu[lgmenu], "B%s", text);
+					 sprintf (&buffmenu[lgmenu], "B%s", tmp);
 				       nbsubitems++;
 				     } 
+#ifdef _I18N_
+				   TtaFreeMemory (tmp);
+#endif /* _I18N_ */
 				   lgmenu += length;
 				 } 
 			       /* next child of OPTGROUP */
@@ -1434,7 +1514,7 @@ void SelectOneOption (Document doc, Element el)
 			     for (i = 0; i < nbsubitems; i++)
 			       if (subSelected[nbsubmenus][i])
 #ifdef _WINDOWS
-				 WIN_TtaSetToggleMenu (BaseDialog + OptionMenu + (nbsubmenus * nbOldEntries) + 1, i, TRUE, FrMainRef [currentFrame]);
+				 WIN_TtaSetToggleMenu (BaseDialog + OptionMenu + (nbsubmenus * nbOldEntries) + 1, i, TRUE, FrMainRef [ActiveFrame]);
 #else  /* !_WINDOWS */
 			         TtaSetToggleMenu (BaseDialog+OptionMenu+nbsubmenus+1, i, TRUE);
 #endif /* _WINDOWS */
@@ -1453,7 +1533,13 @@ void SelectOneOption (Document doc, Element el)
 #endif /* !_WINDOWS */
 	       TtaShowDialogue (BaseDialog + OptionMenu, FALSE);
 	       /* wait for an answer from the user */
-	       TtaWaitShowDialogue ();
+	       if (nbsubmenus == 0)
+		 TtaWaitShowProcDialogue ();
+	       else
+		 TtaWaitShowDialogue ();
+	       /* destroy the dialogue */
+	       TtaDestroyDialogue (BaseDialog + OptionMenu);
+
 	       if (ReturnOption >= 0 && ReturnOptionMenu >= 0)
 		 {
 		   /* make the returned option selected */
@@ -1495,6 +1581,8 @@ void SelectOneOption (Document doc, Element el)
 		       /* switch Amaya buttons and menus */
 		       DocStatusUpdate (doc, modified);
 		     }
+		   /* clear the selection (which may happen from clicking on the dialogue) */
+		   TtaUnselect (doc);
 		 }
 	     }
 	 }
