@@ -2,13 +2,9 @@
 #include "HTML.h"
 static ElemMapping    XHTMLElemMappingTable[] =
 {
-   /* This table MUST be in alphabetical order */
+   /* This table MUST be in alphabetical order and in lower case */
  
-   {"XMLPI", SPACE, HTML_EL_XMLPI, L_OtherValue, FALSE},
-   {"XMLPI_line", SPACE, HTML_EL_PI_line, L_OtherValue, FALSE},
-   {"XMLcomment", SPACE, HTML_EL_Comment_, L_OtherValue, FALSE},
-   {"XMLcomment_line", SPACE, HTML_EL_Comment_line, L_OtherValue, FALSE},
-   {"a", SPACE, HTML_EL_Anchor, L_Basic, TRUE},
+   {"a", SPACE, HTML_EL_Anchor, L_BasicValue, TRUE},
    {"abbr", SPACE, HTML_EL_ABBR, L_BasicValue, TRUE},
    {"acronym", SPACE, HTML_EL_ACRONYM, L_BasicValue, TRUE},
    {"address", SPACE, HTML_EL_Address, L_BasicValue, FALSE},
@@ -25,6 +21,8 @@ static ElemMapping    XHTMLElemMappingTable[] =
    {"button", SPACE, HTML_EL_BUTTON_, L_StrictValue, TRUE},
    {"c", SPACE, HTML_EL_TEXT_UNIT, L_OtherValue, TRUE}, /* used by the trans.c */
    {"caption", SPACE, HTML_EL_CAPTION, L_BasicValue, FALSE},
+   {"cdata", SPACE, HTML_EL_CDATA, L_OtherValue, FALSE},
+   {"cdata_line", SPACE, HTML_EL_CDATA_line, L_OtherValue, FALSE},
    {"center", SPACE, HTML_EL_Center, L_TransitionalValue, FALSE},
    {"cite", SPACE, HTML_EL_Cite, L_BasicValue, TRUE},
    {"code", SPACE, HTML_EL_Code, L_BasicValue, TRUE},
@@ -36,8 +34,11 @@ static ElemMapping    XHTMLElemMappingTable[] =
    {"dir", SPACE, HTML_EL_Directory, L_StrictValue, FALSE},
    {"div", SPACE, HTML_EL_Division, L_BasicValue, FALSE},
    {"dl", SPACE, HTML_EL_Definition_List, L_BasicValue, FALSE},
+   {"doctype", SPACE, HTML_EL_DOCTYPE, L_OtherValue, FALSE},
+   {"doctype_line", SPACE, HTML_EL_DOCTYPE_line, L_OtherValue, FALSE},
    {"dt", SPACE, HTML_EL_Term, L_BasicValue, FALSE},
    {"em", SPACE, HTML_EL_Emphasis, L_BasicValue, TRUE},
+   {"embed",  'E', HTML_EL_Embed_, L_TransitionalValue, TRUE},
    {"fieldset", SPACE, HTML_EL_FIELDSET, L_StrictValue, FALSE},
    {"font", SPACE, HTML_EL_Font_, L_TransitionalValue, TRUE},
    {"form", SPACE, HTML_EL_Form, L_BasicValue, FALSE},
@@ -60,12 +61,12 @@ static ElemMapping    XHTMLElemMappingTable[] =
    {"input", 'E', HTML_EL_Text_Input, L_BasicValue, TRUE},
    {"input", 'E', HTML_EL_Password_Input, L_BasicValue, TRUE},
    {"input", 'E', HTML_EL_File_Input, L_BasicValue, TRUE},
-   {"input", 'E', HTML_EL_Checkbox_Input, L_BasicValue, FALSE},
-   {"input", 'E', HTML_EL_Radio_Input, L_BasicValue, FALSE},
-   {"input", 'E', HTML_EL_Submit_Input, L_BasicValue, FALSE},
-   {"input", 'E', HTML_EL_Reset_Input, L_BasicValue, FALSE},
+   {"input", 'E', HTML_EL_Checkbox_Input, L_BasicValue, TRUE},
+   {"input", 'E', HTML_EL_Radio_Input, L_BasicValue, TRUE},
+   {"input", 'E', HTML_EL_Submit_Input, L_BasicValue, TRUE},
+   {"input", 'E', HTML_EL_Reset_Input, L_BasicValue, TRUE},
    {"input", 'E', HTML_EL_Button_Input, L_BasicValue, TRUE},
-   {"input", 'E', HTML_EL_Hidden_Input, L_BasicValue, FALSE},
+   {"input", 'E', HTML_EL_Hidden_Input, L_BasicValue, TRUE},
    {"ins", SPACE, HTML_EL_INS, L_StrictValue, TRUE},
    {"isindex", 'E', HTML_EL_ISINDEX, L_TransitionalValue, FALSE},
    {"kbd", SPACE, HTML_EL_Keyboard, L_BasicValue, TRUE},
@@ -80,7 +81,7 @@ static ElemMapping    XHTMLElemMappingTable[] =
    {"meta", 'E', HTML_EL_META, L_BasicValue, FALSE},
    {"noframes", SPACE, HTML_EL_NOFRAMES, L_TransitionalValue, FALSE},
    {"noscript", SPACE, HTML_EL_NOSCRIPT, L_StrictValue, FALSE},
-   {"object", SPACE, HTML_EL_Object, L_BasicValue, FALSE},
+   {"object", SPACE, HTML_EL_Object, L_BasicValue, TRUE},
    {"ol", SPACE, HTML_EL_Numbered_List, L_BasicValue, FALSE},
    {"optgroup", SPACE, HTML_EL_OptGroup, L_StrictValue, TRUE},
    {"option", SPACE, HTML_EL_Option, L_BasicValue, TRUE},
@@ -90,16 +91,16 @@ static ElemMapping    XHTMLElemMappingTable[] =
    {"plaintext", SPACE, HTML_EL_Preformatted, L_TransitionalValue, FALSE},/* converted to PRE */
    {"pre", SPACE, HTML_EL_Preformatted, L_BasicValue, FALSE},
    {"q", SPACE, HTML_EL_Quotation, L_BasicValue, TRUE},
-   {"rb", SPACE, HTML_EL_rb, L_RubyValue, TRUE},
+   {"rb", SPACE, HTML_EL_rb, L_RubyValue, FALSE},
    {"rbc", SPACE, HTML_EL_rbc, L_RubyValue, FALSE},
    {"rp", SPACE, HTML_EL_rp, L_RubyValue, FALSE},
-   {"rt", SPACE, HTML_EL_rt, L_RubyValue, TRUE},
+   {"rt", SPACE, HTML_EL_rt, L_RubyValue, FALSE},
    {"rtc", SPACE, HTML_EL_rtc1, L_RubyValue, FALSE},
    {"rtc", SPACE, HTML_EL_rtc2, L_RubyValue, FALSE},
    {"ruby", SPACE, HTML_EL_simple_ruby, L_RubyValue, TRUE},
    {"s", SPACE, HTML_EL_Struck_text, L_StrictValue, TRUE},
    {"samp", SPACE, HTML_EL_Sample, L_BasicValue, TRUE},
-   {"script", SPACE, HTML_EL_SCRIPT, L_StrictValue, FALSE},
+   {"script", SPACE, HTML_EL_SCRIPT_, L_StrictValue, FALSE},
    {"select", SPACE, HTML_EL_Option_Menu, L_BasicValue, TRUE},
    {"small", SPACE, HTML_EL_Small_text, L_StrictValue, TRUE},
    {"span", SPACE, HTML_EL_Span, L_BasicValue, TRUE},
@@ -122,6 +123,10 @@ static ElemMapping    XHTMLElemMappingTable[] =
    {"u", SPACE, HTML_EL_Underlined_text, L_StrictValue, TRUE},
    {"ul", SPACE, HTML_EL_Unnumbered_List, L_BasicValue, FALSE},
    {"var", SPACE, HTML_EL_Variable, L_BasicValue, TRUE},
+   {"xmlcomment", SPACE, HTML_EL_Comment_, L_OtherValue, FALSE},
+   {"xmlcomment_line", SPACE, HTML_EL_Comment_line, L_OtherValue, FALSE},
+   {"xmlpi", SPACE, HTML_EL_XMLPI, L_OtherValue, FALSE},
+   {"xmlpi_line", SPACE, HTML_EL_PI_line, L_OtherValue, FALSE},
    {"xmp", SPACE, HTML_EL_Preformatted, L_TransitionalValue, FALSE},  /* converted to PRE */
    {"", SPACE, 0, L_BasicValue, FALSE}	  /* Last entry. Mandatory */
 };
@@ -139,10 +144,11 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"accesskey", "", 'A', HTML_ATTR_accesskey, L_BasicValue},
    {"action", "", 'A', HTML_ATTR_Script_URL, L_BasicValue},
    {"align", "applet", 'A', HTML_ATTR_Alignment, L_TransitionalValue},
-   {"align", "caption", 'A', HTML_ATTR_Position, L_StrictValue},
+   {"align", "caption", 'A', HTML_ATTR_Position, L_TransitionalValue},
    {"align", "col", 'A', HTML_ATTR_Cell_align, L_StrictValue},
    {"align", "colgroup", 'A', HTML_ATTR_Cell_align, L_StrictValue},
    {"align", "div", 'A', HTML_ATTR_TextAlign, L_TransitionalValue},
+   {"align", "embed", 'A', HTML_ATTR_Alignment, L_TransitionalValue},
    {"align", "h1", 'A', HTML_ATTR_TextAlign, L_TransitionalValue},
    {"align", "h2", 'A', HTML_ATTR_TextAlign, L_TransitionalValue},
    {"align", "h3", 'A', HTML_ATTR_TextAlign, L_TransitionalValue},
@@ -170,7 +176,8 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"axis", "", 'A', HTML_ATTR_axis, L_BasicValue},
 
    {"background", "", 'A', HTML_ATTR_background_, L_TransitionalValue},
-   {"bgcolor", "", 'A', HTML_ATTR_BackgroundColor, L_TransitionalValue},
+   {"bgcolor", "", 'A', HTML_ATTR_BackgroundColor, L_StrictValue},
+   {"border", "embed", 'A', HTML_ATTR_Img_border, L_TransitionalValue},
    {"border", "image", 'A', HTML_ATTR_Img_border, L_TransitionalValue},
    {"border", "img", 'A', HTML_ATTR_Img_border, L_TransitionalValue},
    {"border", "object", 'A', HTML_ATTR_Img_border, L_TransitionalValue},
@@ -221,6 +228,7 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"height", "img", 'A', HTML_ATTR_Height_, L_BasicValue},
    {"height", "object", 'A', HTML_ATTR_Height_, L_BasicValue},
    {"height", "", 'A', HTML_ATTR_Height_, L_TransitionalValue},
+   {"hidden", "embed", 'A', HTML_ATTR_EmbedHidden, L_TransitionalValue},
    {"href", "", 'A', HTML_ATTR_HREF_, L_BasicValue},
    {"hreflang", "", 'A', HTML_ATTR_hreflang, L_BasicValue},
    {"hspace", "", 'A', HTML_ATTR_hspace, L_TransitionalValue},
@@ -230,8 +238,7 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"ismap", "", 'A', HTML_ATTR_ISMAP, L_StrictValue},
 
    {"label", "", 'A', HTML_ATTR_label, L_BasicValue},
-   {"lang", "", 'A', HTML_ATTR_Langue, L_BasicValue},
-   {"language", "script", 'A', HTML_ATTR_script_language, L_BasicValue},
+   {"lang", "", 'A', HTML_ATTR_Language, L_BasicValue},
    {"link", "body", 'A', HTML_ATTR_LinkColor, L_BasicValue},
    {"longdesc", "", 'A', HTML_ATTR_longdesc, L_BasicValue},
 
@@ -243,6 +250,7 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"multiple", "", 'A', HTML_ATTR_Multiple, L_BasicValue},
 
    {"name", "applet", 'A', HTML_ATTR_applet_name, L_TransitionalValue},
+   {"name", "embed", 'A', HTML_ATTR_EmbedName, L_TransitionalValue},
    {"name", "frame", 'A', HTML_ATTR_FrameName, L_TransitionalValue},
    {"name", "iframe", 'A', HTML_ATTR_FrameName, L_TransitionalValue},
    {"name", "meta", 'A', HTML_ATTR_meta_name, L_BasicValue},
@@ -288,7 +296,7 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"scheme", "meta", 'A', HTML_ATTR_scheme, L_BasicValue},
    {"scope", "", 'A', HTML_ATTR_scope, L_BasicValue},
    {"scrolling", "", 'A', HTML_ATTR_scrolling, L_TransitionalValue},
-   {"selected", "", 'A', HTML_ATTR_Selected, L_BasicValue},
+   {"selected", "option", 'A', HTML_ATTR_Selected, L_BasicValue},
    {"shape", "", 'A', HTML_ATTR_shape, L_StrictValue},
    {"size", "basefont", 'A', HTML_ATTR_BaseFontSize, L_TransitionalValue},
    {"size", "font", 'A', HTML_ATTR_Font_size, L_TransitionalValue},
@@ -297,6 +305,7 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"size", "select", 'A', HTML_ATTR_MenuSize, L_BasicValue},
    {"span", "col", 'A', HTML_ATTR_span_, L_StrictValue},
    {"span", "colgroup", 'A', HTML_ATTR_span_, L_StrictValue},
+   {"src", "embed", 'A', HTML_ATTR_SRC, L_TransitionalValue},
    {"src", "frame", 'A', HTML_ATTR_FrameSrc, L_TransitionalValue},
    {"src", "iframe", 'A', HTML_ATTR_FrameSrc, L_TransitionalValue},
    {"src", "script", 'A', HTML_ATTR_script_src, L_StrictValue},
@@ -311,6 +320,7 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"text", "", 'A', HTML_ATTR_TextColor, L_BasicValue},
    {"title", "", 'A', HTML_ATTR_Title, L_BasicValue},
    {"type", "button", 'A', HTML_ATTR_Button_type, L_StrictValue},
+   {"type", "embed", 'A', HTML_ATTR_Embed_type, L_TransitionalValue},
    {"type", "li", 'A', HTML_ATTR_ItemStyle, L_BasicValue},
    {"type", "link", 'A', HTML_ATTR_Link_type, L_BasicValue},
    {"type", "a", 'A', HTML_ATTR_Link_type, L_BasicValue},
@@ -341,6 +351,7 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"width", "applet", 'A', HTML_ATTR_Width__, L_TransitionalValue},
    {"width", "col", 'A', HTML_ATTR_Width__, L_StrictValue},
    {"width", "colgroup", 'A', HTML_ATTR_Width__, L_StrictValue},
+   {"width", "embed", 'A', HTML_ATTR_Width__, L_TransitionalValue},
    {"width", "hr", 'A', HTML_ATTR_Width__, L_TransitionalValue},
    {"width", "iframe", 'A', HTML_ATTR_Width__, L_TransitionalValue},
    {"width", "image", 'A', HTML_ATTR_Width__, L_TransitionalValue},
@@ -357,6 +368,153 @@ AttributeMapping XHTMLAttributeMappingTable[] =
    {"", "", EOS, 0, L_OtherValue}		/* Last entry. Mandatory */
 };
 
+/* Mapping table of HTML attribute values */
+AttrValueMapping XhtmlAttrValueMappingTable[] =
+{
+   {HTML_ATTR_dir, "ltr", HTML_ATTR_dir_VAL_ltr_},
+   {HTML_ATTR_dir, "rtl", HTML_ATTR_dir_VAL_rtl_},
+
+   {HTML_ATTR_TextAlign, "left", HTML_ATTR_TextAlign_VAL_left_},
+   {HTML_ATTR_TextAlign, "center", HTML_ATTR_TextAlign_VAL_center_},
+   {HTML_ATTR_TextAlign, "right", HTML_ATTR_TextAlign_VAL_right_},
+   {HTML_ATTR_TextAlign, "justify", HTML_ATTR_TextAlign_VAL_justify_},
+
+   {HTML_ATTR_Align, "left", HTML_ATTR_Align_VAL_left_},
+   {HTML_ATTR_Align, "center", HTML_ATTR_Align_VAL_center_},
+   {HTML_ATTR_Align, "right", HTML_ATTR_Align_VAL_right_},
+
+   {HTML_ATTR_LAlign, "top", HTML_ATTR_LAlign_VAL_Top_},
+   {HTML_ATTR_LAlign, "bottom", HTML_ATTR_LAlign_VAL_Bottom_},
+   {HTML_ATTR_LAlign, "left", HTML_ATTR_LAlign_VAL_Left_},
+   {HTML_ATTR_LAlign, "right", HTML_ATTR_LAlign_VAL_Right_},
+
+   {HTML_ATTR_Clear, "left", HTML_ATTR_Clear_VAL_Left_},
+   {HTML_ATTR_Clear, "right", HTML_ATTR_Clear_VAL_Right_},
+   {HTML_ATTR_Clear, "all", HTML_ATTR_Clear_VAL_All_},
+   {HTML_ATTR_Clear, "none", HTML_ATTR_Clear_VAL_None_},
+
+   {HTML_ATTR_NumberStyle, "1", HTML_ATTR_NumberStyle_VAL_Arabic_},
+   {HTML_ATTR_NumberStyle, "a", HTML_ATTR_NumberStyle_VAL_LowerAlpha},
+   {HTML_ATTR_NumberStyle, "A", HTML_ATTR_NumberStyle_VAL_UpperAlpha},
+   {HTML_ATTR_NumberStyle, "i", HTML_ATTR_NumberStyle_VAL_LowerRoman},
+   {HTML_ATTR_NumberStyle, "I", HTML_ATTR_NumberStyle_VAL_UpperRoman},
+
+   {HTML_ATTR_BulletStyle, "disc", HTML_ATTR_BulletStyle_VAL_disc},
+   {HTML_ATTR_BulletStyle, "square", HTML_ATTR_BulletStyle_VAL_square},
+   {HTML_ATTR_BulletStyle, "circle", HTML_ATTR_BulletStyle_VAL_circle},
+
+   {HTML_ATTR_ItemStyle, "1", HTML_ATTR_ItemStyle_VAL_Arabic_},
+   {HTML_ATTR_ItemStyle, "a", HTML_ATTR_ItemStyle_VAL_LowerAlpha},
+   {HTML_ATTR_ItemStyle, "A", HTML_ATTR_ItemStyle_VAL_UpperAlpha},
+   {HTML_ATTR_ItemStyle, "i", HTML_ATTR_ItemStyle_VAL_LowerRoman},
+   {HTML_ATTR_ItemStyle, "I", HTML_ATTR_ItemStyle_VAL_UpperRoman},
+   {HTML_ATTR_ItemStyle, "disc", HTML_ATTR_ItemStyle_VAL_disc},
+   {HTML_ATTR_ItemStyle, "square", HTML_ATTR_ItemStyle_VAL_square},
+   {HTML_ATTR_ItemStyle, "circle", HTML_ATTR_ItemStyle_VAL_circle},
+
+   {HTML_ATTR_Button_type, "button", HTML_ATTR_Button_type_VAL_button},
+   {HTML_ATTR_Button_type, "submit", HTML_ATTR_Button_type_VAL_submit},
+   {HTML_ATTR_Button_type, "reset", HTML_ATTR_Button_type_VAL_reset},
+
+   {HTML_ATTR_frame, "void", HTML_ATTR_frame_VAL_void},
+   {HTML_ATTR_frame, "above", HTML_ATTR_frame_VAL_above},
+   {HTML_ATTR_frame, "below", HTML_ATTR_frame_VAL_below},
+   {HTML_ATTR_frame, "hsides", HTML_ATTR_frame_VAL_hsides},
+   {HTML_ATTR_frame, "lhs", HTML_ATTR_frame_VAL_lhs},
+   {HTML_ATTR_frame, "rhs", HTML_ATTR_frame_VAL_rhs},
+   {HTML_ATTR_frame, "vsides", HTML_ATTR_frame_VAL_vsides},
+   {HTML_ATTR_frame, "box", HTML_ATTR_frame_VAL_box},
+   {HTML_ATTR_frame, "border", HTML_ATTR_frame_VAL_border},
+
+   {HTML_ATTR_frameborder, "0", HTML_ATTR_frameborder_VAL_Border0},
+   {HTML_ATTR_frameborder, "1", HTML_ATTR_frameborder_VAL_Border1},
+
+   {HTML_ATTR_scrolling, "yes", HTML_ATTR_scrolling_VAL_Yes_},
+   {HTML_ATTR_scrolling, "no", HTML_ATTR_scrolling_VAL_No_},
+   {HTML_ATTR_scrolling, "auto", HTML_ATTR_scrolling_VAL_auto_},
+
+   {HTML_ATTR_rules_, "none", HTML_ATTR_rules__VAL_none_},
+   {HTML_ATTR_rules_, "groups", HTML_ATTR_rules__VAL_groups},
+   {HTML_ATTR_rules_, "rows", HTML_ATTR_rules__VAL_rows},
+   {HTML_ATTR_rules_, "cols", HTML_ATTR_rules__VAL_cols},
+   {HTML_ATTR_rules_, "all", HTML_ATTR_rules__VAL_all},
+
+   {HTML_ATTR_Cell_align, "left", HTML_ATTR_Cell_align_VAL_Cell_left},
+   {HTML_ATTR_Cell_align, "center", HTML_ATTR_Cell_align_VAL_Cell_center},
+   {HTML_ATTR_Cell_align, "right", HTML_ATTR_Cell_align_VAL_Cell_right},
+   {HTML_ATTR_Cell_align, "justify", HTML_ATTR_Cell_align_VAL_Cell_justify},
+   {HTML_ATTR_Cell_align, "char", HTML_ATTR_Cell_align_VAL_Cell_char},
+
+   {HTML_ATTR_Alignment, "top", HTML_ATTR_Alignment_VAL_Top_},
+   {HTML_ATTR_Alignment, "middle", HTML_ATTR_Alignment_VAL_Middle_},
+   {HTML_ATTR_Alignment, "bottom", HTML_ATTR_Alignment_VAL_Bottom_},
+   {HTML_ATTR_Alignment, "left", HTML_ATTR_Alignment_VAL_Left_},
+   {HTML_ATTR_Alignment, "right", HTML_ATTR_Alignment_VAL_Right_},
+
+   {HTML_ATTR_METHOD, "get", HTML_ATTR_METHOD_VAL_Get_},
+   {HTML_ATTR_METHOD, "post", HTML_ATTR_METHOD_VAL_Post_},
+
+   {HTML_ATTR_Position, "top", HTML_ATTR_Position_VAL_Position_top},
+   {HTML_ATTR_Position, "bottom", HTML_ATTR_Position_VAL_Position_bottom},
+   {HTML_ATTR_Position, "left", HTML_ATTR_Position_VAL_Position_left},
+   {HTML_ATTR_Position, "right", HTML_ATTR_Position_VAL_Position_right},
+
+   {HTML_ATTR_Row_valign, "top", HTML_ATTR_Row_valign_VAL_Row_top},
+   {HTML_ATTR_Row_valign, "middle", HTML_ATTR_Row_valign_VAL_Row_middle},
+   {HTML_ATTR_Row_valign, "bottom", HTML_ATTR_Row_valign_VAL_Row_bottom},
+   {HTML_ATTR_Row_valign, "baseline", HTML_ATTR_Row_valign_VAL_Row_baseline},
+
+   {HTML_ATTR_Cell_valign, "top", HTML_ATTR_Cell_valign_VAL_Cell_top},
+   {HTML_ATTR_Cell_valign, "middle", HTML_ATTR_Cell_valign_VAL_Cell_middle},
+   {HTML_ATTR_Cell_valign, "bottom", HTML_ATTR_Cell_valign_VAL_Cell_bottom},
+   {HTML_ATTR_Cell_valign, "baseline", HTML_ATTR_Cell_valign_VAL_Cell_baseline},
+
+   {HTML_ATTR_shape, "rect", HTML_ATTR_shape_VAL_rectangle},
+   {HTML_ATTR_shape, "circle", HTML_ATTR_shape_VAL_circle},
+   {HTML_ATTR_shape, "poly", HTML_ATTR_shape_VAL_polygon},
+
+   {HTML_ATTR_valuetype, "data", HTML_ATTR_valuetype_VAL_data_},
+   {HTML_ATTR_valuetype, "ref", HTML_ATTR_valuetype_VAL_ref},
+   {HTML_ATTR_valuetype, "object", HTML_ATTR_valuetype_VAL_object_},
+
+   {HTML_ATTR_EmbedHidden, "yes", HTML_ATTR_EmbedHidden_VAL_Yes_},
+   {HTML_ATTR_EmbedHidden, "no", HTML_ATTR_EmbedHidden_VAL_No_},
+
+/* HTML attribute TYPE generates a Thot element */
+   {DummyAttribute, "button", HTML_EL_Button_Input},
+   {DummyAttribute, "checkbox", HTML_EL_Checkbox_Input},
+   {DummyAttribute, "file", HTML_EL_File_Input},
+   {DummyAttribute, "hidden", HTML_EL_Hidden_Input},
+   {DummyAttribute, "image", HTML_EL_PICTURE_UNIT},
+   {DummyAttribute, "password", HTML_EL_Password_Input},
+   {DummyAttribute, "radio", HTML_EL_Radio_Input},
+   {DummyAttribute, "reset", HTML_EL_Reset_Input},
+   {DummyAttribute, "submit", HTML_EL_Submit_Input},
+   {DummyAttribute, "text", HTML_EL_Text_Input},
+
+/* The following declarations allow the parser to accept boolean attributes */
+/* written "checked=CHECKED"), for instance */
+   {HTML_ATTR_ISMAP, "ismap", HTML_ATTR_ISMAP_VAL_Yes_},
+   {HTML_ATTR_nohref, "nohref", HTML_ATTR_nohref_VAL_Yes_},
+   {HTML_ATTR_COMPACT, "compact", HTML_ATTR_COMPACT_VAL_Yes_},
+   {HTML_ATTR_Multiple, "multiple", HTML_ATTR_Multiple_VAL_Yes_},
+   {HTML_ATTR_Selected, "selected", HTML_ATTR_Selected_VAL_Yes_},
+   {HTML_ATTR_Checked, "checked", HTML_ATTR_Checked_VAL_Yes_},
+   {HTML_ATTR_No_wrap, "nowrap", HTML_ATTR_No_wrap_VAL_no_wrap},
+   {HTML_ATTR_NoShade, "noshade", HTML_ATTR_NoShade_VAL_NoShade_},
+   {HTML_ATTR_declare, "declare", HTML_ATTR_declare_VAL_declare_},
+   {HTML_ATTR_defer, "defer", HTML_ATTR_defer_VAL_Yes_},
+   {HTML_ATTR_disabled, "disabled", HTML_ATTR_disabled_VAL_Yes_},
+   {HTML_ATTR_readonly, "readonly", HTML_ATTR_readonly_VAL_Yes_},
+   {HTML_ATTR_no_resize, "noresize", HTML_ATTR_no_resize_VAL_Yes_},
+
+   /* XML attribute xml:space */
+   {HTML_ATTR_xml_space, "default", HTML_ATTR_xml_space_VAL_xml_space_default},
+   {HTML_ATTR_xml_space, "preserve", HTML_ATTR_xml_space_VAL_xml_space_preserve},
+
+   {0, "", 0}			/* Last entry. Mandatory */
+};
+
 /* Mapping table of Xhtml entities */
 XmlEntity XhtmlEntityTable[] =
 {
@@ -471,6 +629,7 @@ XmlEntity XhtmlEntityTable[] =
 {"empty", 8709},  /* empty set = null set = diameter, U+2205 ISOamso */
 {"emsp", 8195},   /* em space, U+2003 ISOpub */
 {"ensp", 8194},   /* en space, U+2002 ISOpub */
+{"epsi", 949},    /* greek small letter epsilon, U+03B5 ISOgrk3 */
 {"epsilon", 949}, /* greek small letter epsilon, U+03B5 ISOgrk3 */
 {"equiv", 8801},  /* identical to, U+2261 ISOtech */
 {"eta", 951},     /* greek small letter eta, U+03B7 ISOgrk3 */
@@ -565,6 +724,7 @@ XmlEntity XhtmlEntityTable[] =
 {"permil", 8240}, /* per mille sign, U+2030 ISOtech */
 {"perp", 8869},   /* up tack = orthogonal to = perpendicular, U+22A5 ISOtech */
 {"phi", 966},     /* greek small letter phi, U+03C6 ISOgrk3 */
+{"phiv", 981},    /* greek U+03D5 ISOgrk3 */
 {"pi", 960},      /* greek small letter pi, U+03C0 ISOgrk3 */
 {"piv", 982},     /* greek pi symbol, U+03D6 ISOgrk3 */
 {"plusmn", 177},  /* plus-minus sign = plus-or-minus sign, U+00B1 ISOnum */
