@@ -18,8 +18,8 @@
 #include "amaya.h"
 #include "css.h"
 #include "MathML.h"
-#ifdef GRAPHML
-#include "GraphML.h"
+#ifdef _SVG
+#include "SVG.h"
 #endif
 #include "presentation.h"
 
@@ -46,8 +46,8 @@ ThotBool MakeASpan (Element elem, Element *span, Document doc)
   *span = NULL;
   elType = TtaGetElementType (elem);
   if (!strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") ||
-      !strcmp(TtaGetSSchemaName (elType.ElSSchema), "GraphML"))
-    /* it's an HTML or GraphML element */
+      !strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG"))
+    /* it's an HTML or SVG element */
     if (elType.ElTypeNum == HTML_EL_TEXT_UNIT ||
 	elType.ElTypeNum == HTML_EL_Basic_Elem)
       /* it's a text leaf */
@@ -59,8 +59,8 @@ ThotBool MakeASpan (Element elem, Element *span, Document doc)
 	elType = TtaGetElementType (parent);
 	if ((elType.ElTypeNum == HTML_EL_Span &&
 	     !strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML")) ||
-	    (elType.ElTypeNum == GraphML_EL_tspan &&
-	     !strcmp(TtaGetSSchemaName (elType.ElSSchema), "GraphML")))
+	    (elType.ElTypeNum == SVG_EL_tspan &&
+	     !strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG")))
 	   /* element parent is a span element */
 	   {
 	   sibling = elem;
@@ -85,7 +85,7 @@ ThotBool MakeASpan (Element elem, Element *span, Document doc)
 	   if (!strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML"))
 	      elType.ElTypeNum = HTML_EL_Span;
 	   else
-	      elType.ElTypeNum = GraphML_EL_tspan;
+	      elType.ElTypeNum = SVG_EL_tspan;
 	   *span = TtaNewElement (doc, elType);
 	   TtaInsertSibling (*span, elem, FALSE, doc);
 	   TtaRemoveTree (elem, doc);
@@ -206,7 +206,7 @@ void  AttrToSpan (Element elem, Attribute attr, Document doc)
 /*----------------------------------------------------------------------
   GlobalAttrCreated: the user has created a global attribute
   ----------------------------------------------------------------------*/
-void                GlobalAttrCreated (NotifyAttribute * event)
+void GlobalAttrCreated (NotifyAttribute * event)
 {
    /* if the attribute is on a text string, create a SPAN element that encloses
       this text string and move the attribute to that SPAN element */
@@ -216,7 +216,7 @@ void                GlobalAttrCreated (NotifyAttribute * event)
 /*----------------------------------------------------------------------
   GlobalAttrDeleted: the user has removed a global attribute
   ----------------------------------------------------------------------*/
-void                GlobalAttrDeleted (NotifyAttribute * event)
+void GlobalAttrDeleted (NotifyAttribute * event)
 {
    Element	firstChild, lastChild;
 
@@ -229,7 +229,7 @@ void                GlobalAttrDeleted (NotifyAttribute * event)
   AttrClassChanged: the user has created removed or modified a Class
   attribute
   ----------------------------------------------------------------------*/
-void                AttrClassChanged (NotifyAttribute * event)
+void AttrClassChanged (NotifyAttribute * event)
 {
    Element	firstChild, lastChild;
 
@@ -292,7 +292,7 @@ void MovePRule (PRule presRule, Element fromEl, Element toEl, Document doc,
 /*----------------------------------------------------------------------
   SetStyleAttribute.
   ----------------------------------------------------------------------*/
-void                SetStyleAttribute (Document doc, Element elem)
+void SetStyleAttribute (Document doc, Element elem)
 {
    AttributeType       attrType;
    Attribute           styleAttr;
@@ -312,14 +312,14 @@ void                SetStyleAttribute (Document doc, Element elem)
        attrType.AttrTypeNum = MathML_ATTR_style_;
      }
    else
-#ifdef GRAPHML
-   if (strcmp (schName, "GraphML") == 0)
+#ifdef _SVG
+   if (strcmp (schName, "SVG") == 0)
      {
        attrType.AttrSSchema = elType.ElSSchema;
-       attrType.AttrTypeNum = GraphML_ATTR_style_;
+       attrType.AttrTypeNum = SVG_ATTR_style_;
      }
    else
-#endif /* GRAPHML */
+#endif /* _SVG */
       {
 	attrType.AttrSSchema = TtaGetSSchema ("HTML", doc);
 	attrType.AttrTypeNum = HTML_ATTR_Style_;
@@ -366,7 +366,7 @@ void                SetStyleAttribute (Document doc, Element elem)
   A specific PRule will be created or modified by the user for
   a given element. (pre-event)
   ----------------------------------------------------------------------*/
-ThotBool            ChangePRule (NotifyPresentation * event)
+ThotBool ChangePRule (NotifyPresentation *event)
 {
   ElementType	     elType, parentType;
   Element	     el, span, body, root, parent;
@@ -381,6 +381,9 @@ ThotBool            ChangePRule (NotifyPresentation * event)
   int                w, h, unit, value, i;
   ThotBool           ret;
 
+  /* Store the new value into the presentation rule */
+  TtaSetPRuleValue (event->element, event->pRule, event->value, event->document);
+
   el = event->element;
   doc = event->document;
   presType = event->pRuleType;
@@ -506,7 +509,7 @@ ThotBool            ChangePRule (NotifyPresentation * event)
 		  {
 		    /* the new value is the old one plus the delta */
 		    TtaGiveBoxSize (el, doc, 1, unit, &value, &i);
-		    value = TtaGetPRuleValue (presRule);
+		    value = event->value;
 		    if (unit == UnPercent)
 		      sprintf (buffer, "%d%%", value);
 		    else
@@ -532,14 +535,14 @@ ThotBool            ChangePRule (NotifyPresentation * event)
 		  {
 		    /* the new value is the old one plus the delta */
 		    TtaGiveBoxSize (el, doc, 1, unit, &i, &value);
-		    value = TtaGetPRuleValue (presRule);
+		    value = event->value;
 		    attrType.AttrTypeNum = HTML_ATTR_Height_;
 		    attr = TtaGetAttribute (el, attrType);
 		    if (attr == NULL)
 		      {
 			attr = TtaNewAttribute (attrType);
 			TtaAttachAttribute (el, attr, doc);
-		        TtaSetAttributeValue (attr, value, el, doc);
+			TtaSetAttributeText (attr, buffer, el, doc);
 			TtaRegisterAttributeCreate (attr, el, doc);
 		      }
 		    else
@@ -547,10 +550,9 @@ ThotBool            ChangePRule (NotifyPresentation * event)
 			TtaRegisterAttributeReplace (attr, el, doc);
 		        TtaSetAttributeValue (attr, value, el, doc);
 		      }
-		    /* update the associated map */
-		    if (elType.ElTypeNum == HTML_EL_PICTURE_UNIT)
-		      UpdateImageMap (el, doc, -1, h);
+		    CreateAttrHeightPercentPxl (buffer, el, doc, h);
 		  }
+		TtaSetDocumentModified (doc);
 		return TRUE; /* don't let Thot perform normal operation */
 	      }
 	    else if (IsCharacterLevelElement (el)
@@ -596,124 +598,53 @@ ThotBool            ChangePRule (NotifyPresentation * event)
   return (ret);
 }
 
-
 /*----------------------------------------------------------------------
   PRuleDeleted
   A specific PRule has been deleted by the user for a given element
   (post-event)
   ----------------------------------------------------------------------*/
-void                PRuleDeleted (NotifyPresentation * event)
+void PRuleDeleted (NotifyPresentation * event)
 {
   /* set the Style_ attribute ? */
   SetStyleAttribute (event->document, event->element);
 }
 
-
 /*----------------------------------------------------------------------
  AttrLangDeleted
  A Lang attribute has been deleted
  -----------------------------------------------------------------------*/
 void AttrLangDeleted (NotifyAttribute *event)
 {
-   Element	 firstChild, lastChild;
-   AttributeType attrType;
-   Attribute	 attr;
+  Element	 elem, firstChild, lastChild;
+  ElementType    elType;
+  AttributeType  attrType;
+  Attribute	 attr;
 
+  elem = event->element;
+  elType = TtaGetElementType (elem);
   /* if the element is a SPAN without any other attribute, remove the SPAN
      element */
-  DeleteSpanIfNoAttr (event->element, event->document, &firstChild,&lastChild);
-  /* if it's the root (HTML) element, delete the RealLang attribute */
-  if (!TtaGetParent (event->element))
-     /* it's the root element */
-     {
-     attrType.AttrSSchema = event->attributeType.AttrSSchema;
-     attrType.AttrTypeNum = HTML_ATTR_RealLang;
-     attr = TtaGetAttribute (event->element, attrType);
-     if (attr)
-	TtaRemoveAttribute (event->element, attr, event->document);
+  if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+    DeleteSpanIfNoAttr (elem, event->document, &firstChild,&lastChild);
+  /* if it's the root (HTML, SVG, MathML) element, delete the RealLang
+     attribute */
+  if (elem == TtaGetRootElement (event->document))
+    /* it's the root element */
+    {
+      elType = TtaGetElementType (elem);
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	attrType.AttrTypeNum = HTML_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG") == 0)
+	attrType.AttrTypeNum = SVG_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "MathML") == 0)
+	attrType.AttrTypeNum = MathML_ATTR_RealLang;
+      attrType.AttrSSchema = event->attributeType.AttrSSchema;
+      attr = TtaGetAttribute (elem, attrType);
+      if (attr)
+	TtaRemoveAttribute (elem, attr, event->document);
      }
 }
 
-
-/*----------------------------------------------------------------------
- MoveAttrLang
- 
- -----------------------------------------------------------------------*/
-static void MoveAttrLang (Attribute oldAttr, Element *el, Document doc)
-{
-  Element	first, parent, sibling, next, firstChild, lastChild;
-  Attribute	newAttr, attr;
-  AttributeType	attrType;
-  int		kind, len;
-  char	       *value    = TtaGetMemory (ATTRLEN); 
-  char         * oldValue = TtaGetMemory (ATTRLEN);
-  ThotBool	sameLang;
-
-  /* if all siblings have the same LANG attribute, move that attibute to
-     the parent element, unless the parent element has exception Hidden
-     or the parent element is the document element */
-  parent = TtaGetParent (*el);
-  if (parent)
-     if (TtaIsHidden (parent) || !TtaGetParent (parent))
-        parent = NULL;
-  if (parent != NULL)
-     {
-     TtaGiveAttributeType (oldAttr, &attrType, &kind);
-     len = ATTRLEN - 1;
-     TtaGiveTextAttributeValue (oldAttr, oldValue, &len);
-     first = TtaGetFirstChild (parent);
-     sameLang = TRUE;
-     sibling = first;
-     while (sibling != NULL && sameLang)
-	{
-	if (sibling != *el)
-	   {
-	   attr = TtaGetAttribute (sibling, attrType);
-	   if (attr == NULL)
-	      sameLang = FALSE;
-	   else
-	      {
-	      len = ATTRLEN - 1;
-	      TtaGiveTextAttributeValue (attr, value, &len);
-	      if (strcasecmp(oldValue, value) != 0)
-		 sameLang = FALSE;
-	      }
-	   }
-	if (sameLang)
-	   TtaNextSibling (&sibling);
-	}
-     if (sameLang)
-        /* all sibling have the same LANG attribute */
-	{
-	/* delete the LANG attribute for all siblings */
-	sibling = first;
-	while (sibling != NULL)
-	   {
-	   attr = TtaGetAttribute (sibling, attrType);
-	   TtaRemoveAttribute (sibling, attr, doc);
-	   next = sibling;
-	   TtaNextSibling (&next);
-	   DeleteSpanIfNoAttr (sibling, doc, &firstChild, &lastChild);
-	   sibling = next;
-	   }
-	/* associate a LANG attribute with the parent element */
-	newAttr = TtaGetAttribute (parent, attrType);
-	if (newAttr == NULL)
-	  {
-	    newAttr = TtaNewAttribute (attrType);
-	    TtaAttachAttribute (parent, newAttr, doc);
-	  }
-	TtaSetAttributeText (newAttr, oldValue, parent, doc);
-	*el = parent;
-	/* do it again for the parent element */
-        MoveAttrLang (newAttr, el, doc);
-	}
-     }
-  TtaFreeMemory (value);
-  TtaFreeMemory (oldValue);
-}
-
-
 /*----------------------------------------------------------------------
  AttrLangCreated
  A Lang attribute has been created
@@ -721,17 +652,14 @@ static void MoveAttrLang (Attribute oldAttr, Element *el, Document doc)
 void AttrLangCreated (NotifyAttribute *event)
 {
   Element	elem;
-  int		len;
   ElementType   elType;
+  int		len;
   AttributeType attrType;
   Attribute	attr;
   char	       *value = TtaGetMemory (ATTRLEN); 
 
-  /* move the LANG attribute to the parent element if all sibling have the
-     same attribute with the same value */
   elem = event->element;
-  MoveAttrLang (event->attribute, &elem, event->document);
-
+  elType = TtaGetElementType (elem);
   len = ATTRLEN - 1;
   TtaGiveTextAttributeValue (event->attribute, value, &len);
   if (strcasecmp(value, "Symbol") == 0)
@@ -739,18 +667,25 @@ void AttrLangCreated (NotifyAttribute *event)
        a language */
     TtaRemoveAttribute (elem, event->attribute, event->document);      
   else
-    /* if the LANG attribute is on a text string, create a SPAN element that
-       encloses this text string and move the LANG attribute to that SPAN
-       element */
-    AttrToSpan (elem, event->attribute, event->document);
-  /* if it's the root (HTML) element, create a RealLang attribute too */
-  elType = TtaGetElementType (elem);
-  if (elType.ElTypeNum == HTML_EL_HTML &&
-      !strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML"))
-     /* it's the HTML root element */
+    /* if the LANG attribute is on a text string in a HTML document, create
+       a SPAN element that encloses this text string and move the LANG
+       attribute to that SPAN element */
+    {
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	AttrToSpan (elem, event->attribute, event->document);
+    }
+  /* if it's the root (HTML, SVG, MathML) element, create a RealLang
+     attribute too */
+  if (elem == TtaGetRootElement (event->document))
+     /* it's the root element */
      {
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	attrType.AttrTypeNum = HTML_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG") == 0)
+	attrType.AttrTypeNum = SVG_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "MathML") == 0)
+	attrType.AttrTypeNum = MathML_ATTR_RealLang;
      attrType.AttrSSchema = event->attributeType.AttrSSchema;
-     attrType.AttrTypeNum = HTML_ATTR_RealLang;
      attr = TtaNewAttribute (attrType);
      TtaAttachAttribute (elem, attr, event->document);
      TtaSetAttributeValue (attr, HTML_ATTR_RealLang_VAL_Yes_, elem,
@@ -766,23 +701,31 @@ void AttrLangCreated (NotifyAttribute *event)
 void AttrLangModified (NotifyAttribute *event)
 {
   Element	elem;
+  ElementType   elType;
   AttributeType attrType;
   Attribute	attr;
 
   elem = event->element;
-  /* if it's the root (HTML) element, create a RealLang attribute */
-  if (!TtaGetParent (elem))
-     /* it's the root element */
-     {
-     attrType.AttrSSchema = event->attributeType.AttrSSchema;
-     attrType.AttrTypeNum = HTML_ATTR_RealLang;
-     attr = TtaGetAttribute (elem, attrType);
-     if (!attr)
+  /* if it's the root (HTML, SVG, MathML) element, create a RealLang
+     attribute */
+  if (elem == TtaGetRootElement (event->document))
+    /* it's the root element */
+    {
+      elType = TtaGetElementType (elem);
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	attrType.AttrTypeNum = HTML_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG") == 0)
+	attrType.AttrTypeNum = SVG_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "MathML") == 0)
+	attrType.AttrTypeNum = MathML_ATTR_RealLang;
+      attrType.AttrSSchema = event->attributeType.AttrSSchema;
+      attr = TtaGetAttribute (elem, attrType);
+      if (!attr)
 	{
-        attr = TtaNewAttribute (attrType);
-        TtaAttachAttribute (elem, attr, event->document);
-        TtaSetAttributeValue (attr, HTML_ATTR_RealLang_VAL_Yes_, elem,
-			      event->document);
+	  attr = TtaNewAttribute (attrType);
+	  TtaAttachAttribute (elem, attr, event->document);
+	  TtaSetAttributeValue (attr, HTML_ATTR_RealLang_VAL_Yes_, elem,
+				event->document);
 	}
      }  
 }
@@ -794,19 +737,26 @@ void AttrLangModified (NotifyAttribute *event)
 ThotBool AttrLangShouldBeDeleted (NotifyAttribute *event)
 {
   Element	elem;
+  ElementType   elType;
   AttributeType attrType;
   Attribute	attr;
 
   elem = event->element;
   /* if it's the root (HTML) element, delete the RealLang attribute */
-  if (!TtaGetParent (elem))
-     /* it's the root element */
-     {
-     attrType.AttrSSchema = event->attributeType.AttrSSchema;
-     attrType.AttrTypeNum = HTML_ATTR_RealLang;
-     attr = TtaGetAttribute (elem, attrType);
-     if (attr)
+  if (elem == TtaGetRootElement (event->document))
+    /* it's the root element */
+    {
+      elType = TtaGetElementType (elem);
+      if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "HTML") == 0)
+	attrType.AttrTypeNum = HTML_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "SVG") == 0)
+	attrType.AttrTypeNum = SVG_ATTR_RealLang;
+      else if (strcmp(TtaGetSSchemaName (elType.ElSSchema), "MathML") == 0)
+	attrType.AttrTypeNum = MathML_ATTR_RealLang;
+      attrType.AttrSSchema = event->attributeType.AttrSSchema;
+      attr = TtaGetAttribute (elem, attrType);
+      if (attr)
 	TtaRemoveAttribute (elem, attr, event->document);
-     }  
+    }  
   return FALSE; /* let Thot perform normal operation */
 }
