@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -14,11 +14,13 @@
 /* Amaya includes  */
 #define THOT_EXPORT extern
 #include "amaya.h"
+#include "fetchXMLname_f.h"
 #include "undo.h"
 #include "MathML.h"
+#include "HTMLedit_f.h"
+#include "HTMLtable_f.h"
 #include "MathMLbuilder_f.h"
 #include "Mathedit_f.h"
-#include "HTMLtable_f.h"
 
 static Element      CurrentRow = NULL;
 static Element      CurrentDeletedRow = NULL;
@@ -487,7 +489,7 @@ ThotBool RemoveColumn (Element colhead, Document doc, ThotBool ifEmpty,
 	  while (row && empty)
 	    {
 	      cell = GetCellFromColumnHead (row, colhead, inMath);
-	      if (cell != NULL && TtaGetVolume (cell) != 0)
+	      if (cell != NULL && TtaGetElementVolume (cell) != 0)
 		empty = FALSE;
 	      else
 		{
@@ -600,13 +602,12 @@ void CheckAllRows (Element table, Document doc, ThotBool placeholder,
 {
   Element            *colElement;
   Element             row, nextRow, firstrow, colhead, prevColhead;
-  Element             cell, nextCell, group, new;
-  Element             differredHSpan[MAX_COLS];
+  Element             cell, nextCell, group, prevGroup, new;
   ElementType         elType;
   AttributeType       attrTypeHSpan, attrTypeVSpan, attrType;
   Attribute           attr;
-  int                *colVSpan, hspan, differredRef[MAX_COLS];
-  int                 span, cRef, cNumber;
+  int                *colVSpan;
+  int                 span, cRef, cNumber, extracol;
   int                 i, rowType;
   ThotBool            inMath;
 
@@ -615,7 +616,6 @@ void CheckAllRows (Element table, Document doc, ThotBool placeholder,
 
   colElement = TtaGetMemory (sizeof (Element) * MAX_COLS);
   colVSpan = TtaGetMemory (sizeof (int) * MAX_COLS);
-
   /* store the list of colheads */
   elType = TtaGetElementType (table);
   inMath = !TtaSameSSchemas (elType.ElSSchema,
@@ -641,8 +641,8 @@ void CheckAllRows (Element table, Document doc, ThotBool placeholder,
       cNumber++;
     }
   cell = NULL;
-  /* number of differed colspan rules */
-  hspan = 0;
+  /* number of extra columns */
+  extracol = 0;
   attrType.AttrSSchema = elType.ElSSchema;
   attrTypeHSpan.AttrSSchema = elType.ElSSchema;
   if (inMath)
@@ -759,6 +759,7 @@ void CheckAllRows (Element table, Document doc, ThotBool placeholder,
 		    i++;
 		    }
 		  if (nextCell)
+		    extracol++;
 		    /* Create additional Column_heads */
 		    while (i < span)
 		      {
@@ -772,15 +773,6 @@ void CheckAllRows (Element table, Document doc, ThotBool placeholder,
 		      colVSpan[cRef] = colVSpan[cRef-1];
 		      i++;
 		      }
-		  else if (i < span && hspan < MAX_COLS &&
-			   colVSpan[cRef] == 0)
-		    {
-		    /* differs the management of the colspan */
-		     differredHSpan[hspan] = cell;
-		     differredRef[hspan] = cRef - i + 1;
-		     hspan++;
-		     span = 1;
-		    }
 		  if (span > 1)
 		    {
 		    /* set attribute ColExt */
@@ -865,59 +857,37 @@ void CheckAllRows (Element table, Document doc, ThotBool placeholder,
 	if (group != NULL)
 	  {
 	  elType = TtaGetElementType (group);
-	  if (elType.ElTypeNum == HTML_EL_Table_foot)
-	    /* don't look for rows in the Table_foot! */
-	    row = NULL;
+	  /* skip comments, PIs and invalid elements */
+	  while (group &&
+		 (elType.ElTypeNum == HTML_EL_TEXT_UNIT ||
+		  elType.ElTypeNum == HTML_EL_Invalid_element ||
+		  elType.ElTypeNum == HTML_EL_Comment_ ||
+		  elType.ElTypeNum == HTML_EL_XMLPI))
+	    {
+	      prevGroup = group;
+	      TtaNextSibling (&group);
+	      if (elType.ElTypeNum == HTML_EL_TEXT_UNIT)
+		TtaDeleteTree (prevGroup, doc);
+	      if (group)
+		elType = TtaGetElementType (group);
+	    }
+	  if (group)
+	    {
+	      elType = TtaGetElementType (group);
+	      if (elType.ElTypeNum == HTML_EL_Table_foot)
+		/* don't look for rows in the Table_foot! */
+		row = NULL;
+	      else
+		row = TtaGetFirstChild (group);
+	    }
 	  else
-	    row = TtaGetFirstChild (group);
+	    row = NULL;
 	  }
 	else
 	  row = NULL;
 	}
       }
     }
-  /* now manage differed colspan rules */
-  for (i = 0; i < hspan; i++)
-    {
-    /* if there an attribute colspan for that cell, update attribute ColExt */
-    cell = differredHSpan[i];
-    attr = TtaGetAttribute (cell, attrTypeHSpan);
-    span = TtaGetAttributeValue (attr);
-    cRef = differredRef[i];
-    if (span + cRef >= cNumber)
-      {
-      span = cNumber - cRef;
-      /* the span value is not correct, set the right value */
-      TtaSetAttributeValue (attr, span, cell, doc);
-      }
-    if (span > 1)
-      {
-      /* set attribute ColExt */
-      if (inMath)
-	attrType.AttrTypeNum = MathML_ATTR_MColExt;
-      else
-	attrType.AttrTypeNum = HTML_ATTR_ColExt;
-      attr = TtaGetAttribute (cell, attrType);
-      if (attr == NULL)
-	{
-	attr = TtaNewAttribute (attrType);
-	TtaAttachAttribute (cell, attr, doc);
-	}
-      if (attr != NULL)
-	TtaSetAttributeReference (attr, cell, doc, colElement[cRef + span - 1],
-				  doc);
-      /* remove extra cells */
-      nextCell = cell;
-      TtaNextSibling (&nextCell);
-      while (span > 1 && nextCell != NULL)
-	{
-	cell = nextCell;
-	TtaNextSibling (&nextCell);
-	TtaDeleteTree (cell, doc);
-	span--;
-	}
-      }
-    }
 
   /* if there are some empty columns at the end, remove them */
   if (deleteLastEmptyColumns && cNumber > 0)
@@ -1291,6 +1261,8 @@ void                CellPasted (NotifyElement * event)
 
    cell = event->element;
    doc = event->document;
+   if (!ElementOKforProfile (cell, doc))
+     return;
    row = TtaGetParent (cell);
    if (row == CurrentPastedRow)
      /* this cell is part of the pasted row */
@@ -1625,6 +1597,9 @@ void                RowPasted (NotifyElement * event)
 
   row = event->element;
   doc = event->document;
+  if (!ElementOKforProfile (row, doc))
+    return;
+
   elType = TtaGetElementType (row);
   inMath = TtaSameSSchemas (elType.ElSSchema,
 			    TtaGetSSchema ("MathML", doc));
