@@ -40,8 +40,13 @@
 #define mGeneralEmpty2       8
 #define mGeneralEmpty3       9
 #define mGeneralEmpty4       10
-#define mToggleGeneral       11
-#define MAX_GENERALMENU_DLG  12
+#define mGeneralAccessKey    11
+#define mToggleGeneral       12
+#define MAX_GENERALMENU_DLG  13
+
+#define BrowseMenu          1
+#define mToggleBrowse       2
+#define MAX_BROWSEMENU_DLG  3
 
 #define PublishMenu         1
 #define mTogglePublish      2
@@ -86,7 +91,9 @@ static void BuildProfileSelector();
 #define mAnnotPostServer        3
 #define mAnnotServers           4
 #define mToggleAnnot            5
-#define mAnnotAutoLoad          6
-#define MAX_ANNOTMENU_DLG       7
+#define mAnnotLAutoLoad         6
+#define mAnnotRAutoLoad         7
+#define mAnnotRAutoLoadRst      8
+#define MAX_ANNOTMENU_DLG       9
 
 #endif /* _MENUCONF.h */
