@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -66,7 +66,7 @@ ThotBool  LoadRemoteStyleSheet (char *url, Document doc, Element el,
   if (IsW3Path (completeURL))
     {
       /* check against double inclusion */
-      oldcss = SearchCSS (0, completeURL);
+      oldcss = SearchCSS (0, completeURL, NULL);
       if (oldcss != NULL)
 	strcpy (localfile, oldcss->localName);
       else
@@ -107,10 +107,10 @@ ThotBool  LoadRemoteStyleSheet (char *url, Document doc, Element el,
 
 
 /*----------------------------------------------------------------------
-   LoadUserStyleSheet : Load the user Style Sheet found in it's    
+   LoadUserStyleSheet : Load the user Style Sheet found in his/her   
    home directory or the default one in THOTDIR.           
   ----------------------------------------------------------------------*/
-void                LoadUserStyleSheet (Document doc)
+void LoadUserStyleSheet (Document doc)
 {
   CSSInfoPtr          css;
   struct stat         buf;
@@ -139,7 +139,7 @@ void                LoadUserStyleSheet (Document doc)
       ptr = GetLocalPath (0, UserCSS);
       TtaFileCopy (UserCSS, ptr);
       /* allocate a new Presentation structure */ 
-      css = AddCSS (0, doc, CSS_USER_STYLE, UserCSS, ptr);
+      css = AddCSS (0, doc, CSS_USER_STYLE, UserCSS, ptr, NULL);
       TtaFreeMemory (ptr);
       ptr = css->localName;
     }
@@ -189,7 +189,7 @@ void                LoadUserStyleSheet (Document doc)
       /* parse the whole thing and free the buffer */
       if (buffer != NULL)
 	{
-	  ReadCSSRules (doc, css, buffer, 0, FALSE);
+	  ReadCSSRules (doc, css, buffer, css->url, 0, FALSE, NULL, NULL);
 	  TtaFreeMemory (buffer);
 	}
     }
@@ -199,7 +199,7 @@ void                LoadUserStyleSheet (Document doc)
   AttrMediaChanged: the user has created removed or modified a Media
   attribute
   ----------------------------------------------------------------------*/
-void                AttrMediaChanged (NotifyAttribute *event)
+void AttrMediaChanged (NotifyAttribute *event)
 {
    ElementType         elType;
    Element             el;
@@ -248,18 +248,23 @@ void                AttrMediaChanged (NotifyAttribute *event)
        /* load the stylesheet file found here ! */
        NormalizeURL (name2, doc, completeURL, tempname, NULL);
        TtaFreeMemory (name2);
-       css = SearchCSS (0, completeURL);
-       if (css->media[doc] != media)
+       css = SearchCSS (0, completeURL, NULL);
+       if (css && css->media[doc] != media)
 	 {
 	   /* something changed and we are not printing */
 	   if ((media == CSS_ALL || media == CSS_SCREEN) &&
 	       (css->media[doc] == CSS_PRINT || css->media[doc] == CSS_OTHER))
-	     LoadStyleSheet (completeURL, doc, el, NULL, media);
+	     LoadStyleSheet (completeURL, doc, el, NULL, media,
+			     css->category == CSS_USER_STYLE);
 	   else
 	     {
 	       if ((media == CSS_PRINT || media == CSS_OTHER) &&
 	       (css->media[doc] == CSS_ALL || css->media[doc] == CSS_SCREEN))
-		 RemoveStyleSheet (completeURL, doc, FALSE, FALSE);
+		 {
+		   if (elType.ElTypeNum != HTML_EL_STYLE_)
+		     el = NULL;
+		   RemoveStyleSheet (completeURL, doc, FALSE, FALSE, el);
+		 }
 	       /* only update the CSS media info */
 	       css->media[doc] = media;
 	     }
@@ -277,8 +282,10 @@ void                AttrMediaChanged (NotifyAttribute *event)
   directory.
   Return the list of temporary file names. That list includes:
   - first the User style sheet
-  - after expternal style sheets
-  - the the Document style sheet
+  - the external style sheets linked to the document
+  - the style sheet embedded in a style element within the document
+  Each filename is preceded by "u" and a space if it's a user style sheet,
+  by "a" and a space if it's an author style sheet.
   The returned string should be freed by the caller.
   ----------------------------------------------------------------------*/
 char *CssToPrint (Document doc, char *printdir)
@@ -307,6 +314,8 @@ char *CssToPrint (Document doc, char *printdir)
 	    else
 	      /* that external or user style sheet concerns the document */
 	      length += strlen (css->localName) + 1;
+	    /* add room for "a " or "u " in front of each filename */
+	    length += 2;
 	    }
 	  css = css->NextCSS;
 	}
@@ -323,13 +332,13 @@ char *CssToPrint (Document doc, char *printdir)
 		  css->category == CSS_USER_STYLE)
 		{
 		  /* add that file name to the list */
+		  strcpy (&ptr[length], "u ");
+		  length += 2;
 		  strcpy (&ptr[length], css->localName);
 		  length += strlen (css->localName);
 		  ptr[length++] = SPACE;
-		  css = NULL;
 		}
-	      else
-		css = css->NextCSS;
+	      css = css->NextCSS;
 	    }
 
 	  /* Add after the list of external CSS files */
@@ -340,6 +349,8 @@ char *CssToPrint (Document doc, char *printdir)
 		  css->category == CSS_EXTERNAL_STYLE)
 		{
 		  /* add that file name to the list */
+		  strcpy (&ptr[length], "a ");
+		  length += 2;
 		  strcpy (&ptr[length], css->localName);
 		  length += strlen (css->localName);
 		  ptr[length++] = SPACE;
@@ -361,8 +372,8 @@ char *CssToPrint (Document doc, char *printdir)
 #ifdef _SVG
 	  else if (!strcmp (name, "SVG"))
 	    {
-	    elType.ElTypeNum = SVG_EL_style__;
-	    head = el;
+	      elType.ElTypeNum = SVG_EL_style__;
+	      head = el;
 	    }
 #endif /* _SVG */
 	  else
@@ -386,6 +397,8 @@ char *CssToPrint (Document doc, char *printdir)
 		      if (file)
 			{
 			  /* add that file name to the list */
+			  strcpy (&ptr[length], "a ");
+			  length += 2;
 			  strcpy (&ptr[length], tempfile);
 			  length += strlen (tempfile);
 			  ptr[length++] = SPACE;
@@ -417,7 +430,7 @@ char *CssToPrint (Document doc, char *printdir)
 /*----------------------------------------------------------------------
    Callback procedure for dialogue events.                            
   ----------------------------------------------------------------------*/
-static void         CallbackCSS (int ref, int typedata, char *data)
+static void CallbackCSS (int ref, int typedata, char *data)
 {
   CSSInfoPtr          css;
   PInfoPtr            pInfo;
@@ -430,40 +443,41 @@ static void         CallbackCSS (int ref, int typedata, char *data)
     {
     case CSSForm:
       TtaDestroyDialogue (ref);
-      if (val == 1 && CSSpath != NULL)
+      if (val == 1 && CSSpath)
 	{
 	  switch (CSScase)
 	    {
 	    case 1:
 	      /* display the CSS file */
-	      GetHTMLDocument (CSSpath, NULL, 0, 0, CE_CSS, FALSE,
-			       NULL, NULL);
+	      GetAmayaDoc (CSSpath, NULL, 0, 0, CE_CSS, FALSE, NULL,
+			   NULL, UTF_8);
 	      break;
 	    case 2:
 	      /* disable the CSS file, but not remove */
 	      if (!strcmp (CSSpath, TtaGetMessage (AMAYA, AM_LOCAL_CSS)))
-		RemoveStyleSheet (NULL, CSSdocument, TRUE, FALSE);
+		RemoveStyleSheet (NULL, CSSdocument, TRUE, FALSE, NULL);
 	      else
-		RemoveStyleSheet (CSSpath, CSSdocument, TRUE, FALSE);
+		RemoveStyleSheet (CSSpath, CSSdocument, TRUE, FALSE, NULL);
       	      break;
 	    case 3:
 	      /* enable the CSS file */
 	      if (!strcmp (CSSpath, TtaGetMessage (AMAYA, AM_LOCAL_CSS)))
 		{
-		  css = SearchCSS (CSSdocument, NULL);
+		  css = SearchCSS (CSSdocument, NULL, NULL);
 		  css ->enabled[CSSdocument] = TRUE;
 		  EnableStyleElement (CSSdocument);
 		}
 	      else
 		{
-		  css = SearchCSS (0, CSSpath);
-		  css ->enabled[CSSdocument] = TRUE;
+		  css = SearchCSS (0, CSSpath, NULL);
+		  css->enabled[CSSdocument] = TRUE;
 		  /* apply CSS rules */
 		  if (UserCSS && !strcmp (CSSpath, UserCSS))
 		    LoadUserStyleSheet (CSSdocument);
 		  else
 		    LoadStyleSheet (CSSpath, CSSdocument, NULL, NULL,
-				    css->media[CSSdocument]);
+				    css->media[CSSdocument],
+				    css->category == CSS_USER_STYLE);
 		}
       	      break;
 	    case 4:
@@ -507,8 +521,7 @@ static void         CallbackCSS (int ref, int typedata, char *data)
       CSSpath = NULL;
       break;
     case CSSSelect:
-      if (CSSpath != NULL)
-	TtaFreeMemory (CSSpath);
+      TtaFreeMemory (CSSpath);
       length = strlen (data);
       CSSpath = TtaGetMemory (length + 1);
       strcpy (CSSpath, data);      
@@ -521,7 +534,7 @@ static void         CallbackCSS (int ref, int typedata, char *data)
 /*----------------------------------------------------------------------
    InitCSS                                                         
   ----------------------------------------------------------------------*/
-void                InitCSS (void)
+void InitCSS (void)
 {
    /* initialize the dialogs */
    BaseCSS = TtaSetCallback (CallbackCSS, MAX_CSS_REF);
@@ -531,7 +544,7 @@ void                InitCSS (void)
 /*----------------------------------------------------------------------
    InitCSSDialog list downloaded CSS files
   ----------------------------------------------------------------------*/
-static void         InitCSSDialog (Document doc, char *s)
+static void InitCSSDialog (Document doc, char *s)
 {
   CSSInfoPtr          css;
   char                buf[400];
@@ -565,26 +578,32 @@ static void         InitCSSDialog (Document doc, char *s)
 	      CSScase == 4)
 	    {
 	      /* filter enabled and disabled entries */
-	      /* build the CSS list */
+	      /* build the CSS list:
+		 use the dialogue encoding for buf and UTF-8 for CSS path  */
 	      if (css->category == CSS_DOCUMENT_STYLE)
-		ptr = TtaGetMessage (AMAYA, AM_LOCAL_CSS);
-	      else if (css->url == NULL)
-		ptr = css->localName;
+		ptr = TtaStrdup (TtaGetMessage (AMAYA, AM_LOCAL_CSS));
 	      else
-		ptr = css->url;
-
+		{
+		  if (css->url == NULL)
+		    ptr = TtaConvertMbsToByte (css->localName,
+					      TtaGetDefaultCharset ());
+		  else
+		    ptr = TtaConvertMbsToByte (css->url,
+					      TtaGetDefaultCharset ());
+		}
 	      len = strlen (ptr);
 	      len++;
 	      if (size < len)
 		break;
 	      strcpy (&buf[index], ptr);
+	      TtaFreeMemory (ptr);
 	      index += len;
 	      nb++;
 	      size -= len;
 	      if (select == -1 &&
 		  (CSScase < 4 || css->category == CSS_EXTERNAL_STYLE))
 		{
-		  if (CSSpath != NULL)
+		  if (CSSpath)
 		    TtaFreeMemory (CSSpath);
 		  if (css->category == CSS_DOCUMENT_STYLE)
 		    CSSpath = TtaStrdup (TtaGetMessage (AMAYA, AM_LOCAL_CSS));
@@ -610,7 +629,7 @@ static void         InitCSSDialog (Document doc, char *s)
       if (nb >= 10)
 	i = 10;
       else
-	i = nb;
+	i = nb + 1;
       TtaNewSizedSelector (BaseCSS + CSSSelect, BaseCSS + CSSForm,
 			   TtaGetMessage (AMAYA, AM_CSS_FILE),
 			   nb, buf, 200, i, NULL, FALSE, TRUE);
