@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 2000
+ *  (c) COPYRIGHT MIT and INRIA, 2000-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -29,8 +29,12 @@
 #define THOT_EXPORT extern
 #include "amaya.h"
 #include "MathML.h"
-#include "GraphML.h"
+#include "SVG.h"
 #include "XLink.h"
+#include "init_f.h"
+#ifdef ANNOTATIONS
+#include "Annot.h"
+#endif /* ANNOTATIONS */
 #include "XPointer.h"
 #include "XPointerparse_f.h"
 #undef THOT_EXPORT
@@ -50,7 +54,9 @@ struct _XPathItem {
 typedef XPathItem * XPathList;
 
 /* the thotlib element type used to identify a text node */
-#define THOT_TEXT_UNIT  1
+#define THOT_TEXT_UNIT   1
+#define THOT_GRAPH_UNIT  2
+#define THOT_SYMBOL_UNIT 3
 
 typedef enum _selMode {
   SEL_START_POINT=1,
@@ -59,6 +65,17 @@ typedef enum _selMode {
   SEL_RANGE_TO=64
 } selMode;
 
+typedef enum _selType {
+  SEL_FIRST_EL=0,
+  SEL_LAST_EL=1
+} selType;
+
+static char * xptr_buffer; /* temporary buffer where the user may store
+			      an XPointer */
+
+static Element RootElement; /* the root of the tree we're parsing (so that
+			    we can stop the parsing */
+
 /*----------------------------------------------------------------------
   StrACat
 
@@ -91,58 +108,68 @@ static void StrACat (char ** dest, const char * src)
     }
 }
 
-/*----------------------------------------------------------------------
-  AdjustSelMode
-
-  Makes sure that the index to a text element doesn't point outside
-  of the length of the text.
-  Returns the adjusted index, if any.
-  ----------------------------------------------------------------------*/
-static void AdjustSelMode (Element el, int *start, int index, selMode *mode)
-{
-  int len;
-
-  len = TtaGetTextLength (el);
-  if (*start > 0)
-    {
-      *mode |= SEL_STRING_RANGE;
-      if (*start > len)
-	{
-	  *mode |= SEL_END_POINT;
-	  *start = len;
-	}
-      else if (*start > index)
-	*mode |= SEL_START_POINT;
-    }
-}
-
 /*----------------------------------------------------------------------
   CountInLineChars
 
   returns the number of characters that may be found in inline
   sibling elements from element Mark, such as in <p>an 
   <strong>inline</strong> example</p>.
+  If any inline sibling elements are found, the Mark element is updated
+  to point to its parent.
   ----------------------------------------------------------------------*/
-static int CountInlineChars (Element mark)
+static int CountInlineChars (Element *mark, int firstCh, selMode *mode)
 {
   ElementType elType;
   Element el;
+  Element parent;
   int count = 0;
 
-  elType = TtaGetElementType (mark);
+  if (*mark == NULL)
+    return 0;
+
+  elType = TtaGetElementType (*mark);
 
   if (elType.ElTypeNum != THOT_TEXT_UNIT)
     return 0;
 
-  el = TtaGetParent (mark);
+  parent = TtaGetParent (*mark);
+  el = parent;
   while (1)
     {
       el = TtaSearchTypedElement (elType, SearchForward, el);
-      if (el == mark)
+      if (el == *mark)
 	break;
       count += TtaGetTextLength (el);
     }
 
+  if (count > 0) 
+    {
+
+      /* if we were at an empty end text element, but there were some adjacent
+	 text sibling elements, adjust the selection mode flag */
+      if (firstCh == 0)
+	{
+	  *mode |=  SEL_STRING_RANGE;
+	  /* START or END point? */
+	  el = *mark;
+	  TtaNextSibling (&el);
+	  if (el)
+	    {
+	      elType = TtaGetElementType (el);
+	      if (elType.ElTypeNum == THOT_TEXT_UNIT)
+		*mode |= SEL_START_POINT;
+	      else
+		*mode |= SEL_END_POINT;
+	    }
+	  else
+	    *mode |= SEL_END_POINT;
+	}
+
+      /* now point to the parent */
+      /* @@ JK: if count? */
+      *mark = parent;
+    }
+
   return count;
 }
 
@@ -155,11 +182,139 @@ static int CountInlineChars (Element mark)
 static ThotBool ElIsHidden (Element el)
 {
   ElementType elType;
+  ThotBool result;
 
   if (!el)
     return FALSE;
+
   elType = TtaGetElementType (el);
-  return (TtaHasHiddenException (elType));
+
+  result = TtaHasHiddenException (elType);
+
+  /* ignore all elements that were added by Amaya (they don't have
+     a line number */
+  if (!result)
+    result = (TtaGetElementLineNumber (el) == 0);
+
+
+  return result;
+}
+
+/*----------------------------------------------------------------------
+  ElIsXLink
+
+  returns TRUE if the element is our special Xlink element, false
+  otherwise/
+  ----------------------------------------------------------------------*/
+static ThotBool ElIsXLink (Element el)
+{
+  ElementType elType;
+  char *schema_name;
+
+  if (!el)
+    return FALSE;
+
+  elType = TtaGetElementType (el);
+  schema_name = TtaGetSSchemaName (elType.ElSSchema);
+
+  if (!strcmp (schema_name, "XLink")
+      && elType.ElTypeNum == XLink_EL_XLink)
+    return TRUE;
+  else
+    return FALSE;
+}
+
+/*----------------------------------------------------------------------
+  AdjustSelMode
+
+  Makes sure that the index to a text element doesn't point outside
+  of the length of the text.
+  If we had selected an XLink element, we chose a correct element and
+  selection mode.
+  Returns the adjusted index, if any.
+  ----------------------------------------------------------------------*/
+static void AdjustSelMode (Element *el, int *start, int index, selMode *mode, selType type)
+{
+  int len;
+  ElementType elType;
+  Element tmp_el;
+
+  /* skip the hidden XLInk element */
+  if (ElIsXLink (*el))
+    {
+      tmp_el = *el;
+      if (type == SEL_FIRST_EL)
+	{
+	  do
+	    {
+	      TtaNextSibling (&tmp_el);
+	    }
+	  while (tmp_el && ElIsXLink (tmp_el));
+	  if (tmp_el)
+	    {
+	      /* we use the first non XLink sibling. If it's of type
+	         text, we use 1 as the value of start */
+	      elType = TtaGetElementType (tmp_el);
+	      if (elType.ElTypeNum == THOT_TEXT_UNIT)
+		*start = 1;
+	    }
+	  else
+	    {
+	      /* we use the parent */
+	      tmp_el = TtaGetParent (*el);
+	      *start = 0;
+	    }
+	}
+      else
+	{
+	  do 
+	    {
+	      TtaPreviousSibling (&tmp_el);
+	    } 
+	  while (tmp_el && ElIsXLink (tmp_el));
+
+	  if (tmp_el)
+	    {
+	      /* we use the precedent non-xlink sibling. If it's of
+		 type text, we give start the value of the last char
+		 of that sibling */
+	      elType = TtaGetElementType (tmp_el);
+	      if (elType.ElTypeNum == THOT_TEXT_UNIT)
+		{
+		  len = TtaGetTextLength (tmp_el);
+		  *start = len + 1;
+		}
+	    }
+	  else
+	    {
+	      /* we use the parent */
+	      tmp_el = TtaGetParent (*el);
+	      *start = 0;
+	    }
+	}
+      *el = tmp_el;
+    }
+	  
+  if (*start > 0)
+    {
+      len = TtaGetTextLength (*el);
+      *mode |= SEL_STRING_RANGE;
+      if (*start > len)
+	{
+	  *mode |= SEL_END_POINT;
+	  *start = len;
+	}
+      else if (*start > index)
+	{
+	  if (type == SEL_FIRST_EL)
+	    *mode |= SEL_START_POINT;
+	  else
+	    {
+	      *mode |= SEL_END_POINT;
+	      *start = *start - 1;
+	    }
+	}
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -181,14 +336,14 @@ static char * GetIdValue (Element el)
   elType = TtaGetElementType (el);
   attrType.AttrSSchema = elType.ElSSchema;
   schema_name = TtaGetSSchemaName (elType.ElSSchema);
-  if (!ustrcmp (schema_name, TEXT("XLink")))
+  if (!strcmp (schema_name, "XLink"))
     /* ignore all XLink elements (they are only annotation
        related, and invisible to the document */
     return NULL;
-  else if (!ustrcmp (schema_name, TEXT("MathML")))
+  else if (!strcmp (schema_name, "MathML"))
     attrType.AttrTypeNum = MathML_ATTR_id;
-  else if (!ustrcmp (schema_name, TEXT("GraphML")))
-    attrType.AttrTypeNum = GraphML_ATTR_id;
+  else if (!strcmp (schema_name, "SVG"))
+    attrType.AttrTypeNum = SVG_ATTR_id;
   else
     attrType.AttrTypeNum = HTML_ATTR_ID;
 
@@ -197,7 +352,7 @@ static char * GetIdValue (Element el)
     {
       /* there's an ID attribute */
       len = TtaGetTextAttributeLength (attr) + 1;
-      value = TtaAllocString (len);
+      value = TtaGetMemory (len);
       TtaGiveTextAttributeValue (attr, value, &len);
       
     }
@@ -250,6 +405,37 @@ static ThotBool TestElName (Element el, char *name)
     return FALSE;
 }
 
+/*----------------------------------------------------------------------
+  AGetRootElement
+
+  returns the first root element of the document.
+  ----------------------------------------------------------------------*/
+Element AGetRootElement (Document doc)
+{
+  Element el;
+
+  if (doc == 0)
+    return NULL;
+
+  el = TtaGetRootElement (doc);
+
+#ifdef ANNOTATIONS
+  if (DocumentTypes[doc] == docAnnot)
+    {
+      ElementType elType;
+
+      /* use the first child of the Annotation body as the root element */
+      elType = TtaGetElementType (el);
+      elType.ElTypeNum = Annot_EL_Body;
+      el = TtaSearchTypedElement (elType, SearchInTree, 
+				   el);
+      if (el)
+	el = TtaGetFirstChild (el);
+    }
+#endif /* ANNOTATIONS */
+  return el;
+}
+
 /*----------------------------------------------------------------------
   AGetParent
 
@@ -260,7 +446,8 @@ static Element AGetParent (Element el)
 {
   Element parent;
 
-  if (!el)
+  /* stop if we're already at the Root Element */
+  if (!el || el == RootElement)
     return NULL;
 
   parent = el;
@@ -270,6 +457,11 @@ static Element AGetParent (Element el)
     }
   while (parent && ElIsHidden (parent));
 
+#if 0
+  if (parent  && !TtaGetParent (parent))
+    parent = NULL;
+#endif
+
   return parent;
 }
 
@@ -281,15 +473,16 @@ static Element AGetParent (Element el)
   Because of the way that the Thot tree is built, we need a special
   algorithm to get the childs we would find in a DOM tree (proposed by
   VQ):
-  starting from an element, we try to find the first brother.
-  If there are no brother, we get the parent. If the parent is not
-  hidden, there are no other brothers.
+  starting from an element, we try to find the first sibling.
+  If there are no siblings, we get the parent. If the parent is not
+  hidden, there are no other brothers, and we return NULL.
   If the parent is hidden, then we try to get the brothers of this parent.
   For each brother of the parent, we try to get the last child.
   ----------------------------------------------------------------------*/
 static void PreviousSibling (Element *el)
 {
   Element sibling;
+  Element tmp_el;
 
   if (!el || !*el)
     return;
@@ -301,7 +494,16 @@ static void PreviousSibling (Element *el)
     {
       /* if the element is hidden, return the latest child */
       if (ElIsHidden (sibling))
-	  sibling = TtaGetLastChild (sibling);
+	{
+	  tmp_el = TtaGetLastChild (sibling);
+	  if (!tmp_el)
+	    {
+	      /* there was no child, let's try the next brother */
+	      PreviousSibling (&sibling);
+	    }
+	  else
+	    sibling = tmp_el;
+	}
       *el = sibling;
     }
   else
@@ -310,13 +512,13 @@ static void PreviousSibling (Element *el)
 	 on each parent, element until we find a child or the
 	 the first non-hidden parent */
       sibling = TtaGetParent (*el);
-      if (ElIsHidden (sibling))
-	{
+
+      /* we only continue searching if the parent is hidden */
+      if (sibling && ElIsHidden (sibling))
 	  PreviousSibling (&sibling);
-	  *el = sibling;
-	}
       else
-	*el = NULL;
+	sibling = NULL;
+      *el = sibling;
     }
   return;
 }
@@ -346,11 +548,12 @@ Element SearchAttrId (Element root, char *val)
       
       /* recursive search all the children of root */
       el = TtaGetFirstChild (root);
-      if (!el)
-	break;
-      result = SearchAttrId (el, val);
-      if (result)
-	break;
+      if (el)
+	{
+	  result = SearchAttrId (el, val);
+	  if (result)
+	    break;
+	}
       /* try the same procedure on all the siblings of root */
       TtaNextSibling (&root);
     }
@@ -380,20 +583,23 @@ Element SearchSiblingIndex (Element root, char *el_name, int *index)
     {
       /* if the element is hidden, call the algorithm recursively from 
 	 this point */
-      if (ElIsHidden (sibling))
-	  {
-	    child = TtaGetFirstChild (sibling);
-	    result = SearchSiblingIndex (child, el_name, index);
-	    if (result)
-	      return result;
-	  }
-      /* test the current node */
-      else if (TestElName (sibling, el_name))
+      if (!TestElName (sibling, "Annotation")) /* ignore our XLink element */
 	{
-	  /* we found the element */
-	  if (*index == 1)
-	    return sibling;
-	  (*index)--;
+	  if (ElIsHidden (sibling))
+	    {
+	      child = TtaGetFirstChild (sibling);
+	      result = SearchSiblingIndex (child, el_name, index);
+	      if (result)
+		return result;
+	    }
+	  /* test the current node */
+	  else if (TestElName (sibling, el_name))
+	    {
+	      /* we found the element */
+	      if (*index == 1)
+		return sibling;
+	      (*index)--;
+	    }
 	}
       /* go to the next sibling */
       TtaNextSibling (&sibling);
@@ -519,15 +725,7 @@ static char * XPathList2Str (XPathList *xpath_list, int firstCh, int len, int mo
   It's up to the caller to free the returned string.
   Returns NULL in case of failure.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static char *XPointer_ThotEl2XPath (Element start, int firstCh, int len, selMode mode, ThotBool firstF)
-#else
-static char *XPointer_ThotEl2XPath (start, firstCh, selMode mode, len, firstF)
-Element start;
-int firstCh;
-int len;
-ThotBool firstF;
-#endif /* __STDC__ */
 {
   Element el, prev;
   ElementType elType, prevElType;
@@ -539,13 +737,14 @@ ThotBool firstF;
 
   /* if the user selected a text, adjust the start/end indexes according
      to its siblings inlined text */
-  firstCh += CountInlineChars (start);
+  firstCh += CountInlineChars (&start, firstCh, &mode);
 
   el = start;
-  /* if we chose a hidden element, climb up */
-  if (ElIsHidden (el))
-    el = AGetParent (el);
-  
+  elType = TtaGetElementType (el);
+  /* if we chose a hidden element or a GRAPH UNIT (SVG), climb up */
+  if (ElIsHidden (el) || elType.ElTypeNum == THOT_GRAPH_UNIT)
+      el = AGetParent (el);
+
   /* browse the tree */
   while (el)
     {
@@ -604,13 +803,7 @@ ThotBool firstF;
   N.B., the view parameter isn't used, but it's included to be coherent
   with the function API.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 char * XPointer_build (Document doc, View view, ThotBool useDocRoot)
-#else
-char * XPointer_build (doc, view, useDocRoot)
-Document doc;
-View view;
-#endif
 {
   Element     firstEl, lastEl;
 
@@ -621,9 +814,10 @@ View view;
   char       *lastXpath = NULL;
   ElementType elType;
 
-  CHAR_T     *schemaName;
+  char     *schemaName;
 
-  selMode    mode = 0;
+  selMode    firstMode = 0;
+  selMode    lastMode = 0;
 
   /* @@ debug */
   char       *xptr_expr = NULL;
@@ -632,23 +826,24 @@ View view;
   /* only do this operation on XML and HTML documents */
   /* @@ JK: should be a function in AHTURLTools */
   schemaName = TtaGetSSchemaName (elType.ElSSchema);
-  if (ustrcmp(schemaName, TEXT("HTML"))
-      && ustrcmp(schemaName, TEXT("XHTML"))
-      && ustrcmp(schemaName, TEXT("XML"))
-      && ustrcmp(schemaName, TEXT("MathML"))
-      && ustrcmp(schemaName, TEXT("GraphML")))
+  if (strcmp(schemaName, "HTML")
+      && strcmp(schemaName, "XHTML")
+      && strcmp(schemaName, "XML")
+      && strcmp(schemaName, "MathML")
+      && strcmp(schemaName, "SVG")
+      && strcmp(schemaName, "Annot"))
     return NULL;
 
   /* is the document selected? */
   if (useDocRoot)
     {
-      firstEl = TtaGetMainRoot (doc);
+      firstEl = AGetRootElement (doc);
       if (!firstEl)
 	return NULL; /* something went wrong */
       firstLen = 0;
       firstCh = 0;
       lastEl = NULL;
-      mode  = mode & ~(SEL_START_POINT | SEL_END_POINT);
+      firstMode  = firstMode & ~(SEL_START_POINT | SEL_END_POINT);
     }
   else
     {
@@ -661,11 +856,19 @@ View view;
 #ifdef DEBUG_XPOINTER
       printf ("first Ch is %d, i is %d\n", firstCh, i);
 #endif 
-      AdjustSelMode (firstEl, &firstCh, i, &mode);
+      AdjustSelMode (&firstEl, &firstCh, i, &firstMode, SEL_FIRST_EL);
       
       if (firstEl == NULL)
 	return NULL; /* ERROR, there is no selection */
 
+      if  (TtaGetElementLineNumber (firstEl) == 0)
+	{
+	  /* ERROR, don't annotate end elements that Amaya added
+	     internally, but that don't exist in the document */
+	  InitInfo ("XPointer", TtaGetMessage (AMAYA, AM_NO_SXPOINTER));
+	  return NULL;
+	}
+
       /* is it a caret or an extension selection? */
       if (TtaIsSelectionEmpty ())
 	lastEl = NULL;
@@ -675,30 +878,65 @@ View view;
 #ifdef DEBUG_XPOINTER
 	  printf ("last Ch is %d, i is %d\n", lastCh, i);
 #endif 
-	  AdjustSelMode (lastEl, &lastCh, i, &mode);
+	  AdjustSelMode (&lastEl, &lastCh, i, &lastMode, SEL_LAST_EL);
+
+	  if  (TtaGetElementLineNumber (lastEl) == 0)
+	    {
+	      /* ERROR, don't annotate end elements that Amaya added
+		 internally, but that don't exist in the document */
+	      InitInfo ("XPointer", TtaGetMessage (AMAYA, AM_NO_EXPOINTER));
+	      return NULL;
+	    }
+
 	}
+
       /* if the selection is in the same element, adjust the first element's
 	 length */
       if (firstEl == lastEl)
 	{
-	  firstLen = lastCh - firstCh + 1;
+	  /* if we have an empty selection, ignore the last element and its data */
+	  if (lastCh != firstCh)
+	    {
+	      firstLen = lastCh - firstCh + 1;
+	      firstMode  = firstMode & ~(SEL_START_POINT | SEL_END_POINT);
+	    }
+	  else  /* the selection is only a caret */
+	    {
+	      firstLen = 1;
+	    }
 	  lastEl = NULL;
-	  mode  = mode & ~(SEL_START_POINT | SEL_END_POINT);
 	}
       else
 	{
 	  firstLen = 1;
-	  mode |= SEL_RANGE_TO;
+	  firstMode |= SEL_RANGE_TO;
+	  lastMode |= SEL_RANGE_TO;
 	}
     }
 
-  firstXpath = XPointer_ThotEl2XPath (firstEl, firstCh, firstLen, mode, TRUE);
+  /* @@ JK: We don't know yet how to handle annotations on symbols, so we just
+     forbid making XPointers on them, for the moment @@ */
+  elType = TtaGetElementType (firstEl);
+  if (elType.ElTypeNum == THOT_SYMBOL_UNIT)
+    return NULL;
+  if (lastEl)
+    {
+      elType = TtaGetElementType (lastEl);
+      if (elType.ElTypeNum == THOT_SYMBOL_UNIT)
+	return NULL;
+    }
+
+  /* remember the root of the tree we are annotating, so that we can stop
+     walking the tree when we reach it */
+  RootElement = AGetRootElement (doc);
+
+  firstXpath = XPointer_ThotEl2XPath (firstEl, firstCh, firstLen, firstMode, TRUE);
 #ifdef DEBUG_XPOINTER
   fprintf (stderr, "\nfirst xpointer is %s", firstXpath);
 #endif  
   if (lastEl)
     {
-      lastXpath = XPointer_ThotEl2XPath (lastEl, lastCh, 1, mode, FALSE);
+      lastXpath = XPointer_ThotEl2XPath (lastEl, lastCh, 1, lastMode, FALSE);
 #ifdef DEBUG_XPOINTER
       fprintf (stderr, "\nlast xpointer is %s\n", lastXpath);
 #endif  
@@ -711,15 +949,18 @@ View view;
     }
 
   /* calculate the length of the xptr buffer */
-  i = sizeof ("xpointer()/range-to()") + strlen (firstXpath) 
-	      + ((lastEl) ? strlen (lastXpath) : 0) + 1;
-  xptr_expr = TtaGetMemory (i);
-
-  if (lastEl)
-      sprintf (xptr_expr, "xpointer(%s/range-to(%s))", firstXpath, lastXpath);
-  else
-    sprintf (xptr_expr, "xpointer(%s)", firstXpath);
-
+  if (firstXpath)
+    {
+      i = sizeof ("xpointer()/range-to()") + strlen (firstXpath) 
+	+ ((lastEl) ? strlen (lastXpath) : 0) + 1;
+      xptr_expr = TtaGetMemory (i);
+      
+      if (lastEl)
+	sprintf (xptr_expr, "xpointer(%s/range-to(%s))", firstXpath, lastXpath);
+      else
+	sprintf (xptr_expr, "xpointer(%s)", firstXpath);
+    }
+  
   TtaFreeMemory (firstXpath);
   if (lastEl)
     TtaFreeMemory (lastXpath);
@@ -740,5 +981,51 @@ View view;
 }
 
 
+/*----------------------------------------------------------------------
+  XPointer_bufferStore
+  Stores into a local buffer the full XPointer corresponding to the
+  current selection or caret position.
+  ----------------------------------------------------------------------*/
+void XPointer_bufferStore (Document doc, View view)
+{
+  char *xptr;
+
+  /* we need to make a full URL */
+  if (!DocumentURLs[doc] || *DocumentURLs[doc] == EOS)
+    return;
+
+  /* free the precedent buffer */
+  XPointer_bufferFree ();
+  /* build  the XPointer */
+  xptr = XPointer_build (doc, view, FALSE);
+  /* make a full URI */
+  if (xptr)
+    {
+      xptr_buffer = TtaGetMemory (strlen (DocumentURLs[doc]) + strlen (xptr) + 2);
+      sprintf (xptr_buffer, "%s#%s", DocumentURLs[doc], xptr);
+    }
+}
+
+/*----------------------------------------------------------------------
+  XPointer_buffer
+  Returns the buffer that may contain a stored XPointer. The buffer
+  may be empty.
+  ----------------------------------------------------------------------*/
+char * XPointer_buffer (void)
+{
+  return xptr_buffer;
+}
 
 
+/*----------------------------------------------------------------------
+  XPointer_bufferFree
+  Frees the memory associated to the local buffer
+  ----------------------------------------------------------------------*/
+void XPointer_bufferFree (void)
+{
+  if (xptr_buffer)
+    {
+      TtaFreeMemory (xptr_buffer);
+      xptr_buffer = NULL;
+    }
+}
