@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 2000
+ *  (c) COPYRIGHT MIT and INRIA, 2000-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -29,8 +29,9 @@
 #include <string.h>
 
 
+/* To turn on debugging */
+/* #define XPTR_ACTION_DEBUG */
 /* #define XPTR_PARSE_DEBUG */
-#undef XPTR_ACTION_DEBUG
 
 #define BSIZE 128       /* size of the lexical analyzer temporary buffer */
 
@@ -128,7 +129,24 @@ static void AddChild (XPointerContextPtr ctx, char *node)
 
 #ifdef XPTR_ACTION_DEBUG
   printf ("Adding child %s\n", node);
-#endif	  
+#endif
+
+  if ((unsigned) curNode->nodeSize < strlen (node) + 1)
+    {
+      if (curNode->node == NULL)
+	{
+	  curNode->nodeSize = 30 + strlen (node) * 2 + 1;
+	  curNode->node = (char *) TtaGetMemory (sizeof (char) * curNode->nodeSize);
+	}
+      else
+	{
+	  curNode->nodeSize = curNode->nodeSize + strlen (node) * 2 + 1;
+	  /* we do a free, rather than a calloc because we're not interested
+	     in keeping whatever was stored in this buffer */
+	  TtaFreeMemory (curNode->node);
+	  curNode->node =  (char *) TtaGetMemory (sizeof (char) * (curNode->nodeSize));
+	}
+    }
   strcpy (curNode->node, node);
 }
 
@@ -154,7 +172,7 @@ static void GotoChild (XPointerContextPtr ctx)
       return;
     }
 
-  if (curNode->node[0])
+  if (curNode->node && curNode->node[0])
     {
 #ifdef XPTR_ACTION_DEBUG
       printf ("Going to child: %s, index %d\n", curNode->node, curNode->index);
@@ -243,9 +261,9 @@ static void StringRange (XPointerContextPtr ctx, int startC, int len)
 	  curNode->processed = 1;
 	  curNode->startC = pos;
 	  if (len > 1)
-	    curNode->endC = pos + len - 1;
+	    curNode->endC = pos + len;
 	  else
-	    curNode->endC = curNode->startC;
+	    curNode->endC = curNode->startC - 1;
 	  curNode->type |= STRING_RANGE;
 	}
     }
@@ -261,7 +279,7 @@ static void StartPoint (XPointerContextPtr ctx)
   nodeInfo *curNode = ctx->curNode;
 
   curNode->type |= START_POINT;
-  curNode->endC = curNode->startC -1;
+  curNode->endC = curNode->startC - 1;
 }
 
 static void EndPoint (XPointerContextPtr ctx)
@@ -270,7 +288,7 @@ static void EndPoint (XPointerContextPtr ctx)
 
   curNode->type |= END_POINT;
   curNode->startC++;
-  curNode->endC = curNode->startC -1;
+  curNode->endC = curNode->startC;
 }
 
 /************************************************** 
@@ -659,7 +677,12 @@ static void InitSymtable (XPointerContextPtr ctx)
 
 /**************************************************
   PUBLIC API 
-**************************************************/						  
+**************************************************/
+
+/*----------------------------------------------------------------------
+  SelectNode
+  Initialize a selection
+  ----------------------------------------------------------------------*/
 static void SelectNode (Document doc, nodeInfo *node)
 {
   if (node->type & STRING_RANGE)
@@ -668,6 +691,10 @@ static void SelectNode (Document doc, nodeInfo *node)
     TtaSelectElement (doc, node->el);
 }
 
+/*----------------------------------------------------------------------
+  SelectToNode
+  Extend a selection
+  ----------------------------------------------------------------------*/
 static void SelectToNode (Document doc, nodeInfo *node)
 {
   TtaExtendSelection (doc, node->el, node->endC);
@@ -686,7 +713,7 @@ void XPointer_select (XPointerContextPtr ctx)
   if (TtaGetSelectedDocument () == ctx->doc)
     TtaUnselect (ctx->doc);
 
-  root = TtaGetMainRoot (ctx->doc);
+  root = TtaGetRootElement (ctx->doc);
   /* only do the selection if it's not the root */
   if (ctx->nodeStart.el != root)
     {
@@ -706,6 +733,11 @@ void XPointer_free (XPointerContextPtr ctx)
   
   if (ctx->error)
     TtaFreeMemory (ctx->error);
+  if (ctx->nodeStart.node)
+    TtaFreeMemory (ctx->nodeStart.node);
+  if (ctx->nodeEnd.node)
+    TtaFreeMemory (ctx->nodeEnd.node);
+
   TtaFreeMemory (ctx);
 }
 
@@ -828,7 +860,7 @@ XPointerContextPtr XPointer_parse (Document doc, char *buffer)
   context->curNode = &(context->nodeStart);
   /* and initialize the document root */
   context->doc = doc;
-  context->root = TtaGetMainRoot (doc);
+  context->root = AGetRootElement (doc);
 
   /* start parsing */
   context->lookahead = LexAn (context);
