@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -28,19 +28,43 @@
 #include "styleparser_f.h"
 
 
+/*----------------------------------------------------------------------
+  ReallocUTF8String
+  If such convertion is needed, the string url is reallocated with
+  the converted string.
+  ----------------------------------------------------------------------*/
+char *ReallocUTF8String (char *url, Document doc)
+{
+#ifndef _I18N_
+  unsigned char *tmp;
+
+  if (!url || *url == EOS)
+    return NULL;
+  /* does the URL contain chars > 127 ? */
+  tmp = url;
+  while (*tmp)
+    {
+      if (*tmp > 127)
+        break;
+      tmp++;
+    }
+  if (*tmp == EOS)
+    return url;  /* no such chars found */
+  tmp = TtaConvertByteToMbs (url, TtaGetDocumentCharset (doc));
+  if (tmp)
+    TtaFreeMemory (url);
+  return tmp;
+#else /* _I18N_ */
+  return url;
+#endif /* _I18N_ */
+}
+
 /*----------------------------------------------------------------------
   GetPExtension returns the Presentation Extension Schema associated with
   the document doc and the structure sSchema
   At the same time, this funciton updates the css context.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-PSchema         GetPExtension (Document doc, SSchema sSchema, CSSInfoPtr css)
-#else
-PSchema         GetPExtension (doc, sSchema, css)
-Document        doc;
-SSchema         sSchema;
-CSSInfoPtr      css;
-#endif
+PSchema GetPExtension (Document doc, SSchema sSchema, CSSInfoPtr css)
 {
   CSSInfoPtr          oldcss;
   PInfoPtr            pInfo;
@@ -50,13 +74,14 @@ CSSInfoPtr      css;
   ElementType	      elType;
   AttributeType       attrType;
   Attribute           attr;
-  CHAR_T              buffer[MAX_LENGTH];
+  char                buffer[MAX_LENGTH];
   int                 length;
   ThotBool            found, before;
 
   if (sSchema == NULL)
     sSchema = TtaGetDocumentSSchema (doc);
   pInfo = css->infos;
+  nextLink = NULL;
   found = FALSE;
   pIS = NULL;
   while (pInfo != NULL && !found)
@@ -103,7 +128,8 @@ CSSInfoPtr      css;
     }
 
   /* create the presentation schema for this structure */
-  nSchema = TtaNewPSchema ();
+  nSchema = TtaNewPSchema (sSchema, css->category == CSS_USER_STYLE,
+			   css->refEl);
   pSchema = TtaGetFirstPSchema (doc, sSchema);
   pIS->PiPSchema = nSchema;
   /* chain the presentation schema at the right position */
@@ -149,7 +175,7 @@ CSSInfoPtr      css;
 		      /* get a buffer for the attribute value */
 		      length = MAX_LENGTH;
 		      TtaGiveTextAttributeValue (attr, buffer, &length);
-		      found = (!ustrcasecmp (buffer, TEXT("STYLESHEET")) || !ustrcasecmp (buffer, TEXT("STYLE")));
+		      found = (!strcasecmp (buffer, "STYLESHEET") || !strcasecmp (buffer, "STYLE"));
 		    }
 		  if (found)
 		    {
@@ -206,7 +232,7 @@ CSSInfoPtr      css;
 			  /* get a buffer for the attribute value */
 			  length = MAX_LENGTH;
 			  TtaGiveTextAttributeValue (attr, buffer, &length);
-			  found = (!ustrcasecmp (buffer, TEXT("STYLESHEET")) || !ustrcasecmp (buffer, TEXT("STYLE")));
+			  found = (!strcasecmp (buffer, "STYLESHEET") || !strcasecmp (buffer, "STYLE"));
 			}
 		      /* search if the previous CSS has a presentation schema */
 		      if (found)
@@ -325,16 +351,8 @@ CSSInfoPtr      css;
 /*----------------------------------------------------------------------
    AddCSS adds a new CSS context in the list.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-CSSInfoPtr      AddCSS (Document doc, Document docRef, CSSCategory category, STRING url, STRING localName)
-#else
-CSSInfoPtr      AddCSS (doc, docRef, category, url, localName)
-Document        doc;
-Document        docRef;
-CSSCategory     category;
-STRING          url;
-STRING          localName;
-#endif
+CSSInfoPtr AddCSS (Document doc, Document docRef, CSSCategory category,
+		   char *url, char *localName, Element styleElement)
 {
   CSSInfoPtr          css, prev;
   int                 i;
@@ -343,8 +361,10 @@ STRING          localName;
   if (css != NULL)
     {
       css->doc = doc;
-      css->localName = TtaWCSdup (localName);
-      css->url = TtaWCSdup (url);
+      css->url = TtaStrdup (url);
+      css->localName = TtaStrdup (localName);
+      css->styleEl = styleElement;
+      css->refEl = NULL;
       css->category = category;
 
       /* that CSS is only used by the document docRef */
@@ -378,27 +398,26 @@ STRING          localName;
    SearchCSS searchs the css corresponding to the specific url (doc == 0)
    or the CSS_DOCUMENT_STYLE css of the document.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-CSSInfoPtr          SearchCSS (Document doc, STRING url)
-#else
-CSSInfoPtr          SearchCSS (doc, url)
-Document            doc;
-STRING               url;
-#endif
+CSSInfoPtr SearchCSS (Document doc, char *url, Element styleEl)
 {
   CSSInfoPtr          css = CSSList;
  
   while (css != NULL)
     {
-      if (url && ((css->url && !ustrcmp (url, css->url)) ||
-		  (css->localName && !ustrcmp (url, css->localName))))
+      if (url && ((css->url && !strcmp (url, css->url)) ||
+		  (css->localName && !strcmp (url, css->localName))))
 	/* an external CSS */
 	return css;
       else if (doc != 0 && css->doc == doc)
 	/* a document CSS */
-	return css;
-      else
-	css = css->NextCSS;
+	{
+	  if (!styleEl)
+	    /* any document CSS is ok */
+	    return css;
+	  else if (css->styleEl == styleEl)
+	    return css;
+	}
+      css = css->NextCSS;
     }
   return css;
 }
@@ -410,15 +429,8 @@ STRING               url;
    If this CSS is no longer used the context and attached information
    are freed.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void     UnlinkCSS (CSSInfoPtr css, Document doc, ThotBool disabled, ThotBool removed)
-#else
-static void     UnlinkCSS (css, doc, disabled, removed)
-CSSInfoPtr      css;
-Document        doc;
-ThotBool        disabled;
-ThotBool        removed;
-#endif
+static void UnlinkCSS (CSSInfoPtr css, Document doc, ThotBool disabled,
+		       ThotBool removed)
 {
   CSSInfoPtr          prev;
   PInfoPtr            pInfo, prevInfo;
@@ -457,10 +469,10 @@ ThotBool        removed;
 		pIS = pInfo->PiSchemas;
 		if (pIS->PiPSchema)
 		  {
-		    TtaUnlinkPSchema (pIS->PiPSchema, pInfo->PiDoc, pIS->PiSSchema);
-		    TtaCleanStylePresentation (NULL, pIS->PiPSchema, pInfo->PiDoc);
-		    /* remove presentation schemas */
-		    TtaRemovePSchema (pIS->PiPSchema, pInfo->PiDoc, pIS->PiSSchema);
+		    TtaCleanStylePresentation (pIS->PiPSchema, pInfo->PiDoc,
+					       pIS->PiSSchema);
+		    TtaUnlinkPSchema (pIS->PiPSchema, pInfo->PiDoc,
+				      pIS->PiSSchema);
 		    pInfo->PiSchemas = pIS->PiSNext;
 		    TtaFreeMemory (pIS);
 		  }
@@ -506,12 +518,7 @@ ThotBool        removed;
 /*----------------------------------------------------------------------
    RemoveDocCSSs removes all CSS information linked with the document.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                RemoveDocCSSs (Document doc)
-#else
-void                RemoveDocCSSs (doc)
-Document            doc;
-#endif
+void RemoveDocCSSs (Document doc)
 {
   CSSInfoPtr          css, next;
  
@@ -537,15 +544,8 @@ Document            doc;
    It could be an external CSS file linked with the document (url not NULL)
    or the document Style element.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void            RemoveStyleSheet (STRING url, Document doc, ThotBool disabled, ThotBool removed)
-#else
-void            RemoveStyleSheet (url, doc, disabled, removed)
-STRING          url;
-Document        doc;
-ThotBool        disabled;
-ThotBool        removed;
-#endif
+void  RemoveStyleSheet (char *url, Document doc, ThotBool disabled,
+			ThotBool removed, Element styleEl)
 {
   CSSInfoPtr          css;
   ThotBool            found;
@@ -554,11 +554,13 @@ ThotBool        removed;
   found = FALSE;
   while (css != NULL && !found)
     {
-      if (url && ((css->url && !ustrcmp (url, css->url)) ||
-		  (css->localName && !ustrcmp (url, css->localName))))
+      if (url && ((css->url && !strcmp (url, css->url)) ||
+		  (css->localName && !strcmp (url, css->localName))))
 	/* an external CSS */
 	found = TRUE;
-      else if (!url && css->category == CSS_DOCUMENT_STYLE && css->documents[doc])
+      else if (!url && css->category == CSS_DOCUMENT_STYLE &&
+	       css->documents[doc] &&
+	       (styleEl == NULL || css->styleEl == styleEl))
 	found = TRUE;
       else
 	css = css->NextCSS;
@@ -568,31 +570,26 @@ ThotBool        removed;
     UnlinkCSS (css, doc, disabled, removed);
 }
 
-
 /*----------------------------------------------------------------------
   GetStyleContents returns a buffer that contains the whole text of the
   style element el. It returns NULL if the element is empty.
   The buffer should be freed by the caller.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-STRING              GetStyleContents (Element el)
-#else
-STRING              GetStyleContents (el)
-Element             el;
-#endif
+char *GetStyleContents (Element el)
 {
   ElementType         elType;
   Element             text;
   Language            lang;
-  STRING              buffer;
+  char               *buffer;
   int                 length, i, j;
 
   buffer = NULL;
-  length = TtaGetElementVolume (el) + 1;
+  /* get enough space to store UTF-8 characters */
+  length = TtaGetElementVolume (el) * 6 + 1;
   if (length > 1)
     {
       /* get the length of the included text */
-      buffer = TtaAllocString (length);
+      buffer = TtaGetMemory (length);
 
       /* fill the buffer */
       elType = TtaGetElementType (el);
@@ -616,26 +613,19 @@ Element             el;
   The parameter el gives the element which links the CSS or NULL.
   The parameter css gives the CSS context which imports this CSS file.
   The parameter media gives the application limits of the CSS.
+  The parameter user is true when it's a User style sheet. It's false
+  when it's an authr style sheet
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void        LoadStyleSheet (STRING url, Document doc, Element el, CSSInfoPtr css, CSSmedia media)
-#else
-void        LoadStyleSheet (url, doc, el, css, media)
-STRING      url;
-Document    doc;
-Element     el;
-CSSInfoPtr  css;
-CSSmedia    media;
-#endif
+void LoadStyleSheet (char *url, Document doc, Element el, CSSInfoPtr css,
+		     CSSmedia media, ThotBool user)
 {
   CSSInfoPtr          oldcss;
   PInfoPtr            pInfo;
   struct stat         buf;
   FILE               *res;
-  CHAR_T              tempfile[MAX_LENGTH];
-  CHAR_T              tempURL[MAX_LENGTH];
-  CHAR_T*             buffer = NULL;
-  char*               tmpBuff;
+  char                tempfile[MAX_LENGTH];
+  char                tempURL[MAX_LENGTH];
+  char               *tmpBuff;
   int                 len;
   ThotBool            import, printing;
 
@@ -644,13 +634,19 @@ CSSmedia    media;
   if (TtaGetViewFrame (doc, 1) != 0 || TtaIsPrinting ())
     {
       LoadRemoteStyleSheet (url, doc, el, css, tempURL, tempfile);
-      oldcss = SearchCSS (0, tempURL);
+      oldcss = SearchCSS (0, tempURL, NULL);
       if (oldcss == NULL || oldcss->category != CSS_EXTERNAL_STYLE)
 	{
 	  /* It could be a @import CSS */
 	  if (css == NULL)
 	    /* It's a new CSS file: allocate a new Presentation structure */
-	    css = AddCSS (0, doc, CSS_EXTERNAL_STYLE, tempURL, tempfile);
+	    {
+	    if (user)
+	      css = AddCSS (0, doc, CSS_USER_STYLE, tempURL, tempfile, NULL);
+	    else
+	      css = AddCSS (0, doc, CSS_EXTERNAL_STYLE, tempURL, tempfile,
+			    NULL);
+	    }
 	  oldcss = css;
 	  oldcss->media[doc] = media;
 	}
@@ -700,7 +696,7 @@ CSSmedia    media;
       if ( pInfo->PiSchemas == NULL || import)
 	{
 	  /* load the resulting file in memory */
-	  res = ufopen (tempfile, TEXT("r"));
+	  res = fopen (tempfile, "r");
 	  if (res == NULL)
 	    {
 	      TtaSetStatus (doc, 1, TtaGetMessage (AMAYA, AM_CANNOT_LOAD), tempURL);
@@ -718,7 +714,7 @@ CSSmedia    media;
 		return;
 	      }
 
-      tmpBuff = TtaGetMemory (buf.st_size + 1000);
+	  tmpBuff = TtaGetMemory (buf.st_size + 1000);
 	  if (tmpBuff == NULL)
 	    {
 	      TtaSetStatus (doc, 1, TtaGetMessage (AMAYA, AM_CANNOT_LOAD), tempURL);
@@ -736,14 +732,8 @@ CSSmedia    media;
 	  tmpBuff[buf.st_size] = 0;
 	  fclose (res);
 
-#     ifdef _I18N_
-	  buffer = TtaAllocString (buf.st_size + 1000);
-      mbstowcs (buffer, tmpBuff, buf.st_size + 1000);
-#     else  /* !_I18N_ */
-      buffer = tmpBuff;
-#     endif /* !_I18N_ */
-	  ReadCSSRules (doc, oldcss, buffer, FALSE);
-	  TtaFreeMemory (buffer);
+	  ReadCSSRules (doc, oldcss, tmpBuff, tempURL, 0, FALSE, NULL, NULL);
+	  TtaFreeMemory (tmpBuff);
 	}
     }
 }
