@@ -257,6 +257,7 @@
 #define CHARSETDIALOG                   553
 #define SAVEASDIALOG2                   554
 #define INITCONFIRMDIALOG1              555
+#define BROWSEMENU                      556
 #define ID_CONFIRM                      1000
 #define ID_DONE                         1001
 #define IDC_URLEDIT                     1002
@@ -580,8 +581,8 @@
 #define IDC_BGIMAGES                    1424
 #define IDC_DOUBLECLICK                 1425
 #define IDC_ENABLEFTP                   1426
-#define IDC_SHOWTARGETS                 1427
-#define IDC_NUMBERS                     1428
+#define IDC_SHOWADDRESS                 1427
+#define IDC_SHOWTARGET                  1428
 #define IDC_CRLF                        1429
 #define IDC_LOSTUPDATECHECK             1430
 #define IDC_NOPROXY                     1430
@@ -590,6 +591,7 @@
 #define IDC_ENABLECACHE                 1433
 #define IDC_SAFEPUTREDIRECT             1433
 #define IDC_CACHEPROTECTEDDOCS          1434
+#define IDC_USEXHTMLMIMETYPE            1434
 #define IDC_CACHEDISCONNECTEDMODE       1435
 #define IDC_CACHEEXPIREIGNORE           1436
 #define IDC_CACHEDIRECTORY              1437
@@ -716,6 +718,9 @@
 #define IDC_UTF8                        1568
 #define IDC_ISOL1                       1569
 #define ID_SHOW                         1570
+#define IDC_LOADIMG                     1572
+#define IDC_SHOWBUTTONS                 1573
+#define IDC_NUMBER                      1574
 #define ID_BUTTON40017                  40017
 #define ID_BUTTON40018                  40018
 #define ID_BUTTON40019                  40019
@@ -762,7 +767,7 @@
 #ifndef APSTUDIO_READONLY_SYMBOLS
 #define _APS_NEXT_RESOURCE_VALUE        554
 #define _APS_NEXT_COMMAND_VALUE         40052
-#define _APS_NEXT_CONTROL_VALUE         1572
+#define _APS_NEXT_CONTROL_VALUE         1575
 #define _APS_NEXT_SYMED_VALUE           101
 #endif
 #endif
