@@ -1,6 +1,6 @@
 /*
  *
- *  COPYRIGHT MIT and INRIA, 1996.
+ *  COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -17,19 +17,14 @@
 #define THOT_EXPORT extern
 #include "amaya.h"
 #include "css.h"
-
 #include "document.h"
 #include "view.h"
 
 
 /* content of the attr name of the meta tag defining the doc's destination URL */
-#if defined(_I18N_) || defined(__JIS__)
-#   define META_TEMPLATE_NAME L"AMAYA_TEMPLATE"
-#else /* defined(_I18N_) || defined(__JIS__) */
-#     define META_TEMPLATE_NAME "AMAYA_TEMPLATE"
-#endif /* defined(_I18N_) || defined(__JIS__) */
+#define META_TEMPLATE_NAME "AMAYA_TEMPLATE"
 /* URL of the script providing templates (for reload) */
-static CHAR_T* script_URL;
+static char   *script_URL;
 
 #include "init_f.h"
 
@@ -37,33 +32,21 @@ static CHAR_T* script_URL;
 /*----------------------------------------------------------------------
    NewTemplate: Opens a template form on the remote template server
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void               NewTemplate (Document doc, View view)
-#else  /* __STDC__ */
-void               NewTemplate (doc, view)
-Document doc;
-View view;
-#endif /* __STDC__ */
-
+void NewTemplate (Document doc, View view)
 {
-  STRING    url;
+  char      *url;
 
   url = TtaGetEnvString ("TEMPLATE_URL");
   if (url)
-    GetHTMLDocument (url, NULL, 0, 0, CE_ABSOLUTE, FALSE, NULL, NULL);
+    GetAmayaDoc (url, NULL, 0, 0, CE_ABSOLUTE, FALSE, NULL, NULL,
+		 TtaGetDefaultCharset ());
 }
 /*----------------------------------------------------------------------
    OpenTemplateDocument: Process the meta of a template document,
      changes the URL, try to save it and warn the user if it cannot be
      saved.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void               OpenTemplateDocument (Document doc)
-#else  /* __STDC__ */
-void               OpenTemplateDocument (doc)
-Document doc;
-#endif /* __STDC__ */
-
+void OpenTemplateDocument (Document doc)
 {
   ElementType         metaElType;
   Element             metaEl;
@@ -72,9 +55,9 @@ Document doc;
   ThotBool            found = FALSE;
   ThotBool            ok = FALSE;
   int                 length;
-  CHAR_T              buffer[MAX_LENGTH];
-  CHAR_T             *URLdir;
-  CHAR_T             *URLname;
+  char              buffer[MAX_LENGTH];
+  char             *URLdir;
+  char             *URLname;
 
   metaElType.ElSSchema = TtaGetDocumentSSchema (doc);
   metaElType.ElTypeNum = HTML_EL_META;
@@ -92,7 +75,7 @@ Document doc;
             {
               length = MAX_LENGTH - 1;
  	      TtaGiveTextAttributeValue (metaAttr, buffer, &length);
-              found = !ustrcmp (buffer, META_TEMPLATE_NAME);
+              found = !strcmp (buffer, META_TEMPLATE_NAME);
             }
         }
     }
@@ -118,9 +101,9 @@ Document doc;
       if (ok)
         {
           if (script_URL == NULL)
-            script_URL = TtaWCSdup (DocumentURLs[doc]);
+            script_URL = TtaStrdup (DocumentURLs[doc]);
 	  TtaFreeMemory (DocumentURLs[doc]);
-	  DocumentURLs[doc] = TtaWCSdup (buffer);
+	  DocumentURLs[doc] = TtaStrdup (buffer);
           DocumentMeta[doc]->method = CE_TEMPLATE;
 	  TtaSetTextZone (doc, 1, 1, DocumentURLs[doc]);
 	  TtaSetDocumentUnmodified (doc);
@@ -134,7 +117,7 @@ Document doc;
 	
       /* set the document name and dir */
       
-      URLname = ustrrchr (buffer, URL_SEP);
+      URLname = strrchr (buffer, URL_SEP);
       if (URLname)
 	{
 	  URLname[0] = EOS;
@@ -146,7 +129,7 @@ Document doc;
 	}
       else
 	{
-	  TtaSetDocumentDirectory (doc, TEXT(""));
+	  TtaSetDocumentDirectory (doc, "");
 	  TtaSetDocumentName (doc, buffer);
 	}
       SetWindowTitle (doc, doc, 0);
@@ -158,15 +141,9 @@ Document doc;
   ReloadTemplateParams : restores the script URL and method into meta
   to reload a template
   ----------------------------------------------------------------------*/
-#ifdef __STDC__    
-void        ReloadTemplateParams (CHAR_T **docURL, ClickEvent *method)
-#else /* __STDC__ */
-void        ReloadTemplateParams (docURL, method)
-CHAR_T    **docURL;
-ClickEvent *method;
-#endif /* __STDC__ */
+void ReloadTemplateParams (char **docURL, ClickEvent *method)
 {
    *method = CE_FORM_GET;
    TtaFreeMemory (*docURL);
-   *docURL = TtaWCSdup (script_URL); 
+   *docURL = TtaStrdup (script_URL); 
 }
