@@ -16,7 +16,9 @@
 
 /* annotlib includes */
 #include "annotlib.h"
+
 #include "ANNOTfiles_f.h"
+#include "ANNOTlink_f.h"
 #include "ANNOTschemas_f.h"
 #include "ANNOTevent_f.h"
 #include "ANNOTtools_f.h"
@@ -24,6 +26,8 @@
 /* Amaya includes */
 #include "init_f.h"
 #include "AHTURLTools_f.h"
+#include "HTMLsave_f.h"
+#include "uconvert.h"
 
 /* schema includes */
 #include "Annot.h"
@@ -78,7 +82,7 @@ Document ANNOT_NewDocument (Document doc, AnnotMode mode)
   else
     tmp = "annotation";
 
-  annotDoc = InitDocView (0, tmp, docAnnot, 0, FALSE);
+  annotDoc = InitDocAndView (0, tmp, docAnnot, 0, FALSE, L_OtherValue);
 
   if (annotDoc == 0) 
     {
@@ -93,13 +97,71 @@ Document ANNOT_NewDocument (Document doc, AnnotMode mode)
       DocumentURLs[annotDoc] = fname;
       DocumentMeta[annotDoc] = DocumentMetaDataAlloc ();
       DocumentMeta[annotDoc]->method = CE_ABSOLUTE;
+      DocumentMeta[annotDoc]->xmlformat = TRUE;
       DocumentSource[annotDoc] = 0;
 
+      /* set the charset to be UTF-8 by default */
+      TtaSetDocumentCharset (annotDoc, TtaGetCharset ("UTF-8"));
+      DocumentMeta[annotDoc]->charset = TtaStrdup ("UTF-8");
+      DocumentMeta[annotDoc]->content_type = TtaStrdup (AM_XHTML_MIME_TYPE);
+      
+      /* activate, remove buttons from the annot view */
       ANNOT_PrepareAnnotView (annotDoc);
     }  
   return annotDoc;
 }
 
+/*-----------------------------------------------------------------------
+  GetMetaDataInAnnotList
+  If it exists, it returns the annotation object from the annot_list that
+  corresponds to the document_url. Otherwise, it returns NULL.
+  -----------------------------------------------------------------------*/
+static AnnotMeta *GetMetaDataInAnnotList (char *doc_url, List *annot_list)
+{
+  List *ptr;
+  AnnotMeta *annot = NULL;
+  char *annot_url;
+  char *tmp_doc_url = NULL;
+  char *body_url;
+
+  if (!IsW3Path (doc_url) && !IsFilePath (doc_url))
+    tmp_doc_url = LocalToWWW (doc_url);
+  else
+    tmp_doc_url = doc_url;
+
+  /* get a pointer to the annot list */
+  ptr = annot_list;
+  while (ptr)
+    {
+      annot = (AnnotMeta *) ptr->object;
+
+      if (annot->annot_url)
+	annot_url = annot->annot_url;
+      else
+	annot_url = NULL;
+
+      /* RRS: newly created local annotations have only a body URI */
+      if (annot->body_url)
+	body_url = annot->body_url;
+      else
+	body_url = NULL;
+
+      /* @@ maybe we could add calls to NormalizeFile here */
+      if ((annot_url && !strcasecmp (tmp_doc_url, annot_url))
+	  || (body_url && !strcasecmp (tmp_doc_url, body_url)))
+	break;
+      ptr = ptr->next;
+    }
+  
+  if (tmp_doc_url != doc_url)
+      TtaFreeMemory (tmp_doc_url);
+
+  if (ptr)
+    return annot;
+  else
+    return NULL;
+}
+
 /*-----------------------------------------------------------------------
   GetMetaData
   returns the annotation object that corresponds to the annotation
@@ -107,49 +169,22 @@ Document ANNOT_NewDocument (Document doc, AnnotMode mode)
   -----------------------------------------------------------------------*/
 AnnotMeta *GetMetaData (Document doc, Document doc_annot)
 {
-  List *ptr;
   AnnotMeta *annot = NULL;
 
-  /* get a pointer to the annot list */
-  ptr = AnnotMetaData[doc].annotations;
-  while (ptr)
-    {
-      annot = (AnnotMeta *) ptr->object;
-      /* @@ maybe we could add calls to NormalizeFile here */
-      if ((annot->annot_url 
-	   && (!strcasecmp (DocumentURLs[doc_annot], annot->annot_url)
-	       || !strcasecmp (DocumentURLs[doc_annot], annot->annot_url + 7)))
-	  /* RRS: newly created local annotations have only a body URI */
-	  || (annot->body_url
-	      &&  (!strcasecmp (DocumentURLs[doc_annot], annot->body_url) 
-		   || !strcasecmp (DocumentURLs[doc_annot], annot->body_url + 7))))
-	break;
-      ptr = ptr->next;
-    }
-  
-  if (ptr)
+  /* search the annotation list */
+  annot = GetMetaDataInAnnotList (DocumentURLs[doc_annot], 
+				 AnnotMetaData[doc].annotations);
+
+  if (annot)
     return annot;
 
 #ifdef ANNOT_ON_ANNOT
   if (AnnotMetaData[doc].thread)
-    ptr = AnnotMetaData[doc].thread->annotations;
-  else
-    ptr = NULL;
-  while (ptr)
-    {
-      annot = (AnnotMeta *) ptr->object;
-      /* @@ maybe we could add calls to NormalizeFile here */
-      if ((annot->annot_url 
-	   && (!strcasecmp (DocumentURLs[doc_annot], annot->annot_url)
-	       || !strcasecmp (DocumentURLs[doc_annot], annot->annot_url + 7)))
-	  /* RRS: newly created local annotations have only a body URI */
-	  || (annot->body_url
-	      &&  (!strcasecmp (DocumentURLs[doc_annot], annot->body_url) 
-		   || !strcasecmp (DocumentURLs[doc_annot], annot->body_url + 7))))
-	break;
-      ptr = ptr->next;
-    }
-  if (ptr)
+    /* search the thread list associated to the annotation document */
+    annot = GetMetaDataInAnnotList (DocumentURLs[doc_annot], 
+				    AnnotMetaData[doc].thread->annotations);
+
+  if (annot)
     return annot;
 #endif /* ANNOT_ON_ANNOT */
 
@@ -162,13 +197,7 @@ AnnotMeta *GetMetaData (Document doc, Document doc_annot)
   its body.
   (@@ JK: weird function!)
   -----------------------------------------------------------------------*/
-#ifdef __STDC__
 void ANNOT_LoadAnnotation (Document doc, Document docAnnot)
-#else /* __STDC__*/
-void ANNOT_LoadAnnotation (doc, annotDoc)
-     Document doc;
-     Document annotDoc;
-#endif /* __STDC__*/
 {
   AnnotMeta *annot;
 
@@ -176,18 +205,28 @@ void ANNOT_LoadAnnotation (doc, annotDoc)
      we copy it from the existing metadata */
   annot = GetMetaData (doc, docAnnot);
   if (annot)
-    ANNOT_InitDocumentStructure (doc, docAnnot, annot, ANNOT_initNone);
+    {
+      ANNOT_InitDocumentStructure (doc, docAnnot, annot, ANNOT_initNone);
+      /* save the annot_url to which we should reply (or annotate)
+	 this annotation */
+      if (AnnotMetaData[docAnnot].annot_url)
+	TtaFreeMemory (AnnotMetaData[docAnnot].annot_url);
+      /* Locally stored annotations have a parser generated URI, that
+	 changes each time they are parsed. We use the body URL instead
+	 to reply to them.
+	 Remote annotations have a correct url, so we use it. When
+         pubishing an annotation, we'll do the necessary changes */
+      if (annot->annot_url && IsW3Path (annot->annot_url))
+	AnnotMetaData[docAnnot].annot_url = TtaStrdup (annot->annot_url);
+      else
+	AnnotMetaData[docAnnot].annot_url = TtaStrdup (annot->body_url);
+    }
 }
 
 /*-----------------------------------------------------------------------
   ANNOT_ReloadAnnotMeta
   -----------------------------------------------------------------------*/
-#ifdef __STDC__
 void ANNOT_ReloadAnnotMeta (Document annotDoc)
-#else /* __STDC__*/
-void ANNOT_ReloadAnnotMeta (annotDoc)
-     Document annotDoc;
-#endif /* __STDC__*/
 {
   Document source_doc; 
   AnnotMeta *annot;
@@ -198,7 +237,15 @@ void ANNOT_ReloadAnnotMeta (annotDoc)
   annot = GetMetaData (source_doc, annotDoc);
   if (!annot)
     return;
+  /* clear the list of annotations, threads, and filters related to
+     this annotation, as it is a reload */
+  ANNOT_FreeDocumentResource (annotDoc);
   /* initialize the meta data */
+  if (annot->annot_url && IsW3Path (annot->annot_url))
+    AnnotMetaData[annotDoc].annot_url = TtaStrdup (annot->annot_url);
+  else
+    AnnotMetaData[annotDoc].annot_url = TtaStrdup (annot->body_url);
+
   ANNOT_InitDocumentStructure (source_doc, annotDoc, annot, ANNOT_initNone);
 }
 
@@ -213,12 +260,15 @@ void  ANNOT_InitDocumentMeta (Document doc, Document docAnnot, AnnotMeta *annot,
   Element        root, head, el;
   Attribute      attr;
   AttributeType  attrType;
-  STRING         user;
-  STRING         doc_anchor;
-  STRING         source_url;
+  char          *user;
+  char          *doc_anchor;
+  char          *source_url;
   char          *cdate;
   char          *mdate;
   char          *type;
+#ifdef ANNOT_ON_ANNOT
+  Document       thread_doc;
+#endif /* ANNOT_ON_ANNOT */
 
   user = annot->author;
   source_url = annot->source_url;
@@ -232,7 +282,16 @@ void  ANNOT_InitDocumentMeta (Document doc, Document docAnnot, AnnotMeta *annot,
 			  a service may forget to give us a type. */
 
   /* save the docid of the annotated document */
-  DocumentMeta[docAnnot]->source_doc = doc;
+#ifdef ANNOT_ON_ANNOT
+  /* for annotations, the source document is the document where the thread is found */
+  thread_doc = 0;
+  if (AnnotMetaData[docAnnot].thread)
+    thread_doc = AnnotThread_searchThreadDoc (DocumentURLs[docAnnot]);
+  if (thread_doc > 0)
+    DocumentMeta[docAnnot]->source_doc = thread_doc;
+  else
+#endif /* ANNOT_ON_ANNOT */
+    DocumentMeta[docAnnot]->source_doc = doc;
 
   /*
   ** initialize the METADATA 
@@ -242,6 +301,11 @@ void  ANNOT_InitDocumentMeta (Document doc, Document docAnnot, AnnotMeta *annot,
   root = TtaGetRootElement (docAnnot);
   elType = TtaGetElementType (root);
 
+  /* specify if the annotation body is an XML document. We systematically
+   convert HTML into XHTML when saving */
+  if (DocumentMeta[docAnnot]->xmlformat || annot->bodyType == docHTML)
+    Annot_SetXMLBody (docAnnot);
+
   /* point to the metadata structure */
   elType.ElTypeNum = Annot_EL_Description;
   head = TtaSearchTypedElement (elType, SearchInTree, root);
@@ -255,27 +319,45 @@ void  ANNOT_InitDocumentMeta (Document doc, Document docAnnot, AnnotMeta *annot,
   if (annot->creator)
     {
       RDFStatementP s;
+      char *tmp;
 
       s = ANNOT_FindRDFStatement (annot->creator->statements, PROP_firstName);
       if (s)
 	{
+#ifdef _I18N_
+	  tmp = (char *) s->object->name;
+#else
+	  tmp = TtaConvertMbsToByte ((char *) s->object->name, ISO_8859_1);
+#endif /* _I18N_ */
 	  elType.ElTypeNum = Annot_EL_CreatorGivenName;
 	  el = TtaSearchTypedElement (elType, SearchInTree, head);
 	  el = TtaGetFirstChild (el);
-	  TtaSetTextContent (el, (char *) s->object->name,
+	  TtaSetTextContent (el, tmp,
 			     TtaGetDefaultLanguage (),
 			     docAnnot);
+#ifndef _I18N_
+	  TtaFreeMemory (tmp);
+#endif /* _I18N_ */
 	}
 
       s = ANNOT_FindRDFStatement (annot->creator->statements, PROP_name);
       if (s)
 	{
+#ifdef _I18N_
+	  tmp = (char *) s->object->name;
+#else
+	  tmp = TtaConvertMbsToByte ((char *) s->object->name, ISO_8859_1);
+#endif /* _I18N_ */
+
 	  elType.ElTypeNum = Annot_EL_CreatorFamilyName;
 	  el = TtaSearchTypedElement (elType, SearchInTree, head);
 	  el = TtaGetFirstChild (el);
-	  TtaSetTextContent (el, (char *) s->object->name,
+	  TtaSetTextContent (el, (char *) tmp,
 			     TtaGetDefaultLanguage (),
 			     docAnnot);
+#ifndef _I18N_
+	  TtaFreeMemory (tmp);
+#endif /* _I18N_ */
 	}
 
       s = ANNOT_FindRDFStatement (annot->creator->statements, PROP_Email);
@@ -320,23 +402,14 @@ void  ANNOT_InitDocumentMeta (Document doc, Document docAnnot, AnnotMeta *annot,
   attrType.AttrTypeNum = Annot_ATTR_HREF_;
   attr = TtaNewAttribute (attrType);
   TtaAttachAttribute (el, attr, docAnnot);
-#if 0
-    {
-      doc_anchor = TtaGetMemory (strlen (DocumentURLs[doc])
-				 + strlen (user)
-				 + strlen (DocumentURLs[docAnnot])
-				 + 20);
-      sprintf (doc_anchor, "%s#%s_%s_%s", DocumentURLs[doc],
-	       ANNOT_ANAME, user, DocumentURLs[docAnnot]);
-      annot->name = doc_anchor;
-    }
-#endif
-  doc_anchor = TtaGetMemory (strlen (DocumentURLs[doc])
+
+  doc_anchor = TtaGetMemory (strlen (annot->source_url)
 			     + strlen (annot->name) 
 			     + 20);
-  sprintf (doc_anchor, "%s#%s", DocumentURLs[doc], annot->name);
+  sprintf (doc_anchor, "%s#%s", annot->source_url, annot->name);
   TtaSetAttributeText (attr, doc_anchor, el, docAnnot);
   TtaFreeMemory (doc_anchor);
+
   /* use the source_doc_title parameter as the value of the source
      document field */
   el = TtaGetFirstChild (el);
@@ -345,6 +418,15 @@ void  ANNOT_InitDocumentMeta (Document doc, Document docAnnot, AnnotMeta *annot,
   /* RDF type metadata */
   elType.ElTypeNum = Annot_EL_RDFtype;
   el = TtaSearchTypedElement (elType, SearchInTree, head);
+  /* to set up the text (reply or annotation type), we use an attribute in this
+   element */
+  if (annot->inReplyTo)
+    {
+      attrType.AttrTypeNum = Annot_ATTR_isReply;
+      attr = TtaGetAttribute (el, attrType);
+      TtaSetAttributeValue (attr, Annot_ATTR_isReply_VAL_YES_, el, docAnnot);
+    }
+  /* set the rdf type value itself */
   el = TtaGetFirstChild (el);
   TtaSetTextContent (el, type,
 		     TtaGetDefaultLanguage (), docAnnot);
@@ -364,6 +446,8 @@ void  ANNOT_InitDocumentBody (Document docAnnot, char *source_doc_title)
   /*
   ** HTML initialization
   */
+  /* we first add the HTML tree */
+  ANNOT_CreateBodyTree (docAnnot, docHTML);
 
   /* we find the the HTML nature */
   root = TtaGetRootElement (docAnnot);
@@ -431,6 +515,7 @@ void  ANNOT_InitDocumentBody (Document docAnnot, char *source_doc_title)
     }
 }
 
+#ifdef ANNOT_ON_ANNOT
 /*-----------------------------------------------------------------------
   ANNOT_ThreadItem_new
   -----------------------------------------------------------------------*/
@@ -447,10 +532,14 @@ static Element ANNOT_ThreadItem_new (Document doc)
   /* return the element we just created */
   return thread_item;
 }
+#endif /* ANNOT_ON_ANNOT */
 
+#ifdef ANNOT_ON_ANNOT
 /*-----------------------------------------------------------------------
   ANNOT_ThreadItem_init
   Inits a thread item default fields.
+  If useSource == TRUE, we're initializing the thread, so the first
+  item points to the root of thread.
   -----------------------------------------------------------------------*/
 static void ANNOT_ThreadItem_init (Element thread_item, Document doc, AnnotMeta *annot_doc, ThotBool useSource)
 {
@@ -458,12 +547,18 @@ static void ANNOT_ThreadItem_init (Element thread_item, Document doc, AnnotMeta
   ElementType         elType;
   Attribute           attr;
   AttributeType       attrType;
-  char               *href;
   char               *tmp;
   AnnotMeta          *annot;
 
-  attrType.AttrSSchema = elType.ElSSchema = TtaGetSSchema ("Annot", doc);
+  if (useSource && DocumentMeta[doc]->source_doc > 0)
+    annot = GetMetaData (DocumentMeta[doc]->source_doc, doc);
+  else
+    annot = annot_doc;
+  
+  if (!annot)
+    tmp = NULL;
 
+  attrType.AttrSSchema = elType.ElSSchema = TtaGetSSchema ("Annot", doc);
   /*
   **  initialize the reverse link 
   */
@@ -474,19 +569,36 @@ static void ANNOT_ThreadItem_init (Element thread_item, Document doc, AnnotMeta
     TtaRemoveAttribute (thread_item, attr, doc);
   attr = TtaNewAttribute (attrType);
   TtaAttachAttribute (thread_item, attr, doc);
-  tmp = (useSource) ? annot_doc->source_url : annot_doc->body_url,
-  href = TtaGetMemory (strlen (tmp) + 20);
-  sprintf (href, "%s", tmp);
-  TtaSetAttributeText (attr, href, thread_item, doc);
-  TtaFreeMemory (href);
+  TtaSetAttributeText (attr, annot->body_url, thread_item, doc);
 
-  if (useSource)
-    annot = GetMetaData (DocumentMeta[doc]->source_doc, doc);
+  /* initialize the Annot_HREF_. This attribute helps
+     sort the thread items in conjunction with the inreplyto URL */
+  attrType.AttrTypeNum = Annot_ATTR_Annot_HREF_;
+  /* remove the previous reverse link if it exists already */
+  attr = TtaGetAttribute (thread_item, attrType);
+  if (attr)
+    TtaRemoveAttribute (thread_item, attr, doc);
+  attr = TtaNewAttribute (attrType);
+  TtaAttachAttribute (thread_item, attr, doc);
+  if (annot->annot_url && !IsFilePath (annot->annot_url))
+    tmp = annot->annot_url;
   else
-    annot = annot_doc;
+    tmp = annot->body_url;
+  TtaSetAttributeText (attr, tmp, thread_item, doc);
 
-  if (!annot)
-    tmp = NULL;
+  /* mark the thread item as orphan or not */
+  if (annot->is_orphan_item)
+    {
+      attrType.AttrTypeNum = Annot_ATTR_Orphan_item;
+      /* remove the previous reverse link if it exists already */
+      attr = TtaGetAttribute (thread_item, attrType);
+      if (!attr)
+	{
+	  attr = TtaNewAttribute (attrType);
+	  TtaAttachAttribute (thread_item, attr, doc);
+	}
+      TtaSetAttributeValue (attr, Annot_ATTR_Orphan_item_VAL_Yes_, thread_item, doc);
+    }
 
   /* put the type of the annotation */
   elType.ElTypeNum = Annot_EL_TI_Type;
@@ -537,6 +649,7 @@ static void ANNOT_ThreadItem_init (Element thread_item, Document doc, AnnotMeta
 	TtaSetTextContent (el, tmp, TtaGetDefaultLanguage (), doc);
     }
 }
+#endif /* ANNOT_ON_ANNOT */
 
 /*-----------------------------------------------------------------------
   ANNOT_AddThreadItem
@@ -554,7 +667,7 @@ Element ANNOT_AddThreadItem (Document doc, AnnotMeta *annot)
 {
 #ifdef ANNOT_ON_ANNOT
   ElementType    elType;
-  Element        root, thread_item, el;
+  Element        root, thread_item, el, rootOfThread;
   Attribute      attr;
   AttributeType  attrType;
   char          *url;
@@ -567,7 +680,7 @@ Element ANNOT_AddThreadItem (Document doc, AnnotMeta *annot)
   el = TtaSearchTypedElement (elType, SearchInTree, root);
 
   if (el)
-    root = el;
+    rootOfThread = el;
   else
     {
       /* create the thread element */
@@ -576,24 +689,25 @@ Element ANNOT_AddThreadItem (Document doc, AnnotMeta *annot)
       if (!el)
 	return NULL; /* couldn't find a body! */
       elType.ElTypeNum = Annot_EL_Thread;
-      root =  TtaNewTree (doc, elType, "");
+      rootOfThread =  TtaNewTree (doc, elType, "");
       /* insert the thread */
-      TtaInsertSibling (root, el, FALSE, doc);
+      TtaInsertSibling (rootOfThread, el, FALSE, doc);
       /* and initialize the root of the thread */
-      el = TtaGetFirstChild (root);
+      el = TtaGetFirstChild (rootOfThread);
       ANNOT_ThreadItem_init (el, doc,  annot, TRUE);
     }
 
   /* try to find where to insert the element */
   attrType.AttrSSchema = elType.ElSSchema;
-  attrType.AttrTypeNum = Annot_ATTR_HREF_;
+  attrType.AttrTypeNum = Annot_ATTR_Annot_HREF_;
+  root = rootOfThread;
   TtaSearchAttribute (attrType, SearchForward, root, &el, &attr);
   while (el)
     {
       i = TtaGetTextAttributeLength (attr) + 1;
       url = TtaGetMemory (i);
       TtaGiveTextAttributeValue (attr, url, &i);
-      if (!strcasecmp (url, annot->source_url))
+      if (!strcasecmp (url, annot->inReplyTo))
 	{
 	  TtaFreeMemory (url);
 	  break;
@@ -602,11 +716,13 @@ Element ANNOT_AddThreadItem (Document doc, AnnotMeta *annot)
       root = el;
       TtaSearchAttribute (attrType, SearchForward, root, &el, &attr);
     }
-  root = el;
   
-  if (!root)
-    /* we can't insert a reply to nothing! */
-    return NULL;
+  if (el)
+    root = el;
+  else
+    /* we didn't find the ReplyTo, so we insert it as a child of the root 
+     @@ and maybe add an attribute (unknown thread... ) */
+    root = TtaGetFirstChild (rootOfThread);
       
   /* find the container and insert it */
   el = TtaGetLastChild (root);
@@ -639,6 +755,54 @@ Element ANNOT_AddThreadItem (Document doc, AnnotMeta *annot)
 #endif /* ANNOT_ON_ANNOT */
 }
 
+/*-----------------------------------------------------------------------
+  ANNOT_DeleteThread
+  Experiment the S/P threads.
+
+  Deletes the thread of thread_doc, if it exists
+  -----------------------------------------------------------------------*/
+void ANNOT_DeleteThread (Document thread_doc)
+{
+  Element el;
+  ElementType elType;
+
+  /* we find the the Thread element and make it our root */
+  el = TtaGetRootElement (thread_doc);
+  elType = TtaGetElementType (el);
+  elType.ElTypeNum = Annot_EL_Thread;
+  el = TtaSearchTypedElement (elType, SearchInTree, el);
+
+  if (el)
+    TtaDeleteTree (el, thread_doc);
+}
+
+/*-----------------------------------------------------------------------
+  ANNOT_BuildThread
+  Experiment the S/P threads.
+
+  Builds a thread on thread_doc
+  -----------------------------------------------------------------------*/
+void ANNOT_BuildThread (Document thread_doc)
+{
+#ifdef ANNOT_ON_ANNOT
+  List *annot_list;
+  AnnotMeta *annot;
+
+  annot_list = AnnotThread[thread_doc].annotations;
+  /* sort the thread */
+  AnnotThread_sortThreadList (&annot_list);
+  AnnotThread[thread_doc].annotations = annot_list;
+
+  /* erase previous thread */
+  while (annot_list)
+    {
+      annot = (AnnotMeta *) annot_list->object;
+      ANNOT_AddThreadItem (thread_doc, annot);
+      annot_list = annot_list->next;
+    }
+#endif /* ANNOT_ON_ANNOT */
+}
+
 /*-----------------------------------------------------------------------
   ANNOT_ToggleThread
   Selects an item in the thread view.
@@ -668,26 +832,29 @@ void ANNOT_ToggleThread (Document thread_doc, Document annot_doc,
       /* grr... again a local compare problem! */
       elType.ElTypeNum = Annot_EL_TI_desc;
       ti_desc = TtaSearchTypedElement (elType, SearchInTree, el);
-      attrType.AttrTypeNum = Annot_ATTR_Selected_;
-      attr = TtaGetAttribute (ti_desc, attrType);
-      if (!strcasecmp (url, DocumentURLs[annot_doc])
-	  || !strcasecmp (url + sizeof ("file:/"), DocumentURLs[annot_doc]))
+      if (ti_desc)
 	{
-	  if (turnOn && !attr)
+	  attrType.AttrTypeNum = Annot_ATTR_Selected_;
+	  attr = TtaGetAttribute (ti_desc, attrType);
+	  if (!strcasecmp (url, DocumentURLs[annot_doc])
+	      || !strcasecmp (url + sizeof ("file:/"), DocumentURLs[annot_doc]))
 	    {
-	      /* turn on the selected status */
-	      attr = TtaNewAttribute (attrType);
-	      TtaAttachAttribute (ti_desc, attr, thread_doc);
-	      TtaSetAttributeValue (attr, Annot_ATTR_Selected__VAL_Yes_, ti_desc, thread_doc);
+	      if (turnOn && !attr)
+		{
+		  /* turn on the selected status */
+		  attr = TtaNewAttribute (attrType);
+		  TtaAttachAttribute (ti_desc, attr, thread_doc);
+		  TtaSetAttributeValue (attr, Annot_ATTR_Selected__VAL_Yes_, ti_desc, thread_doc);
+		}
+	      if (!turnOn && attr)
+		/* turn off the selected status */
+		TtaRemoveAttribute (ti_desc, attr, thread_doc);
+	    }
+	  else if (strcasecmp (url, DocumentURLs[annot_doc]) && attr)
+	    {
+	      /* turn off the selected status */
+	      TtaRemoveAttribute (ti_desc, attr, thread_doc);
 	    }
-	  if (!turnOn && attr)
-	    /* turn off the selected status */
-	    TtaRemoveAttribute (ti_desc, attr, thread_doc);
-	}
-      else if (strcasecmp (url, DocumentURLs[annot_doc]) && attr)
-	{
-	  /* turn off the selected status */
-	  TtaRemoveAttribute (ti_desc, attr, thread_doc);
 	}
       TtaFreeMemory (url);
       root = el;
@@ -716,6 +883,8 @@ Document ANNOT_GetThreadDoc (Document thread_doc)
   doc_annot = 0;
   /* we find the the Thread element and make it our root */
   root = TtaGetRootElement (thread_doc);
+  if (!root)
+    return 0;
   elType = TtaGetElementType (root);
   attrType.AttrSSchema = elType.ElSSchema;
   attrType.AttrTypeNum = Annot_ATTR_Selected_;
@@ -752,10 +921,12 @@ Document ANNOT_GetThreadDoc (Document thread_doc)
 }
 
 /*-----------------------------------------------------------------------
-  ANNOT_UpdateThread
+  ANNOT_UpdateThreadItem
   Updates the metadata of a thread item.
+  @@ If we published the root of the thread, update all references
+  to this item.
   -----------------------------------------------------------------------*/
-void ANNOT_UpdateThread (Document doc, AnnotMeta *annot)
+void ANNOT_UpdateThreadItem (Document doc, AnnotMeta *annot, char *body_url)
 {
 #ifdef ANNOT_ON_ANNOT
   Document       thread_doc;
@@ -770,11 +941,10 @@ void ANNOT_UpdateThread (Document doc, AnnotMeta *annot)
     return;
 
   /* find the document where the thread is being shown */
-  if (AnnotMetaData[doc].thread)
-    thread_doc = AnnotThread_searchRoot (AnnotMetaData[doc].thread->rootOfThread);
-  else
-    thread_doc = 0;
+  if (!annot->inReplyTo)
+    return;
 
+  thread_doc = AnnotThread_searchRoot (AnnotMetaData[doc].thread->rootOfThread);
   if (thread_doc == 0)
     return;
 
@@ -789,7 +959,7 @@ void ANNOT_UpdateThread (Document doc, AnnotMeta *annot)
       i = TtaGetTextAttributeLength (attr) + 1;
       url = TtaGetMemory (i);
       TtaGiveTextAttributeValue (attr, url, &i);
-      if (!strcasecmp (url, annot->body_url))
+      if (!strcasecmp (url, body_url))
 	{
 	  TtaFreeMemory (url);
 	  break;
@@ -810,8 +980,8 @@ void ANNOT_UpdateThread (Document doc, AnnotMeta *annot)
    Initializes an annotation document by adding a BODY part
    and adding META elements for title, author, date, and type
   -----------------------------------------------------------------------*/
-void  ANNOT_InitDocumentStructure (Document doc, Document docAnnot, 
-				   AnnotMeta *annot, AnnotMode mode)
+void ANNOT_InitDocumentStructure (Document doc, Document docAnnot, 
+				  AnnotMeta *annot, AnnotMode mode)
 {
   char *source_doc_title;
   char *text;
@@ -828,7 +998,6 @@ void  ANNOT_InitDocumentStructure (Document doc, Document docAnnot,
 
   if (mode & ANNOT_initATitle)
     {
-      annot->title = TtaStrdup (source_doc_title);
       if (mode & ANNOT_isReplyTo)
 	text = "Reply to ";
       else
@@ -868,25 +1037,18 @@ void  ANNOT_InitDocumentStructure (Document doc, Document docAnnot,
    Initializes an annotation document by adding a BODY part
    and adding META elements for title, author, date, and type
   -----------------------------------------------------------------------*/
-
-#ifdef __STDC__
-void  ANNOT_InitDocumentStructure (Document docAnnot, Document document)
-#else /* __STDC__*/
-void  ANNOT_InitDocumentStructure (docAnnot, document)
-     Document docAnnot;
-     Document document;
-#endif /* __STDC__*/
+void ANNOT_InitDocumentStructure (Document docAnnot, Document document)
 {
-  ElementType elType;
-  Element     root, head, body, el, di, tl, top, child, meta;
+  ElementType  elType;
+  Element      root, head, body, el, di, tl, top, child, meta;
   Attribute           attr;
   AttributeType       attrType;
-  time_t      curDate;
+  time_t       curDate;
   struct tm   *localDate;
-  STRING      strDate;
-  STRING      doc_anchor;
-  char      tempfile[MAX_LENGTH];
-  char       *annot_user;
+  char        *strDate;
+  char        *doc_anchor;
+  char         tempfile[MAX_LENGTH];
+  char        *annot_user;
 
   annot_user = GetAnnotUser ();
 
@@ -1044,13 +1206,7 @@ void  ANNOT_InitDocumentStructure (docAnnot, document)
    Removes all  unused buttons and menus). Returns the number of the opened 
    view or 0 in case of failure.
   -----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void ANNOT_PrepareAnnotView (Document document)
-#else /* __STDC__*/
-void ANNOT_PrepareAnnotView (document)
-     Document document;
-#endif /* __STDC__*/
 {
   View view;
 
@@ -1068,7 +1224,10 @@ void ANNOT_PrepareAnnotView (document)
     /*    TtaSetMenuOff (document, view, Help_); */
     TtaSetItemOff (document, view, File, BBack);
     TtaSetItemOff (document, view, File, BForward);
-    TtaSetItemOff (document, view, File, BHtml);
+    TtaSetItemOff (document, view, File, BHtmlBasic);
+    TtaSetItemOff (document, view, File, BHtmlStrict);
+    TtaSetItemOff (document, view, File, BHtml11);
+    TtaSetItemOff (document, view, File, BHtmlTransitional);
     TtaSetItemOff (document, view, File, BTemplate);
     TtaSetItemOff (document, view, File, BCss);
     TtaSetItemOff (document, view, File, BOpenDoc);
@@ -1076,6 +1235,24 @@ void ANNOT_PrepareAnnotView (document)
     TtaSetItemOff (document, view, File, BSaveAs);
     TtaSetTextZone (document, view, 1, DocumentURLs[document]);
 
+    /* views */
+    TtaSetItemOff (document, view, Views, TShowTextZone);
+    /* TtaSetItemOff (document, view, Views, BShowStructure); */
+    TtaSetItemOff (document, view, Views, BShowAlternate);
+    TtaSetItemOff (document, view, Views, BShowSource);
+    TtaSetItemOff (document, view, Views, BShowLinks);
+
+    /* annotations */
+#ifdef ANNOT_ON_ANNOT
+    TtaSetItemOn (document, view, Annotations_, BAnnotateSelection);
+    TtaSetItemOn (document, view, Annotations_, BAnnotateDocument);
+    TtaSetItemOn (document, view, Annotations_, BReplyToAnnotation);
+#else
+    TtaSetItemOff (document, view, Annotations_, BAnnotateSelection);
+    TtaSetItemOff (document, view, Annotations_, BAnnotateDocument);
+    TtaSetItemOff (document, view, Annotations_, BReplyToAnnotation);
+#endif /* ANNOT_ON_ANNOT */
+
 #if 0
     TtaAddTextZone (document, 1, TtaGetMessage (AMAYA, AM_TITLE), TRUE,
 		    NULL);
@@ -1083,9 +1260,7 @@ void ANNOT_PrepareAnnotView (document)
     TtaAddTextZone (document, 1, TtaGetMessage (AMAYA, AM_LOCATION), TRUE,
 		    NULL);
     TtcSwitchCommands (document, view);
-    TtaSetItemOff (document, view, File, BReload);
 
-    TtaSetMenuOff (document, view, Views);
     TtaSetMenuOff (document, view, Special);
 #endif
   }
@@ -1096,18 +1271,12 @@ void ANNOT_PrepareAnnotView (document)
   ANNOT_LocalSave
   Saves the annotation document doc_annot to the local filesystem
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool            ANNOT_LocalSave (Document doc_annot, char *html_filename)
-#else  /* __STDC__ */
-ThotBool            ANNOT_LocalSave (doc_annot, html_filename)
-Document            doc_annot;
-char             *html_filename;
-
-#endif /* __STDC__ */
+ThotBool ANNOT_LocalSave (Document doc_annot, char *html_filename)
 {
   Element el;
   ElementType elType;
   AnnotMeta *annot;
+  ThotBool result;
 
   annot = GetMetaData (DocumentMeta[doc_annot]->source_doc, doc_annot);
   if (!annot)
@@ -1136,14 +1305,15 @@ char             *html_filename;
   el = TtaGetFirstChild (el);
   TtaSetTextContent (el, annot->mdate, TtaGetDefaultLanguage (), doc_annot); 
 
-  return TtaExportDocument (doc_annot, html_filename, "AnnotT");
+  /* set up the charset and namespaces */
+  SetNamespacesAndDTD (doc_annot);
+  result = TtaExportDocumentWithNewLineNumbers (doc_annot, html_filename, 
+					       "AnnotT");
+
+  /* update the annotation icon to the new type */
+  if (result && !Annot_IsReplyTo (doc_annot))
+      LINK_UpdateAnnotIcon (DocumentMeta[doc_annot]->source_doc, annot);
+
+  return result;
 }
 
-
-
-
-
-
-
-
-
