@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1999.
+ *  (c) COPYRIGHT MIT and INRIA, 1999-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  * 
  */
@@ -36,8 +36,10 @@
 #include "XLink.h"
 #include "HTML.h"
 #include "MathML.h"
-#include "GraphML.h"
+#include "SVG.h"
 
+/* thotlib includes */
+#include "content.h"
 
 /*-----------------------------------------------------------------------
   LINK_CreateAName
@@ -82,7 +84,10 @@ char  *LINK_GetAnnotationIndexFile (char *source_url)
   char buffer[MAX_LENGTH];
   char *index_file;
   FILE *fp;
-  
+
+  if (!source_url)
+    return NULL;
+
   annot_dir = GetAnnotDir ();
   annot_main_index = GetAnnotMainIndex ();
   annot_main_index_file = TtaGetMemory (strlen (annot_dir) 
@@ -130,6 +135,86 @@ char  *LINK_GetAnnotationIndexFile (char *source_url)
   return (index_file);
 }
 
+/*-----------------------------------------------------------------------
+   LINK_UpdateAnnotationIndexFile
+   old_source_url and new_source_url must be complete URLs.
+   -----------------------------------------------------------------------*/
+void LINK_UpdateAnnotationIndexFile (char *old_source_url, char *new_source_url)
+{
+  char *annot_dir;
+  char *annot_main_index;
+  char *annot_main_index_file;
+  char *annot_new_main_index_file;
+  char url[MAX_LENGTH];
+  char index_file[MAX_LENGTH];
+  char buffer[MAX_LENGTH];
+  char *tmp_url;
+  char *tmp_index_file;
+  FILE *fp_old, *fp_new;
+  ThotBool move;
+
+  annot_dir = GetAnnotDir ();
+  annot_main_index = GetAnnotMainIndex ();
+  annot_main_index_file = TtaGetMemory (strlen (annot_dir) 
+					+ strlen (annot_main_index) 
+					+ 10);
+  sprintf (annot_main_index_file, "%s%c%s", 
+	    annot_dir, 
+	    DIR_SEP,  
+	    annot_main_index);
+  annot_new_main_index_file = TtaGetMemory (strlen (annot_dir) 
+					    + strlen (annot_main_index) 
+					    + 11);
+  sprintf (annot_new_main_index_file, "%s%ct%s", 
+	    annot_dir, 
+	    DIR_SEP,  
+	    annot_main_index);
+
+  move = FALSE;
+  if (TtaFileExist (annot_main_index_file))
+    {
+      if ((fp_old = fopen (annot_main_index_file, "r")))
+	{
+	  if ((fp_new = fopen (annot_new_main_index_file, "w")))
+	    {
+	      while (fgets (buffer, MAX_LENGTH, fp_old))
+		{
+		  sscanf (buffer, "%s %s\n", url, index_file);
+		  /* convert local URLs into file: ones */
+		  if (!IsFilePath (url) && !IsW3Path (url))
+		    tmp_url = LocalToWWW (url);
+		  else
+		    tmp_url = url;
+		  if (!IsFilePath (index_file) && !IsW3Path (index_file))
+		    tmp_index_file = LocalToWWW (index_file);
+		  else
+		    tmp_index_file = index_file;
+		  if (!strcasecmp (old_source_url, tmp_url))
+		    fprintf (fp_new, "%s %s\n", new_source_url, tmp_index_file);
+		  else
+		    fprintf (fp_new, "%s %s\n", tmp_url, tmp_index_file);
+		  if (tmp_url != url)
+		    TtaFreeMemory (tmp_url);
+		  if (tmp_index_file != index_file)
+		    TtaFreeMemory (tmp_index_file);
+		}
+	      fclose (fp_new);
+	      move = TRUE;
+	    }
+	  fclose (fp_old);
+	}
+    }
+  
+  if (move)
+    {
+      TtaFileCopy (annot_new_main_index_file, annot_main_index_file);
+      TtaFileUnlink (annot_new_main_index_file);
+    }
+
+  TtaFreeMemory (annot_main_index_file);
+  TtaFreeMemory (annot_new_main_index_file);
+}
+
 /*-----------------------------------------------------------------------
   AddAnnotationIndexFile 
   adds a new entry to the main annot index file
@@ -171,6 +256,109 @@ static void AddAnnotationIndexFile (char *source_url, char *index_file)
   TtaFreeMemory (annot_main_index_file);
 }
 
+/*-----------------------------------------------------------------------
+  LINK_AddAnnotIcon
+  If the annotation is not orphan, adds an annotation icon to a source 
+  document, an annotation link pointing to the annotation.
+  Returns TRUE if the annotation could be attached and FALSE if the
+  annotation became orphan.
+  -----------------------------------------------------------------------*/
+void LINK_AddAnnotIcon (Document source_doc, Element anchor, AnnotMeta *annot)
+{
+  Element       el;
+  char          iconName[MAX_LENGTH];
+  char          previous[MAX_LENGTH];
+  RDFStatementP iconS = (RDFStatementP) NULL;
+  int           len;
+  Language      lang;
+
+  el = TtaGetFirstChild (anchor);
+  
+  if (!PROP_usesIcon)
+    PROP_usesIcon = ANNOT_FindRDFResource (&annot_schema_list,
+					   USESICON_PROPNAME,
+					   TRUE);
+
+  if (annot->type)
+    iconS = ANNOT_FindRDFStatement (annot->type->statements, PROP_usesIcon);
+  
+  if (iconS)
+    {
+      /* @@@ JK? */
+#if 0
+      sprintf (s, "%s%camaya%cannot.png",
+	       TtaGetEnvString ("THOTDIR"), DIR_SEP, DIR_SEP);
+      iconName = s;
+#endif
+      /* @@@ JK? */
+      strcpy (iconName, iconS->object->name);
+      WWWToLocal (iconName);
+      /* expand $THOTDIR and $APP_HOME */
+      if (strncmp (iconName, "$THOTDIR", 8) == 0)
+	{
+	  char temp[MAX_LENGTH];
+	  char* thotdir = TtaGetEnvString ("THOTDIR");
+	  strcpy (temp, iconName);
+	  strcpy (iconName, thotdir);
+	  strcat (iconName, temp+8);
+	}
+      else
+	if (strncmp (iconName, "$APP_HOME", 9) == 0)
+	  {
+	    char temp[MAX_LENGTH];
+	    char* app_home = TtaGetEnvString ("APP_HOME");
+	    strcpy (temp, iconName);
+	    strcpy (iconName, app_home);
+	    strcat (iconName, temp+9);
+	  }
+    }
+  else
+      sprintf (iconName, "%s%camaya%cannot.gif",
+	       TtaGetEnvString ("THOTDIR"), DIR_SEP, DIR_SEP);
+
+  /* only substitute the icon name if it has changed */
+  len = TtaGetTextLength (el);
+  if (len > 0 && len < (int) (sizeof (previous)))
+    {
+      TtaGiveTextContent (el, previous, &len, &lang);
+      previous[len] = EOS;
+    }
+  else
+    previous[0] = EOS;
+
+  if (previous[0] == EOS || strcasecmp (iconName, previous))
+    {
+      int docModified;
+
+      docModified = TtaIsDocumentModified (source_doc);
+      TtaSetPictureContent (el, iconName, SPACE, source_doc, "image/gif");
+      if (!docModified)
+	{
+	  TtaSetDocumentUnmodified (source_doc);
+	  DocStatusUpdate (source_doc, docModified);
+	}
+    }
+}
+
+/*-----------------------------------------------------------------------
+  LINK_UpdateAnnotIcon
+  When changing annotation types, updates the icon that corresponds to the
+  type.
+  -----------------------------------------------------------------------*/
+void LINK_UpdateAnnotIcon (Document source_doc, AnnotMeta *annot)
+{
+  Element anchor;
+
+  if (!annot || !annot->name || annot->is_orphan)
+    return;
+
+  anchor = SearchAnnotation (source_doc, annot->name);
+  if (!anchor)
+    return;
+
+  LINK_AddAnnotIcon (source_doc, anchor, annot);
+}
+
 /*-----------------------------------------------------------------------
   LINK_AddLinkToSource
   Adds to a source document, an annotation link pointing to the annotation.
@@ -183,14 +371,12 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
   Element       el, first, anchor;
   AttributeType attrType;
   Attribute     attr;
-  char       *annot_user;
-  char       *docSchemaName;
-  int          c1, cN;
-  int          check_mode;
-  SSchema      XLinkSchema;
+  char         *docSchemaName;
+  int           c1, cN;
+  int           check_mode;
+  SSchema       XLinkSchema;
 
-  annot_user = GetAnnotUser ();
-  
+  /*annot_user = GetAnnotUser ();*/
   if (annot->xptr)
     {
       XPointerContextPtr ctx;
@@ -222,7 +408,7 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
   
 #ifdef ANNOT_ON_ANNOT
   /* don't add the Xlink element for replies */
-  if (annot->isReplyTo)
+  if (annot->inReplyTo)
     return (!(annot->is_orphan));
 #endif /* ANNOT_ON_ANNOT */
 
@@ -230,7 +416,7 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
   XLinkSchema = GetXLinkSSchema (source_doc);
   elType.ElSSchema = XLinkSchema;
   elType.ElTypeNum = XLink_EL_XLink;
-  anchor = TtaNewElement (source_doc, elType);
+  anchor = TtaNewTree (source_doc, elType, "");
   
   /*
   ** insert the anchor in the document tree
@@ -239,6 +425,8 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
   firstElType = TtaGetElementType (first);
   docSchemaName = TtaGetSSchemaName (firstElType.ElSSchema);
 
+  check_mode = TtaGetStructureChecking (source_doc);
+  TtaSetStructureChecking (FALSE, source_doc);
   /* @@ JK: this doesn't seem useful anymore */
   /* is the user trying to annotate an annotation? */
   el = TtaGetTypedAncestor (first, elType);
@@ -261,12 +449,9 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
 	    break;
 	  el = TtaGetParent (el);
 	}
-      check_mode = TtaGetStructureChecking (source_doc);
-      TtaSetStructureChecking (FALSE, source_doc);
       TtaInsertFirstChild (&anchor, el, source_doc);
-      TtaSetStructureChecking (check_mode, source_doc);
     }
-  else if (!strcmp (docSchemaName, "GraphML"))
+  else if (!strcmp (docSchemaName, "SVG"))
     {
       /* An annotation on a GraphMl structure. We backtrace the tree
 	 until we find the Math root element and then add the annotation as
@@ -275,16 +460,13 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
       while (el)
 	{
 	  elType = TtaGetElementType (el);
-	  if (elType.ElTypeNum == GraphML_EL_GraphML)
+	  if (elType.ElTypeNum == SVG_EL_SVG)
 	    break;
 	  el = TtaGetParent (el);
 	}
-      check_mode = TtaGetStructureChecking (source_doc);
-      TtaSetStructureChecking (FALSE, source_doc);
       TtaInsertFirstChild (&anchor, el, source_doc);
-      TtaSetStructureChecking (check_mode, source_doc);
     }
-  else 
+  else if (!strcmp (docSchemaName, "HTML"))
     {
       /* it's an HTML document */
       el = first;
@@ -294,7 +476,7 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
 	    || elType.ElTypeNum == HTML_EL_TITLE
 	    || elType.ElTypeNum == HTML_EL_BASE
 	    || elType.ElTypeNum == HTML_EL_META
-	    || elType.ElTypeNum == HTML_EL_SCRIPT
+	    || elType.ElTypeNum == HTML_EL_SCRIPT_
 	    || elType.ElTypeNum == HTML_EL_STYLE_
 	    || elType.ElTypeNum == HTML_EL_BODY)
 	{
@@ -383,19 +565,21 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
 	      
 	      len = TtaGetTextLength (first);
 
-	      if (cN > len)
-		/* add it to the end */
+	      if (cN > len && c1 == cN)
+		/* add it to the end (a caret) */
 		TtaInsertSibling (anchor, first, FALSE, source_doc);
 	      else
 		{
 		  /* add it in the middle */
-		  TtaSplitText (first, c1 - 1, source_doc);
+		  TtaSplitText (first, c1, source_doc);
 		  TtaNextSibling (&first);
 		  TtaInsertSibling (anchor, first, TRUE, source_doc);
 		}
 	    }
 	}
     }
+
+  TtaSetStructureChecking (check_mode, source_doc);
   /* add the Xlink attribute */
   SetXLinkTypeSimple (anchor, source_doc, FALSE);
 
@@ -408,7 +592,7 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
 
 #ifdef ANNOT_ON_ANNOT
   /* @@ JK: Systematically hiding the thread annotations */
-  if (annot->isReplyTo)
+  if (annot->inReplyTo)
     {
       attrType.AttrTypeNum = XLink_ATTR_AnnotIsHidden;
       attr = TtaNewAttribute (attrType);
@@ -438,7 +622,8 @@ ThotBool LINK_AddLinkToSource (Document source_doc, AnnotMeta *annot)
   TtaAttachAttribute (anchor, attr, source_doc);
   /* set the ID value (anchor endpoint) */
   TtaSetAttributeText (attr, annot->name, anchor, source_doc);
-
+  /* add the annotation icon */
+  LINK_AddAnnotIcon (source_doc, anchor, annot);
   return (!(annot->is_orphan));
 }
 
@@ -458,26 +643,44 @@ void LINK_RemoveLinkFromSource (Document source_doc, Element el)
    name of the annotation file, and the Xpointer that specifies the 
    selected  elements on the document
   -----------------------------------------------------------------------*/
-void LINK_SaveLink (Document source_doc)
+void LINK_SaveLink (Document source_doc, ThotBool isReplyTo)
 {
   char   *indexName, *doc_url;
-  List   *annot_list;
+  List   *annot_list = NULL;
 #ifdef ANNOT_ON_ANNOT
-  AnnotThreadList *thread;
+  Document rootDoc;
+  AnnotThreadList *thread = NULL;
 #endif /* ANNOT_ON_ANNOT */
 
-  annot_list = AnnotMetaData[source_doc].annotations;  
 #ifdef ANNOT_ON_ANNOT
-  thread = AnnotMetaData[source_doc].thread;
-#endif /* ANNOT_ON ANNOT */
-
-#ifdef ANNOT_ON_ANNOT
-  /* Open the annotation index */
-  if (thread)
-    doc_url = thread->rootOfThread;
+  if (isReplyTo)
+    {
+      thread = AnnotMetaData[source_doc].thread;
+      if (thread)
+	{
+	  rootDoc = AnnotThread_searchRoot (thread->rootOfThread);
+	  annot_list = AnnotMetaData[rootDoc].annotations;  
+	  doc_url = AnnotMetaData[rootDoc].annot_url;
+	  /* doc_url = DocumentURLs[rootDoc]; */
+	}
+      /* JK: what shall we do if there is no thread, return. signal an error? */
+      else
+	return;
+    }
   else
 #endif /* ANNOT_ON ANNOT */
-    doc_url = DocumentURLs[source_doc];
+    {
+#ifdef ANNOT_ON_ANNOT
+      /* the threads that have a root of thread on source_doc */
+      thread = &AnnotThread[source_doc];
+#endif /* ANNOT_ON ANNOT */
+      /* the annotations on this URL */
+      annot_list = AnnotMetaData[source_doc].annotations;
+      if (DocumentTypes[source_doc] == docAnnot &&  AnnotMetaData[source_doc].annot_url)
+	doc_url = AnnotMetaData[source_doc].annot_url;
+      else
+	doc_url = DocumentURLs[source_doc];
+    }
 
   indexName = LINK_GetAnnotationIndexFile (doc_url);
   if (!indexName)
@@ -486,21 +689,37 @@ void LINK_SaveLink (Document source_doc)
       AddAnnotationIndexFile (doc_url, indexName);
     }
 
-  /* write the update annotation list */
+  /* write the updated annotation list */
 #ifdef ANNOT_ON_ANNOT
   AnnotList_writeIndex (indexName, annot_list, (thread) ? thread->annotations : NULL);
-#else
+#else /* ANNOT_ON_ANNOT */
   AnnotList_writeIndex (indexName, annot_list, NULL);
 #endif /* ANNOT_ON_ANNOT */
+
   TtaFreeMemory (indexName);
 }
 
+/*-----------------------------------------------------------------------
+   LINK_UpdateLink
+   Updates the local index reference of an annotation that was posted
+   but that still has local annotations attached to it.
+  -----------------------------------------------------------------------*/
+void LINK_UpdateLink (Document source_doc, ThotBool isReplyTo)
+{
+  char   *indexName, *doc_url;
+  List   *annot_list = NULL;
+  
+  annot_list = AnnotMetaData[source_doc].annotations;  
+  doc_url = DocumentURLs[source_doc];
+  indexName = LINK_GetAnnotationIndexFile (doc_url);
+}
+
 /*-----------------------------------------------------------------------
    LINK_DeleteLink
    For a given source doc, deletes the index entry from the main index and
    removes the documents index file.
   -----------------------------------------------------------------------*/
-void LINK_DeleteLink (Document source_doc)
+void LINK_DeleteLink (Document source_doc, ThotBool isReplyTo)
 {
   char   *doc_index;
   char buffer[255];
@@ -508,14 +727,17 @@ void LINK_DeleteLink (Document source_doc)
   char *main_index;
   char *main_index_file_old;
   char *main_index_file_new;
+  char *source_doc_url;
   int len;
   int error;
-  FILE *fp_old;
+  FILE *fp_old = NULL;
   FILE *fp_new;
   
 
   /* get the annotation index */
   doc_index = LINK_GetAnnotationIndexFile (DocumentURLs[source_doc]);
+  if (!doc_index)
+    doc_index = LINK_GetAnnotationIndexFile (AnnotMetaData[source_doc].annot_url);
   if (!doc_index)
     return;
 
@@ -544,24 +766,37 @@ void LINK_DeleteLink (Document source_doc)
   if (!(fp_new = fopen (main_index_file_new, "w")))
     error++;
 
-  if (!error && !(fp_old = fopen (main_index_file_old, "r")))
+  if (!error)
     {
-      fclose (fp_new);
-      error++;
+      if (!(fp_old = fopen (main_index_file_old, "r")))
+	{
+	  fclose (fp_new);
+	  error++;
+	}
     }
 
   if (!error)
     {
+      if (IsW3Path (DocumentURLs[source_doc])
+	  || IsFilePath (DocumentURLs[source_doc]))
+	source_doc_url = DocumentURLs[source_doc];
+      else
+	source_doc_url = ANNOT_MakeFileURL (DocumentURLs[source_doc]);
+		    
+		    
       /* search and remove the index entry */
       len = strlen (DocumentURLs[source_doc]);
       while (fgets (buffer, sizeof (buffer), fp_old))
 	{
-	  if (strncmp (DocumentURLs[source_doc], buffer, len))
+	  if (strncmp (source_doc_url, buffer, len))
 	    fputs (buffer, fp_new);
 	}
       fclose (fp_new);
       fclose (fp_old);
       
+      if (source_doc_url != DocumentURLs[source_doc])
+	TtaFreeMemory (source_doc_url);
+
       /* rename the main index file */
       TtaFileUnlink (main_index_file_old);
       rename (main_index_file_new, main_index_file_old);
@@ -578,41 +813,59 @@ void LINK_DeleteLink (Document source_doc)
   LINK_CreateMeta
   -----------------------------------------------------------------------*/
 AnnotMeta *LINK_CreateMeta (Document source_doc, Document annot_doc, AnnotMode mode)
+
 {
   AnnotMeta   *annot;
   char      *annot_user;
-  
+  char      *source_doc_url; /* the url we will use to refer to the source doc */
+
   /*
   **  Make a new annotation entry, add it to annotlist, and initialize it.
   */
-  annot =  AnnotMeta_new ();
-  if (!IsW3Path (DocumentURLs[annot_doc]) && 
+
+#ifdef ANNOT_ON_ANNOT
+  if (DocumentTypes[source_doc] == docAnnot && AnnotMetaData[source_doc].annot_url)
+    {
+      /* we reply to an annotation (using it's metadata). However, we annotate the
+	 body of an annotation, not its metadata */
+      if (mode & ANNOT_isReplyTo)
+	source_doc_url = AnnotMetaData[source_doc].annot_url;
+      else
+	source_doc_url = DocumentURLs[source_doc];
+    }
+  else
+#endif /* ANNOT_ON_ANNOT */
+    source_doc_url = DocumentURLs[source_doc];
+
+
+  /* download the local annotations, if they do exist, but mark them
+     invisible */
+  if (!IsW3Path (DocumentURLs[source_doc]) && 
       (!AnnotMetaData[source_doc].annotations 
        && !AnnotMetaData[source_doc].local_annot_loaded))
     {
       char *annotIndex;
 
-      /* download the local annotations, if they do exist, but mark them
-	 invisible */
-      annotIndex = LINK_GetAnnotationIndexFile (DocumentURLs[source_doc]);
+      annotIndex = LINK_GetAnnotationIndexFile (source_doc_url);
       LINK_LoadAnnotationIndex (source_doc, annotIndex, FALSE);
       TtaFreeMemory (annotIndex);
       AnnotMetaData[source_doc].local_annot_loaded = TRUE;
     }
 
-  if (IsW3Path (DocumentURLs[source_doc])
-      || IsFilePath(DocumentURLs[source_doc]))
-    annot->source_url = TtaStrdup (DocumentURLs[source_doc]);
+  annot =  AnnotMeta_new ();
+  if (IsW3Path (source_doc_url)
+      || IsFilePath(source_doc_url))
+    annot->source_url = TtaStrdup (source_doc_url);
   else
     {
-      annot->source_url = ANNOT_MakeFileURL (DocumentURLs[source_doc]);
+      annot->source_url = ANNOT_MakeFileURL (source_doc_url);
     }
   
   /* get the current date... cdate = mdate at this stage */
   annot->cdate = StrdupDate ();
   annot->mdate = TtaStrdup (annot->cdate);
 
-  /* Annotation type */
+  /* Annotation type -- will be reassigned below for Reply */
   annot->type = DEFAULT_ANNOTATION_TYPE;
 
   /* Annotation XPointer */
@@ -620,7 +873,7 @@ AnnotMeta *LINK_CreateMeta (Document source_doc, Document annot_doc, AnnotMode m
 
   annot_user = GetAnnotUser ();
   annot->author = TtaStrdup (annot_user);
-  annot->content_type = TtaStrdup ("text/html");
+  annot->content_type = TtaStrdup (AM_XHTML_MIME_TYPE);
   if (!IsW3Path (DocumentURLs[annot_doc]) 
       && !IsFilePath (DocumentURLs[annot_doc]))
     {
@@ -640,25 +893,33 @@ AnnotMeta *LINK_CreateMeta (Document source_doc, Document annot_doc, AnnotMode m
   LINK_CreateAName (annot);
 
 #ifdef ANNOT_ON_ANNOT
+  /* initialize the annot_url we will use to reference this annotation*/
+  AnnotMetaData[annot_doc].annot_url = TtaStrdup (annot->body_url);
+
   if (mode & ANNOT_isReplyTo)
     {
+      char *source_annot_url;
+
+      annot->type = DEFAULT_REPLY_TYPE;
+
+      source_annot_url = AnnotMetaData[source_doc].annot_url;
       /* initialize the thread info */
       if (!AnnotMetaData[source_doc].thread)
 	{
 	  AnnotMetaData[source_doc].thread = &AnnotThread[source_doc];
-	  AnnotThread[source_doc].rootOfThread = TtaStrdup (annot->source_url);
+	  AnnotThread[source_doc].rootOfThread = TtaStrdup (source_annot_url);
 	}
       AnnotMetaData[annot_doc].thread = AnnotMetaData[source_doc].thread;
       annot->thread = AnnotMetaData[source_doc].thread;
       AnnotMetaData[source_doc].thread->references++;
-      annot->inReplyTo = TtaStrdup (annot->source_url);
+      annot->inReplyTo = TtaStrdup (source_annot_url);
       /* maybe useless */
       annot->rootOfThread = TtaStrdup (AnnotMetaData[source_doc].thread->rootOfThread);
       /* and add the annotation to the list of threads */
       List_add (&(AnnotMetaData[source_doc].thread->annotations), (void *) annot);
     }
   else
-#endif /* ANONT_ON_ANNOT */
+#endif /* ANNOT_ON_ANNOT */
     {
       /* add the annotation to the list of annotations */
       List_add (&(AnnotMetaData[source_doc].annotations), (void *) annot);
@@ -674,16 +935,47 @@ AnnotMeta *LINK_CreateMeta (Document source_doc, Document annot_doc, AnnotMode m
   -----------------------------------------------------------------------*/
 void LINK_DelMetaFromMemory (Document doc)
 {
+  /* @@ JK remove this?
   if (!AnnotMetaData[doc].annotations)
     return;
+  */
 
   AnnotList_free (AnnotMetaData[doc].annotations);
   AnnotMetaData[doc].annotations = NULL;
+  AnnotMetaData[doc].local_annot_loaded = FALSE;
   /* delete any previous filters */
   AnnotFilter_deleteAll (doc);
   /* we no longer need this part of the RDF model; it holds only
      for the annotations of this document */
   SCHEMA_FreeRDFModel (&AnnotMetaData[doc].rdf_model);
+
+  if (AnnotMetaData[doc].annot_url)
+    {
+      TtaFreeMemory (AnnotMetaData[doc].annot_url);
+      AnnotMetaData[doc].annot_url = NULL;
+    }
+
+#ifdef ANNOT_ON_ANNOT 
+  /* @@ JK: maybe remove the references too */
+  AnnotMetaData[doc].thread = NULL;
+  
+  /* free the data associated with the thread */
+  if (AnnotThread[doc].annotations)
+    {
+      /* @@ JK: we need a function to erase all the annotations in the list */
+      /*
+	AnnotList_delAnnot (&(AnnotMetaData[source_doc].annotations),
+	annot->body_url, FALSE);
+      */
+      AnnotList_free (AnnotThread[doc].annotations);
+      AnnotThread[doc].annotations = NULL;
+    }
+  if (AnnotThread[doc].rootOfThread)
+    {
+      TtaFreeMemory (AnnotThread[doc].rootOfThread);
+      AnnotThread[doc].rootOfThread = NULL;
+    }
+#endif /* ANNOT_ON_ANNOT */
 }
 
 /*-----------------------------------------------------------------------
@@ -731,7 +1023,11 @@ void LINK_LoadAnnotationIndex (Document doc, char *annotIndex, ThotBool mark_vis
 
   while (list_ptr)
     {
+      char *url;
+      AnnotMetaDataSearch searchType;
+
       annot = (AnnotMeta *) list_ptr->object;
+      
       /* don't add an annotation if it's already on the list */
       /* @@ JK: later, Ralph will add code to delete the old one.
 	 We should remove the old annotations and preserve the newer
@@ -739,8 +1035,25 @@ void LINK_LoadAnnotationIndex (Document doc, char *annotIndex, ThotBool mark_vis
 	 and that we'll have to close it without saving... or just update
 	 the metadata... */
       
+      if (IsW3Path (annot->annot_url))
+	{
+	  url = annot->annot_url;
+	  searchType = AM_ANNOT_URL;
+	}
+      else
+	{
+	  url = annot->body_url;
+	  searchType = AM_BODY_URL;
+	}
       old_annot = AnnotList_searchAnnot (AnnotMetaData[doc].annotations,
-					 annot->annot_url, AM_ANNOT_URL);
+					     url, searchType);
+#ifdef ANNOT_ON_ANNOT
+      /* do the same thing to avoid duplicating the thread items */
+      if (!old_annot && AnnotThread[doc].annotations)
+	old_annot = AnnotList_searchAnnot (AnnotThread[doc].annotations,
+					   url, searchType);
+#endif /* ANNOT_ON_ANNOT */
+
       if (!old_annot)
 	{
 	  if (mark_visible)
@@ -755,7 +1068,7 @@ void LINK_LoadAnnotationIndex (Document doc, char *annotIndex, ThotBool mark_vis
 	  else
 	    annot->is_visible = FALSE;
 #ifdef ANNOT_ON_ANNOT
-	  if (annot->isReplyTo)
+	  if (annot->inReplyTo)
 	    {
 	      doc_thread = AnnotThread_searchRoot (annot->rootOfThread);
 	      /* if there's no other thread, then use the source doc as the
@@ -768,7 +1081,8 @@ void LINK_LoadAnnotationIndex (Document doc, char *annotIndex, ThotBool mark_vis
 	      else
 		thread = &AnnotThread[doc_thread];
 
-	      if (!thread)
+	      /* there was no thread. Create a new one if it's the same rootOfThread document */
+	      if (!thread && Annot_isSameURL (AnnotMetaData[doc].annot_url, annot->rootOfThread))
 		{
 		  /* add the root of thread (used by load index later on) */
 		  AnnotThread[doc].rootOfThread = 
@@ -776,12 +1090,18 @@ void LINK_LoadAnnotationIndex (Document doc, char *annotIndex, ThotBool mark_vis
 		  AnnotThread[doc].references = 1;
 		  thread = &AnnotThread[doc_thread];
 		}
+
 	      /* add and show the thread item */
-	      List_add (&(thread->annotations), (void *) annot);
-	      if (!AnnotMetaData[doc].thread)
-		AnnotMetaData[doc].thread = thread;
-	      annot->thread = thread;
-	      ANNOT_AddThreadItem (doc_thread, annot);
+	      if (thread)
+		{
+		  List_add (&(thread->annotations), (void *) annot);
+		  if (!AnnotMetaData[doc].thread)
+		    AnnotMetaData[doc].thread = thread;
+		  annot->thread = thread;
+		  /* ANNOT_AddThreadItem (doc_thread, annot); */
+		}
+	      else
+		Annot_free (annot);
 	    }
 	  else
 #endif /* ANNOT_ON_ANNOT */
@@ -792,6 +1112,13 @@ void LINK_LoadAnnotationIndex (Document doc, char *annotIndex, ThotBool mark_vis
       List_delFirst (&list_ptr);
     }
 
+#ifdef ANNOT_ON_ANNOT
+  /* erase and redisplay the thread items */
+  ANNOT_DeleteThread (doc);
+  if (AnnotThread[doc].annotations)
+    ANNOT_BuildThread (doc);
+#endif /* ANNOT_ON_ANNOT */
+
   /* show the document */
   if (dispMode == DisplayImmediately)
     TtaSetDisplayMode (doc, dispMode);
@@ -802,7 +1129,6 @@ void LINK_LoadAnnotationIndex (Document doc, char *annotIndex, ThotBool mark_vis
       InitInfo ("Annotation load", 
 		"There were some orphan annotations. You may See them with the Links view.");
     }
-
 }
 
 /*-----------------------------------------------------------------------
