@@ -15,16 +15,20 @@
  */
 
 /**
-w3c_algaeQuery=(ask '((?p ?s ?o)) :collect '(?p ?s ?o))
+   w3c_algaeQuery=(ask '((?p ?s ?o)) :collect '(?p ?s ?o))
+**/
 
- **/
-
-#define THOT_EXPORT extern
-#include "HTML.h"
+/* annotlib includes */
 #include "annotlib.h"
-#include "HTMLhistory_f.h"
 #include "ANNOTmenu.h"
-#
+#include "ANNOTschemas_f.h"
+#include "ANNOTevent_f.h"
+#include "ANNOTtools_f.h"
+
+/* amaya includes */
+#include "HTMLhistory_f.h"
+
+/* windows includes */
 #ifdef _WINDOWS
 #include "resource.h"
 #include "wininclude.h"
@@ -35,8 +39,11 @@ w3c_algaeQuery=(ask '((?p ?s ?o)) :collect '(?p ?s ?o))
 extern HINSTANCE hInstance;
 #endif /* _WINDOWS */
 
+/* schema includes */
+#include "XLink.h"
+
 /* common local variables */
-static CHAR_T  s[MAX_LENGTH]; /* general purpose buffer */
+static char  s[MAX_LENGTH]; /* general purpose buffer */
 
 /************************************************************
  ** Local custom query variables
@@ -44,7 +51,7 @@ static CHAR_T  s[MAX_LENGTH]; /* general purpose buffer */
 
 static int     CustomQueryBase;
 static ThotBool CustomQueryFlag;
-static CHAR_T  *AlgaeText;
+static char  *AlgaeText;
 
 /************************************************************
  ** Local annotation filter variables
@@ -54,7 +61,7 @@ static int      AnnotFilterBase;
 /* copies of the doc and view from which the menu was invoked */
 static Document AnnotFilterDoc;
 static View     AnnotFilterView;
-static CHAR_T   AnnotSelItem[MAX_LENGTH];
+static char   AnnotSelItem[MAX_LENGTH];
 static int      AnnotSelIndex;
 static SelType  AnnotSelType;
 
@@ -66,7 +73,7 @@ static HWND       FilterHwnd = NULL;
 
 typedef struct _typeSelector
 {
-  CHAR_T *name;
+  char *name;
   RDFResourceP type;
 } TypeSelector;
 
@@ -76,12 +83,12 @@ typedef struct _typeSelector
   callback of the annot custom query menu
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-static void         CustomQueryCallbackDialog (int ref, int typedata, CHAR_T *data)
+static void         CustomQueryCallbackDialog (int ref, int typedata, char *data)
 #else
 static void         CustomQueryCallbackDialog (ref, typedata, data)
 int                 ref;
 int                 typedata;
-CHAR_T              *data;
+char              *data;
 
 #endif /* __STDC__ */
 {
@@ -116,7 +123,7 @@ CHAR_T              *data;
 	      CustomQueryFlag = FALSE;
 	      TtaFreeMemory (AlgaeText);
 	      AlgaeText = NULL;
-	      TtaSetTextForm (CustomQueryBase + mFreeText, TEXT(""));
+	      TtaSetTextForm (CustomQueryBase + mFreeText, "");
 	      TtaSetMenuForm (CustomQueryBase + mExpertMode, 0);
 	      break;
 
@@ -175,7 +182,7 @@ View         view;
 {
 #ifndef _WINDOWS
    int              i;
-   CHAR_T          *ptr;
+   char          *ptr;
 
    /* initialize the base if it hasn't yet been done */
    if (!CustomQueryBase)
@@ -185,12 +192,12 @@ View         view;
    /* Create the dialogue form */
    i = 0;
    strcpy (&s[i], TtaGetMessage (AMAYA, AM_APPLY_BUTTON));
-   i += ustrlen (&s[i]) + 1;
+   i += strlen (&s[i]) + 1;
    strcpy (&s[i], TtaGetMessage (AMAYA, AM_DEFAULT_BUTTON));
 
    TtaNewSheet (CustomQueryBase + CustomQueryMenu,
 		TtaGetViewFrame (document, view),
-		TEXT("Query Customization Menu"),
+		"Query Customization Menu",
 		2, s, FALSE, 10, 'L', D_DONE);
 
    /* @@ JK: removed the following menus as they're not ready for the
@@ -198,54 +205,54 @@ View         view;
 #if 0
    TtaNewLabel (CustomQueryBase + mUsersGroups, 
 		CustomQueryBase + CustomQueryMenu,
-	       TEXT("Users and groups"));
+	       "Users and groups");
 
    TtaNewTextForm (CustomQueryBase + mUser1,
 		   CustomQueryBase + CustomQueryMenu,
-		   TEXT("user1"),
+		   "user1",
 		   20,
 		   1,
 		   TRUE);
 
    TtaNewTextForm (CustomQueryBase + mUser2,
 		   CustomQueryBase + CustomQueryMenu,
-		   TEXT("user2"),
+		   "user2",
 		   20,
 		   1,
 		   TRUE);
 
    TtaNewLabel (CustomQueryBase + mTime, 
 		CustomQueryBase + CustomQueryMenu,
-		TEXT("Time"));
+		"Time");
 
    TtaNewTextForm (CustomQueryBase + mBtime,
 		   CustomQueryBase + CustomQueryMenu,
-		   TEXT("Begin"),
+		   "Begin",
 		   20,
 		   1,
 		   TRUE);
 
    TtaNewTextForm (CustomQueryBase + mEtime,
 		   CustomQueryBase + CustomQueryMenu,
-		   TEXT("End"),
+		   "End",
 		   20,
 		   1,
 
    TtaNewLabel (CustomQueryBase + mExpertMode,
 		CustomQueryBase + CustomQueryMenu,
-		TEXT("Expert mode"));
+		"Expert mode");
 		   TRUE);
 #else
   /* create the radio buttons for choosing a selector */
   i = 0;
-  strcpy (&s[i], TEXT("BUse standard query"));
-  i += ustrlen (&s[i]) + 1;
-  strcpy (&s[i], TEXT("BUse free algae query with the following text"));
+  strcpy (&s[i], "BUse standard query");
+  i += strlen (&s[i]) + 1;
+  strcpy (&s[i], "BUse free algae query with the following text");
 
   TtaNewSubmenu (CustomQueryBase + mExpertMode,
 		 CustomQueryBase + CustomQueryMenu,
 		 0,
-		 TEXT("Query type"),
+		 "Query type",
 		 2,
 		 s,
 		 NULL,
@@ -254,7 +261,7 @@ View         view;
 
    TtaNewTextForm (CustomQueryBase + mFreeText,
 		   CustomQueryBase + CustomQueryMenu,
-		   TEXT("(%u stands for the URL of the document that's being browsed)"),
+		   "(%u stands for the URL of the document that's being browsed)",
 		   70,
 		   5,
 		   TRUE);
@@ -291,7 +298,7 @@ View         view;
   Filter.
   ------------------------------------------------------------------*/
 #ifdef _WINDOWS
-static void WIN_AnnotFilterNewSelector (Document doc, CHAR_T *entries, int nb_entries)
+static void WIN_AnnotFilterNewSelector (Document doc, char *entries, int nb_entries)
 {
   int index = 0;
   int i = 0;
@@ -304,7 +311,7 @@ static void WIN_AnnotFilterNewSelector (Document doc, CHAR_T *entries, int nb_en
       SendDlgItemMessage (FilterHwnd, IDC_FILTERSEL, LB_INSERTSTRING,
 		  i, (LPARAM) &entries[index]); 
       /* @@ JK: what does this mean? Longueur de l'intitule ?? */
-      index += ustrlen (&entries[index]) + 1;
+      index += strlen (&entries[index]) + 1;
       i++;
     }
 }
@@ -325,8 +332,8 @@ SelType  selector;
   int                   i;
   List                  *list_item;
   AnnotFilterData       *filter;
-  CHAR_T                *name;
-  CHAR_T                 status_char;
+  char                *name;
+  char                 status_char;
   int                    status;
 	   
 
@@ -348,7 +355,7 @@ SelType  selector;
     }
 
   nb_entries = 0;
-  ustrcpy (s, TEXT(""));
+  strcpy (s, "");
   i = 0;
   while (list_item)
      {
@@ -363,25 +370,25 @@ SelType  selector;
 	   switch (status)
 	     {
 	     case 2: /* partial */
-	       status_char = TEXT('-');
+	       status_char = '-';
 	       break;
 
 	     default:
 	     case 1: /* show */
-	       status_char = TEXT(' ');
+	       status_char = ' ';
 	       break;
 
 	     case 0: /* hide */
-	       status_char = TEXT('*');
+	       status_char = '*';
 	       break;
 	     }
 	   if (selector == BY_TYPE)
 	     name = ANNOT_GetLabel (&annot_schema_list,
 				    (RDFResourceP) filter->object);
 	   else
-	     name = (CHAR_T *) filter->object;
-	   usprintf (&s[i], TEXT("%c%s"), status_char, name);
-	   i += ustrlen (&s[i]) + 1;
+	     name = (char *) filter->object;
+	   sprintf (&s[i], "%c%s", status_char, name);
+	   i += strlen (&s[i]) + 1;
 	   nb_entries++;
 	 }
        list_item = list_item->next;
@@ -407,20 +414,20 @@ SelType  selector;
   ChangeAnnotVisibility
 ------------------------------------------------------------------*/
 #ifdef __STDC__
-static void ChangeAnnotVisibility (Document doc, SelType selector, CHAR_T *object, ThotBool show)
+static void ChangeAnnotVisibility (Document doc, SelType selector, char *object, ThotBool show)
 #else
 static void ChangeAnnotVisibility (doc, selector, object, show)
 Document doc;
 SelType selector;
-CHAR_T *object;
+char *object;
 ThotBool show;
 
 #endif /* __STDC__ */
 {
   List               *list_item;
   AnnotFilterData    *filter;
-  CHAR_T             *annot_url;
-  CHAR_T             *name;
+  char             *annot_url;
+  char             *name;
   AnnotMeta          *annot;
   int                 length;
   ThotBool            annot_show = TRUE;
@@ -436,10 +443,10 @@ ThotBool show;
   int		      position;
   int		      distance;
 
-  if (AnnotSelItem[0] == WC_EOS)
+  if (AnnotSelItem[0] == EOS)
     return;
 
-  XLinkSchema = TtaGetSSchema (TEXT("XLink"), doc);
+  XLinkSchema = TtaGetSSchema ("XLink", doc);
   if (!XLinkSchema)
     /* there are no xlinks in this document */
     return;
@@ -472,9 +479,9 @@ ThotBool show;
 	name = ANNOT_GetLabel (&annot_schema_list,
 			       (RDFResourceP) filter->object);
       else
-	name = (CHAR_T *) filter->object;
+	name = (char *) filter->object;
 
-      if (filter && !ustrcasecmp (name, object + 1))
+      if (filter && !strcasecmp (name, object + 1))
 	{
 	  filter->show = show;
 	  break;
@@ -495,7 +502,7 @@ ThotBool show;
     TtaSetDisplayMode (doc, NoComputedDisplay);
 
   /* initialize */
-  el = TtaGetMainRoot (doc);
+  el = TtaGetRootElement (doc);
   elType.ElSSchema = XLinkSchema;
   elType.ElTypeNum = XLink_EL_XLink;
   attrType.AttrSSchema = XLinkSchema;
@@ -510,7 +517,7 @@ ThotBool show;
 	/* this looks like an error! */
 	continue;
       length = TtaGetTextAttributeLength (attr) + 1;
-      annot_url = TtaAllocString (length);
+      annot_url = TtaGetMemory (length);
       TtaGiveTextAttributeValue (attr, annot_url, &length);
       
       /* now look in the filters to see if we need to hide it or not */
@@ -519,6 +526,12 @@ ThotBool show;
       TtaFreeMemory (annot_url);
       if (!annot || !(annot->is_visible) || annot->is_orphan)
 	continue;
+
+#ifdef ANNOT_ON_ANNOT
+      /* skip the reply annotations */
+      if (annot->inReplyTo)
+	continue;
+#endif /* ANNOT_ON_ANNOT */
       
       switch (selector) 
 	{
@@ -574,9 +587,9 @@ ThotBool show;
   BuildAnnotFilterSelector (doc, selector);
   /* update the selector text */
   if (show)
-    AnnotSelItem[0] = TEXT(' ');
+    AnnotSelItem[0] = ' ';
   else
-    AnnotSelItem[0] = TEXT(' ');
+    AnnotSelItem[0] = ' ';
 #ifndef _WINDOWS
   TtaSetSelector (AnnotFilterBase + mFilterSelector, -1, AnnotSelItem);
 #endif /* _WINDOWS */
@@ -606,7 +619,7 @@ ThotBool show;
   int		      position;
   int		      distance;
 
-  XLinkSchema = TtaGetSSchema (TEXT("XLink"), document);
+  XLinkSchema = TtaGetSSchema ("XLink", document);
   if (!XLinkSchema)
     /* there are no xlinks in this document */
     return;
@@ -641,7 +654,7 @@ ThotBool show;
     TtaSetDisplayMode (document, NoComputedDisplay);
 
   /* initialize */
-  el = TtaGetMainRoot (document);
+  el = TtaGetRootElement (document);
   elType.ElTypeNum = XLink_EL_XLink;
   elType.ElSSchema = XLinkSchema;
   attrType.AttrSSchema = XLinkSchema;
@@ -680,9 +693,9 @@ ThotBool show;
   /* redisplay the current selector */
   BuildAnnotFilterSelector (document, AnnotSelType);
   /* and clear the selector text */
-  AnnotSelItem[0] = WC_EOS;
+  AnnotSelItem[0] = EOS;
 #ifndef _WINDOWS
-  TtaSetSelector (AnnotFilterBase + mFilterSelector, -1, TEXT(""));
+  TtaSetSelector (AnnotFilterBase + mFilterSelector, -1, "");
 #endif /* !_WINDOWS */
 }
 
@@ -708,13 +721,34 @@ LPARAM lParam;
     {
     case WM_INITDIALOG:
 	  FilterHwnd = hwnDlg;
-	  /* write the dialogue text */
-      SetWindowText (hwnDlg, "Annotation Local Filter Menu");
-      SetWindowText (GetDlgItem (hwnDlg, ID_ANNOTSHOW), "Show");
-      SetWindowText (GetDlgItem (hwnDlg, ID_ANNOTHIDE), "Hide");
-      SetWindowText (GetDlgItem (hwnDlg, ID_ANNOTSHOWALL), "Show all");
-	  SetWindowText (GetDlgItem (hwnDlg, ID_ANNOTHIDEALL), "Hide all");
+	  /* 
+	  ** write the dialogue text
+	  */
+	  /* window title */
+      SetWindowText (hwnDlg, TtaGetMessage (AMAYA, AM_AFILTER));
+	  /* buttons */
+      SetWindowText (GetDlgItem (hwnDlg, ID_ANNOTSHOW), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_SHOW));
+      SetWindowText (GetDlgItem (hwnDlg, ID_ANNOTHIDE), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_HIDE));
+      SetWindowText (GetDlgItem (hwnDlg, ID_ANNOTSHOWALL), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_SHOW_ALL));
+      SetWindowText (GetDlgItem (hwnDlg, ID_ANNOTHIDEALL), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_HIDE_ALL));
       SetWindowText (GetDlgItem (hwnDlg, ID_DONE), TtaGetMessage (LIB, TMSG_DONE));
+      /* filter options */
+      SetWindowText (GetDlgItem (hwnDlg, IDC_TFILTEROPTIONS), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_OPTIONS));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_FILTERBYAUTHOR), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_BYAUTHOR));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_FILTERBYTYPE), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_BYTYPE));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_FILTERBYSERVER), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_BYSERVER));
+	  /* help label */
+	  SetWindowText (GetDlgItem (hwnDlg, IDC_TFILTERHELP), 
+		     TtaGetMessage (AMAYA, AM_AFILTER_HELP));
+
       /* display the by author items */
       BuildAnnotFilterSelector (AnnotFilterDoc, AnnotSelType);
       /* select the by author radio button */
@@ -821,12 +855,12 @@ LPARAM lParam;
    callback of the AnnotFilter menu
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-static void         AnnotFilterCallbackDialog (int ref, int typedata, CHAR_T * data)
+static void         AnnotFilterCallbackDialog (int ref, int typedata, char * data)
 #else
 static void         AnnotFilterCallbackDialog (ref, typedata, data)
 int                 ref;
 int                 typedata;
-CHAR_T             *data;
+char             *data;
 #endif /* __STDC__ */
 {
   int val;
@@ -874,9 +908,9 @@ CHAR_T             *data;
 	case mFilterSelector:
 	  /* copy what was selected */
 	  if (data)
-	    ustrcpy (AnnotSelItem, data);
+	    strcpy (AnnotSelItem, data);
 	  else
-	    AnnotSelItem[0] = WC_EOS;
+	    AnnotSelItem[0] = EOS;
 	  break;
 	  
 	case mSelectFilter:
@@ -884,12 +918,12 @@ CHAR_T             *data;
 	  /* @@ here I need to have a pointer in memory to the
 	     annotation document... means I'll only be able to have
 	     one such annotation dialogue at the time */
-	  if (AnnotSelType != val)
+	  if (AnnotSelType != (SelType) val)
 	    {
-	      AnnotSelType = val;
-	      AnnotSelItem[0] = WC_EOS;
+	      AnnotSelType = (SelType) val;
+	      AnnotSelItem[0] = EOS;
 	      BuildAnnotFilterSelector (AnnotFilterDoc, val);
-	      TtaSetSelector (AnnotFilterBase + mFilterSelector, -1, TEXT(""));
+	      TtaSetSelector (AnnotFilterBase + mFilterSelector, -1, "");
 	    }
 	  break;
 
@@ -932,48 +966,53 @@ View                view;
      find this info in the callback */
   AnnotFilterDoc = document;
   AnnotFilterView = view;
-  AnnotSelItem[0] = WC_EOS;
+  AnnotSelItem[0] = EOS;
   AnnotSelIndex = -1;
   AnnotSelType = 0;
 
   /* Create the dialogue form */
 #ifndef _WINDOWS
   i = 0;
-  strcpy (&s[i], TEXT("Show"));
-  i += ustrlen (&s[i]) + 1;
-  strcpy (&s[i], TEXT("Hide"));
-  i += ustrlen (&s[i]) + 1;
-  strcpy (&s[i], TEXT("Show all"));
-  i += ustrlen (&s[i]) + 1;
-  strcpy (&s[i], TEXT("Hide all"));
+  strcpy (&s[i], TtaGetMessage (AMAYA, AM_AFILTER_SHOW));
+  i += strlen (&s[i]) + 1;
+  strcpy (&s[i], TtaGetMessage (AMAYA, AM_AFILTER_HIDE));
+  i += strlen (&s[i]) + 1;
+  strcpy (&s[i], TtaGetMessage (AMAYA, AM_AFILTER_SHOW_ALL));
+  i += strlen (&s[i]) + 1;
+  strcpy (&s[i], TtaGetMessage (AMAYA, AM_AFILTER_HIDE_ALL));
 
   TtaNewSheet (AnnotFilterBase + AnnotFilterMenu, 
 	       TtaGetViewFrame (document, view),
-	       TEXT("Annotation Filter  "), 4, s, TRUE, 2, 'L', 
+	       TtaGetMessage (AMAYA, AM_AFILTER), 4, s, TRUE, 2, 'L', 
 	       D_DONE);
   
   /* an empty text */
   TtaNewLabel (AnnotFilterBase + mAnnotFilterEmpty1,
 	       AnnotFilterBase + AnnotFilterMenu,
-	       TEXT("                 "));
+	       "                 ");
 
   /* the * = filter message */
   TtaNewLabel (AnnotFilterBase + mAnnotFilterLabelStars,
 	       AnnotFilterBase + AnnotFilterMenu,
-	       TEXT("     a * prefix means hidden"));
+	       TtaGetMessage (AMAYA, AM_AFILTER_HELP));
 	       
   /* create the radio buttons for choosing a selector */
-  i = 0;
-  strcpy (&s[i], TEXT("BBy author"));
-  i += ustrlen (&s[i]) + 1;
-  strcpy (&s[i], TEXT("BBy type"));
-  i += ustrlen (&s[i]) + 1;
-  strcpy (&s[i], TEXT("BBy server"));
+  s[0] = 'B';
+  i = 1;
+  strcpy (&s[i], TtaGetMessage (AMAYA, AM_AFILTER_BYAUTHOR));
+  i += strlen (&s[i]) + 1;
+  s[i] = 'B';
+  i++;
+  strcpy (&s[i], TtaGetMessage (AMAYA, AM_AFILTER_BYTYPE));
+  i += strlen (&s[i]) + 1;
+  s[i] = 'B';
+  i++;
+  strcpy (&s[i], TtaGetMessage (AMAYA, AM_AFILTER_BYSERVER));
 
   TtaNewSubmenu (AnnotFilterBase + mSelectFilter, 
 		 AnnotFilterBase + AnnotFilterMenu,
 		 0,
-		 TEXT("Filter options"),
+		 TtaGetMessage (AMAYA, AM_AFILTER_OPTIONS),
 		 3,
 		 s,
 		 NULL,
@@ -1022,10 +1061,15 @@ Document doc;
   nb_entries = 0;
   if (typesList)
     List_delAll (&typesList, List_delCharObj);
-  ustrcpy (s, TEXT(""));
+  strcpy (s, "");
   i = 0;
 
-  annotClass = ANNOTATION_CLASS;
+#ifdef ANNOT_ON_ANNOT
+  if (Annot_IsReplyTo (doc))
+    annotClass = THREAD_REPLY_CLASS;
+  else
+#endif /* ANNOT_ON_ANNOT */
+    annotClass = ANNOTATION_CLASS;
 
   if (annotClass && annotClass->class)
     {
@@ -1040,9 +1084,9 @@ Document doc;
 	  t->name = ANNOT_GetLabel(&annot_schema_list, subType);
 	  List_add (&typesList, (void *) t);
 
-	  usprintf (&s[i], "B%s", t->name);
-	  i += ustrlen (&s[i]);
-	  s[i] = WC_EOS;
+	  sprintf (&s[i], "B%s", t->name);
+	  i += strlen (&s[i]);
+	  s[i] = EOS;
 	  i++;
 	  nb_entries++;
 	}
@@ -1056,9 +1100,9 @@ Document doc;
 	t->name = ANNOT_GetLabel(&annot_schema_list, annotClass);
 	List_add (&typesList, (void *) t);
 
-	ustrcpy (s, t->name);
-	i = ustrlen (s);
-	s[i] = WC_EOS;
+	strcpy (s, t->name);
+	i = strlen (s);
+	s[i] = EOS;
 	nb_entries = 1;
       }
 
