@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1999-2001.
+ *  (c) COPYRIGHT MIT and INRIA, 1999-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  * 
  */
@@ -653,14 +653,13 @@ AnnotMeta *AnnotList_searchAnnot (List *list, char *url, AnnotMetaDataSearch sea
 	  break;
 
 #if 0 /* This case is no longer necessary given FixFileURL() */
-#endif
-
 	case AM_BODY_FILE:
 	  if (annot->body_url && IsFilePath (annot->body_url))
 	    ptr = annot->body_url + sizeof ("file://") - 1;
 	  else
 	    ptr = annot->body_url;
 	  break;
+#endif
 
 	case AM_ANAME:
 	  ptr = annot->name;
@@ -737,6 +736,45 @@ char *AnnotList_searchAnnotURL (Document source_doc, char *body_url)
     return NULL;
 }
 
+/*------------------------------------------------------------
+   AnnotList_searchBody
+   Returns the annot meta corresponding to the body URL
+   or NULL if it doesn't exist
+   ------------------------------------------------------------*/
+AnnotMeta *AnnotList_searchBody (Document annot_doc)
+{
+  AnnotMeta *annot;
+  char *body_url;
+  char *ptr;
+  Document source_doc;
+  
+  if (DocumentTypes[annot_doc] != docAnnot)
+    return NULL;
+
+  source_doc = DocumentMeta[annot_doc]->source_doc;
+  body_url = DocumentURLs[annot_doc];
+
+  if (!IsW3Path (body_url) && !IsFilePath (body_url))
+    ptr = ANNOT_MakeFileURL (body_url);
+  else
+    ptr = body_url;
+
+  annot = AnnotList_searchAnnot (AnnotMetaData[source_doc].annotations,
+				 ptr,
+				 AM_BODY_URL);
+#ifdef ANNOT_ON_ANNOT
+  if (!annot && AnnotThread[source_doc].annotations)
+    annot = AnnotList_searchAnnot (AnnotThread[source_doc].annotations,
+				   ptr,
+				   AM_BODY_URL);
+#endif /* ANNOT_ON_ANNOT */
+
+  if (ptr != body_url)
+    TtaFreeMemory (ptr);
+
+  return (annot);
+}
+
 /*------------------------------------------------------------
   AnnotList_delAnnot
   Searches for annotation with URL url and, if found, deletes it
@@ -942,20 +980,26 @@ Document AnnotThread_searchThreadDoc (char *annot_url)
 #ifdef ANNOT_ON_ANNOT
   int i;
   AnnotMetaDataSearch searchType;
+  char *tmp_url;
 
-  if (IsW3Path (annot_url))
-    searchType = AM_BODY_URL;
+  if (IsW3Path (annot_url) || IsFilePath (annot_url))
+    tmp_url = annot_url;
   else
-    searchType = AM_BODY_FILE;
+    tmp_url = LocalToWWW (annot_url);
+  searchType = AM_BODY_URL;
 
   for (i = 0; i < DocumentTableLength; i++)
     {
       if (!AnnotThread[i].rootOfThread)
 	continue;
       if (AnnotList_searchAnnot (AnnotThread[i].annotations, 
-				 annot_url, searchType))
+				 tmp_url, searchType))
 	break;
     }
+
+  if (tmp_url != annot_url)
+    TtaFreeMemory (tmp_url);
+
   return (i == DocumentTableLength) ? 0 : i;
 #else
   return 0;
@@ -1615,7 +1659,7 @@ static void  Annot_dumpCommonMeta (AnnotMeta *annot, FILE *fp)
 #ifdef _I18N_
       tmp = annot->title;
 #else
-      tmp = TtaConvertIsoToMbs (annot->title, ISO_8859_1);
+      tmp = TtaConvertByteToMbs (annot->title, ISO_8859_1);
 #endif /* _I18N_ */
       fprintf (fp,
 	       "<d:title>%s</d:title>\n",
@@ -1630,7 +1674,7 @@ static void  Annot_dumpCommonMeta (AnnotMeta *annot, FILE *fp)
 #ifdef _I18N_
       tmp = annot->author;
 #else
-      tmp = TtaConvertIsoToMbs (annot->author, ISO_8859_1);
+      tmp = TtaConvertByteToMbs (annot->author, ISO_8859_1);
 #endif /* _I18N_ */
       fprintf (fp,
 	       "<d:creator>%s</d:creator>\n",
@@ -1748,6 +1792,7 @@ char * ANNOT_PreparePostBody (Document doc)
   char tmp_str[80];
   char *rdf_tmpfile, *ptr;
   char *html_tmpfile;
+  char *content_type;
 
   AnnotMeta *annot;
   unsigned long content_length;
@@ -1793,13 +1838,15 @@ char * ANNOT_PreparePostBody (Document doc)
   Annot_dumpCommonMeta (annot, fp);
 
   /* the body of the annotation prologue */
+  content_type = (DocumentMeta[doc]->content_type) 
+    ? DocumentMeta[doc]->content_type : "text/html";
   fprintf (fp,
 	   "<a:body>\n"
 	   "<r:Description>\n"
 	   "<http:ContentType>%s</http:ContentType>\n"
 	   "<http:ContentLength>%ld</http:ContentLength>\n"
 	   "<http:Body r:parseType=\"Literal\">\n",
-	   "text/html",
+	   content_type,
 	   content_length);
 
  /* 
@@ -2228,11 +2275,11 @@ void ANNOT_SetType (Document doc, RDFResourceP type)
   Element          el;
   ElementType      elType;
   char          *url;
-  char          *ptr;
   int              i;
   AnnotMeta       *annot;
   char          *type_name;
-  
+  ThotBool       in_thread = FALSE;
+
   if (!type)
     return;
 
@@ -2251,55 +2298,43 @@ void ANNOT_SetType (Document doc, RDFResourceP type)
   TtaSetTextContent (el, type_name,
 		     TtaGetDefaultLanguage (), doc);
 
-  /* update the metadata. We need to find the source document to find
-   the metadata */
-  el = TtaGetRootElement (doc);
-  elType.ElTypeNum = Annot_EL_SourceDoc;
-  el = TtaSearchTypedElement (elType, SearchInTree, el);
-  if (!el)
-    return;
-  url = SearchAttributeInEl (doc, el, Annot_ATTR_HREF_, "Annot");
-  if (!url)
-    return;
-  ptr = strchr (url, '#');
-  if (ptr)
-    *ptr = EOS;
-  if (IsFilePath (url))
-    WWWToLocal (url);
+  /* update the metadata */
+  if (!IsFilePath (DocumentURLs[doc])
+      && !IsW3Path (DocumentURLs[doc]))
+    url = ANNOT_MakeFileURL (DocumentURLs[doc]);
+  else
+    url = DocumentURLs[doc];
+
   for (i = 1; i <= DocumentTableLength; i++)
     {
-      if (DocumentURLs[i] && !strcmp (url, DocumentURLs[i]))
-	{
-	  /* we found the source document, we now search and update
-	     the annotation meta data */
-	  /* @@ this doesn't work yet... we need to make file:/// here
-	   and I need to remove the document modified thingy, as we have
-	  to tell the user that he needs to save the document */
-	  if (!IsW3Path (DocumentURLs[doc]) 
-	      && !IsFilePath (DocumentURLs[doc]))
-	    {
-	      /* @@ add the file:// (why it wasn't there before? */
-	      ptr = ANNOT_MakeFileURL (DocumentURLs[doc]);
-	    }
-	  else
-	    ptr = NULL;
-	  annot = AnnotList_searchAnnot (AnnotMetaData[i].annotations,
-					 (ptr) ? ptr : DocumentURLs[doc],
-					 AM_BODY_URL);
+      /* find the annotation link in the source document that corresponds
+	 to this annotation */
+      annot = AnnotList_searchAnnot (AnnotMetaData[i].annotations,
+				     url, AM_BODY_URL);
 #ifdef ANNOT_ON_ANNOT
-	  if (!annot && AnnotMetaData[i].thread)
-	    annot = AnnotList_searchAnnot (AnnotMetaData[i].thread->annotations,
-					   (ptr) ? ptr : DocumentURLs[doc],
-					   AM_BODY_URL);
-#endif /* ANNOT_ON_ANNOT */
-	  if (ptr)
-	    TtaFreeMemory (ptr);
+      if (!annot && AnnotMetaData[i].thread)
+	{
+	  annot = AnnotList_searchAnnot (AnnotMetaData[i].thread->annotations,
+					 url, AM_BODY_URL);
 	  if (annot)
-	      annot->type = type;
-	  break;
+	    in_thread = TRUE;
 	}
+#endif /* ANNOT_ON_ANNOT */
+      if (annot)
+	break;
     }
-  TtaFreeMemory (url);
+
+  if (annot)
+    {
+      annot->type = type;
+#ifdef ANNOT_ON_ANNOT
+      if (in_thread)
+	/* update the reply type in the thread */
+	ANNOT_UpdateThreadItem (doc, annot, annot->body_url);
+#endif /* ANNOT_ON_ANNOT */
+    }
+  if (url != DocumentURLs[doc])
+    TtaFreeMemory (url);
 }
 
 /*-----------------------------------------------------------------------
@@ -2384,15 +2419,60 @@ Element ANNOT_GetHTMLRoot (Document doc, ThotBool getFirstChild)
     }
   else 
     el = NULL;
-
+  
   return (el);
 }
 
 /*-----------------------------------------------------------------------
-  ANNOT_CreateHTMLRoot
+  ANNOT_GetBodySSchema
+  Returns the S Schema associated with the body.
+  -----------------------------------------------------------------------*/
+SSchema ANNOT_GetBodySSchema (Document doc)
+{
+  ElementType elType;
+  Element     el;
+  SSchema     result = NULL;
+
+  el = ANNOT_GetHTMLRoot (doc, TRUE);
+
+  if (el)
+    {
+      elType = TtaGetElementType (el);
+      result = elType.ElSSchema;
+    }
+
+  return result;
+}
+
+/*-----------------------------------------------------------------------
+  ANNOT_GetBodySSchemaName
+  Returns the S Schema name associated with the body.
+  -----------------------------------------------------------------------*/
+char * ANNOT_GetBodySSchemaName (Document doc)
+{
+  ElementType elType;
+  Element     el;
+  char       *result = NULL;
+
+  el = ANNOT_GetHTMLRoot (doc, TRUE);
+
+  if (el)
+    {
+      elType = TtaGetElementType (el);
+      result = TtaGetSSchemaName (elType.ElSSchema);
+    }
+
+  if (!result)
+    result = "";
+
+  return result;
+}
+
+/*-----------------------------------------------------------------------
+  ANNOT_CreateBodyTree
   Adds the first HTML element to the annotation body.
   -----------------------------------------------------------------------*/
-void ANNOT_CreateHTMLTree (Document doc)
+void ANNOT_CreateBodyTree (Document doc, DocumentType bodyType)
 {
   ElementType elType;
   Element body, el;
@@ -2405,14 +2485,34 @@ void ANNOT_CreateHTMLTree (Document doc)
       elType = TtaGetElementType (el);
       elType.ElTypeNum = Annot_EL_Body;
       body = TtaSearchTypedElement (elType, SearchInTree, el);
-      body = TtaGetLastChild (el);
-      elType.ElSSchema = GetXHTMLSSchema (doc);
-      elType.ElTypeNum = HTML_EL_HTML;
+      /* body = TtaGetLastChild (el); */
+      if (bodyType == docSVG)
+	{
+	  elType.ElSSchema = GetSVGSSchema (doc);
+	  elType.ElTypeNum = SVG_EL_SVG;
+	}
+      else if (bodyType == docMath)
+	{
+	  elType.ElSSchema = GetMathMLSSchema (doc);
+	  elType.ElTypeNum = MathML_EL_MathML;
+	}
+      else if (bodyType == docText)
+	{
+	  elType.ElSSchema =  GetTextSSchema (doc);
+	  elType.ElTypeNum = TextFile_EL_TextFile;
+	}
+      else /* consider the default case to be HTML */
+	{
+	  elType.ElSSchema = GetXHTMLSSchema (doc);
+	  elType.ElTypeNum = HTML_EL_HTML;
+	}
       el = TtaNewTree (doc, elType, "");
       result = TtaCanInsertFirstChild (elType, body, doc);
       oldStructureChecking = TtaGetStructureChecking (doc);
       TtaSetStructureChecking (FALSE, doc);
       TtaInsertFirstChild (&el, body, doc);
+      /* set attribute dir on the Document element */
+      AddDirAttributeToDocEl (doc);
       TtaSetStructureChecking (oldStructureChecking, doc);
     }
 }
@@ -2426,7 +2526,7 @@ void WWWToLocal (char *url)
 {
   char *tmp;
   char *tmp_url;
-  char target[10];
+  char target[MAX_LENGTH];
 
   ThotBool free_tmp_url;
 
@@ -2601,3 +2701,109 @@ Document Annot_IsDocumentLoaded (Document annot_doc, char *source_annot_url, cha
 
   return doc;
 }
+
+/*-----------------------------------------------------------------------
+  Annot_bodyType
+  Returns the document type of the body of an annotation
+  -----------------------------------------------------------------------*/
+DocumentType ANNOT_bodyType (Document annot_doc)
+{
+  Document source_doc;
+  AnnotMeta *annot;
+
+  source_doc = DocumentMeta[annot_doc]->source_doc;
+  annot =  GetMetaData (source_doc, annot_doc);
+  if (annot)
+    return (annot->bodyType);
+  else
+    return docHTML;
+}
+
+/*-----------------------------------------------------------------------
+  Annot_bodyType_set
+  Sets the document type of the body of an annotation
+  -----------------------------------------------------------------------*/
+void ANNOT_bodyType_set (Document annot_doc, DocumentType bodyType)
+{
+  Document source_doc;
+  AnnotMeta *annot = NULL;
+  int i;
+
+  if (DocumentMeta[annot_doc] && DocumentMeta[annot_doc]->source_doc != 0)
+    {
+      source_doc = DocumentMeta[annot_doc]->source_doc;
+      annot =  GetMetaData (source_doc, annot_doc);
+    }
+  else /* we don't know the source document, we scan everywhere to find it */
+    {
+      for (i=1; i< MAX_DOCUMENTS; i++)
+	{
+	  if (i == annot_doc)
+	    continue;
+	  if (!DocumentMeta[i])
+	    continue;
+	  annot = GetMetaData (i, annot_doc);
+	  if (annot)
+	    break;
+	}
+    }
+
+  if (annot)
+    annot->bodyType = bodyType;
+}
+
+/*-----------------------------------------------------------------------
+  Annot_SetXMLBody
+  Changes the setting of the document to say that the body of the
+  annotation is a document of type XML.
+  -----------------------------------------------------------------------*/
+void Annot_SetXMLBody (Document doc)
+{
+  Element el;
+  Attribute      attr;
+  AttributeType  attrType;
+  int            oldStructureChecking;
+
+  if (DocumentTypes[doc] != docAnnot)
+    return;
+
+  el = TtaGetMainRoot (doc);
+
+  attrType.AttrSSchema = TtaGetDocumentSSchema (doc);
+  attrType.AttrTypeNum =  Annot_ATTR_XmlBody;
+
+  attr = TtaGetAttribute (el, attrType); 
+
+  if (!attr)  /* attach a new attribute */
+    {
+      oldStructureChecking = TtaGetStructureChecking (doc);
+      TtaSetStructureChecking (0, doc);
+      attr = TtaNewAttribute (attrType);
+      TtaAttachAttribute (el, attr, doc);
+      TtaSetStructureChecking ((ThotBool) oldStructureChecking, doc);
+    }
+}
+
+/*-----------------------------------------------------------------------
+  Annot_DocumentURL
+  Returns a string that has a copy of the document URL. If the document
+  has any parameters, they are concatenated to it.
+  The caller has to free the returned pointer.
+  -----------------------------------------------------------------------*/
+char * Annot_DocumentURL (Document doc)
+{
+ 
+  char *ptr;
+
+  if (DocumentMeta[doc] && DocumentMeta[doc]->form_data)
+    {
+      int len;
+      len = strlen (DocumentURLs[doc] + strlen (DocumentMeta[doc]->form_data) + 2);
+      ptr = TtaGetMemory (len);
+      sprintf (ptr, "%s?%s", DocumentURLs[doc], DocumentMeta[doc]->form_data);
+    }
+  else
+    ptr = TtaStrdup (DocumentURLs[doc]);
+
+  return ptr;
+}
