@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA 1996-2001
+ *  (c) COPYRIGHT INRIA 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -359,7 +359,7 @@ void GetNextToken (indLine start, indLine *wi, indLine *wl, SyntacticType *wn)
 		       (inputLine[j] >= '0' && inputLine[j] <= '9'
 		       should be:
 		       if (iswalnum (inputLine[j])) to see if inputLine[j] is
-		       alphabetical or numerical value @@@@@@@@@@ */
+		       scriptical or numerical value @@@@@@@@@@ */
 
 		    if (!((inputLine[j] >= 'A' && inputLine[j] <= 'Z') || 
 			  (inputLine[j] >= 'a' && inputLine[j] <= 'z') || 
@@ -822,7 +822,7 @@ void InitSyntax (char *fileName)
   char                pnomcourt[200];
   int                 l;
   int                 ruleptr;
-  int                 currule;
+  int                 currule = 1;
   ThotBool            readingKeywordTable;
   ThotBool             fileOK;
 
@@ -847,8 +847,7 @@ void InitSyntax (char *fileName)
 	  j = 0;
 	  do
 	    {
-	      /* fileOK = TtaReadByte (grmFile, &inputLine[j]); */
-	      fileOK = TtaReadWideChar (grmFile, &inputLine[j], ISO_8859_1);
+	      fileOK = TtaReadByte (grmFile, &inputLine[j]);
 	      j++;
 	    }
 #ifdef _WINDOWS
