@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2000
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -23,9 +23,10 @@
 #define THOT_EXPORT
 #include "platform_tv.h"
 
-#include "readprs_f.h"
-#include "memory_f.h"
 #include "fileaccess_f.h"
+#include "memory_f.h"
+#include "readprs_f.h"
+#include "readstr_f.h"
 
 static PtrSSchema   pSchemaStr;
 static PtrPSchema   pSchemaPrs;
@@ -38,25 +39,16 @@ CntrItem           *pCp1;
 PresConstant       *pPr1;
 PresVariable       *pPres1;
 PresVarItem        *pVa1;
-PresentationBox    *pBo1;
+PtrPresentationBox  pBo1;
 TtAttribute        *pAt1;
 AttributePres      *pRP1;
 NumAttrCase        *pCa1;
-ThotBool             Transm;
+ThotBool            Transm;
 
 /*----------------------------------------------------------------------
    wrnb ecrit au terminal l'entier nb.                             
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnb (int nb)
-
-#else  /* __STDC__ */
-static void         wrnb (nb)
-int                 nb;
-
-#endif /* __STDC__ */
-
 {
    if (nb < 0)
      {
@@ -77,20 +69,10 @@ int                 nb;
       printf ("999999999");
 }
 
-
 /*----------------------------------------------------------------------
    wrlevel ecrit au terminal le niveau relatif n.                 
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrlevel (Level n)
-
-#else  /* __STDC__ */
-static void         wrlevel (n)
-Level               n;
-
-#endif /* __STDC__ */
-
 {
    switch (n)
 	 {
@@ -130,16 +112,7 @@ Level               n;
 /*----------------------------------------------------------------------
    wrdistunit ecrit le nom d'une unite' de distance.               
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrdistunit (TypeUnit u)
-
-#else  /* __STDC__ */
-static void         wrdistunit (u)
-TypeUnit            u;
-
-#endif /* __STDC__ */
-
 {
    switch (u)
 	 {
@@ -164,20 +137,10 @@ TypeUnit            u;
 	 }
 }
 
-
 /*----------------------------------------------------------------------
    wrrepere ecrit au terminal le repere de boite r.                
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrrepere (BoxEdge r)
-
-#else  /* __STDC__ */
-static void         wrrepere (r)
-BoxEdge             r;
-
-#endif /* __STDC__ */
-
 {
    switch (r)
 	 {
@@ -211,20 +174,10 @@ BoxEdge             r;
 	 }
 }
 
-
 /*----------------------------------------------------------------------
    wrnom ecrit au terminal le nom n.                               
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnom (Name n)
-
-#else  /* __STDC__ */
-static void         wrnom (n)
-Name                n;
-
-#endif /* __STDC__ */
-
 {
    int                 i;
 
@@ -249,38 +202,19 @@ Name                n;
 /*----------------------------------------------------------------------
    wrnomregle ecrit au terminal le nom de la regle de numero r.    
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnomregle (int r)
-
-#else  /* __STDC__ */
-static void         wrnomregle (r)
-int                 r;
-
-#endif /* __STDC__ */
-
 {
    if (r > 0)
-      wrnom (pSchemaStr->SsRule[r - 1].SrName);
+      wrnom (pSchemaStr->SsRule->SrElem[r - 1]->SrName);
 }
 
-
 /*----------------------------------------------------------------------
    wrnomattr ecrit au terminal le nom de l'attribut de numero a.   
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnomattr (int a)
-
-#else  /* __STDC__ */
-static void         wrnomattr (a)
-int                 a;
-
-#endif /* __STDC__ */
-
 {
    if (a != 0)
-      wrnom (pSchemaStr->SsAttribute[abs (a) - 1].AttrName);
+      wrnom (pSchemaStr->SsAttribute->TtAttr[abs (a) - 1]->AttrName);
 }
 
 
@@ -288,36 +222,16 @@ int                 a;
    wrnomboite ecrit au terminal le nom de la boite de presentation 
    de numero b.                                            
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnomboite (int b)
-
-#else  /* __STDC__ */
-static void         wrnomboite (b)
-int                 b;
-
-#endif /* __STDC__ */
-
 {
    if (b > 0)
-      wrnom (pSchemaPrs->PsPresentBox[b - 1].PbName);
+      wrnom (pSchemaPrs->PsPresentBox->PresBox[b - 1]->PbName);
 }
 
-
 /*----------------------------------------------------------------------
    wrModeHerit ecrit au terminal un mode d'heritage.               
   ----------------------------------------------------------------------*/
-
-
-#ifdef __STDC__
 static void         wrModeHerit (InheritMode M)
-
-#else  /* __STDC__ */
-static void         wrModeHerit (M)
-InheritMode         M;
-
-#endif /* __STDC__ */
-
 {
    switch (M)
 	 {
@@ -339,20 +253,10 @@ InheritMode         M;
 	 }
 }
 
-
 /*----------------------------------------------------------------------
    wrsize ecrit au terminal la regle de taille pointee par pR.     
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrsize (PtrPRule pR)
-
-#else  /* __STDC__ */
-static void         wrsize (pR)
-PtrPRule            pR;
-
-#endif /* __STDC__ */
-
 {
    if (pR->PrPresMode == PresInherit)
      {
@@ -404,21 +308,11 @@ PtrPRule            pR;
    printf (";");
 }
 
-
 /*----------------------------------------------------------------------
    wrfontstyle ecrit au terminal la regle d'heritage ou la valeur  
    entiere de la regle pointee par pR.                     
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrfontstyle (PtrPRule pR)
-
-#else  /* __STDC__ */
-static void         wrfontstyle (pR)
-PtrPRule            pR;
-
-#endif /* __STDC__ */
-
 {
    if (pR->PrPresMode == PresInherit)
      {
@@ -516,6 +410,35 @@ PtrPRule            pR;
 		     putchar (pR->PrChrValue);
 		     break;
 	       }
+      else if (pR->PrType == PtDirection)
+	 switch (pR->PrChrValue)
+	       {
+		  case 'L':
+		     printf ("ltr");
+		     break;
+		  case 'R':
+		     printf ("rtl");
+		     break;
+		  default:
+		     putchar (pR->PrChrValue);
+		     break;
+	       }
+      else if (pR->PrType == PtUnicodeBidi)
+	 switch (pR->PrChrValue)
+	       {
+		  case 'N':
+		     printf ("Normal");
+		     break;
+		  case 'E':
+		     printf ("Embed");
+		     break;
+		  case 'O':
+		     printf ("Override");
+		     break;
+		  default:
+		     putchar (pR->PrChrValue);
+		     break;
+	       }
       else if (pR->PrType == PtLineStyle)
 	 switch (pR->PrChrValue)
 	       {
@@ -532,6 +455,28 @@ PtrPRule            pR;
 		     putchar (pR->PrChrValue);
 		     break;
 	       }
+      else if (pR->PrType == PtDisplay)
+	 switch (pR->PrChrValue)
+	       {
+		  case 'I':
+		     printf ("inline");
+		     break;
+		  case 'B':
+		     printf ("block");
+		     break;
+		  case 'L':
+		     printf ("listItem");
+		     break;
+		  case 'R':
+		     printf ("runIn");
+		     break;
+		  case 'C':
+		     printf ("compact");
+		     break;
+		  case 'M':
+		     printf ("marker");
+		     break;
+	       }
       else if (pR->PrType == PtBorderTopStyle ||
 	       pR->PrType == PtBorderRightStyle ||
 	       pR->PrType == PtBorderBottomStyle ||
@@ -580,21 +525,11 @@ PtrPRule            pR;
    printf (";");
 }
 
-
 /*----------------------------------------------------------------------
    wrnbherit ecrit au terminal la regle d'heritage ou la valeur    
    entiere de la regle pointee par pR.                     
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrnbherit (PtrPRule pR)
-
-#else  /* __STDC__ */
-static void         wrnbherit (pR)
-PtrPRule            pR;
-
-#endif /* __STDC__ */
-
 {
    if (pR->PrPresMode == PresInherit)
      if (pR->PrInhPercent)
@@ -635,21 +570,11 @@ PtrPRule            pR;
    printf (";");
 }
 
-
 /*----------------------------------------------------------------------
    wrminind ecrit au terminal une regle du type Indent, Size, Margin*, etc.  
    La regle a ecrire est pointee par pR.                                  
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrminind (PtrPRule pR)
-
-#else  /* __STDC__ */
-static void         wrminind (pR)
-PtrPRule            pR;
-
-#endif /* __STDC__ */
-
 {
    if (pR->PrPresMode == PresInherit)
       wrnbherit (pR);
@@ -674,33 +599,24 @@ PtrPRule            pR;
      }
 }
 
-
 /*----------------------------------------------------------------------
    wrpos ecrit au terminal la position pos.                        
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrpos (PosRule pos, ThotBool Def)
-
-#else  /* __STDC__ */
-static void         wrpos (pos, Def)
-PosRule             pos;
-ThotBool             Def;
-
-#endif /* __STDC__ */
-
 {
    if (Def)
-      if (pos.PoPosDef == NoEdge)
+     {
+       if (pos.PoPosDef == NoEdge)
 	 printf (" NULL");
-      else
-	{
+       else
+	 {
 	   wrrepere (pos.PoPosDef);
 	   printf (" = ");
-	}
+	 }
+     }
    if (!Def || pos.PoPosDef != NoEdge)
      {
-	wrlevel (pos.PoRelation);
+       wrlevel (pos.PoRelation);
 	if (pos.PoNotRel)
 	   printf (" NOT");
 	printf (" ");
@@ -710,6 +626,10 @@ ThotBool             Def;
 	   wrnomboite (pos.PoRefIdent);
 	else if (pos.PoRefKind == RkAttr)
 	   wrnomattr (pos.PoRefIdent);
+	else if (pos.PoRefKind == RkAnyElem)
+	   printf ("AnyElem");
+	else if (pos.PoRefKind == RkAnyBox)
+	   printf ("AnyBox");
 	printf (". ");
 	wrrepere (pos.PoPosRef);
 	if (pos.PoDistance != 0)
@@ -730,21 +650,10 @@ ThotBool             Def;
    printf (";");
 }
 
-
 /*----------------------------------------------------------------------
    wrdimens ecrit au terminal la dimension Dim.                    
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrdimens (DimensionRule Dim, ThotBool Hauteur)
-
-#else  /* __STDC__ */
-static void         wrdimens (Dim, Hauteur)
-DimensionRule       Dim;
-ThotBool             Hauteur;
-
-#endif /* __STDC__ */
-
 {
    if (Dim.DrPosition)
       wrpos (Dim.DrPosRule, True);
@@ -775,6 +684,10 @@ ThotBool             Hauteur;
 		wrnomboite (Dim.DrRefIdent);
 	     else if (Dim.DrRefKind == RkAttr)
 		wrnomattr (Dim.DrRefIdent);
+	     else if (Dim.DrRefKind == RkAnyElem)
+	       printf ("AnyElem");
+	     else if (Dim.DrRefKind == RkAnyBox)
+	       printf ("AnyBox");
 	     printf (". ");
 	     if ((Dim.DrSameDimens && Hauteur) || (!Dim.DrSameDimens && !Hauteur))
 		printf ("Height");
@@ -813,20 +726,10 @@ ThotBool             Hauteur;
      }
 }
 
-
 /*----------------------------------------------------------------------
    wrCondition                                             
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrCondition (PtrCondition pCond)
-
-#else  /* __STDC__ */
-static void         wrCondition (pCond)
-PtrCondition        pCond;
-
-#endif /* __STDC__ */
-
 {
    if (!pCond->CoNotNegative)
       printf ("NOT ");
@@ -877,6 +780,9 @@ PtrCondition        pCond;
 	    case PcEmpty:
 	       printf ("PcEmpty ");
 	       break;
+	    case PcRoot:
+	       printf ("PcRoot ");
+	       break;
 	    case PcEven:
 	       printf ("Even");
 	       break;
@@ -926,11 +832,23 @@ PtrCondition        pCond;
 		 }
 	       break;
 	    case PcElemType:
-	       wrnomregle (pCond->CoTypeElAttr);
+	       wrnomregle (pCond->CoTypeElem);
 	       printf (" ");
 	       break;
+	    case PcInheritAttribute:
+	       printf ("Inherited ");
 	    case PcAttribute:
-	       wrnomattr (pCond->CoTypeElAttr);
+	       wrnomattr (pCond->CoTypeAttr);
+	       if (pCond->CoTestAttrValue)
+		 {
+		   printf (" = ");
+		   if (pSchemaStr->SsAttribute->TtAttr[pCond->CoTypeAttr - 1]->AttrType == AtTextAttr)
+		     wrnom (pCond->CoAttrTextValue);
+		   else if (pSchemaStr->SsAttribute->TtAttr[pCond->CoTypeAttr - 1]->AttrType == AtEnumAttr)
+		     wrnom (pSchemaStr->SsAttribute->TtAttr[pCond->CoTypeAttr - 1]->AttrEnumValue[pCond->CoAttrValue - 1]);
+		   else
+		     wrnb (pCond->CoAttrValue);
+		 }
 	       printf (" ");
 	       break;
 	    default:
@@ -950,16 +868,7 @@ PtrCondition        pCond;
    wrFonctPres ecrit au terminal la fonction de presentation       
    contenue dans la regle pointee par pR.                  
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrFonctPres (PtrPRule pR)
-
-#else  /* __STDC__ */
-static void         wrFonctPres (pR)
-PtrPRule            pR;
-
-#endif /* __STDC__ */
-
 {
    int                 i;
 
@@ -1072,20 +981,10 @@ PtrPRule            pR;
    printf (";");
 }
 
-
 /*----------------------------------------------------------------------
    wrajust ecrit au terminal la regle d'ajustement des lignes.     
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrajust (PtrPRule pR)
-
-#else  /* __STDC__ */
-static void         wrajust (pR)
-PtrPRule            pR;
-
-#endif /* __STDC__ */
-
 {
    if (pR->PrPresMode == PresInherit)
       wrnbherit (pR);
@@ -1104,48 +1003,32 @@ PtrPRule            pR;
 	       case AlignLeftDots:
 		  printf ("LeftWithDots;");
 		  break;
+	       case AlignJustify:
+		  printf ("Justify;");
+		  break;
 	    }
 }
 
-
 /*----------------------------------------------------------------------
-   wrjustif ecrit la regle de justification ou hyphenation pointee 
-   par pR.                                                 
+   wrboolean ecrit la regle booleenne pointee par pR.
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
-static void         wrjustif (PtrPRule pR)
-
-#else  /* __STDC__ */
-static void         wrjustif (pR)
-PtrPRule            pR;
-
-#endif /* __STDC__ */
-
+static void         wrboolean (PtrPRule pR)
 {
    if (pR->PrPresMode == PresInherit)
       wrnbherit (pR);
    if (pR->PrPresMode == PresImmediate)
-      if (pR->PrJustify)
+     {
+      if (pR->PrBoolValue)
 	 printf ("Yes;");
       else
 	 printf ("No;");
+     }
 }
 
-
 /*----------------------------------------------------------------------
    WriteCounterStyle ecrit au terminal un style de compteur.            
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         WriteCounterStyle (CounterStyle St)
-
-#else  /* __STDC__ */
-static void         WriteCounterStyle (St)
-CounterStyle        St;
-
-#endif /* __STDC__ */
-
 {
    switch (St)
 	 {
@@ -1167,21 +1050,11 @@ CounterStyle        St;
 	 }
 }
 
-
 /*----------------------------------------------------------------------
    wrsuiteregles ecrit au terminal la suite de regles chainees dont 
    RP pointe sur la regle de tete.                         
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 static void         wrsuiteregles (PtrPRule RP)
-
-#else  /* __STDC__ */
-static void         wrsuiteregles (RP)
-PtrPRule            RP;
-
-#endif /* __STDC__ */
-
 {
    PtrCondition        pCond;
 
@@ -1215,136 +1088,44 @@ PtrPRule            RP;
 	  }
 	switch (RP->PrType)
 	      {
-		 case PtFunction:
-		    wrFonctPres (RP);
-		    break;
 		 case PtVisibility:
 		    printf ("Visibility: ");
 		    wrnbherit (RP);
 		    break;
-		 case PtDepth:
-		    printf ("Depth: ");
-		    wrnbherit (RP);
+		 case PtFunction:
+		    wrFonctPres (RP);
 		    break;
-		 case PtFillPattern:
-		    printf ("FillPattern: ");
-		    wrnbherit (RP);
+		 case PtVertOverflow:
+		    printf ("VertOverflow: ");
+		    wrboolean (RP);
 		    break;
-		 case PtBackground:
-		    printf ("Background: ");
-		    wrnbherit (RP);
+		 case PtHorizOverflow:
+		    printf ("HorizOverflow: ");
+		    wrboolean (RP);
 		    break;
-		 case PtForeground:
-		    printf ("Foreground: ");
-		    wrnbherit (RP);
+		 case PtVertRef:
+		    printf ("VertRef: ");
+		    wrpos (RP->PrPosRule, False);
 		    break;
-		 case PtBorderTopColor:
-		    printf ("BorderTopColor: ");
-	            if (RP->PrPresMode == PresImmediate &&
-			!RP->PrAttrValue && RP->PrIntValue == -2)
-		       printf ("transparent;");
-		    else if (RP->PrPresMode == PresImmediate &&
-			     !RP->PrAttrValue && RP->PrIntValue == -1)
-		       printf ("foreground;");
-		    else
-		       wrnbherit (RP);
+		 case PtHorizRef:
+		    printf ("HorizRef: ");
+		    wrpos (RP->PrPosRule, False);
 		    break;
-		 case PtBorderRightColor:
-		    printf ("BorderRightColor: ");
-	            if (RP->PrPresMode == PresImmediate &&
-			!RP->PrAttrValue && RP->PrIntValue == -2)
-		       printf ("transparent;");
-		    else if (RP->PrPresMode == PresImmediate &&
-			     !RP->PrAttrValue && RP->PrIntValue == -1)
-		       printf ("foreground;");
-		    else
-		       wrnbherit (RP);
+		 case PtHeight:
+		    printf ("Height: ");
+		    wrdimens (RP->PrDimRule, True);
 		    break;
-		 case PtBorderBottomColor:
-		    printf ("BorderBottomColor: ");
-	            if (RP->PrPresMode == PresImmediate &&
-			!RP->PrAttrValue && RP->PrIntValue == -2)
-		       printf ("transparent;");
-		    else if (RP->PrPresMode == PresImmediate &&
-			     !RP->PrAttrValue && RP->PrIntValue == -1)
-		       printf ("foreground;");
-		    else
-		       wrnbherit (RP);
+		 case PtWidth:
+		    printf ("Width: ");
+		    wrdimens (RP->PrDimRule, False);
 		    break;
-		 case PtBorderLeftColor:
-		    printf ("BorderLeftColor: ");
-	            if (RP->PrPresMode == PresImmediate &&
-			!RP->PrAttrValue && RP->PrIntValue == -2)
-		       printf ("transparent;");
-		    else if (RP->PrPresMode == PresImmediate &&
-			     !RP->PrAttrValue && RP->PrIntValue == -1)
-		       printf ("foreground;");
-		    else
-		       wrnbherit (RP);
+		 case PtVertPos:
+		    printf ("VertPos: ");
+		    wrpos (RP->PrPosRule, True);
 		    break;
-		 case PtFont:
-		    printf ("Font: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtStyle:
-		    printf ("Style: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtWeight:
-		    printf ("Weight: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtUnderline:
-		    printf ("Underline: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtThickness:
-		    printf ("Thickness: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtLineStyle:
-		    printf ("LineStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBorderTopStyle:
-		    printf ("BorderTopStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBorderRightStyle:
-		    printf ("BorderRightStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBorderBottomStyle:
-		    printf ("BorderBottomStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBorderLeftStyle:
-		    printf ("BorderLeftStyle: ");
-		    wrfontstyle (RP);
-		    break;
-		 case PtBreak1:
-		    printf ("NoBreak1: ");
-		    wrminind (RP);
-		    break;
-		 case PtBreak2:
-		    printf ("NoBreak2: ");
-		    wrminind (RP);
-		    break;
-		 case PtIndent:
-		    printf ("Indent: ");
-		    wrminind (RP);
-		    break;
-		 case PtSize:
-		    printf ("Size: ");
-		    wrsize (RP);
-		    break;
-		 case PtLineSpacing:
-		    printf ("LineSpacing: ");
-		    wrminind (RP);
-		    break;
-		 case PtLineWeight:
-		    printf ("LineWeight: ");
-		    wrminind (RP);
+		 case PtHorizPos:
+		    printf ("HorizPos: ");
+		    wrpos (RP->PrPosRule, True);
 		    break;
 		 case PtMarginTop:
 		    printf ("MarginTop: ");
@@ -1394,92 +1175,208 @@ PtrPRule            RP;
 		    printf ("BorderLeftWidth: ");
 		    wrminind (RP);
 		    break;
-		 case PtVertRef:
-		    printf ("VertRef: ");
-		    wrpos (RP->PrPosRule, False);
+		 case PtBorderTopColor:
+		    printf ("BorderTopColor: ");
+	            if (RP->PrPresMode == PresImmediate &&
+			!RP->PrAttrValue && RP->PrIntValue == -2)
+		       printf ("transparent;");
+		    else if (RP->PrPresMode == PresImmediate &&
+			     !RP->PrAttrValue && RP->PrIntValue == -1)
+		       printf ("foreground;");
+		    else
+		       wrnbherit (RP);
 		    break;
-		 case PtHorizRef:
-		    printf ("HorizRef: ");
-		    wrpos (RP->PrPosRule, False);
+		 case PtBorderRightColor:
+		    printf ("BorderRightColor: ");
+	            if (RP->PrPresMode == PresImmediate &&
+			!RP->PrAttrValue && RP->PrIntValue == -2)
+		       printf ("transparent;");
+		    else if (RP->PrPresMode == PresImmediate &&
+			     !RP->PrAttrValue && RP->PrIntValue == -1)
+		       printf ("foreground;");
+		    else
+		       wrnbherit (RP);
 		    break;
-		 case PtVertPos:
-		    printf ("VertPos: ");
-		    wrpos (RP->PrPosRule, True);
+		 case PtBorderBottomColor:
+		    printf ("BorderBottomColor: ");
+	            if (RP->PrPresMode == PresImmediate &&
+			!RP->PrAttrValue && RP->PrIntValue == -2)
+		       printf ("transparent;");
+		    else if (RP->PrPresMode == PresImmediate &&
+			     !RP->PrAttrValue && RP->PrIntValue == -1)
+		       printf ("foreground;");
+		    else
+		       wrnbherit (RP);
 		    break;
-		 case PtHorizPos:
-		    printf ("HorizPos: ");
-		    wrpos (RP->PrPosRule, True);
+		 case PtBorderLeftColor:
+		    printf ("BorderLeftColor: ");
+	            if (RP->PrPresMode == PresImmediate &&
+			!RP->PrAttrValue && RP->PrIntValue == -2)
+		       printf ("transparent;");
+		    else if (RP->PrPresMode == PresImmediate &&
+			     !RP->PrAttrValue && RP->PrIntValue == -1)
+		       printf ("foreground;");
+		    else
+		       wrnbherit (RP);
 		    break;
-		 case PtHeight:
-		    printf ("Height: ");
-		    wrdimens (RP->PrDimRule, True);
+		 case PtBorderTopStyle:
+		    printf ("BorderTopStyle: ");
+		    wrfontstyle (RP);
 		    break;
-		 case PtWidth:
-		    printf ("Width: ");
-		    wrdimens (RP->PrDimRule, False);
+		 case PtBorderRightStyle:
+		    printf ("BorderRightStyle: ");
+		    wrfontstyle (RP);
 		    break;
-		 case PtJustify:
-		    printf ("Justify: ");
-		    wrjustif (RP);
+		 case PtBorderBottomStyle:
+		    printf ("BorderBottomStyle: ");
+		    wrfontstyle (RP);
 		    break;
-		 case PtHyphenate:
-		    printf ("Hyphenate: ");
-		    wrjustif (RP);
+		 case PtBorderLeftStyle:
+		    printf ("BorderLeftStyle: ");
+		    wrfontstyle (RP);
 		    break;
-		 case PtVertOverflow:
-		    printf ("VertOverflow: ");
-		    wrjustif (RP);
+		 case PtSize:
+		    printf ("Size: ");
+		    wrsize (RP);
 		    break;
-		 case PtHorizOverflow:
-		    printf ("HorizOverflow: ");
-		    wrjustif (RP);
+		 case PtStyle:
+		    printf ("Style: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtWeight:
+		    printf ("Weight: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtFont:
+		    printf ("Font: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtUnderline:
+		    printf ("Underline: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtThickness:
+		    printf ("Thickness: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtIndent:
+		    printf ("Indent: ");
+		    wrminind (RP);
+		    break;
+		 case PtLineSpacing:
+		    printf ("LineSpacing: ");
+		    wrminind (RP);
+		    break;
+		 case PtDepth:
+		    printf ("Depth: ");
+		    wrnbherit (RP);
 		    break;
 		 case PtAdjust:
 		    printf ("Adjust: ");
 		    wrajust (RP);
 		    break;
+		 case PtDirection:
+		    printf ("Direction: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtUnicodeBidi:
+		    printf ("UnicodeBidi: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtLineStyle:
+		    printf ("LineStyle: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtLineWeight:
+		    printf ("LineWeight: ");
+		    wrminind (RP);
+		    break;
+		 case PtFillPattern:
+		    printf ("FillPattern: ");
+		    wrnbherit (RP);
+		    break;
+		 case PtBackground:
+		    printf ("Background: ");
+		    wrnbherit (RP);
+		    break;
+		 case PtForeground:
+		    printf ("Foreground: ");
+		    wrnbherit (RP);
+		    break;
+		 case PtHyphenate:
+		    printf ("Hyphenate: ");
+		    wrboolean (RP);
+		    break;
+	         case PtPageBreak:
+		    printf ("PageBreak: ");
+		    wrboolean (RP);
+		    break;
+	         case PtLineBreak:
+		    printf ("LineBreak: ");
+		    wrboolean (RP);
+		    break;
+	         case PtGather:
+		    printf ("Gather: ");
+		    wrboolean (RP);
+		    break;
+	         case PtXRadius:
+		    printf ("XRadius: ");
+		    wrboolean (RP);
+		    break;
+	         case PtYRadius:
+		    printf ("YRadius: ");
+		    wrboolean (RP);
+		    break;
+	         case PtDisplay:
+		    printf ("Display: ");
+		    wrfontstyle (RP);
+		    break;
+		 case PtBreak1:
+		    printf ("NoBreak1: ");
+		    wrminind (RP);
+		    break;
+		 case PtBreak2:
+		    printf ("NoBreak2: ");
+		    wrminind (RP);
+		    break;
 		 case PtPictInfo:
 		    break;
 	      }
+	if (RP->PrDuplicate)
+	  printf ("  {duplicate}");
 	printf ("\n");		/* passe a la regle suivante */
 	RP = RP->PrNextPRule;
      }
 }
 
-
 /*----------------------------------------------------------------------
    main                                                            
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 int                 main (int argc, char **argv)
-
-#else  /* __STDC__ */
-int                 main (argc, argv)
-int                 argc;
-char              **argv;
-
-#endif /* __STDC__ */
-
 {
    int                 i, k;
    PtrHostView         pHostView;
 
    TtaInitializeAppRegistry (argv[0]);
 
-   i = TtaGetMessageTable (TEXT("libdialogue"), TMSG_LIB_MSG_MAX);
-   PRS = TtaGetMessageTable (TEXT("prsdialogue"), PRS_MSG_MAX);
+   i = TtaGetMessageTable ("libdialogue", TMSG_LIB_MSG_MAX);
+   PRS = TtaGetMessageTable ("prsdialogue", PRS_MSG_MAX);
    /* recupere d'abord le nom du schema a lister */
    filename[0] = '\0';
-   if (argc != 2)
+   if (argc != 3)
       goto Usage;
    argv++;
    strcpy (filename, *argv);
-   /* indique que le chargement du schema de presentation doit commencer par */
-   /* le chargement du schema de structure correspondant */
+   /* chargement du schema de structure */
    GetSchStruct (&pSchemaStr);
-   pSchemaStr->SsRootElem = 0;
-   /* lit le schema de presentation et le schema de structure correspondant */
+   if (!ReadStructureSchema (filename, pSchemaStr))
+     {
+      TtaDisplaySimpleMessage (FATAL, PRS, UNKNOWN_FILE);
+      return 1;
+     }
+   argv++;
+   strcpy (filename, *argv);
+   /* lit le schema de presentation */
    pSchemaPrs = ReadPresentationSchema (filename, pSchemaStr);
    if (pSchemaPrs == NULL)
       TtaDisplaySimpleMessage (FATAL, PRS, UNKNOWN_FILE);
@@ -1490,7 +1387,7 @@ char              **argv;
 	printf ("\n");
 	printf ("{ In each block, rules are listed in the same order as they\n");
 	printf ("are processed by the editor. }\n");
-	/* ecrit au terminal le nom du schema  de structure */
+ /* ecrit au terminal le nom du schema  de structure */
 	printf ("\n");
 	printf ("PRESENTATION ");
 	wrnom (pSchemaStr->SsName);
@@ -1652,7 +1549,7 @@ char              **argv;
 			{
 			   case CharString:
 			      printf ("Text ");
-			      putchar (pPr1->PdAlphabet);
+			      putchar (pPr1->PdScript);
 			      break;
 			   case GraphicElem:
 			      printf ("Graphics ");
@@ -1713,15 +1610,17 @@ char              **argv;
 				   WriteCounterStyle (pVa1->ViStyle);
 				   break;
 				case VarAttrValue:
+				  if (pVa1->ViAttr == 0)
+				    printf (" AttributeValue");
 				   printf (" VALUE(");
 				   wrnomattr (pVa1->ViAttr);
 				   WriteCounterStyle (pVa1->ViStyle);
 				   break;
 				case VarDate:
-				   printf (" VarDate");
+				   printf (" Date");
 				   break;
 				case VarFDate:
-				   printf (" VarFDate");
+				   printf (" FDate");
 				   break;
 				case VarDirName:
 				   printf (" DirName");
@@ -1733,7 +1632,7 @@ char              **argv;
 				   printf (" ElemName");
 				   break;
 				case VarAttrName:
-				   printf (" VarAttrName");
+				   printf (" AttributeName");
 				   break;
 				case VarPageNumber:
 				   printf (" VALUE(PageNumber(");
@@ -1763,27 +1662,12 @@ char              **argv;
 	     printf ("\n");
 	     for (i = 1; i <= pSc1->PsNPresentBoxes; i++)
 	       {
-		  pBo1 = &pSc1->PsPresentBox[i - 1];
+		  pBo1 = pSc1->PsPresentBox->PresBox[i - 1];
 		  wrnom (pBo1->PbName);
 		  printf (":	{%3d }\n", i);
 		  if (pBo1->PbFirstPRule != NULL)
 		     printf ("   BEGIN\n");
 		  wrsuiteregles (pBo1->PbFirstPRule);
-		  printf ("   PageBreak: ");
-		  if (pBo1->PbAcceptPageBreak)
-		     printf ("Yes;\n");
-		  else
-		     printf ("No;\n");
-		  printf ("   LineBreak: ");
-		  if (pBo1->PbAcceptLineBreak)
-		     printf ("Yes;\n");
-		  else
-		     printf ("No;\n");
-		  printf ("   Gather: ");
-		  if (pBo1->PbBuildAll)
-		     printf ("Yes;\n");
-		  else
-		     printf ("No;\n");
 		  /* ecrit le contenu de la boite */
 		  if (pBo1->PbContent != FreeContent)
 		    {
@@ -1798,14 +1682,7 @@ char              **argv;
 				   printf ("Cste");
 				   wrnb (pBo1->PbContConstant);
 				   break;
-				case ContElement:
-				   wrnomregle (pBo1->PbContElem);
-				   printf (" {referenced by ");
-				   wrnomregle (pBo1->PbContRefElem);
-				   printf ("}");
-				   break;
 				case FreeContent:
-				   ;
 				   break;
 			     }
 
@@ -1836,36 +1713,19 @@ char              **argv;
 	printf ("\nRULES\n\n");
 	for (El = 1; El <= pSchemaStr->SsNRules; El++)
 	  {
-	     if (pSchemaStr->SsRule[El - 1].SrConstruct == CsPairedElement)
-		if (pSchemaStr->SsRule[El - 1].SrFirstOfPair)
+	    if (pSchemaStr->SsRule->SrElem[El - 1]->SrConstruct == CsPairedElement)
+	      {
+		if (pSchemaStr->SsRule->SrElem[El - 1]->SrFirstOfPair)
 		   printf ("First ");
 		else
 		   printf ("Second ");
+	      }
 	     wrnomregle (El);
 	     printf (":\n");
-	     if (pSc1->PsElemPRule[El - 1] != NULL)
+	     if (pSc1->PsElemPRule->ElemPres[El - 1])
 		printf ("   BEGIN\n");
-	     wrsuiteregles (pSc1->PsElemPRule[El - 1]);
-	     printf ("   PageBreak: ");
-	     if (pSc1->PsAcceptPageBreak[El - 1])
-		printf ("Yes;\n");
-	     else
-		printf ("No;\n");
-	     printf ("   LineBreak: ");
-	     if (pSc1->PsAcceptLineBreak[El - 1])
-		printf ("Yes;\n");
-	     else
-		printf ("No;\n");
-	     printf ("   Gather: ");
-	     if (pSc1->PsBuildAll[El - 1])
-		printf ("Yes;\n");
-	     else
-		printf ("No;\n");
-	     if (pSc1->PsInPageHeaderOrFooter[El - 1])
-		printf ("   { displayed in top or bottom of page }\n");
-	     if (pSc1->PsAssocPaginated[El - 1])
-		printf ("   { with pages }\n");
-	     if (pSc1->PsElemPRule[El - 1] != NULL)
+	     wrsuiteregles (pSc1->PsElemPRule->ElemPres[El - 1]);
+	     if (pSc1->PsElemPRule->ElemPres[El - 1])
 		printf ("   END;\n");
 	     printf ("\n");
 	  }
@@ -1877,9 +1737,9 @@ char              **argv;
 	     printf ("\n");
 	     for (Attr = 1; Attr <= pSchemaStr->SsNAttributes; Attr++)
 	       {
-	       pAt1 = &pSchemaStr->SsAttribute[Attr - 1];
-	       pRP1 = pSc1->PsAttrPRule[Attr - 1];
-	       for (k = pSc1->PsNAttrPRule[Attr - 1]; k-- > 0;
+	       pAt1 = pSchemaStr->SsAttribute->TtAttr[Attr - 1];
+	       pRP1 = pSc1->PsAttrPRule->AttrPres[Attr - 1];
+	       for (k = pSc1->PsNAttrPRule->Num[Attr - 1]; k-- > 0;
 		    pRP1 = pRP1->ApNextAttrPres)
 		  if (pRP1 != NULL)
 		    {
@@ -1913,6 +1773,7 @@ char              **argv;
 					  }
 					else if (pCa1->CaLowerBound != -MAX_INT_ATTR_VAL - 1
 						 || pCa1->CaUpperBound != MAX_INT_ATTR_VAL + 1)
+					  {
 					   if (pCa1->CaLowerBound != -MAX_INT_ATTR_VAL - 1)
 					     {
 						printf (">");
@@ -1923,6 +1784,7 @@ char              **argv;
 						printf ("<");
 						wrnb (pCa1->CaUpperBound + 1);
 					     }
+					  }
 					printf (":\n");
 					if (pCa1->CaFirstPRule == NULL)
 					   printf ("   BEGIN END;\n");
@@ -2041,7 +1903,7 @@ char              **argv;
 	/* ecrit les regles de transmission de valeurs d'elements */
 	if (pSc1->PsNTransmElems > 0)
 	   for (i = 1; i <= pSchemaStr->SsNRules; i++)
-	      if (pSc1->PsElemTransmit[i - 1] > 0)
+	      if (pSc1->PsElemTransmit->Num[i - 1] > 0)
 		{
 		   if (!Transm)
 		     {
@@ -2051,9 +1913,9 @@ char              **argv;
 		   printf ("   ");
 		   wrnomregle (i);
 		   printf (" To ");
-		   wrnom (pSc1->PsTransmElem[pSc1->PsElemTransmit[i - 1] - 1].TeTargetAttr);
+		   wrnom (pSc1->PsTransmElem[pSc1->PsElemTransmit->Num[i - 1] - 1].TeTargetAttr);
 		   printf ("(");
-		   wrnomregle (pSc1->PsTransmElem[pSc1->PsElemTransmit[i - 1] - 1].TeTargetDoc);
+		   wrnomregle (pSc1->PsTransmElem[pSc1->PsElemTransmit->Num[i - 1] - 1].TeTargetDoc);
 		   printf (");\n");
 		}
 
@@ -2063,7 +1925,7 @@ char              **argv;
    exit (0);
 
  Usage:
-   fprintf (stderr, "usage : %s <input-file>\n", argv[0]);
+   fprintf (stderr, "usage : %s <S schema> <P schema>\n", argv[0]);
    exit (1);
 }
 /* End Of Module Printprs */
