@@ -1,6 +1,6 @@
-/* 
+/*
  *
- *  (c) COPYRIGHT INRIA 1996-2001
+ *  (c) COPYRIGHT INRIA 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -15,6 +15,7 @@
  *
  * Author: V. Quint
  *         R. Guetari: Windows adaptation
+ *         I. Vatton: new exceptions
  *
  */
 
@@ -362,9 +363,9 @@ static void      AllocateNewRule (PtrSSchema pSS)
 }
 
 /*----------------------------------------------------------------------
-   Undefined                                                       
+   UndefinedElem
   ----------------------------------------------------------------------*/
-static void         Undefined (int n)
+static void         UndefinedElem (int n)
 {
    int                 j;
    SrcIdentDesc       *pIdent;
@@ -418,7 +419,7 @@ static void         ChangeOneRule (PtrSRule pRule)
 	  {
 	     if (Identifier[pRule->SrInclusion[j] - MAX_BASIC_TYPE - 1].
 		 SrcIdentDefRule == 0)
-	       Undefined (pRule->SrInclusion[j] - MAX_BASIC_TYPE);
+	       UndefinedElem (pRule->SrInclusion[j] - MAX_BASIC_TYPE);
 	     pRule->SrInclusion[j] =
 	       Identifier[pRule->SrInclusion[j] - MAX_BASIC_TYPE - 1].
 	               SrcIdentDefRule;
@@ -435,7 +436,7 @@ static void         ChangeOneRule (PtrSRule pRule)
 	  {
 	     if (Identifier[pRule->SrExclusion[j] - MAX_BASIC_TYPE - 1].
 		 SrcIdentDefRule == 0)
-	       Undefined (pRule->SrExclusion[j] - MAX_BASIC_TYPE);
+	       UndefinedElem (pRule->SrExclusion[j] - MAX_BASIC_TYPE);
 	     pRule->SrExclusion[j] =
 	       Identifier[pRule->SrExclusion[j] - MAX_BASIC_TYPE - 1].
 	                      SrcIdentDefRule;
@@ -454,7 +455,7 @@ static void         ChangeOneRule (PtrSRule pRule)
 	   {
 	     if (Identifier[pRule->SrReferredType - MAX_BASIC_TYPE - 1].
 		 SrcIdentDefRule == 0)
-	       Undefined (pRule->SrReferredType - MAX_BASIC_TYPE);
+	       UndefinedElem (pRule->SrReferredType - MAX_BASIC_TYPE);
 	     pRule->SrReferredType =
 	       Identifier[pRule->SrReferredType - MAX_BASIC_TYPE - 1].
 	               SrcIdentDefRule;
@@ -468,7 +469,7 @@ static void         ChangeOneRule (PtrSRule pRule)
 	 {
 	   if (Identifier[pRule->SrIdentRule - MAX_BASIC_TYPE - 1].
 	       SrcIdentDefRule == 0)
-	     Undefined (pRule->SrIdentRule - MAX_BASIC_TYPE);
+	     UndefinedElem (pRule->SrIdentRule - MAX_BASIC_TYPE);
 	   pRule->SrIdentRule =
 	     Identifier[pRule->SrIdentRule - MAX_BASIC_TYPE - 1].
 	               SrcIdentDefRule;
@@ -482,7 +483,7 @@ static void         ChangeOneRule (PtrSRule pRule)
 	 {
 	   if (Identifier[pRule->SrListItem - MAX_BASIC_TYPE - 1].
 	       SrcIdentDefRule == 0)
-	     Undefined (pRule->SrListItem - MAX_BASIC_TYPE);
+	     UndefinedElem (pRule->SrListItem - MAX_BASIC_TYPE);
 	   pRule->SrListItem = Identifier[pRule->SrListItem-MAX_BASIC_TYPE-1].
 		       SrcIdentDefRule;
 	 }
@@ -498,7 +499,7 @@ static void         ChangeOneRule (PtrSRule pRule)
 	       {
 		 if (Identifier[pRule->SrChoice[j] - MAX_BASIC_TYPE - 1].
 		     SrcIdentDefRule == 0)
-		   Undefined (pRule->SrChoice[j] - MAX_BASIC_TYPE);
+		   UndefinedElem (pRule->SrChoice[j] - MAX_BASIC_TYPE);
 		 pRule->SrChoice[j] =
 		   Identifier[pRule->SrChoice[j] - MAX_BASIC_TYPE - 1].
 		              SrcIdentDefRule;
@@ -516,7 +517,7 @@ static void         ChangeOneRule (PtrSRule pRule)
 	     {
 	       if (Identifier[pRule->SrComponent[j] - MAX_BASIC_TYPE - 1].
 		   SrcIdentDefRule == 0)
-		 Undefined (pRule->SrComponent[j] - MAX_BASIC_TYPE);
+		 UndefinedElem (pRule->SrComponent[j] - MAX_BASIC_TYPE);
 	       pRule->SrComponent[j] =
 		 Identifier[pRule->SrComponent[j] - MAX_BASIC_TYPE - 1].
 		           SrcIdentDefRule;
@@ -570,7 +571,7 @@ static void         ChangeRules ()
 		{
 		   if (Identifier[pAttr->AttrTypeRef - MAX_BASIC_TYPE - 1].
 		       SrcIdentDefRule == 0)
-		     Undefined (pAttr->AttrTypeRef - MAX_BASIC_TYPE);
+		     UndefinedElem (pAttr->AttrTypeRef - MAX_BASIC_TYPE);
 		   pAttr->AttrTypeRef =
 		     Identifier[pAttr->AttrTypeRef - MAX_BASIC_TYPE - 1].
 		                  SrcIdentDefRule;
@@ -1666,6 +1667,9 @@ static void         ProcessToken (indLine wi, indLine wl, SyntacticCode c,
        case KWD_NoCreate:
 	 ExceptionNum (ExcNoCreate, True, False, False, wi);
 	 break;
+       case KWD_NoReplicate:
+	 ExceptionNum (ExcNoReplicate, True, False, False, wi);
+	 break;
        case KWD_NoHMove:
 	 ExceptionNum (ExcNoHMove, True, False, False, wi);
 	 break;
@@ -2862,7 +2866,10 @@ int main (int argc, char **argv)
    char                msg [800];
    int                 ndx, pIndex = 0;
    HANDLE              cppLib;
-   FARPROC             ptrMainProc;
+   /* FARPROC             ptrMainProc; */
+   typedef int (*MYPROC) (HWND, int, char **, int *);
+   MYPROC              ptrMainProc; 
+
 #else  /* !_WINDOWS */
    char                cmd[800];
 #endif /* _WINDOWS */
@@ -3002,7 +3009,7 @@ int main (int argc, char **argv)
 	     } 
 #ifdef _WINDOWS
            cppLib = LoadLibrary ("cpp");
-           ptrMainProc = GetProcAddress (cppLib, "CPPmain");
+           ptrMainProc = (MYPROC) GetProcAddress (cppLib, "CPPmain");
            i = ptrMainProc (hwnd, pIndex, cmd, &_CY_);
            FreeLibrary (cppLib);
            for (ndx = 0; ndx < pIndex; ndx++)
