@@ -70,7 +70,7 @@ char*    ThotPath;
 char*    currentFile;
 char*    currentDestFile;
 char*    BinFiles [100];
-char*    TbStrings [2] = {"Open (Ctrl+O)", "Build (F7)"};
+char*    TbStrings [2] = {"Open a specif file (Ctrl+O)", "Build (F7)"};
 
 DWORD    dwStatusBarStyles = WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | CCS_BOTTOM | SBARS_SIZEGRIP;
 
@@ -238,8 +238,10 @@ int Makefile (HWND hwnd, char *fileName)
 {
     FILE      *f; 
     HINSTANCE  hLib;
-    FARPROC    ptrMainProc;
-    char       msg [1024];
+	/* FARPROC ptrMainProc; */
+    typedef    int (*MYPROC) (HWND, HWND, int, char **, int *);
+    MYPROC     ptrMainProc;    
+	char       msg [1024];
     char       seps[]   = " \t=$(\n\r)";
     char       string [1024];
     char      *args [100];
@@ -402,7 +404,7 @@ int Makefile (HWND hwnd, char *fileName)
 		       hLib = LoadLibrary ("app");
 		       if (!hLib)
 			 return FATAL_EXIT_CODE;
-		       ptrMainProc = GetProcAddress (hLib, "APPmain");
+		       ptrMainProc = (MYPROC) GetProcAddress (hLib, "APPmain");
 		       if (!ptrMainProc)
 			 {
 			   FreeLibrary (hLib);
@@ -494,7 +496,7 @@ int Makefile (HWND hwnd, char *fileName)
 		       hLib = LoadLibrary ("prs");
 		       if (!hLib)
 			 return FATAL_EXIT_CODE;
-		       ptrMainProc = GetProcAddress (hLib, "PRSmain");
+		       ptrMainProc = (MYPROC) GetProcAddress (hLib, "PRSmain");
 		       if (!ptrMainProc)
 			 {
 			   FreeLibrary (hLib);
@@ -636,7 +638,7 @@ int Makefile (HWND hwnd, char *fileName)
 		       hLib = LoadLibrary ("str");
 		       if (!hLib)
 			 return FATAL_EXIT_CODE;
-		       ptrMainProc = GetProcAddress (hLib, "STRmain");
+		       ptrMainProc = (MYPROC) GetProcAddress (hLib, "STRmain");
 		       if (!ptrMainProc)
 			 {
 			   FreeLibrary (hLib);
@@ -779,7 +781,7 @@ int Makefile (HWND hwnd, char *fileName)
 		       if (!hLib)
 			 return FATAL_EXIT_CODE;
 		       
-		       ptrMainProc = GetProcAddress (hLib, "TRAmain");
+		       ptrMainProc = (MYPROC) GetProcAddress (hLib, "TRAmain");
 		       if (!ptrMainProc)
 			 {
 			   FreeLibrary (hLib);
@@ -1059,9 +1061,12 @@ int Makefile (HWND hwnd, char *fileName)
 }
 
 /*----------------------------------------------------------------------
+ CompilersWndProc
   ----------------------------------------------------------------------*/
 LRESULT CALLBACK CompilersWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam)
 {
+  char         *s;
+  int           l;
   int           result = COMP_SUCCESS; 
   int           status, cx, cy;
   int           cyStatus, cyTB;
@@ -1074,13 +1079,13 @@ LRESULT CALLBACK CompilersWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM l
     case WM_CREATE:
       menuBar = CreateMenu ();
       popupMenu = CreateMenu ();
-      AppendMenu (popupMenu, MF_STRING, OPEN, "&Open\tCtrl+O");
+      AppendMenu (popupMenu, MF_STRING, OPEN, "&Open a specific file\tCtrl+O");
       AppendMenu (popupMenu, MF_STRING, COMPILE, "&Build\tF7");
       AppendMenu (popupMenu, MF_SEPARATOR, 0, NULL);
       AppendMenu (popupMenu, MF_STRING, QUIT, "&Quit\tAlt+F4");
       AppendMenu (menuBar, MF_POPUP, (UINT)popupMenu, "&File");
       SetMenu (hwnd, menuBar);
-      EnableMenuItem (popupMenu, COMPILE, MFS_GRAYED);
+      EnableMenuItem (popupMenu, COMPILE, MFS_ENABLED);
       
       hWndToolBar = CreateToolbarEx (hwnd, WS_CHILD | WS_BORDER | WS_VISIBLE | TBSTYLE_TOOLTIPS | CCS_ADJUSTABLE,
 				     ID_TOOLBAR, 2, g_hInstance, COMP_TOOLBAR, (LPCTBBUTTON)&tbButtons,
@@ -1152,10 +1157,30 @@ LRESULT CALLBACK CompilersWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM l
       return 0;
 
     case WM_KEYDOWN:
-      if ((wParam == VK_F7) && fileToOpen && (fileToOpen[0] != 0))
+      if ((wParam == VK_F7))
 	{
 	  SetCursor(LoadCursor(NULL, IDC_WAIT));
 	  SetWindowText (hEdit, "");
+	  if (fileToOpen[0] == EOS)
+	  {
+		s = TtaGetEnvString ("PWD");
+		if (s)
+		{
+			strcpy (fileToOpen, s);
+			l = strlen (fileToOpen) - 1;
+			while (l > 0 && fileToOpen[l] != '\\')
+              l--;
+			if (l > 0)
+			{
+			  while (l > 0 && fileToOpen[l] != '\\')
+				l--;
+			}
+			fileToOpen[l] = EOS;
+			strcat (fileToOpen, "\\amaya");
+			_chdir (fileToOpen);
+		}
+		strcat (fileToOpen, "\\amaya.mkf");
+	  }
 	  result = Makefile (hEdit, fileToOpen);
 	  SetCursor(LoadCursor(NULL, IDC_ARROW));
 	  if (result == FATAL_EXIT_CODE)
@@ -1251,6 +1276,26 @@ LRESULT CALLBACK CompilersWndProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM l
 	case COMPILE: 
 	  SetCursor(LoadCursor(NULL, IDC_WAIT));
 	  SetWindowText (hEdit, "");
+	  if (fileToOpen[0] == EOS)
+	  {
+		s = TtaGetEnvString ("PWD");
+		if (s)
+		{
+			strcpy (fileToOpen, s);
+			l = strlen (fileToOpen) - 1;
+			while (l > 0 && fileToOpen[l] != '\\')
+              l--;
+			if (l > 0)
+			{
+			  while (l > 0 && fileToOpen[l] != '\\')
+				l--;
+			}
+			fileToOpen[l] = EOS;
+			strcat (fileToOpen, "\\amaya");
+			_chdir (fileToOpen);
+		}
+		strcat (fileToOpen, "\\amaya.mkf");
+	  }
 	  result = Makefile (hEdit, fileToOpen);
 	  SetCursor(LoadCursor(NULL, IDC_ARROW));
 	  if (result == FATAL_EXIT_CODE)
@@ -1314,6 +1359,7 @@ int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
   /*** sprintf (WorkPath, "%s\\amaya", BinPath); ***/
   
   ThotPath = TtaGetEnvString ("THOTDIR");
+  /* register window class */
   wndClass.style         = CS_HREDRAW | CS_VREDRAW;
   wndClass.lpfnWndProc   = (WNDPROC) CompilersWndProc;
   wndClass.cbClsExtra    = 0;
@@ -1329,16 +1375,20 @@ int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
   if (!RegisterClassEx (&wndClass))
     return FALSE;
   
+   /* create window */
   hwnd = CreateWindowEx (0, szAppName, "Thot compilers",
 			 DS_MODALFRAME | WS_POPUP |
-			 WS_OVERLAPPEDWINDOW | WS_VISIBLE | WS_CAPTION | WS_SYSMENU,
+			 WS_OVERLAPPEDWINDOW | WS_VISIBLE |
+			 WS_CAPTION | WS_SYSMENU,
 			 0, 0,
 			 600, 650,
 			 NULL, NULL, hInstance, NULL);
   
+  /* display window */
   ShowWindow (hwnd, SW_SHOWNORMAL);
   UpdateWindow (hwnd);
-  
+
+  /* process messages */
   while (GetMessage (&msg, NULL, 0, 0))
     {
       TranslateMessage (&msg);
