@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA  1996-2001
+ *  (c) COPYRIGHT INRIA  1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -30,13 +30,7 @@ static BinFile      outfile;
 /*----------------------------------------------------------------------
    WriteShort							
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WriteShort (int n)
-#else  /* __STDC__ */
-static void         WriteShort (n)
-int                 n;
-
-#endif /* __STDC__ */
+static void WriteShort (int n)
 {
    TtaWriteByte (outfile, (char) (n / 256));
    TtaWriteByte (outfile, (char) (n % 256));
@@ -45,13 +39,7 @@ int                 n;
 /*----------------------------------------------------------------------
    WriteSignedShort						
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WriteSignedShort (int n)
-#else  /* __STDC__ */
-static void         WriteSignedShort (n)
-int                 n;
-
-#endif /* __STDC__ */
+static void WriteSignedShort (int n)
 {
    if (n >= 0)
       WriteShort (n);
@@ -62,19 +50,13 @@ int                 n;
 /*----------------------------------------------------------------------
    WriteName							
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WriteName (Name n)
-#else  /* __STDC__ */
-static void         WriteName (n)
-Name                n;
-
-#endif /* __STDC__ */
+static void WriteName (Name n)
 {
    int                 i;
 
    i = 0;
    do
-      TtaWriteWideChar (outfile, n[i++], ISO_8859_1 /* UTF_8 */);
+      TtaWriteByte (outfile, n[i++]);
    while (n[i - 1] != '\0');
 }
 
@@ -82,13 +64,7 @@ Name                n;
 /*----------------------------------------------------------------------
    WriteRulePtr							
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WriteRulePtr (PtrPRule p)
-#else  /* __STDC__ */
-static void         WriteRulePtr (p)
-PtrPRule            p;
-
-#endif /* __STDC__ */
+static void WriteRulePtr (PtrPRule p)
 {
    if (p == NULL)
       TtaWriteByte (outfile, '\0');
@@ -100,13 +76,7 @@ PtrPRule            p;
 /*----------------------------------------------------------------------
    WriteBoolean							
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WriteBoolean (ThotBool b)
-#else  /* __STDC__ */
-static void         WriteBoolean (b)
-ThotBool             b;
-
-#endif /* __STDC__ */
+static void WriteBoolean (ThotBool b)
 {
    if (b)
       TtaWriteByte (outfile, '\1');
@@ -117,14 +87,7 @@ ThotBool             b;
 /*----------------------------------------------------------------------
    WriteUnit ecrit une unite dans le fichier sur un octet          
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WriteUnit (TypeUnit unit)
-#else  /* __STDC__ */
-static void         WriteUnit (unit)
-BinFile             file;
-TypeUnit            unit;
-
-#endif /* __STDC__ */
+static void WriteUnit (TypeUnit unit)
 {
    switch (unit)
 	 {
@@ -156,13 +119,7 @@ TypeUnit            unit;
 /*----------------------------------------------------------------------
    WritePRuleType  ecrit un type de regle de presentation		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                WritePRuleType (PRuleType ruleType)
-#else  /* __STDC__ */
-void                WritePRuleType (ruleType)
-PRuleType           ruleType;
-
-#endif /* __STDC__ */
 {
    switch (ruleType)
 	 {
@@ -172,6 +129,12 @@ PRuleType           ruleType;
 	    case PtFunction:
 	       TtaWriteByte (outfile, C_PR_FUNCTION);
 	       break;
+	    case PtHorizOverflow:
+	       TtaWriteByte (outfile, C_PR_HORIZOVERFLOW);
+	       break;
+	    case PtVertOverflow:
+	       TtaWriteByte (outfile, C_PR_VERTOVERFLOW);
+	       break;
 	    case PtVertRef:
 	       TtaWriteByte (outfile, C_PR_VREF);
 	       break;
@@ -253,12 +216,6 @@ PRuleType           ruleType;
 	    case PtSize:
 	       TtaWriteByte (outfile, C_PR_SIZE);
 	       break;
-	    case PtUnderline:
-	       TtaWriteByte (outfile, C_PR_UNDERLINE);
-	       break;
-	    case PtThickness:
-	       TtaWriteByte (outfile, C_PR_UNDER_THICK);
-	       break;
 	    case PtStyle:
 	       TtaWriteByte (outfile, C_PR_STYLE);
 	       break;
@@ -268,11 +225,11 @@ PRuleType           ruleType;
 	    case PtFont:
 	       TtaWriteByte (outfile, C_PR_FONT);
 	       break;
-	    case PtBreak1:
-	       TtaWriteByte (outfile, C_PR_BREAK1);
+	    case PtUnderline:
+	       TtaWriteByte (outfile, C_PR_UNDERLINE);
 	       break;
-	    case PtBreak2:
-	       TtaWriteByte (outfile, C_PR_BREAK2);
+	    case PtThickness:
+	       TtaWriteByte (outfile, C_PR_UNDER_THICK);
 	       break;
 	    case PtIndent:
 	       TtaWriteByte (outfile, C_PR_INDENT);
@@ -280,14 +237,17 @@ PRuleType           ruleType;
 	    case PtLineSpacing:
 	       TtaWriteByte (outfile, C_PR_LINESPACING);
 	       break;
+	    case PtDepth:
+	       TtaWriteByte (outfile, C_PR_DEPTH);
+	       break;
 	    case PtAdjust:
 	       TtaWriteByte (outfile, C_PR_ADJUST);
 	       break;
-	    case PtHyphenate:
-	       TtaWriteByte (outfile, C_PR_HYPHENATE);
+	    case PtDirection:
+	       TtaWriteByte (outfile, C_PR_DIRECTION);
 	       break;
-	    case PtDepth:
-	       TtaWriteByte (outfile, C_PR_DEPTH);
+	    case PtUnicodeBidi:
+	       TtaWriteByte (outfile, C_PR_UNICODEBIDI);
 	       break;
 	    case PtLineStyle:
 	       TtaWriteByte (outfile, C_PR_LINESTYLE);
@@ -304,14 +264,8 @@ PRuleType           ruleType;
 	    case PtForeground:
 	       TtaWriteByte (outfile, C_PR_FOREGROUND);
 	       break;
-	    case PtHorizOverflow:
-	       TtaWriteByte (outfile, C_PR_HORIZOVERFLOW);
-	       break;
-	    case PtVertOverflow:
-	       TtaWriteByte (outfile, C_PR_VERTOVERFLOW);
-	       break;
-	    case PtGather:
-	       TtaWriteByte (outfile, C_PR_GATHER);
+	    case PtHyphenate:
+	       TtaWriteByte (outfile, C_PR_HYPHENATE);
 	       break;
 	    case PtPageBreak:
 	       TtaWriteByte (outfile, C_PR_PAGEBREAK);
@@ -319,6 +273,19 @@ PRuleType           ruleType;
 	    case PtLineBreak:
 	       TtaWriteByte (outfile, C_PR_LINEBREAK);
 	       break;
+	    case PtGather:
+	       TtaWriteByte (outfile, C_PR_GATHER);
+	       break;
+
+	    case PtDisplay:
+	       TtaWriteByte (outfile, C_PR_DISPLAY);
+	       break;
+	    case PtBreak1:
+	       TtaWriteByte (outfile, C_PR_BREAK1);
+	       break;
+	    case PtBreak2:
+	       TtaWriteByte (outfile, C_PR_BREAK2);
+	       break;
 	    default:
 	       fprintf (stderr, "Invalid rule type %X\n", (int)ruleType);
 	       break;
@@ -329,14 +296,7 @@ PRuleType           ruleType;
 /*----------------------------------------------------------------------
    WritePresMode ecrit un mode de calcul dans le fichier		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WritePresMode (PresMode mode)
-
-#else  /* __STDC__ */
-static void         WritePresMode (mode)
-PresMode            mode;
-
-#endif /* __STDC__ */
+static void WritePresMode (PresMode mode)
 {
    switch (mode)
 	 {
@@ -359,13 +319,7 @@ PresMode            mode;
 /*----------------------------------------------------------------------
    WriteInheritMode       ecrit un type d'heritage dans le fichier	
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WriteInheritMode (InheritMode mode)
-#else  /* __STDC__ */
-static void         WriteInheritMode (mode)
-InheritMode         mode;
-
-#endif /* __STDC__ */
+static void WriteInheritMode (InheritMode mode)
 {
    switch (mode)
 	 {
@@ -394,15 +348,7 @@ InheritMode         mode;
 /*----------------------------------------------------------------------
    WriteFunctionType  ecrit un type de fonction de presentation	
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WriteFunctionType (FunctionType functType, ThotBool rep)
-
-#else  /* __STDC__ */
-static void         WriteFunctionType (functType, rep)
-FunctionType        functType;
-ThotBool             rep;
-
-#endif /* __STDC__ */
+static void WriteFunctionType (FunctionType functType, ThotBool rep)
 {
    switch (functType)
 	 {
@@ -482,13 +428,7 @@ ThotBool             rep;
 /*----------------------------------------------------------------------
    WriteAlignment    ecrit un mode d'alignement de lignes		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteAlignment (BAlignment align)
-#else  /* __STDC__ */
-static void         WriteAlignment (align)
-BAlignment          align;
-
-#endif /* __STDC__ */
 {
    switch (align)
 	 {
@@ -517,13 +457,7 @@ BAlignment          align;
 /*----------------------------------------------------------------------
    WritePresCondition      ecrit un type de condition		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                WritePresCondition (PresCondition cond)
-#else  /* __STDC__ */
-void                WritePresCondition (cond)
-PresCondition       cond;
-
-#endif /* __STDC__ */
 {
    switch (cond)
 	 {
@@ -596,6 +530,9 @@ PresCondition       cond;
 	    case PcAttribute:
 	       TtaWriteByte (outfile, C_COND_HAS_ATTR);
 	       break;
+	    case PcInheritAttribute:
+	       TtaWriteByte (outfile, C_COND_INHERIT_ATTR);
+	       break;
 	    case PcNoCondition:
 	       TtaWriteByte (outfile, C_COND_NOCOND);
 	       break;
@@ -612,13 +549,7 @@ PresCondition       cond;
 /*----------------------------------------------------------------------
    WriteCounterValue ecrit la nature de la condition sur un compteur 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteCounterValue (CounterValue val)
-#else  /* __STDC__ */
-static void         WriteCounterValue (val)
-CounterValue        val;
-
-#endif /* __STDC__ */
 {
    switch (val)
 	 {
@@ -641,13 +572,7 @@ CounterValue        val;
 /*----------------------------------------------------------------------
    WriteArithRel     ecrit la relation de la condition Within	
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteArithRel (ArithRel rel)
-#else  /* __STDC__ */
-static void         WriteArithRel (rel)
-ArithRel            rel;
-
-#endif /* __STDC__ */
 {
    switch (rel)
 	 {
@@ -669,13 +594,7 @@ ArithRel            rel;
 /*----------------------------------------------------------------------
    WriteBoxEdge      ecrit un repere de boite			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteBoxEdge (BoxEdge edge)
-#else  /* __STDC__ */
-static void         WriteBoxEdge (edge)
-BoxEdge             edge;
-
-#endif /* __STDC__ */
 {
    switch (edge)
 	 {
@@ -716,13 +635,7 @@ BoxEdge             edge;
 /*----------------------------------------------------------------------
    WriteLevel     ecrit un niveau relatif de boite			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteLevel (Level level)
-#else  /* __STDC__ */
-static void         WriteLevel (level)
-Level               level;
-
-#endif /* __STDC__ */
 {
    switch (level)
 	 {
@@ -766,13 +679,7 @@ Level               level;
 /*----------------------------------------------------------------------
    WriteCounterOp  ecrit un type d'operation sur compteur		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteCounterOp (CounterOp op)
-#else  /* __STDC__ */
-static void         WriteCounterOp (op)
-CounterOp           op;
-
-#endif /* __STDC__ */
 {
    switch (op)
 	 {
@@ -798,13 +705,7 @@ CounterOp           op;
 /*----------------------------------------------------------------------
    WriteBasicType ecrit un type de base				
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteBasicType (BasicType typ)
-#else  /* __STDC__ */
-static void         WriteBasicType (typ)
-BasicType           typ;
-
-#endif /* __STDC__ */
 {
    switch (typ)
 	 {
@@ -836,13 +737,7 @@ BasicType           typ;
 /*----------------------------------------------------------------------
    WriteVariableType    ecrit un type de variable			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                WriteVariableType (VariableType typ)
-#else  /* __STDC__ */
-void                WriteVariableType (typ)
-VariableType        typ;
-
-#endif /* __STDC__ */
 {
    switch (typ)
 	 {
@@ -886,13 +781,7 @@ VariableType        typ;
 /*----------------------------------------------------------------------
    WriteCounterStyle ecrit un style de compteur			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteCounterStyle (CounterStyle style)
-#else  /* __STDC__ */
-static void         WriteCounterStyle (style)
-CounterStyle        style;
-
-#endif /* __STDC__ */
 {
    switch (style)
 	 {
@@ -921,13 +810,7 @@ CounterStyle        style;
 /*----------------------------------------------------------------------
    WriteContentType   ecrit un type de contenu			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                WriteContentType (ContentType typ)
-#else  /* __STDC__ */
-void                WriteContentType (typ)
-ContentType         typ;
-
-#endif /* __STDC__ */
 {
    switch (typ)
 	 {
@@ -950,13 +833,7 @@ ContentType         typ;
 /*----------------------------------------------------------------------
    WriteAttrComparType						
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteAttrComparType (AttrComparType typ)
-#else  /* __STDC__ */
-static void         WriteAttrComparType (typ)
-AttrComparType      typ;
-
-#endif /* __STDC__ */
 {
    switch (typ)
 	 {
@@ -975,13 +852,7 @@ AttrComparType      typ;
 /*----------------------------------------------------------------------
    WriteRefKind						
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WriteRefKind (RefKind kind)
-#else  /* __STDC__ */
-static void         WriteRefKind (kind)
-RefKind      kind;
-
-#endif /* __STDC__ */
 {
    switch (kind)
 	 {
@@ -994,6 +865,12 @@ RefKind      kind;
 	    case RkAttr:
 	       TtaWriteByte (outfile, C_KIND_ATTRIBUTE);
 	       break;
+	    case RkAnyElem:
+	       TtaWriteByte (outfile, C_KIND_ANYELEM);
+	       break;
+	    case RkAnyBox:
+	       TtaWriteByte (outfile, C_KIND_ANYBOX);
+	       break;
 	    default:
 	       fprintf (stderr, "Invalid reference %X\n", kind);
 	       break;
@@ -1003,13 +880,7 @@ RefKind      kind;
 /*----------------------------------------------------------------------
    WritePosRule   ecrit un positionnement relatif			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         WritePosRule (PosRule posRule)
-#else  /* __STDC__ */
-static void         WritePosRule (posRule)
-PosRule             posRule;
-
-#endif /* __STDC__ */
 {
    WriteBoxEdge (posRule.PoPosDef);
    WriteBoxEdge (posRule.PoPosRef);
@@ -1028,13 +899,7 @@ PosRule             posRule;
    WritePRules   ecrit la chaine de regle de presentation qui	
    commence par la regle pointee par pPRule.			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                WritePRules (PtrPRule pPRule)
-#else  /* __STDC__ */
-void                WritePRules (pPRule)
-PtrPRule            pPRule;
-
-#endif /* __STDC__ */
+void                WritePRules (PtrPRule pPRule, PtrSSchema pSS)
 {
    PtrPRule            currentRule;
    PtrCondition        pCond;
@@ -1077,8 +942,19 @@ PtrPRule            pPRule;
 			   }
 			 break;
 		      case PcElemType:
+			 WriteSignedShort (pCond->CoTypeElem);
+			 break;
 		      case PcAttribute:
-			 WriteSignedShort (pCond->CoTypeElAttr);
+		      case PcInheritAttribute:
+			 WriteSignedShort (pCond->CoTypeAttr);
+			 WriteBoolean (pCond->CoTestAttrValue);
+			 if (pCond->CoTestAttrValue)
+			   {
+			     if (pSS->SsAttribute->TtAttr[pCond->CoTypeAttr]->AttrType == AtTextAttr)
+			       WriteName (pCond->CoAttrTextValue);
+			     else
+			       WriteSignedShort (pCond->CoAttrValue);
+			   }
 			 break;
 		      default:
 			 break;
@@ -1087,6 +963,7 @@ PtrPRule            pPRule;
 	  }
 	WritePresCondition (PcNoCondition);
 	WriteShort (currentRule->PrViewNum);
+	WriteBoolean (currentRule->PrDuplicate);
 	WritePresMode (currentRule->PrPresMode);
 	switch (currentRule->PrPresMode)
 	      {
@@ -1138,7 +1015,10 @@ PtrPRule            pPRule;
 			     case PtWeight:
 			     case PtUnderline:
 			     case PtThickness:
+			     case PtDirection:
+			     case PtUnicodeBidi:
 			     case PtLineStyle:
+			     case PtDisplay:
 			     case PtBorderTopStyle:
 			     case PtBorderRightStyle:
 			     case PtBorderBottomStyle:
@@ -1227,25 +1107,14 @@ PtrPRule            pPRule;
    WritePresentationSchema    cree le fichier de sortie et y ecrit	
    le schema de presentation					
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
-ThotBool             WritePresentationSchema (Name fileName, PtrPSchema pPSch, PtrSSchema pSS)
-
-#else  /* __STDC__ */
-ThotBool             WritePresentationSchema (fileName, pPSch, pSS)
-Name                fileName;
-PtrPSchema          pPSch;
-PtrSSchema          pSS;
-
-#endif /* __STDC__ */
-
+ThotBool WritePresentationSchema (Name fileName, PtrPSchema pPSch, PtrSSchema pSS)
 {
    Counter            *pCntr;
    CntrItem           *pCItem;
    PresConstant       *pConst;
    PresVariable       *pVar;
    PresVarItem        *pVarItem;
-   PresentationBox    *pBox;
+   PtrPresentationBox  pBox;
    AttributePres      *pAttPres;
    NumAttrCase        *pCase;
    int                 i, j, k;
@@ -1354,10 +1223,10 @@ PtrSSchema          pSS;
      {
 	pConst = &pPSch->PsConstant[i];
 	WriteBasicType (pConst->PdType);
-	TtaWriteByte (outfile, pConst->PdAlphabet);
+	TtaWriteByte (outfile, pConst->PdScript);
 	j = 0;
 	do
-	   TtaWriteWideChar (outfile, pConst->PdString[j++], ISO_8859_1 /* UTF_8 */);
+	   TtaWriteByte (outfile, pConst->PdString[j++]);
 	while (pConst->PdString[j - 1] != '\0');
      }
 
@@ -1398,7 +1267,7 @@ PtrSSchema          pSS;
    /* ecrit les boites de presentation */
    for (i = 0; i < pPSch->PsNPresentBoxes; i++)
      {
-	pBox = &pPSch->PsPresentBox[i];
+	pBox = pPSch->PsPresentBox->PresBox[i];
 	WriteName (pBox->PbName);
 	WriteRulePtr (pBox->PbFirstPRule);
 	WriteBoolean (pBox->PbPageFooter);
@@ -1423,16 +1292,16 @@ PtrSSchema          pSS;
 
    /* ecrit les presentations des attributs */
    for (i = 0; i < pSS->SsNAttributes; i++)
-      WriteShort (pPSch->PsNAttrPRule[i]);
+      WriteShort (pPSch->PsNAttrPRule->Num[i]);
 
    for (i = 0; i < pSS->SsNAttributes; i++)
      {
-	pAttPres = pPSch->PsAttrPRule[i];
-	for (k = pPSch->PsNAttrPRule[i]; k-- > 0;
+	pAttPres = pPSch->PsAttrPRule->AttrPres[i];
+	for (k = pPSch->PsNAttrPRule->Num[i]; k-- > 0;
 	     pAttPres = pAttPres->ApNextAttrPres)
 	  {
 	     WriteShort (pAttPres->ApElemType);
-	     switch (pSS->SsAttribute[i].AttrType)
+	     switch (pSS->SsAttribute->TtAttr[i]->AttrType)
 		   {
 		      case AtNumAttr:
 			 WriteShort (pAttPres->ApNCases);
@@ -1453,7 +1322,7 @@ PtrSSchema          pSS;
 			 WriteRulePtr (pAttPres->ApTextFirstPRule);
 			 break;
 		      case AtEnumAttr:
-			 for (j = 0; j <= pSS->SsAttribute[i].AttrNEnumValues; j++)
+			 for (j = 0; j <= pSS->SsAttribute->TtAttr[i]->AttrNEnumValues; j++)
 			    WriteRulePtr (pAttPres->ApEnumFirstPRule[j]);
 			 break;
 		   }
@@ -1463,38 +1332,38 @@ PtrSSchema          pSS;
    /* ecrit la table des pointeurs de regle de chaque type du schema de */
    /* structure */
    for (i = 0; i < pSS->SsNRules; i++)
-      WriteRulePtr (pPSch->PsElemPRule[i]);
+      WriteRulePtr (pPSch->PsElemPRule->ElemPres[i]);
 
    /* ecrit toutes les regles de presentation */
    /* ecrit les regles standard */
-   WritePRules (pPSch->PsFirstDefaultPRule);
+   WritePRules (pPSch->PsFirstDefaultPRule, pSS);
 
    /* ecrit les regles des boites */
    for (i = 0; i < pPSch->PsNPresentBoxes; i++)
-      WritePRules (pPSch->PsPresentBox[i].PbFirstPRule);
+      WritePRules (pPSch->PsPresentBox->PresBox[i]->PbFirstPRule, pSS);
 
    /* ecrit les regles des attributs */
    for (i = 0; i < pSS->SsNAttributes; i++)
      {
-	pAttPres = pPSch->PsAttrPRule[i];
-	for (k = pPSch->PsNAttrPRule[i]; k-- > 0;
+	pAttPres = pPSch->PsAttrPRule->AttrPres[i];
+	for (k = pPSch->PsNAttrPRule->Num[i]; k-- > 0;
 	     pAttPres = pAttPres->ApNextAttrPres)
 	  {
-	     switch (pSS->SsAttribute[i].AttrType)
+	     switch (pSS->SsAttribute->TtAttr[i]->AttrType)
 		   {
 		      case AtNumAttr:
 			 for (j = 0; j < pAttPres->ApNCases; j++)
-			    WritePRules (pAttPres->ApCase[j].CaFirstPRule);
+			   WritePRules (pAttPres->ApCase[j].CaFirstPRule, pSS);
 			 break;
 		      case AtReferenceAttr:
-			 WritePRules (pAttPres->ApRefFirstPRule);
+			 WritePRules (pAttPres->ApRefFirstPRule, pSS);
 			 break;
 		      case AtTextAttr:
-			 WritePRules (pAttPres->ApTextFirstPRule);
+			 WritePRules (pAttPres->ApTextFirstPRule, pSS);
 			 break;
 		      case AtEnumAttr:
-			 for (j = 0; j <= pSS->SsAttribute[i].AttrNEnumValues; j++)
-			    WritePRules (pAttPres->ApEnumFirstPRule[j]);
+			 for (j = 0; j <= pSS->SsAttribute->TtAttr[i]->AttrNEnumValues; j++)
+			    WritePRules (pAttPres->ApEnumFirstPRule[j], pSS);
 			 break;
 		   }
 	  }
@@ -1502,19 +1371,19 @@ PtrSSchema          pSS;
 
    /* ecrit les regles des elements */
    for (i = 0; i < pSS->SsNRules; i++)
-      WritePRules (pPSch->PsElemPRule[i]);
+      WritePRules (pPSch->PsElemPRule->ElemPres[i], pSS);
 
    for (i = 0; i < pSS->SsNAttributes; i++)
-      WriteShort (pPSch->PsNHeirElems[i]);
+      WriteShort (pPSch->PsNHeirElems->Num[i]);
 
    for (i = 0; i < pSS->SsNRules; i++)
-      WriteShort (pPSch->PsNInheritedAttrs[i]);
+      WriteShort (pPSch->PsNInheritedAttrs->Num[i]);
 
    for (i = 0; i < pSS->SsNAttributes; i++)
-      WriteShort (pPSch->PsNComparAttrs[i]);
+      WriteShort (pPSch->PsNComparAttrs->Num[i]);
 
    for (i = 0; i < pSS->SsNRules; i++)
-      WriteShort (pPSch->PsElemTransmit[i]);
+      WriteShort (pPSch->PsElemTransmit->Num[i]);
    WriteShort (pPSch->PsNTransmElems);
    for (i = 0; i < pPSch->PsNTransmElems; i++)
      {
