@@ -1,15 +1,15 @@
 /*
  *
- *  (c) COPYRIGHT INRIA 1997.
+ *  (c) COPYRIGHT INRIA 1997-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
  
- /*
+/*
  * HTML tables
  */
 
- /*
+/*
  *
  * Author: I. Vatton (INRIA)
  */
@@ -46,31 +46,21 @@ static ThotBool          Lock = FALSE;
 #include "buildboxes_f.h"
 #include "buildlines_f.h"
 #include "createabsbox_f.h"
+#include "font_f.h"
 #include "exceptions_f.h"
 #include "frame_f.h"
 #include "memory_f.h"
 #include "tree_f.h"
 #define MAX_COLROW 50
 
-#ifdef __STDC__
 static ThotBool SetCellWidths (PtrAbstractBox cell, PtrAbstractBox table, int frame);
-#else
-static ThotBool SetCellWidths (/*PtrAbstractBox cell, PtrAbstractBox table, int frame*/);
-#endif
 
 /*----------------------------------------------------------------------
   DifferFormatting registers differed table formatting in the right
   order:
   The ancestor before its child.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void       DifferFormatting (PtrAbstractBox table, PtrAbstractBox cell, int frame)
-#else  /* __STDC__ */
-static void       DifferFormatting (table, cell, frame)
-PtrAbstractBox    table;
-PtrAbstractBox    cell;
-int               frame;
-#endif /* __STDC__ */
+static void  DifferFormatting (PtrAbstractBox table, PtrAbstractBox cell, int frame)
 {
   PtrLockRelations    pLockRel;
   PtrLockRelations    pPreviousLockRel;
@@ -148,13 +138,7 @@ int               frame;
    IsDifferredTable
    Return TRUE if this table is already registered.        
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static ThotBool   IsDifferredTable(PtrAbstractBox table, PtrAbstractBox cell)
-#else  /* __STDC__ */
-static ThotBool   IsDifferredTable (table, cell)
-PtrAbstractBox    table;
-PtrAbstractBox    cell;
-#endif /* __STDC__ */
 {
   PtrLockRelations    pLockRel;
   int                 i;
@@ -188,13 +172,7 @@ PtrAbstractBox    cell;
   When pRoot is not Null, the returned abstract box has to be included
   within the pRoot.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static PtrAbstractBox NextSiblingAbsBox (PtrAbstractBox pAb, PtrAbstractBox pRoot)
-#else  /* __STDC__ */
-static PtrAbstractBox NextSiblingAbsBox (pAb, pRoot)
-PtrAbstractBox      pAb;
-PtrAbstractBox      pRoot;
-#endif /* __STDC__ */
 {
   ThotBool           skip;
 
@@ -228,13 +206,7 @@ PtrAbstractBox      pRoot;
 /*----------------------------------------------------------------------
    GetTaRBlock allocates table relations
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void       GetTaRBlock (PtrTabRelations *pBlock)
-#else  /* __STDC__ */
-static void       GetTaRBlock (pBlock)
-PtrTabRelations  *pBlock;
-#endif /* __STDC__ */
-
 {
   PtrTabRelations   pNewBlock;
   int               i;
@@ -255,13 +227,7 @@ PtrTabRelations  *pBlock;
 /*----------------------------------------------------------------------
    FreeTaRBlock frees table relations
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void       FreeTaRBlock (PtrTabRelations pBlock)
-#else  /* __STDC__ */
-static void       FreeTaRBlock (pBlock)
-PtrTabRelations   pBlock;
-#endif /* __STDC__ */
-
+static void FreeTaRBlock (PtrTabRelations pBlock)
 {
   if (pBlock != NULL)
     {
@@ -274,13 +240,7 @@ PtrTabRelations   pBlock;
   BuildColOrRowList builds the list of columns or rows included within
   a table
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void      BuildColOrRowList (PtrAbstractBox table, BoxType colrow)
-#else
-static void      BuildOrColRowList (table, colrow)
-PtrAbstractBox   table;
-BoxType          colrow;
-#endif
+static void BuildColOrRowList (PtrAbstractBox table, BoxType colrow)
 {
   PtrTabRelations     pTabRel, pOldTabRel;
   PtrTabRelations     pPreviousTabRel;
@@ -307,21 +267,21 @@ BoxType          colrow;
 
   /* search each column or row box within the table */
   pAb = table;
-  while (pAb != NULL)
+  while (pAb)
     {
-      if (pAb->AbBox != NULL &&
+      if (pAb->AbBox && !pAb->AbDead &&
 	  (pAb->AbBox->BxType == BoRow || pAb->AbBox->BxType == BoColumn))
 	/* skip over the element contents */
 	pAb = NextSiblingAbsBox (pAb, table);
       else
 	pAb = SearchNextAbsBox (pAb, table);
 
-      if (pAb != NULL && !pAb->AbDead && pAb->AbBox != NULL &&
+      if (pAb && pAb->AbBox &&
 	  pAb->AbBox->BxType == BoTable && !pAb->AbPresentationBox)
 	/* it's an included table, skip over this table */
 	pAb = NextSiblingAbsBox (pAb, table);
 
-      if (pAb != NULL && pAb->AbBox != NULL)
+      if (pAb && pAb->AbBox)
 	{
 	  if (pAb->AbBox->BxType == BoRow && colrow == BoColumn)
 	    /* stop the process */
@@ -414,82 +374,6 @@ BoxType          colrow;
     }
 }
 
-#ifdef IV
-/*----------------------------------------------------------------------
-  SetAttrWidthPxl changes the AttrWidthPxl value.
-  ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void     SetAttrWidthPxl (PtrAbstractBox pAb, int width)
-#else
-static void     SetAttrWidthPxl (pAb, width)
-PtrAbstractBox  pAb;
-int             width;
-#endif
-{
-  PtrSSchema          pSS;
-  PtrAttribute        pAttr;
-  int                 attr;
-  ThotBool             found;
-
-  found = FALSE;
-  /* look for ExcNewWidth attribute */
-  pSS = pAb->AbElement->ElStructSchema;
-  attr = GetAttrWithException (ExcNewWidth, pSS);
-  if (attr != 0)
-    {
-      pAttr = pAb->AbElement->ElFirstAttr;
-      found = FALSE;
-      while (!found && pAttr != NULL)
-	if (pAttr->AeAttrNum == attr &&
-	    pAttr->AeAttrSSchema->SsCode == pSS->SsCode &&
-	    (pAttr->AeAttrType == AtNumAttr || pAttr->AeAttrType == AtEnumAttr))
-	  {
-	    found = TRUE;
-            pAttr->AeAttrValue = width;
-	  }
-	else
-	  pAttr = pAttr->AeNext;
-    }
-}
-
-
-/*----------------------------------------------------------------------
-  SetAttrWidthPercent changes the AttrWidthPercent value.
-  ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void     SetAttrWidthPercent (PtrAbstractBox pAb, int width)
-#else
-static void     SetAttrWidthPercent (pAb, width)
-PtrAbstractBox  pAb;
-int             width;
-#endif
-{
-  PtrSSchema          pSS;
-  PtrAttribute        pAttr;
-  int                 attr;
-  ThotBool             found;
-
-  found = FALSE;
-  /* look for ExcNewWidth attribute */
-  pSS = pAb->AbElement->ElStructSchema;
-  attr = GetAttrWithException (ExcNewPercentWidth, pSS);
-  if (attr != 0)
-    {
-      pAttr = pAb->AbElement->ElFirstAttr;
-      found = FALSE;
-      while (!found && pAttr != NULL)
-	if (pAttr->AeAttrNum == attr &&
-	    pAttr->AeAttrSSchema->SsCode == pSS->SsCode &&
-	    (pAttr->AeAttrType == AtNumAttr || pAttr->AeAttrType == AtEnumAttr))
-	  {
-	    found = TRUE;
-            pAttr->AeAttrValue = width;
-	  }
-	else
-	  pAttr = pAttr->AeNext;
-    }
-}
-#endif /* IV */
 
 /*----------------------------------------------------------------------
   GiveAttrWidth returns TRUE if the element width is given
@@ -499,14 +383,8 @@ int             width;
   The parameter percent returns the value of the attribute which has
   exception ExcNewPercentWidth or 0
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static ThotBool  GiveAttrWidth (PtrAbstractBox pAb, int *width, int *percent)
-#else
-static ThotBool  GiveAttrWidth (pAb, width, percent)
-PtrAbstractBox  pAb;
-int            *width;
-int            *percent;
-#endif
+static ThotBool  GiveAttrWidth (PtrAbstractBox pAb, int zoom,
+				int *width, int *percent)
 {
   PtrSSchema          pSS;
   PtrAttribute        pAttr;
@@ -530,7 +408,7 @@ int            *percent;
 	    (pAttr->AeAttrType == AtNumAttr || pAttr->AeAttrType == AtEnumAttr))
 	  {
 	    found = TRUE;
-            *width = pAttr->AeAttrValue;
+            *width = PixelValue (pAttr->AeAttrValue, UnPixel, NULL, zoom);
 	  }
 	else if (pAttr->AeAttrNum == attrPercent &&
 	    pAttr->AeAttrSSchema->SsCode == pSS->SsCode &&
@@ -542,6 +420,20 @@ int            *percent;
 	else
 	  pAttr = pAttr->AeNext;
     }
+  if (!pAb->AbWidth.DimIsPosition &&
+      pAb->AbWidth.DimUnit == UnPercent)
+    {
+      found = TRUE;
+      *width = 0;
+      *percent = pAb->AbWidth.DimValue;
+    }
+  else if (!pAb->AbWidth.DimIsPosition &&
+	  pAb->AbWidth.DimValue > 0)
+    {
+      found = TRUE;
+      *width = PixelValue (pAb->AbWidth.DimValue, pAb->AbWidth.DimUnit, NULL, zoom);
+      *percent = 0;
+    }
   return (found);
 }
 
@@ -549,13 +441,7 @@ int            *percent;
 /*----------------------------------------------------------------------
   CheckRowHeights checks row-spanned cells with related rows.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void     CheckRowHeights (PtrAbstractBox table, int frame)
-#else
-static void     CheckRowHeights (table, frame)
-PtrAbstractBox  table;
-int             frame;
-#endif
 {
   PtrAttribute        pAttr;
   PtrSSchema          pSS = NULL;
@@ -668,7 +554,8 @@ int             frame;
 			   pAttr->AeAttrSSchema = pSS;
 			   pAttr->AeAttrNum = attrHeight;
 			   pAttr->AeAttrType = AtNumAttr;
-			   pAttr->AeAttrValue = rowList[k]->AbBox->BxH + height;
+			   pAttr->AeAttrValue = LogicalValue (rowList[k]->AbBox->BxH + height,
+				   UnPixel, NULL, ViewFrameTable[frame - 1].FrMagnification);
 			   AttachAttrWithValue (rowList[k]->AbElement, pDoc, pAttr);
 			   DeleteAttribute (NULL, pAttr);
 			   /* update the row box */
@@ -695,25 +582,15 @@ int             frame;
 
 /*----------------------------------------------------------------------
   CheckTableWidths
-  Test the coherence between 
-  t.
+  Test the coherence between min, max, width constraint and the current
+  width.
+  The parameter freely lets know that the width can be increased.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void     CheckTableWidths (PtrAbstractBox table, int frame)
-#else
-static void     CheckTableWidths (table, frame)
-PtrAbstractBox  table;
-int             cNumber;
-PtrAbstractBox *colBox;
-int            *colWidth;
-int            *colPercent;
-int             frame;
-ThotBool         force;
-#endif
+static void CheckTableWidths (PtrAbstractBox table, int frame, ThotBool freely)
 {
   PtrAbstractBox      pCell;
   PtrAbstractBox     *colBox;
-  PtrBox              pBox, box;
+  PtrBox              pBox, box = NULL;
   PtrTabRelations     pTabRel;
   int                *colWidth, *colPercent;
   int                 cNumber, cRef;
@@ -734,6 +611,9 @@ ThotBool         force;
 
   /* get the number of columns */
   pBox = table->AbBox;
+  /* check if we're really handling a table */
+  if (pBox && pBox->BxType != BoTable)
+    return;
   pTabRel = pBox->BxColumns;
   cNumber = 0;
   while (pTabRel != NULL)
@@ -751,7 +631,7 @@ ThotBool         force;
   pCell = GetParentCell (pBox);
 
   /* get the inside table width */
-  constraint = GiveAttrWidth (table, &width, &percent);
+  constraint = GiveAttrWidth (table, ViewFrameTable[frame - 1].FrMagnification, &width, &percent);
   if (!constraint)
     /* limit given by available space */
     width = table->AbEnclosing->AbBox->BxW - mbp;
@@ -851,6 +731,7 @@ ThotBool         force;
 	  sumPercent = (100 - delta) * width / 100;
 	  for (cRef = 0; cRef < cNumber; cRef++)
 	    if (colPercent[cRef])
+	      {
 	      if (colPercent[cRef] <= i)
 		{
 		  colPercent[cRef] = 0;
@@ -859,6 +740,7 @@ ThotBool         force;
 		}
 	      else if (colPercent[cRef] > i)
 		colPercent[cRef] -= i;
+	      }
 	}
       else
 	sumPercent = sumPercent * width / 100;
@@ -894,7 +776,7 @@ printf ("Width[%d]=%d\n", cRef, box->BxWidth);
 #endif
 	}
     }
-  else if (min + sum + sumPercent >= width && pCell == NULL)
+  else if (min + sum + sumPercent >= width && (freely || pCell == NULL))
     {
       /* assign the minimum width, or the percent, or the width */
       width = min + sum + sumPercent;
@@ -959,7 +841,8 @@ printf ("Width[%d]=%d\n", cRef, box->BxWidth);
 		    {
 		      /* use the max instead of the min + delta */
 		      delta = delta + box->BxMinWidth - box->BxMaxWidth;
-			n--;
+		      colWidth[cRef] = box->BxMaxWidth;
+		      n--;
 		    }
 		}
 	    }
@@ -1017,7 +900,7 @@ printf ("Width[%d]=%d\n", cRef, box->BxWidth);
 #ifdef TAB_DEBUG
 printf(">>>>>>>>>>>>>>>>>%d\n", table->AbBox->BxWidth);
 #endif
-  if (table->AbBox->BxRows != NULL && table->AbBox->BxRows->TaRTable[0]->AbEnclosing != NULL)
+  if (table->AbBox->BxRows && table->AbBox->BxRows->TaRTable[0]->AbEnclosing)
     RecordEnclosing (table->AbBox->BxRows->TaRTable[0]->AbEnclosing->AbBox, FALSE);
   else
     RecordEnclosing (table->AbBox, FALSE);
@@ -1033,14 +916,12 @@ printf(">>>>>>>>>>>>>>>>>%d\n", table->AbBox->BxWidth);
 
   The table width changes, we need to propagate the change.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void     ChangeTableWidth (PtrAbstractBox table, int frame)
-#else
-static void     ChangeTableWidth (table, frame)
-PtrAbstractBox  table;
-int             frame;
-#endif
+static void ChangeTableWidth (PtrAbstractBox table, int frame)
 {
+  /* table formatting in the main view only */
+  if (FrameTable[frame].FrView != 1)
+    return;
+
   if (Lock)
     /* the table formatting is locked */
     DifferFormatting (table, NULL, frame);
@@ -1050,7 +931,7 @@ int             frame;
   else
     {
       /* Now check the table size */
-      CheckTableWidths (table, frame);
+      CheckTableWidths (table, frame, FALSE);
       CheckRowHeights (table, frame);
     }
 }
@@ -1059,16 +940,8 @@ int             frame;
   GiveCellWidths returns the minimum width, the maximum width, the
   constrained width and the percent width of a specific cell.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void     GiveCellWidths (PtrAbstractBox cell, int *min, int *max, int *width, int *percent)
-#else
-static void     GiveCellWidths (cell, min, max)
-PtrAbstractBox  cell;
-int            *min;
-int            *max;
-int            *width;
-int            *percent;
-#endif
+static void GiveCellWidths (PtrAbstractBox cell, int frame, int *min, int *max,
+			    int *width, int *percent)
 {
   PtrAbstractBox      pAb;
   PtrBox              box;
@@ -1095,8 +968,8 @@ int            *percent;
 	}
       else
 	pAb = SearchNextAbsBox (pAb, cell);
-      if (pAb != NULL && !pAb->AbDead &&
-	  pAb->AbBox != NULL && !pAb->AbPresentationBox)
+      if (pAb && !pAb->AbDead &&
+	  pAb->AbBox && !pAb->AbPresentationBox)
 	{
 	  /* diff between cell's and box's position */
 	  if (pAb->AbBox->BxHorizEdge == Left ||
@@ -1141,9 +1014,15 @@ int            *percent;
   mbp += box->BxRMargin + box->BxRBorder + box->BxRPadding;
   *min = *min + mbp;
   *max = *max + mbp;
-  GiveAttrWidth (cell, width, percent);
+  GiveAttrWidth (cell, ViewFrameTable[frame - 1].FrMagnification, width, percent);
   if (*width)
-    *width = *width + mbp;
+    {
+      *width = *width + mbp;
+      if (*width > *min)
+	*min = *width;
+      if (*width > *max)
+	*max = *width;
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -1156,13 +1035,7 @@ int            *percent;
   - TaRTPercent = the contrained width in percent
   Return TRUE if any table width is modified
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static ThotBool SetTableWidths (PtrAbstractBox table, int frame)
-#else
-static ThtoBool SetTableWidths (table, frame)
-PtrAbstractBox  table;
-int             frame;
-#endif
 {
   PtrAttribute        pAttr;
   PtrSSchema          pSS;
@@ -1189,9 +1062,14 @@ int             frame;
   pSS = table->AbElement->ElStructSchema;
   /* how many columns */
   pBox = table->AbBox;
+  /* check if we're really handling a table */
+  if (pBox && pBox->BxType != BoTable)
+    return FALSE;
   pTabRel = pBox->BxColumns;
   cNumber = 0;
   change = FALSE;
+  span = 1;
+  width = 0;
   while (pTabRel != NULL)
     {
       for (i = 0; i < MAX_RELAT_DIM &&
@@ -1318,16 +1196,19 @@ int             frame;
 				    }
 				}
 			      if (pAttr->AeAttrNum == attrHSpan &&
-				       pAttr->AeAttrSSchema->SsCode == pSS->SsCode)
+				  pAttr->AeAttrSSchema->SsCode == pSS->SsCode)
 				{
 				  foundH = TRUE;
-				  if (pAttr->AeAttrType == AtEnumAttr || pAttr->AeAttrType == AtNumAttr)
-				    {
-				      span = pAttr->AeAttrValue;
-				      /* it could be an invalid span */
-				      if (cRef + span > cNumber)
-					span = cNumber - cRef;
-				    }
+				  if (pAttr->AeAttrType == AtEnumAttr ||
+				      pAttr->AeAttrType == AtNumAttr)
+				     /* ignore values less or equal to 1 */
+				     if (pAttr->AeAttrValue > 1)
+				       {
+				         span = pAttr->AeAttrValue;
+				         /* it could be an invalid span */
+				         if (cRef + span > cNumber)
+					   span = cNumber - cRef;
+				       }
 				}
 			      pAttr = pAttr->AeNext;
 			    }
@@ -1336,7 +1217,7 @@ int             frame;
 		      cell = pAb;
 		      box = cell->AbBox;
 		      /* get the min and max and constrained widths */
-		      GiveCellWidths (cell, &min, &max, &cellWidth, &percent);
+		      GiveCellWidths (cell, frame, &min, &max, &cellWidth, &percent);
 		      if (box->BxMinWidth != min)
 			box->BxMinWidth = min;
 		      if (box->BxMaxWidth != max)
@@ -1411,6 +1292,8 @@ int             frame;
 	{
 	  realMin += colBox[cRef]->AbBox->BxMinWidth;
 	  realMax += colBox[cRef]->AbBox->BxMaxWidth;
+	  if (colPercent[cRef] == 0 && colWidth[cRef] == 0)
+	    span++;
 	}
 	  
       /* compare min and max values */
@@ -1446,7 +1329,7 @@ int             frame;
 	}
     }
 
-  GiveAttrWidth (table, &width, &percent);
+  GiveAttrWidth (table, ViewFrameTable[frame - 1].FrMagnification, &width, &percent);
   /* now update column boxes */
   pTabRel = pBox->BxColumns;
   cRef = 0;
@@ -1536,6 +1419,7 @@ int             frame;
 	  if (cell)
 	    {
 	      /* propagate changes to the enclosing table */
+	      CheckTableWidths (table, frame, TRUE);
 	      row = SearchEnclosingType (cell, BoRow);
 	      if (row  && row->AbBox)
 		table = (PtrAbstractBox) row->AbBox->BxTable;
@@ -1552,14 +1436,7 @@ int             frame;
   If TRUE updates table widths.
   Return TRUE if any table width is modified
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static ThotBool SetCellWidths (PtrAbstractBox cell, PtrAbstractBox table, int frame)
-#else
-static ThtoBool SetCellWidths (cell, table, frame)
-PtrAbstractBox  cell;
-PtrAbstractBox  table;
-int             frame;
-#endif
 {
   PtrBox              box;
   int                 min, max;
@@ -1567,7 +1444,7 @@ int             frame;
   ThotBool            reformat;
 
   box = cell->AbBox;
-  GiveCellWidths (cell, &min, &max, &width, &percent);
+  GiveCellWidths (cell, frame, &min, &max, &width, &percent);
   if (width && width < min)
     /* a constrained width it must be greater than the minimum */
     width = min;
@@ -1594,24 +1471,25 @@ int             frame;
 
   The cell height changes, we need to propagate the change.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void      UpdateCellHeight (PtrAbstractBox cell, int frame)
-#else
-static void      UpdateCellHeight (cell, frame)
-PtrAbstractBox   cell;
-int              frame;
-#endif
+static void UpdateCellHeight (PtrAbstractBox cell, int frame)
 {
   PtrAbstractBox      table;
   PtrAbstractBox      row;
 
-  /* get row and table elements */
-  row = SearchEnclosingType (cell, BoRow);
-  if (row  && row->AbBox)
+  /* table formatting in the main view only */
+  if (FrameTable[frame].FrView != 1)
+    return;
+
+  if (!Lock)
     {
-      table = (PtrAbstractBox) row->AbBox->BxTable;
-      if (table)
-	CheckRowHeights (table, frame);
+      /* get row and table elements */
+      row = SearchEnclosingType (cell, BoRow);
+      if (row  && row->AbBox)
+	{
+	  table = (PtrAbstractBox) row->AbBox->BxTable;
+	  if (table)
+	    CheckRowHeights (table, frame);
+	}
     }
 }
 
@@ -1621,18 +1499,15 @@ int              frame;
 
   The column width changes, we need to propagate the change.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void      UpdateColumnWidth (PtrAbstractBox cell, PtrAbstractBox col, int frame)
-#else
-static void      UpdateColumnWidth (cell, col, frame)
-PtrAbstractBox   cell;
-PtrAbstractBox   col;
-int              frame;
-#endif
+static void UpdateColumnWidth (PtrAbstractBox cell, PtrAbstractBox col, int frame)
 {
   PtrAbstractBox      table;
   PtrAbstractBox      row;
 
+  /* table formatting in the main view only */
+  if (FrameTable[frame].FrView != 1)
+    return;
+
   /* look for the table */
   table = NULL;
   if (col && col->AbBox)
@@ -1661,8 +1536,12 @@ int              frame;
 	      /* there a change within a specific cell */
 	      if (SetCellWidths (cell, table, frame))
 		{
+#ifdef TAB_DEBUG
+		  if (table->AbBox->BxCycles > 0)
+		    printf ("table in progress\n");
+#endif
 		  /* Now check the table size */
-		  CheckTableWidths (table, frame);
+		  CheckTableWidths (table, frame, TRUE);
 		  CheckRowHeights (table, frame);
 		}
 	    }
@@ -1677,18 +1556,15 @@ int              frame;
    - a column (table == NULL && col != NULL)
    - a row (table == NULL && row != NULL)
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void      UpdateTable (PtrAbstractBox table, PtrAbstractBox col, PtrAbstractBox row, int frame)
-#else
-static void      UpdateTable (table, col, row, frame)
-PtrAbstractBox   table;
-PtrAbstractBox   col;
-PtrAbstractBox   row;
-int              frame;
-#endif
+static void UpdateTable (PtrAbstractBox table, PtrAbstractBox col,
+			 PtrAbstractBox row, int frame)
 {
   PtrAbstractBox      pAb;
 
+  /* table formatting in the main view only */
+  if (FrameTable[frame].FrView != 1)
+    return;
+
   if (table == NULL)
     {
       /* look for the table */
@@ -1715,7 +1591,7 @@ int              frame;
   else
     pAb = table;
 
-  if (pAb && pAb->AbBox && !pAb->AbNew && !pAb->AbDead)
+  if (pAb && pAb->AbBox && !pAb->AbNew && !IsDead (pAb))
     {
       /* the table box has been created */
       if (table || col)
@@ -1737,7 +1613,7 @@ int              frame;
 	  if (SetTableWidths (table, frame))
 	    {
 	      /* Now check the table size */
-	      CheckTableWidths (table, frame);
+	      CheckTableWidths (table, frame, TRUE);
 	      CheckRowHeights (table, frame);
 	    }
 	}
@@ -1748,44 +1624,37 @@ int              frame;
 /*----------------------------------------------------------------------
    ClearTable removes table information
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void      ClearTable (PtrAbstractBox table)
-#else
-static void      ClearTable (table)
-PtrAbstractBox   table;
-#endif
+static void ClearTable (PtrAbstractBox table)
 {
   PtrTabSpan          pTabSpan;
   PtrBox              pBox;
 
   /* free specific blocks */
   pBox = table->AbBox;
-  FreeTaRBlock (pBox->BxColumns);
-  pBox->BxColumns = NULL;
-  FreeTaRBlock (pBox->BxRows);
-  pBox->BxRows = NULL;
-  /* remove the list of vertical spanned cells */
-  pTabSpan = pBox->BxSpans;
-  while (pTabSpan != NULL)
+  if (pBox)
     {
-      pBox->BxSpans = pTabSpan->TaSpanNext;
-      TtaFreeMemory (pTabSpan);
+      FreeTaRBlock (pBox->BxColumns);
+      pBox->BxColumns = NULL;
+      FreeTaRBlock (pBox->BxRows);
+      pBox->BxRows = NULL;
+      /* remove the list of vertical spanned cells */
       pTabSpan = pBox->BxSpans;
+      while (pTabSpan)
+	{
+	  pBox->BxSpans = pTabSpan->TaSpanNext;
+	  TtaFreeMemory (pTabSpan);
+	  pTabSpan = pBox->BxSpans;
+	}
     }
 }
 
 
 /*----------------------------------------------------------------------
-   ClearTable removes table information
+   IsFirstColumn returns result = TRUE if cel is within the first
+   column of the table.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void      IsFirstColumn (PtrAbstractBox cell, PtrAbstractBox table, ThotBool *result)
-#else
-static void      IsFirstColumn (cell, table, result)
-PtrAbstractBox   cell;
-PtrAbstractBox   table;
-oolean          *result;
-#endif
+static void IsFirstColumn (PtrAbstractBox cell, PtrAbstractBox table,
+			   ThotBool *result)
 {
   PtrAbstractBox      col, firstcol;
   PtrAttribute        pAttr;
@@ -1838,7 +1707,7 @@ void    TtaLockTableFormatting ()
   ----------------------------------------------------------------------*/
 static void    UnlockTableFormatting ()
 {
-  PtrLockRelations    pLockRel;
+  PtrLockRelations    pLockRel, first;
   PtrAbstractBox      table, cell;
   Propagation         savpropage;
   int                 i;
@@ -1857,7 +1726,9 @@ static void    UnlockTableFormatting ()
 	the contrained width of each column and table starting
 	form the most embedded to the enclosing table
       */
-      pLockRel = DifferedChecks;
+      first = DifferedChecks;
+      DifferedChecks = NULL;
+      pLockRel = first;
       while (pLockRel->LockRNext != NULL)
 	pLockRel = pLockRel->LockRNext;
       while (pLockRel != NULL)
@@ -1870,6 +1741,9 @@ static void    UnlockTableFormatting ()
 	      if (table && table->AbElement)
 		{
 		  cell = pLockRel->LockRCell[i];
+		  if (IsDead (table))
+		    /* nothing to do more on this table */
+		    pLockRel->LockRTable[i] = NULL;
 		  if (cell && cell->AbBox)
 		    {
 		      /* there is a change within a specific cell */
@@ -1878,10 +1752,8 @@ static void    UnlockTableFormatting ()
 			pLockRel->LockRTable[i] = NULL;
 		    }
 		  else
-		    {
-		      /* there is a change within a specific cell */
-		      SetTableWidths (table, pLockRel->LockRFrame[i]);
-		    }
+		    /* there is a change within a specific cell */
+		    SetTableWidths (table, pLockRel->LockRFrame[i]);
 		}
 	      /* next entry */
 	      i--;
@@ -1894,7 +1766,7 @@ static void    UnlockTableFormatting ()
 	Second, reformat all tables starting
 	form the enclosing table to the most embedded
       */
-      pLockRel = DifferedChecks;
+      pLockRel = first;
       while (pLockRel != NULL)
 	{
 	  /* Manage all locked tables */
@@ -1905,7 +1777,7 @@ static void    UnlockTableFormatting ()
 	      if (table && table->AbElement)
 		{
 		  /*pLockRel->LockRTable[i] = NULL;*/
-		  CheckTableWidths (table, pLockRel->LockRFrame[i]);
+		  CheckTableWidths (table, pLockRel->LockRFrame[i], FALSE);
 		  /* need to propagate to enclosing boxes */
 		  ComputeEnclosing (pLockRel->LockRFrame[i]);
 		}
@@ -1919,7 +1791,7 @@ static void    UnlockTableFormatting ()
       /*
 	Then, check all table heighs form the most embedded to the enclosing table
       */
-      pLockRel = DifferedChecks;
+      pLockRel = first;
       while (pLockRel->LockRNext != NULL)
 	pLockRel = pLockRel->LockRNext;
       while (pLockRel != NULL)
@@ -1945,10 +1817,10 @@ static void    UnlockTableFormatting ()
 
       /* Now, free allocated blocks */
       Propagate = savpropage;
-      while (DifferedChecks != NULL)
+      while (first != NULL)
 	{
-	  pLockRel = DifferedChecks;
-	  DifferedChecks = DifferedChecks->LockRNext;
+	  pLockRel = first;
+	  first = pLockRel->LockRNext;
 	  TtaFreeMemory (pLockRel);
 	}
     }
@@ -1958,7 +1830,7 @@ static void    UnlockTableFormatting ()
   TtaUnlockTableFormatting reformats all locked tables
   Redisplay the selection and update scrolling bars
   ----------------------------------------------------------------------*/
-void    TtaUnlockTableFormatting ()
+void TtaUnlockTableFormatting ()
 {
   UnlockTableFormatting ();
 }
@@ -1967,12 +1839,7 @@ void    TtaUnlockTableFormatting ()
 /*----------------------------------------------------------------------
   TtaGiveTableFormattingLock gives the status of the table formatting lock.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void      TtaGiveTableFormattingLock (ThotBool *lock)
-#else  /* __STDC__ */
-void      TtaGiveTableFormattingLock (lock)
-ThotBool *lock;
-#endif /* __STDC__ */
+void TtaGiveTableFormattingLock (ThotBool *lock)
 {
   /* check if we're not processing a TtaUnlockTableFormatting */
   if (DifferedChecks != NULL)
@@ -1985,7 +1852,7 @@ ThotBool *lock;
 /*----------------------------------------------------------------------
    TableHLoadResources : connect resources for managing HTML tables
   ----------------------------------------------------------------------*/
-void                TableHLoadResources ()
+void TableHLoadResources ()
 {
 
    if (ThotLocalActions[T_checktable] == NULL)
