@@ -69,9 +69,9 @@ ThotBool TtaReadByte (BinFile file, unsigned char *bval)
 ThotBool TtaReadWideChar (BinFile file, CHAR_T *bval)
 {
 #ifdef _I18N_
-  int           nbBytesToRead;
+  int           nbBytesToRead, i;
   unsigned char car;
-  unsigned char res;
+  wchar_t       res;
 
   if (TtaReadByte (file, &car) == 0)
     {
@@ -96,33 +96,41 @@ ThotBool TtaReadWideChar (BinFile file, CHAR_T *bval)
   /* See how many bytes to read to build a wide character */
   switch (nbBytesToRead)
     {        /** WARNING: There is not break statement between cases */
-    case 6: res += car;
+    case 6:
+      res += car;
       res <<= 6;
       TtaReadByte (file, &car);
-    case 5: res += car;
+    case 5:
+      res += car;
       res <<= 6;
       TtaReadByte (file, &car);
     case 4: res += car;
       res <<= 6;
       TtaReadByte (file, &car);
-    case 3: res += car;
+    case 3:
+      res += car;
       res <<= 6;
       TtaReadByte (file, &car);
-    case 2: res += car;
+    case 2:
+      res += car;
       res <<= 6;
       TtaReadByte (file, &car);
     case 1: res += car;
     }
-  res -= offset[nbBytesToRead - 1];
 
+  /* Leading bits in each byte are not masked or checked, but
+     accounted for by subtracting the appropriate offset value.
+     This assumes that the input is correct :-(. */
+   i = offset[nbBytesToRead - 1];
+  res -= (wchar_t) i;
   if (res <= 0xFFFF)
     *bval = res;
   else 
     *bval = '?';    
   return (TRUE);
-#else  /* !_I18N_ */
+#else  /* _I18N_ */
   return TtaReadByte (file, bval);
-#endif /* !_I18N_ */
+#endif /* _I18N_ */
 }
 
 /*----------------------------------------------------------------------
@@ -295,7 +303,7 @@ BinFile TtaReadOpen (CONST char *filename)
   ----------------------------------------------------------------------*/
 void TtaReadClose (BinFile file)
 {
-  if (file != NULL)
+  if (file)
     fclose (file);
 }
 
@@ -318,7 +326,7 @@ BinFile TtaWriteOpen (CONST char *filename)
   ----------------------------------------------------------------------*/
 void TtaWriteClose (BinFile file)
 {
-  if (file != NULL)
+  if (file)
     fclose (file);
 }
 
