@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -123,6 +123,9 @@ PtrDict             PtFree_Dict;
 #include "memory_f.h"
 #include "fileaccess_f.h"
 
+
+
+#ifndef _DEBUG
 /*----------------------------------------------------------------------
    TtaGetMemory
 
@@ -163,6 +166,25 @@ void TtaFreeMemory (void *ptr)
       free (ptr);
 }
 
+/*----------------------------------------------------------------------
+   TtaReAlloc increases the size of the memory block.                        
+  ----------------------------------------------------------------------*/
+void *TtaRealloc (void *ptr, unsigned int n)
+{
+   void               *res;
+
+   if (n == 0)
+      n++;
+   res = realloc (ptr, (size_t) n);
+#ifndef _WINDOWS 
+   if (!res)
+     TtaDisplaySimpleMessage (FATAL, LIB, TMSG_NOT_ENOUGH_MEMORY);
+#endif /* _WINDOWS */
+   return res;
+}
+
+#endif /*_DEBUG*/
+
 /*----------------------------------------------------------------------
    FreeAll frees all allocated memory
   ----------------------------------------------------------------------*/
@@ -386,23 +408,6 @@ char *TtaStrdup (char* str)
    return (res);
 }
 
-/*----------------------------------------------------------------------
-   TtaReAlloc increases the size of the memory block.                        
-  ----------------------------------------------------------------------*/
-void *TtaRealloc (void *ptr, unsigned int n)
-{
-   void               *res;
-
-   if (n == 0)
-      n++;
-   res = realloc (ptr, (size_t) n);
-#ifndef _WINDOWS 
-   if (!res)
-     TtaDisplaySimpleMessage (FATAL, LIB, TMSG_NOT_ENOUGH_MEMORY);
-#endif /* _WINDOWS */
-   return res;
-}
-
 /*----------------------------------------------------------------------
    InitEditorMemory initializes block contexts.                                                  
   ----------------------------------------------------------------------*/
@@ -920,6 +925,45 @@ void FreeAbstractBox (PtrAbstractBox pAb)
    NbUsed_AbsBox--;
 }
 
+/*----------------------------------------------------------------------
+  FreeNamespaceDeclarations
+  Free all namespace declarations
+  ----------------------------------------------------------------------*/
+void FreeNamespaceDeclarations (PtrDocument pDoc)
+{
+  PtrNsUriDescr    uriDecl, nextUriDecl;
+  PtrNsPrefixDescr prefixDecl, nextPrefixDecl;
+  
+  if (pDoc->DocNsUriDecl == NULL)
+    return;
+  
+  uriDecl = pDoc->DocNsUriDecl;
+  while (uriDecl != NULL)
+    {
+      nextUriDecl = uriDecl->NsNextUriDecl;
+      prefixDecl = uriDecl->NsPtrPrefix;
+      while (prefixDecl != NULL)
+	{
+	  nextPrefixDecl = prefixDecl->NsNextPrefixDecl;
+	  if (prefixDecl->NsPrefixName != NULL)
+	    {
+	      TtaFreeMemory (prefixDecl->NsPrefixName);
+	      prefixDecl->NsPrefixName = NULL;
+	    }
+	  TtaFreeMemory (prefixDecl);
+	  prefixDecl = nextPrefixDecl;
+	}
+      if (uriDecl->NsUriName != NULL)
+	{
+	  TtaFreeMemory (uriDecl->NsUriName);
+	  uriDecl->NsUriName = NULL;
+	}
+      TtaFreeMemory (uriDecl);
+      uriDecl = nextUriDecl;
+    } 
+  pDoc->DocNsUriDecl = NULL;
+}
+
 /*----------------------------------------------------------------------
    GetDocument alloue un descripteur de document.                  
   ----------------------------------------------------------------------*/
@@ -974,6 +1018,7 @@ void FreeDocument (PtrDocument pDoc)
        pDoc->DocViewModifiedAb[i] = NULL;
      }
    pDoc->DocLabels = NULL;
+   FreeNamespaceDeclarations (pDoc);
 #ifdef DEBUG_MEMORY
    TtaFreeMemory (pDoc);
 #else
@@ -993,7 +1038,10 @@ void GetAttributePres (AttributePres **pAP)
 
   pNewAP = (AttributePres *) TtaGetMemory (sizeof (AttributePres));
   if (pNewAP)
-    memset (pNewAP, 0, sizeof (AttributePres));
+    {
+      memset (pNewAP, 0, sizeof (AttributePres));
+      pNewAP->ApMatch = 0;
+    }
   *pAP = pNewAP;
 }
 
@@ -1404,6 +1452,8 @@ void GetPresentRule (PtrPRule * pRP)
   if (pNewRP)
     {
       memset (pNewRP, 0, sizeof (PresRule));
+      pNewRP->PrImportant = FALSE;
+      pNewRP->PrDuplicate = FALSE;
       pNewRP->PrNextPRule = NULL;
       pNewRP->PrSpecifAttr = 0;
       pNewRP->PrSpecifAttrSSchema = NULL;
@@ -1698,7 +1748,9 @@ PtrBox FreeBox (PtrBox pBox)
 
    /* get next child */
    if (pBox->BxType == BoSplit ||
-       pBox->BxType == BoPiece || pBox->BxType == BoDotted)
+       pBox->BxType == BoPiece ||
+       pBox->BxType == BoScript ||
+       pBox->BxType == BoDotted)
      NextBox = pBox->BxNexChild;
    else
      NextBox = NULL;
@@ -1831,9 +1883,9 @@ void GetSearchContext (PtrSearchContext * pSearch)
 	  s->SDocument = NULL;
 	  s->STree = 0;
 	  s->SStartElement = NULL;
-	  s->SStartChar = 0;
+	  s->SStartChar = 1;
 	  s->SEndElement = NULL;
-	  s->SEndChar = 0;
+	  s->SEndChar = 1;
 	  s->SStartToEnd = TRUE;
 	  s->SWholeDocument = FALSE;
 	}
