@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -181,7 +181,7 @@ void SplitTextElement (PtrElement pEl, int rank, PtrDocument pDoc,
 	       }
 	     /* longueur +1 du texte qui restera dans le dernier buffer (pBuf)
 	        de la premiere partie */
-	     len = rank - i;
+	     len = rank - i - 1;
 	     pBuf1 = pEl2->ElText;
 	     pBuf1->BuNext = pBuf->BuNext;
 	     if (pBuf1->BuNext != NULL)
@@ -190,38 +190,36 @@ void SplitTextElement (PtrElement pEl, int rank, PtrDocument pDoc,
 	     i = 0;
 	     do
 	       {
-		  pBuf1->BuContent[i] = pBuf->BuContent[len + i - 1];
+		  pBuf1->BuContent[i] = pBuf->BuContent[len + i];
 		  i++;
 	       }
 	     while (pBuf1->BuContent[i - 1] != EOS);
 	     pBuf1->BuLength = i - 1;
-	     pBuf1 = pBuf;
 	     /* met a jour le dernier buffer de la premiere partie */
-	     pBuf1->BuNext = NULL;
-	     pBuf1->BuContent[len - 1] = EOS;
-	     pBuf1->BuLength = len - 1;
+	     pBuf->BuNext = NULL;
+	     pBuf->BuContent[len] = EOS;
+	     pBuf->BuLength = len;
 	     /* essaie de vider ce dernier buffer dans le precedent */
-	     if (pBuf1->BuPrevious != NULL)
-		/* il y a un buffer precedent */
-		if (pBuf1->BuPrevious->BuLength + pBuf1->BuLength < THOT_MAX_CHAR)
-		   /* ca peut tenir */
-		  {
-		     i = 0;
-		     while (i <= pBuf1->BuLength)
-		       {
-			  pBuf1->BuPrevious->BuContent[pBuf1->BuPrevious->BuLength + i] =
-			     pBuf1->BuContent[i];
-			  i++;
-		       }
-		     pBuf1->BuPrevious->BuLength += pBuf1->BuLength;
-		     /* supprime le dernier buffer de la premiere partie : il est */
-		     /* vide */
-		     pBuf1->BuPrevious->BuNext = NULL;
-		     FreeTextBuffer (pBuf);
-		  }
+	     pBuf1 = pBuf->BuPrevious;
+	     if (pBuf1 &&
+		 pBuf1->BuLength + pBuf->BuLength <= FULL_BUFFER)
+	       /* il y a un buffer precedent et ca peut tenir */
+	       {
+		 i = 0;
+		 while (i <= pBuf->BuLength)
+		   {
+		     pBuf1->BuContent[pBuf1->BuLength + i] = pBuf->BuContent[i];
+		     i++;
+		   }
+		 pBuf1->BuLength += pBuf->BuLength;
+		 /* supprime le dernier buffer de la premiere partie : il est */
+		 /* vide */
+		 pBuf1->BuNext = NULL;
+		 FreeTextBuffer (pBuf);
+	       }
 	     /* accroche les buffers de la deuxieme partie a leur element */
 	     pBuf = pEl2->ElText;
-	     while (pBuf != NULL)
+	     while (pBuf)
 		pBuf = pBuf->BuNext;
 	     if (withAppEvent)
 	       {
@@ -260,7 +258,7 @@ void SplitTextElement (PtrElement pEl, int rank, PtrDocument pDoc,
 /*--------------------------------------------------------------------
   MergeTextElements   si l'element pEl est une feuille de texte et si
   son suivant est egalement une feuille de texte dans le meme
-  alphabet, reunit les deux elements sucessifs en un seul.
+  script, reunit les deux elements sucessifs en un seul.
   Le deuxieme element, devenu vide, n'est pas libere' et un
   pointeur sur cet element est retourne dans pFreeEl.
   Le parametre booleen removeAbsBox indique s'il faut liberer les
@@ -350,7 +348,7 @@ ThotBool MergeTextElements (PtrElement pEl, PtrElement *pFreeEl, PtrDocument pDo
 		pBuf1 = pBuf1->BuNext;
 	      /* premier buffer du deuxieme element */
 	      pBuf2 = pEl2->ElText;
-	      if (pBuf1->BuLength + pBuf2->BuLength < THOT_MAX_CHAR)
+	      if (pBuf1->BuLength + pBuf2->BuLength <= FULL_BUFFER)
 		/* copie le contenu du premier buffer du 2eme element */
 		/* a la fin du dernier buffer du 1er element */
 		{
@@ -689,20 +687,20 @@ void CopyTextToText (PtrTextBuffer pSrceBuf, PtrTextBuffer pCopyBuf, int *len)
   PtrTextBuffer       pTBSrce, pTBDest;
 
   *len = 0;
-  if (pSrceBuf != NULL && pCopyBuf != NULL)
+  if (pSrceBuf && pCopyBuf)
     {
       /* cherche le dernier buffer de la chaine des buffers destination */
       pTBDest = pCopyBuf;
-      while (pTBDest->BuNext != NULL)
+      while (pTBDest->BuNext)
 	pTBDest = pTBDest->BuNext;
       pTBSrce = pSrceBuf;
       /* parcourt la chaine des buffers source */
-      while (pTBSrce != NULL)
+      while (pTBSrce)
 	{
 	  if (pTBSrce->BuLength > 0)
 	    /* ce buffer source n'est pas vide, on le copie */
 	    {
-	      if (THOT_MAX_CHAR - 1 - pTBDest->BuLength <= pTBSrce->BuLength)
+	      if (FULL_BUFFER - pTBDest->BuLength < pTBSrce->BuLength)
 		/* pas assez de place dans le buffer destination */
 		/* pour copier tout le texte du buffer source, on */
 		/* prend un nouveau buffer destination */
@@ -739,7 +737,7 @@ int CopyMBs2Buffer (unsigned char *src, PtrTextBuffer pBuf, int pos, int max)
     {
 #ifdef _I18N_
       l = 0;
-      while (pos < THOT_MAX_CHAR - 1 && max > l && src[0] != EOS)
+      while (pos < FULL_BUFFER && max > l && src[0] != EOS)
 	{
 	  l += TtaMBstringToWC (&src, &pBuf->BuContent[pos]);
 	  pos++;
@@ -749,7 +747,7 @@ int CopyMBs2Buffer (unsigned char *src, PtrTextBuffer pBuf, int pos, int max)
 	max = 0;
 #else /* _I18N_ */
       /* length of copied text in that buffer */
-      l = THOT_MAX_CHAR - 1 - pos;
+      l = FULL_BUFFER - pos;
       if (max < l)
 	l = max;
       if (l > 0)
@@ -1091,27 +1089,28 @@ int TtaGetTextLength (Element element)
   else if (pEl->ElLeafType != LtText && pEl->ElLeafType != LtPicture)
     TtaError (ERR_invalid_element_type);
   else
-    length = pEl->ElTextLength;
-#ifdef _I18N_
-  pBuf = pEl->ElText;
-  l = 0;
-  while (pBuf != NULL && length > 0)
     {
-      i = 0;
-      while (i < pBuf->BuLength && i < length)
+      length = pEl->ElTextLength;
+#ifdef _I18N_
+      pBuf = pEl->ElText;
+      l = 0;
+      while (pBuf != NULL && length > 0)
 	{
-	  ptr = c;
-	  l += TtaWCToMBstring (pBuf->BuContent[i], &ptr);
-	  i++;
+	  i = 0;
+	  while (i < pBuf->BuLength && i < length)
+	    {
+	      ptr = c;
+	      l += TtaWCToMBstring (pBuf->BuContent[i], &ptr);
+	      i++;
+	    }
+	  /* next buffer */
+	  length -= pBuf->BuLength;
+	  pBuf = pBuf->BuNext;
 	}
-      /* next buffer */
-      length -= pBuf->BuLength;
-      pBuf = pBuf->BuNext;
-    }
-  return l;
-#else /* _I18N_ */
-  return length;
+      length = l;
 #endif /* _I18N_ */
+    }
+  return length;
 }
 
 /*----------------------------------------------------------------------
@@ -1138,9 +1137,6 @@ void TtaGiveTextContent (Element element, unsigned char *buffer, int *length,
   PtrTextBuffer       pBuf;
   unsigned char      *ptr;
   int                 len, l;
-#ifdef _I18N_
-  int                 i;
-#endif /* _I18N_ */
 
   UserErrorCode = 0;
   pEl = (PtrElement) element;
