@@ -1,19 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2001
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
- 
 /*
  * References search
  *
@@ -49,7 +40,6 @@
 #include "scroll_f.h"
 #include "views_f.h"
 #include "callback_f.h"
-#include "viewcommands_f.h"
 #include "exceptions_f.h"
 #include "absboxes_f.h"
 #include "memory_f.h"
@@ -107,19 +97,10 @@
    mais ce document n'est pas charge' (cela ne se  	
    produit que si processNotLoaded est TRUE).       	
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-PtrReference        NextReferenceToEl (PtrElement pEl, PtrDocument pDoc, ThotBool processNotLoaded, PtrReference pPrevRef, PtrDocument * pDocRef, PtrExternalDoc * pExtDoc, ThotBool nextExtDoc)
-#else  /* __STDC__ */
-PtrReference        NextReferenceToEl (pEl, pDoc, processNotLoaded, pPrevRef, pDocRef, pExtDoc, nextExtDoc)
-PtrElement          pEl;
-PtrDocument         pDoc;
-ThotBool            processNotLoaded;
-PtrReference        pPrevRef;
-PtrDocument        *pDocRef;
-PtrExternalDoc     *pExtDoc;
-ThotBool            nextExtDoc;
-
-#endif /* __STDC__ */
+PtrReference  NextReferenceToEl (PtrElement pEl, PtrDocument pDoc,
+				 ThotBool processNotLoaded,
+				 PtrReference pPrevRef, PtrDocument * pDocRef,
+				 PtrExternalDoc * pExtDoc, ThotBool nextExtDoc)
 {
    PtrReference        pRef;
 
@@ -180,18 +161,9 @@ ThotBool            nextExtDoc;
    references dans le document designe' par        
    pExtDoc, mais ce document n'est pas charge'     
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                FindReference (PtrReference * pPrevRef, PtrDocument * pDocPrevRef, PtrElement * pReferredEl, PtrDocument * pDocReferredEl, PtrExternalDoc * pExtDoc, ThotBool nextExtDoc)
-#else  /* __STDC__ */
-void                FindReference (pPrevRef, pDocPrevRef, pReferredEl, pDocReferredEl, pExtDoc, nextExtDoc)
-PtrReference       *pPrevRef;
-PtrDocument        *pDocPrevRef;
-PtrElement         *pReferredEl;
-PtrDocument        *pDocReferredEl;
-PtrExternalDoc     *pExtDoc;
-ThotBool            nextExtDoc;
-
-#endif /* __STDC__ */
+void FindReference (PtrReference * pPrevRef, PtrDocument * pDocPrevRef,
+		    PtrElement * pReferredEl, PtrDocument * pDocReferredEl,
+		    PtrExternalDoc * pExtDoc, ThotBool nextExtDoc)
 {
    PtrElement          firstSel;
    PtrElement          lastSel;
@@ -212,11 +184,9 @@ ThotBool            nextExtDoc;
       /* pas de reference courante */
      {
 	/* prend la selection courante */
-	ok = GetCurrentSelection (&pSelDoc, &firstSel, &lastSel, &firstChar, &lastChar);
-	if (!ok)
-	   /* pas de selection, message 'Selectionnez' */
-	   TtaDisplaySimpleMessage (INFO, LIB, TMSG_SEL_EL);
-	else
+	ok = GetCurrentSelection (&pSelDoc, &firstSel, &lastSel, &firstChar,
+				  &lastChar);
+	if (ok)
 	   /* cherche le premier element reference' qui englobe la */
 	   /* selection courante */
 	  {
@@ -298,17 +268,13 @@ void                FindReferredEl ()
    PtrAttribute        pAttr;
    PtrReference        pRef;
    DocumentIdentifier  docIdent;
-   PtrDocument         pDoc;
    int                 nMenuItems;
-   CHAR_T              menuBuf[MAX_TXT_LEN];
+   char                menuBuf[MAX_TXT_LEN];
    int                 menuBufLen;
    PtrReference        pRefTable[MAX_ITEM_MENU_REF];
-   int                 chosenItem;
 
    /* y-a-t'il une selection au niveau editeur ou mediateur ? */
-   if (!GetCurrentSelection (&pSelDoc, &firstSel, &lastSel, &firstChar, &lastChar))
-      TtaDisplaySimpleMessage (INFO, LIB, TMSG_SEL_EL);
-   else
+   if (GetCurrentSelection (&pSelDoc, &firstSel, &lastSel, &firstChar, &lastChar))
      {
 	pEl = NULL;
 	pRef = NULL;
@@ -322,8 +288,9 @@ void                FindReferredEl ()
 	     {
 		pRefTable[nMenuItems] = firstSel->ElReference;
 		nMenuItems++;
-		ustrcpy (menuBuf + menuBufLen, firstSel->ElStructSchema->SsRule[firstSel->ElTypeNumber - 1].SrName);
-		menuBufLen += ustrlen (menuBuf) + 1;
+		strcpy (menuBuf + menuBufLen,
+			firstSel->ElStructSchema->SsRule->SrElem[firstSel->ElTypeNumber - 1]->SrName);
+		menuBufLen += strlen (menuBuf) + 1;
 	     }
 	if (firstSel->ElSource != NULL)
 	   /* la selection est un element inclus */
@@ -345,8 +312,9 @@ void                FindReferredEl ()
 		     {
 			pRefTable[nMenuItems] = pAttr->AeAttrReference;
 			nMenuItems++;
-			ustrcpy (menuBuf + menuBufLen, pAttr->AeAttrSSchema->SsAttribute[pAttr->AeAttrNum - 1].AttrName);
-			menuBufLen += ustrlen (pAttr->AeAttrSSchema->SsAttribute[pAttr->AeAttrNum - 1].AttrName) + 1;
+			strcpy (menuBuf + menuBufLen,
+				pAttr->AeAttrSSchema->SsAttribute->TtAttr[pAttr->AeAttrNum - 1]->AttrName);
+			menuBufLen += strlen (pAttr->AeAttrSSchema->SsAttribute->TtAttr[pAttr->AeAttrNum - 1]->AttrName) + 1;
 		     }
 	     pAttr = pAttr->AeNext;
 	  }
@@ -354,47 +322,42 @@ void                FindReferredEl ()
 	   pRef = NULL;
 	else if (nMenuItems == 1)
 	   pRef = pRefTable[0];
+#ifdef IV
 	else
 	  {
-	     BuildReferenceMenu (menuBuf, nMenuItems, &chosenItem);
-	     pRef = pRefTable[chosenItem];
+	    BuildReferenceMenu (menuBuf, nMenuItems, &chosenItem);
+	    pRef = pRefTable[chosenItem];
 	  }
 	if (pRef != NULL)
 	   /* c'est bien une reference qui est selectionnee */
 	  {
 	     /* l'element reference' est pointe' par pEl */
 	     pEl = ReferredElement (pRef, &docIdent, &pDoc);
-	     if (pEl == NULL)
+	     if (pEl == NULL &&
 		/* il n'y a pas d'element reference' */
-		if (DocIdentIsNull (docIdent) || pDoc != NULL)
-		   /* ce n'est pas une reference externe ou c'est une reference vide */
-		   TtaDisplaySimpleMessage (INFO, LIB, TMSG_EMPTY_REF);
-		else
-		   /* l'element reference' est dans un autre document qui */
-		   /* n'est pas charge' */
-		   /* on proposera ce nom comme nom par defaut lorsque */
-		   /* l'utilisateur demandera a ouvrir un document */
-		  {
-		    LoadDocument (&pDoc, docIdent);
-		    if (pDoc != NULL)
-		      {
-			/* annule la selection */
-			TtaClearViewSelections ();
-			/* le chargement du document a reussi */
-			pEl = ReferredElement (pRef, &docIdent, &pDoc);
-			/* s'il s'agit d'une inclusion de */
-			/* document, applique les regles Transmit */
-			ApplyTransmitRules (pRef->RdElement, pSelDoc);
-		      }
-		  }
-	     else if (IsASavedElement (pEl))
+		 !DocIdentIsNull (docIdent) && pDoc == NULL)
+	       /* l'element reference' est dans un autre document qui */
+	       /* n'est pas charge' */
+	       /* on proposera ce nom comme nom par defaut lorsque */
+	       /* l'utilisateur demandera a ouvrir un document */
 	       {
-		 /* l'element reference est-il dans le buffer de sauvegarde ? */
-		 pEl = NULL;
-		 /* message 'CsReference vide' */
-		 TtaDisplaySimpleMessage (INFO, LIB, TMSG_EMPTY_REF);
+		 LoadDocument (&pDoc, docIdent);
+		 if (pDoc != NULL)
+		   {
+		     /* annule la selection */
+		     TtaClearViewSelections ();
+		     /* le chargement du document a reussi */
+		     pEl = ReferredElement (pRef, &docIdent, &pDoc);
+		     /* s'il s'agit d'une inclusion de */
+		     /* document, applique les regles Transmit */
+		     ApplyTransmitRules (pRef->RdElement, pSelDoc);
+		   }
 	       }
+	     else if (IsASavedElement (pEl))
+	       /* l'element reference est-il dans le buffer de sauvegarde ? */
+	       pEl = NULL;
 	  }
+#endif /* IV */
 
 	if (pEl != NULL)
 	  {
@@ -409,147 +372,12 @@ void                FindReferredEl ()
      }
 }
 
-
-/*----------------------------------------------------------------------
-   RegisterPastedReferredElem					
-   	L'element pEl vient d'etre colle'				
-   dans le document pDoc et l'original vient d'un document		
-   different. S'il possede des elements reference's par d'autres   
-   documents, on note dans le contexte du document pDoc que cet    
-   element a change' de document. Ce sera utile lorsqu'on sauvera  
-   le document pDoc pour mettre a jour les fichiers .REF des       
-   documents referencant les elements deplace's.                   
-  ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         RegisterPastedReferredElem (PtrElement pEl, PtrDocument pDoc, LabelString oldLabel)
-#else  /* __STDC__ */
-static void         RegisterPastedReferredElem (pEl, pDoc, oldLabel)
-PtrElement          pEl;
-PtrDocument         pDoc;
-LabelString         oldLabel;
-
-#endif /* __STDC__ */
-{
-   PtrChangedReferredEl pChnRef;
-
-   if (pEl->ElReferredDescr != NULL)
-      if (pEl->ElReferredDescr->ReExtDocRef != NULL)
-	 /* l'element est reference' par d'autres documents */
-	{
-	   /* acquiert un descripteur et le remplit */
-	   GetChangedReferredEl (&pChnRef);
-	   strncpy (pChnRef->CrOldLabel, oldLabel, MAX_LABEL_LEN);
-	   strncpy (pChnRef->CrNewLabel, pEl->ElLabel, MAX_LABEL_LEN);
-	   CopyDocIdent (&pChnRef->CrOldDocument, DocOfSavedElements->DocIdent);
-	   CopyDocIdent (&pChnRef->CrNewDocument, pDoc->DocIdent);
-	   pChnRef->CrReferringDoc = NULL;
-	   /* chaine ce descripteur */
-	   pChnRef->CrNext = pDoc->DocChangedReferredEl;
-	   pDoc->DocChangedReferredEl = pChnRef;
-	   /* copie la liste des documents qui referencent l'element */
-	   CopyDescrExtDoc (pEl, pChnRef);
-	}
-}
-
-/*----------------------------------------------------------------------*/
-  /* UpdateReferences      */    
-   /* traitement particulier a faire sur les references si */
-   /* la copie n'est pas dans le meme document que l'original */
-   /* et que l'on a fait un copier/coller d'un element LtReference */
-   /* ou d'un element portant un attribut reference, */
-/*----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void                UpdateReferences (PtrElement pRoot, PtrDocument pDoc, PtrReference pRef)
-#else  /* __STDC__ */
-static void                updateReferences (pRoot, pDoc, pRef)
-PtrElement          pRoot;
-PtrDocument         pDoc;
-PtrReference        pRef;
-
-#endif /* __STDC__ */
-
-{
-   PtrElement          pElRef;
-   DocumentIdentifier  docIdentRef;
-   PtrDocument         pDocRef;
-   ThotBool            sameDoc;
-
-	   if (pRef->RdReferred != NULL)
-	     {
-		pElRef = ReferredElement (pRef, &docIdentRef, &pDocRef);
-		if (pElRef == NULL && DocIdentIsNull (docIdentRef))
-		   /* la reference ne designe rien */
-		   pRef->RdReferred = NULL;
-		else
-		  {
-		     if (pElRef != NULL)
-			if (!IsASavedElement (pElRef))
-			   if (IsWithinANewElement (pElRef))
-			      /* l'element reference' est aussi colle' */
-			      pDocRef = pDoc;
-		     if (pRef->RdInternalRef)
-			/* l'original etait une reference interne a son document */
-			/* d'origine, l'element reference' appartient donc au */
-			/* document d'origine */
-			pDocRef = DocOfSavedElements;
-		     else
-			/* l'original etait une reference externe a son document */
-			/* d'origine */
-		     if (SameDocIdent (pDoc->DocIdent, docIdentRef))
-			/* l'element reference' et la reference sont dans le */
-			/* meme document */
-			pDocRef = pDoc;
-		     if (pDocRef != NULL)
-			/* le document contenant l'element reference' est charge' */
-		       {
-			  /* si l'original n'est pas une reference interne, */
-			  /* l'element reference' ne peut pas avoir ete copie' en */
-			  /* meme temps */
-			  if (!pRef->RdInternalRef)
-			    {
-			     /* lie la reference a l'element designe' par l'original */
-		             if (pRef->RdAttribute != NULL)
-		               pRoot = NULL;
-			     SetReference (pRoot, pRef->RdAttribute, pElRef, pDoc, pDocRef, FALSE, FALSE);
-			    }
-			  else
-			     /* la reference originale etait une reference interne */
-			    {
-			       sameDoc = FALSE;
-			       if (pElRef != NULL)
-				  if (!IsASavedElement (pElRef))
-				     /* l'element reference' est aussi colle', on ne fait */
-				     /* rien : ce cas est traite' plus haut */
-				     sameDoc = IsWithinANewElement (pElRef);
-			       if (!sameDoc)
-
-				  /* etablit le lien entre la reference copie'e et */
-				  /* l'element reference */
-				 {
-				   if (pRef->RdAttribute != NULL)
-				     pRoot = NULL;
-				  SetReference (pRoot, pRef->RdAttribute, pElRef, pDoc, pDocRef, FALSE, FALSE);
-				 }
-			    }
-		       }
-		  }
-	     }
-}
-
 /*----------------------------------------------------------------------
    CheckReferences        On vient de coller le sous-arbre de racine	
    pRoot dans le document pDoc. Verifie la coherence des elements	
    reference's et des references presents dans ce sous-arbre.      
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void                CheckReferences (PtrElement pRoot, PtrDocument pDoc)
-#else  /* __STDC__ */
-void                CheckReferences (pRoot, pDoc)
-PtrElement          pRoot;
-PtrDocument         pDoc;
-
-#endif /* __STDC__ */
-
 {
    PtrElement          pEl, pElRef, pSource;
    PtrReference        pRef, pNextRef;
@@ -571,20 +399,21 @@ PtrDocument         pDoc;
 	ConvertIntToLabel (l, pRoot->ElLabel);
      }
    if (pRoot->ElReferredDescr != NULL)
-      /* l'original de l'element colle' est reference' ou possede un label */
+      /* l'original de l'element colle' est reference' */
      {
 	if (pRoot->ElReferredDescr->ReExternalRef)
 	   pSource = NULL;
 	else
-	   pSource = pRoot->ElReferredDescr->ReReferredElem;	/* l'element original */
+	   /* l'element original */
+	   pSource = pRoot->ElReferredDescr->ReReferredElem;
 
 	pExtDoc = NULL;
 	pRoot->ElReferredDescr = NULL;
-	/* l'element colle' n'est pas reference', il prend le label de son     */
-	/* element original (celui qui est dans le tampon Couper-Copier)       */
-	/* si la commande precedente etait Couper et s'il ne change pas de     */
-	/* document. Dans les autres cas (commande precedente Copier ou        */
-	/* changement de document), il prend un nouveau label.                 */
+	/* l'element colle' n'est pas reference', il prend le label de son
+	   element original (celui qui est dans le tampon Couper-Copier)
+	   si la commande precedente etait Couper et s'il ne change pas de
+	   document. Dans les autres cas (commande precedente Copier ou
+	   changement de document), il prend un nouveau label. */
 
 	/* alloue a l'element un descripteur d'element reference' */
 	pRoot->ElReferredDescr = NewReferredElDescr (pDoc);
@@ -595,33 +424,35 @@ PtrDocument         pDoc;
 	   strncpy (pRoot->ElLabel, pSource->ElLabel, MAX_LABEL_LEN);
 	if (!ChangeLabel && pSource != NULL)
 	  {
-	     /* l'element prend les memes descripteurs de documents */
-	     /* externes referencants que l'element original */
-	     pExtDoc = pSource->ElReferredDescr->ReExtDocRef;
-	     while (pExtDoc != NULL)
-	       {
-		  /* on ne considere pas le document lui-meme comme externe... */
-		  if (!SameDocIdent (pExtDoc->EdDocIdent, pDoc->DocIdent))
-		     AddDocOfExternalRef (pRoot, pExtDoc->EdDocIdent, pDoc);
-		  pExtDoc = pExtDoc->EdNext;
-	       }
+	    /* l'element prend les memes descripteurs de documents */
+	    /* externes referencants que l'element original */
+	    pExtDoc = pSource->ElReferredDescr->ReExtDocRef;
+	    while (pExtDoc != NULL)
+	      {
+		/* on ne considere pas le document lui-meme comme externe... */
+		if (!SameDocIdent (pExtDoc->EdDocIdent, pDoc->DocIdent))
+		  AddDocOfExternalRef (pRoot, pExtDoc->EdDocIdent, pDoc);
+		pExtDoc = pExtDoc->EdNext;
+	      }
 	  }
 
 	/* on cherche toutes les references a l'element original et on les */
 	/* fait pointer sur l'element colle'. */
 	/* cherche d'abord la premiere reference */
-	pRef = NextReferenceToEl (pSource, DocOfSavedElements, FALSE, NULL, &pDocRef, &pExtDoc, TRUE);
-	pNextDocRef = pDocRef;	/* a priori la reference suivante sera dans le */
-	/* meme document */
+	pRef = NextReferenceToEl (pSource, DocOfSavedElements, FALSE, NULL,
+				  &pDocRef, &pExtDoc, TRUE);
+	/* a priori la reference suivante sera dans le meme document */
+	pNextDocRef = pDocRef;
 	while (pRef != NULL)
 	  {
 	     /* cherche la reference suivante a l'original avant de modifier */
 	     /* la reference courante */
-	     pNextRef = NextReferenceToEl (pSource, DocOfSavedElements, FALSE, pRef, &pNextDocRef, &pExtDoc, TRUE);
+	     pNextRef = NextReferenceToEl (pSource, DocOfSavedElements, FALSE,
+					   pRef, &pNextDocRef, &pExtDoc, TRUE);
 	     /* traite la reference courante */
-	     /* si elle est dans le tampon, on n'y touche pas : sa copie dans */
-	     /* le document ou on colle a deja ete traitee ou sera traitee dans */
-	     /* cette boucle */
+	     /* si elle est dans le tampon, on n'y touche pas : sa copie dans
+	        le document ou on colle a deja ete traitee ou sera traitee
+		dans cette boucle */
 	     if (!IsASavedElement (pRef->RdElement))
 		/* on fait pointer la reference sur l'element colle' */
 	       {
@@ -631,7 +462,8 @@ PtrDocument         pDoc;
 		     pElemRef = NULL;
 		  else
 		     pElemRef = pRef->RdElement;
-		  SetReference (pElemRef, pRef->RdAttribute, pRoot, pDocRef, pDoc, FALSE, FALSE);
+		  SetReference (pElemRef, pRef->RdAttribute, pRoot, pDocRef,
+				pDoc, FALSE, FALSE);
 		  /* si c'est une reference par attribut, verifie la */
 		  /* validite de l'attribut dans le cas des extensions de */
 		  /* cellule des tableaux */
@@ -643,10 +475,6 @@ PtrDocument         pDoc;
 	     pRef = pNextRef;	/* passe a la reference suivante */
 	     pDocRef = pNextDocRef;
 	  }
-	if (!ChangeLabel && pDoc != DocOfSavedElements)
-	   /* l'element a change' de document, on le note dans le */
-	   /* contexte de son nouveau document */
-	   RegisterPastedReferredElem (pRoot, pDoc, oldLabel);
      }
    /* Traite les attributs de type reference porte's par l'element */
    pAttr = pRoot->ElFirstAttr;
@@ -693,9 +521,6 @@ PtrDocument         pDoc;
 				   }
 			       }
 			   }
-		     /* on traite le cas des references qui pointent sur de
-			nouveaux elements */
-		     UpdateReferences (pRoot, pDoc, pAttr->AeAttrReference);
 		     if (delAttr)
 			DeleteAttribute (pRoot, pAttr);
 		   }  
@@ -727,20 +552,6 @@ PtrDocument         pDoc;
 	  }
      }
 
-   /* on n'a pas de traitement particulier a faire sur les references si */
-   /* la copie est dans le meme document que l'original */
-   if (DocOfSavedElements != pDoc)
-     {
-	/* l'element traite' est-il une reference ? */
-	if (pRoot->ElTerminal && pRoot->ElLeafType == LtReference)
-	   /* c'est un element reference */
-	   pRef = pRoot->ElReference;
-	else			/* c'est peut-etre une inclusion */
-	   pRef = pRoot->ElSource;
-	if (pRef != NULL)	/* c'est une reference, on */
-	                        /*  met a jour la reference si besoin */
-	   UpdateReferences (pRoot, pDoc, pRef);
-     }
    if (!pRoot->ElTerminal && pRoot->ElSource == NULL)
       /* ce n'est ni une inclusion ni un terminal, on traite tous les fils */
      {
