@@ -51,7 +51,14 @@ extern LPCTSTR iconID;
 
 static ThotWindow   Color_Window = 0;
 static ThotWidget   Color_Palette;
+#ifdef _GTK
+static ThotWidget   Color_Palette_Extended = NULL;
+static ThotBool     ApplyFg = FALSE;
+static ThotBool     ApplyBg = FALSE;
+#endif /* _GTK */
+#ifndef _GTK
 static ThotGC       GCkey;
+#endif /* !_GTK */
 static int          LastBg;
 static int          LastFg;
 static int          FgColor, BgColor;
@@ -63,6 +70,9 @@ static ThotBool     PalMessageSet1 = TRUE;
 static ThotBool     IsRegistered = FALSE;
 #endif
 
+#ifdef _GTK
+#include "gtk-functions.h"
+#endif /* _GTK */
 
 #include "appdialogue_f.h"
 #include "actions_f.h"
@@ -81,7 +91,14 @@ static ThotBool     IsRegistered = FALSE;
   ----------------------------------------------------------------------*/
 static void ThotSelectPalette (int bground, int fground)
 {
-#ifndef _GTK
+#ifdef _GTK
+   BgColor = bground;
+   FgColor = fground;
+
+   /* redraw the palette */
+   gtk_widget_queue_draw (GTK_WIDGET(Color_Palette));
+   return;
+#else /* !_GTK */
 #ifdef _WINDOWS
    BgColor = bground;
    FgColor = fground;
@@ -92,10 +109,10 @@ static void ThotSelectPalette (int bground, int fground)
 
    if (TtWDepth == 1)
       /* Affiche le nom des couleurs sur un ecran N&B */
-      wcase = CharacterWidth ('m', FontDialogue) * 12;
+      wcase = CharacterWidth (109, FontDialogue) * 12;
    else
       /* Affiche les couleurs sur un ecran couleur */
-      wcase = CharacterWidth ('m', FontDialogue) * 4;
+      wcase = CharacterWidth (109, FontDialogue) * 4;
 
    hcase = FontHeight (FontDialogue);
 
@@ -167,7 +184,7 @@ static void ThotSelectPalette (int bground, int fground)
 }
 
 
-
+#ifndef _GTK
 /*----------------------------------------------------------------------
    KillPalette
    kills the palette.
@@ -177,15 +194,186 @@ static void KillPalette (ThotWidget w, int index, caddr_t call_d)
    Color_Palette = 0;
    Color_Window = 0;
 }
+#endif /* !_GTK */
+
+#ifdef _GTK
+/*----------------------------------------------------------------------
+  ColorExposeGTK redraw the color of the button color
+  data is the id color
+  widget is the drawing area
+  ----------------------------------------------------------------------*/
+gboolean ColorsExposeGTK (GtkWidget *widget, GdkEventExpose *ev, gpointer data)
+{
+  int color = (int) data;
+  GdkWindow *window = GTK_WIDGET(widget)->window;
+  gdk_rgb_gc_set_foreground (TtLineGC, ColorPixel (color));
+  gdk_draw_rectangle (window, TtLineGC, TRUE, 0, 0, 20, 20);
+
+  /*  fground = LastFg;
+  bground = LastBg;
+  LastFg = -1;
+  LastBg = -1;*/
+  /*  ThotSelectPalette (bground, fground);*/
+  return TRUE;
+}
+
+/*----------------------------------------------------------------------
+  ColorExposeFgBgGTK redraw the color of the foreground and background color
+  data is the @ id color
+  widget is the drawing area
+  ----------------------------------------------------------------------*/
+gboolean ColorsExposeFgBgGTK (GtkWidget *widget, GdkEventExpose *ev, gpointer data)
+{
+  int color = *((int *)data);
+  GdkWindow *window = GTK_WIDGET(widget)->window;
+  if (color==-1) color=0;
+  gdk_rgb_gc_set_foreground (TtLineGC, ColorPixel (color));
+  gdk_draw_rectangle (window, TtLineGC, TRUE, 0, 0, 40, 40);
+  return TRUE;
+}
+
+/*----------------------------------------------------------------------
+  2 callback to synchronize toggle selection button
+  foreground color
+  background color
+  ----------------------------------------------------------------------*/
+void ButtonFgToggledGTK (GtkWidget *w, GdkEventButton *event, gpointer data)
+{
+  GtkToggleButton * fgbutton;
+  GtkToggleButton * bgbutton;
+  fgbutton = (GtkToggleButton *)gtk_object_get_data (GTK_OBJECT(w), "FgToggle");
+  bgbutton = (GtkToggleButton *)gtk_object_get_data (GTK_OBJECT(w), "BgToggle");
+  
+  ApplyFg = TRUE;
+  ApplyBg = FALSE;
+  gtk_toggle_button_set_active (bgbutton, FALSE);
+  gtk_toggle_button_set_active (fgbutton, TRUE);
+}
+
+void ButtonBgToggledGTK (GtkWidget *w, GdkEventButton *event, gpointer data)
+{
 
 
+  GtkToggleButton * fgbutton;
+  GtkToggleButton * bgbutton;
+  fgbutton = (GtkToggleButton *)gtk_object_get_data (GTK_OBJECT(w), "FgToggle");
+  bgbutton = (GtkToggleButton *)gtk_object_get_data (GTK_OBJECT(w), "BgToggle");
+
+  ApplyBg = TRUE;
+  ApplyFg = FALSE;
+  gtk_toggle_button_set_active (fgbutton, FALSE); 
+  gtk_toggle_button_set_active (bgbutton, TRUE);
+}
+
+/*----------------------------------------------------------------------
+   ColorsPress
+   handles a click on the color palette
+  ----------------------------------------------------------------------*/
+gboolean ColorsPressGTK (GtkWidget *widget, gpointer data)
+{
+  int color = (int)data;
+  
+  if (ApplyFg)
+    {
+      /* on assigne le foreground */
+      FgColor = color;
+      if (applyToSelection)
+	{
+	  ModifyColor (color, FALSE);
+	  ThotSelectPalette (BgColor, color);
+	}
+    }
+
+   if (ApplyBg)
+    {
+      /* on assigne le background */
+      BgColor = color;
+      if (applyToSelection)
+	{
+	  ModifyColor (color, TRUE);
+	  ThotSelectPalette (color, FgColor);
+	}
+      
+    }
+  return TRUE;
+}
+
+/*----------------------------------------------------------------------
+   Apply the current selected fg and bg color to the current selected element
+   FgColor is the foreground id color
+   BgColor is the background id color
+  ----------------------------------------------------------------------*/
+gboolean ApplyColorToSelectedElementGTK (GtkWidget *widget, gpointer data)
+{
+  if (applyToSelection)
+    {
+      ModifyColor (FgColor, FALSE);
+      ModifyColor (BgColor, TRUE);
+    }
+  return TRUE;
+}
+
+/*----------------------------------------------------------------------
+   Apply the selected color to the fg or bg widget
+   FgColor is the foreground id color
+   BgColor is the background id color
+   data is the color_selection reference
+  ----------------------------------------------------------------------*/
+gboolean ApplyExtendedColorGTK (GtkWidget *widget, gpointer data)
+{
+  gdouble couleur[4];
+  unsigned short red;
+  unsigned short green;
+  unsigned short blue;
+  int color_id;
+  gtk_color_selection_get_color (GTK_COLOR_SELECTION (data), couleur);
+  red = couleur[0]*255;
+  green = couleur[1]*255;
+  blue = couleur[2]*255;
+  
+  color_id = TtaGetThotColor (red, green, blue);
+  if (ApplyBg)
+    {
+      ThotSelectPalette (color_id, FgColor);
+    }
+  if (ApplyFg)
+    {
+      ThotSelectPalette (BgColor, color_id);
+    }
+  return TRUE;
+}
+
+
+/*----------------------------------------------------------------------
+   Get the selected element bg and fg colors
+   FgColor is the foreground id color
+   BgColor is the background id color
+  ----------------------------------------------------------------------*/
+gboolean GetSelectedElementColorGTK (GtkWidget *widget, gpointer data)
+{
+  PtrDocument         pSelDoc;
+  PtrElement          pFirstSel, pLastSel;
+  int                 firstChar, lastChar;
+  ThotBool            selectionOK;
+  
+  selectionOK = GetCurrentSelection (&pSelDoc, &pFirstSel, &pLastSel, &firstChar, &lastChar);
+  if (!selectionOK) return TRUE;
+  
+  BgColor = pFirstSel->ElAbstractBox[0]->AbBackground;
+  FgColor = pFirstSel->ElAbstractBox[0]->AbForeground;
+  
+  ThotSelectPalette (BgColor, FgColor);
+  return TRUE;
+}
+#endif /* _GTK */
+
+#ifndef _GTK
 /*----------------------------------------------------------------------
    ColorsExpose
    redisplays a color keyboard.
   ----------------------------------------------------------------------*/
 static void ColorsExpose ()
 {
-#ifndef _GTK
    int                 max, y, w, h;
 #ifndef _WINDOWS 
    char               *ptr;
@@ -197,10 +385,10 @@ static void ColorsExpose ()
 
    if (TtWDepth == 1)
       /* Affiche le nom des couleurs sur un ecran N&B */
-      wcase = CharacterWidth ('m', FontDialogue) * 12;
+      wcase = CharacterWidth (109, FontDialogue) * 12;
    else
       /* Affiche les couleurs sur un ecran couleur */
-      wcase = CharacterWidth ('m', FontDialogue) * 4;
+      wcase = CharacterWidth (109, FontDialogue) * 4;
 
    w = wcase * COLORS_COL;
    hcase = FontHeight (FontDialogue);
@@ -220,14 +408,12 @@ static void ColorsExpose ()
    WChaine (Color_Window, ptr,
 	    (w / 2) - (XTextWidth ((XFontStruct *) SmallFontDialogue, ptr, strlen (ptr)) / 2),
 	    0, FontDialogue, TtLineGC);
-
    /* grille */
    XSetLineAttributes (TtDisplay, TtLineGC, 1, LineSolid, CapButt, JoinMiter);
    for (x = wcase; x < w; x += wcase)
       XDrawLine (TtDisplay, Color_Window, TtLineGC, x, y, x, h);
    for (y = hcase; y < h; y += hcase)
       XDrawLine (TtDisplay, Color_Window, TtLineGC, 0, y, w, y);
-
    /* items */
    if (TtWDepth == 1)
       for (i = 0; i < max; i++)
@@ -246,7 +432,6 @@ static void ColorsExpose ()
 	   XSetForeground (TtDisplay, TtLineGC, ColorPixel (i));
 	   XFillRectangle (TtDisplay, Color_Window, TtLineGC, x + 1, y + 1, wcase - 2, hcase - 2);
 	}
-
    /* show the current selection */
    fground = LastFg;
    bground = LastBg;
@@ -254,10 +439,10 @@ static void ColorsExpose ()
    LastBg = -1;
    ThotSelectPalette (bground, fground);
 #endif /* _WINDOWS */
-#endif /* _GTK */
 }
+#endif /*!_GTK*/
 
-
+#ifndef _GTK
 /*----------------------------------------------------------------------
    ColorsPress
    handles a click on the color palette
@@ -270,10 +455,10 @@ static void ColorsPress (int button, int x, int y)
 
   if (TtWDepth == 1)
     /* Affiche le nom des couleurs sur un ecran N&B */
-    wcase = CharacterWidth ('m', FontDialogue) * 12;
+    wcase = CharacterWidth (109, FontDialogue) * 12;
   else
     /* Affiche les couleurs sur un ecran couleur */
-    wcase = CharacterWidth ('m', FontDialogue) * 4;
+    wcase = CharacterWidth (109, FontDialogue) * 4;
   hcase = FontHeight (FontDialogue);
   /* Regarde si on n'a pas clique dans le titre */
   if (y < hcase)
@@ -345,7 +530,7 @@ static void ColorsPress (int button, int x, int y)
 	}
     }
 }
-
+#endif /* !_GTK */
 
 #ifdef _WINDOWS
 /*----------------------------------------------------------------------
@@ -620,7 +805,11 @@ LRESULT CALLBACK ThotColorPaletteWndProc (HWND hwnd, UINT iMsg,
   ----------------------------------------------------------------------*/
 static void EndPalette (ThotWidget w, int index, caddr_t call_d)
 {
+#ifndef _GTK
   XtPopdown (Color_Palette);
+#else /* !_GTK */
+  gtk_widget_hide (Color_Palette);
+#endif /* _GTK */
 }
 
 /*----------------------------------------------------------------------
@@ -641,10 +830,96 @@ void ColorsEvent (ThotEvent * event)
 }
 #endif /* !_WINDOWS */
 
+#ifdef _GTK
+/*----------------------------------------------------------------------
+   CreateColorSelectionGTK
+   creates the color selection palette (used to select a specifique color)
+  ----------------------------------------------------------------------*/
+gboolean CreateExtendedColorSelectionGTK (GtkWidget *widget, gpointer data)
+{
+  ThotWidget tmpw_vbox;
+  ThotWidget tmpw_colsel;
+  ThotWidget tmpw_frame;
+  ThotWidget tmpw_hbox;
+  ThotWidget tmpw_button;
+
+  if (Color_Palette_Extended)
+    {
+      gtk_widget_show_all (Color_Palette_Extended);
+      return TRUE;
+    }
+
+  /* create the window */
+  Color_Palette_Extended = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_position (GTK_WINDOW (Color_Palette_Extended), GTK_WIN_POS_MOUSE);
+  gtk_window_set_title (GTK_WINDOW (Color_Palette_Extended), TtaGetMessage (LIB, TMSG_COLORS));
+  ConnectSignalGTK (GTK_OBJECT (Color_Palette_Extended),
+		    "delete_event",
+		    GTK_SIGNAL_FUNC(gtk_true),
+		    (gpointer)NULL);
+  
+  /* create the vbox whitch containe all the widget */
+  tmpw_vbox = gtk_vbox_new (FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (tmpw_vbox), 5);
+  gtk_container_add (GTK_CONTAINER (Color_Palette_Extended), tmpw_vbox);
+  /* create the frame for decoration */
+  tmpw_frame = gtk_frame_new (NULL);
+  gtk_box_pack_start (GTK_BOX (tmpw_vbox), tmpw_frame, TRUE, TRUE, 0);
+  
+  /* create the color selection */
+  tmpw_colsel = gtk_color_selection_new ();
+  /* set a high refresh rate (for powerfull computers) */
+  gtk_color_selection_set_update_policy (GTK_COLOR_SELECTION (tmpw_colsel), GTK_UPDATE_CONTINUOUS);
+  /* no opacity */
+  gtk_color_selection_set_opacity (GTK_COLOR_SELECTION (tmpw_colsel), FALSE);
+  /* set the curent edited color */
+  /* gtk_color_selection_set_color (GTK_COLOR_SELECTION (tmpw_colsel), 0);*/
+  /* to get the curent color, use: gtk_colorselection_get_color (colorsel, *color) */
+  /* to connect a callback when the curent color change use the following code :*/
+  /*   ConnectSignalGTK (GTK_OBJECT (tmpw_colsel),
+       "color_changed",
+       GTK_SIGNAL_FUNC(callback_du_changement_de_couleur),
+	(gpointer)tmpw_colsel);*/
+  gtk_container_add (GTK_CONTAINER (tmpw_frame), tmpw_colsel);
+  
+  /* create the hbox for buttons */
+  tmpw_hbox = gtk_hbox_new (FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (tmpw_vbox), tmpw_hbox, FALSE, FALSE, 0);
+  /* add the buttons */
+  tmpw_button = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_DONE));
+  GTK_WIDGET_SET_FLAGS (GTK_WIDGET(tmpw_button), GTK_CAN_DEFAULT);
+  gtk_widget_grab_default (GTK_WIDGET(tmpw_button));
+  gtk_box_pack_start (GTK_BOX (tmpw_hbox), tmpw_button, FALSE, FALSE, 0);
+  ConnectSignalGTK (GTK_OBJECT (tmpw_button),
+		    "clicked",
+		    GTK_SIGNAL_FUNC (CloseExtendedColorSelectionGTK),
+		    (gpointer)NULL);
+  tmpw_button = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_APPLY));
+  GTK_WIDGET_SET_FLAGS (GTK_WIDGET(tmpw_button), GTK_CAN_DEFAULT);
+  gtk_widget_grab_default (GTK_WIDGET(tmpw_button));
+  gtk_box_pack_start (GTK_BOX (tmpw_hbox), tmpw_button, FALSE, FALSE, 0);
+  ConnectSignalGTK (GTK_OBJECT (tmpw_button),
+		    "clicked",
+		    GTK_SIGNAL_FUNC (ApplyExtendedColorGTK),
+		    (gpointer)tmpw_colsel);
+
+  /* show it */
+  gtk_widget_show_all(GTK_WIDGET(Color_Palette_Extended));
+  return TRUE;
+}
+/*----------------------------------------------------------------------
+  Close Extended Color Selector
+  ----------------------------------------------------------------------*/
+gboolean CloseExtendedColorSelectionGTK (GtkWidget *widget, gpointer data)
+{
+  gtk_widget_hide (Color_Palette_Extended);
+  return TRUE;
+}
+#endif /* _GTK */
 
 /*----------------------------------------------------------------------
    ThotCreatePalette
-   creates the color palette.
+   creates the color palette. (user choose colors into a predefined palette)
   ----------------------------------------------------------------------*/
 ThotBool ThotCreatePalette (int x, int y)
 {
@@ -707,7 +982,7 @@ ThotBool ThotCreatePalette (int x, int y)
 
    xfont = XmFontListCreate ((XFontStruct *) FontDialogue, XmSTRING_DEFAULT_CHARSET);
    if (SmallFontDialogue == NULL)
-      SmallFontDialogue = ReadFont ('L', 'H', 0, 9, UnPoint);
+      SmallFontDialogue = ReadFont ('L', 2, 0, 9, UnPoint);
 
    n = 0;
    sprintf (string, "+%d+%d", x, y);
@@ -807,10 +1082,10 @@ ThotBool ThotCreatePalette (int x, int y)
    n = 0;
    if (TtWDepth == 1)
       /* Affiche le nom des couleurs sur un ecran N&B */
-      width = CharacterWidth ('m', FontDialogue) * 12 * COLORS_COL;
+      width = CharacterWidth (109, FontDialogue) * 12 * COLORS_COL;
    else
       /* Affiche les couleurs sur un ecran couleur */
-      width = CharacterWidth ('m', FontDialogue) * 4 * COLORS_COL;
+      width = CharacterWidth (109, FontDialogue) * 4 * COLORS_COL;
    height = ((NumberOfColors () + COLORS_COL - 1) / COLORS_COL + 1) * FontHeight (FontDialogue);
 
 /*** Cree un DrawingArea pour contenir les touches de la palette ***/
@@ -911,9 +1186,224 @@ ThotBool ThotCreatePalette (int x, int y)
    /* pas de selection precedente */
    LastBg = -1;
    LastFg = -1;
+#else /* !_GTK */
+   /* create the color selection in GTK, it's possible to add some elements by adding it into the tmpw_vbox container */
+  GtkWidget *vbox1;
+  GtkWidget *vbox2;
+  GtkWidget *frame3;
+  GtkWidget *frame2;
+  GtkWidget *vbox4;
+  GtkWidget *hbox4;
+  GtkWidget *vbox5 = None;
+  GtkWidget *button4;
+  GtkWidget *hbox1;
+  GtkWidget *button1;
+  GtkWidget *button2;
+  GtkWidget *button3;
+  GtkWidget *color_draw;
+  GtkWidget *color_button;
+  GtkWidget *fixed1;
+  GtkWidget *bg_color_draw;
+  GtkWidget *fg_color_draw;
+  int i, j;
+
+  Color_Palette = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (Color_Palette), TtaGetMessage (LIB, TMSG_COLORS));
+  gtk_window_set_position (GTK_WINDOW (Color_Palette), GTK_WIN_POS_MOUSE);
+  ConnectSignalGTK (GTK_OBJECT (Color_Palette),
+		    "delete_event",
+		    GTK_SIGNAL_FUNC(gtk_true),
+		    (gpointer)NULL);
+
+  vbox1 = gtk_vbox_new (FALSE, 0);
+  gtk_widget_show (vbox1);
+  gtk_container_add (GTK_CONTAINER (Color_Palette), vbox1);
+
+  vbox2 = gtk_vbox_new (FALSE, 2);
+  gtk_widget_show (vbox2);
+  gtk_container_add (GTK_CONTAINER (vbox1), vbox2);
+
+
+  /******** The foreground and background zone *********/
+  frame3 = gtk_frame_new ("");
+  gtk_widget_show (frame3);
+  gtk_box_pack_start (GTK_BOX (vbox2), frame3, TRUE, TRUE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (frame3), 4);
+  fixed1 = gtk_fixed_new ();
+  gtk_widget_show (fixed1);
+  gtk_container_add (GTK_CONTAINER (frame3), fixed1);
+
+  /* background */
+  bg_color_draw = gtk_toggle_button_new();
+  gtk_widget_show (bg_color_draw);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (bg_color_draw), FALSE);
+  ApplyBg = FALSE;
+  gtk_container_set_border_width (GTK_CONTAINER (bg_color_draw), 5);
+  gtk_fixed_put (GTK_FIXED (fixed1), bg_color_draw, 40, 40);
+
+  /* foreground */
+  fg_color_draw = gtk_toggle_button_new();
+  gtk_widget_show (fg_color_draw);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fg_color_draw), TRUE);
+  ApplyFg = TRUE;
+  gtk_container_set_border_width (GTK_CONTAINER (fg_color_draw), 5);
+  gtk_fixed_put (GTK_FIXED (fixed1), fg_color_draw, 10, 10);
+
+  /* drawing area background */
+  color_draw = gtk_drawing_area_new ();
+  gtk_drawing_area_size (GTK_DRAWING_AREA (color_draw), 40, 40);
+  gtk_container_add (GTK_CONTAINER (bg_color_draw), color_draw);
+  /* connecte the data to the drawing area */
+  gtk_object_set_data (GTK_OBJECT(color_draw), "FgToggle", (gpointer)fg_color_draw);
+  gtk_object_set_data (GTK_OBJECT(color_draw), "BgToggle", (gpointer)bg_color_draw);
+  gtk_widget_set_events (GTK_WIDGET (color_draw), GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
+  /* connecte the expose event to redraw the good color */
+  ConnectSignalGTK (GTK_OBJECT (color_draw),
+		    "expose_event",
+		    GTK_SIGNAL_FUNC (ColorsExposeFgBgGTK),
+		    (gpointer)&BgColor);
+  ConnectSignalGTK (GTK_OBJECT (color_draw),
+		    "button_press_event",
+		    GTK_SIGNAL_FUNC (ButtonBgToggledGTK),
+		    (gpointer)NULL);
+
+  /* drawing area foreground */
+  color_draw = gtk_drawing_area_new ();
+  gtk_drawing_area_size (GTK_DRAWING_AREA (color_draw), 40, 40);
+  gtk_container_add (GTK_CONTAINER (fg_color_draw), color_draw);
+  /* connecte the data to the drawing area */
+  gtk_object_set_data (GTK_OBJECT(color_draw), "FgToggle", (gpointer)fg_color_draw);
+  gtk_object_set_data (GTK_OBJECT(color_draw), "BgToggle", (gpointer)bg_color_draw);
+  gtk_widget_set_events (GTK_WIDGET (color_draw), GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
+  /* connecte the expose event to redraw the good color */
+  ConnectSignalGTK (GTK_OBJECT (color_draw),
+		    "expose_event",
+		    GTK_SIGNAL_FUNC (ColorsExposeFgBgGTK),
+		    (gpointer)&FgColor);
+  /* connect synchronize callback */
+  ConnectSignalGTK (GTK_OBJECT (color_draw),
+		    "button_press_event",
+		    GTK_SIGNAL_FUNC (ButtonFgToggledGTK),
+		    (gpointer)NULL);
+
+
+  /******** the color palette *******/
+  frame2 = gtk_frame_new (TtaGetMessage (LIB, TMSG_CPCOLORBUTTON));
+  gtk_widget_show (frame2);
+  gtk_box_pack_start (GTK_BOX (vbox2), frame2, TRUE, TRUE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (frame2), 5);
+
+  /* vbox witch contain all the colors */
+  vbox4 = gtk_vbox_new (FALSE, 5);
+  gtk_widget_show (vbox4);
+  gtk_container_add (GTK_CONTAINER (frame2), vbox4);
+  gtk_container_set_border_width (GTK_CONTAINER (vbox4), 4);
+
+  /* Default color */
+  /* hbox witch contain the default color */
+  hbox4 = gtk_hbox_new (FALSE, 0);
+  gtk_widget_show (hbox4);
+  gtk_box_pack_start (GTK_BOX (vbox4), hbox4, TRUE, TRUE, 0);
+  /* create a color button (default) */
+  color_button = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_STD_COLORS));
+  gtk_widget_show (color_button);
+  /* on connecte le callback pour l'activation de la couleur par default */
+  /* on passe la couleur -1 */
+  ConnectSignalGTK (GTK_OBJECT (color_button),
+		    "clicked",
+		    GTK_SIGNAL_FUNC(ColorsPressGTK),
+		    (gpointer)-1);
+  gtk_box_pack_start (GTK_BOX (hbox4), color_button, FALSE, FALSE, 0);
+
+  /* hbox witch contain all the colors columns */
+  hbox4 = gtk_hbox_new (FALSE, 0);
+  gtk_widget_show (hbox4);
+  gtk_box_pack_start (GTK_BOX (vbox4), hbox4, TRUE, TRUE, 0);
+  j = 0;
+  for (i = 1; i<=NumberOfColors (); i++)
+    {
+      if (i-1 == COLORS_COL*j)
+	{
+	  /* vbox witch contain one color column */
+	  vbox5 = gtk_vbox_new (FALSE, 0);
+	  gtk_widget_show (vbox5);
+	  gtk_box_pack_start (GTK_BOX (hbox4), vbox5, FALSE, FALSE, 0);
+	  j++;
+	}
+      /* create a color button */
+      color_button = gtk_button_new ();
+      gtk_widget_show (color_button);
+      color_draw = gtk_drawing_area_new ();
+      gtk_drawing_area_size (GTK_DRAWING_AREA (color_draw), 20, 20);
+      gtk_container_add (GTK_CONTAINER (color_button), color_draw);
+      gtk_widget_set_events (GTK_WIDGET (color_draw), GDK_EXPOSURE_MASK);
+      /* connecte the expose event to redraw the good color */
+      ConnectSignalGTK (GTK_OBJECT (color_draw),
+			"expose_event",
+			GTK_SIGNAL_FUNC (ColorsExposeGTK),
+			(gpointer)i);
+      /* connecte the clicked action to cahnge the color */
+      ConnectSignalGTK (GTK_OBJECT (color_button),
+			"clicked",
+			GTK_SIGNAL_FUNC(ColorsPressGTK),
+			(gpointer)i);
+      gtk_box_pack_start (GTK_BOX (vbox5), GTK_WIDGET(color_button), FALSE, FALSE, 0);
+    }
+
+
+  /******** All the action buttons *********/
+  button4 = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_CPMOREBUTTON));
+  gtk_widget_show (button4);
+  gtk_box_pack_start (GTK_BOX (vbox4), button4, FALSE, FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (button4), 2);
+  ConnectSignalGTK (GTK_OBJECT (button4),
+		    "clicked",
+		    GTK_SIGNAL_FUNC (CreateExtendedColorSelectionGTK),
+		    (gpointer)NULL);
+
+  hbox1 = gtk_hbox_new (TRUE, 0);
+  gtk_widget_show (hbox1);
+  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 5);
+
+  button1 = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_APPLY));
+  gtk_widget_show (button1);
+  gtk_box_pack_start (GTK_BOX (hbox1), button1, FALSE, FALSE, 0); 
+  gtk_container_set_border_width (GTK_CONTAINER (button1), 2);
+  ConnectSignalGTK (GTK_OBJECT (button1),
+		    "clicked",
+		    GTK_SIGNAL_FUNC (ApplyColorToSelectedElementGTK),
+		    (gpointer)NULL);
+
+  button2 = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_CPGETBUTTON));
+  gtk_widget_show (button2);
+  gtk_box_pack_start (GTK_BOX (hbox1), button2, FALSE, FALSE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (button2), 2);
+  ConnectSignalGTK (GTK_OBJECT (button2),
+		    "clicked",
+		    GTK_SIGNAL_FUNC (GetSelectedElementColorGTK),
+		    (gpointer)NULL);
+
+  button3 = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_DONE));
+  gtk_container_set_border_width (GTK_CONTAINER (button3), 2);
+  gtk_widget_show (button3);
+  gtk_box_pack_start (GTK_BOX (hbox1), button3, FALSE, FALSE, 0);
+  ConnectSignalGTK (GTK_OBJECT (button3),
+		    "clicked",
+		    GTK_SIGNAL_FUNC (EndPalette),
+		    (gpointer)NULL);
+
+  /* show all the dialogues */
+  gtk_widget_set_uposition (Color_Palette, x, y);
+  gtk_widget_show_all(GTK_WIDGET(Color_Palette));
+  Color_Window = GTK_WIDGET (Color_Palette)->window;
+  
+  /* pas de selection precedente */
+  LastBg = -1;
+  LastFg = -1;
 #endif /* _GTK */
 #endif /* _WINDOWS */
-   return TRUE;
+  return TRUE;
 }
 
 
@@ -958,15 +1448,23 @@ void TtcChangeColors (Document document, View view)
      {
 	/* Cree la palette si elle n'existe pas */
 	if (Color_Window == 0)
-	  {	    
+	  {
 	    ConfigKeyboard (&KbX, &KbY);
 #ifndef _WINDOWS
 	    ThotCreatePalette (KbX, KbY);
 #endif /* _WINDOWS */
 	  }
 #ifndef _WINDOWS
+#ifndef _GTK
 	else
 	  XtPopup (Color_Palette, XtGrabNonexclusive);
+#else /* !_GTK */
+	else
+	  {
+	    gtk_widget_show_all (GTK_WIDGET(Color_Palette));
+	    gdk_window_raise (GTK_WIDGET(Color_Palette)->window);
+	  }
+#endif /* _GTK */
 #endif /* _WINDOWS */
 	
 	/* recherche le pave concerne */
