@@ -49,9 +49,8 @@ extern int              errno;
 #include "registry_f.h"
 #include "textcommands_f.h"
 
-
-
 #ifndef _WINDOWS
+#ifndef _GTK
 /*----------------------------------------------------------------------
  * XWindowError is the X-Windows non-fatal errors handler.
  ----------------------------------------------------------------------*/
@@ -79,6 +78,7 @@ static int XWindowFatalError (Display * dpy)
     (*ThotLocalActions[T_backuponfatal]) ();
   return (0);
 }
+#endif /* !_GTK */
 #endif /* _WINDOWS */
 
 
@@ -136,7 +136,14 @@ static void         InitCurs ()
    HCurs = XCreateFontCursor (TtDisplay, XC_sb_h_double_arrow);
    HVCurs = XCreateFontCursor (TtDisplay, /*XC_fleur*/XC_plus);
    WaitCurs = XCreateFontCursor (TtDisplay, XC_watch);
-#endif
+#else /* !_GTK */
+   ArrowCurs = gdk_cursor_new (GDK_LEFT_PTR);
+   WindowCurs = gdk_cursor_new (GDK_HAND2);
+   VCurs = gdk_cursor_new (GDK_SB_V_DOUBLE_ARROW);
+   HCurs = gdk_cursor_new (GDK_SB_H_DOUBLE_ARROW);
+   HVCurs = gdk_cursor_new (GDK_PLUS);
+   WaitCurs = gdk_cursor_new (GDK_CLOCK);
+#endif /* _GTK */
 #endif /* _WINDOWS */
 }
 
@@ -156,7 +163,7 @@ void TtaUpdateEditorColors (void)
   found = FindColor (0, name, "ForegroundColor", "Black", &Black_Color);
   /* selection colors */
   found = FindColor (0, name, "InserPointColor", "Red", &Select_Color);
-  found = FindColor (0, name, "DocSelectColor", "LightGold3", &Select_Color);
+  found = FindColor (0, name, "DocSelectColor", "LightCoral3", &Select_Color);
   /* The reference color */
   found = FindColor (0, name, "ActiveBoxColor", "Red", &(Box_Color));
   /* color for read-only sections */
@@ -181,13 +188,12 @@ static void InitColors (char* name)
 #ifdef _GTK
    GdkVisual          *vptr;
    GdkVisualType       vinfo;
-   int                 test;
 #else /* _GTK */
    XVisualInfo        *vptr;
    XVisualInfo         vinfo;
-#endif /* _GTK */
    ThotColorStruct     col;
    int                 i;
+#endif /* _GTK */
 
 #ifdef _GTK
    vptr = gdk_visual_get_best ();
@@ -198,23 +204,8 @@ static void InitColors (char* name)
      TtIsTrueColor = FALSE;
    /* Depending on the display Black and White order may be inverted */
   // if (XWhitePixel (TtDisplay, TtScreen) == 0)
-  test =  gdk_color_white (TtCmap, &cwhite);
-  test = gdk_color_black (TtCmap, &cblack);
-   if (cwhite.pixel == 0)
-     {
-       /* je ne sais pas ce qu'il faut faire */
-       /*if (!XAllocNamedColor (TtDisplay, TtCmap, "White", &cwhite, &col))
-	   TtaDisplaySimpleMessage (FATAL, LIB, TMSG_NOT_ENOUGH_MEMORY);
-	if (!XAllocNamedColor (TtDisplay, TtCmap, "Black", &cblack, &col))
-	   TtaDisplaySimpleMessage (FATAL, LIB, TMSG_NOT_ENOUGH_MEMORY);
-     }
-   else
-     {
-	if (!XAllocNamedColor (TtDisplay, TtCmap, "Black", &cblack, &col))
-	   TtaDisplaySimpleMessage (FATAL, LIB, TMSG_NOT_ENOUGH_MEMORY);
-	if (!XAllocNamedColor (TtDisplay, TtCmap, "White", &cwhite, &col))
-	   TtaDisplaySimpleMessage (FATAL, LIB, TMSG_NOT_ENOUGH_MEMORY)*/;
-     }
+  gdk_color_white (TtCmap, (GdkColor *)&cwhite);
+  gdk_color_black (TtCmap, (GdkColor *)&cblack);
    /* Initialize colors for the application */
    /*   Black_Color  = cblack.pixel;
 	FgMenu_Color = Select_Color = cblack.pixel;
@@ -291,6 +282,7 @@ static void InitGraphicContexts (void)
   gdk_rgb_gc_set_background (TtWhiteGC, Black_Color);
   gdk_rgb_gc_set_foreground (TtWhiteGC, White_Color);
   gdk_gc_set_function (TtWhiteGC, GDK_COPY); 
+  gdk_gc_set_exposures (TtWhiteGC, TRUE);
 
  
   /* Create a Graphic Context to write black on white. */
@@ -307,7 +299,7 @@ static void InitGraphicContexts (void)
   gdk_rgb_gc_set_foreground (TtLineGC, Black_Color);
   gdk_rgb_gc_set_background (TtLineGC, White_Color);
   gdk_gc_set_function (TtLineGC, GDK_COPY);
-  gdk_gc_set_tile (TtLineGC, pix);
+  gdk_gc_set_tile (TtLineGC, (GdkPixmap *)pix);
 
   /* Another Graphic Context to write black on white, for dialogs. */
   TtDialogueGC = gdk_gc_new (DefaultDrawable);
@@ -337,7 +329,7 @@ static void InitGraphicContexts (void)
   gdk_gc_set_function (TtGreyGC, GDK_COPY);
   gdk_gc_set_fill (TtGreyGC, GDK_TILED);
 
-  gdk_pixmap_unref (pix);
+  gdk_pixmap_unref ((GdkPixmap *)pix);
 #else /* _GTK */
    valuemask = GCForeground | GCBackground | GCFunction;
    white = ColorNumber ("White");
@@ -404,6 +396,46 @@ void ThotInitDisplay (char* name, int dx, int dy)
    if (TtWDepth == 1)
      TtWDepth = GetDeviceCaps (TtDisplay, BITSPIXEL);
 
+   /* test for 15 or 16 bits. Adapted from:
+       Tim Lesher http://mlarchive.ima.com/windev/1999/0273.html */
+   if (TtWDepth == 16)
+   {
+    LPBITMAPINFOHEADER pDib;
+	HDC hdc;
+	HBITMAP hbm;
+    int iDibSize;
+
+	iDibSize = sizeof( BITMAPINFOHEADER ) + 256 * sizeof ( DWORD );
+	pDib = (LPBITMAPINFOHEADER) malloc( iDibSize );
+    memset( pDib, 0, iDibSize );
+        
+    /* Use old-style BMIH for compatibility, and leave biBitCount zero */
+    pDib->biSize = sizeof( BITMAPINFOHEADER );
+        
+    /* Need to call GetDIBits twice: the first one just fills in the */
+    /* biBitCount member; the second fills in the bitfields or palette */
+    hdc = GetDC (NULL);
+    hbm = CreateCompatibleBitmap (hdc, 1, 1);
+    GetDIBits (hdc, hbm, 0, 1, NULL, (LPBITMAPINFO) pDib, DIB_RGB_COLORS);
+    GetDIBits (hdc, hbm, 0, 1, NULL, (LPBITMAPINFO) pDib, DIB_RGB_COLORS);
+    DeleteObject (hbm);
+    if (!ReleaseDC (NULL, hdc))
+      WinErrorBox (NULL, "ReleaseDC: ThotInitDisplay");
+
+	/* printf( "Current video mode is %lu-bit.\n", pDib->biBitCount ); */
+
+    if (BI_BITFIELDS == pDib->biCompression)
+    {
+        DWORD * pdwFields = (DWORD*) (pDib+1);
+       /* 0xf800: == 565 BGR;
+	      0x7c00  == 555 BGR; */
+
+		if (pdwFields[0] == 0x7c00)
+            TtWDepth = 15;
+	}
+    free( pDib );
+   }
+
    InitDocColors (name);
    InitColors (name);   
    if (WIN_LastBitmap)
@@ -426,7 +458,7 @@ void ThotInitDisplay (char* name, int dx, int dy)
    DefaultWindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_realize (DefaultWindow);
    DefaultDrawingarea = gtk_drawing_area_new();
-   gtk_widget_set_parent (DefaultDrawingarea,DefaultWindow); 
+   gtk_widget_set_parent (DefaultDrawingarea, DefaultWindow); 
    gtk_widget_realize (DefaultDrawingarea);
    DefaultDrawable = DefaultDrawingarea->window;
    gdk_window_get_geometry (DefaultDrawable,&x, &y, &width, &height, &depth);
@@ -493,12 +525,13 @@ void InitDocContexts ()
 }
 
 #ifndef _WINDOWS
+
+#ifndef _GTK
 /*----------------------------------------------------------------------
  *      SelectionEvents handle the X-Windows selection events.
  ----------------------------------------------------------------------*/
 void SelectionEvents (void *ev)
 {
-#ifndef _GTK
    XSelectionRequestEvent *request;
    XSelectionEvent     notify;
    ThotWindow          w, wind;
@@ -598,7 +631,7 @@ void SelectionEvents (void *ev)
 		   else
 		     buffer = partbuffer;
 		   /* paste the content of the selection */
-		   if (ThotLocalActions[T_pasteclipboard] != NULL)
+		   if (ThotLocalActions[T_pasteclipboard])
 		     (*ThotLocalActions[T_pasteclipboard]) (buffer, (int) nbitems);
 		   if (buffer != partbuffer)
 		     TtaFreeMemory (buffer);
@@ -653,7 +686,7 @@ void SelectionEvents (void *ev)
 	 }
        break;
      }
-#endif /* _GTK */
 }
+#endif /* _GTK */
 #endif /* _WINDOWS */
 
