@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,7 +12,7 @@
  *          D. Veillard (INRIA) - Removed X remapping of keys,
  *                                lead to crash in some configurations
  *          R. Guetari (W3C/INRIA) - Previous Windows version
- *
+ *          P. Cheyrou-Lagreze (INRIA) - gtk input
  */
 
 #include "thot_gui.h"
@@ -39,19 +39,11 @@ typedef struct _key
   int                 K_Value;	      /* return key if command = -1          */
   ThotBool            K_Special;      /* TRUE if it's a special key          */
   struct _key        *K_Other;	      /* next entry at the same level        */
-  union
-  {
-    struct _key    *_K_Next_;	      /* 1st complementary touch (1st level) */
-    int             _K_Modifier_;     /* modifier value (2nd level)          */
-    void           *_K_Param_;     /* the parameter of the access key     */
-  } u;
   struct _key        *K_Next;	      /* 1st complementary touch (1st level) */
   int                 K_Modifier;     /* modifier value (2nd level)          */
+  void               *K_Param;        /* the parameter of the access key     */
 }
 KEY;
-#define K_Next u._K_Next_
-#define K_Modifier u._K_Modifier_
-#define K_Param u._K_Param_
 
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
@@ -548,67 +540,66 @@ void WIN_CharTranslation (HWND hWnd, int frame, UINT msg, WPARAM wParam,
   ----------------------------------------------------------------------*/
 gboolean CharTranslationGTK (GtkWidget *w, GdkEventKey* event, gpointer data)
 {
-   int                 status;
-   int                 PicMask;
-   int                 frame;
-   unsigned int        state, save;
-   unsigned char       string[2];
-   KeySym              KS;
-   GtkWidget          *drawing_area;
-   GtkEntry           *textzone;
+  int                 status;
+  int                 PicMask;
+  int                 frame;
+  unsigned int        state, save;
+  unsigned char       string[2];
+  KeySym              KS;
+  GtkWidget          *drawing_area;
+  GtkEntry           *textzone;
 
-   frame = (int) data;
-   if (frame > MAX_FRAME)
-     frame = 0;
-   /* the drawing area is the main zone where keypress event must be active */
-   drawing_area = FrameTable[frame].WdFrame;
-   /* Focus is on all the drawing frame : 
-      Drawing area and his hiden text catcher (for multikey),
-      and the URLtext textzone, 
-      so we must now know where is the focus, 
-      to analyse the meaning of the keypress
-      and setting it on one of the  textfields*/
-   if (FrameTable[frame].Text_Zone[1])
-       {
-	   
-	 if (GTK_WIDGET_HAS_FOCUS (FrameTable[frame].Text_Zone[1]))
-	    { 
-		 /* We're in the url zone*/
-		 return FALSE;
-	    } 
-	 else
-	    { 
-		 /* We're in the drawing so get the hidden textfield adress*/		 
-		 textzone = gtk_object_get_data (GTK_OBJECT (drawing_area), "Text_catcher");
-		 gtk_widget_grab_focus (GTK_WIDGET(textzone));
-	     }
-     }
+  frame = (int) data;
+  if (frame > MAX_FRAME)
+    frame = 0;
+  /* the drawing area is the main zone where keypress event must be active */
+  drawing_area = FrameTable[frame].WdFrame;
+  /* Focus is on all the drawing frame : 
+     Drawing area and his hiden text catcher (for multikey),
+     and the URL text textzone, 
+     so we must now know where is the focus, 
+     to analyse the meaning of the keypress
+     and setting it on one of the  textfields*/
+  if (FrameTable[frame].Text_Zone[1])
+    {
+      if (GTK_WIDGET_HAS_FOCUS (FrameTable[frame].Text_Zone[1]))
+	{ 
+	  /* We're in the url zone*/
+	  return FALSE;
+	} 
+      else
+	{ 
+	  /* We're in the drawing so get the hidden textfield adress*/		 
+	  textzone = gtk_object_get_data (GTK_OBJECT (drawing_area), "Text_catcher");
+	  gtk_widget_grab_focus (GTK_WIDGET(textzone));
+	}
+    }
+  status = 0;
+  /* control, alt and mouse status bits of the state are ignored */
+  state = event->state & (GDK_SHIFT_MASK | GDK_LOCK_MASK | GDK_MOD3_MASK);
 
-   status = 0;
-   /* control, alt and mouse status bits of the state are ignored */
-   state = event->state & (GDK_SHIFT_MASK | GDK_LOCK_MASK | GDK_MOD3_MASK);
-
-   strncpy(string, event->string, 2);
-   KS = event->keyval;
-   if (event->state != state)
-     {
-       save = event->state;
-       event->state = state;
-       state = save;
-     }
-   PicMask = 0;
-   if (state & GDK_SHIFT_MASK)
-      PicMask |= THOT_MOD_SHIFT;
-   /*if (state & GDK_LOCK_MASK)
-     PicMask |= THOT_MOD_SHIFT;*/
-   if (state & GDK_CONTROL_MASK)
-       PicMask |= THOT_MOD_CTRL;
-   if (state & GDK_MOD1_MASK || state & GDK_MOD4_MASK)
-      PicMask |= THOT_MOD_ALT;
-   if (event->keyval == GDK_space)
-     event->length = 1;
-   ThotInput (frame, &string[0], event->length, PicMask, KS);
-   return FALSE;
+  strncpy(string, event->string, 2);
+  KS = event->keyval;
+  if (event->state != state)
+    {
+      save = event->state;
+      event->state = state;
+      state = save;
+    }
+  PicMask = 0;
+  if (state & GDK_SHIFT_MASK)
+    PicMask |= THOT_MOD_SHIFT;
+  /*if (state & GDK_LOCK_MASK)
+    PicMask |= THOT_MOD_SHIFT;*/
+  if (state & GDK_CONTROL_MASK)
+    PicMask |= THOT_MOD_CTRL;
+  if (state & GDK_MOD1_MASK || state & GDK_MOD4_MASK)
+    PicMask |= THOT_MOD_ALT;
+  if (event->keyval == GDK_space)
+    event->length = 1;
+  ThotInput (frame, &string[0], event->length, PicMask, KS);
+  gtk_signal_emit_stop_by_name (GTK_OBJECT(w), "key_press_event");
+  return TRUE;
 }
 
 /*----------------------------------------------------------------------
@@ -659,13 +650,15 @@ gboolean KeyScrolledGTK (GtkWidget *w, GdkEvent* event, gpointer data)
   GdkEventButton      *eventmouse;
   Document            doc; 
   int                 view;
-  int                 x,y;
+  int                 x, y;
   int                 height;
   static int          timer = None; 
-  
+  GdkModifierType state;
+  GtkEntry           *textzone;
+
   frame = (int) data; 
   FrameToView (frame, &doc, &view);
-  gdk_window_get_position(w->parent->parent->parent->window, &x, &y);
+  textzone = 0;
   if (timer != None)
     {
       gtk_timeout_remove (timer);
@@ -680,12 +673,7 @@ gboolean KeyScrolledGTK (GtkWidget *w, GdkEvent* event, gpointer data)
 	{
 	  /* 16 is the pixel size of the scrollbar buttons*/
 	  height = w->allocation.height - 16;
-	  /* 
-	     Gives the real Y 
-	     (eventmouse->y GTK gives bad number on the lower button)
-	     So We use the Window manager one and calculate absolute position...
-	  */
-	  y = eventmouse->y_root - y - w->allocation.y;
+	  gdk_window_get_pointer (w->window, &x, &y, &state);
 	  if (eventmouse->state & GDK_CONTROL_MASK)
 	    {
 	      if (y > height)
@@ -709,13 +697,11 @@ gboolean KeyScrolledGTK (GtkWidget *w, GdkEvent* event, gpointer data)
 	}
 		 
     } 
-  /*
-    Code is here, but didn't manage to always catch those events..
+  /* Code is here, but didn't manage to always catch those events..
     But as CRTL + UP and CTRL + DOWN work like UP and DOWN should...
     The problem is that Key event are catched before, by the Main Window 
     In CharTranslation GTK... We need to redesign all Key catching if we 
-    wanna catch UP and DOWN... As we sadly cannot get mouse position in a key event..
-   */
+    wanna catch UP and DOWN... As we sadly cannot get mouse position in a key event... */
   else if (event->type == GDK_KEY_PRESS)
     {
       eventkey = (GdkEventKey*) event;
@@ -736,6 +722,7 @@ gboolean KeyScrolledGTK (GtkWidget *w, GdkEvent* event, gpointer data)
 	  return TRUE;
 	}
     }
+
   return FALSE;
 }
 #else /* _GTK */
@@ -884,7 +871,7 @@ void ThotInput (int frame, unsigned char *string, unsigned int nb,
       
 	  if (found)
 	    {
-	      value = (unsigned char) ptr->K_Value;
+	      value = ptr->K_Value;
 	      command = ptr->K_Command;
 	    }
 	}
@@ -955,7 +942,7 @@ void ThotInput (int frame, unsigned char *string, unsigned int nb,
 			if (Automata_current == NULL)
 			  {
 			    /* one key shortcut */
-			    value = (unsigned char) ptr->K_Value;
+			    value = ptr->K_Value;
 			    command = ptr->K_Command;
 			  }
 		      }
@@ -1010,6 +997,9 @@ void ThotInput (int frame, unsigned char *string, unsigned int nb,
 	  break;
 	default:
 	  index = -1;
+#ifdef _WINDOWS
+	  nb = 0;
+#endif /* _WINDOWS */
 	  break;
 	}
 
@@ -1041,19 +1031,22 @@ void ThotInput (int frame, unsigned char *string, unsigned int nb,
 	  if (LoadedDocument[document - 1] == SelectedDocument &&
 	      command == CMD_CreateElement)
 	    /* check if the application wants to handle the return */
-	    done = APPKey (TteElemReturn, FirstSelectedElement, document, TRUE);
+	    done = APPKey (TteElemReturn, FirstSelectedElement, document,
+			   TRUE);
 	  else if (LoadedDocument[document - 1] == DocSelectedAttr &&
 	      command == CMD_CreateElement)
 	    /* check if the application wants to handle the return */
-	    done = APPKey (TteElemReturn, AbsBoxSelectedAttr->AbElement, document, TRUE);
+	    done = APPKey (TteElemReturn, AbsBoxSelectedAttr->AbElement,
+			   document, TRUE);
 	  else if (LoadedDocument[document - 1] == SelectedDocument &&
 		   value == TAB)
 	    /* check if the application wants to handle the Tab */
 	    done = APPKey (TteElemTab, FirstSelectedElement, document, TRUE);
 	  else if (LoadedDocument[document - 1] == DocSelectedAttr &&
 		   value == TAB)
-	    /* check if the application wants to handle the return */
-	    done = APPKey (TteElemTab, AbsBoxSelectedAttr->AbElement, document, TRUE);
+	    /* check if the application wants to handle the Tab */
+	    done = APPKey (TteElemTab, AbsBoxSelectedAttr->AbElement,
+			   document, TRUE);
 	  else
 	    done = FALSE;
 	  /* Call action if it's active */
@@ -1073,18 +1066,20 @@ void ThotInput (int frame, unsigned char *string, unsigned int nb,
 	      else if (LoadedDocument[document - 1] == DocSelectedAttr &&
 		       command == CMD_CreateElement)
 		/* check if the application wants to handle the return */
-		APPKey (TteElemReturn, AbsBoxSelectedAttr->AbElement, document, FALSE);
+		APPKey (TteElemReturn, AbsBoxSelectedAttr->AbElement,
+			document, FALSE);
 	      else if (LoadedDocument[document - 1] == SelectedDocument &&
 		       value == TAB)
 		/* post treatment for the application */
 		APPKey (TteElemTab, FirstSelectedElement, document, FALSE);
 	      else if (LoadedDocument[document - 1] == DocSelectedAttr &&
 		       value == TAB)
-		/* check if the application wants to handle the return */
-		APPKey (TteElemTab, AbsBoxSelectedAttr->AbElement, document, FALSE);
+		/* check if the application wants to handle the Tab */
+		APPKey (TteElemTab, AbsBoxSelectedAttr->AbElement, document,
+			FALSE);
 	    }
 	}
-     else if (nb == 0)
+     else if (nb == 0 && !found)
        {
 	 /* Rien a inserer */ 
 	 Automata_current = NULL;
@@ -1103,42 +1098,24 @@ void ThotInput (int frame, unsigned char *string, unsigned int nb,
 	    }
       
 	  /*** Sequence de traitement des espaces ***/
-	  if (value == BREAK_LINE || (InputSpace && value == SHOWN_BREAK_LINE))
+	  if (value == BREAK_LINE || value == THIN_SPACE ||
+	      value == FOUR_PER_EM || value == UNBREAKABLE_SPACE)
 	    {
 	      if (MenuActionList[0].Call_Action)
-		(*MenuActionList[0].Call_Action) (document, view, BREAK_LINE);
+		(*MenuActionList[0].Call_Action) (document, view, value);
 	    }
-	  else if (value == THIN_SPACE || (InputSpace && value == SHOWN_THIN_SPACE))
-	    {
-	      if (MenuActionList[0].Call_Action)
-		(*MenuActionList[0].Call_Action) (document, view, THIN_SPACE);
-	    }
-	  else if (value == HALF_EM || (InputSpace && value == SHOWN_HALF_EM))
-	    {
-	      if (MenuActionList[0].Call_Action)
-		(*MenuActionList[0].Call_Action) (document, view, HALF_EM);
-	    }
-	  else if (value == UNBREAKABLE_SPACE || (InputSpace && value == SHOWN_UNBREAKABLE_SPACE))
-	    {
-	      if (MenuActionList[0].Call_Action)
-		(*MenuActionList[0].Call_Action) (document, view, UNBREAKABLE_SPACE);
-	    }
-	  else if ((InputSpace && value == SHOWN_SPACE))
-	    {
-	      if (MenuActionList[0].Call_Action)
-		(*MenuActionList[0].Call_Action) (document, view, SPACE);
-	    }
-	  else if (value == 9 ||
-		   ( value >= 32 && value < 128) || (value >= 144 && value < 256))
+	  else if (value == 9 || value >= 32)
 	    {
 	      if (LoadedDocument[document - 1] == SelectedDocument &&
 		  value == TAB)
 		/* check if the application wants to handle the Tab */
-		done = APPKey (TteElemTab, FirstSelectedElement, document, TRUE);
+		done = APPKey (TteElemTab, FirstSelectedElement, document,
+			       TRUE);
 	      else if (LoadedDocument[document - 1] == DocSelectedAttr &&
 		       value == TAB)
-		/* check if the application wants to handle the return */
-		done = APPKey (TteElemTab, AbsBoxSelectedAttr->AbElement, document, TRUE);
+		/* check if the application wants to handle the TAB */
+		done = APPKey (TteElemTab, AbsBoxSelectedAttr->AbElement,
+			       document, TRUE);
 	      else
 		done = FALSE;
 	      /* on insere un caractere valide quelque soit la langue */
@@ -1150,8 +1127,9 @@ void ThotInput (int frame, unsigned char *string, unsigned int nb,
 		APPKey (TteElemTab, FirstSelectedElement, document, FALSE);
 	      else if (LoadedDocument[document - 1] == DocSelectedAttr &&
 		       value == TAB)
-		/* check if the application wants to handle the return */
-		APPKey (TteElemTab, AbsBoxSelectedAttr->AbElement, document, FALSE);
+		/* check if the application wants to handle the TAB */
+		APPKey (TteElemTab, AbsBoxSelectedAttr->AbElement, document,
+			FALSE);
 	    }
 	}
     }
@@ -1328,22 +1306,22 @@ void      TtaRemoveAccessKey (Document doc, unsigned int key)
    InitTranslations
    intializes the keybord encoding.
   ----------------------------------------------------------------------*/
-ThotTranslations      InitTranslations (char *appliname)
+ThotTranslations InitTranslations (char *appliname)
 {
-  char*             appHome;	   /* fichier de translation */
-  char              fullName[200];  /* ligne en construction pour motif */
-  char              home[200]; 
-  char              name[80]; 
-  char*               text;	   
-  char*               adr;
-  char                transText [MAX_LENGTH];
+  char               *appHome;	   /* fichier de translation */
+  char                fullName[200];  /* ligne en construction pour motif */
+  char                home[200]; 
+  char                name[80]; 
+  char               *text;	   
+  char               *adr;
+  char                transText[MAX_LENGTH];
   char                ch[80]; 
   char                line[200];  /* ligne en construction pour motif */
   char                equiv[MAX_EQUIV]; /* equivalents caracteres pour motif */
   unsigned int        key1, key2; /* 1ere & 2eme cles sous forme de keysym X */
-  int                 e, i;
+  int                 e, i, j;
   int                 mod1, mod2; /* 1er/ 2eme modifieurs : voir THOT_MOD_xx */
-  int                 len, max;
+  int                 len, max, value;
   FILE               *file;
   ThotTranslations    table = 0;
   ThotBool            isSpecialKey1, isSpecialKey2;
@@ -1574,10 +1552,9 @@ ThotTranslations      InitTranslations (char *appliname)
 		      break;
 		} 
 
-	      /* Est-ce une translation valable pour le texte Motif */
 	      if (i <= 8)
 		{
-		  /* FnCopy la ligne dans le source de la table de translations */
+		  /* Prepare translations for Motif */
 		  strcat (text, line);
 		  if (!strcmp (ch, "TtcInsertChar"))
 		    {
@@ -1605,15 +1582,29 @@ ThotTranslations      InitTranslations (char *appliname)
 
 	      if (i == 0)
 		{
-		  /* C'est l'action insert-string */
-		  /* FnCopy la ligne dans le source de la table de translations */
-		  strcat (text, line);
-		  strcat (text, AsciiTranslate (transText));
-		  strcat (text, "\n");
-		  /* C'est un encodage de caractere */
-		  adr = AsciiTranslate (&transText[len]);
+		  /* action insert-string */
+		  if (transText[len] == '&' && transText[len + 1] == '#')
+		    {
+		      /* it's an entity */
+		      j = 2;
+		      while (transText[len + j] != EOS &&
+			     transText[len + j] != ';' &&
+			     transText[len + j] != ')' &&
+			     transText[len + j] != '"')
+			j++;
+		      transText[len + j] = EOS;
+		      if (transText[len + 2] == 'x')
+			sscanf (&transText[len + 3], "%x", &value);
+		      else
+			sscanf (&transText[len + 2], "%d", &value);
+		    }
+		  else
+		    {
+		      adr = AsciiTranslate (&transText[len]);
+		      value = (unsigned char) adr[0];
+		    }
 		  MemoKey (mod1, key1, isSpecialKey1,
-			   mod2, key2, isSpecialKey2, (unsigned int) adr[0], 0);
+			   mod2, key2, isSpecialKey2, value, 0);
 		}
 	      else if (i < max)
 		{
