@@ -19,6 +19,7 @@
 
 #include "thot_gui.h"
 #include "thot_sys.h"
+
 #include "constmenu.h"
 #include "constmedia.h"
 /* Needed for a glitch with Motif 2.0 see ResizeEvents below ! */
@@ -39,6 +40,7 @@
 #include "wininclude.h"
 #else /* _WINDOWS */
 #include <X11/Intrinsic.h>
+#include <locale.h>
 #endif /* _WINDOWS */
 
 #define MAX_ARGS 20
@@ -74,9 +76,11 @@ static int           mk_state = 0;
 static int           TtaKeyboardMapInstalled = 0;
 
 #ifdef _WINDOWS
-static char          previous_keysym;
+static unsigned char previous_keysym;
 #else  /* !_WINDOWS */
+#ifndef _GTK
 static KeySym        previous_keysym;
+#endif /* !_GTK */
 static KeySym        previous_value = 0;
 static unsigned int  previous_state = 0;
 #endif /* !_WINDOWS */
@@ -91,6 +95,7 @@ static unsigned int  previous_state = 0;
   ----------------------------------------------------------------------*/
 /* #define DEBUG_KEYMAP *//* give debug information when installing keymap */
 /* #define DEBUG_MULTIKEY *//* give debug information when using multikey */
+#ifndef _GTK
 static KeySym       TtaIsoKeySymTab[256] =
 {
    XK_nobreakspace,		/* First keysyms are mapped directly  */
@@ -114,6 +119,7 @@ XK_ocircumflex, XK_otilde, XK_odiaeresis, XK_division, XK_oslash, XK_ugrave,
 XK_uacute, XK_ucircumflex, XK_udiaeresis, XK_yacute, XK_thorn, XK_ydiaeresis,
    NoSymbol			/* Needed, do not remove ! */
 };
+#endif
 #endif /* !_WINDOWS */
 /*
  * definition of a multi-key sequence, is made of three KeySyms :
@@ -125,8 +131,8 @@ XK_uacute, XK_ucircumflex, XK_udiaeresis, XK_yacute, XK_thorn, XK_ydiaeresis,
 typedef struct multi_key
   {
 #ifdef _WINDOWS
-     char                c;
-     char                m;
+     unsigned char       c;
+     unsigned char       m;
      int                 r;
 #else  /* !_WINDOWS */
      KeySym              c;
@@ -137,6 +143,7 @@ typedef struct multi_key
 Multi_Key;
 
 #ifndef _WINDOWS
+#ifndef _GTK
 /*
  * tab containing the xtended multi-key sequences.
  * It's a one dimentionnal array of multi-key sequences
@@ -158,7 +165,7 @@ static Multi_Key    emk_tab[] =
 /*
  * The ISO 8859 standard includes several 8-bit extensions to
  * the  ASCII  character  set  (also  known  as ISO 646-IRV).
- * Especially important is ISO 8859-1,  the  "Latin  Alphabet
+ * Especially important is ISO 8859-1,  the  "Latin  Script
  * No.  1",  which  has  become  widely  implemented  and may
  * already  be  seen   as   the   de-facto   standard   ASCII
  * replacement.
@@ -270,7 +277,7 @@ static Multi_Key    emk_tab[] =
    {XK_apostrophe, XK_apostrophe, XK_acute},	/* acute */
    {XK_comma, XK_comma, XK_cedilla},	/* cedilla */
    {XK_quotedbl, XK_quotedbl, XK_diaeresis},	/* diaeresis */
-/* Accented Alphabet */
+/* Accented Script */
    {XK_A, XK_grave, XK_Agrave},	/* Agrave */
    {XK_A, XK_acute, XK_Agrave},	/* Aacute */
    {XK_A, XK_apostrophe, XK_Aacute},	/* Aacute */
@@ -365,8 +372,10 @@ static Multi_Key    emk_tab[] =
 };
 
 #define ExtNB_MK (int)((sizeof(emk_tab) / sizeof(Multi_Key)))
+#endif /* !_GTK */
 #endif /* !_WINDOWS */
 
+#ifndef _GTK
 /*
  * tab containing the multi-key sequences.
  * It's a one dimentionnal array of multi-key sequences
@@ -506,15 +515,18 @@ static Multi_Key    mk_tab[] =
 };
 
 #define NB_MK (int)((sizeof(mk_tab) / sizeof(Multi_Key)))
+#endif /* !_GTK */
 
 #ifndef _WINDOWS
+#ifndef _GTK
 static Display     *TtaDisplay = NULL;
 static int          TtaNbIsoKeySym = 0;
-static int          TtaNbKeySymPerKeyCode = 0;
 static int          TtaModifierNumber = 0;
+static KeyCode      TtaMode_switchKeyCode = NoSymbol;
+#endif
+static int          TtaNbKeySymPerKeyCode = 0;
 static int          TtaMinKeyCode = 0;
 static int          TtaMaxKeyCode = 0;
-static KeyCode      TtaMode_switchKeyCode = NoSymbol;
 static KeySym       TtaKeyboardMap[8 * 256];
 
 /*
@@ -658,6 +670,15 @@ void TtaInstallMultiKey ()
     Enable_Multikey = FALSE;
    TtaKeyboardMapInstalled = 1;
 #else  /* _WINDOWS */
+#ifdef _GTK
+   /* GTK Multikey */
+  ptr = TtaGetEnvString ("ENABLE_MULTIKEY");
+  if (ptr != NULL && !strcasecmp (ptr, "yes"))
+    Enable_Multikey = TRUE;
+  else
+      Enable_Multikey = FALSE;
+  TtaKeyboardMapInstalled = 1;
+#else /* !_GTK */
   KeySym             *keymap;
   Display            *dpy = TtaGetCurrentDisplay ();
   KeyCode             keycode;
@@ -676,7 +697,7 @@ void TtaInstallMultiKey ()
     Enable_Multikey = TRUE;
   else
     Enable_Multikey = FALSE;
-
+  setlocale (LC_ALL, "");
   /* load the current keyboard mapping */
   XDisplayKeycodes (dpy, &TtaMinKeyCode, &TtaMaxKeyCode);
   keymap = XGetKeyboardMapping (dpy, TtaMinKeyCode,
@@ -797,6 +818,7 @@ void TtaInstallMultiKey ()
 	TtaKeyboardMap[keycode * TtaNbKeySymPerKeyCode + TtaModifierNumber] = keysym;
      }
    TtaKeyboardMapInstalled = 1;
+#endif /* _GTK */ 
 #endif /* _WINDOWS */
 }
 
@@ -841,7 +863,7 @@ int WIN_TtaHandleMultiKeyEvent (UINT msg, WPARAM wParam, LPARAM lParam, int* k)
   return 1;
 }
 #else /* _WINDOWS */
-
+#ifndef _GTK
 /*----------------------------------------------------------------------
    TtaGetIsoKeysym
 
@@ -1149,6 +1171,7 @@ fprintf (stderr, "      Multikey : <Alt>%c %c\n", previous_keysym, KS);
        return (1);
      }
 }
+#endif /* !_GTK */
 #endif /* !_WINDOWS */
 
 /*----------------------------------------------------------------------
@@ -1164,33 +1187,19 @@ void TtaFetchOneEvent (ThotEvent *ev)
 #else /* _GTK */
   /* do nothing in gtk because we do not need to know event 
      on the queue */
-
-
-  /*
-  while (!gtk_events_pending());
-  ev = gtk_get_current_event ();
-  */
-
-#if 0
-  /* free the precedent event */
-  if (ev) gdk_event_free (ev);
-  /* wait until one event is coming */
-  while ( !gdk_events_pending () );
-  /* get the waiting event */
-  ev = gdk_event_get ();
-#endif
-
 #endif /* !_GTK */
 #endif /* ! _WINDOWS */
 }
 
 #ifndef _WINDOWS
+#ifndef _GTK
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
 static void *TimerCallback (XtPointer cdata, XtIntervalId *id)
 {
   return (0);
 }
+#endif /* !_GTK */
 #endif /* ! _WINDOWS */
 
 /*----------------------------------------------------------------------
@@ -1201,8 +1210,7 @@ static void *TimerCallback (XtPointer cdata, XtIntervalId *id)
   ----------------------------------------------------------------------*/
 void TtaFetchOrWaitEvent (ThotEvent *ev)
 {
-#ifndef _GTK
-#ifndef _WINDOWS
+#if !defined(_GTK) && !defined(_WINDOWS)
    XtInputMask         status;
 
    status = XtAppPending (app_cont);
@@ -1214,10 +1222,7 @@ void TtaFetchOrWaitEvent (ThotEvent *ev)
        XtAppAddTimeOut (app_cont, 1000, (XtTimerCallbackProc) TimerCallback, NULL);
        XtAppNextEvent (app_cont, ev);
      }
-#endif /* ! _WINDOWS */
-#else /* _GTK */
-
-#endif /* !_GTK */
+#endif /* _GTK _WINDOWS*/
 }
 
 /*----------------------------------------------------------------------
@@ -1240,7 +1245,7 @@ ThotBool TtaFetchOneAvailableEvent (ThotEvent *ev)
      }
    else if (status != 0)
      XtAppProcessEvent (app_cont, XtIMAll);
-#endif /* ! _WINDOWS */
+#endif /* _WINDOWS */
 #else /* _GTK */
    if (gtk_events_pending())
      {
@@ -1249,7 +1254,7 @@ ThotBool TtaFetchOneAvailableEvent (ThotEvent *ev)
      }
    else
      return FALSE;
-#endif /* !_GTK */
+#endif /* _GTK */
    return (FALSE);
 }
 
@@ -1289,6 +1294,7 @@ void TtaHandleOneEvent (ThotEvent *ev)
 	  s = XGetAtomName (ev->xany.display, ((XClientMessageEvent *) ev)->data.l[0]);
 	  if (!strcmp (s, "WM_DELETE_WINDOW"))
 	    {
+	      XFree (s);
 	      if (FrRef[0] != 0 &&
 		  XtWindowOfObject (XtParent (FrameTable[0].WdFrame)) == w)
 		TtcQuit (0, 0);
@@ -1311,8 +1317,9 @@ void TtaHandleOneEvent (ThotEvent *ev)
 		      return;
 		  TtaQuit();
 		}
-	      XFree (s);
 	    }
+	  else
+	    XFree (s);
 	}
       else if (!strcmp (s, "THOT_MESSAGES"))
 	{
@@ -1355,7 +1362,7 @@ void TtaHandleOneEvent (ThotEvent *ev)
 #endif /* !_WINDOWS */
 #else /* _GTK */
   gtk_main_iteration_do (TRUE);
-			 /*if (ev) gtk_main_do_event (ev);*/
+  /*if (ev) gtk_main_do_event (ev);*/
   /* a main loop iteration , not blocking */
   /*  gtk_main_iteration_do (FALSE);*/
 #endif /* !_GTK */
@@ -1368,18 +1375,29 @@ void TtaHandleOneEvent (ThotEvent *ev)
 void TtaHandlePendingEvents ()
 {
 #ifndef _WINDOWS
+  static ThotBool         crit_section; /* protect against multiple imbrications of
+					   calls to this function */
 #ifndef _GTK
-   ThotEvent              ev;
+   ThotEvent              ev;  
+#endif /* !_GTK */
 
+  if (crit_section)
+    return;
+  /*crit_section = TRUE;*/
+#ifndef _GTK
    while (TtaFetchOneAvailableEvent(&ev))
      TtaHandleOneEvent (&ev);
 #else /* _GTK */
-     while (gtk_events_pending ())
-       gtk_main_iteration ();
+   while (gtk_events_pending ()) 
+     gtk_main_iteration ();
 #endif /* !_GTK */
+   crit_section = FALSE;
 #endif /* _WINDOWS */
 }
 
+#ifdef _GL
+#include "glwindowdisplay.h"
+#endif /*_GL*/
 /*----------------------------------------------------------------------
    TtaMainLoop
    Starts the main loop for processing all events in an application. This
@@ -1388,15 +1406,9 @@ void TtaHandlePendingEvents ()
 void TtaMainLoop ()
 {
   NotifyEvent         notifyEvt;
-#ifndef _GTK
   ThotEvent           ev;
-#else /* _GTK */
-  ThotEvent           ev=NULL;
-#endif /* !_GTK */
-#ifndef _GTK
-  TtaInstallMultiKey ();
-#endif /* _GTK */
 
+  TtaInstallMultiKey ();
   UserErrorCode = 0;
   /* Sends the message Init.Pre */
   notifyEvt.event = TteInit;
@@ -1411,7 +1423,14 @@ void TtaMainLoop ()
   /* Sends the message Init.Post */
   notifyEvt.event = TteInit;
   CallEventType (&notifyEvt, FALSE);
-  
+#ifdef _GTK
+#ifdef _GL
+  /* First Time drawing (if we don't have focus)  */
+  while (gtk_events_pending ()) 
+    gtk_main_iteration ();    
+  GL_DrawAll (NULL, -1);
+#endif /*_GL*/
+#endif /*_GTK*/
   /* Loop wainting for the events */
   while (1)
     {
@@ -1421,15 +1440,14 @@ void TtaMainLoop ()
       TtaFetchOneEvent (&ev);
 #endif /* _WINDOWS */
       TtaHandleOneEvent (&ev);
+#ifdef _GL
+      /* buffer swapping, when needed*/
+      GL_DrawAll (NULL, -1);
+#endif/*  _GL */
     }
-
-#ifndef _GTK
-#else /* _GTK */
-     while (gtk_events_pending ())
-       gtk_main_iteration ();
-  /* gdk_set_show_events(TRUE);*/
-  gtk_main ();
-#endif /* !_GTK */
+#ifdef _GTK
+  gtk_exit (0);
+#endif /* _GTK */
 }
 
 /*----------------------------------------------------------------------
@@ -1564,16 +1582,14 @@ void TtaSetMultikey (ThotBool value)
   previous_value = 0;
 #endif /* _WINDOWS */
 }
+/*----------------------------------------------------------------------
+   TtaSetMultiKey
+   Return the state of the multikey support
+   Parameters:
+   value :
+  ----------------------------------------------------------------------*/
+ThotBool TtaGetMultikey()
+{
+    return Enable_Multikey;
+}
 /* End Of Module */
-
-
-
-
-
-
-
-
-
-
-
-
