@@ -1,24 +1,15 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2001
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
- 
 /*
  * Latin, Greek, Math keyboards
  *
  * Authors: I. Vatton (INRIA)
- *          R. Guetari (W3C/INRIA) - Unicode and Windows version
+ *          R. Guetari (W3C/INRIA) - Windows version
  *
  */
 
@@ -30,6 +21,9 @@
 #include "appdialogue.h"
 #include "dialog.h"
 #include "message.h"
+#ifdef _WINDOWS
+#include "wininclude.h"
+#endif /* _WINDOWS */
 
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
@@ -39,24 +33,30 @@
 #include "select_tv.h"
 #include "appdialogue_tv.h"
 
+#ifdef _GTK
+#include "gtk-functions.h"
+#endif /* _GTK */
+
 #define MAX_ARGS 20
 #define MAX_KEYBOARD 4
 
 
 typedef struct _item
   {
-     char          name;
+     char          index;
      unsigned char value;
-     char*         legend;
+     char         *legend;
   }
 ITEM;
 
 
 static ThotWidget   Keyboards[MAX_KEYBOARD];
-static ptrfont      KbFonts[MAX_KEYBOARD];
+static PtrFont      KbFonts[MAX_KEYBOARD];
 static int          KbX = 800;
 static int          KbY = 100;
+#ifndef _GTK
 static ThotGC       GCkey;
+#endif /* !_GTK */
 
 /* data for the keyboards */
 /*iso */
@@ -321,29 +321,6 @@ static ITEM         Items_Isol[] =
 };
 
 
-static ITEM         Items_Symb[] =
-{
-   {83, 'S', NULL},		/* sum */
-   {80, 'P', NULL},		/* product */
-   {85, 'U', NULL},		/* union */
-   {73, 'I', NULL},		/* intersection */
-   {105, 'i', NULL},		/* integral */
-   {99, 'c', NULL},		/* circle integral */
-   {100, 'd', NULL},		/* double integral */
-   {114, 'r', NULL},		/* root */
-   {60, '<', NULL},		/* < arrow */
-   {62, '>', NULL},		/* > arrow */
-   {118, 'V', NULL},		/* V arrow */
-   {94, '^', NULL},		/* ^ arrow */
-   {91, '[', NULL},		/* */
-   {93, ']', NULL},		/* */
-   {40, '(', NULL},		/* */
-   {41, ')', NULL},		/* */
-   {123, '{', NULL},		/* */
-   {125, '}', NULL},		/* */
-};
-
-
 static ITEM         Items_Graph[] =
 {
    {'C', 'C', NULL},		/* oval circle */
@@ -392,38 +369,19 @@ static ITEM         Items_Graph[] =
 #include "font_f.h"
 #include "structselect_f.h"
 #include "textcommands_f.h"
-#include "windowdisplay_f.h"
+#include "xwindowdisplay_f.h"
 
-#ifdef _WINDOWS
-#ifdef __STDC__
-extern void CreateGreekKeyboardDlgWindow (HWND);
-#else /* __STDC__ */
-extern void CreateGreekKeyboardDlgWindow ();
-#endif /* __STDC__ */
-#endif /* _WINDOWS */
 
+#ifndef _GTK
 /*----------------------------------------------------------------------
    WChar
    displays character ch at position <x,y> of window w using the character
    policy font.
    Function func indicates if it's an active box (1) or not (0).
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         WChar (ThotWindow w, CHAR_T ch, int x, int y, int func, ptrfont font, int disp, ThotGC GClocal)
-#else  /* __STDC__ */
-static void         WChar (w, ch, x, y, func, font, disp, GClocal)
-ThotWindow          w;
-CHAR_T                ch;
-int                 x;
-int                 y;
-int                 func;
-ptrfont             font;
-int                 disp;
-ThotGC              GClocal;
-
-#endif /* __STDC__ */
+static void WChar (ThotWindow w, char ch, int x, int y, int func,
+		   PtrFont font, int disp, ThotGC GClocal)
 {
-#ifndef _GTK
    int                 length;
 
    length = 1;
@@ -434,58 +392,45 @@ ThotGC              GClocal;
    FontOrig (font, ch, &x, &y);
    XDrawString (TtDisplay, w, GClocal, x, y, &ch, length);
 #endif
-#endif /* _GTK */
 }
+#endif /* _GTK */
 
 /*----------------------------------------------------------------------
    KbdEndDisplay
    Ends the display of a keyboard.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         KbdEndDisplay (ThotWidget w, int index, caddr_t call_d)
-
-#else  /* __STDC__ */
-static void         KbdEndDisplay (w, index, call_d)
-ThotWidget          w;
-int                 index;
-caddr_t             call_d;
-
-#endif /* __STDC__ */
+static void KbdEndDisplay (ThotWidget w, int index, caddr_t call_d)
 {
 #ifndef _GTK
 #ifndef _WINDOWS
    XtPopdown (Keyboards[index]);
 #endif /* _WINDOWS */
-#endif /* _GTK */
+#else /* _GTK */
+   gtk_widget_hide (GTK_WIDGET(Keyboards[index]));
+#endif /* !_GTK */
 }
 
 /*----------------------------------------------------------------------
-   KbdCallbackHandler
-   handles the keyboard keys.
+  KbdCallbackHandler handles the keyboard keys.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void         KbdCallbackHandler (ThotWidget w, int param, caddr_t call_d)
-#else  /* __STDC__ */
-void         KbdCallbackHandler (w, param, call_d)
-ThotWidget          w;
-int                 param;
-caddr_t             call_d;
-
-#endif /* __STDC__ */
+void KbdCallbackHandler (ThotWidget w, int param, caddr_t call_d)
 {
-#ifndef _GTK
-   UCHAR_T       car;
+   CHAR_T              car;
    ThotWidget          wp;
    int                 i;
 
    /* Recupere la table des items */
-   car = (UCHAR_T) param % 256;
+   car = (CHAR_T) param % 256;
    /* Recupere le widget de la palette */
-#  ifndef _WINDOWS
+#ifndef _GTK
+#ifndef _WINDOWS
    wp=XtParent(XtParent(XtParent(XtParent(w))));
-#  else  /* _WINDOWS */
+#else  /* _WINDOWS */
    wp = GetParent (GetParent (GetParent (GetParent (w))));
-#  endif /* _WINDOWS */
+#endif /* _WINDOWS */
+#else /* _GTK */
+   wp = GTK_WIDGET (w)->parent->parent->parent->parent;
+#endif /* !_GTK */
    /* met a jour l'indicateur de palette */
    if(Keyboards[KeyboardMode] != wp)
      {
@@ -494,85 +439,61 @@ caddr_t             call_d;
        if(Keyboards[i] == wp)
          KeyboardMode = i;
      }
-   /* Insere le caractere selectionne */
+   /* Insert the selected character */
+#ifdef _I18N_
+   if (KeyboardMode == 3)
+      /* give the unicode value instead of the symbol index */
+      car = TtaGetWCFromChar ((unsigned char) car, ISO_SYMBOL);
+#endif /* _I18N_ */
    if (ThotLocalActions[T_insertchar] != NULL)
       (*ThotLocalActions[T_insertchar]) (ActiveFrame, car, KeyboardMode);
-#endif /* _GTK */
 }
 
-
 #ifndef _WINDOWS
-/*----------------------------------------------------------------------
-   ExposeKbd
-   displays the keyboard keys
-  ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         ExposeKbd (ThotWidget w, int param, XmDrawnButtonCallbackStruct * infos)
-#else  /* __STDC__ */
-static void         ExposeKbd (w, param, infos)
-ThotWidget          w;
-int                 param;
-XmDrawnButtonCallbackStruct *infos;
-
-#endif /* __STDC__ */
-{
 #ifndef _GTK
-   int                 y;
-   int                 i, kb;
-   ITEM               *it;
+/*----------------------------------------------------------------------
+  ExposeKbd displays the keyboard keys
+  ----------------------------------------------------------------------*/
+static void ExposeKbd (ThotWidget w, int param,
+		       XmDrawnButtonCallbackStruct *infos)
+{
+  int                 y;
+  int                 i, kb;
+  ITEM               *it;
 
-   /* Recupere la table des items */
-   kb = param / 256;
-   switch (kb)
-	 {
-	    case 0:
-	       it = Items_Symb;
-	       break;
-	    case 1:
-	       it = Items_Graph;
-	       break;
-	    case 2:
-	       it = Items_Isol;
-	       break;
-	    case 4:
-	       it = Items_Grec;
-	       break;
-	    default:
-	       return;
-	 }
-   y = 4;
-   i = param % 256;		/* indice dans la table des items */
-   it = (ITEM *) ((int) it + (sizeof (ITEM) * i));
-   WChar (infos->window, it->name, CharacterWidth (87, FontDialogue), y, GXcopy, KbFonts[kb], 0, GCkey);
-   if (it->legend)
-     {
-	y = FontHeight (KbFonts[kb]);
-	WChaine (infos->window, it->legend, 4, y, FontDialogue, GCkey);
-     }
-#endif /* _GTK */
+  /* Recupere la table des items */
+  kb = param / 256;
+  switch (kb)
+    {
+    case 1:
+      it = Items_Graph;
+      break;
+    case 2:
+      it = Items_Isol;
+      break;
+    case 4:
+      it = Items_Grec;
+      break;
+    default:
+      return;
+    }
+  y = 4;
+  i = param % 256;		/* indice dans la table des items */
+  it = (ITEM *) ((int) it + (sizeof (ITEM) * i));
+  WChar (infos->window, it->index, CharacterWidth (87, FontDialogue), y, GXcopy, KbFonts[kb], 0, GCkey);
+  if (it->legend)
+    {
+      y = FontHeight (KbFonts[kb]);
+      WChaine (infos->window, it->legend, 4, y, FontDialogue, GCkey);
+    }
 }
 
-
 /*----------------------------------------------------------------------
-   CreateKeyboard
-   creates a keyboard.
+   CreateKeyboard creates a keyboard.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         CreateKeyboard (int number, STRING title, ptrfont pFont, int col, int x, int y, ITEM * items, int nbitem)
-#else  /* __STDC__ */
-static void         CreateKeyboard (number, title, font, col, x, y, items, nbitem)
-int                 number;
-STRING              title;
-ptrfont             pFont;
-int                 col;
-int                 x;
-int                 y;
-ITEM               *items;
-int                 nbitem;
-
-#endif /* __STDC__ */
+static void CreateKeyboard (int number, char *title, PtrFont pFont,
+			    int col, int x, int y, ITEM *items, int nbitem)
 {
-#ifndef _GTK
    int                 n;
    int                 i;
    register ITEM      *it;
@@ -581,7 +502,7 @@ int                 nbitem;
    Arg                 args[MAX_ARGS];
    XmString            title_string;
    XmFontList          xfont;
-   CHAR_T                string[10];
+   char                string[10];
    int                 param;
    XGCValues           GCmodel;
 
@@ -595,12 +516,8 @@ int                 nbitem;
    n++;
    XtSetArg (args[n], XmNuseAsyncGeometry, TRUE);
    n++;
-#ifdef OLD
-   w = XtCreateWidget (title, topLevelShellWidgetClass, XtParent (FrameTable[0].WdFrame), args, n);
-#else
    w = XtCreatePopupShell (title, applicationShellWidgetClass, RootShell, args, n);
-#endif
-/*** Cree le clavier dans sa frame ***/
+   /*** Cree le clavier dans sa frame ***/
    n = 0;
    XtSetArg (args[n], XmNbackground, BgMenu_Color);
    n++;
@@ -614,7 +531,7 @@ int                 nbitem;
    n++;
    XtSetArg (args[n], XmNautoUnmanage, FALSE);
    n++;
-/**XtSetArg(args[n], XmNdefaultPosition, FALSE); n++;**/
+   /**XtSetArg(args[n], XmNdefaultPosition, FALSE); n++;**/
    XtSetArg (args[n], XmNmarginWidth, 0);
    n++;
    XtSetArg (args[n], XmNmarginHeight, 0);
@@ -626,8 +543,8 @@ int                 nbitem;
    XtManageChild (w);
    XmStringFree (title_string);
 
-/*** Cree un Row-Column pour mettre le bouton Quit ***/
-/*** en dessous des touches du clavier.    ***/
+   /*** Cree un Row-Column pour mettre le bouton Quit ***/
+   /*** en dessous des touches du clavier.    ***/
    n = 3;
    XtSetArg (args[n], XmNadjustLast, FALSE);
    n++;
@@ -642,7 +559,7 @@ int                 nbitem;
    w = XmCreateRowColumn (w, "Dialogue", args, n);
    XtManageChild (w);
 
-/*** Cree un Row-Column pour contenir les touches du clavier ***/
+   /*** Cree un Row-Column pour contenir les touches du clavier ***/
    n = 3;
    XtSetArg (args[n], XmNadjustLast, FALSE);
    n++;
@@ -661,7 +578,7 @@ int                 nbitem;
    row = XmCreateRowColumn (w, "Dialogue", args, n);
    XtManageChild (row);
 
-/*** Cree un Row-Column pour contenir le bouton Quit ***/
+   /*** Cree un Row-Column pour contenir le bouton Quit ***/
    n = 3;
    XtSetArg (args[n], XmNorientation, XmHORIZONTAL);
    n++;
@@ -677,7 +594,7 @@ int                 nbitem;
    GCmodel.background = BgMenu_Color;
    GCkey = XCreateGC (TtDisplay, TtRootWindow, GCForeground | GCBackground | GCFunction, &GCmodel);
 
-/*** Cree le bouton Quit ***/
+   /*** Cree le bouton Quit ***/
    n = 0;
    XtSetArg (args[n], XmNbackground, BgMenu_Color);
    n++;
@@ -723,7 +640,7 @@ int                 nbitem;
 	n++;
 	for (i = 0; i < nbitem; i++, it++)
 	  {
-	     string[0] = it->name;
+	     string[0] = it->index;
 	     string[1] = EOS;
 	     w = XmCreatePushButton (row, string, args, n);
 	     XtManageChild (w);
@@ -745,7 +662,7 @@ int                 nbitem;
 
 	for (i = 0; i < nbitem; i++, it++)
 	  {
-	     string[0] = it->name;
+	     string[0] = it->index;
 	     string[1] = '\n';
 	     w = XmCreateDrawnButton (row, "", args, n);
 	     XtManageChild (w);
@@ -753,37 +670,149 @@ int                 nbitem;
 	     XtAddCallback (w, XmNexposeCallback, (XtCallbackProc) ExposeKbd, (XtPointer) (param + i));
 	  }
      }
-#endif /* _GTK */
 }
+#else /* _GTK */
+/*----------------------------------------------------------------------
+  CreateKeyboard creates a keyboard.
+  ----------------------------------------------------------------------*/
+static void CreateKeyboard (int number, char *title, PtrFont pFont,
+			    int col, int x, int y, ITEM *items, int nbitem)
+{
+  int                 i,j;
+  register ITEM      *it;
+  ThotWidget          w;
+  GtkWidget *vbox1;
+  GtkWidget *vbox5=None;
+  GtkWidget *frame2;
+  GtkWidget *hbox1;
+  GtkWidget *hbox2;
+  GtkWidget *button1;
+  GtkWidget *label;
+  char                string[10];
+  int                 param;
+  
+  
+  w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title (GTK_WINDOW (w), title);
+  gtk_window_set_position (GTK_WINDOW (w), GTK_WIN_POS_MOUSE);
+  ConnectSignalGTK (GTK_OBJECT (w),
+		    "delete_event",
+		    GTK_SIGNAL_FUNC(gtk_true),
+		    (gpointer)NULL);
+  
+  vbox1 = gtk_vbox_new (FALSE, 0);
+  gtk_widget_show (vbox1);
+  gtk_container_add (GTK_CONTAINER (w), vbox1);
+   
+  frame2 = gtk_frame_new (NULL);
+  gtk_widget_show (frame2);
+  gtk_box_pack_start (GTK_BOX (vbox1), frame2, TRUE, TRUE, 0);
+  gtk_container_set_border_width (GTK_CONTAINER (frame2), 5);
+  
+  hbox1 = gtk_hbox_new (FALSE, 5);
+  gtk_widget_show (hbox1);
+  gtk_container_add (GTK_CONTAINER (frame2), hbox1);
+  gtk_container_set_border_width (GTK_CONTAINER (hbox1), 5);
+  
+  hbox2 = gtk_hbox_new (FALSE, 5);
+  gtk_widget_show (hbox2);
+  gtk_container_add (GTK_CONTAINER (vbox1), hbox2);
+   
+  button1 = gtk_button_new_with_label (TtaGetMessage (LIB, TMSG_CANCEL));
+  gtk_widget_show (button1);
+  GTK_WIDGET_SET_FLAGS (GTK_WIDGET(button1), GTK_CAN_DEFAULT);
+  gtk_widget_grab_default(GTK_WIDGET(button1));
+  gtk_box_pack_start (GTK_BOX (hbox2), button1, TRUE, TRUE, 2); 
+  gtk_container_set_border_width (GTK_CONTAINER (button1), 2);
+  ConnectSignalGTK (GTK_OBJECT (button1),
+		    "clicked",
+		    GTK_SIGNAL_FUNC (KbdEndDisplay),
+		    (gpointer)number);
+
+  sprintf (string, "+%d+%d", x, y);
+  /* Affiche les differents boutons du clavier */
+  it = items;
+  /* Prepare les parametres des procedures KbdCallbackHandler et ExposeKbd */
+  param = number * 256;	/* indice du clavier */
+  
+  if (it->legend == 0)
+    {
+      j = 0;
+      /* Un clavier sans legende */
+      for (i = 0; i < nbitem; i++, it++)
+	{
+	  if (i == 10 * j)
+	    {
+	      /* vbox witch contain one color column */
+	      vbox5 = gtk_vbox_new (TRUE, 5);
+	      gtk_widget_show (vbox5);
+	      gtk_box_pack_start (GTK_BOX (hbox1), vbox5, TRUE, TRUE, 0);
+	      j++;
+	    }
+	  string[0] = it->index;
+	  string[1] = EOS;
+	  button1 = gtk_button_new ();
+	  gtk_widget_show (button1);
+	  gtk_box_pack_start (GTK_BOX(vbox5), button1, TRUE, TRUE, 0);
+	  label = gtk_label_new (string);
+	  label->style = gtk_style_copy (label->style);
+	  label->style->font = pFont;
+	  gtk_widget_show (label);
+	  gtk_container_add (GTK_CONTAINER (button1), label);
+	  ConnectSignalGTK (GTK_OBJECT (button1),
+			    "clicked",
+			    GTK_SIGNAL_FUNC (KbdCallbackHandler),
+			    (gpointer)(param + (int)(it->value)));
+	}
+    }
+  else
+    {
+      j = 0;
+      /* Un clavier avec legende */
+      for (i = 0; i < nbitem; i++, it++)
+	{
+	  if (i == 10 * j)
+	    {
+	      /* vbox witch contain one color column */
+	      vbox5 = gtk_vbox_new (TRUE, 5);
+	      gtk_widget_show (vbox5);
+	      gtk_box_pack_start (GTK_BOX (hbox1), vbox5, TRUE, TRUE, 0);
+	      j++;
+	    }
+	  
+	  string[0] = it->index;
+	  string[1] = '\n';
+	  button1 = gtk_button_new ();
+	  gtk_widget_show (button1);
+	  gtk_box_pack_start (GTK_BOX(vbox5), button1, TRUE, TRUE, 0);
+	  label = gtk_label_new (string);
+	  label->style = gtk_style_copy (label->style);
+	  label->style->font = pFont;
+	  gtk_widget_show (label);
+	  ConnectSignalGTK (GTK_OBJECT (button1),
+			    "clicked",
+			    GTK_SIGNAL_FUNC (KbdCallbackHandler),
+			    (gpointer)(param + (int)(it->value)));
+	}
+    }
+  gtk_widget_show_all (w);
+  Keyboards[number] = w;
+}
+#endif /* _GTK */
 
 
 /*----------------------------------------------------------------------
    LoadKbd
    loads a keyboard.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         LoadKbd (int number)
-#else  /* __STDC__ */
-static void         LoadKbd (number)
-int                 number;
-#endif /* __STDC__ */
+static void LoadKbd (int number)
 {
-#ifndef _GTK
-  ptrfont             pFontAc;
-  ptrfont             pFontIg;
+  PtrFont             pFontAc;
+  PtrFont             pFontIg;
 
   ConfigKeyboard (&KbX, &KbY);
   switch (number)
     {
-    case 0:		/* Symboles */
-      if (SymbolIcons != NULL)
-	{
-	  KbFonts[number] = SymbolIcons;
-	  CreateKeyboard (number, TtaGetMessage (LIB, TMSG_MATH_SYMBOLS),
-			  SymbolIcons, 3, KbX, KbY, Items_Symb,
-			  sizeof (Items_Symb) / sizeof (ITEM));
-	}
-      break;
     case 1:		/* Graphiques */
       if (GraphicsIcons != NULL)
 	{
@@ -795,7 +824,7 @@ int                 number;
 	}
       break;
     case 2:		/* ISO latin 1 */
-      pFontAc = ReadFont ('L', 'T', 0, 14, UnPoint);
+      pFontAc = ReadFont ('L', 1, 0, 14, UnPoint);
       if (!pFontAc)
 	pFontAc = FontDialogue;
       KbFonts[number] = pFontAc;
@@ -803,17 +832,17 @@ int                 number;
 	CreateKeyboard (number, TtaGetMessage (LIB, TMSG_LATIN_ALPHABET), pFontAc, 13,
 			KbX, KbY, Items_Isol, sizeof (Items_Isol) / sizeof (ITEM));
       break;
-    case 3:		/* Grec */
-      pFontIg = ReadFont ('G', 'T', 0, 14, UnPoint);
+    case 3:		/* Greek */
+      pFontIg = ReadFont ('G', 1, 0, 14, UnPoint);
+      /*pFontIg = LoadFont ("-ttf-esstixone-*-*-*-*-14-*-*-*-*-*-*-*");*/
       if (!pFontIg)
 	pFontIg = FontDialogue;
       if (pFontIg != NULL)
-	KbFonts[number] = pFontIg;	/* Grec */
+	KbFonts[number] = pFontIg;	/* Greek */
       CreateKeyboard (number, TtaGetMessage (LIB, TMSG_GREEK_ALPHABET), pFontIg, 16,
 		      KbX, KbY, Items_Grec, sizeof (Items_Grec) / sizeof (ITEM));
       break;
     }
-#endif /* _GTK */
 }
 #endif /* _WINDOWS */
 
@@ -822,15 +851,8 @@ int                 number;
    KeyboardMap
    maps a keyboard.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                KeyboardMap (int kb)
-#else  /* __STDC__ */
-void                KeyboardMap (kb)
-int                 kb;
-
-#endif /* __STDC__ */
+void KeyboardMap (int kb)
 {
-#ifndef _GTK
 #ifndef _WINDOWS
    if (kb >= 0 && kb < MAX_KEYBOARD)
      {
@@ -839,21 +861,24 @@ int                 kb;
 	   LoadKbd (kb);
 	if (Keyboards[kb] != 0)
 	  {
+#ifndef _GTK
 	    XtPopup (Keyboards[kb], XtGrabNonexclusive);
 	    XMapRaised (TtDisplay, XtWindowOfObject (Keyboards[kb]));
+#else /* _GTK */
+	    gtk_widget_show_all (GTK_WIDGET(Keyboards[kb]));
+	    gdk_window_raise (GTK_WIDGET(Keyboards[kb])->window);
+#endif /* !_GTK */
 	  }
      }
 #endif /* _WINDOWS */
-#endif /* _GTK */
 }
 
 /*----------------------------------------------------------------------
   GraphicsLoadResource
   Initializes the keyboards.
   ----------------------------------------------------------------------*/
-void                GraphicsLoadResources ()
+void GraphicsLoadResources ()
 {
-#ifndef _GTK
    int                 i;
 
    if (ThotLocalActions[T_keyboard] == NULL)
@@ -863,60 +888,40 @@ void                GraphicsLoadResources ()
 	/* Initialise la table des claviers */
 	for (i = 0; i < MAX_KEYBOARD; i++)
 	   Keyboards[i] = 0;
-	SymbolIcons = NULL;
-
 	if (SmallFontDialogue == NULL)
-	   SmallFontDialogue = ReadFont ('L', 'H', 0, 9, UnPoint);
+	   SmallFontDialogue = ReadFont ('L', 2, 0, 9, UnPoint);
 	if (SmallFontDialogue == NULL)
 	   SmallFontDialogue = FontDialogue;
      }
 
-#   ifndef _WINDOWS
+#ifndef _WINDOWS
    GraphicsIcons = LoadFont ("ivgraf");
-#   endif /* _WINDOWS */
+#endif /* _WINDOWS */
    if (GraphicsIcons == NULL)
-     {
-       /*Fonte 'ivgraf' inaccessible */
-       TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_LIB_MISSING_FILE), "ivgraf");
-       GraphicsIcons = FontDialogue;
-     }
-#endif /* _GTK */
+     GraphicsIcons = FontDialogue;
 }
 
 /*----------------------------------------------------------------------
   KeyboardsLoadResource
   Initializes the keyboards.
   ----------------------------------------------------------------------*/
-void                KeyboardsLoadResources ()
+void KeyboardsLoadResources ()
 {
-#ifndef _GTK
-   int                 i;
+  int                 i;
 
-   if (ThotLocalActions[T_keyboard] == NULL)
-     {
-	TteConnectAction (T_keyboard, (Proc) KeyboardMap);
+  if (ThotLocalActions[T_keyboard] == NULL)
+    {
+      TteConnectAction (T_keyboard, (Proc) KeyboardMap);
+      /* Initialise la table des claviers */
+      for (i = 0; i < MAX_KEYBOARD; i++)
+	Keyboards[i] = 0;
+      GraphicsIcons = NULL;
 
-	/* Initialise la table des claviers */
-	for (i = 0; i < MAX_KEYBOARD; i++)
-	   Keyboards[i] = 0;
-	GraphicsIcons = NULL;
-
-	if (SmallFontDialogue == NULL)
-	   SmallFontDialogue = ReadFont ('L', 'H', 0, 9, UnPoint);
-	if (SmallFontDialogue == NULL)
-	   SmallFontDialogue = FontDialogue;
-     }
-
-#   ifndef _WINDOWS
-    SymbolIcons = LoadFont ("ivsymb");
-    if (SymbolIcons == NULL) {
-       /*Fonte 'ivsymb' inaccessible */
-       TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_LIB_MISSING_FILE), "ivsymb");
-       SymbolIcons = FontDialogue;
-	} 
-#   endif /* _WINDOWS */
-
-#endif /* _GTK */
+      if (SmallFontDialogue == NULL)
+	SmallFontDialogue = ReadFont ('L', 2, 0, 9, UnPoint);
+      if (SmallFontDialogue == NULL)
+	SmallFontDialogue = FontDialogue;
+    }
 }
 
 
@@ -924,24 +929,14 @@ void                KeyboardsLoadResources ()
    TtcDisplayMathKeyboard
    displays the math keyboard
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtcDisplayMathKeyboard (Document document, View view)
-
-#else  /* __STDC__ */
-void                TtcDisplayMathKeyboard (document, view)
-Document            document;
-View                view;
-
-#endif /* __STDC__ */
+void TtcDisplayMathKeyboard (Document document, View view)
 {
-#ifndef _GTK
    KeyboardsLoadResources ();
    /* Enregistre la position pour le dialogue */
-#  ifndef _WINDOWS
+#ifndef _WINDOWS
    TtaSetDialoguePosition ();
-#  endif /* !_WINDOWS */
+#endif /* !_WINDOWS */
    TtaSetCurrentKeyboard (0);
-#endif /* _GTK */
 }
 
 
@@ -949,24 +944,14 @@ View                view;
    TtcDisplayGraphicsKeyboard
    displays the graphics keyboard
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtcDisplayGraphicsKeyboard (Document document, View view)
-
-#else  /* __STDC__ */
-void                TtcDisplayGraphicsKeyboard (document, view)
-Document            document;
-View                view;
-
-#endif /* __STDC__ */
+void TtcDisplayGraphicsKeyboard (Document document, View view)
 {
-#ifndef _GTK
    KeyboardsLoadResources ();
    /* Enregistre la position pour le dialogue */
-#  ifndef _WINDOWS
+#ifndef _WINDOWS
    TtaSetDialoguePosition ();
-#  endif /* !_WINDOWS */
+#endif /* !_WINDOWS */
    TtaSetCurrentKeyboard (1);
-#endif /* _GTK */
 }
 
 
@@ -974,49 +959,29 @@ View                view;
    TtcDisplayLatinKeyboard
    displays the latin keyboard
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtcDisplayLatinKeyboard (Document document, View view)
-
-#else  /* __STDC__ */
-void                TtcDisplayLatinKeyboard (document, view)
-Document            document;
-View                view;
-
-#endif /* __STDC__ */
+void TtcDisplayLatinKeyboard (Document document, View view)
 {
-#ifndef _GTK
    KeyboardsLoadResources ();
    /* Enregistre la position pour le dialogue */
-#  ifndef _WINDOWS 
+#ifndef _WINDOWS 
    TtaSetDialoguePosition ();
-#  endif /* !_WINDOWS */
+#endif /* !_WINDOWS */
    TtaSetCurrentKeyboard (2);
-#endif /* _GTK */
 }
 
 /*----------------------------------------------------------------------
    TtcDisplayGreekKeyboard
    displays the greek keyboard 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtcDisplayGreekKeyboard (Document document, View view)
-
-#else  /* __STDC__ */
-void                TtcDisplayGreekKeyboard (document, view)
-Document            document;
-View                view;
-
-#endif /* __STDC__ */
+void TtcDisplayGreekKeyboard (Document document, View view)
 {
-#ifndef _GTK
    KeyboardsLoadResources ();
    /* Enregistre la position pour le dialogue */
-#  ifndef _WINDOWS 
+#ifndef _WINDOWS 
    TtaSetDialoguePosition ();
-#  endif /* !_WINDOWS */
+#endif /* !_WINDOWS */
    TtaSetCurrentKeyboard (3);
-#  ifdef _WINDOWS
+#ifdef _WINDOWS
    CreateGreekKeyboardDlgWindow (NULL);
-#  endif /* _WINDOWS */
-#endif /* _GTK */
+#endif /* _WINDOWS */
 }
