@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -95,7 +95,11 @@ static void Print (char *name, char *dir, char *thotSch, char *thotDoc,
    char                   *ptr;
 #ifdef _WINDOWS
    HINSTANCE               hLib;
-   FARPROC                 ptrMainProc;
+   /* FARPROC                 ptrMainProc; */
+   typedef void (*MYPROC)(HWND, int, char **, HDC, ThotBool,
+			      int, char *, char *, HINSTANCE, ThotBool);
+   MYPROC  ptrMainProc;
+
    char                    tmp[MAX_TXT_LEN];
    char                   *printArgv [100];
    int                     printArgc = 0;
@@ -311,7 +315,7 @@ static void Print (char *name, char *dir, char *thotSch, char *thotDoc,
        sprintf (printArgv[printArgc], "-w%ld", FrRef[0]);
        printArgc++;
 #else  /* _WINDOWS */
-       sprintf (&cmd[i], " -w%ld", FrRef[0]);
+       sprintf (&cmd[i], " -w%u", (unsigned int) FrRef[0]);
 #endif /* _WINDOWS */
      }
    else
@@ -326,7 +330,7 @@ static void Print (char *name, char *dir, char *thotSch, char *thotDoc,
 	   sprintf (printArgv[printArgc], "-w%ld", FrRef[frame]);
 	   printArgc++;
 #else  /* _WINDOWS */
-	   sprintf (&cmd[i], " -w%ld", FrRef[frame]);
+	   sprintf (&cmd[i], " -w%u", (unsigned int) FrRef[frame]);
 #endif /* _WINDOWS */
 	 }
        else
@@ -336,7 +340,7 @@ static void Print (char *name, char *dir, char *thotSch, char *thotDoc,
 	   sprintf (printArgv[printArgc], "-w%ld", FrRef[0]);
 	   printArgc++;
 #else  /* _WINDOWS */
-	   sprintf (&cmd[i], " -w%ld", FrRef[0]);
+	   sprintf (&cmd[i], " -w%u", (unsigned int) FrRef[0]);
 #endif /* _WINDOWS */
 	 }
      }
@@ -489,54 +493,52 @@ static void Print (char *name, char *dir, char *thotSch, char *thotDoc,
    if (cssToPrint != NULL && cssToPrint[0] != EOS)
      {
        i = 0;
-       /* skip leading spaces */ 
-       while (cssToPrint[i] == SPACE)
-	 i++;
-       /* insert the first flag */
-       if (cssToPrint[i] != EOS)
+       while (cssToPrint[i] != EOS)
 	 {
-	   /* insert the flag -v before each view name */
-#ifdef _WINDOWS
-	   printArgv[printArgc] = TtaStrdup ("-css");
-	   printArgc++;
-	   printArgv[printArgc] = TtaGetMemory (50);
-	   j = 0;
-#else  /* _WINDOWS */
-	   j = strlen (cmd);
-           sprintf (&cmd[j], " -css ");
-	   j = strlen (cmd);
-#endif /* _WINDOWS */
-
-	   while (cssToPrint[i] != EOS)
+	   /* skip leading spaces */ 
+	   while (cssToPrint[i] == SPACE)
+	     i++;
+	   if (cssToPrint[i] != 'a' && cssToPrint[i] != 'u')
+	     /* ERROR */
+	     cssToPrint[i] = EOS;
+	   else
 	     {
-	       /* is it a space? */
-	       if (cssToPrint[i] == SPACE)
-		 {
-		   i++;
-		   /* skip multiple spaces */
-		   while (cssToPrint[i] == SPACE)
-		     i++;
-		   if (cssToPrint[i] != EOS)
-		     {
+	       /* insert the flag -cssa or -cssu before each stylesheet name */
 #ifdef _WINDOWS
-		       printArgv[printArgc][j++] = EOS;
-		       printArgc++;
-		       printArgv[printArgc] = TtaStrdup ("-css");
-		       printArgc++;
-		       printArgv[printArgc] = TtaGetMemory (50);
-		       j = 0;
+	       if (cssToPrint[i] == 'a')
+		 /* it's an author stylesheet */
+		 printArgv[printArgc] = TtaStrdup ("-cssa");
+	       else if (cssToPrint[i] == 'u')
+		 /* it's an user stylesheet */
+		 printArgv[printArgc] = TtaStrdup ("-cssu");
+	       printArgc++;
 #else  /* _WINDOWS */
-		       j = strlen (cmd);
-		       sprintf (&cmd[j], " -css ");
-		       j = strlen (cmd);
-#endif /* _WINDOWS */
-		     }
-		 }
+	       j = strlen (cmd);
+	       if (cssToPrint[i] == 'a')
+		 sprintf (&cmd[j], " -cssa ");
 	       else
+		 sprintf (&cmd[j], " -cssu ");
+	       j = strlen (cmd);
+#endif /* _WINDOWS */
+	       /* skip the flag "a" or "u" */
+	       i++;
+	     }
+	   /* skip spaces after the flag */
+	   while (cssToPrint[i] == SPACE && cssToPrint[i] != EOS)
+	     i++;
+           if (cssToPrint[i] != EOS)
+	     /* there is a file name after the flag */
+	     {
+#ifdef _WINDOWS
+	       printArgv[printArgc] = TtaGetMemory (50);
+	       j = 0;
+#endif /* _WINDOWS */
+	       while (cssToPrint[i] != SPACE && cssToPrint[i] != EOS)
 		 {
 		   /* copy the character */
 #ifdef _WINDOWS
 		   printArgv[printArgc][j++] = cssToPrint[i];
+		   printArgv[printArgc][j] = EOS;
 #else /* _WINDOWS */
 		   cmd[j++] = cssToPrint[i];
 		   cmd[j] = EOS;
@@ -544,13 +546,12 @@ static void Print (char *name, char *dir, char *thotSch, char *thotDoc,
 		   /* process next char */
 		   i++;
 		 }
+#ifdef _WINDOWS
+	       printArgc++;
+#endif /* _WINDOWS */
 	     }
 	 }
-#ifdef _WINDOWS
-       printArgv[printArgc][j] = EOS;
-       printArgc++;
-#endif /* _WINDOWS */
-   }
+     }
    /* transmit the path or source file */
 #ifdef _WINDOWS 
    printArgv[printArgc] = TtaStrdup ("-removedir");
@@ -563,7 +564,7 @@ static void Print (char *name, char *dir, char *thotSch, char *thotDoc,
    hLib = LoadLibrary ("thotprinter");
    if (!hLib)
       return /* FATAL_EXIT_CODE */;
-   ptrMainProc = GetProcAddress (hLib, "PrintDoc");
+   ptrMainProc = (MYPROC) GetProcAddress (hLib, "PrintDoc");
    if (!ptrMainProc)
      {
        FreeLibrary (hLib);
@@ -572,7 +573,8 @@ static void Print (char *name, char *dir, char *thotSch, char *thotDoc,
    
    EnableWindow  (FrRef[frame], FALSE);
 
-   ptrMainProc (FrRef[frame], printArgc, printArgv,
+   if (TtPrinterDC)
+   (ptrMainProc) (FrRef[frame], printArgc, printArgv,
 		TtPrinterDC, TtIsTrueColor, 
 		TtWDepth, name, dir, hInstance, buttonCommand);
    FreeLibrary (hLib);
@@ -776,6 +778,8 @@ ThotBool TtaGetPrinterDC (ThotBool reuse, int *orientation, int *paper)
       if (Pdlg.hDevMode)
 	{
 	  lpDevMode = (LPDEVMODE) GlobalLock (Pdlg.hDevMode);
+	  if (!lpDevMode)
+		  return FALSE;
 	  TtPrinterDC = CreateDC (lpDriverName, lpDeviceName, lpPortName, lpDevMode);
 	  if (lpDevMode->dmOrientation == DMORIENT_LANDSCAPE)
 	    /* landscape */
@@ -800,6 +804,8 @@ ThotBool TtaGetPrinterDC (ThotBool reuse, int *orientation, int *paper)
       if (Pdlg.hDevMode)
 	{
 	  lpDevMode = (LPDEVMODE) GlobalLock (Pdlg.hDevMode);
+	  if (!lpDevMode)
+		 return FALSE;
 	  if (lpDevMode->dmOrientation == DMORIENT_LANDSCAPE)
 	    /* landscape */
 	    *orientation = 1;
