@@ -1,6 +1,6 @@
 /*
  *
- *  COPYRIGHT MIT and INRIA, 1996.
+ *  COPYRIGHT MIT and INRIA, 1996-2001.
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -17,7 +17,7 @@
 
 /* Included headerfiles */
 #include "thot_sys.h"
-#include "ustring.h"
+#include "string.h"
 #include "constmedia.h"
 #include "typemedia.h"
 #include "appdialogue.h"
@@ -25,13 +25,21 @@
 #include "registry.h"
 #include "profiles.h"
 
-#include "ustring_f.h"
+#define PROFILE_START              '<'
+#define PROFILE_END                '>'
+#define DOCTYPE_START              '{'
+#define DOCTYPE_END                '}'
+#define MODULE_START               '['
+#define MODULE_END                 ']'
+#define MODULE_REF                 '+'
+#define EDITING_REF                '&'
+
 
 #define MAX_ENTRIES 10
 typedef struct _Profile_Ctl *PtrProCtl;
 typedef struct _ProElement
 {
-  char*                ProName;      /* Name of the entry */
+  char                *ProName;      /* Name of the entry */
   PtrProCtl            ProSubModule; /* Pointer to a sub-module context */
   ThotBool             ProIsModule;    /* TRUE if it is a sub-module */
   ThotBool             ProEdit;      /* TRUE if it's a editing function */
@@ -50,13 +58,18 @@ typedef struct _Profile_Ctl
 /* Profiles table contains the name of all the available profiles */
 static PtrProCtl            ProfileTable = NULL;
 static int                  NbProfiles = 0;
+/* Profiles table contains the name of all the available profiles */
+static PtrProCtl            DoctypeTable = NULL;
+static int                  NbDoctypes = 0;
 /* Modules table contains the name of all the available modules */
 static PtrProCtl            ModuleTable = NULL;
 static int                  NbModules = 0;
 /* Functions table contains current list of available functions */
 static PtrProCtl            FunctionTable = NULL;
 static int                  NbFunctions = 0;
-static char**               SortedFunctionTable = NULL;
+static char               **SortedFunctionTable = NULL;
+static int                 *FunctionMask = NULL;
+static ThotBool            *FunctionRO = NULL;
 
 /* The first context of the current module or profile in progress */
 static PtrProCtl            CurrentModule;
@@ -75,48 +88,44 @@ static ThotBool             EnableEdit = TRUE;
 /*********************************************************
   String functions : set of useful functions for strings
 ***********************************************************/
-/*----------------------------------------------------------------------
-  SkipNewLineSymbol : Remove the EOF ('end of line') character
-----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void   SkipNewLineSymbol (char* Astring)
-#else  /* !__STDC__ */
-static void   SkipNewLineSymbol (Astring)
-char*         Astring;
-#endif /* !__STDC__ */
-{
-  int         c = 0;
-
-  while (Astring[c] != EOS && Astring[c] != EOL)
-    c++;
-  if (Astring[c] == EOL)
-    Astring[c] = EOS;    
-}
 
 
 /*----------------------------------------------------------------------
   SkipAllBlanks:  Remove all the spaces, tabulations and return
   returns (CR) and "end of line" characters.
 ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void   SkipAllBlanks (char* Astring)
-#else  /* !__STDC__ */
-static void   SkipAllBlanks (Astring)
-char*         string;
-#endif /* !__STDC__ */
+static void SkipAllBlanks (char *Astring)
 {
   int         c = 0;
   int         nbsp = 0;
-  
+
+  /* locate the biginning of the string */
+  while (Astring[c + nbsp] != EOS &&
+	 (Astring[c + nbsp] == SPACE || Astring[c + nbsp] == TAB ||
+	  Astring[c + nbsp] == __CR__))
+    nbsp++;
+  /* suppress blanks before */
+  while (Astring[c] != EOS)
+    {
+      Astring[c] = Astring[c + nbsp];
+      c++;
+    }
+  /* suppress blanks after the string */
+  while (c > 0 &&
+	 (Astring[c - 1] == SPACE || Astring[c - 1] == TAB ||
+	  Astring[c - 1] == __CR__ || Astring[c - 1] == EOL))
+    {
+      c--;
+      Astring[c] = EOS;
+    }
   do
     {
-      while (Astring[c+nbsp] == WC_SPACE || Astring[c+nbsp] == WC_TAB || Astring[c+nbsp] == WC_CR)
+      while (Astring[c + nbsp] == SPACE || Astring[c + nbsp] == TAB ||
+	     Astring[c + nbsp] == __CR__)
 	nbsp++;
-      Astring[c] = Astring[c+nbsp];
-    } 
-  while (Astring[c++] != WC_EOS);
-
-  SkipNewLineSymbol (Astring);
+      Astring[c] = Astring[c + nbsp];
+    }
+  while (Astring[c++] != EOS);
 }
 
 
@@ -131,16 +140,9 @@ char*         string;
   The parameter ctxt points to the first ctxt of the table.
   Return the first context of the new module.
 ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static PtrProCtl   AddInTable (char* name, ThotBool isModule, ThotBool edit, PtrProCtl subModule, int number, PtrProCtl ctxt)
-#else  /* !__STDC__ */
-static PtrProCtl   AddInTable (name, isModule, edit, subModule, number, ctxt)
-char*              name;
-ThotBool           isModule;
-ThotBool           edit;
-int                number;
-PtrProCtl          table;
-#endif /* !__STDC__ */
+static PtrProCtl AddInTable (char *name, ThotBool isModule, ThotBool edit,
+			      PtrProCtl subModule, int number,
+			      PtrProCtl ctxt)
 {
   PtrProCtl           current, prev, new;
   int                 i;
@@ -191,12 +193,7 @@ PtrProCtl          table;
   AddModule inserts a new module in the module table.
   Return the first context of the new module.
 ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static PtrProCtl    AddModule (char* name)
-#else  /* !__STDC__ */
-static PtrProCtl    AddModule (name)
-char*               name;
-#endif /* !__STDC__ */
+static PtrProCtl AddModule (char *name)
 {
   PtrProCtl     new = NULL;
 
@@ -216,12 +213,7 @@ char*               name;
   AddProfile inserts a new profile in the profile table.
   Return the first context of the new profile.
 ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static PtrProCtl    AddProfile (char* name)
-#else  /* !__STDC__ */
-static PtrProCtl    AddProfile (name)
-char*               name;
-#endif /* !__STDC__ */
+static PtrProCtl AddProfile (char *name)
 {
   PtrProCtl     new = NULL;
 
@@ -239,19 +231,32 @@ char*               name;
   return new;
 }
 
+/*----------------------------------------------------------------------
+  AddDoctype inserts a new doctype profile in the table.
+  Return the first context of the new profile.
+----------------------------------------------------------------------*/
+static PtrProCtl AddDoctype (char *name)
+{
+  PtrProCtl     new = NULL;
+
+  /* Register the new profile */
+  if (NbDoctypes == 0)
+    {
+      DoctypeTable = (PtrProCtl) TtaGetMemory (sizeof (Profile_Ctl));
+      memset (DoctypeTable, 0, sizeof (Profile_Ctl));
+    }
+  new = AddInTable (name, TRUE, FALSE, NULL, NbDoctypes, DoctypeTable);
+  NbDoctypes++;
+  return new;
+}
+
 
 /*----------------------------------------------------------------------
   DeleteTable frees all contexts allocated for the table.
   The parameter ctxt points to the first context of the table.
   The parameter recursive is TRUE if the referred context must be freed.
 ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void    DeleteTable (PtrProCtl ctxt, ThotBool recursive)
-#else  /* __STDC__ */
-static void    DeleteTable (ctxt, recursive)
-PtrProCtl      ctxt;
-ThotBool       recursive;
-#endif /* __STDC__ */
+static void DeleteTable (PtrProCtl ctxt, ThotBool recursive)
 {
   PtrProCtl           next;
   int                 i;
@@ -278,12 +283,7 @@ ThotBool       recursive;
   SearchModule searchs a module in the module table and return the
   pointer to current entry.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static ProElement  *SearchModule (char* name)
-#else  /* !__STDC__ */
-static ProElement  *SearchModule (name) 
-char*               name;
-#endif /* !__STDC__ */
+static ProElement *SearchModule (char *name)
 {
   PtrProCtl        current;
   int              i = 0;
@@ -316,12 +316,7 @@ char*               name;
   ProcessDefinition : Recursive function that helps
   building the profile table
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void    ProcessDefinition (char* element)
-#else  /* !__STDC__ */
-static void    ProcessDefinition (element)
-char*         element;
-#endif /* !__STDC__ */ 
+static void ProcessDefinition (char *element)
 {
   ProElement      *pEntry;
   PtrProCtl        ctxt;
@@ -341,6 +336,20 @@ char*         element;
       CurrentModule = AddProfile (&element[1]);
       CurrentEntries = 0;
     }
+  else if (*element == DOCTYPE_START)
+    {
+      /*
+       * It's a document profile definition -> insert it in the doctype table
+       * Remove the start tag and the end tag
+       */
+      i = 1;
+      while (element[i] != DOCTYPE_END)
+	i++;
+      element[i] = EOS;
+      /* The new profile in progress */
+      CurrentModule = AddDoctype (&element[1]);
+      CurrentEntries = 0;
+    }
   else if (*element == MODULE_START)
     {
       /*
@@ -403,12 +412,7 @@ char*         element;
    AddFunctions keep in the function table the list of functions
    declared in the list of contexts.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void AddFunctions (PtrProCtl ctxt)
-#else  /* !__STDC__ */
-static void AddFunctions (ctxt)
-PtrProCtl   ctxt;
-#endif /* !__STDC__ */
+static void AddFunctions (PtrProCtl ctxt, PtrProCtl functionTable)
 {
   int           i;
 
@@ -419,10 +423,12 @@ PtrProCtl   ctxt;
 	{
 	  if (ctxt->ProEntries[i].ProIsModule)
 	    /* add functions of the sub-module */
-	    AddFunctions (ctxt->ProEntries[i].ProSubModule);
+	    AddFunctions (ctxt->ProEntries[i].ProSubModule, functionTable);
 	  else
 	    {
-	      AddInTable (ctxt->ProEntries[i].ProName, FALSE, ctxt->ProEntries[i].ProEdit, NULL, NbFunctions, FunctionTable);
+	      AddInTable (ctxt->ProEntries[i].ProName, FALSE,
+			  ctxt->ProEntries[i].ProEdit, NULL,
+			  NbFunctions, functionTable);
 	      NbFunctions++;
 	    }
 	  /* next entry */
@@ -437,26 +443,33 @@ PtrProCtl   ctxt;
 /*----------------------------------------------------------------------
   SortFunctionTable generates the function table in ascending
   order.
+  The parameter checkEdit will return TRUE if alomst one editing function
+  is available in the generated table.
   ----------------------------------------------------------------------*/
-static void SortFunctionTable ()
+static void SortFunctionTable (PtrProCtl ctxt)
 {
-  PtrProCtl     ctxt;
-  char*         ptr;
+  char         *ptr;
   int           i, j, index;
+  ThotBool      RO;
 
   /* copy the list of contexts in a large table */
-  SortedFunctionTable = (char**) TtaGetMemory (NbFunctions * sizeof (char*));
+  SortedFunctionTable = (char **) TtaGetMemory (NbFunctions * sizeof (char *));
+  FunctionRO = (ThotBool *) TtaGetMemory (NbFunctions * sizeof (ThotBool));
   index = 0;
-  ctxt = FunctionTable;
   EnableEdit = FALSE;
   while (ctxt && index < NbFunctions)
     {
       i = 0;
       while (i < MAX_ENTRIES && ctxt->ProEntries[i].ProName)
 	{
-	  if (!EnableEdit && ctxt->ProEntries[i].ProEdit)
-	    /* there is almost one editing function */
-	    EnableEdit = TRUE;
+	  if (ctxt->ProEntries[i].ProEdit)
+	    {
+	      /* there is almost one editing function */
+	      EnableEdit = TRUE;
+	      FunctionRO[index] = FALSE;
+	    }
+	  else
+	    FunctionRO[index] = TRUE;
 	  SortedFunctionTable[index] = TtaStrdup (ctxt->ProEntries[i].ProName);
 	  index++;
 	  /* next entry */
@@ -476,27 +489,51 @@ static void SortFunctionTable ()
       ptr = SortedFunctionTable[index];
       SortedFunctionTable[index] = SortedFunctionTable[i];
       SortedFunctionTable[i] = ptr;
-    }
-
-  /* delete the function table */
-  if (FunctionTable)
-    {
-      DeleteTable (FunctionTable, FALSE);
-      FunctionTable = NULL;
+      RO =  FunctionRO[index];
+      FunctionRO[index] =  FunctionRO[i];
+      FunctionRO[i] = RO;
     }
 }
 
 
-/*----------------------------------------------------------------------
-  Prof_BelongTable searchs a function in the function table and return TRUE
-  if the function exists.
+/*-----------------------------------------------------------------------
+   FunctionInProfile returns TRUE if the function is defined in the
+   document profile.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool    Prof_BelongTable (char* name)
-#else  /* !__STDC__ */
-ThotBool    Prof_BelongTable (name) 
-char*       name;
-#endif /* !__STDC__ */
+static ThotBool FunctionInProfile (char *name, PtrProCtl ctxt)
+{
+  int           i;
+
+  while (ctxt)
+    {
+      i = 0;
+      while (i < MAX_ENTRIES && ctxt->ProEntries[i].ProName)
+	{
+	  if (ctxt->ProEntries[i].ProIsModule)
+	    {
+	      if (FunctionInProfile (name, ctxt->ProEntries[i].ProSubModule))
+		/* stop if the function is founs this sub-module */
+		return TRUE;
+	    }
+	  else if (!strcmp (name, ctxt->ProEntries[i].ProName))
+	    {
+	      /* the function is declared in that document profile */
+	      return TRUE;
+	    }
+	  /* next entry */
+	  i++;
+	}
+      /* next context */
+      ctxt = ctxt->ProNext;
+    }
+  return FALSE;
+}
+
+/*----------------------------------------------------------------------
+  Prof_BelongTable searchs a function in the function table and returns
+  TRUE if the function exists.
+  ----------------------------------------------------------------------*/
+ThotBool Prof_BelongTable (char *name)
 {
   int              left, right, middle, i;
   ThotBool         found = FALSE;
@@ -524,30 +561,92 @@ char*       name;
 }
 
 
+/*----------------------------------------------------------------------
+  Prof_BelongDoctype searchs a function in the function table.
+  Returns:
+  -1: the function is unknown
+  0: the function must be hidden
+  1: the function is accepted
+  ----------------------------------------------------------------------*/
+int Prof_BelongDoctype (char *name, int docProfile, ThotBool RO)
+{
+  int              left, right, middle, i;
+
+  if (NbFunctions == 0 || docProfile == 0)
+    /* All functions are allowed */
+    return -1;
+
+  /* Dichotomic search */
+  left = middle = 0;
+  right = NbFunctions - 1;
+ 
+  while (left <= right)
+    {
+      middle = (right + left) / 2;
+      i = strcmp (SortedFunctionTable[middle], name);
+      if (i == 0)
+	{
+	  /* check the profile value */
+	  if (FunctionMask[middle] == 0)
+	    /* not defind */
+	    return -1;
+	  else if (FunctionMask[middle] & docProfile)
+	    {
+	      if (RO && !FunctionRO[middle])
+		/* refused */
+		return 0;
+	      else
+		/* acceped */
+		return 1;
+	    }
+	  else
+	    /* refused */
+	    return 0;
+	}
+      else if (i < 0)
+	left = middle + 1;
+      else
+	right = middle - 1;
+    }
+  return -1;
+}
+
+
 /*-----------------------------------------------------------------------
    Prof_InitTable: Seek the current profile file and init tables
   ----------------------------------------------------------------------*/
-void Prof_InitTable ()
+void Prof_InitTable (char *prof_file)
 {
-  FILE*     profFile;
-  CHAR_T*   ptr;
-  CHAR_T    buffer[MAX_LENGTH];
+  FILE               *profFile;
+  char               *ptr;
+  char                buffer[MAX_LENGTH];
+  int                 i, j;
 
-  /* TODO: use a ISO functions for TtaGetEnvString and TtaStrdup */
-  /* Retreive thot.rc variables and open the profile file */
-  ptr = TtaGetEnvString ("Profiles_File");
+  /* open the profile file */
+  /* if the caller didn't specify any profile, we use the one
+     given in the registry */
+  if (prof_file && *prof_file)
+    ptr = prof_file;
+  else
+    ptr = TtaGetEnvString ("Profiles_File");
 
   if (ptr && *ptr)
     {
       if (SearchFile (ptr, 2, buffer))
 	{
-	  profFile = ufopen (buffer, TEXT("r"));
+	  profFile = fopen (buffer, "r");
 	  /* what is the current active profile */  
 	  ptr = TtaGetEnvString ("Profile");
 	  if (ptr && *ptr)
-	    wc2iso_strcpy (UserProfile, ptr);	
+	    {
+	      if (strstr (ptr, "(editor)"))
+		/* avoid to use a dead profile */
+		strcpy (UserProfile, "Editor");
+	      else
+		strcpy (UserProfile, ptr);
+	    }
 	  else
-	    UserProfile[0] = WC_EOS;
+	    strcpy (UserProfile, "Editor");
 
 	  /* Fill a profile and module tables */
 	  while (fgets (ProfileBuff, sizeof (ProfileBuff), profFile))
@@ -575,16 +674,51 @@ void Prof_InitTable ()
 	{
 	  FunctionTable = (PtrProCtl) TtaGetMemory (sizeof (Profile_Ctl));
 	  memset (FunctionTable, 0, sizeof (Profile_Ctl));
-	  AddFunctions (UserProfContext);
+	  AddFunctions (UserProfContext, FunctionTable);
 	  /* generate a sorted list of available functions */
-	  SortFunctionTable ();
+	  SortFunctionTable (FunctionTable);
+	  /* delete the function table */
+	  if (FunctionTable)
+	    {
+	      DeleteTable (FunctionTable, FALSE);
+	      FunctionTable = NULL;
+	    }
+	}
+    }
+
+  /* associate a mask to each function in the table */
+  FunctionMask = (int *) TtaGetMemory (NbFunctions * sizeof (int));
+  for (i = 0; i < NbFunctions; i++)
+    {
+      FunctionMask[i] = 0;
+      for (j = 0; j < NbDoctypes; j++)
+	{
+	  if (FunctionInProfile (SortedFunctionTable[i],
+				 DoctypeTable->ProEntries[j].ProSubModule))
+	    {
+	      if (strstr (DoctypeTable->ProEntries[j].ProName, "Transitional"))
+		FunctionMask[i] = (FunctionMask[i] | L_TransitionalValue);
+	      else if (strstr (DoctypeTable->ProEntries[j].ProName, "Strict"))
+		FunctionMask[i] = (FunctionMask[i] | L_StrictValue);
+	      else if (strstr (DoctypeTable->ProEntries[j].ProName, "Basic"))
+		FunctionMask[i] = (FunctionMask[i] | L_BasicValue);
+	      else if (strstr (DoctypeTable->ProEntries[j].ProName, "1.1"))
+		FunctionMask[i] = (FunctionMask[i] | L_StrictValue | L_RubyValue);
+	      else if (strstr (DoctypeTable->ProEntries[j].ProName, "Ruby"))
+		FunctionMask[i] = (FunctionMask[i] | L_RubyValue);
+	      else if (strstr (DoctypeTable->ProEntries[j].ProName, "MathML"))
+		FunctionMask[i] = (FunctionMask[i] | L_MathMLValue);
+	    }
 	}
     }
-  /* TODO: remove that clean up as soon as we are able to update Amaya UI */
   /* delete the modules table */
   DeleteTable (ModuleTable, TRUE);
   ModuleTable = NULL;
   NbModules = 0;
+  /* delete the doctype table */
+  DeleteTable (DoctypeTable, TRUE);
+  DoctypeTable = NULL;
+  NbDoctypes = 0;
 }
 
 
@@ -597,12 +731,18 @@ void Prof_FreeTable ()
 
   /* delete the profiles table */
   DeleteTable (ProfileTable, TRUE);
+  ProfileTable = NULL;
   NbProfiles = 0;
   UserProfContext = NULL;
+  /* delete the function table */
   for (i = 0; i < NbFunctions; i++)
     TtaFreeMemory (SortedFunctionTable[i]);
   TtaFreeMemory (SortedFunctionTable);
   SortedFunctionTable = NULL;
+  TtaFreeMemory (FunctionMask);
+  FunctionMask = NULL;
+  TtaFreeMemory (FunctionRO);
+  FunctionRO = NULL;
   NbFunctions = 0;
 }
 
@@ -610,23 +750,18 @@ void Prof_FreeTable ()
 /*----------------------------------------------------------------------
   TtaRebuildProTable: Rebuild the Profiles Table
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void     TtaRebuildProTable (CHAR_T* prof_file)
-#else  /* !__STDC__ */
-void     TtaRebuildProTable (prof_file)
-CHAR_T*  prof_file;
-#endif /* !__STDC__ */
+void TtaRebuildProTable (char *prof_file)
 {
   /* delete the profiles table */
   Prof_FreeTable ();
-  Prof_InitTable ();
+  Prof_InitTable (prof_file);
 }
 
 
 /*----------------------------------------------------------------------
   TtaCanEdit returns TRUE if there is almost one editing function active.
   ----------------------------------------------------------------------*/
-ThotBool    TtaCanEdit ()
+ThotBool TtaCanEdit ()
 {
   return (EnableEdit);
 }
@@ -635,23 +770,18 @@ ThotBool    TtaCanEdit ()
    TtaGetProfileFileName:  Get the text for the profile file name.
    name is a provided buffer of length characters to receive the name.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void     TtaGetProfileFileName (CHAR_T* name, int length)
-#else  /* !__STDC__ */
-void     TtaGetProfileFileName (name, length)
-CHAR_T*  name;
-#endif /* !__STDC__ */
+void TtaGetProfileFileName (char *name, int length)
 {
-  CHAR_T*   ptr;
-  CHAR_T    buffer[MAX_LENGTH];
+  char   *ptr;
+  char    buffer[MAX_LENGTH];
 
   name[0] = EOS;
   ptr = TtaGetEnvString ("Profiles_File");
   if (ptr && *ptr)
     {
       SearchFile (ptr, 2, buffer);
-      if (ustrlen (buffer) < (size_t)length)
-	ustrcpy (name, buffer);
+      if (strlen (buffer) < (size_t)length)
+	strcpy (name, buffer);
     }
 }
 
@@ -659,23 +789,18 @@ CHAR_T*  name;
    TtaGetDefProfileFileName:  Get the text for the default profile file name.
    name is a provided buffer of length characters to receive the name.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void     TtaGetDefProfileFileName (CHAR_T* name, int length)
-#else  /* !__STDC__ */
-void     TtaGetDefProfileFileName (name, length)
-CHAR_T*  name;
-#endif /* !__STDC__ */
+void TtaGetDefProfileFileName (char *name, int length)
 {
-  CHAR_T* ptr;
-  CHAR_T  buffer[200];
+  char *ptr;
+  char  buffer[200];
 
-  name[0] = WC_EOS;
+  name[0] = EOS;
   ptr = TtaGetDefEnvString ("Profiles_File");
   if (ptr && *ptr)
     {
       SearchFile (ptr, 2, buffer);
-      if (ustrlen (buffer) < (size_t)length)
-	ustrcpy (name, buffer);
+      if (strlen (buffer) < (size_t)length)
+	strcpy (name, buffer);
     }
 }
 
@@ -685,12 +810,7 @@ CHAR_T*  name;
    listEntries is a provided list of length pointers.
    Returns the number of items
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-int      TtaGetProfilesItems (char** listEntries, int length)
-#else  /* !__STDC__ */
-int      TtaGetProfilesItems (listEntries, length)
-char**   listEntries;
-#endif /* !__STDC__ */
+int TtaGetProfilesItems (char **listEntries, int length)
 {
   PtrProCtl     ctxt;
   int           nbelem = 0;
@@ -713,32 +833,20 @@ char**   listEntries;
 
 
 /*----------------------------------------------------------------------
-    Prof_ShowSeparator : determines if a separator must be displayed
-    in the menu interface. Two consecutive separators musn't be displayed
+  Prof_ShowSeparator : determines if a separator must be displayed
+  in the menu interface. Two consecutive separators musn't be displayed
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool     Prof_ShowSeparator (Menu_Ctl *ptrmenu, int item, char LastItemType)
-#else  /* __STDC__ */
-ThotBool     Prof_ShowSeparator (*ptrmenu, item, LastItemType)
-Menu_Ctl    *ptrmenu;
-int          item;
-char         LastItemType;
-#endif /* __STDC__ */
+ThotBool Prof_ShowSeparator (Menu_Ctl *ptrmenu, int item, char LastItemType)
 {  
   return !(LastItemType == 'S' || item == 0);
 }
 
 
 /*-----------------------------------------------------------------------
-    Prof_ShowButton : Add a button if the function associated to that button
-    belongs to the user profile
+  Prof_ShowButton : Add a button if the function associated to that button
+  belongs to the user profile
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool    Prof_ShowButton (char* FunctionName)
-#else  /* __STDC__ */
-ThotBool    Prof_ShowButton (FunctionName)
-char*       FunctionName;
-#endif /* __STDC__ */
+ThotBool Prof_ShowButton (char *FunctionName)
 {
   return (Prof_BelongTable (FunctionName));
 }
@@ -748,12 +856,7 @@ char*       FunctionName;
     Prof_ShowSubMenu : Check if a submenu has to be displayed.
     A submenu musn't be displayed if it contains no entry
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool   Prof_ShowSubMenu (Menu_Ctl *ptrsubmenu)
-#else  /* __STDC__ */
-ThotBool   Prof_ShowSubMenu (ptrsubmenu)
-Menu_Ctl  *ptrsubmenu;
-#endif /* __STDC__ */
+ThotBool Prof_ShowSubMenu (Menu_Ctl *ptrsubmenu)
 {
   int      item    = 0;
   
@@ -761,7 +864,7 @@ Menu_Ctl  *ptrsubmenu;
     return FALSE;
   while (item < ptrsubmenu->ItemsNb)
     {
-      if (ptrsubmenu->ItemsList[item].ItemType != TEXT('S'))
+      if (ptrsubmenu->ItemsList[item].ItemType != 'S')
 	return TRUE;
       item ++;
     }
@@ -772,12 +875,7 @@ Menu_Ctl  *ptrsubmenu;
     Prof_ShowMenu : Check if a menu has to be displayed. A menu mustn't be
     displayed if it contains no entry.
  -----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool   Prof_ShowMenu (Menu_Ctl *ptrmenu)
-#else  /* __STDC__ */
-ThotBool   Prof_ShowMenu (ptrmenu)
-Menu_Ctl  *ptrmenu;
-#endif /* __STDC__ */
+ThotBool Prof_ShowMenu (Menu_Ctl *ptrmenu)
 {
   int      item    = 0;
 
@@ -799,9 +897,9 @@ Menu_Ctl  *ptrmenu;
   /* check if the menu is only composed of empty sub menus and separators */
   while (item < ptrmenu->ItemsNb)
     {
-      if (ptrmenu->ItemsList[item].ItemType != TEXT('S'))
+      if (ptrmenu->ItemsList[item].ItemType != 'S')
 	{
-	  if (ptrmenu->ItemsList[item].ItemType == TEXT('M'))
+	  if (ptrmenu->ItemsList[item].ItemType == 'M')
 	    {
 	      if (Prof_ShowSubMenu (ptrmenu->ItemsList[item].SubMenu)) 
 		/* there is at least a non empty sub menu */
