@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -35,34 +35,24 @@
 
 /* pointer to the search domain for the the current command */
 static PtrSearchContext SearchingD = NULL;
-/* document where we are searching a page */
-static PtrDocument   SearchedPageDoc;
 /* document to which the CurrRef belongs */
 static PtrDocument   CurrRefDoc;
 static PtrDocument   DocTextOK;
-
-/* root of the tree where we are searching a page */
-static PtrElement    SearchedPageRoot;
 /* element whose references we are looking for */
 static PtrElement    CurrRefElem;
 static PtrElement    ElemTextOK;
-
-/* document view for which we are searching a page */
-static int           ViewSearchedPageDoc;
-/* presentation scheme view for which we are searching a page */
-static int           SearchedPageSchView;
-/* number of the searched page */
-static int           SearchedPageNumber;
 /* precedent searched string */
 static unsigned char pPrecedentString[THOT_MAX_CHAR];
 /* searched string */
 static unsigned char pSearchedString[THOT_MAX_CHAR];
+static CHAR_T       *SString = NULL;
 /* length of the searched string */
-static int           SearchedStringLen;
+static int           SStringLen;
 /* the replace string */
 static unsigned char pReplaceString[THOT_MAX_CHAR];
+static CHAR_T       *RString = NULL;
 /* length of the replace string */
-static int           ReplaceStringLen;
+static int           RStringLen;
 /* pointer to the current reference */
 static PtrReference  CurrRef;
 
@@ -322,7 +312,7 @@ void CreateSearchDlgWindow (ThotWindow parent)
   InitMenuWhereToSearch 
   inits the "Where to search" submenu.
   ----------------------------------------------------------------------*/
-static void         InitMenuWhereToSearch (int ref)
+static void InitMenuWhereToSearch (int ref)
 {
   int                 i;
   char                string[200];
@@ -341,7 +331,7 @@ static void         InitMenuWhereToSearch (int ref)
 		 NULL, FALSE);
   TtaSetMenuForm (NumMenuOrSearchText, 2);
 }
-#endif /* !_WINDOWS */
+#endif /* _WINDOWS */
 
 
 /*----------------------------------------------------------------------
@@ -366,7 +356,7 @@ void         ResetSearchInDocument (PtrDocument pDoc)
   CallbackWhereToSearch
   callback handler for the "Where to search" submenu.
   ----------------------------------------------------------------------*/
-void                CallbackWhereToSearch (int ref, int val)
+void CallbackWhereToSearch (int ref, int val)
 {
    /* determine le point de depart de la recherche et le point */
    /* de terminaison */
@@ -399,7 +389,7 @@ void                CallbackWhereToSearch (int ref, int val)
   launches the command for searching and/or replacing a text,
   an element type and an attribute for the pDoc document.
   ----------------------------------------------------------------------*/
-void                TtcSearchText (Document document, View view)
+void TtcSearchText (Document document, View view)
 {
   PtrDocument         pDocSel;
   PtrElement          pFirstSel;
@@ -452,7 +442,7 @@ void                TtcSearchText (Document document, View view)
   
   /* zone de saisie du texte de remplacement */
   TtaNewTextForm (NumZoneTextReplace, NumFormSearchText,
-		  TtaGetMessage (LIB, TMSG_REPLACE_BY), 30, 1, TRUE);
+		  TtaGetMessage (LIB, TMSG_REPLACE_BY), 30, 1, FALSE);
   TtaSetTextForm (NumZoneTextReplace, pReplaceString);
   
   /* sous-menu mode de remplacement */
@@ -472,12 +462,14 @@ void                TtcSearchText (Document document, View view)
       TtaNewSubmenu (NumMenuReplaceMode, NumFormSearchText, 0,
 		     TtaGetMessage (LIB, TMSG_REPLACE), 3, string,
 		     NULL, FALSE);
-      if (WithReplace) {
-	if (AutoReplace)
-          TtaSetMenuForm (NumMenuReplaceMode, 2);
-	else
-	  TtaSetMenuForm (NumMenuReplaceMode, 1);
-      } else
+      if (WithReplace)
+	{
+	  if (AutoReplace)
+	    TtaSetMenuForm (NumMenuReplaceMode, 2);
+	  else
+	    TtaSetMenuForm (NumMenuReplaceMode, 1);
+	}
+      else
 	TtaSetMenuForm (NumMenuReplaceMode, 0);
     }
   else
@@ -494,15 +486,12 @@ void                TtcSearchText (Document document, View view)
   
 #ifdef _WINDOWS
   SearchAfter = ok;
-#else /* _WINDOWS */
-  /* efface le label "References dans le document X" */
-  TtaNewLabel (NumLabelAttributeValue, NumFormSearchText, " ");
 #endif /* _WINDOWS */
   SearchLoadResources ();
   /* complete la feuille de dialogue avec les menus de recherche de types */
   /* d'element et d'attributs si la ressource de recherche avec structure */
   /* est chargee */
-  if (ThotLocalActions[T_strsearchconstmenu] != NULL)
+  if (ThotLocalActions[T_strsearchconstmenu])
     (*ThotLocalActions[T_strsearchconstmenu]) (pDoc);
   
   /* active le formulaire */
@@ -521,6 +510,17 @@ void                TtcSearchText (Document document, View view)
 #endif /* _WINDOWS */
 }
 
+/*----------------------------------------------------------------------
+  CleanSearchContext free all context related to search commands.
+  ----------------------------------------------------------------------*/
+static void CleanSearchContext ()
+{
+  FreeSearchContext (&SearchingD);
+  TtaFreeMemory (SString);
+  SString = NULL;
+  TtaFreeMemory (RString);
+  RString = NULL;
+}
 
 /*----------------------------------------------------------------------
   CallbackTextReplace
@@ -552,21 +552,30 @@ void CallbackTextReplace (int ref, int val, char *txt)
 	  ReplaceDone = FALSE;
 	  strcpy (pPrecedentString, pSearchedString);
 	}
-      SearchedStringLen = strlen (pSearchedString);
+      /* convert the string if necessary */
+      TtaFreeMemory (SString);
+      SString = TtaConvertByteToCHAR (pSearchedString, ISO_8859_1);
+      SStringLen = ustrlen (SString);
       break;
     case NumZoneTextReplace:
       /* Chaine a remplacer */
-      strcpy (pReplaceString, txt);
-      ReplaceStringLen = strlen (pReplaceString);
-      /* bascule automatiquement le remplacement */
-      if (!WithReplace && !SearchingD->SDocument->DocReadOnly)
+      if (pReplaceString[0] == EOS && txt[0] != EOS)
 	{
-	  WithReplace = TRUE;
-	  DoReplace = TRUE;
+	  if (!WithReplace && !SearchingD->SDocument->DocReadOnly)
+	    {
+	      WithReplace = TRUE;
+	      DoReplace = TRUE;
 #ifndef _WINDOWS
-	  TtaSetMenuForm (NumMenuReplaceMode, 1);
-#endif /* !_WINDOWS */
+	      TtaSetMenuForm (NumMenuReplaceMode, 1);
+#endif /* _WINDOWS */
+	    }
 	}
+      strcpy (pReplaceString, txt);
+      /* convert the string if necessary */
+      TtaFreeMemory (RString);
+      RString = TtaConvertByteToCHAR (pReplaceString, ISO_8859_1);
+      RStringLen = ustrlen (RString);
+      /* bascule automatiquement le remplacement */
       break;
     case NumToggleUpperEqualLower:
       if (val == 0)
@@ -602,20 +611,24 @@ void CallbackTextReplace (int ref, int val, char *txt)
       break;
     case NumFormSearchText:
       /* Boutons de la feuille de dialogue */
-#ifndef _WINDOWS
-      TtaNewLabel (NumLabelAttributeValue, NumFormSearchText, " ");
-#endif /* !_WINDOWS */
-      if (SearchingD->SDocument == NULL)
+      if (SearchingD->SDocument == NULL ||
+	  SearchingD->SDocument->DocSSchema == NULL)
 	{
-	  TtaDestroyDialogue (NumFormSearchText);
+#ifdef _WINDOWS
+      EndDialog (SearchW, ID_DONE);
+      SearchW = NULL;
+#else /* _WINDOWS */
+      TtaDestroyDialogue (NumFormSearchText);
+#endif /* _WINDOWS */
+	  TtaFreeMemory (SString);
+	  SString = NULL;
+	  TtaFreeMemory (RString);
+	  RString = NULL;
 	  return;
 	}
-      if (SearchingD->SDocument->DocSSchema == NULL)
-	return;
       if (val == 2 && WithReplace && !StartSearch)
 	DoReplace = FALSE;
       else if (val == 0)
-	/* Abandon de la recherche */
 	return;
 
       selectionOK = GetCurrentSelection (&pDocSel, &pFirstSel,
@@ -629,7 +642,11 @@ void CallbackTextReplace (int ref, int val, char *txt)
 	  selectionOK = FALSE;
 	  StartSearch = TRUE;
 	}
-
+      else if (firstChar == 0 && lastChar == 0 && pFirstSel == pLastSel)
+	{
+	  firstChar = 1;
+	  lastChar = pFirstSel->ElVolume + 1;
+	}
       if (StartSearch)
 	pCurrEl = NULL;
       else if (SearchingD->SStartToEnd)
@@ -640,10 +657,10 @@ void CallbackTextReplace (int ref, int val, char *txt)
        /* la recherche est demandee, on recupere les parametres */
       if (ThotLocalActions[T_strsearchgetparams] != NULL)
 	(*ThotLocalActions[T_strsearchgetparams]) (&error, SearchingD);
-      if (!error || SearchedStringLen != 0)
+      if (!error || SStringLen != 0)
 	{
 	  found = FALSE;
-	  if (SearchedStringLen == 0)
+	  if (SStringLen == 0)
 	    {
 	      if (ThotLocalActions[T_strsearchonly] != NULL)
 		(*ThotLocalActions[T_strsearchonly]) (pCurrEl, SearchingD,
@@ -674,11 +691,7 @@ void CallbackTextReplace (int ref, int val, char *txt)
 		    /* on ne remplace pas dans un sous-arbre en */
 		    /* lecture seule */
 		    if (ElementIsReadOnly (pFirstSel))
-#ifndef _WINDOWS
-		      TtaNewLabel (NumLabelAttributeValue, NumFormSearchText,
-				   TtaGetMessage (LIB, TMSG_EL_RO))
-#endif /* !_WINDOWS */
-			;
+		      ;
 		    else if (!pFirstSel->ElIsCopy && pFirstSel->ElText != NULL
 			     && pFirstSel->ElTerminal
 			     && pFirstSel->ElLeafType == LtText)
@@ -695,7 +708,7 @@ void CallbackTextReplace (int ref, int val, char *txt)
 			    if (!AutoReplace)
 			      OpenHistorySequence (SearchingD->SDocument,
 					       pFirstSel, pFirstSel, firstChar,
-					       firstChar+SearchedStringLen-1);
+					       firstChar + SStringLen);
 			    AddEditOpInHistory (pFirstSel,
 						SearchingD->SDocument,
 						TRUE, TRUE);
@@ -703,21 +716,22 @@ void CallbackTextReplace (int ref, int val, char *txt)
 			      CloseHistorySequence (SearchingD->SDocument);
 			    /* effectue le remplacement du texte */
 			    ReplaceString (SearchingD->SDocument, pFirstSel,
-					   firstChar, SearchedStringLen,
-					   pReplaceString, ReplaceStringLen,
+					   firstChar, SStringLen,
+					   RString, RStringLen,
 					   (ThotBool)(!AutoReplace));
 			    ReplaceDone = TRUE;
 			    StartSearch = FALSE;
+			    if (AutoReplace)
+			      lastChar += RStringLen - SStringLen;
 			    /* met eventuellement a jour la borne de */
 			    /* fin du domaine de recherche */
-			    if (pFirstSel == SearchingD->SEndElement)
-			      /* la borne est dans l'element ou` on a */
-			      /* fait le remplacement */
-			      if (SearchingD->SEndChar != 0)
+			    if (pFirstSel == SearchingD->SEndElement &&
+				/* la borne est dans l'element ou` on a */
+				/* fait le remplacement */
+				SearchingD->SEndChar > 1)
 				/* la borne n'est pas a la fin de */
 				/* l'element, on decale la borne */
-				SearchingD->SEndChar +=
-                                          ReplaceStringLen - SearchedStringLen;
+			      SearchingD->SEndChar += RStringLen - SStringLen;
 			    /* recupere les parametres de la nouvelle */
 			    /* chaine */
 			    if (!AutoReplace)
@@ -766,11 +780,8 @@ void CallbackTextReplace (int ref, int val, char *txt)
 		      found = SearchText (SearchingD->SDocument,
 					  &pFirstSel, &firstChar, &pLastSel,
 					  &lastChar, SearchingD->SStartToEnd,
-					  UpperLower, pSearchedString,
-					  SearchedStringLen);
-		      if (found)
-			lastChar--;
-		      
+					  UpperLower, SString,
+					  SStringLen);
 		      foundString = found;
 		      if (found)
 			/* on a trouve' la chaine cherchee */
@@ -795,7 +806,6 @@ void CallbackTextReplace (int ref, int val, char *txt)
 						 lastChar);
 			  /* arrete la boucle de recherche */
 			  stop = TRUE;
-			  lastChar++;
 			}
 		      TextOK = TRUE;
 		      DocTextOK = SearchingD->SDocument;
@@ -826,46 +836,42 @@ void CallbackTextReplace (int ref, int val, char *txt)
 		 else /* Before selection */
 		   TtaSetMenuForm (NumMenuOrSearchText, 0);
 	       }
-#endif /* !_WINDOWS */
+#endif /* _WINDOWS */
 	     StartSearch = FALSE;
 	     if (ThotLocalActions[T_strsearchshowvalattr] != NULL)
 	       (*ThotLocalActions[T_strsearchshowvalattr]) ();
 	    }
 	  else
-	    /* on n'a pas trouve' */
 	    {
+	      /* not found */
+#ifdef _WINDOWS
+	      if (!searchEnd)
+		{
+		  searchEnd = TRUE;
+		  if (WithReplace && ReplaceDone)
+		    {
+		    if (!AutoReplace)
+		      MessageBox (NULL,
+				  TtaGetMessage (LIB, TMSG_NOTHING_TO_REPLACE),
+				  msgCaption, MB_OK | MB_ICONEXCLAMATION);
+		    }
+		  else
+		    MessageBox (NULL, TtaGetMessage (LIB, TMSG_NOT_FOUND),
+				msgCaption, MB_OK | MB_ICONEXCLAMATION);
+		}
+#else /* _WINDOWS */
 	      if (WithReplace && ReplaceDone)
 		{
 		  if (!AutoReplace)
-		    {
-#ifdef _WINDOWS
-		      if (!searchEnd)
-			{
-			  searchEnd = TRUE;
-			  MessageBox (NULL,
-				 TtaGetMessage (LIB, TMSG_NOTHING_TO_REPLACE),
-                                 msgCaption, MB_OK | MB_ICONEXCLAMATION);
-			}
-#else /* !_WINDOWS */
-		      /* message "Plus de remplacement" */
-		      TtaNewLabel (NumLabelAttributeValue, NumFormSearchText,
-				   TtaGetMessage (LIB, TMSG_NOTHING_TO_REPLACE));
-#endif /* !_WINDOWS */
-		    }
+		    TtaDisplayMessage (CONFIRM,
+				       TtaGetMessage (LIB, TMSG_NOTHING_TO_REPLACE),
+				       NULL);
 		}
 	      else
-		/* message "Pas trouve'" */
-#ifdef _WINDOWS
-		if (!searchEnd)
-		  {
-		    searchEnd = TRUE;
-		    MessageBox (NULL, TtaGetMessage (LIB, TMSG_NOT_FOUND),
-				msgCaption, MB_OK | MB_ICONEXCLAMATION);
-		  }
-#else  /* !_WINDOWS */
-	      TtaNewLabel (NumLabelAttributeValue, NumFormSearchText,
-			   TtaGetMessage (LIB, TMSG_NOT_FOUND));
-#endif /* !_WINDOWS */
+		TtaDisplayMessage (CONFIRM,
+				   TtaGetMessage (LIB, TMSG_NOT_FOUND),
+				   NULL);
+#endif /* _WINDOWS */
 	      StartSearch = TRUE;
 	    }
 	}
@@ -877,100 +883,29 @@ void CallbackTextReplace (int ref, int val, char *txt)
     }
 }
 
-/*----------------------------------------------------------------------
-  BuildGoToPageMenu
-  handles the Goto Page number command.
-  ----------------------------------------------------------------------*/
-void BuildGoToPageMenu (PtrDocument pDoc, int docView, int schView)
-{
-   char                  buffTitle[200];
-
-   if (ThotLocalActions[T_searchpage] == NULL)
-      TteConnectAction (T_searchpage, (Proc) CallbackGoToPageMenu);
-
-   /* garde le pointeur sur le document concerne' par la commande */
-   SearchedPageDoc = pDoc;
-   /* garde le  numero de vue (dans le document) de la vue concernee */
-   ViewSearchedPageDoc = docView;
-   /* garde la racine de l'arbre ou on va chercher une page, ainsi que */
-   /* le numero (dans le schema de presentation) de la vue concernee */
-   /* cherche le numero de vue dans le schema de presentation */
-   /* applique' au document */
-   SearchedPageSchView = AppliedView (pDoc->DocDocElement, NULL, pDoc,
-				      docView);
-   SearchedPageRoot = pDoc->DocDocElement;
-   /* compose le titre "Recherche dans le document..." */
-   strcpy (buffTitle, TtaGetMessage (LIB, TMSG_SEARCH_IN));
-   strcat (buffTitle, pDoc->DocDName);
-#ifndef _WINDOWS 
-   /* cree formulaire de saisie du numero de la page cherchee */
-   TtaNewSheet (NumFormSearchPage,  0, buffTitle, 1,
-		TtaGetMessage (LIB, TMSG_LIB_CONFIRM), TRUE, 1, 'L',
-		D_CANCEL);
-
-   /* cree zone de saisie du numero de la page cherchee */
-   TtaNewNumberForm (NumZoneSearchPage, NumFormSearchPage, TtaGetMessage (LIB, TMSG_GOTO_PAGE), 0, 9999, FALSE);
-   /* affiche le formulaire */
-   TtaShowDialogue (NumFormSearchPage, FALSE);
-#endif /* !_WINDOWS */
-}
-
-/*----------------------------------------------------------------------
-  CallbackGoToPageMenu
-  callback handler for the GotoPage menu.
-  ----------------------------------------------------------------------*/
-void CallbackGoToPageMenu (int ref, int val)
-{
-   PtrElement          pPage;
-
-   if (SearchedPageDoc != NULL)
-      if (SearchedPageDoc->DocSSchema != NULL)
-	 /* le document concerne' est toujours la */
-	 switch (ref)
-	       {
-		  case NumZoneSearchPage:
-		     /* zone de saisie du numero de la page cherchee */
-		     SearchedPageNumber = val;
-		     break;
-		  case NumFormSearchPage:
-		     /* formulaire de saisie du numero de la page cherchee */
-		     /* cherche la page */
-		     pPage = SearchPageBreak (SearchedPageRoot,
-					      SearchedPageSchView,
-					      SearchedPageNumber, FALSE);
-		     /* fait afficher la page trouvee en haut de sa frame */
-		     ScrollPageToTop (pPage, ViewSearchedPageDoc,
-				      SearchedPageDoc);
-		     TtaDestroyDialogue (NumFormSearchPage);
-		     break;
-	       }
-}
 
 /*----------------------------------------------------------------------
   SearchLoadResources
   inits the variables of the search commands.
   ----------------------------------------------------------------------*/
-void                SearchLoadResources (void)
+void SearchLoadResources (void)
 {
    if (ThotLocalActions[T_searchtext] == NULL)
      {
 	/* Connecte les actions liees au traitement du search */
 	TteConnectAction (T_searchtext, (Proc) CallbackTextReplace);
 	TteConnectAction (T_locatesearch, (Proc) CallbackWhereToSearch);
-        
-	/*TteConnectAction (T_searchemptyref, (Proc) CallbackSearchEmptyref);
-	  TteConnectAction (T_searchemptyelt, (Proc) CallbackSearchEmptyEl);
-	  TteConnectAction (T_searchrefto, (Proc) CallbackReferenceTo);*/
+	TteConnectAction (T_freesearch, (Proc) CleanSearchContext);
 	CurrRef = NULL;
 	CurrRefDoc = NULL;
 	CurrRefElem = NULL;
 	pExtCurrDoc = NULL;
 	GetSearchContext (&SearchingD);
 	pSearchedString[0] = EOS;
-	SearchedStringLen = 0;
+	SStringLen = 0;
 	UpperLower = TRUE;
 	WithReplace = FALSE;
 	pReplaceString[0] = EOS;
-	ReplaceStringLen = 0;
+	RStringLen = 0;
      }
 }
