@@ -1,15 +1,16 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
- 
-/*----------------------------------------------------------------------
-   
-   Application Program Interface                     
-   Language management                                          
-  ----------------------------------------------------------------------*/
+
+/*
+ *
+ * Languages and Scripts
+ *
+ * Author: I. Vatton(W3C/INRIA)
+ */
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -21,12 +22,13 @@
 
 struct Langue_Ctl   LangTable[MAX_LANGUAGES];
 static char         Langbuffer[2 * MAX_NAME_LENGTH];
+static char         Langbuffer1[2 * MAX_NAME_LENGTH];
+static char         Langbuffer2[2 * MAX_NAME_LENGTH];
 static char         CodeBuffer[2 * MAX_NAME_LENGTH];
 static int          breakPoints[MAX_POINT_COUP];
 static char         StandardLANG[3];
 static char        *dictPath;	/* Environment variable DICOPAR */
 int                 FreeEntry;
-int                 FirstUserLang;
 
 #include "thotmsg_f.h"
 
@@ -217,7 +219,7 @@ char *TtaGetLanguageNameFromCode (char *code)
 	strcpy (Langbuffer, ISO639table[i].fullName);
     }
   if (Langbuffer[0] == EOS)
-    for (i = 0; Langbuffer[0] == EOS && OldLangTable[i].code != EOS; i++)
+    for (i = 0; Langbuffer[0] == EOS && OldLangTable[i].code[0] != EOS; i++)
       {
 	if (!strcasecmp (code, OldLangTable[i].code))
 	  strcpy (Langbuffer, OldLangTable[i].fullName);
@@ -261,167 +263,159 @@ void InitLanguage ()
      {
 	LangTable[i].LangName[0] = EOS;
 	LangTable[i].LangCode[0] = EOS;
-	LangTable[i].LangAlphabet = 'L';
+	LangTable[i].LangScript = 'L';
 	for (j = 0; j < MAX_DICTS; j++)
 	   LangTable[i].LangDict[j] = NULL;
 	LangTable[i].LangPattern[0] = EOS;
 	LangTable[i].LangTabPattern.Charge = 0;
      }
-   /* Loading the default system languages */
-   i = 0;
-   strcpy (LangTable[i].LangName, "ISO_latin_1");
-   strcpy (LangTable[i].LangCode, "x-Latin1");
-   LangTable[i].LangAlphabet = 'L';
-   strcpy (LangTable[i].LangPrincipal, "Usigle");
-   strcpy (LangTable[i].LangSecondary, "Uname");
+   /*
+     Loading the default user languages:
+     Values less than FirstUserLang are scripts (Latin, Greek, Arabic, Hebrew)
+   */
 
-   i = 1;
-   strcpy (LangTable[i].LangName, "ISO_latin_2");
-   strcpy (LangTable[i].LangCode, "x-Latin2");
-   LangTable[i].LangAlphabet = '2';
+   i = 0;
+   strcpy (LangTable[i].LangName, "Arabic");
+   strcpy (LangTable[i].LangCode, "ar");
+   LangTable[i].LangScript = 'A';
    LangTable[i].LangPrincipal[0] = EOS;
    LangTable[i].LangSecondary[0] = EOS;
 
+   i = 1;
+   strcpy (LangTable[i].LangName, "Chinese");
+   strcpy (LangTable[i].LangCode, "zh");
+   LangTable[i].LangScript = 'C';
+   LangTable[i].LangPrincipal[0] = EOS;
+   LangTable[i].LangSecondary[0] = EOS;
+   FreeEntry = 19;
+
    i = 2;
-   strcpy (LangTable[i].LangName, "ISO_latin_9");
-   strcpy (LangTable[i].LangCode, "x-Latin9");
-   LangTable[i].LangAlphabet = '9';
+   strcpy (LangTable[i].LangName, "Czech");
+   strcpy (LangTable[i].LangCode, "cs");
+   LangTable[i].LangScript = '2';
    LangTable[i].LangPrincipal[0] = EOS;
    LangTable[i].LangSecondary[0] = EOS;
 
    i = 3;
-   strcpy (LangTable[i].LangName, "Symbol");
-   strcpy (LangTable[i].LangCode, "x-Symbol");
-   LangTable[i].LangAlphabet = 'G';
-   LangTable[i].LangPrincipal[0] = EOS;
+   strcpy (LangTable[i].LangName, "Dutch");
+   strcpy (LangTable[i].LangCode, "nl");
+   LangTable[i].LangScript = 'L';
+   strcpy (LangTable[i].LangPrincipal, "Nprinc");
    LangTable[i].LangSecondary[0] = EOS;
+   strcpy (LangTable[i].LangPattern, "nederl.ptn");
 
-   /* Loading the default user languages */
-   FirstUserLang = 4;
    i = 4;
-   strcpy (LangTable[i].LangName, "French");
-   strcpy (LangTable[i].LangCode, "fr");
-   LangTable[i].LangAlphabet = 'L';
-   strcpy (LangTable[i].LangPrincipal, "Fprinc");
-   strcpy (LangTable[i].LangSecondary, "Fperso");
-   strcpy (LangTable[i].LangPattern, "francais.ptn");
-
-   i = 5;
    strcpy (LangTable[i].LangName, "English");
    strcpy (LangTable[i].LangCode, "en");
-   LangTable[i].LangAlphabet = 'L';
+   LangTable[i].LangScript = 'L';
    strcpy (LangTable[i].LangPrincipal, "Eprinc");
    strcpy (LangTable[i].LangSecondary, "Eperso");
    strcpy (LangTable[i].LangPattern, "english.ptn");
 
+   i = 5;
+   strcpy (LangTable[i].LangName, "Finnish");
+   strcpy (LangTable[i].LangCode, "fi");
+   LangTable[i].LangScript = 'L';
+   LangTable[i].LangPrincipal[0] = EOS;
+   LangTable[i].LangSecondary[0] = EOS;
+   strcpy (LangTable[i].LangPattern, "finish.ptn");
+
    i = 6;
-   strcpy (LangTable[i].LangName, "American");
-   strcpy (LangTable[i].LangCode, "en-US");
-   LangTable[i].LangAlphabet = 'L';
-   strcpy (LangTable[i].LangPrincipal, "Eprinc");
-   strcpy (LangTable[i].LangSecondary, "Eperso");
-   strcpy (LangTable[i].LangPattern, "american.ptn");
+   strcpy (LangTable[i].LangName, "French");
+   strcpy (LangTable[i].LangCode, "fr");
+   LangTable[i].LangScript = 'L';
+   strcpy (LangTable[i].LangPrincipal, "Fprinc");
+   strcpy (LangTable[i].LangSecondary, "Fperso");
+   strcpy (LangTable[i].LangPattern, "francais.ptn");
 
    i = 7;
    strcpy (LangTable[i].LangName, "German");
    strcpy (LangTable[i].LangCode, "de");
-   LangTable[i].LangAlphabet = 'L';
+   LangTable[i].LangScript = 'L';
    strcpy (LangTable[i].LangPrincipal, "Gprinc");
    LangTable[i].LangSecondary[0] = EOS;
    strcpy (LangTable[i].LangPattern, "deutsch.ptn");
 
    i = 8;
+   strcpy (LangTable[i].LangName, "Greek");
+   strcpy (LangTable[i].LangCode, "el");
+   LangTable[i].LangScript = 'G';
+   LangTable[i].LangPrincipal[0] = EOS;
+   LangTable[i].LangSecondary[0] = EOS;
+
+   i = 9;
+   strcpy (LangTable[i].LangName, "Hebrew");
+   strcpy (LangTable[i].LangCode, "he");
+   LangTable[i].LangScript = 'H';
+   LangTable[i].LangPrincipal[0] = EOS;
+   LangTable[i].LangSecondary[0] = EOS;
+
+   i = 10;
+   strcpy (LangTable[i].LangName, "Icelandic");
+   strcpy (LangTable[i].LangCode, "is");
+   LangTable[i].LangScript = 'L';
+   LangTable[i].LangPrincipal[0] = EOS;
+   LangTable[i].LangSecondary[0] = EOS;
+
+   i = 11;
    strcpy (LangTable[i].LangName, "Italian");
    strcpy (LangTable[i].LangCode, "it");
-   LangTable[i].LangAlphabet = 'L';
+   LangTable[i].LangScript = 'L';
    strcpy (LangTable[i].LangPrincipal, "Iprinc");
    LangTable[i].LangSecondary[0] = EOS;
    strcpy (LangTable[i].LangPattern, "italiano.ptn");
 
-   i = 9;
-   strcpy (LangTable[i].LangName, "Spanish");
-   strcpy (LangTable[i].LangCode, "es");
-   LangTable[i].LangAlphabet = 'L';
-   strcpy (LangTable[i].LangPrincipal, "Sprinc");
+   i = 12;
+   strcpy (LangTable[i].LangName, "Japanese");
+   strcpy (LangTable[i].LangCode, "ja");
+   LangTable[i].LangScript = 'Z';
+   LangTable[i].LangPrincipal[0] = EOS;
    LangTable[i].LangSecondary[0] = EOS;
-   strcpy (LangTable[i].LangPattern, "espanol.ptn");
 
-   i = 10;
+   i = 13;
+   strcpy (LangTable[i].LangName, "Polish");
+   strcpy (LangTable[i].LangCode, "pl");
+   LangTable[i].LangScript = '2';
+   LangTable[i].LangPrincipal[0] = EOS;
+   LangTable[i].LangSecondary[0] = EOS;
+
+   i = 14;
    strcpy (LangTable[i].LangName, "Portuguese");
    strcpy (LangTable[i].LangCode, "pt");
-   LangTable[i].LangAlphabet = 'L';
+   LangTable[i].LangScript = 'L';
    LangTable[i].LangPrincipal[0] = EOS;
    LangTable[i].LangSecondary[0] = EOS;
    strcpy (LangTable[i].LangPattern, "portug.ptn");
 
-   i = 11;
-   strcpy (LangTable[i].LangName, "Dutch");
-   strcpy (LangTable[i].LangCode, "nl");
-   LangTable[i].LangAlphabet = 'L';
-   strcpy (LangTable[i].LangPrincipal, "Nprinc");
+   i = 15;
+   strcpy (LangTable[i].LangName, "Spanish");
+   strcpy (LangTable[i].LangCode, "es");
+   LangTable[i].LangScript = 'L';
+   strcpy (LangTable[i].LangPrincipal, "Sprinc");
    LangTable[i].LangSecondary[0] = EOS;
-   strcpy (LangTable[i].LangPattern, "nederl.ptn");
+   strcpy (LangTable[i].LangPattern, "espanol.ptn");
 
-   i = 12;
+   i = 16;
    strcpy (LangTable[i].LangName, "Swedish");
    strcpy (LangTable[i].LangCode, "sv");
-   LangTable[i].LangAlphabet = 'L';
+   LangTable[i].LangScript = 'L';
    strcpy (LangTable[i].LangPrincipal, "Wprinc");
    LangTable[i].LangSecondary[0] = EOS;
    strcpy (LangTable[i].LangPattern, "swedish.ptn");
 
-   i = 13;
-   strcpy (LangTable[i].LangName, "Finnish");
-   strcpy (LangTable[i].LangCode, "fi");
-   LangTable[i].LangAlphabet = 'L';
-   LangTable[i].LangPrincipal[0] = EOS;
-   LangTable[i].LangSecondary[0] = EOS;
-   strcpy (LangTable[i].LangPattern, "finish.ptn");
-
-   i = 14;
-   strcpy (LangTable[i].LangName, "Greek");
-   strcpy (LangTable[i].LangCode, "el");
-   LangTable[i].LangAlphabet = 'G';
-   LangTable[i].LangPrincipal[0] = EOS;
-   LangTable[i].LangSecondary[0] = EOS;
-
-   i = 15;
-   strcpy (LangTable[i].LangName, "Czech");
-   strcpy (LangTable[i].LangCode, "cs");
-   LangTable[i].LangAlphabet = '2';
-   LangTable[i].LangPrincipal[0] = EOS;
-   LangTable[i].LangSecondary[0] = EOS;
-
    i = 17;
-   strcpy (LangTable[i].LangName, "Polish");
-   strcpy (LangTable[i].LangCode, "pl");
-   LangTable[i].LangAlphabet = '2';
-   LangTable[i].LangPrincipal[0] = EOS;
-   LangTable[i].LangSecondary[0] = EOS;
-
-   i = 18;
    strcpy (LangTable[i].LangName, "Turkish");
    strcpy (LangTable[i].LangCode, "tr");
-   LangTable[i].LangAlphabet = '9';
+   LangTable[i].LangScript = '9';
    LangTable[i].LangPrincipal[0] = EOS;
    LangTable[i].LangSecondary[0] = EOS;
-   FreeEntry = 19;
-
-   i = 19;
-   strcpy (LangTable[i].LangName, "Icelandic");
-   strcpy (LangTable[i].LangCode, "is");
-   LangTable[i].LangAlphabet = 'L';
-   LangTable[i].LangPrincipal[0] = EOS;
-   LangTable[i].LangSecondary[0] = EOS;
-   FreeEntry = 19;
 }
 
 
 /*----------------------------------------------------------------------
    TtaNewLanguage
 
-   Not available for TYPO languages.
-   Declares a new language, its alphabet and optionally the names of the
+   Declares a new language, its script and optionally the names of the
    principal ans secondary dictionaries. All languages used in a Thot
    document must be explicitely declared, except for predefined languages.
    Thot registers all declared languages and allocates a different
@@ -434,43 +428,44 @@ void InitLanguage ()
    has no effect.
    Parameters:
    languageName: name of the language according RFC 1766.
-   languageAlphabet: alphabet to be used for writing that language:
+   languageScript: script to be used for writing that language:
    `L' for ISO-Latin-1, `G' for Symbol (Greek).
    principalDictionary: name of the principal dictionary or NULL.
    secondDictionary: name of the secondary dictionary or NULL.
    Return value:
    identifier of the new language or 0 if the language cannot be added.
   ----------------------------------------------------------------------*/
-Language TtaNewLanguage (char *languageName, char languageAlphabet,
+Language TtaNewLanguage (char *languageName, char languageScript,
 			 char *principalDictionary, char *secondDictionary)
 {
-   int                 i;
+  int                 i;
 
-   i = 0;
-   /* Avoids error cases */
-   if (languageName == NULL)
-      TtaError (ERR_invalid_parameter);
-   else if (languageName[0] == EOS)
-      TtaError (ERR_invalid_parameter);
-   else if (strlen (languageName) >= MAX_NAME_LENGTH)
-      TtaError (ERR_string_too_long);
-   else if (FreeEntry == MAX_LANGUAGES)
-      TtaError (ERR_too_many_languages);
-   else
-     {
-        if (strlen (languageName) == 2 ||
-	   languageName[1] == '-' || languageName[2] == '-')
-	   /* it's an ISO-639 code */
-	  {
+  i = 0;
+  /* Avoids error cases */
+  if (languageName == NULL)
+    TtaError (ERR_invalid_parameter);
+  else if (languageName[0] == EOS)
+    TtaError (ERR_invalid_parameter);
+  else if (strlen (languageName) >= MAX_NAME_LENGTH)
+    TtaError (ERR_string_too_long);
+  else if (FreeEntry == MAX_LANGUAGES)
+    TtaError (ERR_too_many_languages);
+  else
+    {
+      if (strlen (languageName) == 2 ||
+	  languageName[1] == '-' || languageName[2] == '-')
+	/* it's an ISO-639 code */
+	{
 	  for (i = 0; i < FreeEntry; i++)
 	    if (!strcasecmp (languageName, LangTable[i].LangCode))
 	      /* The language is already defined */
 	      return i;
 	  strcpy (LangTable[FreeEntry].LangName, languageName);
-	  strcpy (LangTable[FreeEntry].LangCode, TtaGetLanguageCodeFromName (languageName));
-	  }
-	else
-	  {
+	  strcpy (LangTable[FreeEntry].LangCode,
+		  TtaGetLanguageCodeFromName (languageName));
+	}
+      else
+	{
 	  /* Consults the languages table to see if the language exists. */
 	  for (i = 0; i < FreeEntry; i++)
 	    if (!strcasecmp (languageName, LangTable[i].LangName))
@@ -478,28 +473,28 @@ Language TtaNewLanguage (char *languageName, char languageAlphabet,
 	      return i;
 	  strcpy (LangTable[FreeEntry].LangName, languageName);
 	  strcpy (LangTable[FreeEntry].LangCode, TtaGetLanguageCodeFromName (languageName));
-	  }
-
-	/* Saves the new language */
-	i = FreeEntry;
-	LangTable[i].LangAlphabet = languageAlphabet;
-	if (principalDictionary != NULL)
-	  {
-	     strncpy (LangTable[i].LangPrincipal, principalDictionary, MAX_NAME_LENGTH);
-	     LangTable[i].LangPrincipal[MAX_NAME_LENGTH - 1] = EOS;
-	  }
-	else
-	   LangTable[i].LangPrincipal[0] = EOS;
-	if (secondDictionary != NULL)
-	  {
-	     strncpy (LangTable[i].LangSecondary, secondDictionary, MAX_NAME_LENGTH);
-	     LangTable[i].LangSecondary[MAX_NAME_LENGTH - 1] = EOS;
-	  }
-	else
-	   LangTable[i].LangSecondary[0] = EOS;
-	FreeEntry++;
-     }
-   return i;
+	}
+      
+      /* Saves the new language */
+      i = FreeEntry;
+      LangTable[i].LangScript = languageScript;
+      if (principalDictionary != NULL)
+	{
+	  strncpy (LangTable[i].LangPrincipal, principalDictionary, MAX_NAME_LENGTH);
+	  LangTable[i].LangPrincipal[MAX_NAME_LENGTH - 1] = EOS;
+	}
+      else
+	LangTable[i].LangPrincipal[0] = EOS;
+      if (secondDictionary != NULL)
+	{
+	  strncpy (LangTable[i].LangSecondary, secondDictionary, MAX_NAME_LENGTH);
+	  LangTable[i].LangSecondary[MAX_NAME_LENGTH - 1] = EOS;
+	}
+      else
+	LangTable[i].LangSecondary[0] = EOS;
+      FreeEntry++;
+    }
+  return (Language) (i + FirstUserLang);
 }
 
 
@@ -512,20 +507,20 @@ Language TtaNewLanguage (char *languageName, char languageAlphabet,
   ----------------------------------------------------------------------*/
 void TtaRemoveLanguage (Language language)
 {
-   int                 i;
+  int                 i;
 
-   i = (int) language;
-   if (i >= FreeEntry || i < 0)
-      TtaError (ERR_language_not_found);
-   else
-      {
+  i = (int) language - FirstUserLang;
+  if (i >= FreeEntry || i < 0)
+    TtaError (ERR_language_not_found);
+  else
+    {
       LangTable[i].LangName[0] = EOS;
       LangTable[i].LangCode[0] = EOS;
-      /* don't erase LangAlphabet */
+      /* don't erase LangScript */
       LangTable[i].LangPrincipal[0] = EOS;
       LangTable[i].LangSecondary[0] = EOS;
       LangTable[i].LangPattern[0]   = EOS;
-      }
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -533,14 +528,14 @@ void TtaRemoveLanguage (Language language)
 
    Available for TYPO languages.
    Returns the identifier of a language that matches a language name.
-   Parameters:
    name: name of the language.
    Return value:
-   identifier of that language or 0 if the language is unknown.
+   identifier of that language, 0 if the language is not supported, 
+   -1 for an unknown language.
   ----------------------------------------------------------------------*/
 Language TtaGetLanguageIdFromName (char *name)
 { 
-  char                code[MAX_LENGTH];
+  char                code[MAX_LENGTH], *ptr;
   int                 i;
   ThotBool            again;
 
@@ -554,7 +549,7 @@ Language TtaGetLanguageIdFromName (char *name)
     TtaError (ERR_string_too_long);
   else
     {
-      /* Consults the languages table to see if the language exists. */
+      /* Checks the languages table to see if the language exists */
       again = TRUE;
       while (again && i < FreeEntry)
 	{
@@ -563,9 +558,13 @@ Language TtaGetLanguageIdFromName (char *name)
 	    again = FALSE;
 	  else if (!strcasecmp (name, LangTable[i].LangName))
 	    again = FALSE;
+	  else if (name[2] == '-' &&
+		   !strncasecmp (name, LangTable[i].LangCode, 2))
+	    again = FALSE;
 	  else
 	    i++;
 	}
+
       if (again)
 	{
 	  strcpy (code, TtaGetLanguageCodeFromName (name));
@@ -582,13 +581,15 @@ Language TtaGetLanguageIdFromName (char *name)
 	}
       if (again)
 	{
-	  /* The language does not exist */
-	  i = 0;
-	  TtaError (ERR_language_not_found);
+	  ptr = TtaGetLanguageNameFromCode (name);
+	  if (*ptr == EOS)
+	    return -1;
+	  else
+	    return Latin_Script;
 	}
     }
   /* returned value */
-  return (Language) i;
+  return (Language) (i + FirstUserLang);
 }
 
 /*----------------------------------------------------------------------
@@ -628,88 +629,136 @@ Language TtaGetDefaultLanguage ()
 }
 
 /*----------------------------------------------------------------------
-   TtaGetLanguageIdFromAlphabet
+   TtaGetLanguageIdFromScript
 
-   Not available for TYPO languages.
-   Returns the identifier of the first language that uses a given alphabet.
+   Returns the identifier of the first language that uses a given script
+   `L' -> ISO_latin_1, `G' -> Symbol.
    Parameters:
-   languageAlphabet: the alphabet of interest (`L' = latin, `G' = greek).
+   languageScript: the script of interest.
    Return value:
    identifier of that language or 0 if the language is unknown.
   ----------------------------------------------------------------------*/
-Language TtaGetLanguageIdFromAlphabet (char languageAlphabet)
+Language TtaGetLanguageIdFromScript (char languageScript)
 {
-   int                 i;
-   ThotBool            again;
-
-   i = 0;
-   /* Consults the languages table to see if the language exists. */
-   again = TRUE;
-   while (again && i < FreeEntry)
-     {
-       if (languageAlphabet == LangTable[i].LangAlphabet)
-	 /* The language is already defined */
-	 again = FALSE;
-       else
-	 i++;
-     }
-   
-   if (again)
-     {
-       /* The language does not exist */
-       i = 0;
-       TtaError (ERR_language_not_found);
-     }
-   return (Language) i;
+  /* No user language declared */
+  switch (languageScript)
+    {
+    case 'G':
+      return Greek_Script;
+    case 'A':
+      return Arabic_Script;
+    case 'H':
+      return Hebrew_Script;
+    case 'C':
+      return Cyrillic_Script;
+    case 'Z':
+      return CJK_Script;
+    default:
+      return Latin_Script;
+    }
 }
 
 
 /*----------------------------------------------------------------------
-   TtaGetAlphabet
+   TtaGetCharacterScript
 
-   Not available for TYPO languages. Returns the alphabet of a language.
-   Parameters:
-   languageId: name of the language.
+   Returns the script of the wide character.
    Return value:
-   a character that identifies the alphabet ('L' = latin, 'G' = greek).
+   A character that identifies the script
+   'L' = Latin, 'G' = Greek 'A' = Arabic, etc.
   ----------------------------------------------------------------------*/
-char TtaGetAlphabet (Language languageId)
+char TtaGetCharacterScript (wchar_t c)
+{
+  if (c == SPACE || c == 0x9A /* nbsp */ || c == 0x200D ||
+      c == 0x200E /* lrm */ || c == 0x200F /* rlm */ ||
+      c == 0x202A /* lre */ || c == 0x202B /* rle */ ||
+      c == 0x202D /* lro */ || c == 0x202E /* rlo */ ||
+      c == 0x202C /* pdf */)
+    return ' ';
+    else if (c == 0x25 || c == 0xFF05 /* % */ ||
+	     c == 0x2B || c == 0xFF0B  /* + */ ||
+	     c == 0x2D || c == 0xFF0D  /* - */)
+      return ' ';
+  else if (c == 0x2F || c == 0xFF0F)
+    return ' '; /* a slash */
+  else if ((c >= 0x30 && c <= 0x39) || (c >= 0xFF10 && c < 0xFF1A))
+    return 'D'; /* a digit */
+  else if (c <= 0x32 || (c >= 0xFF01 && c < 0xFF20))
+    return ' '; /* a punctuation */
+  else if (c >= 0x370 && c < 0x400)
+    return 'L'; /* Greek */
+  else if (c >= 0x2AF && c < 0x45F)
+    return 'L'; /* Cyrillic */
+  else if (c >= 0x45F && c < 0x600)
+    return 'H'; /* Hebrew */
+  else if (c >= 0x600 && c < 0x700)
+    return 'A'; /* Arabic */
+  else if (c >= 0x2E80 && c < 0xA000)
+    return 'Z'; /* CJK */
+  else if (c >= 0xFF65 && c < 0xFFDD)
+    return 'Z'; /* CJK */
+  else
+    return 'L'; /* Latin */
+}
+
+/*----------------------------------------------------------------------
+   TtaGetScript
+
+   Returns the script of a language languageId.
+   Return value:
+   A character that identifies the script
+   'L' = Latin, 'G' = Greek 'A' = Arabic, etc.
+  ----------------------------------------------------------------------*/
+char TtaGetScript (Language languageId)
 {
    int                 i;
 
-   i = (int) languageId;
-   /* Verification of the parameter */
-   if (i >= FreeEntry || i < 0)
+   i = (int) languageId - FirstUserLang;
+   if (i < 0)
      {
-	TtaError (ERR_language_not_found);
-	return (EOS);
+      switch (languageId)
+	{
+	case Greek_Script:
+	  return 'G';
+	case  Arabic_Script:
+	  return 'A';
+	case  Hebrew_Script:
+	  return 'H';
+	case Cyrillic_Script:
+	  return 'C';
+	case CJK_Script:
+	  return 'Z';
+	default:
+	  return 'L';
+	}
+      return languageId;
      }
-   return LangTable[i].LangAlphabet;
+   else if (i >= FreeEntry)
+     /* undeclared language */
+     return 'L';
+   return LangTable[i].LangScript;
 }
 
 
 /*----------------------------------------------------------------------
-   TtaGetLanguageName
+  TtaGetLanguageName
 
-   Not available for TYPO languages. Returns the name of a given language.
-   Parameters:
-   languageId: identifier of the language.
-   Return value:
-   the name of the language.
+  Returns the name of a given language.
+  languageId: identifier of the language.
+  Return value:
+  the name of the language.
   ----------------------------------------------------------------------*/
 char *TtaGetLanguageName (Language languageId)
 {
    int                 i;
 
-   i = (int) languageId;
-   if (i >= FreeEntry || i < 0)
-     {
-	TtaError (ERR_language_not_found);
-	Langbuffer[0] = EOS;
-     }
+   i = (int) languageId - FirstUserLang;
+   if (i < 0 || i >= FreeEntry)
+     /* undeclared language */
+     Langbuffer1[0] = EOS;
    else
-      strcpy (Langbuffer, LangTable[i].LangName);
-   return Langbuffer;
+      strcpy (Langbuffer1, LangTable[i].LangName);
+   return Langbuffer1;
 }
 
 
@@ -718,7 +767,6 @@ char *TtaGetLanguageName (Language languageId)
 
    Not available for TYPO languages.
    Returns the RFC-1766 code of a given language.
-   Parameters:
    languageId: identifier of the language.
    Return value:
    the code of the language.
@@ -727,15 +775,13 @@ char *TtaGetLanguageCode (Language languageId)
 {
   int                 i;
 
-  i = (int) languageId;
-  if (i >= FreeEntry || i < 0)
-    {
-      TtaError (ERR_language_not_found);
-      Langbuffer[0] = EOS;
-    }
+  i = (int) languageId - FirstUserLang;
+   if (i < 0 || i >= FreeEntry)
+     /* undeclared language */
+     Langbuffer2[0] = EOS;
   else
-    strcpy (Langbuffer, LangTable[i].LangCode);
-  return Langbuffer;
+    strcpy (Langbuffer2, LangTable[i].LangCode);
+  return Langbuffer2;
 }
 
 
@@ -743,9 +789,7 @@ char *TtaGetLanguageCode (Language languageId)
    TtaGetNumberOfLanguages
 
    Not available for TYPO languages.
-
    Returns the current number of languages known by Thot.
-
    Return value:
    the current number of languages.
   ----------------------------------------------------------------------*/
@@ -759,9 +803,7 @@ int TtaGetNumberOfLanguages ()
    TtaGetFirstUserLanguage
 
    Not available for TYPO languages.
-
    Returns the first user language known by Thot.
-
    Return value:
    the first user language number.
   ----------------------------------------------------------------------*/
@@ -776,7 +818,7 @@ int TtaGetFirstUserLanguage ()
   appropriate structure.
   Return 0 if problem else returns 1 
   ----------------------------------------------------------------------*/
-ThotBool GetPatternList (Language langageId)
+ThotBool GetPatternList (Language langId)
 {
    FILE               *in;
    unsigned char       patternGot[MAX_LET_PATTERN];
@@ -795,7 +837,7 @@ ThotBool GetPatternList (Language langageId)
 
    strcpy (patternFileName, dictPath);
    strcat (patternFileName, DIR_STR);
-   lang = (int) langageId;
+   lang = (int) langId - FirstUserLang;
    ptPattern = LangTable[lang].LangPattern;
    strcat (patternFileName, ptPattern);
    if ((in = fopen (patternFileName, "r")) == NULL)
@@ -828,53 +870,51 @@ ThotBool GetPatternList (Language langageId)
   FoundPatternInList verifies if a string belongs to the pattern list.
   if true, it returns 1 else 0 
   ----------------------------------------------------------------------*/
-static char *FoundPatternInList (Language langageId,
+static char *FoundPatternInList (Language langId,
 				 unsigned char substring[MAX_LET_PATTERN])
 {
-   int                 language;
-   int                 lgstring;
-   int                 i, max;
-   struct PatternList *ptrTabPattern;
+  int                 lgstring;
+  int                 i, max;
+  struct PatternList *ptrTabPattern;
 
-   language = (int) langageId;
-   lgstring = strlen (substring);
-   if (lgstring >= MAX_LET_PATTERN)
-     return (NULL);
-   else
-     {
-       ptrTabPattern = &LangTable[language].LangTabPattern;
-       i = ptrTabPattern->ind_pattern[lgstring];
-       if (i == 0)
-	 return (NULL);
-       else
-	 {
-	   /* search last index */
-	   max = lgstring + 1;
-	   if (max == MAX_LET_PATTERN)
-	     max = ptrTabPattern->NbPatt;
-	   else
-	     {
-	       max = ptrTabPattern->ind_pattern[max];
-	       if (max == 0)
-		 max = ptrTabPattern->NbPatt;
-	     }
-	   while (i < max)
-	     {
-	       if (!strcmp (ptrTabPattern->liste_pattern[i].CarPattern, substring))
-		 return (ptrTabPattern->liste_pattern[i].PoidsPattern);
-	       i++;
-	     }
-	   return (NULL);
-	 }
-     }
+  lgstring = strlen (substring);
+  if (langId < 0 || lgstring >= MAX_LET_PATTERN)
+    return (NULL);
+  else
+    {
+      ptrTabPattern = &LangTable[langId].LangTabPattern;
+      i = ptrTabPattern->ind_pattern[lgstring];
+      if (i == 0)
+	return (NULL);
+      else
+	{
+	  /* search last index */
+	  max = lgstring + 1;
+	  if (max == MAX_LET_PATTERN)
+	    max = ptrTabPattern->NbPatt;
+	  else
+	    {
+	      max = ptrTabPattern->ind_pattern[max];
+	      if (max == 0)
+		max = ptrTabPattern->NbPatt;
+	    }
+	  while (i < max)
+	    {
+	      if (!strcmp (ptrTabPattern->liste_pattern[i].CarPattern, substring))
+		return (ptrTabPattern->liste_pattern[i].PoidsPattern);
+	      i++;
+	    }
+	  return (NULL);
+	}
+    }
 }
 
 
 /*----------------------------------------------------------------------
-   * FoundHyphenPoints: apply Liang algo. on a word and returns the 
-   * hypen points.
+  FoundHyphenPoints: apply Liang algo. on a word and returns the 
+  hypen points.
   ----------------------------------------------------------------------*/
-static void FoundHyphenPoints (Language langageId, char wordToCut[THOT_MAX_CHAR])
+static void FoundHyphenPoints (Language langId, char wordToCut[THOT_MAX_CHAR])
 {
    int                 lang;
    unsigned char       wordToTreat[THOT_MAX_CHAR];	/* "." + wordToCut + "." */
@@ -886,9 +926,9 @@ static void FoundHyphenPoints (Language langageId, char wordToCut[THOT_MAX_CHAR]
    int                 currentPosition;
    int                 i, j;
 
-   lang = (int) langageId;
+   lang = (int) langId - FirstUserLang;
    wordLength = strlen (wordToCut) + 2;
-   if (wordLength > THOT_MAX_CHAR)
+   if (lang < 0 || wordLength > THOT_MAX_CHAR)
      return;
 
    for (i = 0; i < THOT_MAX_CHAR; i++)
@@ -925,17 +965,17 @@ static void FoundHyphenPoints (Language langageId, char wordToCut[THOT_MAX_CHAR]
 
 
 /*----------------------------------------------------------------------
- TtaGetPatternHyphenList 
-   returns a pointer on the list of values representing the hyphen points
-   or NULL 
+  TtaGetPatternHyphenList 
+  returns a pointer on the list of values representing the hyphen points
+  or NULL 
   ----------------------------------------------------------------------*/
 int *TtaGetPatternHyphenList (char word[THOT_MAX_CHAR], Language languageId)
 {
    int                 language;
    int                 i;
 
-   language = (int) languageId;
-   if (word[0] == EOS)
+   language = (int) languageId - FirstUserLang;
+   if (language < 0 || word[0] == EOS)
       return (NULL);
    if (strlen (word) < 2)
       return (NULL);
@@ -957,16 +997,18 @@ int *TtaGetPatternHyphenList (char word[THOT_MAX_CHAR], Language languageId)
 }
 
 /*----------------------------------------------------------------------
- * TtaExistPatternList verifies if a list of patterns is defined
- * for a given language
+  TtaExistPatternList verifies if a list of patterns is defined
+  for a given language
   ----------------------------------------------------------------------*/
-ThotBool            TtaExistPatternList (Language languageId)
+ThotBool TtaExistPatternList (Language languageId)
 {
-   int                 language;
-
-   language = (int) languageId;
-   if (LangTable[language].LangPattern[0] != EOS)
-      return TRUE;
-   else
-      return FALSE;
+  int                 language;
+  
+  language = (int) languageId - FirstUserLang;
+  if (language < 0)
+    return FALSE;
+  else if (LangTable[language].LangPattern[0] != EOS)
+    return TRUE;
+  else
+    return FALSE;
 }
