@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -49,6 +49,12 @@ static PRuleType    ReadrdTypeRegle (BinFile file)
 	    case C_PR_FUNCTION:
 	       return PtFunction;
 	       break;
+	    case C_PR_VERTOVERFLOW:
+	       return PtVertOverflow;
+	       break;
+	    case C_PR_HORIZOVERFLOW:
+	       return PtHorizOverflow;
+	       break;
 	    case C_PR_VREF:
 	       return PtVertRef;
 	       break;
@@ -103,12 +109,6 @@ static PRuleType    ReadrdTypeRegle (BinFile file)
             case C_PR_BORDERLEFTWIDTH:
                return PtBorderLeftWidth;
                break;
-            case C_PR_XRADIUS:
-	       return PtXRadius;
-               break;
-            case C_PR_YRADIUS:
-	       return PtYRadius;
-               break;
             case C_PR_BORDERTOPCOLOR:
                return PtBorderTopColor;
                break;
@@ -142,35 +142,32 @@ static PRuleType    ReadrdTypeRegle (BinFile file)
 	    case C_PR_WEIGHT:
 	       return PtWeight;
 	       break;
+	    case C_PR_FONT:
+	       return PtFont;
+	       break;
 	    case C_PR_UNDERLINE:
 	       return PtUnderline;
 	       break;
 	    case C_PR_UNDER_THICK:
 	       return PtThickness;
 	       break;
-	    case C_PR_FONT:
-	       return PtFont;
-	       break;
-	    case C_PR_BREAK1:
-	       return PtBreak1;
-	       break;
-	    case C_PR_BREAK2:
-	       return PtBreak2;
-	       break;
 	    case C_PR_INDENT:
 	       return PtIndent;
 	       break;
 	    case C_PR_LINESPACING:
 	       return PtLineSpacing;
 	       break;
+	    case C_PR_DEPTH:
+	       return PtDepth;
+	       break;
 	    case C_PR_ADJUST:
 	       return PtAdjust;
 	       break;
-	    case C_PR_HYPHENATE:
-	       return PtHyphenate;
+	    case C_PR_DIRECTION:
+	       return PtDirection;
 	       break;
-	    case C_PR_DEPTH:
-	       return PtDepth;
+	    case C_PR_UNICODEBIDI:
+	       return PtUnicodeBidi;
 	       break;
 	    case C_PR_LINESTYLE:
 	       return PtLineStyle;
@@ -187,14 +184,8 @@ static PRuleType    ReadrdTypeRegle (BinFile file)
 	    case C_PR_FOREGROUND:
 	       return PtForeground;
 	       break;
-	    case C_PR_VERTOVERFLOW:
-	       return PtVertOverflow;
-	       break;
-	    case C_PR_HORIZOVERFLOW:
-	       return PtHorizOverflow;
-	       break;
-	    case C_PR_GATHER:
-	       return PtGather;
+	    case C_PR_HYPHENATE:
+	       return PtHyphenate;
 	       break;
 	    case C_PR_PAGEBREAK:
 	       return PtPageBreak;
@@ -202,6 +193,24 @@ static PRuleType    ReadrdTypeRegle (BinFile file)
 	    case C_PR_LINEBREAK:
 	       return PtLineBreak;
 	       break;
+	    case C_PR_GATHER:
+	       return PtGather;
+	       break;
+            case C_PR_XRADIUS:
+	       return PtXRadius;
+               break;
+            case C_PR_YRADIUS:
+	       return PtYRadius;
+               break;
+	    case C_PR_DISPLAY:
+	       return PtDisplay;
+	       break;
+	    case C_PR_BREAK1:
+	       return PtBreak1;
+	       break;
+	    case C_PR_BREAK2:
+	       return PtBreak2;
+	       break;
 	    default:
 	       error = True;
 	       return PtVisibility;
@@ -536,6 +545,9 @@ static PresCondition ReadPresCondition (BinFile file)
 	    case C_COND_HAS_ATTR:
 	       condtype = PcAttribute;
 	       break;
+	    case C_COND_INHERIT_ATTR:
+	       condtype = PcInheritAttribute;
+	       break;
 	    case C_COND_NOCOND:
 	       condtype = PcNoCondition;
 	       break;
@@ -1011,7 +1023,8 @@ AttrComparType      ReadAttrCompar (BinFile file)
    	ReadPRules    lit une suite de regles chainees et fait pointer	
    	le pointeur pPRule sur la premiere regle de la suite de regles lues	
   ----------------------------------------------------------------------*/
-void      ReadPRules (BinFile file, PtrPRule *pPRule, PtrPRule *pNextPRule)
+static void ReadPRules (BinFile file, PtrPRule *pPRule, PtrPRule *pNextPRule,
+		        PtrSSchema pSS)
 {
    PtrPRule            pPR;
    PtrCondition        pCond;
@@ -1073,8 +1086,23 @@ void      ReadPRules (BinFile file, PtrPRule *pPRule, PtrPRule *pNextPRule)
 				}
 			      break;
 			   case PcElemType:
+			      TtaReadSignedShort (file, &pCond->CoTypeElem);
+			      break;
 			   case PcAttribute:
-			      TtaReadSignedShort (file, &pCond->CoTypeElAttr);
+			   case PcInheritAttribute:
+			      TtaReadSignedShort (file, &pCond->CoTypeAttr);
+			      TtaReadBool (file, &pCond->CoTestAttrValue);
+			      if (pCond->CoTestAttrValue)
+				{
+				  if (pSS->SsAttribute->TtAttr[pCond->CoTypeAttr]->AttrType == AtTextAttr)
+				    {
+				      TtaReadName (file, pCond->CoAttrTextValue);
+				      pCond->CoTextMatch = CoMatch;
+				    }
+				  else
+				    TtaReadSignedShort (file,
+							&pCond->CoAttrValue);
+				} 
 			      break;
 			   default:
 			      break; 
@@ -1082,6 +1110,7 @@ void      ReadPRules (BinFile file, PtrPRule *pPRule, PtrPRule *pNextPRule)
 		  typeCond = ReadPresCondition (file);
 	       }
 	     TtaReadShort (file, &pPR->PrViewNum);
+	     TtaReadBool (file, &pPR->PrDuplicate);
 	     pPR->PrPresMode = ReadPresMode (file);
 	     if (!error)
 		switch (pPR->PrPresMode)
@@ -1140,7 +1169,10 @@ void      ReadPRules (BinFile file, PtrPRule *pPRule, PtrPRule *pNextPRule)
 				     case PtWeight:
 				     case PtUnderline:
 				     case PtThickness:
+				     case PtDirection:
+				     case PtUnicodeBidi:
 				     case PtLineStyle:
+				     case PtDisplay:
 				     case PtBorderTopStyle:
 			             case PtBorderRightStyle:
 			             case PtBorderBottomStyle:
@@ -1274,6 +1306,8 @@ PtrPSchema      ReadPresentationSchema (Name fileName, PtrSSchema pSS)
       /* supprime le suffixe .PRS a la fin du nom de fichier */
       buf[i - 4] = EOS;
       GetSchPres (&pPSch);
+      /* this schema is considered as the User Agent default stylesheet */
+      pPSch->PsOrigin = Agent;
       /* acquiert un bloc pour la prochaine regle lue */
       GetPresentRule (&pNextPRule);
       pNextPRule->PrCond = NULL;
@@ -1398,7 +1432,7 @@ PtrPSchema      ReadPresentationSchema (Name fileName, PtrSSchema pSS)
 	      {
 		pConst = &pPSch->PsConstant[i];
 		pConst->PdType = ReadBasicType (file);
-		if (!TtaReadByte (file, &pConst->PdAlphabet))
+		if (!TtaReadByte (file, &pConst->PdScript))
 		  error = True;
 		j = 0;
 		if (!error)
@@ -1566,13 +1600,13 @@ PtrPSchema      ReadPresentationSchema (Name fileName, PtrSSchema pSS)
 	  /* lit toutes les regles de presentation */
 	  /* lit les regles standard */
 	  if (!error)
-	    ReadPRules (file, &pPSch->PsFirstDefaultPRule, &pNextPRule);
+	    ReadPRules (file, &pPSch->PsFirstDefaultPRule, &pNextPRule, pSS);
 
 	  /* les regles des boites */
 	  if (!error)
 	    for (i = 0; i < pPSch->PsNPresentBoxes; i++)
 	      ReadPRules (file, &pPSch->PsPresentBox->PresBox[i]->PbFirstPRule,
-			  &pNextPRule);
+			  &pNextPRule, pSS);
 
 	  /* lit les regles des attributs */
 	  if (!error)
@@ -1589,20 +1623,20 @@ PtrPSchema      ReadPresentationSchema (Name fileName, PtrSSchema pSS)
 			  case AtNumAttr:
 			    for (j = 0; j < pAttrP->ApNCases; j++)
 			      ReadPRules (file,&pAttrP->ApCase[j].CaFirstPRule,
-					  &pNextPRule);
+					  &pNextPRule, pSS);
 			    break;
 			  case AtReferenceAttr:
 			    ReadPRules (file, &pAttrP->ApRefFirstPRule,
-					&pNextPRule);
+					&pNextPRule, pSS);
 			    break;
 			  case AtTextAttr:
 			    ReadPRules (file, &pAttrP->ApTextFirstPRule,
-					&pNextPRule);
+					&pNextPRule, pSS);
 			    break;
 			  case AtEnumAttr:
 			    for (j = 0; j <= pSS->SsAttribute->TtAttr[i]->AttrNEnumValues; j++)
 			      ReadPRules (file, &pAttrP->ApEnumFirstPRule[j],
-					  &pNextPRule);
+					  &pNextPRule, pSS);
 			    break;
 			  }
 		    }
@@ -1611,7 +1645,8 @@ PtrPSchema      ReadPresentationSchema (Name fileName, PtrSSchema pSS)
 	  /* lit les regles des elements structures */
 	  if (!error)
 	    for (i = 0; i < InitialNElems; i++)
-	      ReadPRules (file, &pPSch->PsElemPRule->ElemPres[i], &pNextPRule);
+	      ReadPRules (file, &pPSch->PsElemPRule->ElemPres[i], &pNextPRule,
+			  pSS);
 
 	  if (!error)
 	    {
