@@ -84,7 +84,7 @@ static TransCondition ReadTransCondition (BinFile file)
 		  cond = TcondPRule;
 		  break;
 	       case C_TR_ALPHABET:
-		  cond = TcondAlphabet;
+		  cond = TcondScript;
 		  break;
 	       case C_TR_ATTRIBUTES:
 		  cond = TcondAttributes;
@@ -685,8 +685,8 @@ static void ReadBlocks (BinFile file, PtrTRuleBlock *pBlock, PtrTRule *pNextTRul
 		  TtaReadSignedShort (file, &pCond->TcAscendRelLevel);
 		  switch (pCond->TcCondition)
 			{
-			   case TcondAlphabet:
-			      TtaReadByte (file, &pCond->TcAlphabet);
+			   case TcondScript:
+			      TtaReadByte (file, &pCond->TcScript);
 			      break;
 			   case TcondWithin:
 			   case TcondFirstWithin:
@@ -699,7 +699,7 @@ static void ReadBlocks (BinFile file, PtrTRuleBlock *pBlock, PtrTRule *pNextTRul
 			   case TcondAttr:
 			      TtaReadShort (file, &pCond->TcAttr);
 			      if (!error)
-				 switch ((*pSS)->SsAttribute[pCond->TcAttr - 1].AttrType)
+				 switch ((*pSS)->SsAttribute->TtAttr[pCond->TcAttr - 1]->AttrType)
 				       {
 					  case AtNumAttr:
 					     TtaReadSignedShort (file, &pCond->TcLowerBound);
@@ -789,15 +789,15 @@ static void ReadAttrTRules (BinFile file, int att, PtrTRuleBlock *pNextBlock,
 			    PtrTRule *pNextTRule, PtrSSchema *pSS,
 			    PtrTSchema *pTSch)
 {
-   AttributeTransl    *pAttrT;
+   PtrAttributeTransl  pAttrT;
    TranslNumAttrCase  *pCase;
    int                 i;
 
    if (!error)
      {
-	pAttrT = &(*pTSch)->TsAttrTRule[att];
+	pAttrT = (*pTSch)->TsAttrTRule->TsAttrTransl[att];
 	TtaReadShort (file, &pAttrT->AtrElemType);
-	switch ((*pSS)->SsAttribute[att].AttrType)
+	switch ((*pSS)->SsAttribute->TtAttr[att]->AttrType)
 	      {
 		 case AtNumAttr:
 		    TtaReadShort (file, &pAttrT->AtrNCases);
@@ -821,7 +821,7 @@ static void ReadAttrTRules (BinFile file, int att, PtrTRuleBlock *pNextBlock,
 				pSS, pNextBlock);
 		    break;
 		 case AtEnumAttr:
-		    for (i = 0; i <= (*pSS)->SsAttribute[att].AttrNEnumValues; i++)
+		    for (i = 0; i <= (*pSS)->SsAttribute->TtAttr[att]->AttrNEnumValues; i++)
 		       ReadBlocks (file, &pAttrT->AtrEnuTRuleBlock[i],
 				   pNextTRule, pSS, pNextBlock);
 		    break;
@@ -834,7 +834,7 @@ static void ReadAttrTRules (BinFile file, int att, PtrTRuleBlock *pNextBlock,
 /*----------------------------------------------------------------------
    FreeTRulesAttr libere les regles de traduction d'un attribut.   
   ----------------------------------------------------------------------*/
-static void FreeTRulesAttr (AttributeTransl *pAttrT, TtAttribute *pAttr)
+static void FreeTRulesAttr (PtrAttributeTransl pAttrT, PtrTtAttribute pAttr)
 {
    int                 i;
 
@@ -947,13 +947,13 @@ PtrTSchema ReadTranslationSchema (Name fileName, PtrSSchema pSS)
    TCounter           *pCntr;
    TranslVariable     *pVar;
    TranslVarItem      *pVarItem;
-   AttributeTransl    *pAttrTr;
-   AlphabetTransl     *pAlphTr;
+   PtrAttributeTransl  pAttrTr;
+   ScriptTransl     *pAlphTr;
    StringTransl       *pStringTr;
    PRuleTransl        *pPRuleTr;
    PathBuffer          dirBuffer;
    char                buf[MAX_TXT_LEN];
-   int                 InitialNElems, i, j;
+   int                 InitialNElems, i, j, size;
 
    error = FALSE;
    pTSch = NULL;
@@ -965,9 +965,10 @@ PtrTSchema ReadTranslationSchema (Name fileName, PtrSSchema pSS)
    file = TtaReadOpen (buf);
    if (file == 0)
      {
-	strncpy (buf, fileName, MAX_PATH);
-	strcat (buf, ".TRA");
-	TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_TRA_FILE_INCORRECT), buf);
+       strncpy (buf, fileName, MAX_PATH);
+       strcat (buf, ".TRA");
+       TtaDisplayMessage (INFO, TtaGetMessage (LIB, TMSG_TRA_FILE_INCORRECT),
+			  buf);
      }
    else
      {
@@ -977,6 +978,24 @@ PtrTSchema ReadTranslationSchema (Name fileName, PtrSSchema pSS)
 	     TSchemaError (20);
 	     return NULL;
 	  }
+	/* allocate the element translation table */
+	size = pSS->SsNRules * sizeof (PtrTRuleBlock);
+	pTSch->TsElemTRule = (ElemTransTable*) TtaGetMemory (size);
+	if (pTSch->TsElemTRule)
+	  memset (pTSch->TsElemTRule, 0, size);
+
+	/* allocate the attribute inherit table */
+	size = pSS->SsNRules * sizeof (ThotBool);
+	pTSch->TsInheritAttr = (BlnTable*) TtaGetMemory (size);
+	if (pTSch->TsInheritAttr)
+	  memset (pTSch->TsInheritAttr, 0, size);
+
+	/* allocate the attribute translation table */
+	size = pSS->SsNAttributes * sizeof (PtrAttributeTransl);
+	pTSch->TsAttrTRule = (AttrTransTable*) TtaGetMemory (size);
+	if (pTSch->TsAttrTRule)
+	  memset (pTSch->TsAttrTRule, 0, size);
+
 	GetTRule (&pNextTRule);
 	if (pNextTRule == NULL)
 	  {
@@ -1050,45 +1069,51 @@ PtrTSchema ReadTranslationSchema (Name fileName, PtrSSchema pSS)
 	  {
 	     InitialNElems = pSS->SsFirstDynNature - 1;
 	     for (j = InitialNElems; j < pSS->SsNRules; j++)
-		pTSch->TsElemTRule[j] = NULL;
+		pTSch->TsElemTRule->TsElemTransl[j] = NULL;
 	  }
 	if (!error)
-	   for (i = 0; i < InitialNElems; i++)
-	      pTSch->TsElemTRule[i] = ReadPtrTRuleBlock (file, &pNextBlock);
+	  for (i = 0; i < InitialNElems; i++)
+	    pTSch->TsElemTRule->TsElemTransl[i] =
+	                                ReadPtrTRuleBlock (file, &pNextBlock);
 	if (!error)
-	   for (i = 0; i < InitialNElems; i++)
-	      TtaReadBool (file, &pTSch->TsInheritAttr[i]);
+	  for (i = 0; i < InitialNElems; i++)
+	    TtaReadBool (file, &pTSch->TsInheritAttr->Bln[i]);
 	if (!error)
-	   for (i = 0; i < pSS->SsNAttributes; i++)
-	      if (!error)
+	  for (i = 0; i < pSS->SsNAttributes && !error; i++)
+	    {
+	      GetAttributeTransl (&pAttrTr);
+	      if (!pAttrTr)
+		TSchemaError (25);
+	      else
 		{
-		   pAttrTr = &pTSch->TsAttrTRule[i];
-		   switch (pSS->SsAttribute[i].AttrType)
-			 {
-			    case AtNumAttr:
-			       TtaReadShort (file, &pAttrTr->AtrNCases);
-			       if (!error)
-				  for (j = 0; j < pAttrTr->AtrNCases; j++)
-				     pAttrTr->AtrCase[j].TaTRuleBlock =
-					ReadPtrTRuleBlock (file, &pNextBlock);
-			       break;
-			    case AtTextAttr:
-			       pAttrTr->AtrTxtTRuleBlock =
-				  ReadPtrTRuleBlock (file, &pNextBlock);
-			       break;
-			    case AtReferenceAttr:
-			       pAttrTr->AtrRefTRuleBlock =
-				  ReadPtrTRuleBlock (file, &pNextBlock);
-			       break;
-			    case AtEnumAttr:
-			       for (j = 0; j <= pSS->SsAttribute[i].AttrNEnumValues; j++)
-				  pAttrTr->AtrEnuTRuleBlock[j] =
-				     ReadPtrTRuleBlock (file, &pNextBlock);
-			       break;
-			    default:
-			       break;
-			 }
+		  pTSch->TsAttrTRule->TsAttrTransl[i] = pAttrTr;
+		  switch (pSS->SsAttribute->TtAttr[i]->AttrType)
+		    {
+		    case AtNumAttr:
+		      TtaReadShort (file, &pAttrTr->AtrNCases);
+		      if (!error)
+			for (j = 0; j < pAttrTr->AtrNCases; j++)
+			  pAttrTr->AtrCase[j].TaTRuleBlock =
+			    ReadPtrTRuleBlock (file, &pNextBlock);
+		      break;
+		    case AtTextAttr:
+		      pAttrTr->AtrTxtTRuleBlock =
+			ReadPtrTRuleBlock (file, &pNextBlock);
+		      break;
+		    case AtReferenceAttr:
+		      pAttrTr->AtrRefTRuleBlock =
+			ReadPtrTRuleBlock (file, &pNextBlock);
+		      break;
+		    case AtEnumAttr:
+		      for (j = 0; j <= pSS->SsAttribute->TtAttr[i]->AttrNEnumValues; j++)
+			pAttrTr->AtrEnuTRuleBlock[j] =
+			  ReadPtrTRuleBlock (file, &pNextBlock);
+		      break;
+		    default:
+		      break;
+		    }
 		}
+	    }
 	if (!error)
 	   for (i = 0; i < MAX_TRANSL_PRULE; i++)
 	      if (!error)
@@ -1114,12 +1139,12 @@ PtrTSchema ReadTranslationSchema (Name fileName, PtrSSchema pSS)
 			       ReadPtrTRuleBlock (file, &pNextBlock);
 		     }
 		}
-	TtaReadShort (file, &pTSch->TsNTranslAlphabets);
+	TtaReadShort (file, &pTSch->TsNTranslScripts);
 	if (!error)
-	   for (i = 0; i < pTSch->TsNTranslAlphabets; i++)
+	   for (i = 0; i < pTSch->TsNTranslScripts; i++)
 	     {
-		pAlphTr = &pTSch->TsTranslAlphabet[i];
-		TtaReadByte (file, &pAlphTr->AlAlphabet);
+		pAlphTr = &pTSch->TsTranslScript[i];
+		TtaReadByte (file, &pAlphTr->AlScript);
 		TtaReadShort (file, &pAlphTr->AlBegin);
 		TtaReadShort (file, &pAlphTr->AlEnd);
 	     }
@@ -1156,8 +1181,8 @@ PtrTSchema ReadTranslationSchema (Name fileName, PtrSSchema pSS)
 	/* lit les blocs de regles des elements */
 	if (!error)
 	   for (i = 0; i < InitialNElems; i++)
-	      ReadBlocks (file, &pTSch->TsElemTRule[i], &pNextTRule, &pSS,
-			  &pNextBlock);
+	      ReadBlocks (file, &pTSch->TsElemTRule->TsElemTransl[i],
+			  &pNextTRule, &pSS, &pNextBlock);
 	/* lit les blocs de regles des attributs */
 	for (i = 0; i < pSS->SsNAttributes; i++)
 	   if (!error)
@@ -1200,13 +1225,32 @@ void FreeTranslationSchema (PtrTSchema pTSch, PtrSSchema pSS)
       InitialNElems = pSS->SsFirstDynNature - 1;
    /* libere les blocs de regles  des elements */
    for (i = 0; i < InitialNElems; i++)
-      FreeBlocks (pTSch->TsElemTRule[i]);
+      FreeBlocks (pTSch->TsElemTRule->TsElemTransl[i]);
    /* libere les blocs de regles des attributs */
    for (i = 0; i < pSS->SsNAttributes; i++)
-      FreeTRulesAttr (&pTSch->TsAttrTRule[i], &pSS->SsAttribute[i]);
+     {
+      FreeTRulesAttr (pTSch->TsAttrTRule->TsAttrTransl[i],
+		      pSS->SsAttribute->TtAttr[i]);
+      FreeAttributeTransl (pTSch->TsAttrTRule->TsAttrTransl[i]);
+     }
    /* libere les blocs de regles des presentations */
    for (i = 0; i < MAX_TRANSL_PRULE; i++)
       FreeTRulesPres (i + 1, &pTSch->TsPresTRule[i]);
+   if (pTSch->TsElemTRule)
+     {
+       TtaFreeMemory (pTSch->TsElemTRule);
+       pTSch->TsElemTRule = NULL;
+     }
+   if (pTSch->TsInheritAttr)
+     {
+       TtaFreeMemory (pTSch->TsInheritAttr);
+       pTSch->TsInheritAttr = NULL;
+     }
+   if (pTSch->TsAttrTRule)
+     {
+       TtaFreeMemory (pTSch->TsAttrTRule);
+       pTSch->TsAttrTRule = NULL;
+     }
    /* libere le schema de traduction lui-meme */
-   TtaFreeMemory ( pTSch);
+   FreeSchTra (pTSch, pSS);
 }
