@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -437,12 +437,15 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	rType == PtBorderRightStyle ||
 	rType == PtBorderBottomStyle ||
 	rType == PtBorderLeftStyle ||
+	rType == PtDisplay ||
+	rType == PtVisibility ||
 	rType == PtSize || rType == PtStyle ||
 	rType == PtWeight || rType == PtFont ||
 	rType == PtUnderline || rType == PtThickness ||
 	rType == PtIndent || rType == PtLineSpacing ||
 	rType == PtDepth ||
 	rType == PtAdjust ||
+	rType == PtDirection || rType == PtUnicodeBidi ||
 	rType == PtLineStyle || rType == PtLineWeight ||
 	rType == PtFillPattern ||
 	rType == PtBackground || rType == PtForeground ||
@@ -515,9 +518,21 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	case PtThickness:
 	  TtaWriteByte (pivFile, C_PR_UNDER_THICK);
 	  break;
+	case PtDirection:
+	  TtaWriteByte (pivFile, C_PR_DIRECTION);
+	  break;
+	case PtUnicodeBidi:
+	  TtaWriteByte (pivFile, C_PR_UNICODEBIDI);
+	  break;
 	case PtLineStyle:
 	  TtaWriteByte (pivFile, C_PR_LINESTYLE);
 	  break;
+	case PtDisplay:
+	  TtaWriteByte (pivFile, C_PR_DISPLAY);
+	  break;
+	case PtVisibility:
+	  TtaWriteByte (pivFile, C_PR_VISIBILITY);
+	  break;
         case PtBorderTopStyle:
 	  TtaWriteByte (pivFile, C_PR_BORDERTOPSTYLE);
 	  break;
@@ -645,7 +660,11 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	case PtHorizPos:
 	  PutShort (pivFile, pPRule->PrPosRule.PoPosDef);
 	  PutShort (pivFile, pPRule->PrPosRule.PoPosRef);
+	  PutBoolean (pivFile, pPRule->PrPosRule.PoDistAttr);
 	  PutShort (pivFile, pPRule->PrPosRule.PoRelation);
+	  PutBoolean (pivFile, pPRule->PrPosRule.PoNotRel);
+	  PutShort (pivFile, pPRule->PrPosRule.PoRefKind);
+	  PutShort (pivFile, pPRule->PrPosRule.PoRefIdent);
 	  PutShort (pivFile, abs (pPRule->PrPosRule.PoDistance));
 	  PutUnit (pivFile, pPRule->PrPosRule.PoDistUnit);
 	  PutSign (pivFile, (ThotBool)(pPRule->PrPosRule.PoDistance >= 0));
@@ -722,7 +741,11 @@ static void   PutPresRule (BinFile pivFile, PtrPRule pPRule)
 	case PtWeight:
 	case PtUnderline:
 	case PtThickness:
+	case PtDirection:
+	case PtUnicodeBidi:
+	case PtVisibility:
 	case PtLineStyle:
+	case PtDisplay:
         case PtBorderTopStyle:
         case PtBorderRightStyle:
         case PtBorderBottomStyle:
