@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2001
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -8,7 +8,7 @@
 /*
  *
  * Authors: V. Quint, I. Vatton (INRIA)
- *          R. Guetari (W3C/INRIA) - Unicode and Windows version
+ *          R. Guetari (W3C/INRIA) Windows version
  *
  */	
 
@@ -67,13 +67,14 @@
 #include "schemas_f.h"
 #include "schtrad_f.h"
 #include "searchmenu_f.h"
+#include "structcommands_f.h"
 #include "structschema_f.h"
 #include "structselect_f.h"
 #include "thotmsg_f.h"
 #include "tree_f.h"
 
 static Proc         AppClosingFunction = NULL;
-#define VersionId TEXT ("V2.1")
+#define VersionId "V3.1"
 ThotBool            PrintErrorMessages = TRUE;
 
 #ifdef _WINDOWS
@@ -82,16 +83,12 @@ ThotBool            PrintErrorMessages = TRUE;
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-int                 IdentDocument (PtrDocument pDoc)
-#else  /* __STDC__ */
-int                 IdentDocument (pDoc)
-PtrDocument         pDoc;
-
-#endif /* __STDC__ */
+int IdentDocument (PtrDocument pDoc)
 {
    int                 d;
 
+   if (!pDoc)
+     return 0;
    d = 1;
    while (LoadedDocument[d - 1] != pDoc && d < MAX_DOCUMENTS)
       d++;
@@ -104,49 +101,13 @@ PtrDocument         pDoc;
 /*----------------------------------------------------------------------
    Termine l'insertion de caracteres dans une boite de texte       
   ----------------------------------------------------------------------*/
-void                CloseInsertion ()
+void CloseInsertion ()
 {
    if (ThotLocalActions[T_stopinsert] != NULL)
       (*ThotLocalActions[T_stopinsert]) ();
 }
 
-
-
 #ifndef NODISPLAY
-/*----------------------------------------------------------------------
-   GetViewInfo returns wiew number and assoc state of the          
-   corresponding to the view of the document.             
-   Parameters:                                                     
-   document: the document.                                 
-   view: the view.                                         
-   Return value:                                                   
-   corresponding view number.                              
-   corresponding assoc state.                              
-  ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                GetViewInfo (Document document, View view, int *viewnumber, ThotBool * assoc)
-#else  /* __STDC__ */
-void                GetViewInfo (document, view, viewnumber, assoc)
-Document            document;
-View                view;
-int                *viewnumber;
-ThotBool           *assoc;
-
-#endif /* __STDC__ */
-{
-
-   *assoc = FALSE;
-   *viewnumber = 0;
-
-   if (view < 100)
-      *viewnumber = (int) view;
-   else
-     {
-	*assoc = TRUE;
-	*viewnumber = (int) view - 100;
-     }
-}
-
 
 /*----------------------------------------------------------------------
    GetWindowNumber returns the window corresponding to the view of 
@@ -157,41 +118,28 @@ ThotBool           *assoc;
    Return value:                                                   
    corresponding window.                                   
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-int                 GetWindowNumber (Document document, View view)
-#else  /* __STDC__ */
-int                 GetWindowNumber (document, view)
-Document            document;
-View                view;
-#endif /* __STDC__ */
+int GetWindowNumber (Document document, View view)
 {
-   PtrDocument         pDoc;
-   ThotBool            assoc;
-   int                 aView, win;
+   int                 win;
 
    win = 0;
-   /* Checks parameters */
+   /* Check parameters */
    if (document < 1 || document > MAX_DOCUMENTS)
       TtaError (ERR_invalid_document_parameter);
    else if (LoadedDocument[document - 1] != NULL)
-      if (view < 1 || (view > MAX_VIEW_DOC && view < 100) || view > MAX_ASSOC_DOC + 100)
+     {
+       if (view < 1 || view > MAX_VIEW_DOC)
          TtaError (ERR_invalid_parameter);
       else
-        {
-	pDoc = LoadedDocument[document - 1];
-	GetViewInfo (document, view, &aView, &assoc);
-	if (assoc)
-	   win = pDoc->DocAssocFrame[aView - 1];
-	else
-	   win = pDoc->DocViewFrame[aView - 1];
-        }
+	win = LoadedDocument[document - 1]->DocViewFrame[view - 1];
+     }
    return win;
 }
 
 /*----------------------------------------------------------------------
    TtaIsPrinting returns TRUE is the application is printing.    
   ----------------------------------------------------------------------*/
-ThotBool         TtaIsPrinting ()
+ThotBool TtaIsPrinting ()
 {
   return (Printing);
 }
@@ -199,13 +147,7 @@ ThotBool         TtaIsPrinting ()
 /*----------------------------------------------------------------------
    TtaGetViewFrame returns the frame widget of the document view.    
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotWidget          TtaGetViewFrame (Document document, View view)
-#else  /* __STDC__ */
-ThotWidget          TtaGetViewFrame (document, view)
-Document            document;
-View                view;
-#endif /* __STDC__ */
+ThotWidget TtaGetViewFrame (Document document, View view)
 {
   int                 frame;
 
@@ -224,10 +166,10 @@ View                view;
       return 0;
     }
   else
-#ifndef _WINDOWS
-    return (FrameTable[frame].WdFrame);
-#else  /* _WINDOWS */
+#ifdef _WINDOWS
     return (FrMainRef[frame]);
+#else  /* _WINDOWS */
+    return (FrameTable[frame].WdFrame);
 #endif /* _WINDOWS */
 }
 #endif
@@ -235,7 +177,7 @@ View                view;
 /*----------------------------------------------------------------------
    CoreHandler est un handler d'erreur fatale.                     
   ----------------------------------------------------------------------*/
-static void         ErrorHandler ()
+static void ErrorHandler ()
 {
 #  ifndef _WINDOWS
    signal (SIGBUS, SIG_DFL);
@@ -247,20 +189,22 @@ static void         ErrorHandler ()
 #else
    signal (SIGIOT, SIG_DFL);
 #endif
-   ufprintf (stderr, TtaGetMessage (LIB, TMSG_DEBUG_ERROR));
-
+   fprintf (stderr, TtaGetMessage (LIB, TMSG_DEBUG_ERROR));
    if (ThotLocalActions [T_backuponfatal] != NULL)
-     {
        (*ThotLocalActions [T_backuponfatal]) ();
-     }
-   exit (1);
+   {
+#if defined(_GTK) && !defined(NODISPLAY)
+     gtk_exit (1);
+#endif /* _GTK && !NODISPLAY */	
+     exit (1);
+   }
 }
 
 
 /*----------------------------------------------------------------------
    QuitHandler est un handler pour Interrupt.                      
   ----------------------------------------------------------------------*/
-static void         QuitHandler ()
+static void QuitHandler ()
 {
    signal (SIGINT, ErrorHandler);
 #  ifndef _WINDOWS 
@@ -269,7 +213,12 @@ static void         QuitHandler ()
    signal (SIGTERM, ErrorHandler);
    if (ThotLocalActions [T_backuponfatal] != NULL)
      (*ThotLocalActions [T_backuponfatal]) ();
-   exit (1);
+   {
+#if defined _GTK && !defined(NODISPLAY)
+     gtk_exit (1);
+#endif /* _GTK && !NODISPLAY */	
+     exit (1);
+   }
 #  ifndef _WINDOWS
    signal (SIGINT, QuitHandler);
    signal (SIGQUIT, QuitHandler);
@@ -280,7 +229,7 @@ static void         QuitHandler ()
 /*----------------------------------------------------------------------
    InitErrorHandler initialise le handler de core dump.             
   ----------------------------------------------------------------------*/
-void                InitErrorHandler ()
+void InitErrorHandler ()
 {
 #  ifndef _WINDOWS
    signal (SIGBUS, ErrorHandler);
@@ -304,30 +253,21 @@ void                InitErrorHandler ()
 
    Initializes the Thot editing tool kit for an application. This function must be
    called before any other function of the tool kit.
-
    Parameter:
    applicationName: the argv[0] of the application that requires services
    from the tool kit.  This name is used for accessing the ressources
    defined in file .Xdefaults.
-
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaInitialize (CHAR_T* applicationName)
-#else  /* __STDC__ */
-void                TtaInitialize (applicationName)
-CHAR_T*             applicationName;
-
-#endif /* __STDC__ */
+void TtaInitialize (char *applicationName)
 {
    int                 i;
 
    UserErrorCode = 0;
-   ustrcpy (DefaultDocumentName, TEXT(""));
-   InitEditorMemory ();		/* Initializes the memory managment of the editor */
-   InitNatures ();		/* Initializes the table of Natures */
-
+   strcpy (DefaultDocumentName, "");
+   InitEditorMemory ();	      /* Initializes the memory managment of the editor */
+   InitNatures ();	      /* Initializes the table of Natures */
    FullStructureChecking = FALSE;
-
+   ShowOnePage = FALSE;
    /* Initializes the table of documents */
    for (i = 0; i < MAX_DOCUMENTS; i++)
       LoadedDocument[i] = NULL;
@@ -342,55 +282,47 @@ CHAR_T*             applicationName;
    InitTranslationSchemasTable ();
    /* Read the environment variables concerning the directories */
    InitLanguage ();
-   /* Initializes the color table */
-   NColors = MAX_COLOR;
-   RGB_Table = RGB_colors;
-   Color_Table = Name_colors;
 
 #ifndef NODISPLAY
-#  ifdef _WINDOWS
+#ifdef _WINDOWS
    TtPrinterDC = NULL;
-   WIN_GetDeviceContext (-1);
-   DOT_PER_INCHE = GetDeviceCaps(TtDisplay, LOGPIXELSY);
-   WIN_ReleaseDeviceContext ();
-   /* DOT_PER_INCHE = 72; */
-#  else  /* !_WINDOWS */
-   DOT_PER_INCHE = 72;
-#  endif /* _WINDOWS */
-
+   WIN_Main_Wd = NULL;
+#endif /* _WINDOWS */
+   DOT_PER_INCH = 72;
    numOfJobs = 0;
    /* Initializes patterns */
-   NbPatterns = sizeof (Name_patterns) / sizeof (STRING);
-
+   NbPatterns = sizeof (Name_patterns) / sizeof (char *);
    Patterns = Name_patterns;
 #endif /* NODISPLAY */
    /* load the message table of the Thot Library */
-   i = TtaGetMessageTable (TEXT("libdialogue"), TMSG_LIB_MSG_MAX);
+   i = TtaGetMessageTable ("libdialogue", TMSG_LIB_MSG_MAX);
    switch (i)
-	 {
-	    case 0:
-	       break;
-	    case -1:
-	       fprintf (stderr, "cannot find messages table\n");
-	       exit (1);
-	    default:
-	       fprintf (stderr, "Previous messages table loaded\n");
-	       exit (1);
-	 }
+     {
+     case 0:
+       break;
+     case -1:
+       fprintf (stderr, "cannot find messages table\n");
+#if defined _GTK && !defined(NODISPLAY)
+	gtk_exit (1);
+#endif /* _GTK && !NODISPLAY */	
+       exit (1);
+     default:
+       fprintf (stderr, "Previous messages table loaded\n");
+#if defined _GTK && !defined(NODISPLAY)
+	gtk_exit (1);
+#endif /* _GTK && !NODISPLAY */	
+       exit (1);
+     }
 
    /* init the system error signal handler */
    InitErrorHandler ();
-
 #ifndef NODISPLAY
-
    ConfigInit ();
    /* Initilizes the space character displaying mode */
    ShowSpace = 1;
    InputSpace = 0;
 #endif
-
    CheckAccessLoadResources ();
-
 }
 
 /*----------------------------------------------------------------------
@@ -398,12 +330,7 @@ CHAR_T*             applicationName;
    just before the application will be closed.
    That procedure has no parameters.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaSetApplicationQuit (Proc procedure)
-#else  /* __STDC__ */
-void                TtaSetApplicationQuit (procedure)
-Proc                procedure;
-#endif /* __STDC__ */
+void TtaSetApplicationQuit (Proc procedure)
 {
   AppClosingFunction = procedure;
 }
@@ -414,9 +341,8 @@ Proc                procedure;
 
    Quit the Thot tool kit. No other function of the tool kit can then
    be called by the application.
-
   ----------------------------------------------------------------------*/
-void                TtaQuit ()
+void TtaQuit ()
 {
   PtrDocument       pDoc;
   NotifyDialog      notifyDoc;
@@ -439,6 +365,12 @@ void                TtaQuit ()
 	LoadedDocument[d] = NULL;
       }
 #ifndef NODISPLAY
+  /* remove the contents of the cut buffer related to the document */
+  if (ThotLocalActions[T_freesavedel])
+    (*ThotLocalActions[T_freesavedel]) ();
+  /* free all context related to search commands */
+  if (ThotLocalActions[T_freesearch])
+    (*ThotLocalActions[T_freesearch]) ();
   FreeDocColors ();
   FreeAllMessages ();
   Prof_FreeTable ();
@@ -453,24 +385,22 @@ void                TtaQuit ()
   TtaFreeAppRegistry ();
   if (AppClosingFunction)
     (*AppClosingFunction) ();
-  exit (0);
+  {
+#if defined _GTK && !defined(NODISPLAY)
+    gtk_exit (0);
+#endif /* _GTK && !defined(NODISPLAY) */	
+    exit (0);
+  }
 }
 
 /*----------------------------------------------------------------------
    TtaSetErrorMessages
 
    Indicates to the tool kit whether error messages must be printed or not.
-
    Parameter:
    on: 1 if error messages must be printed, 0 if not.
-
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaSetErrorMessages (int on)
-#else  /* __STDC__ */
-void                TtaSetErrorMessages (on)
-int                 on;
-#endif /* __STDC__ */
+void TtaSetErrorMessages (int on)
 {
    PrintErrorMessages = (ThotBool) on;
 }
@@ -479,12 +409,10 @@ int                 on;
    TtaGetVersion
 
    Returns the identifier of the current version of the Thot editing tool kit.
-
    Return value:
    identifier of the current version.
-
   ----------------------------------------------------------------------*/
-STRING              TtaGetVersion ()
+char *TtaGetVersion ()
 {
    UserErrorCode = 0;
    return (VersionId);
@@ -495,12 +423,10 @@ STRING              TtaGetVersion ()
 
    Returns the error code set by the last call to the Thot editing tool kit.
    See file application.h for the possible values.
-
    Return value:
    last error code, 0 if the last call was successful.
-
   ----------------------------------------------------------------------*/
-int                 TtaGetErrorCode ()
+int TtaGetErrorCode ()
 {
    return UserErrorCode;
 }
@@ -509,129 +435,117 @@ int                 TtaGetErrorCode ()
    TtaGetStrError
 
    Returns a pointer to the message text for a given error code.
-
    Parameter:
    errorCode: an error code.
-
    Return value:
    No return value
-
    See also:
    TtaGetErrorCode
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-STRING              TtaGetStrError (int errorCode)
-#else  /* __STDC__ */
-STRING              TtaGetStrError (errorCode)
-int                 errorCode;
-
-#endif /* __STDC__ */
+char *TtaGetStrError (int errorCode)
 {
-   STRING              strError = NULL;
+   char             *strError = NULL;
 
    switch (errorCode)
 	 {
 	    case ERR_document_name:
-	       strError = TEXT("invalid document name");
+	       strError = "invalid document name";
 	       break;
 	    case ERR_too_many_documents:
-	       strError = TEXT("too many documents");
+	       strError = "too many documents";
 	       break;
 	    case ERR_cannot_read_struct_schema:
-	       strError = TEXT("cannot read structure schema");
+	       strError = "cannot read structure schema";
 	       break;
 	    case ERR_empty_document:
-	       strError = TEXT("empty document");
+	       strError = "empty document";
 	       break;
 	    case ERR_cannot_open_pivot_file:
-	       strError = TEXT("cannot open pivot file");
+	       strError = "cannot open pivot file";
 	       break;
 	    case ERR_invalid_document_parameter:
-	       strError = TEXT("invalid document parameter");
+	       strError = "invalid document parameter";
 	       break;
 	    case ERR_read_only_document:
-	       strError = TEXT("read only document");
-	       break;
-	    case ERR_invalid_associated_root:
-	       strError = TEXT("invalid associated root");
+	       strError = "read only document";
 	       break;
 	    case ERR_invalid_parameter:
-	       strError = TEXT("invalid parameter");
+	       strError = "invalid parameter";
 	       break;
 	    case ERR_not_implemented:
-	       strError = TEXT("feature not implemented");
+	       strError = "feature not implemented";
 	       break;
 	    case ERR_invalid_attribute_value:
-	       strError = TEXT("invalid attribute value");
+	       strError = "invalid attribute value";
 	       break;
 	    case ERR_attribute_element_mismatch:
-	       strError = TEXT("attribute element mismatch");
+	       strError = "attribute element mismatch";
 	       break;
 	    case ERR_buffer_too_small:
-	       strError = TEXT("buffer too small");
+	       strError = "buffer too small";
 	       break;
 	    case ERR_invalid_element_type:
-	       strError = TEXT("invalid element type");
+	       strError = "invalid element type";
 	       break;
 	    case ERR_element_does_not_match_DTD:
-	       strError = TEXT("element does not match DTD");
+	       strError = "element does not match DTD";
 	       break;
 	    case ERR_invalid_attribute_type:
-	       strError = TEXT("invalid attribute type");
+	       strError = "invalid attribute type";
 	       break;
 	    case ERR_duplicate_attribute:
-	       strError = TEXT("duplicate attribute");
+	       strError = "duplicate attribute";
 	       break;
 	    case ERR_mandatory_attribute:
-	       strError = TEXT("mandatory attribute");
+	       strError = "mandatory attribute";
 	       break;
 	    case ERR_cannot_set_link:
-	       strError = TEXT("cannot set link");
+	       strError = "cannot set link";
 	       break;
 	    case ERR_no_selection_in_document:
-	       strError = TEXT("no selection in document");
+	       strError = "no selection in document";
 	       break;
 	    case ERR_incorrect_tree:
-	       strError = TEXT("incorrect tree");
+	       strError = "incorrect tree";
 	       break;
 	    case ERR_cannot_open_view:
-	       strError = TEXT("cannot open view");
+	       strError = "cannot open view";
 	       break;
 	    case ERR_there_are_open_views:
-	       strError = TEXT("there are open views");
+	       strError = "there are open views";
 	       break;
 	    case ERR_cannot_load_pschema:
-	       strError = TEXT("cannot load pschema");
+	       strError = "cannot load pschema";
 	       break;
 	    case ERR_duplicate_presentation_rule:
-	       strError = TEXT("duplicate presentation rule");
+	       strError = "duplicate presentation rule";
 	       break;
 	    case ERR_string_too_long:
-	       strError = TEXT("string too long");
+	       strError = "string too long";
 	       break;
 	    case ERR_cannot_holophrast_a_root:
-	       strError = TEXT("cannot holophrast a root");
+	       strError = "cannot holophrast a root";
 	       break;
 	    case ERR_cannot_holophrast_that_type:
-	       strError = TEXT("cannot holophrast that type");
+	       strError = "cannot holophrast that type";
 	       break;
 	    case ERR_main_window_not_open:
-	       strError = TEXT("main window not open");
+	       strError = "main window not open";
 	       break;
 	    case ERR_element_already_inserted:
-	       strError = TEXT("element already inserted");
+	       strError = "element already inserted";
 	       break;
 	    case ERR_too_many_languages:
-	       strError = TEXT("language table is full");
+	       strError = "language table is full";
 	       break;
 	    case ERR_language_not_found:
-	       strError = TEXT("language not found");
+	       strError = "language not found";
 	       break;
 	    case ERR_no_presentation_schema:
-	       strError = TEXT("no presentation schema");
+	       strError = "no presentation schema";
 	       break;
 	    case ERR_element_has_no_box:
-	       strError = TEXT("element has no box");
+	       strError = "element has no box";
 	       break;
 	    default:
 	       strError = NULL;
@@ -644,20 +558,18 @@ int                 errorCode;
 /*----------------------------------------------------------------------
    ThotExit termine l'application Thot.                             
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                ThotExit (int result)
-#else  /* __STDC__ */
-void                ThotExit (result)
-int                 result;
-
-#endif /* __STDC__ */
+void ThotExit (int result)
 {
    fflush (stderr);
    fflush (stdout);
    if (result)
       ErrorHandler ();
    else
-      exit (result);
+     {
+#if defined _GTK && !defined(NODISPLAY)
+       gtk_exit (result);
+#endif /* _GTK && !NODISPLAY */
+       exit (result);
+     }
 }
 
-/* end of module */
