@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -28,52 +28,54 @@
 #include "dialog.h"
 #include "corrmenu.h"
 #include "appdialogue.h"
+#include "frame.h"
+#ifdef _WINDOWS
+#include "wininclude.h"
+#include "resource.h"
+#endif /* _WINDOWS */
 
 #undef THOT_EXPORT
 #define THOT_EXPORT extern
 #include "edit_tv.h"
 #include "appdialogue_tv.h"
+#include "frame_tv.h"
 #undef THOT_EXPORT
 #define THOT_EXPORT
 #include "spell_tv.h"
 
 /* les variables locales */
 static int          OldNC;
-static int          SpellingBase;
+static int          SpellingBase = 0;
 static ThotBool     FirstStep;
 static ThotBool     ToReplace;
-static CHAR_T       CorrectWord[MAX_WORD_LEN];
-static CHAR_T       CurrentWord[MAX_WORD_LEN];
+static char         CorrectWord[MAX_WORD_LEN];
+static char         CurrentWord[MAX_WORD_LEN];
 static PtrDocument  pDocSel;
-/* static STRING       SpecialChars; */
-
-/* les variables importees de l'editeur */
 
 /* procedures importees de l'editeur */
-#include "spellchecker_f.h"
-#include "dictionary_f.h"
-#include "word_f.h"
-#include "appli_f.h"
 #include "actions_f.h"
+#include "appli_f.h"
+#include "dictionary_f.h"
+#include "font_f.h"
 #include "memory_f.h"
-#include "views_f.h"
-#include "savedoc_f.h"
-#include "structselect_f.h"
 #include "message_f.h"
+#include "spellchecker_f.h"
+#include "structselect_f.h"
+#include "views_f.h"
+#include "word_f.h"
 
-#ifdef _WINDOWS 
-extern HWND wordButton;
-extern HWND hwnListWords;
-extern HWND hwndCurrentWord;
-extern HWND hwndLanguage;
-extern CHAR_T currentWord [30];
-
-#ifdef __STDC__
-extern void CreateSpellCheckDlgWindow (HWND, STRING, STRING, int, int, int,
-				       int, int, int, int);
-#else  /* __STDC__ */
-extern void CreateSpellCheckDlgWindow ();
-#endif /* __STDC__ */
+#ifdef _WINDOWS
+#define IDC_WORDBUTTON    20000
+#define IDC_LANGEDIT      20002
+static ThotWindow   SpellChecker = NULL;
+static ThotWindow   hwnListWords;
+static ThotWindow   hwndCurrentWord;
+static ThotWindow   hwndLanguage;
+static char         currentWord [MAX_WORD_LEN];
+static ThotWindow   wordButton;
+static UINT         itemIndex;
+static int          iLocation;
+static int          iIgnore;
 #endif /* _WINDOWS */
 
 
@@ -81,39 +83,65 @@ extern void CreateSpellCheckDlgWindow ();
    RemoveSpellForm fait disparaitre les formulaires correction    
    s'ils concernent le document pDoc et s'ils sont a l'ecran.      
   ----------------------------------------------------------------------*/
-static void         RemoveSpellForm ()
+static void RemoveSpellForm ()
 {
   if (ChkrRange != NULL)
     {
       FreeSearchContext (&ChkrRange);
+#ifdef _WINDOWS
+      EndDialog (SpellChecker, ID_DONE);
+      hwndLanguage = NULL;
+      SpellChecker = NULL;
+#else /* _WINDOWS */
       TtaDestroyDialogue (SpellingBase + ChkrFormCorrect);
+#endif /* _WINDOWS */
     }
 }
 
 /*----------------------------------------------------------------------
    DisplayWords prepare le selecteur                             
   ----------------------------------------------------------------------*/
-#ifndef _WINDOWS
-#ifdef __STDC__
 static void         DisplayWords (void)
-#else
-static void         DisplayWords ()
-#endif
 {
+#ifdef _WINDOWS
+  int i;
+  
+  SetWindowText (wordButton, ChkrCorrection[0]);
+  SendMessage (hwnListWords, LB_RESETCONTENT, 0, 0);
+  if ((strcmp (ChkrCorrection[1], "$")) == 0)
+    {
+      currentWord [0] = EOS;
+      SetWindowText (hwndCurrentWord, "");
+      SendMessage (hwnListWords, LB_ADDSTRING, 0, (LPARAM) (""));  
+    }
+  else
+    {
+      sprintf (currentWord, "%s", ChkrCorrection[1]);
+      SetWindowText (hwndCurrentWord, ChkrCorrection[1]);
+      for (i = 1; (i <= NC && strcmp (ChkrCorrection[i], "$") != 0);
+	   i++)
+	{
+	  SendMessage (hwnListWords, LB_INSERTSTRING, i - 1,
+		       (LPARAM) ((LPCTSTR)ChkrCorrection[i]));  
+	}
+    }
+  OldNC = NC;
+
+#else /* _WINDOWS */
    int                 i, indx, length;
-   STRING              entry;
-   CHAR_T                BufMenu[MAX_TXT_LEN];
+   char               *entry;
+   char                BufMenu[MAX_TXT_LEN];
 
    /* recopie les propositions */
    indx = 0;
    /* commencer a 1 parce qu'en 0 il y a CurrentWord */
-   for (i = 1; (i <= NC && ustrcmp (ChkrCorrection[i], "$") != 0); i++)
+   for (i = 1; (i <= NC && strcmp (ChkrCorrection[i], "$") != 0); i++)
      {
 	entry = ChkrCorrection[i];
-	length = ustrlen (entry) + 1;
+	length = strlen (entry) + 1;
 	if (length + indx < MAX_TXT_LEN)
 	  {
-	     ustrcpy ((BufMenu) + indx, entry);
+	     strcpy ((BufMenu) + indx, entry);
 	     indx += length;
 	  }
      }
@@ -126,59 +154,241 @@ static void         DisplayWords ()
 		   TtaGetMessage (CORR, Correct), i - 1,
 		   ((i < 2) ? "" : BufMenu), 3, entry, TRUE, FALSE);
    /* selectionner la proposition 0 dans le selecteur - si elle existe */
-   if (ustrcmp (ChkrCorrection[1], "$") != 0)
+   if (strcmp (ChkrCorrection[1], "$") != 0)
       TtaSetSelector (SpellingBase + ChkrSelectProp, -1, ChkrCorrection[1]);
    else
       TtaSetSelector (SpellingBase + ChkrSelectProp, -1, "");
 
    /* le formulaire est maintenant pret a etre affiche' */
    OldNC = NC;
+#endif /* _WINDOWS */
 }
-#endif /* !_WINDOWS */
+
 
 #ifdef _WINDOWS
-#ifdef __STDC__
-void WIN_DisplayWords (void)
-#else  /* __STDC__ */
-void WIN_DisplayWords ()
-#endif /* __STDC__ */
+/*-----------------------------------------------------------------------
+ SpellCheckDlgProc
+ ------------------------------------------------------------------------*/
+static LRESULT CALLBACK SpellCheckDlgProc (ThotWindow hwnDlg, UINT msg,
+					   WPARAM wParam, LPARAM lParam)
 {
-  int i;
-  
-  SetWindowText (wordButton, ChkrCorrection[0]);
-  SendMessage (hwnListWords, LB_RESETCONTENT, 0, 0);
-  if ((ustrcmp (ChkrCorrection[1], TEXT("$"))) == 0)
+  ThotBool ok;	  
+  int  val;
+  char currentRejectedchars [MAX_REJECTED_CHARS];
+
+  switch (msg)
     {
-      currentWord [0] = EOS;
-      SetWindowText (hwndCurrentWord, TEXT(""));
-      SendMessage (hwnListWords, LB_ADDSTRING, 0, (LPARAM) (TEXT("")));  
-    }
-  else
-    {
-      usprintf (currentWord, TEXT("%s"), ChkrCorrection[1]);
-      SetWindowText (hwndCurrentWord, ChkrCorrection[1]);
-      for (i = 1; (i <= NC && ustrcmp (ChkrCorrection[i], TEXT("$")) != 0);
-	   i++)
-	{
-	  SendMessage (hwnListWords, LB_INSERTSTRING, i - 1,
-		       (LPARAM) ((LPCTSTR)ChkrCorrection[i]));  
+    case WM_INITDIALOG:
+      SpellChecker = hwnDlg;
+      hwndLanguage = GetDlgItem (hwnDlg, IDC_LANG);
+      wordButton = GetDlgItem (hwnDlg, IDC_CURWORD);
+	  SetWindowText (hwnDlg, TtaGetMessage (CORR, Correct));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_LABEL), TtaGetMessage (CORR, Correct));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_BEFORE), TtaGetMessage (LIB, TMSG_BEFORE_SEL));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_WITHIN), TtaGetMessage (LIB, TMSG_WITHIN_SEL));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_AFTER), TtaGetMessage (LIB, TMSG_AFTER_SEL));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_WHOLEDOC), TtaGetMessage (LIB, TMSG_IN_WHOLE_DOC));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_NBPROPOSALS), TtaGetMessage (CORR, Number_Propositions));
+      
+      SetWindowText (GetDlgItem (hwnDlg, ID_SKIPNEXT), TtaGetMessage (CORR, Pass_Without));
+      SetWindowText (GetDlgItem (hwnDlg, ID_SKIPDIC), TtaGetMessage (CORR, Pass_With));
+      SetWindowText (GetDlgItem (hwnDlg, ID_REPLACENEXT), TtaGetMessage (CORR, Replace_Without));
+      SetWindowText (GetDlgItem (hwnDlg, ID_REPLACEDIC), TtaGetMessage(CORR, Replace_With));
+      
+      SetWindowText (GetDlgItem (hwnDlg, IDC_IGNORE1), TtaGetMessage (CORR, Capitals));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_IGNORE2), TtaGetMessage (CORR, Arabics));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_IGNORE3), TtaGetMessage (CORR, Romans));
+      SetWindowText (GetDlgItem (hwnDlg, IDC_IGNORE4), TtaGetMessage (CORR, Specials));
+      
+      SetWindowText (GetDlgItem (hwnDlg, IDC_CHECKGROUP), TtaGetMessage (CORR, What));
+      
+      SetWindowText (GetDlgItem (hwnDlg, IDC_IGNOREGROUP), TtaGetMessage (CORR, Ignore));
+      
+      SetWindowText (GetDlgItem (hwnDlg, ID_DONE), TtaGetMessage (LIB, TMSG_DONE));
+      
+      hwnListWords = CreateWindow ("listbox", NULL, WS_CHILD | WS_VISIBLE | LBS_STANDARD,
+				   13, 72, 150, 70, hwnDlg, (HMENU) 1, 
+				   (HINSTANCE) GetWindowLong (hwnDlg, GWL_HINSTANCE), NULL);
+      
+      hwndCurrentWord = CreateWindow ("EDIT", NULL, WS_CHILD | WS_VISIBLE | ES_LEFT | WS_BORDER,
+				      13, 146, 150, 20, hwnDlg, (HMENU) IDC_LANGEDIT, 
+				      (HINSTANCE) GetWindowLong (hwnDlg, GWL_HINSTANCE), NULL);
+      
+      
+      CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
+      CheckDlgButton (hwnDlg, IDC_IGNORE1, BST_CHECKED);
+      CheckDlgButton (hwnDlg, IDC_IGNORE2, BST_CHECKED);
+      CheckDlgButton (hwnDlg, IDC_IGNORE3, BST_CHECKED);
+      CheckDlgButton (hwnDlg, IDC_IGNORE4, BST_CHECKED);
+      
+      SetDlgItemInt (hwnDlg, IDC_EDITPROPOSALS, 3, FALSE);
+      SetDlgItemText (hwnDlg, IDC_EDITIGNORE, RejectedChar);
+      iLocation = 2;
+      SetWindowText (hwndCurrentWord, "");
+      DisplayWords ();
+      break;
+      
+    case WM_CLOSE:
+    case WM_DESTROY:
+      SpellChecker = NULL;
+      hwndLanguage = NULL;
+      EndDialog (hwnDlg, ID_DONE);
+      break;      
+      
+    case WM_COMMAND:
+      if (LOWORD (wParam) == 1 && HIWORD (wParam) == LBN_SELCHANGE) {
+	itemIndex = SendMessage (hwnListWords, LB_GETCURSEL, 0, 0);
+	itemIndex = SendMessage (hwnListWords, LB_GETTEXT, itemIndex, (LPARAM) currentWord);
+	SetDlgItemText (hwnDlg, IDC_LANGEDIT, currentWord);
+      } else if (LOWORD (wParam) == 1 && HIWORD (wParam) == LBN_DBLCLK) {
+	if (LB_ERR == (itemIndex = SendMessage (hwnListWords, LB_GETCURSEL, 0, 0L)))
+	  break;
+	itemIndex = SendMessage (hwnListWords, LB_GETTEXT, itemIndex, (LPARAM) currentWord);
+	SetDlgItemText (hwnDlg, IDC_LANGEDIT, currentWord);
+	ThotCallback (SpellingBase + ChkrSelectProp, STRING_DATA, currentWord);
+	ThotCallback (SpellingBase + ChkrMenuOR, INTEGER_DATA, (char*) iLocation);
+	ThotCallback (SpellingBase + ChkrFormCorrect, INTEGER_DATA, (char*) 3);
+	if (iLocation == 3) {
+	  CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
+	  iLocation = 2;
 	}
+	return 0;
+      } 
+      if (HIWORD (wParam) == EN_UPDATE)
+	{
+	  if (LOWORD (wParam) == IDC_EDITPROPOSALS)
+	    {
+	      val = GetDlgItemInt (hwnDlg, IDC_EDITPROPOSALS, &ok, TRUE);
+	      if (ok)
+		ThotCallback (SpellingBase + ChkrCaptureNC, INTEGER_DATA, (char*) val);
+	    }
+	  else if (LOWORD (wParam) == IDC_EDITIGNORE)
+	    {
+	      GetDlgItemText (hwnDlg, IDC_EDITIGNORE, currentRejectedchars, sizeof (currentRejectedchars) + 1);
+	      ThotCallback (SpellingBase + ChkrSpecial, STRING_DATA, currentRejectedchars);
+	    }
+	  else if (LOWORD (wParam) == IDC_LANGEDIT) 
+	    GetDlgItemText (hwnDlg, IDC_LANGEDIT, currentWord, sizeof (currentWord) + 1);
+	}
+      
+      switch (LOWORD (wParam))
+	{
+	case IDC_BEFORE:
+	  iLocation = 0;
+	  break;
+	  
+	case IDC_WITHIN:
+	  iLocation = 1;
+	  break;
+	  
+	case IDC_AFTER:
+	  iLocation = 2;
+	  break;
+	  
+	case IDC_WHOLEDOC:
+	  iLocation = 3;
+	  break;
+	  
+	case IDC_IGNORE1:
+	  ThotCallback (SpellingBase + ChkrMenuIgnore, INTEGER_DATA, (char*) 0);
+	  break;
+	  
+	case IDC_IGNORE2:
+	  ThotCallback (SpellingBase + ChkrMenuIgnore, INTEGER_DATA, (char*) 1);
+	  break;
+	  
+	case IDC_IGNORE3: 
+	  ThotCallback (SpellingBase + ChkrMenuIgnore, INTEGER_DATA, (char*) 2);
+	  break;
+	  
+	case IDC_IGNORE4:
+	  ThotCallback (SpellingBase + ChkrMenuIgnore, INTEGER_DATA, (char*) 3);
+	  break;
+	  
+	case ID_SKIPNEXT:
+	  ThotCallback (SpellingBase + ChkrSelectProp, STRING_DATA, currentWord);
+	  ThotCallback (SpellingBase + ChkrMenuOR, INTEGER_DATA, (char*) iLocation);
+	  ThotCallback (SpellingBase + ChkrFormCorrect, INTEGER_DATA, (char*) 1);
+	  if (iLocation == 3) {
+	    CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
+	    iLocation = 2;
+	  }
+	  break;
+	  
+	case ID_SKIPDIC:
+	  ThotCallback (SpellingBase + ChkrSelectProp, STRING_DATA, currentWord);
+	  ThotCallback (SpellingBase + ChkrMenuOR, INTEGER_DATA, (char*) iLocation);
+	  ThotCallback (SpellingBase + ChkrFormCorrect, INTEGER_DATA, (char*) 2);
+	  if (iLocation == 3)
+	    {
+	      CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
+	      iLocation = 2;
+	    }
+	  break;
+	  
+	case ID_REPLACENEXT:
+	  ThotCallback (SpellingBase + ChkrSelectProp, STRING_DATA, currentWord);
+	  ThotCallback (SpellingBase + ChkrMenuOR, INTEGER_DATA, (char*) iLocation);
+	  ThotCallback (SpellingBase + ChkrFormCorrect, INTEGER_DATA, (char*) 3);
+	  if (iLocation == 3)
+	    {
+	      CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
+	      iLocation = 2;
+	    }
+	  break;
+	  
+	case ID_REPLACEDIC:
+	  ThotCallback (SpellingBase + ChkrSelectProp, STRING_DATA, currentWord);
+	  ThotCallback (SpellingBase + ChkrMenuOR, INTEGER_DATA, (char*) iLocation);
+	  ThotCallback (SpellingBase + ChkrFormCorrect, INTEGER_DATA, (char*) 4);
+	  if (iLocation == 3)
+	    {
+	      CheckRadioButton (hwnDlg, IDC_BEFORE, IDC_WHOLEDOC, IDC_AFTER);
+	      iLocation = 2;
+	    }
+	  break;
+	  
+	case IDC_WORDBUTTON:
+	  GetWindowText (wordButton, currentWord, MAX_WORD_LEN);
+	  SetWindowText (hwndCurrentWord, currentWord);
+	  break;
+	  
+	case ID_DONE:
+	  SpellChecker = NULL;
+	  hwndLanguage = NULL;
+	  EndDialog (hwnDlg, ID_DONE);
+	  break;	  
+	}
+      break;
+    default: return FALSE;
     }
-  OldNC = NC;
+  return TRUE;
+}
+#else /* _WINDOWS */
+
+/*----------------------------------------------------------------------
+  UnsetEntryMenu
+  displays as non active the "ent" entry of the menu referenced by "ref".
+  ----------------------------------------------------------------------*/
+void UnsetEntryMenu (int ref, int ent)
+{
+   char                fontname[100];
+   char                text[20];
+
+   if (TtWDepth > 1)
+      TtaRedrawMenuEntry (ref, ent, NULL, InactiveB_Color, 0);
+   else
+     {
+	FontIdentifier ('L', 'T', 2, 11, 1, text, fontname);
+	TtaRedrawMenuEntry (ref, ent, fontname, -1, 0);
+     }
 }
 #endif /* _WINDOWS */
 
-
 /*----------------------------------------------------------------------
   TtcSpellCheck  active le formulaire de correction                
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtcSpellCheck (Document doc, View view)
-#else  /* __STDC__ */
-void                TtcSpellCheck (doc, view)
-Document            doc;
-view                view;
-#endif /* __STDC__ */
+void TtcSpellCheck (Document doc, View view)
 {
    PtrDocument         document;
    int                 i;
@@ -186,7 +396,7 @@ view                view;
    PtrElement          pEl1, pElN;
    int                 c1, cN;
    int                 indx;
-   CHAR_T                BufMenu[MAX_TXT_LEN];
+   char                BufMenu[MAX_TXT_LEN];
    ThotBool            ok;
 #endif /* !_WINDOWS */
 
@@ -205,16 +415,16 @@ view                view;
    /* creer la feuille de dialogue de CORRECTION */
    /* attache'e au bouton Confirmer du formulaire (1) CORRIGER */
    indx = 0;		   /* tous les boutons du dialogue de correction */
-   ustrcpy (&BufMenu[indx], TtaGetMessage (CORR, Pass_Without));
-   indx += ustrlen(&BufMenu[indx]) + 1;
-   ustrcpy(&BufMenu[indx], TtaGetMessage(CORR, Pass_With));
-   indx += ustrlen (&BufMenu[indx]) + 1;
-   ustrcpy (&BufMenu[indx], TtaGetMessage (CORR, Replace_Without));
-   indx += ustrlen(&BufMenu[indx]) + 1;
-   ustrcpy(&BufMenu[indx], TtaGetMessage(CORR, Replace_With));
+   strcpy (&BufMenu[indx], TtaGetMessage (CORR, Pass_Without));
+   indx += strlen(&BufMenu[indx]) + 1;
+   strcpy(&BufMenu[indx], TtaGetMessage(CORR, Pass_With));
+   indx += strlen (&BufMenu[indx]) + 1;
+   strcpy (&BufMenu[indx], TtaGetMessage (CORR, Replace_Without));
+   indx += strlen(&BufMenu[indx]) + 1;
+   strcpy(&BufMenu[indx], TtaGetMessage(CORR, Replace_With));
    /* ne pas afficher cette feuille maintenant */
    TtaNewSheet (SpellingBase + ChkrFormCorrect, TtaGetViewFrame (doc, view), 
-		TtaGetMessage (CORR, Correct), 4, BufMenu, FALSE, 4, TEXT('L'),
+		TtaGetMessage (CORR, Correct), 4, BufMenu, FALSE, 4, 'L',
 		D_DONE);
 
    /* initialise le champ langue de correction courante */
@@ -223,9 +433,9 @@ view                view;
 #endif /* !_WINDOWS */
 
    /* Afficher une liste de mots EMPTY */
-   ustrcpy (ChkrCorrection[0], TEXT(" "));
+   strcpy (ChkrCorrection[0], " ");
    for (i = 1; i <= NC; i++)
-      ustrcpy (ChkrCorrection[i], TEXT("$"));
+      strcpy (ChkrCorrection[i], "$");
 
 #ifndef _WINDOWS 
    DisplayWords ();
@@ -233,11 +443,11 @@ view                view;
    /* creer le sous-menu OU dans la feuille OPTIONS */
    indx = 0;			/* Ou commencer la correction ? */
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (LIB, TMSG_BEFORE_SEL));
-   indx += ustrlen (&BufMenu[indx]) + 1;
+   indx += strlen (&BufMenu[indx]) + 1;
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (LIB, TMSG_WITHIN_SEL));
-   indx += ustrlen (&BufMenu[indx]) + 1;
+   indx += strlen (&BufMenu[indx]) + 1;
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (LIB, TMSG_AFTER_SEL));
-   indx += ustrlen (&BufMenu[indx]) + 1;
+   indx += strlen (&BufMenu[indx]) + 1;
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (LIB, TMSG_IN_WHOLE_DOC));
    TtaNewSubmenu (SpellingBase + ChkrMenuOR, SpellingBase + ChkrFormCorrect, 0,
 		  TtaGetMessage (CORR, What), 4, BufMenu, NULL, FALSE);
@@ -272,11 +482,11 @@ view                view;
    /* sous-menu Mots a ignorer */
    indx = 0;
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (CORR, Capitals));
-   indx += ustrlen (&BufMenu[indx]) + 1;
+   indx += strlen (&BufMenu[indx]) + 1;
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (CORR, Arabics));
-   indx += ustrlen (&BufMenu[indx]) + 1;
+   indx += strlen (&BufMenu[indx]) + 1;
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (CORR, Romans));
-   indx += ustrlen (&BufMenu[indx]) + 1;
+   indx += strlen (&BufMenu[indx]) + 1;
    sprintf (&BufMenu[indx], "B%s", TtaGetMessage (CORR, Specials));
    TtaNewToggleMenu (SpellingBase + ChkrMenuIgnore,
 		     SpellingBase + ChkrFormCorrect,
@@ -286,12 +496,12 @@ view                view;
    /* liste des caracteres speciaux dans le formulaire OPTIONS */
    TtaNewTextForm (SpellingBase + ChkrSpecial, SpellingBase + ChkrFormCorrect,
 		   NULL, 20, 1, TRUE);
-   TtaSetTextForm (SpellingBase + ChkrSpecial, TEXT("@#$&+~"));
+   TtaSetTextForm (SpellingBase + ChkrSpecial, "@#$&+~");
 #endif /* _WINDOWS */
 
    /* ne pas ignorer les mots en capitale, chiffres romains */
    /* ou contenant des chiffres arabes,  certains car. speciaux */
-   ustrncpy (RejectedChar, TEXT("@#$&+~"), MAX_REJECTED_CHARS);
+   strncpy (RejectedChar, "@#$&+~", MAX_REJECTED_CHARS);
    /* liste par defaut */
 
    IgnoreUppercase = TRUE;
@@ -323,45 +533,50 @@ view                view;
    /* ne cree pas inutilement le dictionnaire fichier */
    TtaLoadDocumentDictionary (document, (int*) &ChkrFileDict, FALSE);
 
-#ifdef _WINDOWS 
-   CreateSpellCheckDlgWindow (TtaGetViewFrame (doc, view),
-			      TtaGetMessage (CORR, Correct), RejectedChar,
-			      SpellingBase, ChkrSelectProp, ChkrMenuOR, 
-			      ChkrFormCorrect, ChkrMenuIgnore, ChkrCaptureNC,
-			      ChkrSpecial);
-
+#ifdef _WINDOWS  
+  /* to have the same behavior as under Unix, we need to destroy the
+     dialog if it already existed */
+  if (SpellChecker) 
+    {
+      EndDialog (SpellChecker, ID_DONE);
+      SpellChecker = NULL;
+      hwndLanguage = NULL;
+    }
+  DialogBox (hInstance, MAKEINTRESOURCE (SPELLCHECKDIALOG), NULL, (DLGPROC) SpellCheckDlgProc);
 #endif /* _WINDOWS */
 }
 
 
+/*----------------------------------------------------------------------
+  ResetCheckInDocument
+  cleans up the checker domain if it refers a closed document.
+  ----------------------------------------------------------------------*/
+void         ResetCheckInDocument (PtrDocument pDoc)
+{
+  if (ChkrRange && pDoc == ChkrRange->SDocument)
+    RemoveSpellForm ();
+}
+
+
 /*----------------------------------------------------------------------
    SetProposals                                                 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         SetProposals (Language language)
-#else  /* __STDC__ */
-static void         SetProposals (language)
-Language            language;
-#endif /* __STDC__ */
 {
-   CHAR_T             Lab[200];
+   char             Lab[200];
 
    /* calculer les propositions de correction du mot courant */
    GiveProposal (language, ChkrFileDict);
    /* afficher les mots proposes contenus dans ChkrErrWord */
-#ifndef _WINDOWS
    DisplayWords ();
-#else  /* _WINDOWS */
-   WIN_DisplayWords ();
-#endif /* _WINDOWS */
    /* recopier la premiere proposition dans CorrectWord */
-   if (ustrcmp (ChkrCorrection[1], TEXT("$")) != 0)
-      ustrcpy (CorrectWord, ChkrCorrection[1]);
+   if (strcmp (ChkrCorrection[1], "$") != 0)
+      strcpy (CorrectWord, ChkrCorrection[1]);
    else
       CorrectWord[0] = EOS;
 
    /* afficher la langue de correction courante */
-   usprintf (Lab, TEXT("%s: %s"), TtaGetMessage (LIB, TMSG_LANGUAGE),
+   sprintf (Lab, "%s: %s", TtaGetMessage (LIB, TMSG_LANGUAGE),
 	     TtaGetLanguageName (ChkrLanguage));
 #ifdef _WINDOWS
    SetWindowText (hwndLanguage, Lab);
@@ -389,7 +604,7 @@ static ThotBool     StartSpellChecker ()
      {
 	FirstStep = FALSE;
 	ChkrElement = NULL;
-	ChkrIndChar = 0;
+	ChkrIndChar = 1;
      }
 
    /* en tenant compte des options choisies (par defaut) */
@@ -401,14 +616,10 @@ static ThotBool     StartSpellChecker ()
      {
        /* Correction TERMINEE */
 #ifdef _WINDOWS
-       MessageBox (NULL, TtaGetMessage (LIB, TMSG_NOT_FOUND), \
-		   TEXT("Spell checking"), MB_OK | MB_ICONINFORMATION);
+       MessageBox (NULL, TtaGetMessage (CORR, END_CHECK), \
+		   "Spell checking", MB_OK | MB_ICONINFORMATION);
 #else  /* _WINDOWS */
-       TtaDisplaySimpleMessage (INFO, CORR, END_CHECK);
-       /* message 'Pas trouve' dans le formulaire */
-       TtaNewLabel (SpellingBase + ChkrLabelNotFound,
-		    SpellingBase + ChkrFormCorrect,
-		    TtaGetMessage (LIB, TMSG_NOT_FOUND));
+       TtaDisplayMessage (CONFIRM, TtaGetMessage (CORR, END_CHECK), NULL);
 #endif /* _WINDOWS */
        FirstStep = TRUE;
        ok = FALSE;
@@ -420,14 +631,7 @@ static ThotBool     StartSpellChecker ()
 /*----------------------------------------------------------------------
    ApplyCommand traite l'action de correction demandee.             
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static void         ApplyCommand (int val)
-
-#else  /* __STDC__ */
-static void         ApplyCommand (val)
-int                 val;
-
-#endif /* __STDC__ */
 {
    ThotBool            change;
    PtrDocument         document;
@@ -441,10 +645,8 @@ int                 val;
    change = CheckChangeSelection ();
 
    if (document != ChkrRange->SDocument || val == 0)
-     {
-       /* ABANDON de la corrrection */
-       RemoveSpellForm ();
-     }
+     /* ABANDON de la corrrection */
+     RemoveSpellForm ();
    else
      {
        change = TRUE;		/* On passe a priori au mot suivant */
@@ -456,9 +658,11 @@ int                 val;
 	 case 2:
 	   /* Passer apres maj du dictionnaire */
 	   /* mettre CurrentWord dans le dictionnaire */
-	   if (ChkrElement != NULL)	/* Sauf au 1er lancement */
+	   if (ChkrElement != NULL)
+	     {
+	     /* Sauf au 1er lancement */
 	     if (CorrectWord[0] != EOS &&
-		 (ustrcmp (CorrectWord, ChkrCorrection[1]) != 0))
+		 (strcmp (CorrectWord, ChkrCorrection[1]) != 0))
 	       {
 		 /* ajouter le mot corroge dans le dictionaire */
 		 if (!CheckWord (CorrectWord, ChkrLanguage, ChkrFileDict))
@@ -470,6 +674,7 @@ int                 val;
 		 if (!CheckWord (CurrentWord, ChkrLanguage, ChkrFileDict))
 		   AddWord (CurrentWord, &ChkrFileDict);
 	       }
+	     }
 	   break;
 	 case 3:
 	   /* ToReplace */
@@ -505,25 +710,16 @@ int                 val;
 
 
 /*----------------------------------------------------------------------
-   CallbackCorrector                                                 
+  CallbackChecker
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                CallbackCorrector (int ref, int dataType, STRING data)
-#else  /* __STDC__ */
-void                CallbackCorrector (ref, dataType, data)
-int                 ref;
-int                 dataType;
-STRING              data;
-#endif /* __STDC__ */
+void CallbackChecker (int ref, int dataType, char *data)
 {
   PtrElement          pEl1, pElN;
   int                 c1, cN;
 
   if (ref == -1)
-    {
       /* detruit le dialogue du correcteur */
       RemoveSpellForm ();
-    }
   else
     switch (ref - SpellingBase)
       {
@@ -550,7 +746,7 @@ STRING              data;
 	break;
       case ChkrSpecial:
 	/* recopier la liste des car. speciaux dans RejectedChar[] */
-	ustrncpy (RejectedChar, data, MAX_REJECTED_CHARS);
+	strncpy (RejectedChar, data, MAX_REJECTED_CHARS);
 	/* bascule automatiquement l'indicateur IgnoreSpecial */
 	if (!IgnoreSpecial)
 	  {
@@ -566,11 +762,7 @@ STRING              data;
 	if (NC > OldNC && ChkrElement != NULL && CurrentWord[0] != EOS)
 	  SetProposals (ChkrLanguage);
 	else
-#ifndef _WINDOWS 
 	  DisplayWords ();
-#else  /* _WINDOWS */
-	WIN_DisplayWords ();
-#endif /* _WINDOWS */
 	break;
       case ChkrMenuOR:
 	/* definition du sens de correction OU? */
@@ -608,22 +800,23 @@ STRING              data;
 	      /* Est-ce encore vrai */
 	      GetCurrentSelection (&pDocSel, &pEl1, &pElN, &c1, &cN);
 #ifndef _WINDOWS
-	      if (pDocSel == ChkrRange->SDocument) {
-		/* Il faut reactiver les entree */
-		TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 0, NULL, -1, 1);
-		TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 1, NULL, -1, 1);
-		TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 2, NULL, -1, 1);
-	      } 
+	      if (pDocSel == ChkrRange->SDocument)
+		{
+		  /* Il faut reactiver les entree */
+		  TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 0, NULL, -1, 1);
+		  TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 1, NULL, -1, 1);
+		  TtaRedrawMenuEntry (SpellingBase + ChkrMenuOR, 2, NULL, -1, 1);
+		} 
 #endif /* !_WINDOWS */
 	    }
 	break;
       case ChkrSelectProp:
 	/* retour du selecteur de propositions */
 	/* recopier le choix dans CorrectWord */
-	ustrcpy (CorrectWord, data);
+	strcpy (CorrectWord, data);
 	if (CorrectWord[0] != EOS
 	    && CurrentWord != EOS
-	    && ustrcmp (CorrectWord, CurrentWord) != 0)
+	    && strcmp (CorrectWord, CurrentWord) != 0)
 	  ToReplace = TRUE;
 	break;
       case ChkrFormCorrect:
@@ -636,17 +829,14 @@ STRING              data;
 
 
 /*----------------------------------------------------------------------
-   SpellCheckLoadResources charge les ressouces du correcteur       
-   orthographique                                         
+  SpellCheckLoadResources loads resouces of the spellchecker
   ----------------------------------------------------------------------*/
 void                SpellCheckLoadResources ()
 {
-   if (ThotLocalActions[T_corrector] == NULL)
+   if (SpellingBase == 0)
      {
-	TteConnectAction (T_rscorrector, (Proc) CallbackCorrector);
-	TteConnectAction (T_corrector, (Proc) TtcSpellCheck);
-	CORR = TtaGetMessageTable (TEXT("corrdialogue"), MSG_MAX_CHECK);
-	SpellingBase = TtaSetCallback (CallbackCorrector, ChkrMaxDialogue);
+	CORR = TtaGetMessageTable ("corrdialogue", MSG_MAX_CHECK);
+	SpellingBase = TtaSetCallback (CallbackChecker, ChkrMaxDialogue);
 	/* Initialisation des variables globales */
 	ParametrizeChecker ();
 	ChkrFileDict = NULL;
