@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -116,12 +116,11 @@ void VerticalScroll (int frame, int delta, int selection)
 				   pFrame->FrXOrg + lframe, height);
 			  add = RedrawFrameTop (frame, -delta);
 		       }
+		     /* recompute scrolls */
+		     CheckScrollingWidth (frame);
+		     UpdateScrollbars (frame);
 		    }
 
-		  /* recompute scrolls */
-		  CheckScrollingWidth (frame);
-		  UpdateScrollbars (frame);
-
 		  if (selection != 0)
 		     if (add)
 		       {
@@ -384,6 +383,8 @@ void ComputeDisplayedChars (int frame, int *Xpos, int *Ypos, int *width, int *he
        pBox = pFrame->FrAbstractBox->AbBox;
        if (pBox == NULL)
 	 return;
+       if (pFrame->FrAbstractBox->AbElement == NULL)
+	 return;
      }
    else
       return;
@@ -442,7 +443,6 @@ void ComputeDisplayedChars (int frame, int *Xpos, int *Ypos, int *width, int *he
       /* there is nothing on the left */
       *width = l - *Xpos;
 
-
    /***     Vertical scrolling             ***/
    /*      +-------------+                   */
    /*      |  Document   |a                  */
@@ -501,9 +501,11 @@ void ComputeDisplayedChars (int frame, int *Xpos, int *Ypos, int *width, int *he
 	     while (pBoxFirst->BxAbstractBox->AbPresentationBox &&
 		    pBoxFirst != pBoxLast)
 		pBoxFirst = pBoxFirst->BxNext;
-	     while (pBoxLast->BxAbstractBox->AbPresentationBox &&
-		    pBoxFirst != pBoxLast)
-		pBoxLast = pBoxLast->BxPrevious;
+	     if (pBoxFirst != pBoxLast)
+	       while (pBoxLast->BxAbstractBox->AbPresentationBox &&
+		      pBoxLast->BxPrevious->BxAbstractBox &&
+		      pBoxFirst != pBoxLast)
+		 pBoxLast = pBoxLast->BxPrevious;
 
 	     /* Evalue la hauteur et la position Y par rapport a l'image
 		abstraite */
