@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -31,19 +31,14 @@
 /*----------------------------------------------------------------------
    TtaIsDocumentSelected answers TRUE if the document is selected.  
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool            TtaIsDocumentSelected (Document document)
-#else  /* __STDC__ */
-ThotBool            TtaIsDocumentSelected (document)
-Document            document;
-
-#endif /* __STDC__ */
+ThotBool TtaIsDocumentSelected (Document document)
 {
    PtrDocument         docsel;
    PtrElement          firstSelection, lastSelection;
    int                 firstChar, lastChar;
 
-   if (!GetCurrentSelection (&docsel, &firstSelection, &lastSelection, &firstChar, &lastChar))
+   if (!GetCurrentSelection (&docsel, &firstSelection, &lastSelection,
+			     &firstChar, &lastChar))
       return (FALSE);
    else if (LoadedDocument[document - 1] != docsel)
       return (FALSE);
@@ -57,22 +52,13 @@ Document            document;
    Selects a single element. This element is highlighted in all views
    where it can be displayed. If it cannot be displayed in any existing
    view, a new view is eventually open for displaying it.
-
    Parameters:
    document: the document containing the element to be
    selected.
    selectedElement: the element to be selected. NULL for cancelling the
    selection in the document.
-
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaSelectElement (Document document, Element selectedElement)
-#else  /* __STDC__ */
-void                TtaSelectElement (document, selectedElement)
-Document            document;
-Element             selectedElement;
-
-#endif /* __STDC__ */
+void TtaSelectElement (Document document, Element selectedElement)
 {
    DisplayMode         dispMode;
 
@@ -92,7 +78,8 @@ Element             selectedElement;
 	   /* Abort the selection */
 	 (*ThotLocalActions[T_resetsel]) (LoadedDocument[document - 1]);
 	 else
-	   SelectElement (LoadedDocument[document - 1], (PtrElement) selectedElement, TRUE, FALSE);
+	   SelectElement (LoadedDocument[document - 1],
+			  (PtrElement) selectedElement, TRUE, FALSE);
        else
 	 NewSelection (document, selectedElement, 0, 0);
      }
@@ -101,40 +88,51 @@ Element             selectedElement;
 /*----------------------------------------------------------------------
    TtaSelectView posts the selected view in the document
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaSelectView (Document document, View view)
-#else  /* __STDC__ */
-void                TtaSelectView (document, view)
-Document            document;
-View                view;
-
-#endif /* __STDC__ */
+void TtaSelectView (Document document, View view)
 {
   PtrDocument       pDoc;
   int               oldView;
-  ThotBool          assoc;
 
-   UserErrorCode = 0;
-   GetActiveView (&pDoc, &oldView, &assoc);
-   if (LoadedDocument[document - 1] == pDoc && !assoc && oldView != view)
-     SelectedView = view;
+  UserErrorCode = 0;
+  GetActiveView (&pDoc, &oldView);
+  if (LoadedDocument[document - 1] == pDoc && oldView != view)
+    SelectedView = view;
 }
 
 /*----------------------------------------------------------------------
    TtaGetSelectedDocument returns the current selected document
   ----------------------------------------------------------------------*/
-Document   TtaGetSelectedDocument ()
+Document TtaGetSelectedDocument ()
 {
   PtrDocument       pDoc;
   int               oldView;
-  ThotBool          assoc;
 
-   UserErrorCode = 0;
-   GetActiveView (&pDoc, &oldView, &assoc);
-   if (pDoc == NULL)
-     return (0);
-   else
-     return (IdentDocument (pDoc));
+  UserErrorCode = 0;
+  GetActiveView (&pDoc, &oldView);
+  if (pDoc == NULL)
+    return (0);
+  else
+    return (IdentDocument (pDoc));
+}
+
+/*----------------------------------------------------------------------
+   TtaGetActiveView returns the selected document and the active view
+   in that document
+  ----------------------------------------------------------------------*/
+void TtaGetActiveView (Document *doc, View *view)
+{
+  PtrDocument     pDoc;
+  int             docView;
+
+  UserErrorCode = 0;
+  *doc = 0;
+  *view = 0;
+  GetActiveView (&pDoc, &docView);
+  if (pDoc)
+    {
+      *doc = IdentDocument (pDoc);
+      *view = (View) docView;
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -152,16 +150,8 @@ Document   TtaGetSelectedDocument ()
    to be selected.
 
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaSelectString (Document document, Element textElement, int firstCharacter, int lastCharacter)
-#else  /* __STDC__ */
-void                TtaSelectString (document, textElement, firstCharacter, lastCharacter)
-Document            document;
-Element             textElement;
-int                 firstCharacter;
-int                 lastCharacter;
-
-#endif /* __STDC__ */
+void  TtaSelectString (Document document, Element textElement,
+		       int firstCharacter, int lastCharacter)
 {
    DisplayMode         dispMode;
 
@@ -194,71 +184,62 @@ int                 lastCharacter;
    TtaExtendSelection
 
    Extends the current selection to a given element.
-
    Parameters:
    document: the document for which the selection must be extended.
    element: the element to which the selection must be extended.
    lastCharacter: position within this element of the last character
    to be selected. 0 if the whole element must be selected.
-
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaExtendSelection (Document document, Element element, int lastCharacter)
-#else  /* __STDC__ */
-void                TtaExtendSelection (document, element, lastCharacter)
-Document            document;
-Element             element;
-int                 lastCharacter;
-
-#endif /* __STDC__ */
+void TtaExtendSelection (Document document, Element element, int lastCharacter)
 {
-   PtrDocument         pDoc;
-   PtrElement          firstSelection, lastSelection;
-   DisplayMode         dispMode;
-   int                 firstChar, lastChar;
-   ThotBool            ok;
-   ThotBool            abort;
+  PtrDocument         pDoc;
+  PtrElement          firstSelection, lastSelection;
+  DisplayMode         dispMode;
+  int                 firstChar, lastChar;
+  ThotBool            ok;
+  ThotBool            abort;
 
-   UserErrorCode = 0;
-   if (element == NULL)
-      TtaError (ERR_invalid_parameter);
-   else if (((PtrElement) element)->ElParent == NULL)
-      TtaError (ERR_invalid_parameter);
-   /* Checks the parameter document */
-   else if (document < 1 || document > MAX_DOCUMENTS)
-      TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-      TtaError (ERR_invalid_document_parameter);
-   else
-      /* Parameter document is correct */
-     {
-	/* verifies if there is a selection */
-	ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection, &firstChar, &lastChar);
-	dispMode = TtaGetDisplayMode (document);
-	if (dispMode == DisplayImmediately || dispMode == DeferredDisplay)
-	   /* The command can be executed */
-	  {
-	     if (ok)
-		/* verifies that the selection is into the document */
-		ok = (pDoc == LoadedDocument[document - 1]);
-	  }
-	/* verifies if a selection is applied */
-	else if (IsSelectionRegistered (document, &abort))
-	   /* There is an application, the extension is accepted if there is 
-	      a request which is not for aborting */
-	   ok = !abort;
-
-	if (!ok)
-	   /* Error: no selection */
-	   TtaError (ERR_no_selection_in_document);
-	else
-	  {
-	    if (dispMode == DisplayImmediately || dispMode == DeferredDisplay)
-	      ExtendSelection ((PtrElement) element, lastCharacter, FALSE, FALSE, FALSE);
-	    else
-	      NewSelectionExtension (document, element, lastCharacter);
-	  }
-     }
+  UserErrorCode = 0;
+  if (element == NULL)
+    TtaError (ERR_invalid_parameter);
+  else if (((PtrElement) element)->ElParent == NULL)
+    TtaError (ERR_invalid_parameter);
+  /* Checks the parameter document */
+  else if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    /* Parameter document is correct */
+    {
+      /* verifies if there is a selection */
+      ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection,
+				&firstChar, &lastChar);
+      dispMode = TtaGetDisplayMode (document);
+      if (dispMode == DisplayImmediately || dispMode == DeferredDisplay)
+	/* The command can be executed */
+	{
+	  if (ok)
+	    /* verifies that the selection is into the document */
+	    ok = (pDoc == LoadedDocument[document - 1]);
+	}
+      /* verifies if a selection is applied */
+      else if (IsSelectionRegistered (document, &abort))
+	/* There is an application, the extension is accepted if there is 
+	   a request which is not for aborting */
+	ok = !abort;
+      
+      if (!ok)
+	/* Error: no selection */
+	TtaError (ERR_no_selection_in_document);
+      else
+	{
+	  if (dispMode == DisplayImmediately || dispMode == DeferredDisplay)
+	    ExtendSelection ((PtrElement) element, lastCharacter, TRUE, FALSE, FALSE);
+	  else
+	    NewSelectionExtension (document, element, lastCharacter);
+	}
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -266,21 +247,13 @@ int                 lastCharacter;
 
    Add a new element to the current selection.  The new selection
    may then contain separate elements.
-
    Parameters:
    document: the document to which the element belongs.  This element must
 	belong to the same document as the elements selected by previous calls
 	to TtaAddElementToSelection and by the last call to TtaSelectElement.
    element: the element to be added to the current selection
-
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaAddElementToSelection (Document document, Element element)
-#else  /* __STDC__ */
-void                TtaAddElementToSelection (document, element)
-Document            document;
-Element             element;
-#endif /* __STDC__ */
+void TtaAddElementToSelection (Document document, Element element)
 {
    PtrDocument         pDoc;
    PtrElement          firstSelection, lastSelection;
@@ -306,12 +279,14 @@ Element             element;
 				  &firstChar, &lastChar);
 	dispMode = TtaGetDisplayMode (document);
 	if (ok)
+	  {
 	  if (dispMode == NoComputedDisplay)
 	     /* accept only if immediate display mode */
 	     ok = FALSE;
 	  else
 	     /* is the current selection is in the same document? */
 	     ok = (pDoc == LoadedDocument[document - 1]);
+	  }
 	if (!ok)
 	   /* Error: no selection */
 	   TtaError (ERR_no_selection_in_document);
@@ -326,14 +301,12 @@ Element             element;
    If a pair of paired elements is selected, select also all elements
    between the two paired elements. Nothing is done if the current
    selection is not a pair.
-
    Parameters:
    No parameter
-
    Return value:
    No return value
   ----------------------------------------------------------------------*/
-void                TtaSelectInterval ()
+void TtaSelectInterval ()
 {
    SelectPairInterval ();
 }
@@ -343,19 +316,12 @@ void                TtaSelectInterval ()
    TtaUnselect
 
    Turns the selection off for a document.
-
    Parameters:
    document: the document for which the selection must be turned off
-
    Return value:
    No return value
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaUnselect (Document document)
-#else  /* __STDC__ */
-void                TtaUnselect (document)
-Document            document;
-#endif /* __STDC__ */
+void TtaUnselect (Document document)
 {
    if (document < 1 || document > MAX_DOCUMENTS)
      TtaError (ERR_invalid_document_parameter);
@@ -374,17 +340,10 @@ Document            document;
    functions TtaSelectElement, TtaSelectString, TtaExtendSelection and
    TtaUnselect do not update the menus that depend on the selection
    and do not display the selection message. Default mode is withMenu = TRUE.
-
    Parameter:
    withMenu: the new selection mode.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaSetSelectionMode (ThotBool withMenu)
-#else  /* __STDC__ */
-void                TtaSetSelectionMode (withMenu)
-ThotBool            withMenu;
-
-#endif /* __STDC__ */
+void TtaSetSelectionMode (ThotBool withMenu)
 {
    SelectionUpdatesMenus = withMenu;
 }
@@ -395,81 +354,78 @@ ThotBool            withMenu;
    Returns TRUE if there is a current selection and this selection is
    empty (a caret).
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool            TtaIsSelectionEmpty ()
-#else  /* __STDC__ */
-void                TtaIsSelectionEmpty ()
-#endif /* __STDC__ */
+ThotBool TtaIsSelectionEmpty ()
 {
-  if (SelectedDocument == NULL)
+  if (SelectedDocument == NULL && DocSelectedAttr == NULL)
     return (FALSE);
-  else if (SelPosition != TRUE &&
+  else if (SelPosition)
+    return (TRUE);
+  else if (FirstSelectedElement &&
 	   FirstSelectedElement == LastSelectedElement &&
 	   FirstSelectedElement->ElVolume == 0)
     return (TRUE);
   else
-    return (SelPosition);
+    return (FALSE);
 }
 
 /*----------------------------------------------------------------------
-   TtaGiveFirstSelectedElement
+  TtaGiveFirstSelectedElement
 
-   Returns the first element in the current selection in a given document.
-   If this element is a Text element and if only a substring is selected,
-   return also the rank of the first and last characters in the selection.
-
-   Parameter:
-   document: the document for which the selection is asked.
-
-   Return values:
-   selectedElement: the first selected element, NULL if no element is
-   selected in the document.
-   firstCharacter: rank of the first character in the selection, or 0 if the
-   whole element is in the selection.
-   lastCharacter: rank of the last character in the selection, or 0 if the
-   whole element is in the selection.
+  Returns the first element in the current selection in a given document.
+  If this element is a Text element and if only a substring is selected,
+  return also the rank of the first and last characters in the selection.
+  Parameter:
+  document: the document for which the selection is asked.
+  Return values:
+  selectedElement: the first selected element, NULL if no element is
+  selected in the document.
+  firstCharacter: rank of the first selected character in the element, or
+  0 if the whole element is selected.
+  lastCharacter: rank of the last selected character in the element, or
+  0 if the whole element is in the selection, or firstCharacter - 1 for
+  a position.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaGiveFirstSelectedElement (Document document, Element * selectedElement, int *firstCharacter, int *lastCharacter)
-#else  /* __STDC__ */
-void                TtaGiveFirstSelectedElement (document, selectedElement, firstCharacter, lastCharacter)
-Document            document;
-Element            *selectedElement;
-int                *firstCharacter;
-int                *lastCharacter;
-#endif /* __STDC__ */
-
+void TtaGiveFirstSelectedElement (Document document, Element *selectedElement,
+				  int *firstCharacter, int *lastCharacter)
 {
-   PtrDocument         pDoc;
-   PtrElement          firstSelection, lastSelection;
-   int                 firstChar, lastChar;
-   ThotBool            ok;
+  PtrDocument         pDoc;
+  PtrElement          firstSelection, lastSelection;
+  int                 firstChar, lastChar;
+  ThotBool            ok;
 
-   UserErrorCode = 0;
-   *selectedElement = NULL;
-   *firstCharacter = 0;
-   *lastCharacter = 0;
-   /* Checks the parameter document */
-   if (document < 1 || document > MAX_DOCUMENTS)
-     TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-     TtaError (ERR_invalid_document_parameter);
-   else
+  UserErrorCode = 0;
+  *selectedElement = NULL;
+  *firstCharacter = 0;
+  *lastCharacter = 0;
+  /* Checks the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
       /* Parameter document is correct */
-     {
-       ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection, &firstChar, &lastChar);
-       if (ok)
-	 if (pDoc == LoadedDocument[document - 1])
-	   {
-	     *selectedElement = (Element) firstSelection;
-	     *firstCharacter = firstChar;
-	     if (lastSelection == firstSelection)
-	       if (lastChar > 1)
-		 *lastCharacter = lastChar - 1;
-	     if (firstChar != 0 && lastChar == 0)
-	       *lastCharacter = firstSelection->ElVolume;
-	   }
-     }
+      ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection,
+				&firstChar, &lastChar);
+      if (ok)
+	if (pDoc == LoadedDocument[document - 1])
+	  {
+	    *selectedElement = (Element) firstSelection;
+	    *firstCharacter = firstChar;
+	    if (lastSelection == firstSelection)
+	      {
+		/* only one element */
+		if (SelPosition)
+		  /* a position */
+		  *lastCharacter = firstChar - 1;
+		else
+		  *lastCharacter = lastChar;
+	      }
+	    else if (firstChar > 0 && lastChar == 0)
+	      /* the selection starts in the middle of the element */
+	      *lastCharacter = firstSelection->ElVolume;
+	  }
+    }
 }
 
 
@@ -480,11 +436,9 @@ int                *lastCharacter;
    in a given document. If this element is a Text element and if only a
    substring is selected, return also the rank of the first and last
    characters in the selection.
-
    Parameters:
    document: the document for which the selection is asked.
    selectedElement: the current element.
-
    Return parameters:
    selectedElement: the next selected element, NULL if
    this was the last selected element in the document.
@@ -493,52 +447,43 @@ int                *lastCharacter;
    lastCharacter: rank of the last character in the selection, or 0 if the
    whole element is in the selection.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaGiveNextSelectedElement (Document document, Element * selectedElement, int *firstCharacter, int *lastCharacter)
-
-#else  /* __STDC__ */
-void                TtaGiveNextSelectedElement (document, selectedElement, firstCharacter, lastCharacter)
-Document            document;
-Element            *selectedElement;
-int                *firstCharacter;
-int                *lastCharacter;
-
-#endif /* __STDC__ */
-
+void TtaGiveNextSelectedElement (Document document, Element *selectedElement,
+				 int *firstCharacter, int *lastCharacter)
 {
-   PtrDocument         pDoc;
-   PtrElement          pEl;
-   PtrElement          firstSelection, lastSelection;
-   int                 firstChar, lastChar;
-   ThotBool            ok;
+  PtrDocument         pDoc;
+  PtrElement          pEl;
+  PtrElement          firstSelection, lastSelection;
+  int                 firstChar, lastChar;
+  ThotBool            ok;
 
-   UserErrorCode = 0;
-   *firstCharacter = 0;
-   *lastCharacter = 0;
-   /* Checks the parameter document */
-   if (document < 1 || document > MAX_DOCUMENTS)
-     TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-     TtaError (ERR_invalid_document_parameter);
-   else
+  UserErrorCode = 0;
+  *firstCharacter = 0;
+  *lastCharacter = 0;
+  /* Checks the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
       /* Parameter document is correct */
-     {
-	ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection, &firstChar, &lastChar);
-	if (!ok)
-	   *selectedElement = NULL;
-	else if (pDoc != LoadedDocument[document - 1])
-	   *selectedElement = NULL;
-	else
-	  {
-	     pEl = NextInSelection ((PtrElement) * selectedElement, lastSelection);
-	     *selectedElement = (Element) pEl;
-	     if (pEl == lastSelection && lastChar > 0)
-	       {
-		  *firstCharacter = 1;
-		  *lastCharacter = lastChar - 1;
-	       }
-	  }
-     }
+      ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection,
+				&firstChar, &lastChar);
+      if (!ok)
+	*selectedElement = NULL;
+      else if (pDoc != LoadedDocument[document - 1])
+	*selectedElement = NULL;
+      else
+	{
+	  pEl = NextInSelection ((PtrElement) *selectedElement, lastSelection);
+	  *selectedElement = (Element) pEl;
+	  if (pEl == lastSelection && lastChar > 0)
+	    {
+	      *firstCharacter = 1;
+	      *lastCharacter = lastChar;
+	    }
+	}
+    }
 }
 
 
@@ -546,26 +491,14 @@ int                *lastCharacter;
    TtaGiveNextElement
 
    Returns the element that follows a given element in the selection order
-
    Parameters:
    document: the document for which the selection is asked.
    element: the current element.
    last: end of the range to be searched, usually last selected element.
-
    Return parameters:
    element: the next element in the selection order. NULL if not found.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaGiveNextElement (Document document, Element * element, Element last)
-
-#else  /* __STDC__ */
-void                TtaGiveNextElement (document, element, last)
-Document            document;
-Element            *element;
-Element		    last;
-
-#endif /* __STDC__ */
-
+void TtaGiveNextElement (Document document, Element *element, Element last)
 {
    PtrElement          pEl;
 
@@ -587,66 +520,66 @@ Element		    last;
 
 
 /*----------------------------------------------------------------------
-   TtaGiveLastSelectedElement
+  TtaGiveLastSelectedElement
 
-   Returns the last element in the current selection in a given document.
-   If this element is a Text element and if only a substring is selected,
-   return also the rank of the first and last characters in the selection.
-
-   Parameter:
-   document: the document for which the selection is asked.
-
-   Return values:
-   selectedElement: the last selected element, NULL if no element is
-   selected in the document.
-   firstCharacter: rank of the first character in the selection, or 0 if the
-   whole element is in the selection.
-   lastCharacter: rank of the last character in the selection, or 0 if the
-   whole element is in the selection.
+  Returns the last element in the current selection in a given document.
+  If this element is a Text element and if only a substring is selected,
+  return also the rank of the first and last characters in the selection.
+  Parameter:
+  document: the document for which the selection is asked.
+  Return values:
+  selectedElement: the last selected element, NULL if no element is
+  selected in the document.
+  firstCharacter: rank of the first selected character in the element, or
+  0 if the whole element is in the selection.
+  lastCharacter: rank of the last selected character in the element, or
+  0 if the whole element is in the selection, or firstCharacter - 1 for
+  a position.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaGiveLastSelectedElement (Document document, Element * selectedElement, int *firstCharacter, int *lastCharacter)
-
-#else  /* __STDC__ */
-void                TtaGiveLastSelectedElement (document, selectedElement, firstCharacter, lastCharacter)
-Document            document;
-Element            *selectedElement;
-int                *firstCharacter;
-int                *lastCharacter;
-
-#endif /* __STDC__ */
-
+void TtaGiveLastSelectedElement (Document document, Element *selectedElement,
+				 int *firstCharacter, int *lastCharacter)
 {
-   PtrDocument         pDoc;
-   PtrElement          firstSelection, lastSelection;
-   int                 firstChar, lastChar;
-   ThotBool            ok;
+  PtrDocument         pDoc;
+  PtrElement          firstSelection, lastSelection;
+  int                 firstChar, lastChar;
+  ThotBool            ok;
 
-   UserErrorCode = 0;
-   *selectedElement = NULL;
-   *firstCharacter = 0;
-   *lastCharacter = 0;
-   /* Checks the parameter document */
-   if (document < 1 || document > MAX_DOCUMENTS)
-     TtaError (ERR_invalid_document_parameter);
-   else if (LoadedDocument[document - 1] == NULL)
-     TtaError (ERR_invalid_document_parameter);
-   else
+  UserErrorCode = 0;
+  *selectedElement = NULL;
+  *firstCharacter = 0;
+  *lastCharacter = 0;
+  /* Checks the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
       /* Parameter document is correct */
-     {
-       ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection, &firstChar, &lastChar);
-       if (ok)
-	 if (pDoc == LoadedDocument[document - 1])
-	   {
-	     *selectedElement = (Element) lastSelection;
-	     if (lastSelection == firstSelection)
-	       {
-		 *firstCharacter = firstChar;
-		 if (lastChar > 1)
-		   *lastCharacter = lastChar - 1;
-	       }
-	     if (lastChar > 0)
-	       *lastCharacter = lastChar - 1;
-	   }
-     }
+      ok = GetCurrentSelection (&pDoc, &firstSelection, &lastSelection,
+				&firstChar, &lastChar);
+      if (ok)
+	if (pDoc == LoadedDocument[document - 1])
+	  {
+	    *selectedElement = (Element) lastSelection;
+	    if (lastSelection == firstSelection)
+	      {
+		/* only one element */
+		*firstCharacter = firstChar;
+		if (SelPosition)
+		  /* a position */
+		  *lastCharacter = firstChar - 1;
+		else
+		  *lastCharacter = lastChar;
+	      }
+	    else if (lastChar > 0)
+	      {
+		/* the selection ends in the middle of the element */
+		*firstCharacter = 1;
+		*lastCharacter = lastChar;
+	      }
+	  }
+    }
 }
+
+
