@@ -217,11 +217,11 @@ void RedisplayCopies (PtrElement pEl, PtrDocument pDoc, ThotBool redisplay)
 	   /* cherche d'abord le schema de presentation pour l'element */
 	   SearchPresSchema (pEl, &pSchP, &rule, &pSchS, pDoc);
 	   if (pSchP != NULL)
-	      if (pSchP->PsElemTransmit[rule - 1] > 0)
+	      if (pSchP->PsElemTransmit->Num[rule - 1] > 0)
 		 /* il y a une regle Transmit pour ce type d'element */
 		{
 		   /* applique la regle Transmit */
-		   pTrans = &(pSchP->PsTransmElem[pSchP->PsElemTransmit[rule - 1] - 1]);
+		   pTrans = &(pSchP->PsTransmElem[pSchP->PsElemTransmit->Num[rule - 1] - 1]);
 		   TransmitElementContent (pEl, pDoc, pTrans->TeTargetAttr,
 					   pTrans->TeTargetDoc, pSchS);
 		}
@@ -321,7 +321,7 @@ void AssignPairIdentifiers (PtrElement pRoot, PtrDocument pDoc)
    if (pRoot->ElTerminal)
      {
 	if (pRoot->ElLeafType == LtPairedElem)
-	   if (pRoot->ElStructSchema->SsRule[pRoot->ElTypeNumber - 1].SrFirstOfPair)
+	   if (pRoot->ElStructSchema->SsRule->SrElem[pRoot->ElTypeNumber - 1]->SrFirstOfPair)
 	      /* c'est un element de debut de paire */
 	     {
 		/* on cherche son element de fin */
@@ -567,69 +567,6 @@ void DeHolophrast (PtrElement pEl, PtrDocument pDoc)
    RedisplayDocViews (pDoc);
 }
 
-
-/*----------------------------------------------------------------------
-   Assure la presence d'un atome texte de la bonne langue a la      
-   position indiquee (qui doit etre un atome texte), cree l'attribut
-   langue sur cet atome si setLangAttr est vrai.
-  ----------------------------------------------------------------------*/
-void NewTextLanguage (PtrAbstractBox pAb, int charIndex, Language lang,
-		      ThotBool setLangAttr)
-{
-   PtrElement          pEl, pNextEl, pSecond, pNext;
-   PtrDocument         pDoc;
-   PtrAttribute        pAttr;
-   int                 len;
-
-   pEl = pAb->AbElement;
-   if (pEl != NULL && pEl->ElStructSchema != NULL)
-     if (pEl->ElLeafType == LtText)
-     {
-	pDoc = DocumentOfElement (pEl);
-	if (pEl->ElTextLength > 0)
-	  {
-	     pNext = pEl->ElNext;
-	     SplitTextElement (pEl, charIndex, pDoc, FALSE, &pNextEl, FALSE);
-	     BuildAbsBoxSpliText (pEl, pNextEl, pNext, pDoc);
-	     if (pEl->ElTextLength > 0 && pNextEl->ElTextLength > 0)
-	       {
-	          pNext = pNextEl->ElNext;
-		  SplitTextElement (pNextEl, 1, pDoc, FALSE, &pSecond, FALSE);
-		  BuildAbsBoxSpliText (pNextEl, pSecond, pNext, pDoc);
-		  pEl = pNextEl;
-	       }
-	     else if (pEl->ElTextLength > 0)
-		pEl = pNextEl;
-	     AbstractImageUpdated (pDoc);
-	  }
-	if (pEl != NULL && pEl->ElStructSchema != NULL)
-	  {
-	    /* change la langue dans la feuille de texte */
-	    ChangeLanguage (pDoc, pEl, lang, TRUE);
-	    /* met l'attribut langue sur l'element */
-	    GetAttribute (&pAttr);
-	    pAttr->AeAttrSSchema = pEl->ElStructSchema;
-	    pAttr->AeAttrNum = 1;
-	    pAttr->AeDefAttr = FALSE;
-	    pAttr->AeAttrType = AtTextAttr;
-	    if (setLangAttr)
-	      {
-		GetTextBuffer (&pAttr->AeAttrText);
-		CopyStringToText (TtaGetLanguageName (lang), pAttr->AeAttrText, &len);
-	      }
-	    else
-	      {
-		pAttr->AeAttrText = NULL;
-	      }
-	    AttachAttrWithValue (pEl, pDoc, pAttr);
-	    DeleteAttribute (NULL, pAttr);
-	    AbstractImageUpdated (pDoc);
-	    RedisplayDocViews (pDoc);
-	    SelectElement (pDoc, pEl, FALSE, FALSE);
-	  }
-     }
-}
-
 /*----------------------------------------------------------------------
    CompleteElement cree dans l'element pEl les elements absents    
    obligatoires.                                           
@@ -640,7 +577,7 @@ ThotBool CompleteElement (PtrElement pEl, PtrDocument pDoc)
 {
    PtrElement          pComponent, pNewEl, pLastEl, pPrev, pClose;
    PtrElement          pSibling, pPage;
-   SRule              *pSRule;
+   PtrSRule            pSRule;
    NotifyElement       notifyEl;
    Document            doc;
    int                 nElExist, nElems, min, comp, NSiblings;
@@ -648,9 +585,9 @@ ThotBool CompleteElement (PtrElement pEl, PtrDocument pDoc)
 
    ret = FALSE;
    /* regle definissant le type de l'element dans son schema de structure */
-   pSRule = &pEl->ElStructSchema->SsRule[pEl->ElTypeNumber - 1];
+   pSRule = pEl->ElStructSchema->SsRule->SrElem[pEl->ElTypeNumber - 1];
    while (pSRule->SrConstruct == CsIdentity)
-      pSRule = &pEl->ElStructSchema->SsRule[pSRule->SrIdentRule - 1];
+      pSRule = pEl->ElStructSchema->SsRule->SrElem[pSRule->SrIdentRule - 1];
    /* traitement selon le constructeur : seuls les agregats et les listes */
    /* demandent un traitement. */
    doc = IdentDocument (pDoc);
@@ -1223,7 +1160,7 @@ ThotBool BreakElement (PtrElement pElReplicate, PtrElement pSplitEl,
 			     element */
 			  notifyEl.element = (Element) (pE->ElParent);
 			  notifyEl.position = NSiblings;
-			  notifyEl.info = 0;
+			  notifyEl.info = 2;
 			  CallEventType ((NotifyEvent *) (&notifyEl), FALSE);
 			  InsertElementAfter (pPrevEl, pClose);
 			  pPrevEl = pClose;
@@ -1409,7 +1346,7 @@ void SelectRange (PtrDocument pDoc, PtrElement firstSel, PtrElement lastSel,
    else if ((lastChar == 0 && firstChar == 0) || lastSel != firstSel)
       SelectElement (pDoc, firstSel, TRUE, TRUE);
    else
-      SelectString (pDoc, firstSel, 1, lastChar - 1);
+      SelectString (pDoc, firstSel, 1, lastChar);
 
    if (lastSel != firstSel)
      {
@@ -1420,6 +1357,6 @@ void SelectRange (PtrDocument pDoc, PtrElement firstSel, PtrElement lastSel,
       else if (lastChar == 0)
 	 ExtendSelection (lastSel, lastChar, TRUE, FALSE, FALSE);
       else
-	 ExtendSelection (lastSel, lastChar - 1, TRUE, FALSE, FALSE);
+	 ExtendSelection (lastSel, lastChar, TRUE, FALSE, FALSE);
      }
 }
