@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA 1998-2001
+ *  (c) COPYRIGHT INRIA 1998-2002
  *  Please read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -412,9 +412,9 @@ void AddAttrEditOpInHistory (PtrAttribute pAttr, PtrElement pEl,
 			     ThotBool removeWhenUndoing)
 {
    PtrEditOperation	editOp = NULL;
-   PtrAttribute		pCopy, pOldAttr, pElAttr;
+   PtrAttribute		pCopy, pElAttr;
    int                  rank;
-   ThotBool		merge;
+   ThotBool		done;
 
    if (!pEl && !pAttr)
       return;
@@ -424,27 +424,80 @@ void AddAttrEditOpInHistory (PtrAttribute pAttr, PtrElement pEl,
 
    /* if the previous operation recorded is about the same attribute for the
       same element, just modify the previous operation */
-   merge = FALSE;
-   if (!save || !removeWhenUndoing)
-     if (pDoc->DocLastEdit)
-       {
+   done = FALSE;
+   if (pDoc->DocLastEdit)
+     {
        editOp = pDoc->DocLastEdit;
-       if (editOp->EoType == EtAttribute)
-	  if (editOp->EoElement == pEl)
+       if (editOp->EoType == EtAttribute && editOp->EoElement == pEl)
+	 /* the operation previously recorded is related to an attribute for
+	    the same element */
+	 {
+	   if (editOp->EoCreatedAttribute && !editOp->EoSavedAttribute &&
+	       editOp->EoCreatedAttribute->AeAttrSSchema == pAttr->AeAttrSSchema &&
+	       editOp->EoCreatedAttribute->AeAttrNum == pAttr->AeAttrNum)
+	     /* the previous operation was the creation of this attribute */
 	     {
-	     pOldAttr = NULL;
-	     if (save && editOp->EoSavedAttribute == NULL)
-		pOldAttr = editOp->EoCreatedAttribute;
-	     else if (removeWhenUndoing && editOp->EoCreatedAttribute == NULL)
-		pOldAttr = editOp->EoSavedAttribute;
-	     if (pOldAttr)
-		if (pAttr->AeAttrSSchema == pOldAttr->AeAttrSSchema)
-		   if (pAttr->AeAttrNum == pOldAttr->AeAttrNum)
-		      merge = TRUE;
+	       if (!save)
+		 /* creating the same attribute again? Ignore */
+		 done = TRUE;
+	       else if (removeWhenUndoing)
+		 /* replacing the attribute. No need to record it */
+		 done = TRUE;
+	       else
+		 /* deleting the same attribute. Remove the previous record
+		    about creating the attribute */
+		 {
+		   pDoc->DocLastEdit = editOp->EoPreviousOp;
+		   editOp->EoPreviousOp->EoNextOp = NULL;
+		   editOp->EoElement = NULL;
+		   editOp->EoCreatedAttribute = NULL;
+		   TtaFreeMemory (editOp);
+		   done = TRUE;
+		 }
 	     }
-       }
+	   else if (editOp->EoSavedAttribute && !editOp->EoCreatedAttribute &&
+		    editOp->EoSavedAttribute->AeAttrSSchema == pAttr->AeAttrSSchema &&
+		    editOp->EoSavedAttribute->AeAttrNum == pAttr->AeAttrNum)
+	     /* the previous operation was the deletion of this attribute */
+	     {
+	       if (!removeWhenUndoing)
+		 /* deleting the same attribute again? Ignore */
+		 done = TRUE;
+	       else if (save)
+		 /* replacing a deleted attribute? Ignore */
+		 done = TRUE;
+	       else
+		 /* creating the same attribute */
+		 {
+		   editOp->EoCreatedAttribute = pAttr;
+		   done = TRUE;
+		 }
+	     }
+	   else if (editOp->EoSavedAttribute && editOp->EoCreatedAttribute &&
+		    editOp->EoSavedAttribute->AeAttrSSchema == pAttr->AeAttrSSchema &&
+		    editOp->EoSavedAttribute->AeAttrNum == pAttr->AeAttrNum)
+	     /* the previous operation was a replacement of this attribute */
+	     {
+	       if (!save)
+		 /* creating the same attribute? Ignore */
+		 done = TRUE;
+	       else if (removeWhenUndoing)
+		 /* replacing this attribute again */
+		 {
+		   editOp->EoCreatedAttribute = pAttr;
+		   done = TRUE;
+		 }
+	       else
+		 /* deleting this attribute. It's like a creation */
+		 {
+		   editOp->EoCreatedAttribute = NULL;
+		   done = TRUE;
+		 }
+	     }
+	 }
+     }
 
-   if (!merge)
+   if (!done)
      {
        /* create a new operation descriptor in the history */
        editOp = (PtrEditOperation) TtaGetMemory (sizeof (EditOperation));
@@ -468,20 +521,18 @@ void AddAttrEditOpInHistory (PtrAttribute pAttr, PtrElement pEl,
 	   rank++;
 	 }
        editOp->EoAttrRank = rank;
-     }
-
-   if (removeWhenUndoing)
-     editOp->EoCreatedAttribute = pAttr;
-
-   if (save)
-     /* copy the attribute concerned by the operation and attach it to the
-        operation descriptor */
-     {
-       pCopy = AddAttrToElem (NULL, pAttr, NULL);
-       editOp->EoSavedAttribute = pCopy;
-       /* if older editing operations in the history refer to attribute that
-	  has been copied, change these references to the copy */
-       ChangeAttrPointersOlderEdits (editOp, pAttr, pCopy);
+       if (removeWhenUndoing)
+	 editOp->EoCreatedAttribute = pAttr;
+       if (save)
+	 /* copy the attribute concerned by the operation and attach it to the
+	    operation descriptor */
+	 {
+	   pCopy = AddAttrToElem (NULL, pAttr, NULL);
+	   editOp->EoSavedAttribute = pCopy;
+	   /* if older editing operations in the history refer to attribute
+	      that has been copied, change these references to the copy */
+	   ChangeAttrPointersOlderEdits (editOp, pAttr, pCopy);
+	 }
      }
 }
 
@@ -696,7 +747,7 @@ ThotBool CloseHistorySequence (PtrDocument pDoc)
    Cancel the last sequence of editing operations registered in the
    editing history of document.
   ----------------------------------------------------------------------*/
-void                CancelLastSequenceFromHistory (PtrDocument pDoc)
+void CancelLastSequenceFromHistory (PtrDocument pDoc)
 {
    ThotBool	stop;
 
@@ -755,7 +806,7 @@ static void AttachAttr (PtrElement pEl, PtrAttribute pAttr, int rank,
    If reverse, the editing operation descriptor will decribe the reverse
    editing operation when returning.
   ----------------------------------------------------------------------*/
-static void	UndoOperation (ThotBool undo, Document doc, ThotBool reverse)
+static void UndoOperation (ThotBool undo, Document doc, ThotBool reverse)
 {
    PtrEditOperation	editOp;
    PtrElement		pEl, pSibling,
@@ -765,8 +816,10 @@ static void	UndoOperation (ThotBool undo, Document doc, ThotBool reverse)
    PtrDocument		pDoc;
    PtrEditOperation	queue;
    NotifyElement	notifyEl;
+   NotifyOnValue        notifyGraph;
    NotifyAttribute	notifyAttr;
    int			i, nSiblings;
+   ThotBool             replacePoly;
 
    newParent = NULL;
    newPreviousSibling = NULL;
@@ -804,7 +857,7 @@ static void	UndoOperation (ThotBool undo, Document doc, ThotBool reverse)
           if (editOp->EoFirstSelectedEl == editOp->EoLastSelectedEl)
    	     i = editOp->EoLastSelectedChar;
           else
-   	     i = TtaGetTextLength ((Element)(editOp->EoFirstSelectedEl));
+   	     i = TtaGetElementVolume ((Element)(editOp->EoFirstSelectedEl));
           TtaSelectString (doc, (Element)(editOp->EoFirstSelectedEl),
 			   editOp->EoFirstSelectedChar, i);
           }
@@ -932,9 +985,15 @@ static void	UndoOperation (ThotBool undo, Document doc, ThotBool reverse)
          newCreatedElement = NULL;
          newSavedElement = NULL;
 	 }
+      replacePoly = FALSE;
       if (editOp->EoCreatedElement)
          {
          pEl = editOp->EoCreatedElement;
+	 if (editOp->EoSavedElement &&
+	     pEl->ElTerminal && pEl->ElLeafType == LtPolyLine &&
+	     editOp->EoSavedElement->ElTerminal &&
+	     editOp->EoSavedElement->ElLeafType == LtPolyLine)
+	   replacePoly = TRUE;
          /* tell the application that an element will be removed from the
             abstract tree */
          SendEventSubTree (TteElemDelete, pDoc, pEl,
@@ -973,17 +1032,29 @@ static void	UndoOperation (ThotBool undo, Document doc, ThotBool reverse)
       /* insert the saved element in the abstract tree */
       if (editOp->EoSavedElement)
          {
+	 pEl = editOp->EoSavedElement;
          if (editOp->EoPreviousSibling)
-            TtaInsertSibling ((Element)(editOp->EoSavedElement),
+            TtaInsertSibling ((Element)(pEl),
 			      (Element)(editOp->EoPreviousSibling),FALSE, doc);
          else
-            TtaInsertFirstChild ((Element *)&(editOp->EoSavedElement),
+            TtaInsertFirstChild ((Element *)&(pEl),
 				 (Element)(editOp->EoParent), doc);
          /* send event ElemPaste.Post to the application. -1 means that this
 	    is not really a Paste operation but an Undo operation. */
-         NotifySubTree (TteElemPaste, pDoc, editOp->EoSavedElement, -1);
+         NotifySubTree (TteElemPaste, pDoc, pEl, -1);
+/******/
+	 if (replacePoly)
+	   {
+	   notifyGraph.event = TteElemGraphModify;
+	   notifyGraph.document = doc;
+	   notifyGraph.element = (Element)(pEl->ElParent); /***** all ascendants ***/
+	   notifyGraph.target = (Element) pEl;
+	   notifyGraph.value = 0;   /******/
+	   CallEventType ((NotifyEvent *) & notifyGraph, FALSE);
+	   }
+/******/
 	 if (reverse)
-	    newCreatedElement = editOp->EoSavedElement;
+	    newCreatedElement = pEl;
          editOp->EoSavedElement = NULL;
          }
       if (reverse)
