@@ -10,7 +10,6 @@
  * Authors: I. Vatton, N. Layaida (INRIA)
  *          R. Guetari (W3C/INRIA) - Previous Windows version
  */
- 
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
@@ -18,12 +17,11 @@
 #include "frame.h"
 #include "libmsg.h"
 #include "message.h"
-#include "application.h"
-#include "xpm.h"
-#include "thotcolor.h"
 #ifdef _WINDOWS
 #include "winsys.h"
 #endif /* _WINDOWS */
+#include "xpm.h"
+#include "thotcolor.h"
 
 #define THOT_EXPORT extern
 #include "picture_tv.h"
@@ -39,9 +37,17 @@
 #ifdef _WINDOWS
 #include "wininclude.h"
 #endif /* _WINDOWS */
+#include "application.h"
+
 
 #define	MAXCOLORMAPSIZE		256
-#define scale 65536 / MAXCOLORMAPSIZE;
+
+#ifndef _WINDOWS
+#define COLORMAPSCALE 65536 / MAXCOLORMAPSIZE;
+#else/*_WINDOWS*/
+#define COLORMAPSCALE 1;
+#endif /*_WINDOWS*/
+
 #define CM_RED		0
 #define CM_GREEN	1
 #define CM_BLUE		2
@@ -75,12 +81,12 @@ static int          stack[(1 << (MAX_LWZ_BITS)) * 2], *sp = stack;
 /*----------------------------------------------------------------------
   ReadGifImage
   ----------------------------------------------------------------------*/
-static unsigned char* ReadGifImage (FILE *fd, int len, int height,
+static unsigned char *ReadGifImage (FILE *fd, int *w, int *h,
 				    unsigned char clmap[3][MAXCOLORMAPSIZE],
 				    int interlace, int ignore)
 {
   unsigned char       c;
-  int                 v;
+  int                 v, len, height, lines;
   int                 xpos = 0, ypos = 0, pass = 0;
   unsigned char      *data;
   unsigned char      *dptr;
@@ -90,26 +96,22 @@ static unsigned char* ReadGifImage (FILE *fd, int len, int height,
     return (NULL);
   if (LWZReadByte (fd, TRUE, c) < 0)
     return (NULL);
-
    /* If this is an "uninteresting picture" ignore it */
   if (ignore)
-    {
-      while (LWZReadByte (fd, FALSE, c) >= 0)
-	;
-      return (NULL);
-    }
-
+    return (NULL);
+  len = *w;
+  height = *h;
+  lines = 1;
   data = (unsigned char*) TtaGetMemory (sizeof (unsigned char) * len * height);
-
   if (data == NULL)
     return (NULL);
-
   while ((v = LWZReadByte (fd, FALSE, c)) >= 0)
     {
+      /* store the byte value */
       dptr = (unsigned char *) (data + (ypos * len) + xpos);
       *dptr = (unsigned char) v;
       
-      ++xpos;
+      xpos++;
       if (xpos == len)
 	{
 	  xpos = 0;
@@ -131,7 +133,7 @@ static unsigned char* ReadGifImage (FILE *fd, int len, int height,
 	      
 	      if (ypos >= height)
 		{
-		  ++pass;
+		    ++pass;
 		  switch (pass)
 		    {
 		    case 1:
@@ -144,28 +146,36 @@ static unsigned char* ReadGifImage (FILE *fd, int len, int height,
 		      ypos = 1;
 		      break;
 		    default:
-		      goto fini;
+		      break;
 		    }
 		}
 	    }
 	  else
-	    ++ypos;
+	    ypos++;
+
+	  if (ypos >= height)
+	    break;
+	  else
+	    /* read a new line */
+	    lines++;
 	}
-      if (ypos >= height)
-	break;
     }
 
- fini:
   if (LWZReadByte (fd, FALSE, c) >= 0)
-    fprintf (stderr, "gifhandler: too much input data, ignoring extra...\n");
+    {
+      fprintf (stderr, "gifhandler: too much input data, ignoring extra...\n");
+      /* return the number of lines read */
+      *h = lines;
+    }
   return (data);
 }
 
+#ifdef _GL
 /*----------------------------------------------------------------------
   ReadGIF
   ----------------------------------------------------------------------*/
 unsigned char *ReadGIF (FILE *fd, int *w, int *h, int *ncolors, int *cpp,
-			ThotColorStruct colrs[512])
+			ThotColorStruct *colrs)
 {
   unsigned char       buf[16];
   unsigned char      *data;
@@ -207,19 +217,135 @@ unsigned char *ReadGIF (FILE *fd, int *w, int *h, int *ncolors, int *cpp,
 	 return (NULL);
        for (i = 0; i < (int) GifScreen.BitPixel; i++)
 	 {
+	   colrs[i].red   = GifScreen.ColorMap[0][i];
+	   colrs[i].green = GifScreen.ColorMap[1][i];
+	   colrs[i].blue  = GifScreen.ColorMap[2][i];
+	 }
+
+       for (i = GifScreen.BitPixel; i < MAXCOLORMAPSIZE; i++)
+	 {
+	   colrs[i].red = 0;
+	   colrs[i].green = 0;
+	   colrs[i].blue = 0;
+	 }
+     }
+
+   for (;;)
+     {
+       if (!ReadOK (fd, &c, 1))
+	 return (NULL);
+
+       if (c == ';')
+	 { /* GIF terminator */
+	   if (imageCount < imageNumber)
+             return (NULL);
+	   break;
+	 }
+
+       if (c == '!')
+	 { /* Extension */
+	   if (!ReadOK (fd, &c, 1))
+             return (NULL);
+	   DoExtension (fd, c);
+	   continue;
+	 }
+
+       if (c != ',')
+	 continue;
+       ++imageCount;
+       if (!ReadOK (fd, buf, 9))
+	 return (NULL);
+
+       useGlobalColormap = !BitSet (buf[8], LOCALCOLORMAP);
+       bitPixel = 1 << ((buf[8] & 0x07) + 1);
+       *w = LM_to_uint (buf[4], buf[5]);
+       *h = LM_to_uint (buf[6], buf[7]);
+       if (!useGlobalColormap)
+	 {
+	   if (ReadColorMap (fd, bitPixel, localColorMap))
+             return (NULL);
+	   for (i = 0; i < bitPixel; i++)
+	     { 
+	       colrs[i].red   = localColorMap[0][i];
+	       colrs[i].green = localColorMap[1][i];
+	       colrs[i].blue  = localColorMap[2][i];
+	     }
+
+	   for (i = bitPixel; i < MAXCOLORMAPSIZE; i++)
+	     {
+	       colrs[i].red   = 0;
+	       colrs[i].green = 0;
+	       colrs[i].blue  = 0;
+	     }
+	   data = ReadGifImage (fd, w, h, localColorMap,
+				BitSet (buf[8], INTERLACE), imageCount != imageNumber);
+	   return (data);	/* anticipating the exit to prevent gif video */
+	 }
+       else
+	 {
+	   data = ReadGifImage (fd, w, h, GifScreen.ColorMap,
+				BitSet (buf[8], INTERLACE), imageCount != imageNumber);
+	   return (data);	/* anticipating the exit to prevent gif video */
+	 }
+     }
+   return (data);
+}
+#else /*_GL*/
+/*----------------------------------------------------------------------
+  ReadGIF
+  ----------------------------------------------------------------------*/
+unsigned char *ReadGIF (FILE *fd, int *w, int *h, int *ncolors, int *cpp,
+			ThotColorStruct *colrs)
+{
+  unsigned char       buf[16];
+  unsigned char      *data;
+  unsigned char       c;
+  unsigned char       localColorMap[3][MAXCOLORMAPSIZE];
+  int                 useGlobalColormap;
+  int                 bitPixel;
+  int                 imageCount = 0;
+  char                version[4];
+  int                 imageNumber = 1;
+  int                 i;
+
+  *w = 0;
+  *h = 0;
+  data = NULL;
+  if (!ReadOK (fd, buf, 6)) 
+    return (NULL);
+
+  if (strncmp ((char *) buf, "GIF", 3) != 0)
+    return (NULL);
+   strncpy (version, (char *) buf + 3, 3);
+   version[3] = EOS;
+   if ((strcmp (version, "87a") != 0) && (strcmp (version, "89a") != 0))
+     return (NULL);
+   
+   if (!ReadOK (fd, buf, 7))
+     return (NULL);
+
+   GifScreen.Width  = LM_to_uint (buf[0], buf[1]);
+   GifScreen.Height = LM_to_uint (buf[2], buf[3]);
+   GifScreen.BitPixel = 2 << (buf[4] & 0x07);
+   *ncolors = GifScreen.BitPixel;
+   GifScreen.ColorResolution = (((buf[4] & 0x70) >> 3) + 1);
+   GifScreen.Background = buf[5];
+   GifScreen.AspectRatio = buf[6];
+   if (BitSet (buf[4], LOCALCOLORMAP))
+     { /* Global Colormap */
+       if (ReadColorMap (fd, GifScreen.BitPixel, GifScreen.ColorMap))
+	 return (NULL);
+       for (i = 0; i < (int) GifScreen.BitPixel; i++)
+	 {
+	   colrs[i].red   = GifScreen.ColorMap[0][i] * COLORMAPSCALE;
+	   colrs[i].green = GifScreen.ColorMap[1][i] * COLORMAPSCALE;
+	   colrs[i].blue  = GifScreen.ColorMap[2][i] * COLORMAPSCALE;
 #ifndef _WINDOWS
-	   colrs[i].red   = GifScreen.ColorMap[0][i] * scale;
-	   colrs[i].green = GifScreen.ColorMap[1][i] * scale;
-	   colrs[i].blue  = GifScreen.ColorMap[2][i] * scale;
 	   colrs[i].pixel = i;
 #ifndef _GTK
 	   colrs[i].flags = DoRed | DoGreen | DoBlue;
 #endif /* _GTK */
-#else  /* _WINDOWS */
-	   colrs[i].red   = GifScreen.ColorMap[0][i];
-	   colrs[i].green = GifScreen.ColorMap[1][i];
-	   colrs[i].blue  = GifScreen.ColorMap[2][i];
-#endif /* _WINDOWS */
+#endif  /* _WINDOWS */
 	 }
 
        for (i = GifScreen.BitPixel; i < MAXCOLORMAPSIZE; i++)
@@ -272,18 +398,14 @@ unsigned char *ReadGIF (FILE *fd, int *w, int *h, int *ncolors, int *cpp,
              return (NULL);
 	   for (i = 0; i < bitPixel; i++)
 	     {
+	       colrs[i].red   = localColorMap[0][i] * COLORMAPSCALE;
+	       colrs[i].green = localColorMap[1][i] * COLORMAPSCALE;
+	       colrs[i].blue  = localColorMap[2][i] * COLORMAPSCALE;
 #ifndef _WINDOWS
-	       colrs[i].red   = localColorMap[0][i] * scale;
-	       colrs[i].green = localColorMap[1][i] * scale;
-	       colrs[i].blue  = localColorMap[2][i] * scale;
 	       colrs[i].pixel = i;
 #ifndef _GTK
 	       colrs[i].flags = DoRed | DoGreen | DoBlue;
 #endif /* _GTK */
-#else  /* _WINDOWS */
-	       colrs[i].red   = GifScreen.ColorMap[0][i];
-	       colrs[i].green = GifScreen.ColorMap[1][i];
-	       colrs[i].blue  = GifScreen.ColorMap[2][i];
 #endif /* _WINDOWS */
 	     }
 
@@ -299,22 +421,20 @@ unsigned char *ReadGIF (FILE *fd, int *w, int *h, int *ncolors, int *cpp,
 #endif /* _GTK */
 #endif /* _WINDOWS */
 	     }
-	   data = ReadGifImage (fd, LM_to_uint (buf[4], buf[5]),
-				LM_to_uint (buf[6], buf[7]), localColorMap,
+	   data = ReadGifImage (fd, w, h, localColorMap,
 				BitSet (buf[8], INTERLACE), imageCount != imageNumber);
 	   return (data);	/* anticipating the exit to prevent gif video */
 	 }
        else
 	 {
-	   data = ReadGifImage (fd, LM_to_uint (buf[4], buf[5]),
-				LM_to_uint (buf[6], buf[7]), GifScreen.ColorMap,
+	   data = ReadGifImage (fd, w, h, GifScreen.ColorMap,
 				BitSet (buf[8], INTERLACE), imageCount != imageNumber);
 	   return (data);	/* anticipating the exit to prevent gif video */
 	 }
      }
    return (data);
 }
-
+#endif /*_GL*/
 /*----------------------------------------------------------------------
   ReadColorMap
   ----------------------------------------------------------------------*/
@@ -530,35 +650,36 @@ int LWZReadByte (FILE *fd, int flag, int input_code_size)
    return code;
 }
 
-
+#ifndef _GTK
 /*----------------------------------------------------------------------
   highbit returns position of highest set bit in 'ul' as an integer (0-31),
   or -1 if none.     
   ----------------------------------------------------------------------*/
-static int          highbit (unsigned long ul)
+static int highbit (unsigned long ul)
 {
   int                 i;
 
   for (i = 31; ((ul & 0x80000000) == 0) && i >= 0; i--, ul <<= 1) ;
   return i;
 }
+#endif /* !_GTK */
 
 /*----------------------------------------------------------------------
   highbit16 returns position of highest set bit in 'ul' as an integer (0-31),
   or -1 if none.          
   ----------------------------------------------------------------------*/
-int                 highbit16 (unsigned long ul)
+int highbit16 (unsigned long ul)
 {
   int                 i;
 
   for (i = 15; ((ul & 0x8000) == 0) && i >= 0; i--, ul <<= 1) ;
   return i;
 }
-
+#ifndef _GTK
 /*----------------------------------------------------------------------
   nbbits returns the width of a bit PicMask.
   ----------------------------------------------------------------------*/
-static int          nbbits (unsigned long ul)
+static int nbbits (unsigned long ul)
 {
   while (!(ul & 1))
     ul >>= 1;
@@ -603,7 +724,7 @@ static int          nbbits (unsigned long ul)
       return (8);
    }
 }
-
+#endif /* !_GTK */
 #ifndef _WINDOWS
 /*----------------------------------------------------------------------
   Make a shape  of depth 1 for display from image data.
@@ -613,7 +734,9 @@ Pixmap MakeMask (Display *dsp, unsigned char *pixels, int w, int h,
 		 unsigned int bg, int bperpix)
 {
   XImage             *newmask;
+#ifndef _GTK
   ThotGC              tmp_gc;
+#endif /* !_GTK */
   Pixmap              PicMask;
   unsigned short     *spixels;
   unsigned char      *data_ptr, *max_data;
@@ -811,11 +934,12 @@ Pixmap MakeMask (Display *dsp, unsigned char *pixels, int w, int h,
   Make an image of appropriate depth for display from image data.
   The parameter ncolors gives the number of colors in the image.
   ----------------------------------------------------------------------*/
+#ifndef _GTK
 static XImage *MakeImage (Display *dsp, unsigned char *data, int width,
 			  int height, int depth, ThotColorStruct *colrs,
-			  int ncolors, ThotBool withAlpha)
+			  int ncolors, ThotBool withAlpha,
+			  ThotBool grayScale)
 {
-#ifndef _GTK
   XImage             *newimage = NULL;
   unsigned char      *bit_data, *bitp;
   unsigned long       c;
@@ -931,8 +1055,13 @@ static XImage *MakeImage (Display *dsp, unsigned char *data, int width,
 	    {
 	      /* read the RGB from the data descriptor */
 	      r = data[ind++];
-	      g = data[ind++];
-	      b = data[ind++];
+	      if (grayScale)
+		g = b = r;
+	      else
+		{
+		  g = data[ind++];
+		  b = data[ind++];
+		}
 	      if (withAlpha)
 		/* skip the alpha channel */
 		ind++;
@@ -987,8 +1116,13 @@ static XImage *MakeImage (Display *dsp, unsigned char *data, int width,
 	    {
 	      /* read the RGB from the data descriptor */
 	      r = data[ind++];
-	      g = data[ind++];
-	      b = data[ind++];
+	      if (grayScale)
+		g = b = r;
+	      else
+		{
+		  g = data[ind++];
+		  b = data[ind++];
+		}
 	      if (withAlpha)
 		/* skip the alpha channel */
 		ind++;
@@ -1025,8 +1159,8 @@ static XImage *MakeImage (Display *dsp, unsigned char *data, int width,
       return (None);
     }
    return (newimage);
-#endif /* !_GTK */
 }
+#endif /* !_GTK */
 #else /* _WINDOWS */
 /*----------------------------------------------------------------------
   Make an image of appropriate depth for display from image data.
@@ -1034,17 +1168,19 @@ static XImage *MakeImage (Display *dsp, unsigned char *data, int width,
   ----------------------------------------------------------------------*/
 HBITMAP WIN_MakeImage (HDC hDC, unsigned char *data, int width, int height,
 		       int depth, ThotColorStruct *colrs, int ncolors,
-		       ThotBool withAlpha)
+		       ThotBool withAlpha, ThotBool grayScale)
 {
   HBITMAP             newimage;
   unsigned char      *bit_data, *bitp;
-  unsigned char       r, g, b;
+  unsigned char       r, g, b, alpha;
   unsigned int        col;
   int                 temp, w, h, ind;
   int                 linepad;
   int                 bytesperline;
   int                 rshift, gshift, bshift;
+  unsigned short      rmask, gmask, bmask;
 
+  bit_data = NULL;
   switch (depth)
     {
     case 1:
@@ -1077,46 +1213,54 @@ HBITMAP WIN_MakeImage (HDC hDC, unsigned char *data, int width, int height,
       bytesperline = (width + linepad) * depth / 8;
       break;
 
+	case 15:
     case 16:
       bit_data = (unsigned char *) TtaGetMemory (width * height * 2);
       bitp   = bit_data;
       ind = 0; /* pixel index */
+	  if (depth == 15)
+	  {
+      rshift = 1;
+      gshift = 6;
+      bshift = 11;
+	  /* For 15 bit, Windows seems to be using 16 bit pixels */
+	  depth = 16;
+	  }
+	  else
+	  {
       rshift = 0;
       gshift = 5;
       bshift = 11;
+	  }
       for (w = (width * height); w > 0; w--)
 	{
 	  if (ncolors == 0)
 	    {
 	      /* read the RGB from the data descriptor */
 	      r = data[ind++];
-	      g = data[ind++];
-	      b = data[ind++];
-	      if (withAlpha)
-		/* skip the alpha channel */
-		ind++;
-	      if (IS_WIN95)
-	      temp = (((r * 255) & 63488) |
-		      (((g * 255) >> gshift) & 2016) |
-		      (((b * 255) >> bshift) & 31));
+	      if (grayScale)
+		g = b = r;
 	      else
-	      temp = (((r << 8) & 63488) |
-		      (((g << 8) >> gshift) & 2016) |
-		      (((b << 8) >> bshift) & 31));
+		{
+		  g = data[ind++];
+		  b = data[ind++];
+		}
+	      if (withAlpha)
+		    /* skip the alpha channel */
+		   alpha = data[ind++];
 	    }
 	  else
 	    {
 	      /* use one byte per pixel */
 	      col = data[ind++];
-	      if (IS_WIN95)
-		temp = (((colrs[col].red * 255) & 63488) |
-			(((colrs[col].green * 255) >> gshift) & 2016) |
-			((((colrs[col].blue * 255) >> bshift) & 31)));
-	      else
-		temp = (((colrs[col].red << 8) & 63488) |
-			(((colrs[col].green << 8) >> gshift) & 2016) |
-			((((colrs[col].blue << 8) >> bshift) & 31)));
+		  r = colrs[col].red;
+		  g = colrs[col].green;
+		  b = colrs[col].blue;
 	    }
+	  rmask = ((r << 8) >> rshift) & 0xf800;
+	  gmask = ((g << 8) >> gshift) & 0x7e0;
+	  bmask = ((b << 8) >> bshift) & 0x1f;
+	  temp = (rmask | gmask | bmask);
 	  *bitp++ = temp & 0xff;
 	  *bitp++ = (temp >> 8) & 0xff;
 	}
@@ -1133,8 +1277,13 @@ HBITMAP WIN_MakeImage (HDC hDC, unsigned char *data, int width, int height,
 		{
 		  /* read the RGB from the data descriptor */
 		  r = data[ind++];
-		  g = data[ind++];
-		  b = data[ind++];
+		  if (grayScale)
+		    g = b = r;
+		  else
+		    {
+		      g = data[ind++];
+		      b = data[ind++];
+		    }
 		  if (withAlpha)
 		    /* skip the alpha channel */
 		    ind++;
@@ -1168,8 +1317,13 @@ HBITMAP WIN_MakeImage (HDC hDC, unsigned char *data, int width, int height,
 		{
 		  /* read the RGB from the data descriptor */
 		  r = data[ind++];
-		  g = data[ind++];
-		  b = data[ind++];
+		  if (grayScale)
+		    g = b = r;
+		  else
+		    {
+		      g = data[ind++];
+		      b = data[ind++];
+		    }
 		  if (withAlpha)
 		    /* skip the alpha channel */
 		    ind++;
@@ -1187,7 +1341,7 @@ HBITMAP WIN_MakeImage (HDC hDC, unsigned char *data, int width, int height,
 		}
 	      *bitp++ = 0;
 	    }
-	}
+	  }
       break;
     }
 
@@ -1208,7 +1362,7 @@ HBITMAP WIN_MakeImage (HDC hDC, unsigned char *data, int width, int height,
   ----------------------------------------------------------------------*/
 Pixmap DataToPixmap (unsigned char *image_data, int width, int height,
 		     int ncolors, ThotColorStruct *colrs,
-		     ThotBool withAlpha)
+		     ThotBool withAlpha, ThotBool grayScale)
 {
 #ifndef _WINDOWS
 #ifndef _GTK
@@ -1221,13 +1375,12 @@ Pixmap DataToPixmap (unsigned char *image_data, int width, int height,
   if (size == 0)
     return ((Pixmap)NULL); 
   image = MakeImage (TtDisplay, image_data, width, height, TtWDepth, colrs,
-		     ncolors, withAlpha); 
+		     ncolors, withAlpha, grayScale); 
   img = XCreatePixmap (TtDisplay, TtRootWindow, width, height, TtWDepth);
   XPutImage (TtDisplay, img, GCimage, image, 0, 0, 0, 0, width, height);
   XDestroyImage (image);
 #else /* _GTK */
   Pixmap              img;
-  int                 size;
   unsigned long       FgPixel;
   unsigned long       BgPixel;
   ThotColorStruct     gdkFgPixel;
@@ -1238,13 +1391,13 @@ Pixmap DataToPixmap (unsigned char *image_data, int width, int height,
   gdkBgPixel.pixel = gdk_rgb_xpixel_from_rgb (BgPixel);
   /* TODO */
 
-  img = gdk_pixmap_create_from_data (DefaultDrawable, image_data, width, height, TtWDepth, &gdkFgPixel, &gdkBgPixel);
+  img = (Pixmap)gdk_pixmap_create_from_data (DefaultDrawable, image_data, width, height, TtWDepth, (GdkColor *)&gdkFgPixel, (GdkColor *)&gdkBgPixel);
 
 #endif /* !_GTK */
   return (img);
 #else /* _WINDOWS */
   return WIN_MakeImage (TtDisplay, image_data, width, height, TtWDepth,
-	  colrs, ncolors, withAlpha);
+	  colrs, ncolors, withAlpha, grayScale);
 #endif /* _WINDOWS */
 }
 
@@ -1289,13 +1442,19 @@ Drawable GifCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
   Pixmap              pixmap = (Pixmap) 0;
   ThotColorStruct     colrs[256];
 #ifdef _WINDOWS
+#ifndef _GL
   unsigned short      red, green, blue;
+#endif /*_GL*/
 #endif /* _WINDOWS */
   unsigned char      *buffer = NULL;
   unsigned char      *buffer2 = NULL;
   int                 w, h;
   int                 i;
   int                 ncolors, cpp;
+#ifdef _GL
+  unsigned char *ptr;
+  int   x,y;
+#endif /*_GL*/
 
   GifTransparent = -1;
   buffer = ReadGifToData (fn, &w, &h, &ncolors, &cpp, colrs);
@@ -1320,11 +1479,12 @@ Drawable GifCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
     {
       if (*xif == 0 && *yif != 0)
 	*xif = PixelValue (w, UnPixel, NULL, zoom);
-      if (*xif != 0 && *yif == 0)
+      if (*xif != 0 && *yif == 0) 
 	*yif = PixelValue (h, UnPixel, NULL, zoom);
     }
 
 #ifndef _WIN_PRINT
+#ifndef _GL
   if ((*xif != 0 && *yif != 0) && (w != *xif || h != *yif))
     {
       /* xif and yif contain width and height of the box */	  
@@ -1341,11 +1501,40 @@ Drawable GifCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
       w = *xif;
       h = *yif;
     }
+#endif /*_GL*/
 #endif /* _WIN_PRINT */
   
   if (buffer == NULL)
     return ((Drawable) NULL);	
 
+#ifdef _GL
+  ptr = TtaGetMemory (w * h * 4);
+  y = h;
+  while (y--)
+    {
+      buffer2 = buffer + y*w;
+      for (x = 0; x < w; x++)
+	{		
+	  i = *buffer2++;
+	  if (GifTransparent == i)
+	    {
+	      ptr += 3;
+	      *ptr++ = 0;
+	    }
+	  else
+	    {
+	      *ptr++ = colrs[i].red;
+	      *ptr++ = colrs[i].green;
+	      *ptr++ = colrs[i].blue;	
+	      *ptr++ = 255;
+	    }
+	}
+    }
+  ptr -= w*h*4;
+  TtaFreeMemory (buffer);
+  buffer = ptr;
+  pixmap = (Pixmap) buffer;
+#else /* _GL */
   if (GifTransparent >= 0)
     {
 #ifdef _WINDOWS
@@ -1368,8 +1557,10 @@ Drawable GifCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
       imageDesc->PicMask = MakeMask (TtDisplay, buffer, w, h, GifTransparent, 1);
 #endif /* _WINDOWS */
     }
-  pixmap = DataToPixmap (buffer, w, h, ncolors, colrs, FALSE);
+  pixmap = DataToPixmap (buffer, w, h, ncolors, colrs, FALSE, FALSE);
   TtaFreeMemory (buffer);
+#endif /*_GL*/
+  
   if (pixmap == None)
     {
 #ifdef _WINDOWS
@@ -1394,7 +1585,7 @@ Drawable GifCreate (char *fn, PictInfo *imageDesc, int *xif, int *yif,
 void DataToPrint (unsigned char *data, PictureScaling pres, int xif, int yif,
 		  int wif, int hif, int picW, int picH, FILE *fd, int ncolors,
 		  int transparent, int bgColor, ThotColorStruct *colrs,
-		  ThotBool withAlpha)
+		  ThotBool withAlpha, ThotBool grayScale)
 {
   int              delta;
   int              xtmp, ytmp;
@@ -1410,7 +1601,6 @@ void DataToPrint (unsigned char *data, PictureScaling pres, int xif, int yif,
       colrs[transparent].green = green << 8;
       colrs[transparent].blue  = blue << 8;
     }
-  
   xtmp = 0;
   ytmp = 0;
   switch (pres)
@@ -1433,7 +1623,7 @@ void DataToPrint (unsigned char *data, PictureScaling pres, int xif, int yif,
       delta = (hif - picH)/2;
       if (delta > 0)
 	{
-	  yif += delta ;
+	  yif += delta;
 	  hif = picH;
 	}
       else
@@ -1443,16 +1633,12 @@ void DataToPrint (unsigned char *data, PictureScaling pres, int xif, int yif,
 	  picH = hif;
 	} 
 
-      fprintf(fd, "gsave %d -%d translate\n", xif,
-	      yif + hif);
-      fprintf (fd, "%d %d %d %d DumpImage2\n", picW, picH,
-	       wif, hif);
+      fprintf(fd, "gsave %d -%d translate\n", xif, yif + hif);
+      fprintf (fd, "%d %d %d %d DumpImage2\n", picW, picH, wif, hif);
       break;
     case ReScale:
-      fprintf (fd, "gsave %d -%d translate\n", xif,
-	       yif + hif);
-      fprintf (fd, "%d %d %d %d DumpImage2\n", picW, picH,
-	       wif, hif);
+      fprintf (fd, "gsave %d -%d translate\n", xif, yif + hif);
+      fprintf (fd, "%d %d %d %d DumpImage2\n", picW, picH, wif, hif);
       wif = picW;
       hif = picH;
       break;
@@ -1463,7 +1649,19 @@ void DataToPrint (unsigned char *data, PictureScaling pres, int xif, int yif,
   fprintf(fd, "\n");
   if (ncolors == 0)
     {
-      if (withAlpha)
+      if (withAlpha && grayScale)
+	{
+	  /* using 2 bytes per pixel */
+	  xtmp = xtmp * 2;
+	  picW = picW * 2;
+	}
+      else if (grayScale)
+	{
+	  /* using 1 bytes per pixel */
+	  xtmp = xtmp;
+	  picW = picW;
+	}
+      else if (withAlpha)
 	{
 	  /* using 4 bytes per pixel */
 	  xtmp = xtmp * 4;
@@ -1485,8 +1683,13 @@ void DataToPrint (unsigned char *data, PictureScaling pres, int xif, int yif,
 	    {
 	      /* read the RGB from the data descriptor */
 	      r = data[ind++];
-	      g = data[ind++];
-	      b = data[ind++];
+	      if (grayScale)
+		g = b = r;
+	      else
+		{
+		  g = data[ind++];
+		  b = data[ind++];
+		}
 	      if (withAlpha)
 		ind++;
 	      fprintf (fd, "%02x%02x%02x", r, g, b);
@@ -1524,7 +1727,7 @@ void GifPrint (char *fn, PictureScaling pres, int xif, int yif, int wif,
   data = ReadGifToData (fn, &picW, &picH, &ncolors, &cpp, colrs);
   if (data)
     DataToPrint (data, pres, xif, yif, wif, hif, picW, picH, fd, ncolors,
-		 GifTransparent, bgColor, colrs, FALSE);
+		 GifTransparent, bgColor, colrs, FALSE, FALSE);
   TtaFreeMemory (data);
 #endif /* _WINDOWS */
 }
