@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -10,6 +10,7 @@
  * Authors: I. Vatton (INRIA)
  *          N. Layaida (INRIA) - New picture formats
  *          R. Guetari (W3C/INRIA) - Windows version
+ *          P. Cheyrou-lagreze (INRIA) - Opengl Version
  */
 
 #define PNG_SETJMP_SUPPORTED
@@ -17,7 +18,9 @@
 #include "thot_sys.h"
 #include "constmedia.h"
 #include "typemedia.h"
+#ifndef _GTK
 #include "lost.xpm"
+#endif /*_GTK*/
 #include "picture.h"
 #include "frame.h"
 #include "epsflogo.h"
@@ -65,8 +68,256 @@
 #include "xbmhandler_f.h"
 #include "xpmhandler_f.h"
 
+#ifdef _GL
+
+#ifdef _GTK
+#include <gtkgl/gtkglarea.h>
+#endif /*_GTK*/
+#include <GL/gl.h>
+
+
+#ifdef GL_MESA_window_pos
+#define MESA
+#endif
+/* For now not software optimised but quality optimized...
+ if too sloooooow we'll revert*/
+#undef MESA
+
+/*----------------------------------------------------------------------
+ Free video card memory from this texture.
+  ----------------------------------------------------------------------*/
+void FreeGlTexture (PictInfo *Image)
+{
+  if (Image->TextureBind)
+    {
+      if (glIsTexture (Image->TextureBind))
+	glDeleteTextures (1, &(Image->TextureBind));
+      Image->TextureBind = 0;
+      Image->RGBA = False;
+    }
+}
+
+#ifndef MESA
+
+/*----------------------------------------------------------------------
+  p2 :  Lowest power of two bigger than the argument.
+  ----------------------------------------------------------------------*/
+/*All these work on UNSIGNED BITS !!! 
+ if little-big endianess is involved,
+ all those atre wrong !!!*/
+
+#define lowest_bit(x) (x & -x)
+#define is_pow2(x) (x != 0 && x == lowest_bit(x))
+
+static int ceil_pow2_minus_1(unsigned int x)
+{
+  unsigned int i;
+  
+  for (i=1; i; i <<= 1)
+    x |= x >> i;
+  return x;
+}
+#define p2(p) (is_pow2(p)?p:ceil_pow2_minus_1((unsigned int) p) + 1)
+  
+/* Don't know exactly wich is faster...
+ this on is pretty good too...*/
+
+/* 
+int p2(p){
+p -= 1; 
+p |= p >> 16; 
+p |= p >> 8; 
+p |= p >> 4; 
+p |= p >> 2; 
+p |= p >> 1; 
+return p + 1;
+
+otherwise, identical, formulical,
+but VERY VERY VERY slower 
+(int to float, log, ceil, and finally float to int...)
+return 1 << (int) ceilf(logf((float) p) / M_LN2);
+} 
+*/
+
+/*----------------------------------------------------------------------
+  GL_MakeTextureSize : Texture sizes must be power of two
+  ----------------------------------------------------------------------*/
+static void GL_MakeTextureSize(PictInfo *Image, int GL_w, int GL_h)
+{
+  unsigned char      *data, *ptr1, *ptr2;
+  int                 xdiff, x, y, nbpixel;
+
+
+  if (Image->PicPixmap != None)
+	{
+	nbpixel = (Image->RGBA)?4:3;
+	/* In this algo, just remember that a 
+		RGB pixel value is a list of 3 value in source data
+		and 4 for destination RGBA texture */
+	data = TtaGetMemory (sizeof (unsigned char) * GL_w * GL_h * nbpixel);
+	/* Black transparent filling */
+	memset (data, 0, sizeof (unsigned char) * GL_w * GL_h * nbpixel);
+	ptr1 = Image->PicPixmap;
+	ptr2 = data;
+	xdiff = (GL_w - Image->PicWidth) * nbpixel;
+	x = nbpixel * Image->PicWidth;
+	for (y = 0; y < Image->PicHeight; y++)
+	    {
+		/* copy R,G,B,A */
+		memcpy (ptr2, ptr1, x); 
+		/* jump over the black transparent zone*/
+		ptr1 += x;
+		ptr2 += x + xdiff;
+		}	
+	TtaFreeMemory (Image->PicPixmap);
+	Image->PicPixmap = data;
+  }
+}
+
+#endif /*!MESA*/
+
+/* In case of old opengl version where
+ Texture Objects were an extension */
+/* #if defined(GL_VERSION_1_1) */
+/* #define TEXTURE_OBJECT 1 */
+/* #elif defined(GL_EXT_texture_object) */
+/* #define TEXTURE_OBJECT 1 */
+/* #define glBindTexture(A,B)     glBindTextureEXT(A,B) */
+/* #define glGenTextures(A,B)     glGenTexturesEXT(A,B) */
+/* #define glDeleteTextures(A,B)  glDeleteTexturesEXT(A,B) */
+/* #endif */
+
+
+/*----------------------------------------------------------------------
+ GL_TextureMap : map texture on a Quad (sort of a rectangle)
+ Drawpixel Method for software implementation, as it's much faster for those
+ Texture Method for hardware implementation as it's faster and better.
+  ----------------------------------------------------------------------*/
+static void GL_TextureMap (PictInfo *Image, int xFrame, int yFrame, int w, int h)
+{  
+#ifdef MESA
+  int       p2_w, p2_h, GL_h;
+  
+  if (Image->PicPixmap)
+      { 
+	/* The other Way  with texture... 
+	   but without texture power...(mippmapping)
+	   Faster on software implementation
+	   (Actually slower on hardware accelerated system) */
+	GL_h = FrameTable[ActiveFrame].FrHeight;
+	if (xFrame > 0 && yFrame+h < GL_h)
+	  glRasterPos2i (xFrame,  yFrame + h );
+	else
+	  {
+	    /* Raster Pos must be inside viewport 
+	       if not, it's not displayed at all*/
+	    p2_w = p2_h =0;
+	    if (xFrame < 0)
+	      {
+		p2_w = xFrame;
+		xFrame = 0;
+	      }
+	    if (yFrame+h > GL_h && yFrame < GL_h)
+	      { 
+		p2_h = - (yFrame + h - GL_h); 
+		yFrame = GL_h - h; 
+	      } 
+	    glRasterPos2i (xFrame,  yFrame + h); 
+	    glBitmap (0, 0, 0, 0,
+		     p2_w,
+		     p2_h,
+		     NULL);
+	  }
+	glDrawPixels( w, h, 
+		      ((Image->RGBA)?GL_RGBA:GL_RGB), 
+		      GL_UNSIGNED_BYTE, 
+		      Image->PicPixmap); 
+      }
+#else /*!MESA*/
+  int       p2_w, p2_h;
+  GLfloat   GL_w, GL_h;   
+  GLint		Mode;
+  
+  /* Another way is to split texture in 256x256 
+     pieces and render them on different quads
+     Declared to be the faster  */
+  p2_w = p2 (Image->PicWidth);
+  p2_h = p2 (Image->PicHeight);
+  glEnable (GL_TEXTURE_2D); 
+  /* Put texture in 3d card memory */
+  if (!glIsTexture (Image->TextureBind) &&
+      Image->PicPixmap)
+    {      
+      glGenTextures (1, &(Image->TextureBind));
+      glBindTexture (GL_TEXTURE_2D, Image->TextureBind);
+      /*TEXTURE ZOOM : GL_NEAREST is fastest and GL_LINEAR is second fastest*/
+      glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+      glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+      /* How texture is mapped... initially GL_REPEAT
+	 GL_REPEAT, GL_CLAMP, GL_CLAMP_TO_EDGE are another option.. Bench !!*/	    
+      glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
+      glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 
+      /* does current Color modify texture no = GL_REPLACE, 
+	 else => GL_MODULATE, GL_DECAL, ou GL_BLEND */
+      glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+      /* We give te texture to opengl Pipeline system */	    
+      Mode = (Image->RGBA)?GL_RGBA:GL_RGB;		
+      GL_MakeTextureSize(Image, p2_w, p2_h);	
+      glTexImage2D (GL_TEXTURE_2D, 
+		    0, 
+		    Mode, 
+		    p2_w, 
+		    p2_h, 
+		    0,
+		    Mode, 
+		    GL_UNSIGNED_BYTE, 
+		    (GLvoid *) Image->PicPixmap);
+      TtaFreeMemory (Image->PicPixmap);
+      Image->PicPixmap = None;
+    }
+  else
+    {
+      glBindTexture(GL_TEXTURE_2D, Image->TextureBind);
+    }
+  /* We have resized the picture to match a power of 2
+     We don't want to see all the picture, just the w and h 
+     portion*/
+  GL_w = (GLfloat) Image->PicWidth/p2_w;
+  GL_h = (GLfloat) Image->PicHeight/p2_h;   	
+  /* Not sure of the vertex order 
+     (not the faster one, I think) */
+  glBegin (GL_QUADS);
+  /* Texture coordinates are unrelative 
+     to the size of the square */      
+  /* lower left */
+  glTexCoord2i (0,    0); 
+  glVertex2i (xFrame,     yFrame + h);
+  /* upper right*/
+  glTexCoord2f (GL_w, 0.0); 
+  glVertex2i (xFrame + w, yFrame + h);
+  /* lower right */
+  glTexCoord2f (GL_w, GL_h); 
+  glVertex2i (xFrame + w, yFrame); 
+  /* upper left */
+  glTexCoord2f (0.0,  GL_h); 
+  glVertex2i (xFrame,     yFrame);      
+  glEnd ();	
+  /* State disabling */
+  glDisable (GL_TEXTURE_2D); 
+#endif /*MESA*/
+}
+#endif /* _GL */
+
 static char*    PictureMenu;
+#ifdef _GL
+static unsigned char *PictureLogo;
+#else /*_GL*/
+#if !defined (_GTK)
 static Pixmap   PictureLogo;
+#else /*_GTK*/
+static GdkPixmap *PictureLogo;
+#endif /*_GTK*/
+#endif /*_GL*/
 static ThotGC   tiledGC;
 
 char* FileExtension[] = {
@@ -223,19 +474,24 @@ static ThotBool Match_Format (int typeImage, char *fileName)
   ----------------------------------------------------------------------*/
 void FreePixmap (Pixmap pixmap)
 {
-  if (pixmap != None && pixmap != PictureLogo && pixmap != EpsfPictureLogo)
+  if (pixmap != None 
+      && pixmap != (Pixmap) PictureLogo 
+      && pixmap != EpsfPictureLogo)
+#ifdef _GL
+    TtaFreeMemory ((void *)pixmap);
+#else /*_GL*/
 #ifndef _WINDOWS
 #ifndef _GTK
     XFreePixmap (TtDisplay, pixmap);
 #else /* _GTK */
-  {
-   gdk_imlib_free_pixmap ((GdkPixmap *)pixmap);
-  }
+    if (((GdkPixmap *) pixmap) != PictureLogo)
+      gdk_imlib_free_pixmap ((GdkPixmap *) pixmap);
 #endif /* _GTK */
 #else  /* _WINDOWS */
   if (!DeleteObject ((HBITMAP)pixmap))
     WinErrorBox (WIN_Main_Wd, "FreePixmap");
 #endif /* _WINDOWS */
+#endif /*_GL*/
 }
 
 
@@ -316,6 +572,138 @@ static void SetPictureClipping (int *picWArea, int *picHArea, int wFrame,
      }
 }
 
+#ifdef _GL
+/*----------------------------------------------------------------------
+  LayoutPicture performs the layout of pixmap on the screen described
+  by the drawable.
+  if picXOrg or picYOrg are postive, the copy operation is shifted
+  ----------------------------------------------------------------------*/
+static void LayoutPicture (Pixmap pixmap, Drawable drawable, 
+						   int picXOrg, int picYOrg, int w, int h, 
+						   int xFrame, int yFrame, int frame, 
+						   PictInfo *imageDesc, PtrBox box)
+{
+  ViewFrame*        pFrame;
+  PictureScaling    picPresent;
+  int               x, y, clipWidth, clipHeight;
+  int               delta, i, j, iw, jh;
+  
+  if (picXOrg < 0)
+    {
+      xFrame = xFrame - picXOrg;
+      picXOrg = 0;
+    }
+  if (picYOrg < 0)
+    {
+      yFrame = yFrame - picYOrg;
+      picYOrg = 0;
+    }
+  pFrame = &ViewFrameTable[frame - 1];
+  if (pixmap != None)
+    {
+      /* the default presentation depends on the box type */
+      picPresent = imageDesc->PicPresent;
+      if (picPresent == DefaultPres)
+	{
+	  if (box->BxType == BoPicture)
+	    /* an image is rescaled */
+	    picPresent = ReScale;
+	  else
+	    /* a background image is repeated */
+	    picPresent = FillFrame;
+	}
+      switch (picPresent)
+	{
+	case ReScale:
+	  GL_TextureMap (imageDesc, xFrame, yFrame, w ,h);
+	  break;
+	case RealSize:
+	  GL_TextureMap (imageDesc, xFrame, yFrame,
+			 imageDesc->PicWidth, imageDesc->PicHeight);
+	  break;
+	case FillFrame:
+	case XRepeat:
+	case YRepeat:
+	  x          = pFrame->FrClipXBegin;
+          y          = pFrame->FrClipYBegin;
+          clipWidth  = pFrame->FrClipXEnd - x;
+          clipHeight = pFrame->FrClipYEnd - y;
+          x          -= pFrame->FrXOrg;
+          y          -= pFrame->FrYOrg;
+          if (picPresent == FillFrame || picPresent == YRepeat)
+	    {
+	      /* clipping height is done by the box height */
+	      if (y < yFrame)
+		{
+		  /* reduce the height in delta value */
+		  clipHeight = clipHeight + y - yFrame;
+		  y = yFrame;
+		}
+	      if (clipHeight > h)
+		clipHeight = h;
+	    }
+	  else
+	    {
+	      /* clipping height is done by the image height */
+	      delta = yFrame + imageDesc->PicHArea - y;
+	      if (delta <= 0)
+		clipHeight = 0;
+	      else
+		clipHeight = delta;
+	    }	  
+          if (picPresent == FillFrame || picPresent == XRepeat)
+	    {
+	      /* clipping width is done by the box width */
+	      if (x < xFrame)
+		{
+		  /* reduce the width in delta value */
+		  clipWidth = clipWidth + x - xFrame;
+		  x = xFrame;
+		}
+	      if (clipWidth > w)
+		clipWidth = w;
+	    }
+	  else
+	    {
+	      /* clipping width is done by the image width */
+	      delta = xFrame + imageDesc->PicWArea - x;
+	      if (delta <= 0)
+		clipWidth = 0;
+	      else
+		clipWidth = delta;
+	    }          
+	  j = 0;
+	  do
+	    {
+	      i = 0;
+	      do
+		{
+		  /* check if the limits of the copied zone */
+		  if (i + imageDesc->PicWArea <= w)
+		    iw = imageDesc->PicWArea;
+		  else
+		    iw = w - i;
+		  if (j + imageDesc->PicHArea <= h)
+		    jh = imageDesc->PicHArea;
+		  else
+		    jh = h - j;
+		  GL_TextureMap (imageDesc, xFrame, yFrame, iw ,jh);
+		  /*BitBlt (hMemDC, i, j, iw, jh, hOrigDC, 0, 0, SRCCOPY);*/
+		  i += imageDesc->PicWArea;
+		} 
+	      while (i < w);
+	      j += imageDesc->PicHArea;
+	    } 
+	  while (j < h);
+	  break;  
+	default: 
+	  break;
+	}
+    }
+}
+
+#else /*_GL*/
+
 /*----------------------------------------------------------------------
   LayoutPicture performs the layout of pixmap on the screen described
   by the drawable.
@@ -342,19 +730,12 @@ static void LayoutPicture (Pixmap pixmap, Drawable drawable, int picXOrg,
   int               i, j, iw, jh;
   HRGN              hrgn;
 #else /* _WINDOWS */
-#ifndef _GTK
   XRectangle        rect;
+#ifndef _GTK
   XGCValues         values;
   unsigned int      valuemask;
 #endif /* _GTK */
 #endif /* _WINDOWS */
-#ifdef _GTK
-  XRectangle         rect;
-  /*  GdkImlibBorder     rect_border;*/
-  /*GdkImlibImage     *im;*/
-  /*  GdkGCValues       *GCvalues;*/
-
-#endif
 
   if (picXOrg < 0)
     {
@@ -398,25 +779,61 @@ static void LayoutPicture (Pixmap pixmap, Drawable drawable, int picXOrg,
 	      XSetClipOrigin (TtDisplay, TtGraphicGC, xFrame - picXOrg, yFrame - picYOrg);
 	      XSetClipMask (TtDisplay, TtGraphicGC, imageDesc->PicMask);
 	    }
-	  XCopyArea (TtDisplay, pixmap, drawable, TtGraphicGC, picXOrg, picYOrg, w, h, xFrame, yFrame);
-	   if (imageDesc->PicMask)
+	  XCopyArea (TtDisplay, pixmap, drawable, TtGraphicGC, 
+		     picXOrg, picYOrg, w, h, xFrame, yFrame);
+	  if (imageDesc->PicMask)
+	    {
+	      XSetClipMask (TtDisplay, TtGraphicGC, None);
+	      XSetClipOrigin (TtDisplay, TtGraphicGC, 0, 0);
+	    }
+#else /* _GTK */ 
+	   if (w != imageDesc->PicWArea ||
+	       h != imageDesc->PicHArea || 
+	       imageDesc->PicPixmap == NULL)
 	     {
-	       XSetClipMask (TtDisplay, TtGraphicGC, None);
-	       XSetClipOrigin (TtDisplay, TtGraphicGC, 0, 0);
+	       /*W and H is not the same 
+		 as when we load the image*/
+	       if (imageDesc->im && 
+		   w > 0 && h > 0)
+		 {
+		   gdk_imlib_render(imageDesc->im, w, h);
+		   if (imageDesc->PicMask)
+		     {
+		       gdk_gc_set_clip_origin (TtGraphicGC, 
+					       xFrame - picXOrg, 
+					       yFrame - picYOrg);
+		       gdk_gc_set_clip_mask (TtGraphicGC, 
+					     (GdkPixmap *) 
+					     gdk_imlib_move_mask (imageDesc->im));
+		     }		   
+		   gdk_draw_pixmap ((GdkDrawable *)drawable, TtGraphicGC,
+				    (GdkPixmap *) imageDesc->im->pixmap, 
+				    picXOrg, picYOrg, 
+				    xFrame, yFrame, w ,h);
+		 }
 	     }
-#else /* _GTK */
-	   if (imageDesc->PicMask)
+	   else
 	     {
-	       gdk_gc_set_clip_origin (TtGraphicGC, xFrame - picXOrg, yFrame - picYOrg);
-	       gdk_gc_set_clip_mask (TtGraphicGC, (GdkPixmap *)imageDesc->PicMask);
+	       if (imageDesc->PicMask)
+		 {
+		   gdk_gc_set_clip_origin (TtGraphicGC, 
+					   xFrame - picXOrg, 
+					   yFrame - picYOrg);
+		   gdk_gc_set_clip_mask (TtGraphicGC, 
+					 (GdkPixmap *) imageDesc->PicMask);
+		 }
+	       gdk_draw_pixmap ((GdkDrawable *)drawable, TtGraphicGC,
+				(GdkPixmap *) imageDesc->PicPixmap, 
+				picXOrg, picYOrg, 
+				xFrame, yFrame, w ,h);
 	     }
-	   gdk_draw_pixmap ((GdkDrawable *)drawable, TtGraphicGC,(GdkPixmap *) pixmap, picXOrg, picYOrg, xFrame, yFrame, w ,h);
+	   /*Restablish to normal clip*/
 	   if (imageDesc->PicMask)
 	     {
 	       gdk_gc_set_clip_mask (TtGraphicGC, (GdkPixmap *)None);
 	       gdk_gc_set_clip_origin (TtGraphicGC, 0, 0);
 	     }
-#endif /* _GTK */
+#endif /* !_GTK */
 #else /* _WINDOWS */
 	case RealSize:
 	  if (imageDesc->PicMask == -1 || imageDesc->PicType == -1)
@@ -488,6 +905,7 @@ static void LayoutPicture (Pixmap pixmap, Drawable drawable, int picXOrg,
 	    }
 	  if (rect.width > w)
 	    rect.width = w;
+
 #ifndef _GTK
 	  valuemask = GCTile | GCFillStyle | GCTileStipXOrigin | GCTileStipYOrigin;
 	  values.tile = pixmap;
@@ -564,6 +982,7 @@ static void LayoutPicture (Pixmap pixmap, Drawable drawable, int picXOrg,
 			      yFrame,   /* y position drawing */
 			      w,        /* width drawing */
 			      h);       /* height drawing */
+
 	  /* remove clipping */
           rect.x = 0;
           rect.y = 0;
@@ -575,7 +994,7 @@ static void LayoutPicture (Pixmap pixmap, Drawable drawable, int picXOrg,
 	      gdk_gc_set_clip_mask (tiledGC, None);
 	      gdk_gc_set_clip_origin (tiledGC, 0, 0);
 	    }
-#endif /* _GTK */
+#endif /* !_GTK */
 #else  /* _WINDOWS */
           x          = pFrame->FrClipXBegin;
           y          = pFrame->FrClipYBegin;
@@ -626,7 +1045,6 @@ static void LayoutPicture (Pixmap pixmap, Drawable drawable, int picXOrg,
 	      else
 		clipWidth = delta;
 	    }
-	  
           hMemDC  = CreateCompatibleDC (TtDisplay);
           bitmapTiled = CreateCompatibleBitmap (TtDisplay, w, h);
           hOrigDC = CreateCompatibleDC (TtDisplay);
@@ -683,7 +1101,7 @@ static void LayoutPicture (Pixmap pixmap, Drawable drawable, int picXOrg,
 	}
     }
 }
-
+#endif /*_GL*/
 
 /*----------------------------------------------------------------------
    GetPictureFormat returns the format of a file picture           
@@ -747,6 +1165,39 @@ Picture_Report PictureFileOk (char *fileName, int *typeImage)
   return status;
 }
 
+#if defined (_GTK) || defined (_WINDOWS)
+/*----------------------------------------------------------------------
+  Create The logo for lost pictures
+  ----------------------------------------------------------------------*/
+void CreateGifLogo ()
+{
+#if !defined(_GTK) || defined (_GL) 
+  PictInfo            *imageDesc;
+  unsigned long       Bgcolor = 0;
+  int                 xBox = 0;
+  int                 yBox = 0;
+  int                 wBox = 0;
+  int                 hBox = 0;
+  int                 width, height;
+  Drawable            drw;
+  
+  imageDesc = TtaGetMemory (sizeof (PictInfo));
+  drw = (*(PictureHandlerTable[GIF_FORMAT].Produce_Picture)) 
+    (LostPicturePath, imageDesc, &xBox, &yBox, &wBox,
+     &hBox, Bgcolor, &width, &height);
+  TtaFreeMemory (imageDesc);
+  PictureLogo = (unsigned char *) drw;
+#else /*_WIN && _GL*/
+  GdkImlibImage      *im;
+
+  im = gdk_imlib_load_image (LostPicturePath);
+  gdk_imlib_render (im, 40, 40);
+  PictureLogo = gdk_imlib_copy_image (im);
+  TtaFreeMemory (im);
+#endif /*_WIN && _GL*/
+}
+#endif /*_GTK && _WIN*/
+
 /*----------------------------------------------------------------------
    Private Initializations of picture handlers and the visual type 
   ----------------------------------------------------------------------*/
@@ -781,8 +1232,6 @@ void InitPictureHandlers (ThotBool printing)
       gdk_rgb_gc_set_background (GCpicture, White_Color);
       gdk_gc_set_exposures (GCpicture,0);
     }
-  /* create a special logo for lost pictures */
-  /* TODO */
   theVisual = (Visual *) gdk_visual_get_system ();
 #else /* _GTK */
    /* initialize Graphic context to display pictures */
@@ -904,6 +1353,7 @@ void GetPictHandlersList (int *count, char* buffer)
 
 }
 
+#ifndef _GTK
 /*----------------------------------------------------------------------
    SimpleName
 
@@ -938,7 +1388,7 @@ static void SimpleName (char *filename, char *simplename)
       *to++ = *from++;
    *to = EOS;
 }
-
+#endif /*_GTK*/
 
 /*----------------------------------------------------------------------
    DrawEpsBox draws the eps logo into the picture box.            
@@ -1211,8 +1661,9 @@ void DrawPicture (PtrBox box, PictInfo *imageDesc, int frame, int x,
 		                            x + xTranslate, y + yTranslate);
 	    }
 	  else
-	    LayoutPicture (imageDesc->PicPixmap, drawable, picXOrg, picYOrg,
-			   w, h, x + xTranslate, y + yTranslate, frame,
+	    LayoutPicture ((Pixmap) imageDesc->PicPixmap, drawable,
+			   picXOrg, picYOrg, w, h, 
+			   x + xTranslate, y + yTranslate, frame,
 			   imageDesc, box);
 	}
     }
@@ -1401,6 +1852,233 @@ unsigned char *ZoomPicture (unsigned char *cpic, int cWIDE, int cHIGH ,
   return (epic);
 }
 
+#ifdef _GL
+/*----------------------------------------------------------------------
+   Requests the picture handlers to get the corresponding RGB or RGBA buffer
+   and make a Texture or it (aka load into video card memory)   
+  ----------------------------------------------------------------------*/
+void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
+{
+  PathBuffer          fileName;
+  PictureScaling      pres;
+  PtrAbstractBox      pAb;
+  Picture_Report      status;
+  unsigned long       Bgcolor;
+  int                 typeImage;
+  int                 xBox = 0;
+  int                 yBox = 0;
+  int                 wBox =0;
+  int                 hBox = 0;
+  int                 w, h;
+  int                 width, height;
+  int                 left, right, top, bottom;
+
+  /*
+  PictInfo *picture_cached;
+  */
+
+  left = box->BxLMargin + box->BxLBorder + box->BxLPadding;
+  right = box->BxRMargin + box->BxRBorder + box->BxRPadding;
+  top = box->BxTMargin + box->BxTBorder + box->BxTPadding;
+  bottom = box->BxBMargin + box->BxBBorder + box->BxBPadding;
+  pAb = box->BxAbstractBox;
+  if (pAb->AbVisibility < ViewFrameTable[frame - 1].FrVisibility)
+    /* the picture is not visible */
+    return;
+  if (imageDesc->PicFileName == NULL || imageDesc->PicFileName[0] == EOS)
+    return;
+  /* If we're replacing an image*/
+  if (glIsTexture (imageDesc->TextureBind))
+    {
+      glDeleteTextures (1, &(imageDesc->TextureBind));
+      imageDesc->TextureBind = 0;
+    }
+  
+  GetPictureFileName (imageDesc->PicFileName, fileName);
+
+  /*
+  if (PictureAlreadyLoaded (imageDesc, w, h))
+      return;    
+  */
+
+  typeImage = imageDesc->PicType;
+  status = PictureFileOk (fileName, &typeImage);
+  w = 0;
+  h = 0;
+  Bgcolor = ColorPixel (pAb->AbBackground);
+  /* clean up the current image descriptor */
+  /*CleanPictInfo (imageDesc);*/
+  if (status == Supported_Format)
+    {
+      /* Supported format */
+      imageDesc->PicType = typeImage;
+      pres = imageDesc->PicPresent;
+      if (pres == DefaultPres)
+	{
+	  if (box->BxType == BoPicture)
+	    /* an image is rescaled */
+	    pres = ReScale;
+	  else
+	    /* a background image is repeated */
+	    pres = FillFrame;
+	}
+      if ((typeImage == XBM_FORMAT || typeImage == XPM_FORMAT) && pres == ReScale)
+	pres = imageDesc->PicPresent = RealSize;
+      /* picture dimension */
+      if (pAb->AbLeafType == LtCompound)
+	{
+	  /* a background image, draw over the whole box */
+	  w = box->BxWidth;
+	  h = box->BxHeight;
+	}
+      else
+	{
+	  /* draw within the inside box */
+	  w = box->BxW;
+	  h = box->BxH;
+	}
+      
+      if (PictureHandlerTable[typeImage].Produce_Picture != NULL)
+	{
+	  if (typeImage >= InlineHandlers)
+	    {
+	      /* Plugins are displayed in RealSize */
+	      imageDesc->PicPresent = RealSize;
+	      imageDesc->PicWArea = wBox = w;
+	      imageDesc->PicHArea = hBox = h;
+#ifdef _GL
+	      imageDesc->PicPixmap = (unsigned char *) 
+		(*(PictureHandlerTable[typeImage].Produce_Picture)) 
+		(frame, imageDesc, fileName);
+#else /*_GL*/
+	      imageDesc->PicPixmap = (*(PictureHandlerTable[typeImage].Produce_Picture)) 
+		(frame, imageDesc, fileName);
+#endif /*_GL*/	      
+	      xBox = imageDesc->PicXArea;
+	      yBox = imageDesc->PicYArea;
+	    }
+	  else
+	    {
+	      /* xBox and yBox get the box size if picture is */
+	      /* rescaled and receives the position of the picture */
+	      if (pres != ReScale || Printing)
+		{
+		  xBox = 0;
+		  yBox = 0;
+		}
+	      else
+		{
+		  if (box->BxW != 0)
+		    xBox = w;
+		  if(box->BxH != 0)
+		    yBox = h;
+		}
+#ifdef _GL
+	      imageDesc->PicPixmap = (unsigned char *) 
+		(*(PictureHandlerTable[typeImage].Produce_Picture))
+		(fileName, imageDesc, &xBox, &yBox, &wBox, &hBox,
+		 Bgcolor, &width, &height,
+		 ViewFrameTable[frame - 1].FrMagnification);
+#else /*_GL*/
+	      imageDesc->PicPixmap = (*(PictureHandlerTable[typeImage].Produce_Picture))
+		(fileName, imageDesc, &xBox, &yBox, &wBox, &hBox,
+		 Bgcolor, &width, &height,
+		 ViewFrameTable[frame - 1].FrMagnification);
+#endif /*_GL*/
+	    }
+	  /* intrinsic width and height */
+	  imageDesc->PicWidth  = width;
+	  imageDesc->PicHeight = height;
+	}
+    }
+
+  /* Picture didn't load (corrupted, don't exists...)
+     or format isn't supported*/
+  if (imageDesc->PicPixmap == None 
+      && !glIsTexture (imageDesc->TextureBind))
+    {
+      if (PictureLogo == None)
+			/* create a special logo for lost pictures */
+			CreateGifLogo ();
+      imageDesc->PicFileName = TtaStrdup (LostPicturePath);
+      imageDesc->PicType = 3;
+      imageDesc->PicPresent = pres;
+      imageDesc->PicPixmap = PictureLogo;
+      /* convert logo into a texture map 
+	 0r use a display list !!*/
+      wBox = w = 40;
+      hBox = h = 40;
+    }
+  else if (w == 0 || h == 0)
+    {
+      /* one of box size is unknown, keep the image size */
+      if (w == 0)
+	w = wBox;
+      if (h == 0)
+	h = hBox;
+      /* Do you have to extend the clipping */
+      if (pAb->AbLeafType == LtCompound)
+	DefClip (frame, box->BxXOrg, box->BxYOrg,
+		 box->BxXOrg + w, box->BxYOrg + h);
+      else
+	DefClip (frame, box->BxXOrg - left, box->BxYOrg - top,
+		 box->BxXOrg + right + w, box->BxYOrg + bottom + h);
+      if (pAb->AbLeafType == LtPicture)
+	{
+	  /* transmit picture dimensions */
+	  if (!(pAb->AbWidth.DimIsPosition))
+	    {
+	      if (pAb->AbWidth.DimMinimum)
+		/* the rule min is applied to this box */
+		ChangeDefaultWidth (box, box, w, 0, frame);
+	      else if (pAb->AbEnclosing &&
+		       pAb->AbWidth.DimAbRef == pAb->AbEnclosing &&
+		       pAb->AbNext == NULL && pAb->AbPrevious == NULL)
+		/* the parent box should inherit the picture width */
+		ChangeWidth (pAb->AbEnclosing->AbBox,
+			     pAb->AbEnclosing->AbBox, NULL,
+			     w + left + right, 0, frame);
+	    }
+	  if (!(pAb->AbHeight.DimIsPosition))
+	    {
+	      if (pAb->AbHeight.DimMinimum)
+		/* the rule min is applied to this box */
+		ChangeDefaultHeight (box, box, h, frame);
+	      else if (pAb->AbEnclosing &&
+		       pAb->AbWidth.DimAbRef == pAb->AbEnclosing &&
+		       pAb->AbNext == NULL && pAb->AbPrevious == NULL)
+		/* the parent box should inherit the picture height */
+		ChangeHeight (pAb->AbEnclosing->AbBox,
+			      pAb->AbEnclosing->AbBox, NULL,
+			      h + top + bottom + top + bottom, frame);
+	    }
+	}
+    }  
+  if (pres != ReScale || Printing)
+    {
+      imageDesc->PicXArea = xBox;
+      imageDesc->PicYArea = yBox;
+      imageDesc->PicWArea = wBox;
+      imageDesc->PicHArea = hBox;
+    }
+  else
+    {
+      imageDesc->PicXArea = xBox;
+      imageDesc->PicYArea = yBox;
+      imageDesc->PicWArea = w;
+      imageDesc->PicHArea = h;
+    }
+  /* Gif and Png handles transparency 
+     so picture format is RGBA, 
+     all others are RGB*/
+  if (typeImage != GIF_FORMAT 
+      && typeImage != PNG_FORMAT)
+    imageDesc->RGBA = FALSE;
+  else
+    imageDesc->RGBA = TRUE;	
+}
+
+#else /* _GL */    
 
 /*----------------------------------------------------------------------
    Requests the picture handlers to get the corresponding pixmaps    
@@ -1412,7 +2090,11 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
 #ifdef _GTK
 #ifndef _GTK2
   GdkImlibImage      *im = None;
+#ifndef _GL
   GdkPixmap          *drw = None;
+#else /* _GL*/
+  unsigned char*  drw = NULL;
+#endif /* _GL */
 #else /* _GTK2 */
   GdkPixbuf          *im = None;
   GError             *error=NULL;
@@ -1426,7 +2108,7 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
   int                 typeImage;
   int                 xBox = 0;
   int                 yBox = 0;
-  int                 wBox =0;
+  int                 wBox = 0;
   int                 hBox = 0;
   int                 w, h;
   int                 width, height;
@@ -1447,6 +2129,7 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
     return;
 
   GetPictureFileName (imageDesc->PicFileName, fileName);
+
   typeImage = imageDesc->PicType;
   status = PictureFileOk (fileName, &typeImage);
   w = 0;
@@ -1457,6 +2140,11 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
   if (status != Supported_Format)
     {
       pres = RealSize;
+#if defined (_GTK) || defined (_WINDOWS)
+	  if (PictureLogo == None)
+	    /* create a special logo for lost pictures */
+	    CreateGifLogo ();
+#endif 
 #ifdef _WINDOWS
 #ifdef _WIN_PRINT
       if (TtDisplay == NULL)
@@ -1468,18 +2156,18 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
 #else /* _WIN_PRINT */
       imageDesc->PicType = 3;
       imageDesc->PicPresent = pres;
-      drw = (*(PictureHandlerTable[GIF_FORMAT].Produce_Picture)) 
-	(LostPicturePath, imageDesc, &xBox, &yBox, &wBox,
-	 &hBox, Bgcolor, &width, &height);
+      drw = PictureLogo;
 #endif /* _WIN_PRINT */
 #else  /* _WINDOWS */
-#ifdef _GTK
+#ifdef _GTK 
+      imageDesc->PicType = 3;
+      imageDesc->PicPresent = pres;
       drw = (GdkPixmap *) PictureLogo;
-#else /* ! _GTK */
+#else /*_GTK*/
       drw = PictureLogo;
-#endif /* ! _GTK */
-#endif /* _WINDOWS */
       imageDesc->PicType = -1;
+#endif /* ! GTK */
+#endif /* _WINDOWS */
       wBox = w = 40;
       hBox = h = 40;
     }
@@ -1563,9 +2251,29 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
 	      imageDesc->PicHArea = hBox = h;
 #ifndef _GTK2
 	      im = gdk_imlib_load_image (fileName);
+#ifndef _GL
 	      gdk_imlib_render(im, w, h);
 	      drw = gdk_imlib_move_image (im);
 	      imageDesc->PicMask = (Pixmap) gdk_imlib_move_mask (im);
+#else /* _GL */
+	      /* opengl draw in the other way...*/
+	      gdk_imlib_flip_image_vertical (im);
+	      /* opengl texture have size that is a power of 2*/
+#ifndef MESA
+	      /* opengl texture have size that is a power of 2*/
+	      drw = GL_MakeTexture (im->rgb_data, 
+							    im->shape_color.r, 
+							    im->shape_color.g,
+							    im->shape_color.b,
+							    w ,h);
+#else /* MESA*/
+	      drw = GL_MakeTransparentRGB (im->rgb_data, 
+									  im->shape_color.r, 
+									  im->shape_color.g,
+									  im->shape_color.b,
+									  w ,h);
+#endif/*  MESA */
+#endif /* _GL */
 #else /* _GTK2 */
 	      im = gdk_pixbuf_new_from_file(fileName, &error);
 #endif /* _GTK2 */
@@ -1594,27 +2302,67 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
 		(fileName, imageDesc, &xBox, &yBox, &wBox, &hBox,
 		 Bgcolor, &width, &height,
 		 ViewFrameTable[frame - 1].FrMagnification);
+
 #else /* _GTK */
-	      /* load the picture using ImLib */
-	      im = gdk_imlib_load_image (fileName);	      
-	      if (pres == RealSize)
-		{
-		  /* if it's a background, dont rescale the picture */
-		  wBox = im->rgb_width;
-		  hBox = im->rgb_height;
-		}
+	      if (typeImage == EPS_FORMAT)
+		drw = (GdkPixmap *) (*(PictureHandlerTable[typeImage].Produce_Picture))
+		  (fileName, imageDesc, &xBox, &yBox, &wBox, &hBox,
+		   Bgcolor, &width, &height,
+		   ViewFrameTable[frame - 1].FrMagnification);
 	      else
 		{
-		  if (wBox == 0)
-		    wBox = im->rgb_width;
-		  if (hBox == 0)
-		    hBox = im->rgb_height;
+		  /* load the picture using ImLib */
+		  im = gdk_imlib_load_image (fileName);	      
+		  if (pres == RealSize)
+		    {
+		      /* if it's a background, dont rescale the picture */
+		      wBox = im->rgb_width;
+		      hBox = im->rgb_height;
+		      if (xBox == 0)
+			xBox = im->rgb_width;
+		      if (yBox == 0)
+			yBox = im->rgb_height;
+		    }
+		  else
+		    {
+		      if (wBox == 0)
+			wBox = im->rgb_width;
+		      if (hBox == 0)
+			hBox = im->rgb_height;
+		      if (xBox == 0)
+			xBox = im->rgb_width;
+		      if (yBox == 0)
+			yBox = im->rgb_height;
+		    }
+#ifndef _GL
+		  gdk_imlib_render(im,
+				   (gint)xBox,
+				   (gint)yBox);
+		  drw = (GdkPixmap *) gdk_imlib_move_image (im);
+		  imageDesc->PicMask = (Pixmap) gdk_imlib_move_mask (im);
+#else /* _GL */
+		  /* opengl draw in the other way...*/
+		  gdk_imlib_flip_image_vertical (im);
+#ifndef MESA
+		  /* opengl texture have size that is a power of 2*/
+		  drw = GL_MakeTexture (im->rgb_data, 
+					im->shape_color.r, 
+					im->shape_color.g,
+					im->shape_color.b,
+					(gint)wBox ,
+					(gint)hBox);
+#else /* MESA*/
+		  drw = GL_MakeTransparentRGB(im->rgb_data, 
+					      im->shape_color.r, 
+					      im->shape_color.g,
+					      im->shape_color.b,
+					      (gint)wBox, 
+					      (gint)hBox);
+#endif/*  MESA */
+#endif /* _GL */
 		}
-	      gdk_imlib_render(im, (gint)wBox, (gint)hBox);
-	      drw = (GdkPixmap *) gdk_imlib_move_image (im);
-	      imageDesc->PicMask = (Pixmap) gdk_imlib_move_mask (im);
-	      width = im->rgb_width;
-	      height = im->rgb_height;
+	      width = (gint) wBox;
+	      height = (gint) hBox;
 #endif /* _GTK */
 	      /* intrinsic width and height */
 	      imageDesc->PicWidth  = width;
@@ -1626,14 +2374,37 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
        
       if (drw == None)
 	{
-#ifdef _GTK
-	  drw = (GdkPixmap *) PictureLogo;
-#else /* ! _GTK */
-	  drw = PictureLogo;
+#if defined (_GTK) || defined (_WINDOWS)
+	  if (PictureLogo == None)
+	    /* create a special logo for lost pictures */
+	    CreateGifLogo ();
+#endif 
+#ifdef _WINDOWS
+#ifdef _WIN_PRINT
+      if (TtDisplay == NULL)
+	{
+	  /* load the device context into TtDisplay */
+	  WIN_GetDeviceContext (frame);
+	  releaseDC = TRUE;
+	}
+#else /* _WIN_PRINT */
+      imageDesc->PicType = 3;
+      imageDesc->PicPresent = pres;
+      drw = PictureLogo;
+#endif /* _WIN_PRINT */
+#else  /* _WINDOWS */
+#ifdef _GTK 
+      imageDesc->PicType = 3;
+      imageDesc->PicPresent = pres;
+      imageDesc->PicFileName = TtaStrdup (LostPicturePath);
+      drw = (GdkPixmap *) PictureLogo;
+#else /*_GTK*/
+      drw = PictureLogo;
+      imageDesc->PicType = -1;
 #endif /* ! GTK */
-	  imageDesc->PicType = -1;
-	  wBox = w = 40;
-	  hBox = h = 40;
+#endif /* _WINDOWS */
+      wBox = w = 40;
+      hBox = h = 40;
 	}
       else if (w == 0 || h == 0)
 	{
@@ -1696,18 +2467,18 @@ void LoadPicture (int frame, PtrBox box, PictInfo *imageDesc)
       imageDesc->PicWArea = w;
       imageDesc->PicHArea = h;
     }
-#ifdef _GTK
-  imageDesc->PicPixmap = (Pixmap) drw;
-#else /* ! _GTK */
   imageDesc->PicPixmap = drw;
-#endif /* ! GTK */ 
-
+#ifdef _GTK
+  if (im)
+    imageDesc->im = im;
+#endif /*_GTK*/
 #ifdef _WIN_PRINT
   if (releaseDC)
 	  /* release the device context into TtDisplay */
 	  WIN_ReleaseDeviceContext ();
 #endif /* _WIN_PRINT */
 }
+#endif /*_GL*/
 
 
 /*----------------------------------------------------------------------
@@ -1806,3 +2577,57 @@ void LittleXBigEndian (register unsigned char *b, register long n)
      }
    while (--n > 0);
 }
+
+/*----------------------------------------------------------------------
+  GetScreenshot makes a screenshot of Amaya drawing area
+  this function allocates and returns the screenshot
+  ----------------------------------------------------------------------*/
+unsigned char *GetScreenshot (int frame, int x, int y, int width, int height)
+{
+  unsigned char   *screenshot = NULL;
+#ifdef _GTK
+  GdkImage        *View;
+  unsigned char   *pixel, inter;
+  int              k, cpt1, cpt2, line, line2, mi_h, NbOctetsPerLine, i = 0;
+
+
+  View = gdk_image_get ((FrameTable[frame].WdFrame)->window, x, y, width, height);
+  pixel = (unsigned char *) View->mem;
+  k = 4 * View->width * View->height - 4;
+  /* change BGRA in RGBA */
+  while (i < k)
+    {
+      inter = *(pixel + i);
+      *(pixel + i) = *(pixel + i + 2);
+      *(pixel + i + 2) = inter;
+      *(pixel + i + 3) = 255 - *(pixel + i + 3);
+      i += 4;
+    }
+
+  /* Makes mirror picture */
+  pixel = (unsigned char *) View->mem;
+  i = cpt2 = 0;
+  cpt1 = 1;
+  mi_h = View->height / 2;
+  NbOctetsPerLine = View->width * 4;
+  while (cpt1 < mi_h)
+    {
+      cpt2 = 0;
+      line = NbOctetsPerLine * cpt1;
+      line2 = NbOctetsPerLine * (cpt1 - 1);
+      while (cpt2 < NbOctetsPerLine)
+	{
+	  inter = *(pixel + cpt2 + line2);
+	  *(pixel + cpt2 + line2) = *(pixel + k + 4 - line + cpt2);
+	  *(pixel + k + 4 - line + cpt2) = inter;
+	  cpt2++;
+	  i++;
+	}
+      cpt1++;
+    }
+  screenshot = (unsigned char *) TtaGetMemory (k + 4);
+  memcpy (screenshot, View->mem, k + 4);
+  gdk_image_destroy (View);
+#endif /* _GTK */
+  return screenshot;
+}
