@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -70,7 +70,8 @@ extern void TtaSetAttributeValue (Attribute attribute, int value, Element elemen
    document: the document to which the element belongs.
    Must be 0 if element is NULL.
    ---------------------------------------------------------------------- */
-extern void TtaSetAttributeText (Attribute attribute, char *buffer, Element element, Document document);
+extern void TtaSetAttributeText (Attribute attribute, char *buffer,
+				 Element element, Document document);
 
 /* ----------------------------------------------------------------------
    TtaNextAttribute
@@ -105,6 +106,16 @@ extern void TtaNextAttribute (Element element, /*INOUT*/ Attribute *attribute);
    ---------------------------------------------------------------------- */
 extern Attribute TtaGetAttribute (Element element, AttributeType attributeType);
 
+/* ----------------------------------------------------------------------
+   TtaGetAttributeKind
+
+   Returns the kind of the attribute: 0 = Enumerate, 1 = Integer, 2 = Text,
+   3 = CsReference.
+   Parameter:
+   attType: type of the attribute.
+   ---------------------------------------------------------------------- */
+extern int TtaGetAttributeKind (AttributeType attType);
+
 /* ----------------------------------------------------------------------
    TtaGiveAttributeType
 
@@ -112,13 +123,13 @@ extern Attribute TtaGetAttribute (Element element, AttributeType attributeType);
 
    Parameter:
    attribute: the attribute of interest.
-
    Return parameters:
    attributeType: type of the attribute.
    attrKind: kind of the attribute: 0 = Enumerate, 1 = Integer, 2 = Text,
    3 = CsReference
    ---------------------------------------------------------------------- */
-extern void TtaGiveAttributeType (Attribute attribute, AttributeType *attributeType, int *attrKind);
+extern void TtaGiveAttributeType (Attribute attribute, AttributeType *attributeType,
+				  int *attrKind);
 
 /* ----------------------------------------------------------------------
    TtaGiveAttributeTypeFromName
@@ -299,7 +310,21 @@ extern void TtaSearchAttribute (AttributeType searchedAttribute, SearchDomain sc
    elementFound: the element found, or NULL if not found.
    attributeFound: the searched attribute, or NULL if not 
    ---------------------------------------------------------------------- */
-extern void TtaSearchAttributes (AttributeType searchedAtt1, AttributeType searchedAtt2, SearchDomain scope, Element element, Element *elementFound, Attribute *attributeFound);
+extern void TtaSearchAttributes (AttributeType searchedAtt1,
+				 AttributeType searchedAtt2,
+				 SearchDomain scope, Element element,
+				 Element *elementFound, Attribute *attributeFound);
+
+/*----------------------------------------------------------------------
+   TtaGetTypedAttrAncestor
+
+   returns a pointer to the attribute of the first
+   element which encloses pEl and that has an attribute
+   of type attNum. The function returns pElAttr if
+   the search is succesful, NULL otherwise.
+  ----------------------------------------------------------------------*/
+Attribute  TtaGetTypedAttrAncestor (Element pEl, int attNum,
+				    SSchema pSSattr, Element *pElAttr);
 
 #endif /* __CEXTRACT__ */
 
