@@ -1,19 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
- 
 #ifndef _DIALOG_H_
 #define _DIALOG_H_
 
@@ -33,7 +24,6 @@ enum DButtons
   };
 
 #ifndef __CEXTRACT__
-#ifdef __STDC__
 #ifdef _GTK 
 #include <gtk/gtk.h> 
 /*-------------------------------------------------------------------------------
@@ -61,17 +51,18 @@ extern gint ExposeEvent (ThotWidget widget, GdkEventExpose *event, gpointer data
    des entre'es du menu.                                              
    Retourne un code d'erreur.                                         
   ----------------------------------------------------------------------*/
-extern void         TtaNewPulldown (int ref, ThotMenu parent, STRING title, int number, STRING text, char* equiv);
+extern void TtaNewPulldown (int ref, ThotMenu parent, char *title,
+			    int number, char *text, char* equiv);
 
 /*----------------------------------------------------------------------
    TtaSetPulldownOff suspend le pulldown                           
   ----------------------------------------------------------------------*/
-extern void         TtaSetPulldownOff (int ref, ThotWidget parent);
+extern void TtaSetPulldownOff (int ref, ThotWidget parent);
 
 /*----------------------------------------------------------------------
    TtaSetPulldownOn reactive le pulldown                           
   ----------------------------------------------------------------------*/
-extern void         TtaSetPulldownOn (int ref, ThotWidget parent);
+extern void TtaSetPulldownOn (int ref, ThotWidget parent);
 
 /*----------------------------------------------------------------------
    TtaNewPopup cre'e un pop-up menu :                                 
@@ -86,7 +77,11 @@ extern void         TtaSetPulldownOn (int ref, ThotWidget parent);
    Le parame`tre button indique le bouton de la souris qui active le  
    menu : 'L' pour left, 'M' pour middle et 'R' pour right.           
   ----------------------------------------------------------------------*/
-extern void         TtaNewPopup (int ref, ThotWidget parent, STRING title, int number, STRING text, STRING equiv, char button);
+extern void TtaNewPopup (int ref, ThotWidget parent, char *title, int number,
+			 char *text, char *equiv, char button);
+extern void TtaNewScrollPopup (int ref, ThotWidget parent, char *title, 
+			       int number, char *text, char *equiv, 
+                               ThotBool mulitpleOptions, char button);
 
 /*----------------------------------------------------------------------
    TtaNewSubmenu cre'e un sous-menu :                                 
@@ -103,7 +98,8 @@ extern void         TtaNewPopup (int ref, ThotWidget parent, STRING title, int n
    Quand le parame`tre react est vrai, tout changement de se'lection  
    dans le sous-menu est imme'diatement signale' a` l'application.    
   ----------------------------------------------------------------------*/
-extern void         TtaNewSubmenu (int ref, int ref_parent, int entry, STRING title, int number, STRING text, char* equiv, ThotBool react);
+extern void TtaNewSubmenu (int ref, int ref_parent, int entry, char *title,
+			   int number, char *text, char* equiv, ThotBool react);
 
 /*----------------------------------------------------------------------
    TtaNewIconMenu cre'e un sous-menu :                                        
@@ -116,14 +112,15 @@ extern void         TtaNewSubmenu (int ref, int ref_parent, int entry, STRING ti
    Tout changement de se'lection dans le sous-menu est imme'diatement 
    signale' a` l'application.                                         
   ----------------------------------------------------------------------*/
-extern void         TtaNewIconMenu (int ref, int ref_parent, int entry, STRING title, int number, Pixmap * icons, ThotBool horizontal);
+extern void TtaNewIconMenu (int ref, int ref_parent, int entry, char *title,
+			    int number, Pixmap * icons, ThotBool horizontal);
 
 /*----------------------------------------------------------------------
    TtaSetMenuForm fixe la selection dans un sous-menu de formulaire : 
    Le parame`tre ref donne la re'fe'rence du catalogue.               
    Le parame`tre val de'signe l'entre'e se'lectionne'e.               
   ----------------------------------------------------------------------*/
-extern void         TtaSetMenuForm (int ref, int val);
+extern void TtaSetMenuForm (int ref, int val);
 
 /*----------------------------------------------------------------------
    TtaNewToggleMenu cre'e un sous-menu a` choix multiples :           
@@ -139,7 +136,8 @@ extern void         TtaSetMenuForm (int ref, int val);
    Quand le parame`tre react est vrai, tout changement de se'lection  
    dans le sous-menu est imme'diatement signale' a` l'application.    
   ----------------------------------------------------------------------*/
-extern void         TtaNewToggleMenu (int ref, int ref_parent, STRING title, int number, STRING text, STRING equiv, ThotBool react);
+extern void TtaNewToggleMenu (int ref, int ref_parent, char *title, int number,
+			      char *text, char *equiv, ThotBool react);
 
 /*----------------------------------------------------------------------
    TtaSetToggleMenu fixe la selection dans un toggle-menu :           
@@ -148,24 +146,25 @@ extern void         TtaNewToggleMenu (int ref, int ref_parent, STRING title, int
    toutes les entre'es). Le parame`tre on indique que le bouton       
    correspondant doit e^tre allume' (on positif) ou e'teint (on nul). 
   ----------------------------------------------------------------------*/
-extern void         TtaSetToggleMenu (int ref, int val, ThotBool on);
+extern void TtaSetToggleMenu (int ref, int val, ThotBool on);
 
 /*----------------------------------------------------------------------
    TtaChangeMenuEntry modifie l'intitule' texte de l`entre'e entry    
    du menu de'signe' par sa re'fe'rence ref.                          
   ----------------------------------------------------------------------*/
-extern void         TtaChangeMenuEntry (int ref, int entry, STRING texte);
+extern void TtaChangeMenuEntry (int ref, int entry, char *texte);
 
 /*----------------------------------------------------------------------
    TtaRedrawMenuEntry modifie la couleur et/ou la police de l'entre'e 
    entry du menu de'signe' par sa re'fe'rence ref.                    
   ----------------------------------------------------------------------*/
-extern void         TtaRedrawMenuEntry (int ref, int entry, STRING fontname, Pixel color, int activate);
+extern void TtaRedrawMenuEntry (int ref, int entry, char *fontname,
+				Pixel color, int activate);
 
 /*----------------------------------------------------------------------
    TtaDestroyDialogue de'truit le catalogue de'signe' par ref.                
   ----------------------------------------------------------------------*/
-extern void         TtaDestroyDialogue (int ref);
+extern void TtaDestroyDialogue (int ref);
 
 /*----------------------------------------------------------------------
    TtaNewForm cre'e un formulaire :                                   
@@ -179,7 +178,8 @@ extern void         TtaDestroyDialogue (int ref);
    Le parame'tre button indique le bouton de la souris qui active le  
    menu : 'L' pour left, 'M' pour middle et 'R' pour right.           
   ----------------------------------------------------------------------*/
-extern void         TtaNewForm (int ref, ThotWidget parent, STRING title, ThotBool horizontal, int packet, char button, int dbutton);
+extern void TtaNewForm (int ref, ThotWidget parent, char *title,
+			ThotBool horizontal, int packet, char button, int dbutton);
 
 /*----------------------------------------------------------------------
    TtaNewSheet cre'e un feuillet de commande :                        
@@ -198,7 +198,9 @@ extern void         TtaNewForm (int ref, ThotWidget parent, STRING title, ThotBo
    Le parame`tre button indique le bouton de la souris qui active le  
    menu : 'L' pour left, 'M' pour middle et 'R' pour right.           
   ----------------------------------------------------------------------*/
-extern void         TtaNewSheet (int ref, ThotWidget parent, STRING title, int number, STRING text, ThotBool horizontal, int packet, CHAR_T button, int dbutton);
+extern void TtaNewSheet (int ref, ThotWidget parent, char *title, int number,
+			 char *text, ThotBool horizontal, int packet,
+			 char button, int dbutton);
 
 /*----------------------------------------------------------------------
    TtaNewDialogSheet cre'e un feuillet de dialogue :                  
@@ -216,7 +218,8 @@ extern void         TtaNewSheet (int ref, ThotWidget parent, STRING title, int n
    Le parame`tre button indique le bouton de la souris qui active le  
    menu : 'L' pour left, 'M' pour middle et 'R' pour right.           
   ----------------------------------------------------------------------*/
-void                TtaNewDialogSheet (int ref, ThotWidget parent, STRING title, int number, STRING text, ThotBool horizontal, int packet, CHAR_T button);
+void TtaNewDialogSheet (int ref, ThotWidget parent, char *title, int number,
+			char *text, ThotBool horizontal, int packet, char button);
 
 /*----------------------------------------------------------------------
    TtaChangeFormTitle change le titre d'un formulaire ou d'une feuille        
@@ -224,19 +227,39 @@ void                TtaNewDialogSheet (int ref, ThotWidget parent, STRING title,
    Le parame`tre ref donne la re'fe'rence du catalogue.               
    Le parame'tre title donne le titre du catalogue.                   
   ----------------------------------------------------------------------*/
-extern void         TtaChangeFormTitle (int ref, STRING title);
+extern void TtaChangeFormTitle (int ref, char *title);
 
 /*----------------------------------------------------------------------
    TtaAttachForm attache le catalogue au formulaire ou au feuillet    
    dont il de'pend. Les catalogues sont cre'e's attache's.            
   ----------------------------------------------------------------------*/
-extern void         TtaAttachForm (int ref);
+extern void TtaAttachForm (int ref);
 
 /*----------------------------------------------------------------------
    TtaDetachForm detache le catalogue au formulaire ou au feuillet    
    dont il de'pend. Les catalogues sont cre'e's attache's.            
   ----------------------------------------------------------------------*/
-extern void         TtaDetachForm (int ref);
+extern void TtaDetachForm (int ref);
+
+/*----------------------------------------------------------------------
+   TtaNewSizedSelector creates a selector of a given width in a dialogue form:
+   The parameter ref donne la re'fe'rence pour l'application.
+   The parameter title donne le titre du catalogue.             
+   The parameter number donne le nombre d'intitule's.          
+   The parameter text contient la liste des intitule's.   
+   The parameter height donne le nombre d'intitule's visibles a` la
+   fois (hauteur de la fenetree^tre de visualisation).
+   The parameter label ajoute un choix supple'mentaire a` la liste
+   des choix possibles. Ce choix est affiche' se'pare'ment et donc
+   mis en e'vidence. 
+   The parameter withText indique s'il faut creer une zone texte.
+   Quand le parame`tre react est vrai, tout changement de se'lection
+   dans le se'lecteur est imme'diatement signale' a` l'application.
+  ----------------------------------------------------------------------*/
+extern void TtaNewSizedSelector (int ref, int ref_parent, char *title,
+				 int number, char *text, int width, int height,
+				 char *label, ThotBool withText,
+				 ThotBool react);
 
 /*----------------------------------------------------------------------
    TtaNewSelector cre'e un se'lecteur dans un formulaire :            
@@ -253,7 +276,9 @@ extern void         TtaDetachForm (int ref);
    Quand le parame`tre react est vrai, tout changement de se'lection  
    dans le se'lecteur est imme'diatement signale' a` l'application.   
   ----------------------------------------------------------------------*/
-extern void         TtaNewSelector (int ref, int ref_parent, STRING title, int number, STRING text, int height, STRING label, ThotBool withText, ThotBool react);
+extern void TtaNewSelector (int ref, int ref_parent, char *title, int number,
+			    char *text, int height, char *label,
+			    ThotBool withText, ThotBool react);
 
 /*----------------------------------------------------------------------
    TtaSetSelector initialise l'entre'e et/ou le texte du se'lecteur : 
@@ -262,14 +287,14 @@ extern void         TtaNewSelector (int ref, int ref_parent, STRING title, int n
    se'lectionne'e.                                                    
    Le parame`tre text donne le texte si entry vaut -1.                
   ----------------------------------------------------------------------*/
-extern void         TtaSetSelector (int ref, int entry, STRING text);
+extern void TtaSetSelector (int ref, int entry, char *text);
 
 /*----------------------------------------------------------------------
    TtaNewLabel cre'e un intitule' constant dans un formulaire :       
    Le parame`tre ref donne la re'fe'rence du catalogue.               
    Le parame`tre text donne l'intitule'.                              
   ----------------------------------------------------------------------*/
-extern void         TtaNewLabel (int ref, int ref_parent, STRING text);
+extern void TtaNewLabel (int ref, int ref_parent, char *text);
 
 /*----------------------------------------------------------------------
    TtaNewTextForm cre'e une feuille de saisie de texte :              
@@ -280,14 +305,27 @@ extern void         TtaNewLabel (int ref, int ref_parent, STRING text);
    Quand le parame`tre react est vrai, tout changement dans la        
    feuille de saisie est imme'diatement signale' a` l'application.    
   ----------------------------------------------------------------------*/
-extern void         TtaNewTextForm (int ref, int ref_parent, STRING title, int width, int height, ThotBool react);
+extern void TtaNewTextForm (int ref, int ref_parent, char *title, int width,
+			    int height, ThotBool react);
+
+
+/*----------------------------------------------------------------------
+   TtaNewPwdForm creates a dialogue element to input password text:   
+   Parameter ref gives the Thot reference.
+   Parameter title gives the dialogue title.
+   Parameters width and height give the box size. 
+   If the parameter react is TRUE, any change in the input box generates a
+   callback to the application.
+  ----------------------------------------------------------------------*/
+extern void TtaNewPwdForm (int ref, int ref_parent, char *title, int width,
+			   int height, ThotBool react);
 
 /*----------------------------------------------------------------------
    TtaSetTextForm initialise une feuille de saisie de texte :         
    Le parame`tre ref donne la re'fe'rence du catalogue.               
    Le parame`tre text donne la valeur initiale.                       
   ----------------------------------------------------------------------*/
-extern void         TtaSetTextForm (int ref, STRING text);
+extern void TtaSetTextForm (int ref, char *text);
 
 /*----------------------------------------------------------------------
    TtaNewNumberForm cre'e une feuille de saisie de nombre :           
@@ -298,83 +336,55 @@ extern void         TtaSetTextForm (int ref, STRING text);
    Quand le parame`tre react est vrai, tout changement dans           
    la feuille de saisie est imme'diatement signale' a` l'application. 
   ----------------------------------------------------------------------*/
-extern void         TtaNewNumberForm (int ref, int ref_parent, STRING title, int min, int max, ThotBool react);
+extern void TtaNewNumberForm (int ref, int ref_parent, char *title, int min,
+			      int max, ThotBool react);
 
 /*----------------------------------------------------------------------
    TtaSetNumberForm fixe le contenu de la feuille de saisie de texte :        
    Le parame`tre ref donne la re'fe'rence du catalogue.               
    Le parame`tre val donne la valeur initiale.                        
   ----------------------------------------------------------------------*/
-extern void         TtaSetNumberForm (int ref, int val);
+extern void TtaSetNumberForm (int ref, int val);
 
 /*----------------------------------------------------------------------
    TtaSetDialoguePosition me'morise la position actuelle de la souris 
    comme position d'affichage des TtaShowDialogue().               
   ----------------------------------------------------------------------*/
-extern void         TtaSetDialoguePosition (void);
+extern void TtaSetDialoguePosition (void);
 
 /*----------------------------------------------------------------------
    TtaShowDialogue active le catalogue de'signe.                      
   ----------------------------------------------------------------------*/
-extern void         TtaShowDialogue (int ref, ThotBool remanent);
+extern void TtaShowDialogue (int ref, ThotBool remanent);
+
+/*----------------------------------------------------------------------
+   TtaWaitShowProcDialogue waits for a catalogue (that has its own
+   callback handler) to end.
+  ----------------------------------------------------------------------*/
+extern void TtaWaitShowProcDialogue (void);
 
 /*----------------------------------------------------------------------
    TtaWaitShowDialogue attends le retour du catalogue affiche par     
    TtaShowDialogue.                                                   
   ----------------------------------------------------------------------*/
-extern void         TtaWaitShowDialogue (void);
+extern void TtaWaitShowDialogue (void);
 
 /*----------------------------------------------------------------------
    TtaTestWaitShowDialogue retourne Vrai (1) si un TtaWaitShowDialogue        
    est en cours, sinon Faux (0).                                      
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaTestWaitShowDialogue (void);
+extern ThotBool TtaTestWaitShowDialogue (void);
 
 /*----------------------------------------------------------------------
    TtaAbortShowDialogue abandonne le TtaShowDialogue.                 
   ----------------------------------------------------------------------*/
-extern void         TtaAbortShowDialogue (void);
+extern void TtaAbortShowDialogue (void);
 
 /*----------------------------------------------------------------------
    TtaUnmapDialogue desactive le dialogue s'il est actif.             
   ----------------------------------------------------------------------*/
-extern void         TtaUnmapDialogue (int ref);
+extern void TtaUnmapDialogue (int ref);
 
-#else  /* __STDC__ */
-
-extern void         TtaNewPulldown ( /* int ref, ThotWidget parent, char *title, int number, char *text, char *equiv */ );
-extern void         TtaSetPulldownOff ( /* int ref, ThotWidget parent */ );
-extern void         TtaSetPulldownOn ( /* int ref, ThotWidget parent */ );
-extern void         TtaNewPopup ( /* int ref, ThotWidget parent, char *title, int number, char *text, char *equiv, char button */ );
-extern void         TtaNewSubmenu ( /* int ref, int ref_parent, int entry, char *title, int number, char *text, char *equiv, ThotBool react */ );
-extern void         TtaNewIconMenu ( /*int ref, int ref_parent, int entry, char *title, int number, Pixmap *icons,  ThotBool horizontal */ );
-extern void         TtaSetMenuForm ( /* int ref, int val */ );
-extern void         TtaNewToggleMenu ( /* int ref, int ref_parent, char *title, int number, char *text, char *equiv, ThotBool react */ );
-extern void         TtaSetToggleMenu ( /* int ref, int val, ThotBool on */ );
-extern void         TtaChangeMenuEntry ( /* int ref, int entry, char *texte */ );
-extern void         TtaRedrawMenuEntry ( /* int ref, int entry, char *fontname, Pixel color, int activate */ );
-extern void         TtaDestroyDialogue ( /* int ref */ );
-extern void         TtaNewForm (/* int ref, ThotWidget parent, STRING title, ThotBool horizontal, int packet, char button, int dbutton */);
-extern void         TtaNewSheet (/* int ref, ThotWidget parent, STRING title, int number, STRING text, ThotBool horizontal, int packet, CHAR_T button, int dbutton */);
-void                TtaNewDialogSheet (/* int ref, ThotWidget parent, STRING title, int number, STRING text, ThotBool horizontal, int packet, CHAR_T button */);
-extern void         TtaChangeFormTitle ( /* int ref, char *title */ );
-extern void         TtaAttachForm ( /* int ref */ );
-extern void         TtaDetachForm ( /* int ref */ );
-extern void         TtaNewSelector ( /* int ref, int ref_parent, char *title, int number, char *text, int height, char *label, ThotBool withText, ThotBool react */ );
-extern void         TtaSetSelector ( /* int ref, int entry, char *text */ );
-extern void         TtaNewLabel ( /* int ref, int ref_parent, char *text */ );
-extern void         TtaNewTextForm ( /* int ref, int ref_parent, char *title, int width, int height, ThotBool react */ );
-extern void         TtaSetTextForm ( /* int ref, char *text */ );
-extern void         TtaNewNumberForm ( /* int ref, int ref_parent, char *title, int min, int max, ThotBool react */ );
-extern void         TtaSetNumberForm ( /* int ref, int val */ );
-extern void         TtaSetDialoguePosition ( /* void */ );
-extern void         TtaShowDialogue ( /* int ref, ThotBool remanent */ );
-extern void         TtaWaitShowDialogue ( /* void */ );
-extern ThotBool     TtaTestWaitShowDialogue ( /* void */ );
-extern void         TtaAbortShowDialogue ( /* void */ );
-extern void         TtaUnmapDialogue ( /* int ref */ );
-
-#endif /* __STDC__ */
 #endif /* __CEXTRACT__ */
 
 #endif
