@@ -1,19 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2001.
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
- 
 #ifndef _DOCUMENT_H_
 #define _DOCUMENT_H_
 
@@ -32,8 +23,6 @@ typedef int        *SSchema;
 #define		CONFIG_EXCLUSION		4
 
 #ifndef __CEXTRACT__
-#ifdef __STDC__
-
 
 /*----------------------------------------------------------------------
    TtaInitDocument
@@ -55,7 +44,8 @@ typedef int        *SSchema;
    the document that has been created or 0 if the document has not
    been created.
   ----------------------------------------------------------------------*/
-extern Document     TtaInitDocument (CHAR_T* structureSchema, CHAR_T* documentName, Document document);
+extern Document TtaInitDocument (char *structureSchema, char *documentName,
+				 Document document);
 
 /*----------------------------------------------------------------------
    TtaNewDocument
@@ -76,7 +66,7 @@ extern Document     TtaInitDocument (CHAR_T* structureSchema, CHAR_T* documentNa
    the document that has been created or 0 if the document has not
    been created.
   ----------------------------------------------------------------------*/
-extern Document     TtaNewDocument (CHAR_T* structureSchema, CHAR_T* documentName);
+extern Document TtaNewDocument (char *structureSchema, char *documentName);
 
 /*----------------------------------------------------------------------
    TtaOpenDocument
@@ -93,7 +83,7 @@ extern Document     TtaNewDocument (CHAR_T* structureSchema, CHAR_T* documentNam
    the opened document, or 0 if the document cannot be open.
 
   ----------------------------------------------------------------------*/
-extern Document     TtaOpenDocument (STRING documentName, int accessMode);
+extern Document TtaOpenDocument (char *documentName, int accessMode);
 
 /*----------------------------------------------------------------------
    TtaSaveDocument
@@ -113,7 +103,7 @@ extern Document     TtaOpenDocument (STRING documentName, int accessMode);
    removed by the function TtaRemoveDocument.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSaveDocument (Document document, STRING documentName);
+extern void TtaSaveDocument (Document document, char *documentName);
 
 /*----------------------------------------------------------------------
    TtaExportDocument
@@ -132,7 +122,8 @@ extern void         TtaSaveDocument (Document document, STRING documentName);
    
    Return: TRUE if done correctly
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaExportDocument (Document document, STRING fileName, STRING TSchemaName);
+extern ThotBool TtaExportDocument (Document document, char *fileName,
+				   char *TSchemaName);
 
 /*----------------------------------------------------------------------
    TtaExportDocumentWithNewLineNumbers
@@ -153,9 +144,9 @@ extern ThotBool     TtaExportDocument (Document document, STRING fileName, STRIN
 
    Return: TRUE if done correctly
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaExportDocumentWithNewLineNumbers (Document document,
-							 STRING fileName,
-							 STRING TSchemaName);
+extern ThotBool TtaExportDocumentWithNewLineNumbers (Document document,
+						     char *fileName,
+						     char *TSchemaName);
 
 /*----------------------------------------------------------------------
    TtaCloseDocument
@@ -167,7 +158,7 @@ extern ThotBool     TtaExportDocumentWithNewLineNumbers (Document document,
    document: the document to be closed.
 
   ----------------------------------------------------------------------*/
-extern void         TtaCloseDocument (Document document);
+extern void TtaCloseDocument (Document document);
 
 /*----------------------------------------------------------------------
    TtaRemoveDocument
@@ -179,7 +170,7 @@ extern void         TtaCloseDocument (Document document);
    Parameter:
    document: the document to be removed.
   ----------------------------------------------------------------------*/
-extern void         TtaRemoveDocument (Document document);
+extern void TtaRemoveDocument (Document document);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentPath
@@ -195,7 +186,7 @@ extern void         TtaRemoveDocument (Document document);
    the character PATH_SEP.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentPath (STRING path);
+extern void TtaSetDocumentPath (char *path);
 
 /*----------------------------------------------------------------------
    TtaCheckPath
@@ -210,7 +201,7 @@ extern void         TtaSetDocumentPath (STRING path);
    accessed.
 
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaCheckPath (char* path );
+extern ThotBool TtaCheckPath (char *path);
 
 /*----------------------------------------------------------------------
    TtaIsInDocumentPath
@@ -221,7 +212,7 @@ extern ThotBool     TtaCheckPath (char* path );
    directory: the new directory name.
 
   ----------------------------------------------------------------------*/
-extern ThotBool     TtaIsInDocumentPath (CHAR_T* aDirectory);
+extern ThotBool TtaIsInDocumentPath (char *aDirectory);
 
 /*----------------------------------------------------------------------
    TtaAppendDocumentPath
@@ -233,7 +224,7 @@ extern ThotBool     TtaIsInDocumentPath (CHAR_T* aDirectory);
    directory: the new directory name.
 
   ----------------------------------------------------------------------*/
-extern void         TtaAppendDocumentPath (CHAR_T* aDirectory);
+extern void TtaAppendDocumentPath (char *aDirectory);
 
 /*----------------------------------------------------------------------
    TtaSetSchemaPath
@@ -246,7 +237,7 @@ extern void         TtaAppendDocumentPath (CHAR_T* aDirectory);
    the character PATH_SEP.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetSchemaPath (STRING path);
+extern void TtaSetSchemaPath (char *path);
 
 /*----------------------------------------------------------------------
    TtaNewNature
@@ -269,7 +260,8 @@ extern void         TtaSetSchemaPath (STRING path);
    has not been loaded.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaNewNature (Document document, SSchema schema, CHAR_T* natureName, CHAR_T* presentationName);
+extern SSchema TtaNewNature (Document document, SSchema schema,
+			     char *natureName, char *presentationName);
 
 /*----------------------------------------------------------------------
    TtaNewSchemaExtension
@@ -288,7 +280,8 @@ extern SSchema      TtaNewNature (Document document, SSchema schema, CHAR_T* nat
    the extension schema, NULL if the extension schema has not been loaded.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaNewSchemaExtension (Document document, CHAR_T* extensionName, CHAR_T* presentationName);
+extern SSchema TtaNewSchemaExtension (Document document, char *extensionName,
+				      char *presentationName);
 
 /*----------------------------------------------------------------------
    TtaGetSchemaExtension
@@ -304,7 +297,7 @@ extern SSchema      TtaNewSchemaExtension (Document document, CHAR_T* extensionN
    NULL if the extension is not associated with the document.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaGetSchemaExtension (Document document, STRING NomExtension);
+extern SSchema TtaGetSchemaExtension (Document document, char *NomExtension);
 
 /*----------------------------------------------------------------------
    TtaRemoveSchemaExtension
@@ -322,7 +315,8 @@ extern SSchema      TtaGetSchemaExtension (Document document, STRING NomExtensio
    removedAttributes: number of attributes actually removed.
 
   ----------------------------------------------------------------------*/
-extern void         TtaRemoveSchemaExtension (Document document, SSchema extension, int *removedElements, int *removedAttributes);
+extern void TtaRemoveSchemaExtension (Document document, SSchema extension,
+				      int *removedElements, int *removedAttributes);
 
 /*----------------------------------------------------------------------
    TtaSetPSchema
@@ -336,7 +330,7 @@ extern void         TtaRemoveSchemaExtension (Document document, SSchema extensi
    with the document.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetPSchema (Document document, CHAR_T* presentationName);
+extern void TtaSetPSchema (Document document, char *presentationName);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentDirectory
@@ -348,7 +342,7 @@ extern void         TtaSetPSchema (Document document, CHAR_T* presentationName);
    directory: new document directory.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentDirectory (Document document, CHAR_T* directory);
+extern void TtaSetDocumentDirectory (Document document, char *directory);
 
 /*----------------------------------------------------------------------
    TtaRemoveSchemaExtension
@@ -366,7 +360,7 @@ extern void         TtaSetDocumentDirectory (Document document, CHAR_T* director
    removedAttributes: number of attributes actually removed.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentName (Document document, STRING documentName);
+extern void TtaSetDocumentName (Document document, char *documentName);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentAccessMode
@@ -393,7 +387,7 @@ extern void         TtaSetDocumentAccessMode (Document document, int accessMode)
    automatic save of the document into a .BAK file.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentBackUpInterval (Document document, int interval);
+extern void TtaSetDocumentBackUpInterval (Document document, int interval);
 
 /*----------------------------------------------------------------------
    TtaSetNotificationMode
@@ -406,7 +400,7 @@ extern void         TtaSetDocumentBackUpInterval (Document document, int interva
    be notified.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetNotificationMode (Document document, int notificationMode);
+extern void TtaSetNotificationMode (Document document, int notificationMode);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentModified
@@ -418,7 +412,7 @@ extern void         TtaSetNotificationMode (Document document, int notificationM
    document: the document.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentModified (Document document);
+extern void TtaSetDocumentModified (Document document);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentUnmodified
@@ -432,7 +426,7 @@ extern void         TtaSetDocumentModified (Document document);
    document: the document.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentUnmodified (Document document);
+extern void TtaSetDocumentUnmodified (Document document);
 
 /*----------------------------------------------------------------------
    TtaSetDocumentUnupdated
@@ -446,7 +440,7 @@ extern void         TtaSetDocumentUnmodified (Document document);
    document: the document.
 
   ----------------------------------------------------------------------*/
-extern void         TtaSetDocumentUnupdated (Document document);
+extern void TtaSetDocumentUnupdated (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentName
@@ -460,7 +454,7 @@ extern void         TtaSetDocumentUnupdated (Document document);
    name of that document.
 
   ----------------------------------------------------------------------*/
-extern CHAR_T*       TtaGetDocumentName (Document document);
+extern char *TtaGetDocumentName (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentFromName
@@ -474,7 +468,7 @@ extern CHAR_T*       TtaGetDocumentName (Document document);
    the document having that name.
 
   ----------------------------------------------------------------------*/
-extern Document     TtaGetDocumentFromName (STRING documentName);
+extern Document TtaGetDocumentFromName (char *documentName);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentDirectory
@@ -490,7 +484,8 @@ extern Document     TtaGetDocumentFromName (STRING documentName);
    buffer: the document directory.
 
   ----------------------------------------------------------------------*/
-extern void         TtaGetDocumentDirectory (Document document, STRING buffer, int bufferLength);
+extern void TtaGetDocumentDirectory (Document document, char *buffer,
+				     int bufferLength);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentSSchema
@@ -504,7 +499,7 @@ extern void         TtaGetDocumentDirectory (Document document, STRING buffer, i
    the structure schema of that document.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaGetDocumentSSchema (Document document);
+extern SSchema TtaGetDocumentSSchema (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetSSchemaName
@@ -518,7 +513,7 @@ extern SSchema      TtaGetDocumentSSchema (Document document);
    name of that structure schema.
 
   ----------------------------------------------------------------------*/
-extern CHAR_T*        TtaGetSSchemaName (SSchema schema);
+extern char *TtaGetSSchemaName (SSchema schema);
 
 /*----------------------------------------------------------------------
    TtaGetPSchemaName
@@ -533,7 +528,7 @@ extern CHAR_T*        TtaGetSSchemaName (SSchema schema);
    name of the associated presentation schema.
 
   ----------------------------------------------------------------------*/
-extern CHAR_T*       TtaGetPSchemaName (SSchema schema);
+extern char *TtaGetPSchemaName (SSchema schema);
 
 /*----------------------------------------------------------------------
    TtaGetSSchema
@@ -541,7 +536,7 @@ extern CHAR_T*       TtaGetPSchemaName (SSchema schema);
    Returns a structure schema whose name is known and that is used in a
    given document.
 
-   Parameter:
+   Parameters:
    name: the name of the structure schema of interest.
    document: the document that uses this structure schema.
 
@@ -550,7 +545,24 @@ extern CHAR_T*       TtaGetPSchemaName (SSchema schema);
    schema is not loaded or not used by the document.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaGetSSchema (CHAR_T* name, Document document);
+extern SSchema TtaGetSSchema (char *name, Document document);
+
+/*----------------------------------------------------------------------
+   TtaGetSSchemaByUri
+
+   Returns a structure schema whose URI is known and that is used in a
+   given document.
+
+   Parameters:
+   uriName: the URI of the structure schema of interest.
+   document: the document that uses this structure schema.
+
+   Return value:
+   the structure schema having this URI, or NULL if this structure
+   schema is not loaded or not used by the document.
+
+  ----------------------------------------------------------------------*/
+SSchema TtaGetSSchemaByUri (char *uriName, Document document);
 
 /*----------------------------------------------------------------------
    TtaSameSSchemas
@@ -565,7 +577,7 @@ extern SSchema      TtaGetSSchema (CHAR_T* name, Document document);
    0 if both schemas are different, 1 if they are identical.
 
   ----------------------------------------------------------------------*/
-extern int          TtaSameSSchemas (SSchema schema1, SSchema schema2);
+extern int TtaSameSSchemas (SSchema schema1, SSchema schema2);
 
 /*----------------------------------------------------------------------
    TtaGiveSchemasOfDocument
@@ -585,7 +597,9 @@ extern int          TtaSameSSchemas (SSchema schema1, SSchema schema2);
    presentationName: Name of the document presentation schema.
 
   ----------------------------------------------------------------------*/
-extern void         TtaGiveSchemasOfDocument (STRING documentName, /*OUT*/ char* structureName, /*OUT*/ char* presentationName);
+extern void TtaGiveSchemasOfDocument (char *documentName,
+				      /*OUT*/ char* structureName,
+				      /*OUT*/ char* presentationName);
 
 /*----------------------------------------------------------------------
    TtaNextSchemaExtension
@@ -602,7 +616,7 @@ extern void         TtaGiveSchemasOfDocument (STRING documentName, /*OUT*/ char*
    NULL if there is no more schema extension.
 
   ----------------------------------------------------------------------*/
-extern void         TtaNextSchemaExtension (Document document, /*INOUT*/ SSchema * extension);
+extern void TtaNextSchemaExtension (Document document, /*INOUT*/ SSchema * extension);
 
 /*----------------------------------------------------------------------
    TtaNextNature
@@ -620,7 +634,7 @@ extern void         TtaNextSchemaExtension (Document document, /*INOUT*/ SSchema
    NULL if there is no more nature for the document.
 
   ----------------------------------------------------------------------*/
-extern void         TtaNextNature (Document document, /*INOUT*/ SSchema * nature);
+extern void TtaNextNature (Document document, /*INOUT*/ SSchema *nature);
 
 /*----------------------------------------------------------------------
    TtaIsDocumentModified
@@ -637,7 +651,7 @@ extern void         TtaNextNature (Document document, /*INOUT*/ SSchema * nature
    loaded or created, 0 if it has not been modified.
 
   ----------------------------------------------------------------------*/
-extern int          TtaIsDocumentModified (Document document);
+extern int TtaIsDocumentModified (Document document);
 
 /*----------------------------------------------------------------------
    TtaIsDocumentUpdated
@@ -655,7 +669,7 @@ extern int          TtaIsDocumentModified (Document document);
    loaded or created, 0 if it has not been modified.
 
   ----------------------------------------------------------------------*/
-extern int          TtaIsDocumentUpdated (Document document);
+extern int TtaIsDocumentUpdated (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentAccessMode
@@ -668,7 +682,7 @@ extern int          TtaIsDocumentUpdated (Document document);
    0 if access mode is read only, 1 if access mode is read-write.
 
   ----------------------------------------------------------------------*/
-extern int          TtaGetDocumentAccessMode (Document document);
+extern int TtaGetDocumentAccessMode (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentBackUpInterval
@@ -683,7 +697,7 @@ extern int          TtaGetDocumentAccessMode (Document document);
    save of the document into a .BAK file.
 
   ----------------------------------------------------------------------*/
-extern int          TtaGetDocumentBackUpInterval (Document document);
+extern int TtaGetDocumentBackUpInterval (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetNotificationMode
@@ -697,7 +711,7 @@ extern int          TtaGetDocumentBackUpInterval (Document document);
    1 = all elements of created and deleted subtrees must be notified.
 
   ----------------------------------------------------------------------*/
-extern int          TtaGetNotificationMode (Document document);
+extern int TtaGetNotificationMode (Document document);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentPath
@@ -714,7 +728,7 @@ extern int          TtaGetNotificationMode (Document document);
    the character PATH_SEP.
 
   ----------------------------------------------------------------------*/
-extern void         TtaGetDocumentPath (/*OUT*/ STRING buffer, int bufferLength);
+extern void TtaGetDocumentPath (/*OUT*/ char *buffer, int bufferLength);
 
 /*----------------------------------------------------------------------
    TtaGetSchemaPath
@@ -730,7 +744,7 @@ extern void         TtaGetDocumentPath (/*OUT*/ STRING buffer, int bufferLength)
    the character PATH_SEP.
 
   ----------------------------------------------------------------------*/
-extern void         TtaGetSchemaPath (/*OUT*/ STRING buffer, int bufferLength);
+extern void TtaGetSchemaPath (/*OUT*/ char *buffer, int bufferLength);
 
 /*----------------------------------------------------------------------
    TtaGetDocumentOfSavedElements
@@ -746,7 +760,7 @@ extern void         TtaGetSchemaPath (/*OUT*/ STRING buffer, int bufferLength);
    0 if the clipboard is empty.
 
   ----------------------------------------------------------------------*/
-extern Document     TtaGetDocumentOfSavedElements ();
+extern Document TtaGetDocumentOfSavedElements ();
 
 
 /*----------------------------------------------------------------------
@@ -754,7 +768,7 @@ extern Document     TtaGetDocumentOfSavedElements ();
    structure (documents, natures et extensions) qui ont    
    des fichiers de langue dans les directories de schemas. 
   ----------------------------------------------------------------------*/
-extern void TtaConfigReadConfigFiles ( CHAR_T* aSchemaPath );
+extern void TtaConfigReadConfigFiles (char *aSchemaPath);
 
 /*----------------------------------------------------------------------
    TtaConfigSSchemaExternalName retourne dans nameUser le nom     
@@ -763,7 +777,8 @@ extern void TtaConfigReadConfigFiles ( CHAR_T* aSchemaPath );
    Typ indique s'il s'agit d'un schema de document (1), de         
    nature (2) ou d'extension (3).                                  
   ----------------------------------------------------------------------*/
-extern void TtaConfigSSchemaExternalName (/*OUT*/ CHAR_T* nameUser, CHAR_T* nameSchema, int Typ );
+extern void TtaConfigSSchemaExternalName (/*OUT*/ char *nameUser,
+					  char *nameSchema, int Typ );
 
 /*----------------------------------------------------------------------
   TtaGetDocumentCharset gets the document charset
@@ -777,62 +792,38 @@ extern CHARSET TtaGetDocumentCharset (Document document);
 extern void  TtaSetDocumentCharset (Document document,
 				    CHARSET charSet);
 
-#else  /* __STDC__ */
+/*----------------------------------------------------------------------
+  TtaGetDocumentProfile
+  Gets the document profile
+ ----------------------------------------------------------------------*/
+extern int TtaGetDocumentProfile (Document document);
 
-extern Document     TtaInitDocument (/*CHAR_T* structureSchema, CHAR_T* documentName, Document document*/);
-extern Document     TtaNewDocument ( /* CHAR_T* structureSchema, CHAR_T* documentName */ );
-extern Document     TtaOpenDocument ( /* char *documentName, int accessMode */ );
-extern void         TtaSaveDocument ( /* Document document, char *documentName */ );
-extern ThotBool     TtaExportDocument ( /* Document document, char *fileName, char* TSchemaName */ );
-extern ThotBool     TtaExportDocumentWithNewLineNumbers ( /* Document document, char *fileName, char* TSchemaName */ );
-extern void         TtaCloseDocument ( /* Document document */ );
-extern void         TtaRemoveDocument ( /* Document document */ );
-extern void         TtaSetDocumentPath ( /* char *path */ );
-extern ThotBool     TtaCheckPath (/* char *path */);
-extern ThotBool     TtaIsInDocumentPath ( /*CHAR_T* aDirectory */ );
-extern void         TtaAppendDocumentPath ( /*CHAR_T* aDirectory */ );
-extern void         TtaSetSchemaPath ( /* char *path */ );
-extern SSchema      TtaNewNature ( /* Document document, SSchema schema, CHAR_T* natureName, CHAR_T* presentationName */ );
-extern SSchema      TtaNewSchemaExtension ( /* Document document, CHAR_T* extensionName, CHAR_T* presentationName */ );
-extern SSchema      TtaGetSchemaExtension ( /*Document document, char *NomExtension */ );
-extern void         TtaRemoveSchemaExtension ( /* Document document, SSchema extension, int *removedElements, int *removedAttributes */ );
-extern void         TtaSetPSchema ( /* Document document, CHAR_T* presentationName */ );
-extern void         TtaSetDocumentDirectory ( /* Document document, CHAR_T* directory */ );
-extern void         TtaSetDocumentName ( /* Document document, char *documentName */ );
-extern void         TtaSetDocumentAccessMode ( /* Document document, int accessMode */ );
-extern void         TtaSetDocumentBackUpInterval ( /* Document document, int interval */ );
-extern void         TtaSetNotificationMode ( /* Document document, int notificationMode */ );
-extern void         TtaSetDocumentModified ( /* Document document */ );
-extern void         TtaSetDocumentUnmodified ( /* Document document */ );
-extern void         TtaSetDocumentUnupdated ( /* Document document */ );
-extern CHAR_T*      TtaGetDocumentName ( /* Document document */ );
-extern Document     TtaGetDocumentFromName ( /* char *documentName */ );
-extern void         TtaGetDocumentDirectory ( /* Document document, char *buffer, int bufferLength */ );
-extern SSchema      TtaGetDocumentSSchema ( /* Document document */ );
-extern CHAR_T*        TtaGetSSchemaName ( /* SSchema schema */ );
-extern CHAR_T*        TtaGetPSchemaName ( /* SSchema schema */ );
-extern SSchema      TtaGetSSchema ( /* CHAR_T* name, Document document */ );
-extern int          TtaSameSSchemas ( /* SSchema schema1, SSchema schema2 */ );
-extern void         TtaGiveSchemasOfDocument ( /* char *documentName, char *structureName, char *presentationName */ );
-extern void         TtaNextSchemaExtension ( /* Document document, SSchema *extension */ );
-extern void         TtaNextNature ( /* Document document, SSchema *nature */ );
-extern int          TtaIsDocumentModified ( /* Document document */ );
-extern int          TtaIsDocumentUpdated ( /* Document document */ );
-extern int          TtaGetDocumentAccessMode ( /* Document document */ );
-extern int          TtaGetDocumentBackUpInterval ( /* Document document */ );
-extern int          TtaGetNotificationMode ( /* Document document */ );
-extern void         TtaGetDocumentPath ( /* char *buffer, int bufferLength */ );
-extern void         TtaGetSchemaPath ( /* char *buffer, int bufferLength */ );
-extern Document     TtaGetDocumentOfSavedElements ();
+/*----------------------------------------------------------------------
+  TtaSetDocumentProfile
+  Sets the document profile
+ ----------------------------------------------------------------------*/
+extern void TtaSetDocumentProfile (Document document, int profile);
+
+/*----------------------------------------------------------------------
+  TtaSetNamespaceDeclaration
+  Sets one namespace declaration
+ ----------------------------------------------------------------------*/
+void TtaSetNamespaceDeclaration (Document document, Element element,
+				 char *Ns_Prefix, char *Ns_Uri);
+
+/*----------------------------------------------------------------------
+  TtaFreeNamespaceDeclarations
+  Free all the namespaces declarations of a document
+ ----------------------------------------------------------------------*/
+void TtaFreeNamespaceDeclaration (Document document);
+
+
+/*----------------------------------------------------------------------
+  TtaSetUriSSchema
+  Set the schema namespace declaration uri
+ ----------------------------------------------------------------------*/
+void TtaSetUriSSchema (SSchema sSchema, char *sSchemaUri);
 
-extern void         TtaConfigReadConfigFiles (/* CHAR_T* aSchemaPath */);
-extern void         TtaConfigSSchemaExternalName (/* CHAR_T *nameUser,
-                                             CHAR_T *nameSchema,
-                                             int Typ */);
-extern CHARSET      TtaGetDocumentCharset (/* Document document */);
-extern void         TtaSetDocumentCharset (/* Document document,
-					      CHARSET charSet */);
-#endif /* __STDC__ */
 #endif /* __CEXTRACT__ */
 
 #endif
