@@ -11,21 +11,22 @@
 #define MAX_PRO_LENGTH              200       /* Maximum length of a string */
 #define DEF_FILE                   "amaya.profiles"
 
-/* Masks for the profiles */
+/* Masks used for the profiles */
 #define L_Other             0x00 /* No one profile is defined */
-#define L_Basic             0x02 /* accept only Basic values */
-#define L_Strict            0x06 /* accept Strict and Basic values */
-#define L_Xhtml11           0x1E /* accept Strict, Basic and Ruby values */
-#define L_Transitional      0x2E /* accept all values except Ruby */
-#define L_MathML            0x40 /* accept only MathML values */
+#define L_Basic             0x01 /* accept only Basic values */
+#define L_Strict            0x02 /* accept Strict and Basic values */
+#define L_Xhtml11           0x04 /* accept Strict, Basic and Ruby values */
+#define L_Transitional      0x08 /* accept all values except Ruby */
+#define L_MathML            0x10 /* accept only MathML values */
+#define L_SVG               0x20 /* accept only SVG values */
 
 /* Values for the profiles */
-#define L_BasicValue        0x02 /* allowed for all profiles */
-#define L_StrictValue       0x0C /* allowed for Basic and Strict profiles */
-#define L_Xhmli11Value      0x0C /* allowed for Basic, Strict and Xhmli11 profiles */
-#define L_RubyValue         0x10 /* allowed only for Xhmli11 profile */
-#define L_TransitionalValue 0x20 /* allowed only for Transitional profile */
-#define L_MathMLValue       0x7C /* allowed for all profiles except Basic */
+#define L_BasicValue        0x0F /* allowed in all XHTML profiles */
+#define L_StrictValue       0x0E /* allowed in all XHTML profiles except Basic */
+#define L_RubyValue         0x04 /* allowed only in Xhtml11 profile */
+#define L_TransitionalValue 0x08 /* allowed only in Transitional profile */
+#define L_MathMLValue       0x1E /* allowed in MathML profile and  */
+                                 /* XHTML profiles except Basic */
 #define L_OtherValue        0xFF
 
 /*----------------------------------------------------------------------
@@ -39,7 +40,7 @@ extern void TtaRebuildProTable (char *prof_file);
 extern ThotBool TtaCanEdit ();
 
 /*----------------------------------------------------------------------
-   TtaGetProfileFineName:  Get the text for the profile file name.
+   TtaGetProfileFileName:  Get the text for the profile file name.
    name is a provided buffer of length characters to receive the name.
   ----------------------------------------------------------------------*/
 extern void TtaGetProfileFileName (char *name, int length);
