@@ -1,19 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2001.
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
- 
 #ifndef _SELECTION_H_
 #define _SELECTION_H_
 #include "typebase.h"
@@ -21,13 +12,17 @@
 #include "document.h"
 
 #ifndef __CEXTRACT__
-#ifdef __STDC__
 
 /*----------------------------------------------------------------------
    TtaIsDocumentSelected answers TRUE if the document is selected.  
   ----------------------------------------------------------------------*/
 extern ThotBool     TtaIsDocumentSelected (Document document);
 
+/*----------------------------------------------------------------------
+  TtaSelectWord selects the word around the current point in an element
+  ----------------------------------------------------------------------*/
+extern void TtaSelectWord (Element element, int pos, Document doc, View view);
+
 /*----------------------------------------------------------------------
    TtaSelectElement
 
@@ -49,6 +44,12 @@ extern void         TtaSelectElement (Document document, Element selectedElement
   ----------------------------------------------------------------------*/
 extern Document   TtaGetSelectedDocument ();
 
+/*----------------------------------------------------------------------
+   TtaGetActiveView returns the selected document and the active view
+   in that document
+  ----------------------------------------------------------------------*/
+extern void TtaGetActiveView (Document *doc, View *view);
+
 /*----------------------------------------------------------------------
    TtaSelectView posts the selected view in the document
   ----------------------------------------------------------------------*/
@@ -231,25 +232,18 @@ extern void         TtaGiveNextElement (Document document, /*OUT*/ Element * ele
   ----------------------------------------------------------------------*/
 extern void         TtaGiveLastSelectedElement (Document document, /*OUT*/ Element * selectedElement, /*OUT*/ int *firstCharacter, /*OUT*/ int *lastCharacter);
 
-#else  /* __STDC__ */
+/*----------------------------------------------------------------------
+   TtaSetMoveForwardCallback permet de connecter une fonction de
+   l'application a la touche de deplacement du curseur vers l'avant
+  ----------------------------------------------------------------------*/
+extern void  TtaSetMoveForwardCallback (Func callbackFunc);
 
-extern ThotBool     TtaIsDocumentSelected ( /*Document document */ );
-extern Document     TtaGetSelectedDocument ();
-extern void         TtaSelectElement ( /* Document document, Element selectedElement */ );
-extern void         TtaSelectView (/*Document document, View view*/);
-extern void         TtaSelectString ( /* Document document, Element textElement, int firstCharacter, int lastCharacter */ );
-extern void         TtaExtendSelection ( /* Document document, Element element, int lastCharacter */ );
-extern void         TtaAddElementToSelection (/* Document document, Element element */ );
-extern void         TtaSelectInterval ();
-extern void         TtaUnselect ( /* Document document */ );
-extern void         TtaSetSelectionMode ( /* ThotBool withMenu */ );
-extern ThotBool     TtaIsSelectionEmpty ();
-extern void         TtaGiveFirstSelectedElement ( /* Document document, Element *selectedElement, int *firstCharacter, int *lastCharacter */ );
-extern void         TtaGiveNextSelectedElement ( /* Document document, Element *selectedElement, int *firstCharacter, int *lastCharacter */ );
-extern void         TtaGiveNextElement ( /* Document document, Element * element, Element last */ );
-extern void         TtaGiveLastSelectedElement ( /* Document document, Element *selectedElement, int *firstCharacter, int *lastCharacter */ );
+/*----------------------------------------------------------------------
+   TtaSetMoveBackwardCallback permet de connecter une fonction de
+   l'application a la touche de deplacement du curseur vers l'arriere
+  ----------------------------------------------------------------------*/
+extern void  TtaSetMoveBackwardCallback (Func callbackFunc);
 
-#endif /* __STDC__ */
 #endif /* __CEXTRACT__ */
 
 #endif
