@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996-2001
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -21,33 +21,44 @@
 #define __STYLE_H__
 #define MAX_ANCESTORS 5
 
+typedef enum
+{
+  Txtmatch,
+  Txtsubstring,
+  Txtword
+} AttrMatch;
+
 /* two different contexts for generic and specific presentation */
 typedef struct struct_GenericContext
   {
-    Document              doc;	     /* document number */
-    SSchema               schema;    /* associated structure */
-    unsigned int          type;      /* type of element */
-    int                   cssLevel;  /* CSS priority level for generic rules */
-    ThotBool              destroy;   /* destructive mode ? */
+    Document      doc;	          /* document number */
+    SSchema       schema;         /* associated structure */
+    unsigned int  type;           /* type of element */
+    int           cssSpecificity; /* specificity according to selector */
+    ThotBool      important;      /* important rule */
+    ThotBool      destroy;        /* destructive mode ? */
     /*
      * below is the context description.
      */
-    int                   box;	     /* specific presentation box if any */
-    int                   name[MAX_ANCESTORS]; /* ancestors type */
-    int                   names_nb[MAX_ANCESTORS]; /* number of occurences */
-    int                   attrType[MAX_ANCESTORS]; /* type of the attribute */
-    char                 *attrText[MAX_ANCESTORS]; /* attr. or box name */
+    int           box;	                 /* specific presentation box if any */
+    int           name[MAX_ANCESTORS];      /* ancestors type */
+    int           names_nb[MAX_ANCESTORS];  /* number of occurences */
+    int           attrType[MAX_ANCESTORS];  /* type of the attribute */
+    int           attrLevel[MAX_ANCESTORS]; /* level of the attribute */
+    char         *attrText[MAX_ANCESTORS];  /* attr. or box name */
+    AttrMatch     attrMatch[MAX_ANCESTORS]; /* how attr. values match */
   }
 GenericContextBlock, *GenericContext;
 
 typedef struct struct_SpecificContext
   {
-    Document              doc;	     /* document number */
-    SSchema               schema;    /* associated structure */
-    int                   type;      /* type of element */
-    int                   cssLevel;  /* For specific rules: > 0 when the rule
-				        translates a CSS style rule */
-    ThotBool              destroy;  /* destructive mode ? */
+    Document      doc;	          /* document number */
+    SSchema       schema;         /* associated structure */
+    int           type;           /* type of element */
+    int           cssSpecificity; /* For specific rules: > 0 when the rule
+				     translates a CSS style rule */
+    ThotBool      important;      /* important rule */
+    ThotBool      destroy;        /* destructive mode ? */
     /*
      * The end of the block is to be filled with other kind
      * of informations needed to specify the conditions
@@ -129,6 +140,13 @@ typedef struct _PresentationSetting
 #define STYLE_LINEDASHED		2
 #define STYLE_LINEDOTTED		3
 
+#define STYLE_LEFTTORIGHT               1
+#define STYLE_RIGHTTOLEFT               2
+
+#define STYLE_BIDINORMAL                1
+#define STYLE_BIDIEMBED                 2
+#define STYLE_BIDIOVERRIDE              3
+
 #define STYLE_BORDERNONE		1
 #define STYLE_BORDERHIDDEN		2
 #define STYLE_BORDERDOTTED		3
@@ -146,14 +164,19 @@ typedef struct _PresentationSetting
 #define STYLE_ADJUSTLEFT		1
 #define STYLE_ADJUSTRIGHT		2
 #define STYLE_ADJUSTCENTERED		3
-#define STYLE_ADJUSTLEFTWITHDOTS	4
-#define STYLE_ADJUSTJUSTIFY		5
+#define STYLE_ADJUSTJUSTIFY		4
+#define STYLE_ADJUSTLEFTWITHDOTS	5
 
 #define STYLE_INLINE			1
-#define STYLE_NOTINLINE			2
+#define STYLE_BLOCK			2
 
-#define STYLE_HIDE			0
-#define STYLE_DISPLAY			5
+#define STYLE_DISPLAYUNDEFINED          1
+#define STYLE_DISPLAYINLINE             2
+#define STYLE_DISPLAYBLOCK              3
+#define STYLE_DISPLAYLISTITEM           4
+#define STYLE_DISPLAYRUNIN              5
+#define STYLE_DISPLAYCOMPACT            6
+#define STYLE_DISPLAYMARKER             7
 
 #define STYLE_REALSIZE			1
 #define STYLE_SCALE			2
@@ -223,20 +246,23 @@ extern PresentationContext TtaGetSpecificStyleContext (Document doc);
   TtaSetStylePresentation attachs a style rule to an element or to an
   extended presentation schema.
   ----------------------------------------------------------------------*/
-extern int TtaSetStylePresentation (unsigned int type, Element el, PSchema tsch, PresentationContext c, PresentationValue v);
+extern int TtaSetStylePresentation (unsigned int type, Element el, PSchema tsch,
+				    PresentationContext c, PresentationValue v);
 
 /*----------------------------------------------------------------------
   TtaGetStylePresentation returns the style rule attached to an element
   or to an extended presentation schema.
   ----------------------------------------------------------------------*/
-extern int TtaGetStylePresentation (unsigned int type, Element el, PSchema tsch, PresentationContext c, PresentationValue *v);
+extern int TtaGetStylePresentation (unsigned int type, Element el, PSchema tsch,
+				    PresentationContext c, PresentationValue *v);
 
 /*----------------------------------------------------------------------
   ApplyAllSpecificSettings browses all the PRules structures,
   associated to the corresponding Specific Context 
   structure, and calls the given handler for each one.
   ----------------------------------------------------------------------*/
-extern void TtaApplyAllSpecificSettings (Element element, Document doc, SettingsApplyHandler handler, void *param);
+extern void TtaApplyAllSpecificSettings (Element element, Document doc,
+					 SettingsApplyHandler handler, void *param);
 
 /*----------------------------------------------------------------------
   TtaCleanElementPresentation
