@@ -58,43 +58,49 @@ typedef struct struct_ThotGC {
 } WIN_GC_BLK, *WIN_GC, *ThotGC;
 
 typedef struct _FontInfo {
-        int   FiHeight;
-        int   FiAscent;
-        int   FiHeights[256];
-        int   FiWidths[256];
-        int   PFiHeight;
-        int   PFiAscent;
-        int   PFiHeights[256];
-        int   PFiWidths[256];
-        int   highlight; 
-        int   size;
-        char  alphabet; 
-        char  family; 
-        /* HFONT FiFont; */
+  int   FiHeight;
+  int   FiAscent;
+  int   FiFirstChar;
+  int   FiLastChar;
+  int  *FiHeights;
+  int  *FiWidths;
+  int   FiHighlight; 
+  int   FiSize;
+  char  FiScript; 
+  char  FiFamily; 
 } FontInfo;
+typedef FontInfo      *PtrFont;
 
-typedef FontInfo *ptrfont;
-
-typedef void*          ThotWidget;
+typedef void          *ThotWidget;
 typedef HWND           ThotWindow;
-typedef TBBUTTON*      ThotButton;
+typedef TBBUTTON      *ThotButton;
 typedef HMENU          ThotMenu;
 typedef HBITMAP        ThotBitmap;
 typedef COLORREF       ThotColor;
 typedef COLORREF       Pixel;
-typedef void*          Pixmap;
+typedef void          *Pixmap;
 typedef HCURSOR        ThotCursor;
+#ifndef _GL
 typedef HWND           Drawable;
 typedef POINT          ThotPoint;
+#else /* _GL */
+typedef unsigned char* Drawable;
+typedef struct GL_point 
+{
+  float x;
+  float y;
+} ThotPoint;
+#endif /*_GL*/
+
 typedef MSG            ThotEvent;
 typedef MSG            ThotKeyEvent;
 typedef UINT           ThotComposeStatus;
-typedef void*          ThotAppContext;
-typedef void*          ThotTranslations;
-typedef void*          KeySym;
-typedef void*          XtPointer;
-typedef void*          XtIntervalId;
-typedef void*          XtInputId;
+typedef void          *ThotAppContext;
+typedef void          *ThotTranslations;
+typedef void          *KeySym;
+typedef void          *XtPointer;
+typedef void          *XtIntervalId;
+typedef void          *XtInputId;
 typedef int            ThotIcon;
 #define TBBUTTONS_BASE 50000
 #define ThotColorNone  ((COLORREF)~1) /* anything in high byte is bad COLORREF */
@@ -124,7 +130,7 @@ typedef XColor    ThotColorStruct;
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 #include <X11/Intrinsic.h>
-#ifndef _GTK
+#if !defined(_GTK)  && !defined(NODISPLAY)
 #include <Xm/MwmUtil.h>
 #include <Xm/BulletinB.h>
 #include <Xm/CascadeB.h>
@@ -149,7 +155,7 @@ typedef XColor    ThotColorStruct;
 #include <Xm/ToggleB.h>
 #include <Xm/Xm.h>
 #include <Xm/PushBG.h>
-#endif /* !_GTK */
+#endif /* !_GTK && !NODISPLAY */
 /*
  * OWN_XLOOKUPSTRING means we redefine XLookupString function
  * (in api/interface.c) and we need internal informations on
@@ -168,7 +174,11 @@ typedef XColor    ThotColorStruct;
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h> /* for keycode */
+#ifndef _GTK2
 #include <gdk_imlib.h>      /* for load image (jpg, gif, tiff...) */
+#else /* _GTK2 */
+#include <gdk/gdkpixbuf.h>  /* for load image (jpg, gif, tiff...) */
+#endif /* !_GTK2 */
 typedef GtkWidget     *ThotWidget;
 typedef GdkWindow     *ThotWindow;
 typedef ThotWidget     ThotButton;
@@ -177,15 +187,31 @@ typedef GdkBitmap     *ThotBitmap;
 typedef GdkGC         *ThotGC;
 typedef unsigned long  ThotColor;
 typedef XColor         ThotColorStruct;
-typedef GdkFont       *ptrfont;
-typedef Cursor         ThotCursor;
-typedef XPoint         ThotPoint;
-typedef GdkEvent      *ThotEvent;
+typedef GdkFont       *PtrFont;
+typedef GdkCursor     *ThotCursor;
+#ifndef _GL
+typedef GdkPoint       ThotPoint;
+#else /* _GL */
+typedef struct GL_point 
+{
+  float x;
+  float y;
+} ThotPoint;
+#endif /*_GL*/
+typedef GdkEvent       ThotEvent;
 typedef XKeyEvent      ThotKeyEvent;
 typedef XComposeStatus ThotComposeStatus;
 typedef XtAppContext   ThotAppContext;
 typedef XtTranslations ThotTranslations;
-typedef GdkPixmap     *ThotIcon;
+
+typedef struct 
+{
+    GdkBitmap          *mask;
+    GdkPixmap          *pixmap;
+} _Thot_icon;
+
+typedef _Thot_icon    *ThotIcon;
+typedef GdkPixmap     *ThotPixmap;
 #else /* _GTK */
 typedef Widget         ThotWidget;
 typedef Window         ThotWindow;
@@ -195,7 +221,7 @@ typedef Drawable       ThotBitmap;
 typedef GC             ThotGC;
 typedef unsigned long  ThotColor;
 typedef XColor         ThotColorStruct;
-typedef int           *ptrfont;
+typedef int           *PtrFont;
 typedef Cursor         ThotCursor;
 typedef XPoint         ThotPoint;
 typedef XEvent         ThotEvent;
