@@ -1,20 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2002.
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
- 
-
 #ifndef _APPDIALOG_H
 #define _APPDIALOG_H
 
@@ -23,9 +13,9 @@
 /* Structure of an item in the Actions Table */
 typedef struct _Action_Ctl
 {
-  char*         ActionName;    /* External name of the action           */
+  char         *ActionName;    /* External name of the action           */
   Proc 		Call_Action;   /* Address C procedure                   */
-  char*         ActionEquiv;   /* Displayed text for shortcuts          */
+  char         *ActionEquiv;   /* Displayed text for shortcuts          */
   ThotBool	ActionActive[MAX_FRAME];
 }Action_Ctl;
 
@@ -40,7 +30,6 @@ typedef struct _Item_Ctl
     struct
     {
       int	_ItemAction;	/* ID of the linked action		*/
-
     } s0;
     struct
     {
@@ -60,16 +49,16 @@ typedef struct _Menu_Ctl
   ThotBool	MenuSelect;	/* This menu is the select menu		*/
   ThotBool	MenuHelp;	/* This menu is the help menu		*/
   int		ItemsNb;	/* Number of items in the menu menu	*/
-  Item_Ctl	*ItemsList;	/* Pointer to the first item structure	*/
+  Item_Ctl     *ItemsList;	/* Pointer to the first item structure	*/
   struct _Menu_Ctl	*NextMenu;	/* Next menu		    	*/
 }Menu_Ctl;
 
 /* Structure to associate menus and specific structure schemas */
 typedef struct _SchemaMenu_Ctl
 {
-  CHAR_T*   SchemaName;                  /* Structure schema name     */
-  Menu_Ctl* SchemaMenu;	                 /* Pointer to the first menu */
-  struct    _SchemaMenu_Ctl *NextSchema; /* Next association          */
+  char      *SchemaName;                  /* Structure schema name     */
+  Menu_Ctl  *SchemaMenu;	                 /* Pointer to the first menu */
+  struct     _SchemaMenu_Ctl *NextSchema; /* Next association          */
 }SchemaMenu_Ctl;
 
 /*
@@ -85,33 +74,33 @@ typedef struct _SchemaMenu_Ctl
 #define T_stopinsert	4
 #define T_pasteclipboard	5
 #define T_chsplit	6
-#define T_chvisibility	8
-#define T_rsvisibility	9
-#define T_chzoom	10
-#define T_rszoom	11
-#define T_chselect	12
-#define T_chattr	13
-#define T_insertchar	14
-#define T_editfunc	15
-#define T_presentation	16
-#define T_presentchoice	17
-#define T_rsavedoc	18
-#define T_savedoc	19
-#define T_openview	20
-
-#define T_createdoc	21
-#define T_opendoc	22
-#define T_import	23
-#define T_confirmcreate	24
-#define T_confirmclose	25
-#define T_rconfirmclose	26
-#define T_buildpathdocbuffer	27
-#define T_insertpaste	28
-#define T_rcinsertpaste	29
-#define T_raskfornew	30
+#define T_chselect	7
+#define T_chattr	8
+#define T_insertchar	9
+#define T_editfunc	10
+#define T_confirmclose	11
+#define T_rconfirmclose	12
+#define T_insertpaste	13
+#define T_rcinsertpaste	14
+#define T_clearhistory                  15
+#define T_openhistory                   16
+#define T_addhistory                    17
+#define T_attraddhistory                18
+#define T_cancelhistory                 19
+#define T_closehistory                  20
+#define T_lock                          21
+#define T_unlock                        22
+#define T_islock                        23
+#define T_backuponfatal			24
+#define T_switchsel                     25
+#define T_checksel                      26
+#define T_resetsel                      27
+#define T_selstring                     28
+#define T_extendsel                     29
+#define T_showbox                       30
 #define T_searchtext	31
 #define T_locatesearch	32
-#define T_searchpage	33
+#define T_freesearch	33
 #define T_rchoice	34
 #define T_present	35
 #define T_presentstd	36
@@ -122,17 +111,14 @@ typedef struct _SchemaMenu_Ctl
 #define T_rattrlang	41
 #define T_rselect	42
 #define T_emptybox	43
-
-#define T_corrector	44
-#define T_rscorrector	45
-#define T_indexcopy	46
-#define T_indexnew	47
-#define T_indexverif	48
-#define T_indexschema	49
-#define T_rsindex	50
+#define T_freesavedel   44
+#define T_docmodified   45
+#define T_checkHiddenElement		46
+#define T_checkReadOnlyElement		47
+#define T_checkInsertNearElement	48
+#define T_AIupdate			49
+#define T_initevents			50
 #define T_rprint	51
-#define T_rextprint	52
-
 #define T_cancopyorcut	53
 #define T_selectsiblings 54
 #define T_selecbox	55
@@ -141,7 +127,6 @@ typedef struct _SchemaMenu_Ctl
 #define T_enter	        58
 #define T_cmdpaste	59
 #define T_deletenextchar 60
-
 #define T_strsearchconstmenu	61
 #define T_strsearchgetparams	62
 #define T_strsearchonly		63
@@ -168,48 +153,15 @@ typedef struct _SchemaMenu_Ctl
 #define T_cutpage               84
 #define T_deletepage            85
 #define T_deletepageab          86
-#define T_searchemptyelt        87
-#define T_searchemptyref        88
-#define T_searchrefto           89
-
-#define T_checkHiddenElement		90
-#define T_checkReadOnlyElement		91
-#define T_checkInsertNearElement	92
-#define T_rchangetype			93
-#define T_rsurround			94
-#define T_rchangepres			95
-#define T_initevents			96
-#define T_checktable                    97
-#define T_cleartable			98
-#define T_resizetable                   99
-#define T_firstcolumn			100
-#define T_checkcolumn			101
-#define T_checktableheight              102
-#define T_AIupdate			103
-#define T_redisplay			104
-#define T_writedocument                 105
-#define T_setwritedirectory             106
-#define T_transformintotype		108
-#define T_docmodified                   109
-#define T_switchsel                     110
-#define T_checksel                      111
-#define T_resetsel                      112
-#define T_selstring                     113
-#define T_extendsel                     114
-#define T_showbox                       115
-#define T_freesavedel                   116
-#define T_clearhistory                  117
-#define T_openhistory                   118
-#define T_addhistory                    119
-#define T_attraddhistory                120
-#define T_cancelhistory                 121
-#define T_closehistory                  122
-
-#define T_lock                          123
-#define T_unlock                        124
-#define T_islock                        125
-#define T_backuponfatal			126
-#define MAX_LOCAL_ACTIONS               127
+#define T_redisplay		87
+#define T_writedocument         88
+#define T_checktable                    89
+#define T_cleartable			90
+#define T_resizetable                   91
+#define T_firstcolumn			92
+#define T_checkcolumn			93
+#define T_checktableheight              94
+#define MAX_LOCAL_ACTIONS               95
 
 /*
  Predefined idexes of Thot actions which could be linked to shortcuts even
@@ -242,8 +194,16 @@ typedef struct _SchemaMenu_Ctl
 #define CMD_LineDown		25
 #define CMD_ScrollLeft		26
 #define CMD_ScrollRight		27
-#define MAX_INTERNAL_CMD	28
+#define CMD_LineBreak           28
+#define CMD_PreviousWord        29
+#define CMD_NextWord            30
+#define CMD_PreviousSelWord     31
+#define CMD_NextSelWord         32
+#define CMD_SelBeginningOfLine  33
+#define CMD_SelEndOfLine	34
+#define MAX_INTERNAL_CMD	35
 
-#define CST_EquivLineUp TEXT("C Up")
-#define CST_EquivScrollLeft TEXT("C Left")
+#define CST_EquivLineUp "C Up"
+#define CST_EquivScrollLeft "C Left"
 #endif
+
