@@ -1,10 +1,9 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
- 
 
 /*
  * Type declarations for the formatter
@@ -20,7 +19,34 @@
 #include "typecorr.h"
 #include "typetra.h"
 
-typedef UCHAR_T RuleSet[32];
+#ifdef _I18N_
+typedef struct _FontSet      *SpecFont;
+typedef struct _FontSet {
+  SpecFont      NextFontSet;
+  ThotBool      specificFont; /* TRYE when the specific font is used */
+  int           FontFamily;
+  int           FontHighlight;
+  int           FontSize;
+  unsigned int  FontMask;
+  PtrFont       FontIso_1;
+  PtrFont       FontIso_2;
+  PtrFont       FontIso_3;
+  PtrFont       FontIso_4;
+  PtrFont       FontIso_5;
+  PtrFont       FontIso_6;
+  PtrFont       FontIso_7;
+  PtrFont       FontIso_8;
+  PtrFont       FontIso_9;
+  PtrFont       FontIso_13;
+  PtrFont       FontIso_15;
+  PtrFont       FontSymbol;
+  PtrFont       FontUnicode;
+} FontSet;
+#else /* _I18N_ */
+typedef PtrFont               SpecFont;
+#endif /* _I18N_ */
+
+typedef unsigned char RuleSet[32];
 
 /* List of pointers used */
 typedef struct _Box *PtrBox;
@@ -30,11 +56,13 @@ typedef struct _Line *PtrLine;
 /* Type of a described box */
 typedef enum
 {
-  BoComplete, 
-  BoSplit, 
-  BoPiece, 
-  BoBlock, 
-  BoPicture, 
+  BoComplete,
+  BoSplit,
+  BoPiece,
+  BoMulScript,
+  BoScript,
+  BoBlock,
+  BoPicture,
   BoGhost,
   BoDotted,
   BoTable,
@@ -152,10 +180,9 @@ typedef struct _TabUpdate
 
 typedef struct _Box
 {
-  PtrAbstractBox  BxAbstractBox;/* Pointer on the associated abstract box */
-  PtrBox          BxPrevious;	        /* Previous displayable box */
-  PtrBox          BxNext;	        /* Next displayable box */
-  PtrBox          BxNextBackground;	/* Next background box to display */
+  PtrAbstractBox  BxAbstractBox;/* Pointer to the associated abstract box */
+  PtrBox          BxPrevious;	/* Previous displayable box */
+  PtrBox          BxNext;	/* Next displayable box */
   int	          BxNChars;	/* Total number of characters in the box */
   int             BxIndChar;	/* 0 or position of the split box */
   int             BxXOrg;	/* X origin from the root */
@@ -189,7 +216,7 @@ typedef struct _Box
   PtrPosRelations BxPosRelations;	/* Dependencies on positions */
   PtrDimRelations BxWidthRelations;	/* Dependencies in width */
   PtrDimRelations BxHeightRelations;	/* Dependencies in height */
-  ptrfont         BxFont;	        /* Font bound to the box */
+  SpecFont        BxFont;	        /* Font bound to the box */
   PtrBox          BxMoved;	        /* Linking of moved boxes */
   PtrBox          BxHorizInc;	        /* Box linking to the enclosing one */
   PtrBox          BxVertInc;	        /* Box linking to the enclosing one */
@@ -208,7 +235,9 @@ typedef struct _Box
   ThotBool	  BxNew;	        /* Newly displayed box */
   ThotBool	  BxContentHeight;	/* Real height is the content height */
   ThotBool	  BxContentWidth;	/* Real width is the content width */
-  ThotBool        BxShadow;		/* Characters are showed as a set of '*' */
+  ThotBool        BxShadow;		/* Characters are showed as '*' */
+  ThotBool        BxFill;	        /* The box has borders or background */
+  ThotBool        BxDisplay;	        /* The box has borders or background */
   int		  BxRuleHeigth;         /* Content height or minimum */
   int		  BxRuleWidth;	        /* Content width or minimum */
   PtrTextBuffer   BxBuffer;	        /* Pointer on the buffer list */
@@ -222,13 +251,18 @@ typedef struct _Box
       int	 _BxNPixels_;	/* Number of pixels to share out */
       int	 _BxSpaceWidth_;/* >0 of the box is justified */
       int	 _BxFirstChar_;	/* First character in buffer */
+      char       _BxScript_;    /* Script of the current text */
     } s0;
-    struct /* BoPicture */
+    struct /* BoPicture - used for pictures, polylines and paths */
     {
-      int	 *_BxPictInfo_;  /* Image pointer in memory
+      union
+        {
+        int	 *_BxPictInfo_;  /* Image pointer in memory
 				    List of control points for a spline
 				    + twisting ratio of the control
 			            points */
+	PtrPathSeg _BxFirstPathSeg_; /* pointer to the first path segment */
+        } u;
       float 	 _BxXRatio_;
       float 	 _BxYRation_;
     } s1;
@@ -238,8 +272,8 @@ typedef struct _Box
       PtrLine 	 _BxLastLine_;	/* Last line */
       int        _BxMaxWidth_;  /* Width without line wrapping */
       int        _BxMinWidth_;  /* Mininmum width */
-      short      _BxCycles_;    /* count reformatting cycles */
-      short      _BxPacking_;   /* Packing */
+      int        _BxCycles_;    /* count reformatting cycles */
+      int        _BxPacking_;   /* Packing */
     } s2;
     struct /* BoTable BoColumn BoRow */
     {
@@ -247,8 +281,8 @@ typedef struct _Box
       PtrTabRelations	_BxRows_;	/* list of rows within a table */
       int        	_BxMaxWidth_;   /* Width without line wrapping */
       int        	_BxMinWidth_;   /* Mininmum width */
-      short             _BxCycles_;     /* count reformatting cycles */
-      short             _BxPacking_;     /* Packing */
+      int               _BxCycles_;     /* count reformatting cycles */
+      int               _BxPacking_;     /* Packing */
       PtrTabSpan        _BxSpans_;	/* list of spanned cells */
     } s3;
     struct /* BoComplete for Graphics = 'C' */
@@ -264,7 +298,9 @@ typedef struct _Box
 #define BxNPixels u.s0._BxNPixels_
 #define BxSpaceWidth u.s0._BxSpaceWidth_
 #define BxFirstChar u.s0._BxFirstChar_
-#define BxPictInfo u.s1._BxPictInfo_
+#define BxScript u.s0._BxScript_
+#define BxPictInfo u.s1.u._BxPictInfo_
+#define BxFirstPathSeg u.s1.u._BxFirstPathSeg_
 #define BxXRatio u.s1._BxXRatio_
 #define BxYRatio u.s1._BxYRation_
 #define BxFirstLine u.s2._BxFirstLine_
@@ -379,6 +415,7 @@ typedef struct _AbstractBox
   int             AbIndent;	/* Indentation for line breaking */
   int             AbDepth;	/* Abstract box display plane */
   int	          AbVisibility; /* Abstract box visibility degree */
+  int             AbFont;	/* Font family: index in the family table */
   int             AbFontStyle;	/* Font style: roman, italic, oblique */
   int             AbFontWeight;	/* Font weight: normal, bold */
   int             AbSize;	/* Character logical/real size */
@@ -410,8 +447,10 @@ typedef struct _AbstractBox
   int             AbBottomBorder;  /* Bottom Border */
   int             AbLeftBorder;    /* Left Border */
 
-  char            AbFont;	      /* Characteristics of the font used */
+  char            AbDirection;        /* Writing direction */
+  char            AbUnicodeBidi;      /* Unicode Bidi */
   char            AbLineStyle;        /* Line style */
+  char            AbDisplay;          /* Display mode */
   BAlignment      AbAdjust;	      /* Alignment of lines in the box */
   TypeUnit        AbSizeUnit;         /* Unit for the size */
   TypeUnit        AbIndentUnit;       /* Indentation unit */
@@ -431,35 +470,36 @@ typedef struct _AbstractBox
   TypeUnit        AbBottomBorderUnit; /* Bottom Border Unit */
   TypeUnit        AbLeftBorderUnit;   /* Left Border Unit */
 
-  ThotBool        AbJustify;	/* Lines are justified */
-  ThotBool        AbAcceptLineBreak;    /* Can be split in lines */
-  ThotBool        AbAcceptPageBreak;    /* Can be split by page breaks */
-  ThotBool        AbHyphenate;	/* Contents can be hyphenated */
-  ThotBool        AbOnPageBreak;	/* The box crosses the page limit */
-  ThotBool        AbAfterPageBreak;	/* Abstract box beyond the page limit*/
-  ThotBool        AbNotInLine;	/* The abstract box is not part of the line */
-  ThotBool        AbHorizEnclosing;	/* True: horizontally enclosed box */
-  ThotBool        AbVertEnclosing;	/* True: vertically enclosed box */
-  ThotBool        AbCanBeModified;	/* Modification is allowed */
-  ThotBool        AbSelected;		/* The abstract box is selected */
-  ThotBool        AbPresentationBox;	/* A presentation box */
-  ThotBool        AbRepeatedPresBox;	/* A repeated presentation box */
-  ThotBool        AbSensitive;	/* True: the abstract box is active */
-  ThotBool        AbReadOnly;	/* Abstract box is read only */
+  ThotBool        AbBuildAll;         /* All descendant abstract boxes must
+					 always be built */
+  ThotBool        AbAcceptLineBreak;  /* Can be split in lines */
+  ThotBool        AbAcceptPageBreak;  /* Can be split by page breaks */
+  ThotBool        AbHyphenate;	      /* Contents can be hyphenated */
+  ThotBool        AbOnPageBreak;      /* The box crosses the page limit */
+  ThotBool        AbAfterPageBreak;   /* Abstract box beyond the page limit*/
+  ThotBool        AbNotInLine;	 /* The abstract box is not part of the line */
+  ThotBool        AbHorizEnclosing;   /* True: horizontally enclosed box */
+  ThotBool        AbVertEnclosing;    /* True: vertically enclosed box */
+  ThotBool        AbCanBeModified;    /* Modification is allowed */
+  ThotBool        AbSelected;	      /* The abstract box is selected */
+  ThotBool        AbPresentationBox;  /* A presentation box */
+  ThotBool        AbRepeatedPresBox;  /* A repeated presentation box */
+  ThotBool        AbSensitive;	      /* True: the abstract box is active */
+  ThotBool        AbReadOnly;	      /* Abstract box is read only */
   /* Indications of modification for the abstract box */
-  ThotBool        AbNew;	/* Newly created abstract box */
-  ThotBool        AbDead;       /* Abstract box to be destroyed */
-  ThotBool        AbWidthChange;	/* Change the horizontal dimension */
-  ThotBool        AbHeightChange;	/* Change the vertical dimension */
-  ThotBool        AbHorizPosChange;	/* Change the horizontal position */
-  ThotBool        AbVertPosChange;	/* Change the vertical axis */
-  ThotBool        AbHorizRefChange;	/* Change the horizontal axis */
-  ThotBool        AbVertRefChange;	/* Change the vertical positioning */
-  ThotBool        AbSizeChange;	        /* Change the character size */
-  ThotBool        AbAspectChange;       /* Change the graphical aspect: plane,
-					   color, pattern, line style */
-  ThotBool        AbMBPChange;	        /* Change margins, borders, paddings */
-  ThotBool        AbChange;	        /* Change of another type */
+  ThotBool        AbNew;	      /* Newly created abstract box */
+  ThotBool        AbDead;             /* Abstract box to be destroyed */
+  ThotBool        AbWidthChange;      /* Change the horizontal dimension */
+  ThotBool        AbHeightChange;     /* Change the vertical dimension */
+  ThotBool        AbHorizPosChange;   /* Change the horizontal position */
+  ThotBool        AbVertPosChange;    /* Change the vertical axis */
+  ThotBool        AbHorizRefChange;   /* Change the horizontal axis */
+  ThotBool        AbVertRefChange;    /* Change the vertical positioning */
+  ThotBool        AbSizeChange;	      /* Change the character size */
+  ThotBool        AbAspectChange;     /* Change the graphical aspect: plane,
+				         color, pattern, line style */
+  ThotBool        AbMBPChange;	      /* Change margins, borders, paddings */
+  ThotBool        AbChange;	      /* Change of another type */
   LeafType        AbLeafType;
   union
   {
@@ -474,13 +514,13 @@ typedef struct _AbstractBox
     } s0;
     struct /* AbLeafType = LtText */
     {
-      PtrTextBuffer  _AbText_;      /* On first text buffer */
-      Language  _AbLanguage_;  /* Language used */
+      PtrTextBuffer _AbText_;     /* On first text buffer */
+      Language      _AbLang_;     /* Language or script used */
     } s1;
     struct /* AbLeafType = LtGraphics or LtSymbol */
     {
       char     _AbShape_;         /* Drawing code */
-      char     _AbGraphAlphabet_; /* Alphabet used*/
+      char     _AbGraphScript_; /* Script used*/
       char     _AbRealShape_;     /* Effective drawing code */
       int      _AbRx_;            /* Horizontal radius */
       int      _AbRy_;            /* Vertical radius */     
@@ -496,6 +536,10 @@ typedef struct _AbstractBox
       PtrTextBuffer _AbPolyLineBuffer_; /* First buffer */
       char          _AbPolyLineShape_;  /* Drawing type */
     } s4;
+    struct /* AbLeafType = LtPath  */
+    {
+      PtrPathSeg    _AbFirstPathSeg_;   /* First path segment */
+    } s5;
   } u;
 } AbstractBox;
 
@@ -505,9 +549,9 @@ typedef struct _AbstractBox
 #define AbTruncatedHead u.s0._AbTruncatedHead_
 #define AbTruncatedTail u.s0._AbTruncatedTail_
 #define AbText u.s1._AbText_
-#define AbLanguage u.s1._AbLanguage_
+#define AbLang u.s1._AbLang_
 #define AbShape u.s2._AbShape_
-#define AbGraphAlphabet u.s2._AbGraphAlphabet_
+#define AbGraphScript u.s2._AbGraphScript_
 #define AbRealShape u.s2._AbRealShape_
 #define AbRx u.s2._AbRx_
 #define AbRy u.s2._AbRy_
@@ -516,6 +560,7 @@ typedef struct _AbstractBox
 #define AbPictInfo u.s3._AbPictInfo_
 #define AbPolyLineBuffer u.s4._AbPolyLineBuffer_
 #define AbPolyLineShape u.s4._AbPolyLineShape_
+#define AbFirstPathSeg u.s5._AbFirstPathSeg_
 
 typedef struct _ViewFrame
 {
@@ -540,11 +585,9 @@ typedef struct _ViewFrame
 typedef struct _VueDeDoc
 {
   int             VdView;
-  int             VdAssocNum;
   Name            VdViewName;
   PtrSSchema      VdSSchema;
   ThotBool        VdOpen;
-  ThotBool        VdAssoc;
   ThotBool	  VdExist;
   ThotBool        VdNature;
   ThotBool        VdPaginated;
@@ -570,6 +613,6 @@ typedef SearchContext *PtrSearchContext;
 /* Declaration for dialogue */
 
 /* List of the pointers used */
-typedef CHAR_T    FontName[30];
+typedef char    FontName[30];
 
 #endif
