@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -15,7 +15,6 @@
  *
  */
 
-
 /* DEFINITIONS
 
 	A view number is the rank of a view in the table PresentSchema.PsView.
@@ -79,9 +78,9 @@ typedef struct _PosRule
   int		PoDistance;	/* distance between those two sides */
   Level		PoRelation;	/* relative level of the reference element
 				   in the internal representation */
-  ThotBool	PoNotRel;	/* if 'false', PoTypeRefElem or PoRefPresBox
-				   indicates the type of the reference element,
-				   if 'true', the exclusive type. */
+  ThotBool	PoNotRel;	/* if 'false', PoRefIdent indicates the type of
+				   the reference element, if 'true', the
+				   excluded type. */
   ThotBool	PoUserSpecified;/* the distance may be chosen by the user
 				   during the creation */
   RefKind	PoRefKind;	/* the reference is the box of an element,
@@ -194,7 +193,7 @@ typedef struct _PresBoxTable
 /* Alignment of the lines in an abstract box */
 typedef enum
 {
-  AlignLeft, AlignRight, AlignCenter, AlignLeftDots, AlignJustify
+  AlignLeft, AlignRight, AlignCenter, AlignJustify, AlignLeftDots
 } BAlignment;	/* AlignLeftDots = aligned to the left, the last
 		   line is filled with dots */
 
@@ -203,19 +202,20 @@ typedef enum
 {
   /* the order determines the order of the rules in the presentation schema */
   /* Don't forget to change constant MAX_TRANSL_PRULE when adding rules */
-  /* Don't forget to update PR values in the file presentation.h */
-  PtVisibility, PtFunction, PtVertRef, PtHorizRef, PtHeight, PtWidth, 
-  PtVertPos, PtHorizPos,
+  /* Don't forget to update PRxxx values in the file presentation.h */
+  PtVisibility, PtFunction,
+  PtVertOverflow, PtHorizOverflow, 
+  PtVertRef, PtHorizRef, PtHeight, PtWidth, PtVertPos, PtHorizPos,
   PtMarginTop, PtMarginRight, PtMarginBottom, PtMarginLeft,
   PtPaddingTop, PtPaddingRight, PtPaddingBottom, PtPaddingLeft,
   PtBorderTopWidth, PtBorderRightWidth, PtBorderBottomWidth, PtBorderLeftWidth,
   PtBorderTopColor, PtBorderRightColor, PtBorderBottomColor, PtBorderLeftColor,
   PtBorderTopStyle, PtBorderRightStyle, PtBorderBottomStyle, PtBorderLeftStyle,
   PtSize, PtStyle, PtWeight, PtFont, PtUnderline, PtThickness,
-  PtIndent, PtLineSpacing, PtDepth, PtAdjust,
+  PtIndent, PtLineSpacing, PtDepth, PtAdjust, PtDirection, PtUnicodeBidi,
   PtLineStyle, PtLineWeight, PtFillPattern, PtBackground, PtForeground,
-  PtHyphenate, PtVertOverflow, PtHorizOverflow, PtPageBreak, PtLineBreak,
-  PtGather, PtXRadius, PtYRadius,
+  PtHyphenate, PtPageBreak, PtLineBreak, PtGather,
+  PtXRadius, PtYRadius, PtDisplay,
   /* the three following types must be the last ones */
   PtBreak1, PtBreak2, PtPictInfo
 } PRuleType;
@@ -311,7 +311,7 @@ typedef enum
   PcInternalRef, PcCopyRef, PcAnyAttributes, PcFirstAttr, PcLastAttr,
   PcUserPage, PcStartPage, PcComputedPage, PcEmpty, PcRoot, PcEven,
   PcOdd, PcOne, PcInterval, PcWithin, PcElemType, PcAttribute,
-  PcNoCondition, PcDefaultCond
+  PcInheritAttribute, PcNoCondition, PcDefaultCond
 } PresCondition;
 
 /* To interpret the field CoRelation */
@@ -320,6 +320,12 @@ typedef enum
   CondGreater, CondLess, CondEquals
 } ArithRel;
 
+/* To interpret the field CoMatch */
+typedef enum
+{
+  CoMatch, CoSubstring, CoWord
+} CondMatch;
+
 typedef struct _Condition *PtrCondition;
 
 /* A presentation rule application condition */
@@ -356,13 +362,28 @@ typedef struct _Condition
       Name      _CoSSchemaName_;   /* name of the schema where the ancestor is
 				      defined if CoTypeAncestor == 0 */
     } s1;
-    struct			   /* CoCondition = PcElemType or PcAttribute*/
+    struct			   /* CoCondition = PcElemType */
     {
-      int	_CoTypeElAttr_;	   /* PcElemType: type of the element to which
-				      the attribute is attached.
-				      PcAttribute: attribute carried by the
-				      element */
+      int	_CoTypeElem_;	   /* type of the element to which the
+				      attribute is attached */
     } s2;
+    struct			   /* CoCondition = PcAttribute */
+    {
+      int	_CoTypeAttr_;	   /* attribute carried by the element */
+      ThotBool  _CoTestAttrValue_; /* the attribute value must be tested */
+      union
+      {
+	struct        /* numerical or enumerated attribute */
+	{
+	  int	_CoAttrValue_;     /* the value that satisfies the condition */
+	} s0;
+	struct        /* text attribute */
+	{
+	  Name      _CoAttrTextValue_; /* the value that satisfies the condition */
+	  CondMatch _CoTextMatch_;     /* matching condition: substring, word, ... */
+	} s1;
+      } u;
+    } s3;
   } u;
 } Condition;
 
@@ -376,7 +397,12 @@ typedef struct _Condition
 #define CoAncestorRel u.s1._CoAncestorRel_
 #define CoAncestorName u.s1._CoAncestorName_
 #define CoSSchemaName u.s1._CoSSchemaName_
-#define CoTypeElAttr u.s2._CoTypeElAttr_
+#define CoTypeElem u.s2._CoTypeElem_
+#define CoTypeAttr u.s3._CoTypeAttr_
+#define CoTestAttrValue u.s3._CoTestAttrValue_
+#define CoAttrValue u.s3.u.s0._CoAttrValue_
+#define CoAttrTextValue u.s3.u.s1._CoAttrTextValue_
+#define CoTextMatch u.s3.u.s1._CoTextMatch_
 
 /* The presentation rules relative to an object are linked by means of
    the pointer PrNextPRule. This way the presentation rules of a type
@@ -397,9 +423,13 @@ typedef struct _PresRule
 				   number of the attribute to which the
 				   rule corresponds, 0 if the rule is not
 				   derived from an attribute rule */
-  int           PrLevel;        /* CSS priority level for generic rules
-				   For specific rules: > 0 when the rule
-				   translates a CSS style rule */
+  int           PrSpecificity;  /* Specificity of the corresponding CSS rule.
+				   0 if the rule belongs to the default
+				   P Schema */
+  ThotBool      PrImportant;    /* the corresponding CSS rule has flag
+                                   !important */
+  ThotBool      PrDuplicate;    /* duplicate of a conditional rule for an
+				   inherited attribute */
   PtrSSchema    PrSpecifAttrSSchema; /* pointer on the structure schema
                                         defining the attribute PrSpecifAttr */
   PresMode	PrPresMode;	/* computing mode of the value */
@@ -453,7 +483,8 @@ typedef struct _PresRule
 				        -1 means "same color as foreground" */
 	}  s0;
 	struct	/* PRuleType = PtFont, PtStyle, PtWeight, PtUnderline,
-	        	       PtThickness, PtLineStyle,
+	        	       PtThickness, PtDirection, PtUnicodeBidi,
+                               PtLineStyle, PtDisplay,
 	                       PtBorderTopStyle, PtBorderRightStyle,
                                PtBorderBottomStyle, PtBorderLeftStyle */
 	{
@@ -585,7 +616,7 @@ typedef struct _Counter
 typedef struct _PresConstant
 {
   BasicType PdType;                       /* type of the constant */
-  char      PdAlphabet;                   /* alphabet of the constant */
+  char      PdScript;                     /* script of the constant */
   char      PdString[MAX_PRES_CONST_LEN]; /* constant presentation string,
 					     terminated by a NUL character */
 } PresConstant;
@@ -624,34 +655,36 @@ typedef struct _AttributePres
 			   apply, 0 if the rules apply whatever the element
 			   type is */
   struct _AttributePres *ApNextAttrPres; /* the packet of presentation rules
-			   for the next element */
+					    for the next element */
   union
   {
-    struct
+    struct        /* numerical attribute */
     {
       int	  _ApNCases_; /* number of application cases for the
-			   presentation rules */
+				 presentation rules */
       NumAttrCase _ApCase_[MAX_PRES_ATTR_CASE]; /* the cases of application
-			   of the presentation rules */
+						   of the presentation rules */
     } s0;
-    struct
+    struct        /* reference attribute */
     {
       PtrPRule    _ApRefFirstPRule_; /* first rule in the string of rules
-			   to apply for the attribute */
+					to apply for the attribute */
     } s1;
-    struct
+    struct        /* text attribute */
     {
       Name  	  _ApString_;	/* the value triggering the application of the
-			   presentation rules */
-      PtrPRule    _ApTextFirstPRule_;    /* first rule in the string of rules
-			   to apply for this value */
+				   presentation rules */
+      CondMatch   _ApMatch_;    /* matching condition: substring, word, ... */
+      PtrPRule    _ApTextFirstPRule_; /* first rule in the string of rules
+					 to apply for this value */
     } s2;
-    struct
+    struct        /* enumerated attribute */
     {
-      PtrPRule    _ApEnumFirstPRule_[MAX_ATTR_VAL + 1]; /* for each value of
-			   the attribute, in the order of the table
-			   AttrEnumValue, address of the first presentation
-			   rule associated with this value */
+      PtrPRule    _ApEnumFirstPRule_[MAX_ATTR_VAL + 1]; /* for each atribute value,
+							   in the order of the table
+							   AttrEnumValue, ppoints the
+							   first presentation rule
+							   associated with this value */
     } s3;
   } u;
 } AttributePres;
@@ -660,6 +693,7 @@ typedef struct _AttributePres
 #define ApCase u.s0._ApCase_
 #define ApRefFirstPRule u.s1._ApRefFirstPRule_
 #define ApString u.s2._ApString_
+#define ApMatch u.s2._ApMatch_
 #define ApTextFirstPRule u.s2._ApTextFirstPRule_
 #define ApEnumFirstPRule u.s3._ApEnumFirstPRule_
 
@@ -728,6 +762,12 @@ typedef struct _NumberTable
   int           Num[1];
 } NumberTable;
 
+/* origin of a style sheet */
+typedef enum
+{
+  Agent, User, Author
+} StyleSheetOrigin;
+
 typedef struct _PresentSchema *PtrPSchema;
 
 /* a presentation schema loaded in memory */
@@ -738,6 +778,9 @@ typedef struct _PresentSchema
   Name		PsPresentName;		/* name of this presentation schema */
   int	 	PsStructCode;    	/* code identifying the version of this
 					   structure schema */
+  StyleSheetOrigin PsOrigin;            /* origin of this structure schema */
+  void*         PsSubtree;              /* parent of the subtree to which the
+					   presentation rules apply */
   int		PsNViews;	    	/* number of views */
   ViewTable     PsView;    		/* definition of the views */
   PtrHostView   PsHostViewList[MAX_VIEW]; /* for each view defined in ViewTable
