@@ -1,18 +1,9 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2002.
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
-
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
  
 /*
  * Change specific presentation
@@ -58,6 +49,7 @@
 #include "font_f.h"
 #include "memory_f.h"
 #include "presrules_f.h"
+#include "schemas_f.h"
 #include "structcreation_f.h"
 #include "structmodif_f.h"
 #include "structselect_f.h"
@@ -69,99 +61,96 @@
 /*----------------------------------------------------------------------
   ApplyRuleSubTree
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         ApplyRuleSubTree (PtrElement pE, PRuleType ruleType, PtrDocument pDoc, PtrPRule * pPRule, int view, ThotBool display)
-#else  /* __STDC__ */
-static void         ApplyRuleSubTree (pE, ruleType, pDoc, pPRule, view, display)
-PtrElement          pE;
-PRuleType           ruleType;
-PtrDocument         pDoc;
-PtrPRule           *pPRule;
-int                 view;
-ThotBool            display;
-
-#endif /* __STDC__ */
+static void   ApplyRuleSubTree (PtrElement pE, PRuleType ruleType,
+				PtrDocument pDoc, PtrPRule * pPRule,
+				int view, ThotBool display)
 {
-   PtrAbstractBox      pAbb, pAbbF;
-   PtrPSchema          pSPR;
-   PtrAttribute        pAttr;
+  PtrAbstractBox      pAbb, pAbbF;
+  PtrPSchema          pSPR;
+  PtrAttribute        pAttr;
+  ThotBool            complete;
 
-   if (pE->ElTerminal)
-      pE = NULL;
-   else
-      /* on passe au premier fils */
-      pE = pE->ElFirstChild;
-   /* traite tous les fils */
-   while (pE)
-     {
-	/* 1er pave de l'element dans la view */
-	pAbb = pE->ElAbstractBox[view - 1];
-	if (pAbb)
-	   if (pAbb->AbDead)
-	      pAbb = NULL;	/* on ne traite pas les paves morts */
-	if (pAbb == NULL)
-	   /* cet element n'a pas de pave, mais ses descendants en */
-	   /* ont peut etre... */
-	   ApplyRuleSubTree (pE, ruleType, pDoc, pPRule, view, display);
-	else
-	  {
-	     /* il y a un element descendant dont les paves peuvent heriter
-		de pAb. On parcourt ses paves dans la vue */
-	     do
-	       {
-		  *pPRule = SearchRulepAb (pDoc, pAbb, &pSPR, ruleType, FnAny,
-					   TRUE, &pAttr);
-		  if (*pPRule)
+  if (pE->ElTerminal)
+    pE = NULL;
+  else
+    /* on passe au premier fils */
+    pE = pE->ElFirstChild;
+  /* traite tous les fils */
+  while (pE)
+    {
+      /* 1er pave de l'element dans la view */
+      pAbb = pE->ElAbstractBox[view - 1];
+      if (pAbb == NULL)
+	/* no abstract box */
+	if (ruleType == PtVisibility)
+	  /* it's a visibility rule. Try to create the abstract box */
+	  pAbb = AbsBoxesCreate (pE, pDoc, view, True, True, &complete);
+      if (pAbb)
+	if (pAbb->AbDead)
+	  pAbb = NULL;	/* on ne traite pas les paves morts */
+      if (pAbb == NULL)
+	/* cet element n'a pas de pave, mais ses descendants en */
+	/* ont peut etre... */
+	ApplyRuleSubTree (pE, ruleType, pDoc, pPRule, view, display);
+      else
+	{
+	  /* il y a un element descendant dont les paves peuvent heriter
+	     de pAb. On parcourt ses paves dans la vue */
+	  do
+	    {
+	      *pPRule = SearchRulepAb (pDoc, pAbb, &pSPR, ruleType, FnAny,
+				       TRUE, &pAttr);
+	      if (*pPRule)
+		{
+		  if ((*pPRule)->PrPresMode == PresInherit &&
+		      ((*pPRule)->PrInheritMode == InheritParent ||
+		       (*pPRule)->PrInheritMode == InheritGrandFather))
+		    /* la regle de ce pave herite de l'ascendant, */
+		    /* on applique la regle */
+		    if (ApplyRule (*pPRule, pSPR, pAbb, pDoc, pAttr))
+		      {
+			SetChange (pAbb, ruleType);
+			if (display)
+			  RedispAbsBox (pAbb, pDoc);
+			if (!pAbb->AbPresentationBox)
+			  ApplyInherit (ruleType, pAbb, pDoc, display);
+		      }
+		}
+	      if (!pAbb->AbPresentationBox)
+		/* c'est le pave principal de l'element, on traite */
+		/* les paves crees par l'element au niveau inferieur */
+		{
+		  pAbbF = pAbb->AbFirstEnclosed;
+		  while (pAbbF)
 		    {
-		       if ((*pPRule)->PrPresMode == PresInherit &&
-			   ((*pPRule)->PrInheritMode == InheritParent ||
-			    (*pPRule)->PrInheritMode == InheritGrandFather))
-			  /* la regle de ce pave herite de l'ascendant, */
-			  /* on applique la regle */
-			  if (ApplyRule (*pPRule, pSPR, pAbb, pDoc, pAttr))
+		      if (pAbbF->AbElement == pE)
+			{
+			  *pPRule = SearchRulepAb (pDoc, pAbbF, &pSPR,ruleType,
+						   FnAny, TRUE, &pAttr);
+			  if (*pPRule)
 			    {
-			       SetChange (pAbb, ruleType);
-			       if (display)
-				 RedispAbsBox (pAbb, pDoc);
-			       if (!pAbb->AbPresentationBox)
-				 ApplyInherit (ruleType, pAbb, pDoc, display);
+			      if ((*pPRule)->PrPresMode == PresInherit
+				  && (*pPRule)->PrInheritMode == InheritParent)
+				if (ApplyRule (*pPRule, pSPR, pAbbF,
+					       pDoc, pAttr))
+				  {
+				    SetChange (pAbbF, ruleType);
+				    if (display)
+				      RedispAbsBox (pAbbF, pDoc);
+				  }
 			    }
+			}
+		      pAbbF = pAbbF->AbNext;
 		    }
-		  if (!pAbb->AbPresentationBox)
-		     /* c'est le pave principal de l'element, on traite */
-		     /* les paves crees par l'element au niveau inferieur */
-		    {
-		       pAbbF = pAbb->AbFirstEnclosed;
-		       while (pAbbF)
-			 {
-			    if (pAbbF->AbElement == pE)
-			      {
-				 *pPRule = SearchRulepAb (pDoc, pAbbF, &pSPR,
-						ruleType, FnAny, TRUE, &pAttr);
-				 if (*pPRule)
-				   {
-				      if ((*pPRule)->PrPresMode == PresInherit
-					  && (*pPRule)->PrInheritMode == InheritParent)
-					 if (ApplyRule (*pPRule, pSPR, pAbbF,
-							pDoc, pAttr))
-					   {
-					     SetChange (pAbbF, ruleType);
-					     if (display)
-					       RedispAbsBox (pAbbF, pDoc);
-					   }
-				   }
-			      }
-			    pAbbF = pAbbF->AbNext;
-			 }
-		    }
-		  pAbb = pAbb->AbNext;	/* passe au pave suivant */
-	       }
-	       /* on arrete s'il n'y a pas de pave suivant ou si le pave
-		  suivant n'appartient pas a l'element */
-	     while (pAbb && pAbb->AbElement == pE);
-	  }
-	pE = pE->ElNext;	/* on traite l'element suivant */
-     }
+		}
+	      pAbb = pAbb->AbNext;	/* passe au pave suivant */
+	    }
+	  /* on arrete s'il n'y a pas de pave suivant ou si le pave
+	     suivant n'appartient pas a l'element */
+	  while (pAbb && pAbb->AbElement == pE);
+	}
+      pE = pE->ElNext;	/* on traite l'element suivant */
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -170,18 +159,7 @@ ThotBool            display;
    		heritent de cette regle et si oui leur applique		
    		l'heritage et on les r'eaffiche si display est TRUE
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 void         ApplyInherit (PRuleType ruleType, PtrAbstractBox pAb, PtrDocument pDoc, ThotBool display)
-
-#else  /* __STDC__ */
-void         ApplyInherit (ruleType, pAb, pDoc, display)
-PRuleType           ruleType;
-PtrAbstractBox      pAb;
-PtrDocument         pDoc;
-ThotBool            display;
-
-#endif /* __STDC__ */
-
 {
    PtrElement          pEl;
    int                 view;
@@ -358,13 +336,7 @@ ThotBool            display;
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 int          NumTypePRuleAPI (PtrPRule pRule)
-#else  /* __STDC__ */
-int          NumTypePRuleAPI (pRule)
-PRuleType    pRule;
-
-#endif /* __STDC__ */
 {
    switch (pRule->PrType)
 	 {
@@ -455,6 +427,9 @@ PRuleType    pRule;
 	    case PtBorderLeftStyle:
                return PRBorderLeftStyle;
                break;
+	    case PtDisplay:
+	       return PRDisplay;
+	       break;
 	    case PtSize:
 	       return PRSize;
 	       break;
@@ -485,8 +460,11 @@ PRuleType    pRule;
 	    case PtAdjust:
 	       return PRAdjust;
 	       break;
-	    case PtJustify:
-	       return PRJustify;
+	    case PtDirection:
+	       return PRDirection;
+	       break;
+	    case PtUnicodeBidi:
+	       return PRUnicodeBidi;
 	       break;
 	    case PtLineStyle:
 	       return PRLineStyle;
@@ -530,14 +508,7 @@ PRuleType    pRule;
 /*----------------------------------------------------------------------
    	RuleSetIn teste l'appartenance d'un element a` un ensemble.	
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-int                 RuleSetIn (register int num, RuleSet RuleS1)
-#else  /* __STDC__ */
-int                 RuleSetIn (num, RuleS1)
-register int        num;
-RuleSet             RuleS1;
-
-#endif /* __STDC__ */
+int RuleSetIn (register int num, RuleSet RuleS1)
 {
    if (num >= 0 && num < SETSIZE * 8)
       return RuleS1[num >> 3] & (1 << (num & 7));
@@ -549,14 +520,7 @@ RuleSet             RuleS1;
 /*----------------------------------------------------------------------
    	RuleSetPut ajoute un element a` un ensemble.			
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                RuleSetPut (RuleSet RuleS1, int num)
-#else  /* __STDC__ */
-void                RuleSetPut (RuleS1, num)
-RuleSet             RuleS1;
-int                 num;
-
-#endif /* __STDC__ */
+void RuleSetPut (RuleSet RuleS1, int num)
 {
    if (num >= 0 && num < SETSIZE * 8)
       RuleS1[num >> 3] |= (1 << (num & 7));
@@ -566,15 +530,9 @@ int                 num;
 /*----------------------------------------------------------------------
    	RuleSetClr met a` 0 l'ensemble RuleS1.				
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                RuleSetClr (RuleSet RuleS1)
-#else  /* __STDC__ */
-void                RuleSetClr (RuleS1)
-RuleSet             RuleS1;
-
-#endif /* __STDC__ */
+void RuleSetClr (RuleSet RuleS1)
 {
-   USTRING s1;
+   unsigned char *s1;
    int     num;
 
    s1 = RuleS1;
@@ -587,14 +545,7 @@ RuleSet             RuleS1;
    		pEl1 et pEl2 possedent des regles de presentation	
    		specifique.						
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-ThotBool            BothHaveNoSpecRules (PtrElement pEl1, PtrElement pEl2)
-#else  /* __STDC__ */
-ThotBool            BothHaveNoSpecRules (pEl1, pEl2)
-PtrElement          pEl1;
-PtrElement          pEl2;
-
-#endif /* __STDC__ */
+ThotBool BothHaveNoSpecRules (PtrElement pEl1, PtrElement pEl2)
 {
    ThotBool            equal;
 
@@ -610,14 +561,7 @@ PtrElement          pEl2;
    		qui appartient a la view view				
    		et qui n'est pas un pave de presentation.		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-PtrAbstractBox      AbsBoxOfEl (PtrElement pEl, int view)
-#else  /* __STDC__ */
-PtrAbstractBox      AbsBoxOfEl (pEl, view)
-PtrElement          pEl;
-int                 view;
-
-#endif /* __STDC__ */
+PtrAbstractBox AbsBoxOfEl (PtrElement pEl, int view)
 {
    PtrAbstractBox      pAb;
    ThotBool            stop;
@@ -644,15 +588,7 @@ int                 view;
   ApplyNewRule applique au pave courant la regle de		
   presentation specifique qui vient d'etre creee.		
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                ApplyNewRule (PtrDocument pDoc, PtrPRule pPRule, PtrElement pEl)
-#else  /* __STDC__ */
-void                ApplyNewRule (pDoc, pPRule, pEl)
-PtrDocument         pDoc;
-PtrPRule            pPRule;
-PtrElement          pEl;
-
-#endif /* __STDC__ */
+void    ApplyNewRule (PtrDocument pDoc, PtrPRule pPRule, PtrElement pEl)
 {
   PtrAbstractBox      pAb;
   int                 view;
@@ -705,16 +641,8 @@ PtrElement          pEl;
   RemoveSpecifPres supprime toutes les regles de presentation specifiques
   associees a l'element pEl.				
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         RemoveSpecifPres (PtrElement pEl, PtrDocument pDoc, RuleSet rules, int viewToApply)
-#else  /* __STDC__ */
-static void         RemoveSpecifPres (pEl, pDoc, rules, viewToApply)
-PtrElement          pEl;
-PtrDocument         pDoc;
-RuleSet             rules;
-int                 viewToApply;
-
-#endif /* __STDC__ */
+static void RemoveSpecifPres (PtrElement pEl, PtrDocument pDoc,
+			      RuleSet rules, int viewToApply)
 {
    PtrPRule            pPRule, pR, pRS;
    NotifyPresentation  notifyPres;
@@ -794,17 +722,9 @@ int                 viewToApply;
   ApplyPRuleAndRedisplay: apply a presentation rule, update the view
   and apply the inheritance of the rule.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void         ApplyPRuleAndRedisplay (PtrAbstractBox pAb, PtrDocument pDoc, PtrAttribute pAttr, PtrPRule pRP, PtrPSchema pSPR)
-#else  /* __STDC__ */
-static void         ApplyPRuleAndRedisplay (pAb, pDoc, pAttr, pRP, pSPR)
-PtrAbstractBox      pAb;
-PtrDocument         pDoc;
-PtrAttribute        pAttr;
-PtrPRule            pRP;
-PtrPSchema          pSPR;
-
-#endif /* __STDC__ */
+static void ApplyPRuleAndRedisplay (PtrAbstractBox pAb, PtrDocument pDoc,
+				    PtrAttribute pAttr, PtrPRule pRP,
+				    PtrPSchema pSPR)
 {
   if (pRP == NULL)
      return;
@@ -833,17 +753,9 @@ PtrPSchema          pSPR;
   pDoc la regle de presentation standard de type ruleType	
   pour la view viewSch.						
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                ApplyStandardRule (PtrElement pEl, PtrDocument pDoc, PRuleType ruleType, FunctionType funcType, int viewSch)
-#else  /* __STDC__ */
-void                ApplyStandardRule (pEl, pDoc, ruleType, funcType, viewSch)
-PtrElement          pEl;
-PtrDocument         pDoc;
-PRuleType           ruleType;
-FunctionType        funcType;
-int                 viewSch;
-
-#endif /* __STDC__ */
+void  ApplyStandardRule (PtrElement pEl, PtrDocument pDoc,
+			 PRuleType ruleType, FunctionType funcType,
+			 int viewSch)
 {
   PtrPRule            pRP;
   PtrAbstractBox      pAb;
@@ -893,13 +805,7 @@ int                 viewSch;
 /*----------------------------------------------------------------------
   NextAbstractBox: search the next abstract box.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 static PtrAbstractBox     NextAbstractBox (PtrAbstractBox pAb)
-#else  /* __STDC__ */
-static PtrAbstractBox     NextAbstractBox (pAb)
-PtrAbstractBox     pAb;
-#endif /* __STDC__ */
-
 {
   PtrAbstractBox      pNextAb;
 
@@ -928,33 +834,27 @@ PtrAbstractBox     pAb;
   The pPres function presentation rule has been removed for the element
   having abstract box pAb.  Undo that presentation rule for that abstract box.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-static void       RemoveFunctionPRule (PtrPRule pPres, PtrAbstractBox pAb, PtrDocument pDoc)
-#else  /* __STDC__ */
-static void       RemoveFunctionPRule (pPres, pAb, pDoc)
-PtrPRule pPres;
-PtrAbstractBox pAb;
-PtrDocument pDoc;
-#endif /* __STDC__ */
-
+static void RemoveFunctionPRule (PtrPRule pPres, PtrAbstractBox pAb,
+				 PtrDocument pDoc)
 {
-   if (pPres->PrPresFunction == FnBackgroundPicture &&
-       pAb->AbPictBackground != NULL)
-     {
-       TtaFreeMemory ((((PictInfo *) (pAb->AbPictBackground))->PicFileName));
-       FreePictInfo ((PictInfo *) (pAb->AbPictBackground));
-       TtaFreeMemory (pAb->AbPictBackground);
-       pAb->AbPictBackground = NULL;
-     }
-   else if (pPres->PrPresFunction == FnPictureMode &&
-	    pAb->AbPictBackground != NULL)
-       ((PictInfo *) (pAb->AbPictBackground))->PicPresent = FillFrame;
-   else if (pPres->PrPresFunction == FnShowBox)
-       pAb->AbFillBox = FALSE;
+  PictInfo      *image;
 
-   pAb->AbAspectChange = TRUE;
-   RedispAbsBox (pAb, pDoc);
-   /* pDoc->DocViewModifiedAb[view] = pAb; */
+  if (pPres->PrPresFunction == FnBackgroundPicture && pAb->AbPictBackground)
+    {
+      image = (PictInfo *)pAb->AbPictBackground;
+      CleanPictInfo (image);
+      TtaFreeMemory (image->PicFileName);
+      TtaFreeMemory (pAb->AbPictBackground);
+      pAb->AbPictBackground = NULL;
+    }
+  else if (pPres->PrPresFunction == FnPictureMode &&
+	   pAb->AbPictBackground)
+    ((PictInfo *) (pAb->AbPictBackground))->PicPresent = FillFrame;
+  else if (pPres->PrPresFunction == FnShowBox)
+    pAb->AbFillBox = FALSE;
+  
+  pAb->AbAspectChange = TRUE;
+  RedispAbsBox (pAb, pDoc);
 }
 
 /*----------------------------------------------------------------------
@@ -962,17 +862,8 @@ PtrDocument pDoc;
   Redisplay boxes of element pEl that are concerned by removing the
   presentation function pRule
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void            ApplyASpecificStyleRule (PtrPRule pRule, PtrElement pEl, PtrDocument pDoc, ThotBool remove)
-#else  /* __STDC__ */
-void            ApplyASpecificStyleRule (pRule, pEl, pDoc, remove)
-PtrPRule	pRule;
-PtrElement	pEl;
-PtrDocument	pDoc;
-ThotBool        remove;
-
-#endif /* __STDC__ */
-
+void  ApplyASpecificStyleRule (PtrPRule pRule, PtrElement pEl,
+			       PtrDocument pDoc, ThotBool remove)
 {
   PtrAbstractBox  pAb, pParent;
   PtrPRule	  pCurrentRule, pRP;
@@ -982,17 +873,21 @@ ThotBool        remove;
   PRuleType	  ruleType;
   int             viewSch;
   int             view;
-  ThotBool	  done, enclosed;
+  ThotBool	  done, enclosed, complete;
 
-  TtaClearViewSelections ();
   enclosed = FALSE;
   /* do nothing if the document no longer exists */
-  if (pDoc != NULL)
+  if (pDoc)
     /* examine all abstract boxes of element */
     for (view = 0; view < MAX_VIEW_DOC; view++)
     {
       /* the abstract box of the root element */
       pAb = pEl->ElAbstractBox[view];
+      if (pAb == NULL)
+	/* no abstract box */
+	if (pRule->PrType == PtVisibility)
+	  /* it's a visibility rule. Try to create the abstract box */
+	  pAb = AbsBoxesCreate (pEl, pDoc, view+1, True, True, &complete);
       /* the schema view associatde with the current view */
       viewSch = pDoc->DocView[view].DvPSchemaView;
       while (pAb != NULL)
@@ -1095,19 +990,9 @@ ThotBool        remove;
   For each displayed abstract box and each new presention rule
   check if it is concerned by this new pRule.
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void               ApplyAGenericStyleRule (Document doc, PtrSSchema pSS, int elType, int attrType, int presBox, PtrPRule pRule, ThotBool remove)
-#else  /* __STDC__ */
-void               ApplyAGenericStyleRule (doc, pSS, elType, attrType, presBox, pRule, remove)
-Document           doc;
-PtrSSchema         pSS;
-int                elType;
-int                attrType;
-int                presBox;
-PtrPRule           pRule;
-ThotBool           remove;
-#endif /* __STDC__ */
-
+void  ApplyAGenericStyleRule (Document doc, PtrSSchema pSS, int elType,
+			      int attrType, int presBox, PtrPRule pRule,
+			      ThotBool remove)
 {
   PtrPRule        pCurrentRule, pRP;
   PtrPSchema      pSPR;
@@ -1125,8 +1010,6 @@ ThotBool           remove;
   pDoc = LoadedDocument[doc - 1];
   selectionOK = GetCurrentSelection (&pSelDoc, &pFirstSel, &pLastSel,
 				     &firstChar, &lastChar);
-  /* eteint la selection courante */
-  TtaClearViewSelections ();
   /* do nothing if the document no longer exists */
   if (pDoc != NULL)
     /* examine all abstract boxes of elements */
@@ -1143,7 +1026,7 @@ ThotBool           remove;
 	  if (elType > 0)
 	    /* presentation rules are associated with an element type */
 	    found = (pAb->AbElement->ElTypeNumber == elType &&
-	             pAb->AbElement->ElStructSchema->SsCode == pSS->SsCode);
+	             !strcmp (pAb->AbElement->ElStructSchema->SsName, pSS->SsName));
 	  else if (attrType > 0)
 	    {
 	      /* presentation rules are associated with an attribute type */
@@ -1151,7 +1034,7 @@ ThotBool           remove;
 	      while (!found && pAttr != NULL)
 		{
 		  found = (pAttr->AeAttrNum == attrType &&
-			   pAttr->AeAttrSSchema->SsCode == pSS->SsCode);
+			   !strcmp (pAttr->AeAttrSSchema->SsName, pSS->SsName));
 		  if (!found)
 		     pAttr = pAttr->AeNext;
 		}
@@ -1159,7 +1042,7 @@ ThotBool           remove;
 	  else if (presBox > 0)
 	    /* presentation rules are associated with a presentation box */
 	    found = (pAb->AbPresentationBox && pAb->AbTypeNum == presBox &&
-		     pAb->AbPSchema == pSS->SsPSchema);
+		     pAb->AbPSchema == PresentationSchema (pSS, pDoc));
 	  if (found)
 	    {
 	      /* process each presentation rule */
@@ -1210,7 +1093,7 @@ ThotBool           remove;
     }
   /* tente de fusionner les elements voisins et reaffiche les paves */
   /* modifie's et la selection */
-  if (pSelDoc != NULL && pSelDoc == pDoc)
+  if (pDoc && pSelDoc == pDoc)
     SelectRange (pSelDoc, pFirstSel, pLastSel, firstChar, lastChar);
 }
 
@@ -1219,16 +1102,7 @@ ThotBool           remove;
   contenues dans 'RulesS' attachees aux elements du	
   sous-arbre de racine pElRoot				
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                RemoveSpecPresTree (PtrElement pElRoot, PtrDocument pDoc, RuleSet RulesS, int viewToApply)
-#else  /* __STDC__ */
-void                RemoveSpecPresTree (pElRoot, pDoc, RulesS, viewToApply)
-PtrElement          pElRoot;
-PtrDocument         pDoc;
-RuleSet             RulesS;
-int                 viewToApply;
-
-#endif /* __STDC__ */
+void  RemoveSpecPresTree (PtrElement pElRoot, PtrDocument pDoc, RuleSet RulesS, int viewToApply)
 {
    PtrElement          pEl, pNext;
 
@@ -1253,19 +1127,12 @@ int                 viewToApply;
    TtaIsCSSPRule
 
    Check whether a presentation rule is associated with a CSS rule
-
    Parameters:
    pRule: the presentation rule to be tested.
-
    Return value:
    TRUE if pRule is a CSS rule
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
 ThotBool            TtaIsCSSPRule (PRule pRule)
-#else  /* __STDC__ */
-ThotBool            TtaIsCSSPRule (pRule)
-PRule               pRule;
-#endif /* __STDC__ */
 {
    ThotBool         ret;
 
@@ -1275,7 +1142,7 @@ PRule               pRule;
      TtaError (ERR_invalid_parameter);
    else
      /* parameter pRule is correct */
-     ret = (((PtrPRule) pRule)->PrLevel != 0);
+     ret = (((PtrPRule) pRule)->PrSpecificity > 0);
    return ret;
 }
 
@@ -1283,21 +1150,12 @@ PRule               pRule;
    TtaRemovePRule
 
    Removes a presentation rule from an element and release that rule.
-
    Parameters:
    element: the element with which the presentation rule is associated.
    pRule: the presentation rule to be removed.
    document: the document to which the element belongs.
-
   ----------------------------------------------------------------------*/
-#ifdef __STDC__
-void                TtaRemovePRule (Element element, PRule pRule, Document document)
-#else  /* __STDC__ */
-void                TtaRemovePRule (element, pRule, document)
-Element             element;
-PRule               pRule;
-Document            document;
-#endif /* __STDC__ */
+void    TtaRemovePRule (Element element, PRule pRule, Document document)
 {
    PtrPRule            pPres, pPreviousPres;
 
@@ -1347,28 +1205,15 @@ Document            document;
    Returns the first presentation rule associated with a given
    element (if pRule is NULL) or the presentation rule that
    follows a given rule of a given element.
-
    Parameters:
    element: the element of interest.
    pRule: a presentation rule of that element, or NULL
    if the first rule is asked.
-
    Return parameter:
    pRule: the next presentation rule, or NULL if
    pRule is the last rule of the element.
-
   ----------------------------------------------------------------------*/
-
-#ifdef __STDC__
 void                TtaNextPRule (Element element, PRule * pRule)
-
-#else  /* __STDC__ */
-void                TtaNextPRule (element, pRule)
-Element             element;
-PRule              *pRule;
-
-#endif /* __STDC__ */
-
 {
    PtrPRule            nextPRule;
 
