@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -61,7 +61,7 @@
   document pDoc. On envoie le message APP correspondant a		
   l'application et on retourne la reponse de l'application.	
   ----------------------------------------------------------------------*/
-ThotBool PRuleMessagePre (PtrElement pEl, PtrPRule pPRule, PtrDocument pDoc, ThotBool isNew)
+ThotBool PRuleMessagePre (PtrElement pEl, PtrPRule pPRule, int newvalue, PtrDocument pDoc, ThotBool isNew)
 {
    NotifyPresentation  notifyPres;
    PtrPRule            pR, pRPrec;
@@ -75,6 +75,7 @@ ThotBool PRuleMessagePre (PtrElement pEl, PtrPRule pPRule, PtrDocument pDoc, Tho
    notifyPres.document = (Document) IdentDocument (pDoc);
    notifyPres.element = (Element) pEl;
    notifyPres.pRuleType = NumTypePRuleAPI (pPRule);
+   notifyPres.value = newvalue;
    noApply = CallEventType ((NotifyEvent *) & notifyPres, TRUE);
    if (noApply)
       /* l'application demande a l'editeur de ne rien faire */
@@ -298,13 +299,11 @@ void NewPosition (PtrAbstractBox pAb, int X, int xref, int Y, int yref,
 	      if (isPos)
 		dy += value;
 	      
-	      /* modify the distance in the specific rule */
-	      pRuleV->PrPosRule.PoDistance = dy;
 	      /* send the event message to the application */
-	      doitV = !PRuleMessagePre (pEl, pRuleV, pDoc, isNew);
-	      if (!doitV && !isNew)
-		/* reset the previous value */
-		pRuleV->PrPosRule.PoDistance = value;
+	      doitV = !PRuleMessagePre (pEl, pRuleV, dy, pDoc, isNew);
+	      if (doitV || isNew)
+		/* store the new value in the specific rule */
+		pRuleV->PrPosRule.PoDistance = dy;
 	    }
 	   
 	  if (doitV)
@@ -446,13 +445,11 @@ void NewPosition (PtrAbstractBox pAb, int X, int xref, int Y, int yref,
 	      if (isPos)
 		dx += value;
 
-	      /* modify the distance in the specific rule */
-	      pRuleH->PrPosRule.PoDistance = dx;
 	      /* send the event message to the application */
-	      doitH = !PRuleMessagePre (pEl, pRuleH, pDoc, isNew);
-	      if (!doitH && !isNew)
-		/* reset the previous value */
-		pRuleH->PrPosRule.PoDistance = value;
+	      doitH = !PRuleMessagePre (pEl, pRuleH, dx, pDoc, isNew);
+	      if (doitH || isNew)
+		/* store the new value in the specific rule */
+		pRuleH->PrPosRule.PoDistance = dx;
 	    }
 	  if (doitH)
 	    {
@@ -547,6 +544,8 @@ void NewDimension (PtrAbstractBox pAb, int width, int height, int frame,
   histOpen = FALSE;
   doitH = FALSE;
   doitV = FALSE;
+  pRuleH = pRuleV = NULL;
+  attrV = attrH = isDimH = isDimV = FALSE;
   if (dispMode == DisplayImmediately)
     {
       TtaSetDisplayMode (doc, DeferredDisplay);
@@ -700,24 +699,19 @@ void NewDimension (PtrAbstractBox pAb, int width, int height, int frame,
 		}
 	      else if (isDimH)
 		dx += value;
-	      
-	      /* set the absolute value into the rule */
-	      if (isDimH)
-		pRuleH->PrDimRule.DrValue = dx;
-	      else
-		pRuleH->PrDimRule.DrPosRule.PoDistance = dx;
+
 	      if (fromUI)
 		/* send the event message to the application */
-		doitH = !PRuleMessagePre (pEl, pRuleH, pDoc, isNewH);
+		doitH = !PRuleMessagePre (pEl, pRuleH, dx, pDoc, isNewH);
 	      else
 		doitH = TRUE;
-	      if (!doitH && !isNewH)
+	      if (doitH || isNewH)
 		{
-		  /* reset the previous value */
+		  /* store the new value in the specific rule */
 		  if (isDimH)
-		    pRuleH->PrDimRule.DrValue = value;
+		    pRuleH->PrDimRule.DrValue = dx;
 		  else
-		    pRuleH->PrDimRule.DrPosRule.PoDistance = value;
+		    pRuleH->PrDimRule.DrPosRule.PoDistance = dx;		  
 		}
 	    }
 	}
@@ -876,23 +870,18 @@ void NewDimension (PtrAbstractBox pAb, int width, int height, int frame,
 	       else if (isDimV)
 		 dy += value;
 	       
-	       /* set the absolute value into the rule */
-	       if (isDimV)
-		 pRuleV->PrDimRule.DrValue = dy;
+	       if (fromUI)
+		 /* send the event message to the application */
+		 doitV = !PRuleMessagePre (pEl, pRuleV, dy, pDoc, isNewV);
 	       else
-		 pRuleV->PrDimRule.DrPosRule.PoDistance = dy;
-	      if (fromUI)
-		/* send the event message to the application */
-		doitV = !PRuleMessagePre (pEl, pRuleV, pDoc, isNewV);
-	      else
-		doitV = TRUE;
-	       if (!doitV && !isNewV)
+		 doitV = TRUE;
+	       if (doitV || isNewV)
 		 {
-		   /* reset the previous value */
+		   /* store the new value in the specific rule */
 		   if (isDimV)
-		     pRuleV->PrDimRule.DrValue = value;
+		     pRuleV->PrDimRule.DrValue = dy;
 		   else
-		     pRuleV->PrDimRule.DrPosRule.PoDistance = value;
+		     pRuleV->PrDimRule.DrPosRule.PoDistance = dy;
 		 }
 	     }
 	 }
@@ -1040,10 +1029,6 @@ PRule TtaNewPRule (int presentationType, View view, Document document)
 	       pPres->PrType = PtFunction;
 	       pPres->PrPresFunction = FnCreateEnclosing;
 	       break;
-	     case PRLine:
-	       pPres->PrType = PtFunction;
-	       pPres->PrPresFunction = FnLine;
-	       break;
 	     default:
 	       pPres->PrType = (PRuleType) presentationType;
 	       break;
@@ -1108,10 +1093,6 @@ PRule TtaNewPRuleForView (int presentationType, int view, Document document)
 	   pPres->PrType = PtFunction;
 	   pPres->PrPresFunction = FnCreateEnclosing;
 	   break;
-	 case PRLine:
-	   pPres->PrType = PtFunction;
-	   pPres->PrPresFunction = FnLine;
-	   break;
 	 default:
 	   pPres->PrType = (PRuleType) presentationType;
 	   break;
@@ -1190,10 +1171,6 @@ PRule TtaNewPRuleForNamedView (int presentationType, char *viewName,
 	       pPres->PrType = PtFunction;
 	       pPres->PrPresFunction = FnCreateEnclosing;
 	       break;
-	     case PRLine:
-	       pPres->PrType = PtFunction;
-	       pPres->PrPresFunction = FnLine;
-	       break;
 	     default:
 	       pPres->PrType = (PRuleType) presentationType;
 	       break;
@@ -1321,8 +1298,11 @@ void TtaAttachPRule (Element element, PRule pRule, Document document)
    PRWeight: WeightNormal, WeightBold.
    PRUnderline: NoUnderline, Underline, Overline, CrossOut.
    PRThickness: ThinUnderline, ThickUnderline.
+   PRDirection: LeftToRight, RightToLeft.
+   PRUnicodeBidi: Normal, Embed, Override.
    PRLineStyle, PRBorderTopStyle, PRBorderRightStyle, PRBorderBottomStyle,
       PRBorderLeftStyle: SolidLine, DashedLine, DottedLine.
+   PRDisplay: Undefined, Inline, Block, ListItem, RunIn, Compact, Marker.
    PRNoBreak1, PRNoBreak2: a positive integer (distance in points).
    PRIndent: a positive, null or negative integer (indentation in points).
    PRSize: an integer between 6 and 72 (body size in points).
@@ -1504,6 +1484,46 @@ void TtaSetPRuleValue (Element element, PRule pRule, int value, Document documen
 	       break;
 	     }
 	   break;
+	 case PtDirection:
+	   ((PtrPRule) pRule)->PrPresMode = PresImmediate;
+	   switch (value)
+	     {
+	     case LeftToRight:
+	       ((PtrPRule) pRule)->PrChrValue = 'L';
+	       break;
+	     case RightToLeft:
+	       ((PtrPRule) pRule)->PrChrValue = 'R';
+	       break;
+	     default:
+#ifndef NODISPLAY
+	       done = FALSE;
+#endif
+	       TtaError (ERR_invalid_parameter);
+	       break;
+	     }
+	   break;
+	 case PtUnicodeBidi:
+	   ((PtrPRule) pRule)->PrPresMode = PresImmediate;
+	   switch (value)
+	     {
+	     case Normal:
+	       ((PtrPRule) pRule)->PrChrValue = 'N';
+	       break;
+	     case Embed:
+	       ((PtrPRule) pRule)->PrChrValue = 'E';
+	       break;
+	     case Override:
+	       ((PtrPRule) pRule)->PrChrValue = 'O';
+	       break;
+	     default:
+#ifndef NODISPLAY
+	       done = FALSE;
+#endif
+	       TtaError (ERR_invalid_parameter);
+	       break;
+	     }
+	   break;
+
 	 case PtLineStyle:
 	 case PtBorderTopStyle:
 	 case PtBorderRightStyle:
@@ -1530,6 +1550,40 @@ void TtaSetPRuleValue (Element element, PRule pRule, int value, Document documen
 	     }
 	   break;
 
+         case PtDisplay:
+	   ((PtrPRule) pRule)->PrPresMode = PresImmediate;
+	   switch (value)
+	     {
+	     case Undefined:
+	       ((PtrPRule) pRule)->PrChrValue = 'U';
+	       break;
+	     case Inline:
+	       ((PtrPRule) pRule)->PrChrValue = 'I';
+	       break;
+	     case Block:
+	       ((PtrPRule) pRule)->PrChrValue = 'B';
+	       break;
+	     case ListItem:
+	       ((PtrPRule) pRule)->PrChrValue = 'L';
+	       break;
+	     case RunIn:
+	       ((PtrPRule) pRule)->PrChrValue = 'R';
+	       break;
+	     case Compact:
+	       ((PtrPRule) pRule)->PrChrValue = 'C';
+	       break;
+	     case Marker:
+	       ((PtrPRule) pRule)->PrChrValue = 'M';
+	       break;
+	     default:
+#ifndef NODISPLAY
+	       done = FALSE;
+#endif
+	       TtaError (ERR_invalid_parameter);
+	       break;
+	     }
+	   break;
+
 	 case PtBreak1:
 	 case PtBreak2:
 	 case PtIndent:
@@ -1608,12 +1662,12 @@ void TtaSetPRuleValue (Element element, PRule pRule, int value, Document documen
 	     case Centered:
 	       ((PtrPRule) pRule)->PrAdjust = AlignCenter;
 	       break;
-	     case LeftWithDots:
-	       ((PtrPRule) pRule)->PrAdjust = AlignLeftDots;
-	       break;
 	     case Justify:
 	       ((PtrPRule) pRule)->PrAdjust = AlignJustify;
 	       break;
+	     case LeftWithDots:
+	       ((PtrPRule) pRule)->PrAdjust = AlignLeftDots;
+	       break;
 	     default:
 #ifndef NODISPLAY
 	       done = FALSE;
@@ -2312,8 +2366,11 @@ int                 TtaGetPRuleType (PRule pRule)
    PRWeight: WeightNormal, WeightBold.
    PRUnderline: NoUnderline, Underline, Overline, CrossOut.
    PRThickness: ThinUnderline, ThickUnderline.
+   PRDirection: LeftToRight, RightToLeft.
+   PRUnicodeBidi: Normal, Embed, Override.
    PRLineStyle, PRBorderTopStyle, PRBorderRightStyle, PRBorderBottomStyle,
       PRBorderLeftStyle: SolidLine, DashedLine, DottedLine.
+   PRDisplay: Undefined, Inline, Block, ListItem, RunIn, Compact, Marker.
    PRNoBreak1, PRNoBreak2: a positive integer (distance in points).
    PRIndent: a positive, null or negative integer (indentation in points).
    PRSize: an integer between 6 and 72 (body size in points).
@@ -2331,7 +2388,7 @@ int                 TtaGetPRuleType (PRule pRule)
    PRHyphenate: Hyphenation, NoHyphenation.
    PRAdjust: AdjustLeft, AdjustRight, Centered, LeftWithDots, Justify.
   ----------------------------------------------------------------------*/
-int                 TtaGetPRuleValue (PRule pRule)
+int TtaGetPRuleValue (PRule pRule)
 {
   int                 value;
 
@@ -2436,6 +2493,38 @@ int                 TtaGetPRuleValue (PRule pRule)
 	    break;
 	  }
 	break;
+      case PtDirection:
+	switch (((PtrPRule) pRule)->PrChrValue)
+	  {
+	  case 'L':
+	    value = LeftToRight;
+	    break;
+	  case 'R':
+	    value = RightToLeft;
+	    break;
+	  default:
+	    TtaError (ERR_invalid_parameter);
+	    break;
+	  }
+	break;
+      case PtUnicodeBidi:
+	switch (((PtrPRule) pRule)->PrChrValue)
+	  {
+	  case 'N':
+	    value = Normal;
+	    break;
+	  case 'E':
+	    value = Embed;
+	    break;
+	  case 'O':
+	    value = Override;
+	    break;
+	  default:
+	    TtaError (ERR_invalid_parameter);
+	    break;
+	  }
+	break;
+
       case PtLineStyle:
       case PtBorderTopStyle:
       case PtBorderRightStyle:
@@ -2458,6 +2547,35 @@ int                 TtaGetPRuleValue (PRule pRule)
 	  }
 	break;
 
+      case PtDisplay:
+	switch (((PtrPRule) pRule)->PrChrValue)
+	  {
+	  case 'U':
+	    value = Undefined;
+	    break;
+	  case 'I':
+	    value = Inline;
+	    break;
+	  case 'B':
+	    value = Block;
+	    break;
+	  case 'L':
+	    value = ListItem;
+	    break;
+	  case 'R':
+	    value = RunIn;
+	    break;
+	  case 'C':
+	    value = Compact;
+	    break;
+	  case 'M':
+	    value = Marker;
+	    break;
+	  default:
+	    TtaError (ERR_invalid_parameter);
+	    break;
+	  }
+	break;
       case PtBreak1:
       case PtBreak2:
       case PtIndent:
@@ -2513,12 +2631,12 @@ int                 TtaGetPRuleValue (PRule pRule)
 	  case AlignCenter:
 	    value = Centered;
 	    break;
-	  case AlignLeftDots:
-	    value = LeftWithDots;
-	    break;
 	  case AlignJustify:
 	    value = Justify;
 	    break;
+	  case AlignLeftDots:
+	    value = LeftWithDots;
+	    break;
 	  default:
 	    TtaError (ERR_invalid_parameter);
 	    break;
@@ -2549,7 +2667,7 @@ int                 TtaGetPRuleValue (PRule pRule)
    This unit could be UnRelative, UnXHeight, UnPoint, UnPixel, UnPercent.
    Return UnRelative in other cases.
   ----------------------------------------------------------------------*/
-int                 TtaGetPRuleUnit (PRule pRule)
+int TtaGetPRuleUnit (PRule pRule)
 {
   int                 value;
 
@@ -2671,7 +2789,10 @@ int                 TtaSamePRules (PRule pRule1, PRule pRule2)
 			    case PtWeight:
 			    case PtUnderline:
 			    case PtThickness:
+			    case PtDirection:
+			    case PtUnicodeBidi:
 			    case PtLineStyle:
+			    case PtDisplay:
 			    case PtBorderTopStyle:
 			    case PtBorderRightStyle:
 			    case PtBorderBottomStyle:
