@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -515,19 +515,25 @@ void TtaSetAttributeValue (Attribute attribute, int value,
   else if (AttrOfElement (attribute, element))
     {
       if (pAttr->AeAttrType == AtNumAttr)
-	if (abs (value) > 65535)
-	  /* the pivot form represents integers coded on two bytes */
-	  TtaError (ERR_invalid_attribute_value);
-	else
-	  ok = TRUE;
-      else if (pAttr->AeAttrSSchema == NULL)
-	TtaError (ERR_invalid_attribute_type);
-      else if (value < 1)
-	TtaError (ERR_invalid_attribute_value);
-      else if (value > pAttr->AeAttrSSchema->SsAttribute->TtAttr[pAttr->AeAttrNum - 1]->AttrNEnumValues)
-	TtaError (ERR_invalid_attribute_value);
+	{
+	  if (abs (value) > 65535)
+	    /* the pivot form represents integers coded on two bytes */
+	    TtaError (ERR_invalid_attribute_value);
+	  else
+	    if (value != pAttr->AeAttrValue)
+	      ok = TRUE;
+	}
       else
-	ok = TRUE;
+	{
+	  if (pAttr->AeAttrSSchema == NULL)
+	    TtaError (ERR_invalid_attribute_type);
+	  else if (value < 1)
+	    TtaError (ERR_invalid_attribute_value);
+	  else if (value > pAttr->AeAttrSSchema->SsAttribute->TtAttr[pAttr->AeAttrNum - 1]->AttrNEnumValues)
+	    TtaError (ERR_invalid_attribute_value);
+	  else if (value != pAttr->AeAttrValue)
+	    ok = TRUE;
+	}
       if (ok)
 	{
 #ifndef NODISPLAY
@@ -628,6 +634,7 @@ void TtaNextAttribute (Element element, Attribute *attribute)
    *attribute = (Attribute) nextAttribute;
 }
 
+
 /* ----------------------------------------------------------------------
    TtaGiveAttributeType
 
@@ -1213,3 +1220,28 @@ void TtaSearchAttributes (AttributeType searchedAtt1, AttributeType searchedAtt2
       }
     }
 }
+
+/*----------------------------------------------------------------------
+   TtaGetTypedAttrAncestor
+
+   returns a pointer to the attribute of the first
+   element which encloses pEl and that has an attribute
+   of type attNum. The function returns pElAttr if
+   the search is succesful, NULL otherwise.
+  ----------------------------------------------------------------------*/
+Attribute  TtaGetTypedAttrAncestor (Element pEl, int attNum,
+				    SSchema pSSattr, Element *pElAttr)
+{
+  PtrAttribute    pAttr;
+
+  UserErrorCode = 0;
+  pAttr = NULL;
+  if (pEl == NULL)
+    TtaError (ERR_invalid_parameter);
+   else if (attNum == 0)
+    TtaError (ERR_invalid_attribute_type);
+  else
+    pAttr = GetTypedAttrAncestor ((PtrElement) pEl, attNum,
+				  (PtrSSchema) pSSattr, (PtrElement *) pElAttr);
+  return ((Attribute) pAttr);
+}
