@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -46,6 +46,7 @@
 #include "documentapi_f.h"
 #include "exceptions_f.h"
 #include "frame_f.h"
+#include "language_f.h"
 #include "memory_f.h"
 #include "presvariables_f.h"
 #include "references_f.h"
@@ -579,13 +580,12 @@ void ChangeLanguage (PtrDocument pDoc, PtrElement pEl, Language lang,
    int                 view;
    PtrElement          pChild;
    PtrAbstractBox      pAbsBox;
-   Language	       oldElLang;
 
    if (pEl == NULL)
       return;
 
    if (!pEl->ElTerminal)
-      /* l'element n'est pas une feuille, on traite son sous-arbre */
+      /* this element is not a leaf */
      {
 	pChild = pEl->ElFirstChild;
 	while (pChild != NULL)
@@ -596,49 +596,40 @@ void ChangeLanguage (PtrDocument pDoc, PtrElement pEl, Language lang,
 	     pChild = pChild->ElNext;
 	  }
      }
-   else
-      /* this element is a leaf */
-      if (pEl->ElLeafType == LtText &&
-	  pEl->ElLanguage != lang && (pEl->ElLanguage >= 4 || force))
-       /* That's a text leaf in a different language */
-	/*
-	  Changes the language of the element:
-	  lang < 4 when a specific alphabet like symbol
-	*/
-       if (pEl->ElTextLength == 0 ||
-	   TtaGetAlphabet (pEl->ElLanguage) == TtaGetAlphabet (lang))
-	{
-	  oldElLang = pEl->ElLanguage;
-	   pEl->ElLanguage = lang;
-	   /* parcourt toutes les vues du document pour changer les */
-	   /* paves de l'element */
-	   for (view = 0; view < MAX_VIEW_DOC; view++)
-	      if (pEl->ElAbstractBox[view] != NULL)
-		 /* l'element a au moins un pave dans la vue */
-		{
-		   pAbsBox = pEl->ElAbstractBox[view];
-		   /* saute les paves de presentation de l'element */
-		   while (pAbsBox->AbPresentationBox &&
-			  pAbsBox->AbElement == pEl)
-		      pAbsBox = pAbsBox->AbNext;
-		   if (!pAbsBox->AbDead)
-		      /* traite le pave' principal de l'element */
-		      /* change la langue du pave */
-		     {
-			pAbsBox->AbLanguage = lang;
-			if (force ||
-			    TtaGetAlphabet (oldElLang) != TtaGetAlphabet(lang))
-			  /* cette langue s'ecrit dans un alphabet different */
-			  /* ou la langue est forcee */
-			  {
-			    pAbsBox->AbChange = TRUE;
-			    /* conserve le pointeur sur le pave a reafficher */
-			    pDoc->DocViewModifiedAb[view] =
-			      Enclosing (pAbsBox, pDoc->DocViewModifiedAb[view]);
-			  }
-		     }
-		}
-	}
+   else if (pEl->ElLeafType == LtText &&
+	    pEl->ElLanguage != lang &&
+	    (pEl->ElLanguage >= TtaGetFirstUserLanguage () || force))
+     /* That's a text leaf in a different language */
+     /* Changes the language of the element:
+	When lang < TtaGetFirstUserLanguage (), lang actually represents a
+	script, not a real language. */
+     if (pEl->ElTextLength == 0 ||
+	 TtaGetScript (pEl->ElLanguage) == TtaGetScript (lang))
+       {
+	 pEl->ElLanguage = lang;
+	 /* parcourt toutes les vues du document pour changer les */
+	 /* paves de l'element */
+	 for (view = 0; view < MAX_VIEW_DOC; view++)
+	   if (pEl->ElAbstractBox[view] != NULL)
+	     /* l'element a au moins un pave dans la vue */
+	     {
+	       pAbsBox = pEl->ElAbstractBox[view];
+	       /* saute les paves de presentation de l'element */
+	       while (pAbsBox->AbPresentationBox &&
+		      pAbsBox->AbElement == pEl)
+		 pAbsBox = pAbsBox->AbNext;
+	       if (!pAbsBox->AbDead)
+		 /* traite le pave' principal de l'element */
+		 /* change la langue du pave */
+		 {
+		   pAbsBox->AbLang = lang;
+		   pAbsBox->AbChange = TRUE;
+		   /* conserve le pointeur sur le pave a reafficher */
+		   pDoc->DocViewModifiedAb[view] =
+		     Enclosing (pAbsBox, pDoc->DocViewModifiedAb[view]);
+		 }
+	     }
+       }
 }
 
 
@@ -904,29 +895,45 @@ void AttachAttrWithValue (PtrElement pEl, PtrDocument pDoc,
   create = (pAttr == NULL);
 
   /* est-ce une suppression d'attribut */
-  suppress = TRUE;
+  suppress = FALSE;
   switch (pNewAttr->AeAttrType)
     {
     case AtEnumAttr:
-      if (pNewAttr->AeAttrValue != 0)
-	suppress = FALSE;
+      if (pNewAttr->AeAttrValue == 0)
+	suppress = TRUE;
+      else
+	if (pAttr && pNewAttr->AeAttrValue == pAttr->AeAttrValue)
+	  /* the existing attribute already has this value */
+	  return;
       break;
 
     case AtNumAttr:
-      if (pNewAttr->AeAttrValue >= -MAX_INT_ATTR_VAL
-	  && pNewAttr->AeAttrValue <= MAX_INT_ATTR_VAL)
-	suppress = FALSE;
+      if (pNewAttr->AeAttrValue < -MAX_INT_ATTR_VAL ||
+	  pNewAttr->AeAttrValue > MAX_INT_ATTR_VAL)
+	suppress = TRUE;
+      else
+	if (pAttr && pNewAttr->AeAttrValue == pAttr->AeAttrValue)
+	  /* the existing attribut already has this value */
+	  return;
       break;
 
     case AtTextAttr:
-      if (pNewAttr->AeAttrText != NULL)
-	suppress = FALSE;
+      if (pNewAttr->AeAttrText == NULL)
+	suppress = TRUE;
+      else
+	if (pAttr && TextsEqual (pAttr->AeAttrText, pNewAttr->AeAttrText))
+	  return;
       break;
 
     case AtReferenceAttr:
-      if (pNewAttr->AeAttrReference != NULL)
-	if (pNewAttr->AeAttrReference->RdReferred != NULL)
-	  suppress = FALSE;
+      if (pNewAttr->AeAttrReference == NULL ||
+	  pNewAttr->AeAttrReference->RdReferred == NULL)
+	  suppress = TRUE;
+      else
+	if (pAttr && pAttr->AeAttrReference &&
+	    pAttr->AeAttrReference->RdReferred ==
+	                      pNewAttr->AeAttrReference->RdReferred)
+	  return;
       break;
 
     default:
@@ -1229,7 +1236,11 @@ Attribute TtaGetAttribute (Element element, AttributeType attributeType)
 int TtaGetTextAttributeLength (Attribute attribute)
 {
    int                 length;
-   PtrTextBuffer       pBT;
+#ifdef _I18N_
+  unsigned char       c[10], *ptr;
+  int                 i;
+#endif /* _I18N_ */
+   PtrTextBuffer       pBuf;
 
    UserErrorCode = 0;
    length = 0;
@@ -1239,11 +1250,21 @@ int TtaGetTextAttributeLength (Attribute attribute)
      TtaError (ERR_invalid_attribute_type);
    else
      {
-       pBT = ((PtrAttribute) attribute)->AeAttrText;
-       while (pBT != NULL)
+       pBuf = ((PtrAttribute) attribute)->AeAttrText;
+       while (pBuf != NULL)
 	 {
-	   length += pBT->BuLength;
-	   pBT = pBT->BuNext;
+#ifdef _I18N_
+	  i = 0;
+	  while (i < pBuf->BuLength)
+	    {
+	      ptr = c;
+	      length += TtaWCToMBstring (pBuf->BuContent[i], &ptr);
+	      i++;
+	    }
+#else /* _I18N_ */
+	   length += pBuf->BuLength;
+#endif /* _I18N_ */
+	   pBuf = pBuf->BuNext;
 	 }
      }
    return length;
@@ -1273,3 +1294,44 @@ void TtaGiveTextAttributeValue (Attribute attribute, char *buffer, int *length)
     *length = CopyBuffer2MBs (((PtrAttribute) attribute)->AeAttrText, 0,
 			     buffer, *length);
 }
+
+/* ----------------------------------------------------------------------
+   TtaGetAttributeKind
+
+   Returns the kind of the attribute: 0 = Enumerate, 1 = Integer, 2 = Text,
+   3 = CsReference.
+   Parameter:
+   attType: type of the attribute.
+   ---------------------------------------------------------------------- */
+int TtaGetAttributeKind (AttributeType attType)
+{
+  PtrSSchema          pSS;
+  int                 attrKind = 2;
+
+  UserErrorCode = 0;
+  if (attType.AttrSSchema == NULL || attType.AttrTypeNum <= 0 ||
+      attType.AttrTypeNum > ((PtrSSchema)(attType.AttrSSchema))->SsNAttributes)
+    TtaError (ERR_invalid_parameter);
+  else
+    {
+      pSS = (PtrSSchema)(attType.AttrSSchema);
+      switch (pSS->SsAttribute->TtAttr[attType.AttrTypeNum - 1]->AttrType)
+	{
+	case AtEnumAttr:
+	  attrKind = 0;
+	  break;
+	case AtNumAttr:
+	  attrKind = 1;
+	  break;
+	case AtTextAttr:
+	  attrKind = 2;
+	  break;
+	case AtReferenceAttr:
+	  attrKind = 3;
+	  break;
+	default:
+	  break;
+	}
+    } 
+  return attrKind;
+}
