@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1999-2001.
+ *  (c) COPYRIGHT INRIA, 1999-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -14,8 +14,9 @@
 #include <stdlib.h>
 #include "thot_sys.h"
 
+#include "registry_f.h"
 #include "ustring_f.h"
- 
+
 unsigned long offset[6] = {
   0x00000000UL,
   0x00003080UL,
@@ -36,56 +37,136 @@ static CharsetCode CharsetCodeTable[] =
     {"ascii",            US_ASCII},
     {"iso646-US",        US_ASCII},
     {"us",               US_ASCII},
-    {"iso-8859-1",      ISO_8859_1},
-    {"iso_8859-1:1987", ISO_8859_1},
-    {"latin1",          ISO_8859_1},
-    {"iso-8859-2",      ISO_8859_2},
-    {"iso_8859-2:1987", ISO_8859_2},
-    {"latin2",          ISO_8859_2},
-    {"iso-8859-3",      ISO_8859_3},
-    {"iso_8859-3:1988", ISO_8859_3},
-    {"latin3",          ISO_8859_3},
-    {"iso-8859-4",      ISO_8859_4},
-    {"iso_8859-4:1988", ISO_8859_4},
-    {"latin4",          ISO_8859_4},
-    {"iso-8859-5",         ISO_8859_5},
-    {"iso_8859-5:1988",    ISO_8859_5},
+    {"iso-8859-1",       ISO_8859_1},
+    {"iso_8859-1:1987",  ISO_8859_1},
+    {"latin1",           ISO_8859_1},
+    {"iso-8859-2",       ISO_8859_2},
+    {"iso_8859-2:1987",  ISO_8859_2},
+    {"latin2",           ISO_8859_2},
+    {"iso-8859-3",       ISO_8859_3},
+    {"iso_8859-3:1988",  ISO_8859_3},
+    {"latin3",           ISO_8859_3},
+    {"iso-8859-4",       ISO_8859_4},
+    {"iso_8859-4:1988",  ISO_8859_4},
+    {"latin4",           ISO_8859_4},
+    {"iso-8859-5",       ISO_8859_5},
+    {"iso_8859-5:1988",  ISO_8859_5},
     {"iso-8859-6",       ISO_8859_6},
     {"iso_8859-6:1987",  ISO_8859_6},
     {"iso-8859-6",       ISO_8859_6},
     {"arabic",           ISO_8859_6},
     {"ECMA-114",         ISO_8859_6},
-    {"iso-8859-7",      ISO_8859_7},
-    {"iso_8859-7:1987", ISO_8859_7},
-    {"greek",           ISO_8859_7},
-    {"ECMA-118",        ISO_8859_7},
+    {"iso-8859-7",       ISO_8859_7},
+    {"iso_8859-7:1987",  ISO_8859_7},
+    {"greek",            ISO_8859_7},
+    {"ECMA-118",         ISO_8859_7},
     {"iso-8859-8",       ISO_8859_8},
     {"iso_8859-8:1988",  ISO_8859_8},
     {"hebrew",           ISO_8859_8},
-    {"iso-8859-9",      ISO_8859_9},
-    {"iso_8859-9:1989", ISO_8859_9},
-    {"latin5",          ISO_8859_9},
+    {"iso-8859-9",       ISO_8859_9},
+    {"iso_8859-9:1989",  ISO_8859_9},
+    {"latin5",           ISO_8859_9},
+    {"iso-8859-10",      ISO_8859_10},
+    {"iso-8859-15",      ISO_8859_15},
 
-    {"UNICODE-1-1", UNICODE_1_1},
-    {"csUnicode11", UNICODE_1_1},
+    {"UNICODE-1-1",      UNICODE_1_1},
+    {"csUnicode11",      UNICODE_1_1},
 
     {"UNICODE-1-1-UTF-7", UNICODE_1_1_UTF_7},
     {"csUnicode11UTF7",   UNICODE_1_1_UTF_7},
 
-    {"UTF-7", UTF_7},
-    {"UTF-8", UTF_8},
-    {"windows-1250", WINDOWS_1250},
-    {"windows-1251", WINDOWS_1251},
-    {"windows-1252", WINDOWS_1252},
-    {"windows-1253", WINDOWS_1253},
-    {"windows-1254", WINDOWS_1254},
-    {"windows-1255", WINDOWS_1255},
-    {"windows-1256", WINDOWS_1256},
-    {"windows-1257", WINDOWS_1257},
-    {"windows-1258", WINDOWS_1258},
+    {"UTF-7",            UTF_7},
+    {"UTF-8",            UTF_8},
+    {"windows-1250",     WINDOWS_1250},
+    {"windows-1251",     WINDOWS_1251},
+    {"windows-1252",     WINDOWS_1252},
+    {"windows-1253",     WINDOWS_1253},
+    {"windows-1254",     WINDOWS_1254},
+    {"windows-1255",     WINDOWS_1255},
+    {"windows-1256",     WINDOWS_1256},
+    {"windows-1257",     WINDOWS_1257},
+    {"windows-1258",     WINDOWS_1258},
+
+    {"koi8-r",           KOI8_R},
+    {"iso-2022-jp",      ISO_2022_JP},
+    {"euc-jp",           EUC_JP},
+    {"shift_jis",        SHIFT_JIS},
+    {"shift-jis",        SHIFT_JIS},
+    {"x-sjis",           SHIFT_JIS},
     {"", UNDEFINED_CHARSET}
 };
 
+#if defined(_I18N_)
+/*----------------------------------------------------------------------
+  ----------------------------------------------------------------------*/
+wchar_t Ttowlower(wchar_t wc)
+{
+  if (wc < 128) return tolower(wc);
+  else return wc;
+}
+#endif /* _I18N_ */
+
+#if defined(_I18N_) && !defined(_WINDOWS)
+/*----------------------------------------------------------------------
+  ----------------------------------------------------------------------*/
+wchar_t *wcstok(wchar_t *s, const wchar_t *delim, wchar_t **last)
+{
+  wchar_t      *spanp;
+  int           c, sc;
+  wchar_t      *tok;
+
+  if (s == NULL && (s = *last) == NULL)
+    return NULL;
+
+  /*
+   * Skip (span) leading delimiters (s += strspn(s, delim), sort of).
+   */
+ cont:
+  c = *s++;
+  for (spanp = (wchar_t *)delim; (sc = *spanp++) != 0; )
+    {
+      if (c == sc)
+	goto cont;
+    }
+  
+  if (c == 0)		/* no non-delimiter characters */
+    {
+      *last = NULL;
+      return NULL;
+    }
+  tok = s - 1;
+  
+  /*
+   * Scan token (scan for delimiters: s += strcspn(s, delim), sort of).
+   * Note that delim must have one NUL; we stop if we see that, too.
+   */
+  for (;;)
+    {
+      c = *s++;
+      spanp = (wchar_t *)delim;
+      do
+	{
+	  if ((sc = *spanp++) == c)
+	    {
+	      if (c == 0)
+		{
+		  s = NULL;
+		}
+	      else
+		{
+		  wchar_t *w = s - 1;
+		  *w = '\0';
+		}
+	      *last = s;
+	      return tok;
+	    }
+	}
+      while (sc != 0);
+    }
+  /* NOTREACHED */
+}
+#endif /* _I18N_ && !_WINDOWS */
+
 
 /*-------------------------------------------------------------
   ustrcasecmp: compare two strings without regard to case.
@@ -97,26 +178,21 @@ int ustrcasecmp (const CHAR_T *str1, const CHAR_T *str2)
   /* Compatibility of _wcsicmp: WIN NT WIN 9x */
   return (int) _wcsicmp ((wchar_t*)str1, (wchar_t*)str2);
 #else  /* _WINDOWS */
-  int len1 = ustrlen (str1);
-  int len2 = ustrlen (str2);
-  int i    = 0;
+  int     i = 0;
+  wchar_t ch1, ch2;
 
-  if (len1 != len2)
-    return (len1 - len2);
-  while (str1[i])
+  while (str1[i] != EOS && str2[i] != EOS)
     {
-      if (towlower (str1[i]) != towlower (str2[i]))
-	return (str1[i] - str2[i]);
+      ch1 = Ttowlower(str1[i]);
+      ch2 = Ttowlower(str2[i]);
+      if (ch1 != ch2)
+	return (ch1 - ch2);
       i++;
     }
-  return 0;
+  return (str1[i] - str2[i]);
 #endif /* _WINDOWS */
 #else  /* _I18N_ */
-#ifdef _WINDOWS
-  return (unsigned int) _stricmp ((char*)str1, (char*)str2);
-#else  /* _WINDOWS */
   return (unsigned int) strcasecmp ((char*)str1, (char*)str2);
-#endif /* _WINDOWS */
 #endif /* _I18N_ */
 }
 
@@ -227,8 +303,8 @@ int ustrncasecmp (const CHAR_T *str1, const CHAR_T *str2,
 	{
 	  if (str1[ndx] < 255 && str2[ndx] < 255)
 	    {
-	      c1 = towlower (str1[ndx]);
-	      c2 = towlower (str2[ndx]);
+	      c1 = Ttowlower (str1[ndx]);
+	      c2 = Ttowlower (str2[ndx]);
 	      if (c1 != c2)
 		return c1 - c2;
 	    }
@@ -297,9 +373,9 @@ CHAR_T *ustrrchr (const CHAR_T *str, CHAR_T c)
 {
 #ifdef _I18N_
   /* Compatibility of wcsrchr: ANSI, WIN NT and WIN 9x */
-  return (CHAR_T*) (wcsrchr ((wchar_t*)str, (int)c));
+  return (CHAR_T*) (wcsrchr ((wchar_t*)str, (wchar_t)c));
 #else  /* _I18N_ */
-  return (CHAR_T*) strrchr ((char*)str, (int)c);
+  return (CHAR_T*) strrchr ((char*)str, (char)c);
 #endif /* _I18N_ */
 }
 
@@ -355,6 +431,21 @@ CHARSET TtaGetCharset (char *charsetname)
 }
 
 
+/*----------------------------------------------------------------------
+  TtaGetDefaultCharset gives the default charset 
+  ----------------------------------------------------------------------*/
+CHARSET TtaGetDefaultCharset ()
+{
+  char     *charsetname;
+
+  /* it should be given by the system locale */
+  charsetname = TtaGetEnvString ("Default_Charset");
+  if (charsetname)
+    return TtaGetCharset (charsetname);
+  else
+    return ISO_8859_1;
+}
+
 /*----------------------------------------------------------------------
   TtaGetCharsetName gives the constant string of the charset ISO name.
   ----------------------------------------------------------------------*/
