@@ -44,37 +44,38 @@
 
 
 #define MaxAsc 100
-
+static char BoxTypeName[MAX_TXT_LEN];
 
 /*----------------------------------------------------------------------
    AbsBoxType  rend un pointeur sur un buffer qui contient           
    le type de l'element de structure auquel correspond le  
    pave pointe' par pAb.                                  
   ----------------------------------------------------------------------*/
-CHAR_T*             AbsBoxType (PtrAbstractBox pAb, ThotBool origName)
+char *AbsBoxType (PtrAbstractBox pAb, ThotBool origName)
 {
   PtrElement       pEl;
-  CHAR_T*          text;
 
-  text = TtaAllocString (MAX_TXT_LEN);
   if (pAb == NULL)
-    ustrcpy (text, " ");
+    strcpy (BoxTypeName, " ");
   else
     {
       pEl = pAb->AbElement;
       /* copie le nom du type d'element structure auquel appartient la boite */
       if (origName)
-	ustrcpy (text, pEl->ElStructSchema->SsRule[pEl->ElTypeNumber - 1].SrOrigName);
+	strcpy (BoxTypeName,
+		pEl->ElStructSchema->SsRule->SrElem[pEl->ElTypeNumber - 1]->SrOrigName);
       else
-	ustrcpy (text, pEl->ElStructSchema->SsRule[pEl->ElTypeNumber - 1].SrName);
+	strcpy (BoxTypeName,
+		pEl->ElStructSchema->SsRule->SrElem[pEl->ElTypeNumber - 1]->SrName);
       if (pAb->AbPresentationBox)
 	/* Ajoute le nom du type de boite de presentation */
 	{
-	  ustrcat (text, ".");
-	  ustrcat (text, pAb->AbPSchema->PsPresentBox[pAb->AbTypeNum - 1].PbName);
+	  strcat (BoxTypeName, ".");
+	  strcat (BoxTypeName,
+		  pAb->AbPSchema->PsPresentBox->PresBox[pAb->AbTypeNum - 1]->PbName);
 	}
     }
-  return (text);
+  return (BoxTypeName);
 }
 
 
@@ -83,12 +84,13 @@ CHAR_T*             AbsBoxType (PtrAbstractBox pAb, ThotBool origName)
    FreeAbView libere, pour une seule vue, tous les paves englobes par le
    pave pointe par pAb, lui-meme compris.
   ----------------------------------------------------------------------*/
-void                FreeAbView (PtrAbstractBox pAb, int frame)
+void FreeAbView (PtrAbstractBox pAb, int frame)
 {
   PtrAbstractBox      pAbb, pAbbNext;
   PtrTextBuffer       pBT, pNextBT;
   PtrPathSeg          pPa, pNextPa;
   PtrDelayedPRule     pDelPR, pNextDelPR;
+  PictInfo           *image;
   ThotBool            libAb;
 
   if (pAb != NULL)
@@ -174,7 +176,8 @@ void                FreeAbView (PtrAbstractBox pAb, int frame)
 	    if (pAb->AbPresentationBox)
 	      {
 		/* c'est une boite de presentation image */
-		FreePictInfo ((PictInfo *)(pAb->AbPictInfo));
+		image = (PictInfo *)pAb->AbPictInfo;
+		CleanPictInfo (image);
 		TtaFreeMemory (pAb->AbPictInfo);
 		pAb->AbPictInfo = NULL;
 	      }
@@ -183,9 +186,10 @@ void                FreeAbView (PtrAbstractBox pAb, int frame)
 	    if (pAb->AbPictBackground != NULL)
 	      {
 		/* in this particular case we need to free filename */
-		TtaFreeMemory (((PictInfo *)(pAb->AbPictBackground))->PicFileName);
+		image = (PictInfo *)pAb->AbPictBackground;
+		TtaFreeMemory (image->PicFileName);
 		/* ce n'est pas un element image */
-		FreePictInfo ((PictInfo *)(pAb->AbPictBackground));
+		CleanPictInfo (image);
 		TtaFreeMemory (pAb->AbPictBackground);
 		pAb->AbPictBackground = NULL;
 	      }
@@ -803,136 +807,6 @@ static void  SupprAbsBoxes (PtrAbstractBox pAbbRoot, PtrDocument pDoc, ThotBool
      }			/* fin dvol > 0 */
 }
 
-/*----------------------------------------------------------------------
-   AddVolView ajuste le volume de la vue dont le pave pAbbRoot      
-   est le pave racine.                                     
-   VolOpt est le volume optimum de cette vue.              
-   ElMilieu est l'element qui devrait etre au milieu       
-   de la vue.                                              
-     pAbbRoot est une vraie racine d'image abstraite        
-  ----------------------------------------------------------------------*/
-static void AddVolView (int VolOpt, PtrAbstractBox pAbbRoot,
-			PtrElement pElMiddle, PtrDocument pDoc)
-{
-   ThotBool            add, suppress, midHead, stop;
-   int                 view, volprec, dvol;
-   PtrAbstractBox      pAbbPrevious, pAbbParent;
-
-   /* evalue d'abord s'il faut ajouter ou supprimer des paves */
-   add = FALSE;
-   suppress = FALSE;
-   if (IsBreakable (pAbbRoot, pDoc))
-     {
-      if (pAbbRoot->AbVolume < VolOpt - VolOpt / 8)
-	 /* il faudrait creer de nouveaux paves dans cette vue */
-	 add = TRUE;
-      else if (pAbbRoot->AbVolume > VolOpt + VolOpt / 8)
-	 /* il faudrait supprimer des paves dans cette vue */
-	 suppress = TRUE;
-     }
-   if (add || suppress)
-     {
-	/* calcule la position actuelle de pElMiddle dans la vue, pour */
-	/* savoir s'il faut modifier le debut ou la fin de la vue */
-	view = pAbbRoot->AbDocView;
-	if (pElMiddle->ElAbstractBox[view - 1] == NULL)
-	   /* pElMiddle n'a pas de pave dans cette vue */
-	   /* cherche le 1er element englobant pElMiddle qui ait un pave */
-	   /* dans la vue */
-	  {
-	     stop = FALSE;
-	     do
-	       {
-		  pElMiddle = pElMiddle->ElParent;
-		  if (pElMiddle == NULL)
-		    {
-		       stop = TRUE;
-		       pElMiddle = pAbbRoot->AbElement;
-		    }
-		  else if (pElMiddle->ElAbstractBox[view - 1] != NULL)
-		     stop = TRUE;
-	       }
-	     while (!stop);
-	  }
-	/* pElMiddle a un pave dans cette vue */
-	/* calcule le volume des paves qui le precedent */
-	volprec = 0;
-	pAbbParent = pElMiddle->ElAbstractBox[view - 1];
-	do
-	  {
-	     pAbbPrevious = pAbbParent->AbPrevious;
-	     while (pAbbPrevious != NULL)
-	       {
-		  volprec += pAbbPrevious->AbVolume;
-		  pAbbPrevious = pAbbPrevious->AbPrevious;
-	       }
-	     pAbbParent = pAbbParent->AbEnclosing;
-	  }
-	while (pAbbParent != NULL);
-	midHead = volprec < VolOpt / 2;
-	if (add)
-	   /* il faut ajouter des paves */
-	   if (midHead)
-	      /* l'element qui devrait etre au milieu est dans la 1ere moitie' */
-	      if (pAbbRoot->AbTruncatedHead)
-		 /* le debut de l'image n'est pas complete, on ajoute en tete */
-		 AddAbsBoxes (pAbbRoot, pDoc, TRUE);
-	      else
-		{
-		   /* le debut de l'image est complete */
-		   /* on ajoute en queue, si la queue n'est pas complete */
-		   if (pAbbRoot->AbTruncatedTail)
-		      AddAbsBoxes (pAbbRoot, pDoc, FALSE);
-		}
-	   else
-	      /* l'element qui devrait etre au milieu est dans la 2eme moitie' */
-	   if (pAbbRoot->AbTruncatedTail)
-	      /* la fin de l'image n'est pas complete, on ajoute en queue */
-	      AddAbsBoxes (pAbbRoot, pDoc, FALSE);
-	   else
-	     {
-		/* la fin de l'image est complete, on ajoute en tete si le */
-		/* idebut n'est pas complete */
-		if (pAbbRoot->AbTruncatedHead)
-		   AddAbsBoxes (pAbbRoot, pDoc, TRUE);
-	     }
-	else if (suppress)
-	   /* supprime des paves en queue si l'element qui devrait etre au */
-	   /* milieu est dans la premiere moitie de l'image, et inversement */
-	  {
-	     dvol = pAbbRoot->AbVolume - VolOpt;
-	     SupprAbsBoxes (pAbbRoot, pDoc, (ThotBool)(!midHead), &dvol);
-	  }
-     }
-}
-
-/*----------------------------------------------------------------------
-   AdjustVolume pour toutes les vues du document pointe' par pDoc  
-   ajuste (reduit ou augmente) le volume des images        
-   abstraites en conservant l'element pointe par pEl au    
-   milieu (ou a peu pres) de l'image abstraite.            
-  ----------------------------------------------------------------------*/
-void                AdjustVolume (PtrElement pEl, PtrDocument pDoc)
-{
-  int                 view;
-
-  if (pEl != NULL && pDoc != NULL)
-    {
-      for (view = 0; view < MAX_VIEW_DOC; view++)
-	{
-	  /* traite toutes les vues */
-	  if (pDoc->DocView[view].DvPSchemaView > 0)
-	    /* la vue existe */
-	    {
-	      pDoc->DocViewFreeVolume[view] = pDoc->DocViewVolume[view] -
-		                           pDoc->DocViewRootAb[view]->AbVolume;
-	      AddVolView (pDoc->DocViewVolume[view], pDoc->DocViewRootAb[view],
-			  pEl, pDoc);
-	    }
-	}
-    }
-}
-
 /*----------------------------------------------------------------------
   IncreaseVolume Le Mediateur augmente de dVol le volume affichable  
   dans la fenetre ViewFrame.
