@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001.
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -40,10 +40,11 @@
   SetYCompleteForOutOfStruct checks if an external box has a Y position
   that depends on that box.
   ----------------------------------------------------------------------*/
-static void SetYCompleteForOutOfStruct (PtrBox pBox, int visibility, int y, int frame)
+static void SetYCompleteForOutOfStruct (PtrBox pBox, int visibility,
+					int y, int frame)
 {
   PtrPosRelations     pPosRel;
-  BoxRelation        *pRelation;
+  BoxRelation        *pRel;
   ThotBool            notEmpty;
   int                 i;
 
@@ -54,22 +55,14 @@ static void SetYCompleteForOutOfStruct (PtrBox pBox, int visibility, int y, int
       notEmpty = (pPosRel->PosRTable[i].ReBox != NULL);
       while (i < MAX_RELAT_POS && notEmpty)
 	{
-	  pRelation = &pPosRel->PosRTable[i];
-	  if (pRelation->ReBox->BxAbstractBox != NULL)
-	    /* Relation hors-struture sur l'origine de la boite */
-	    if (pRelation->ReOp == OpVertDep &&
-		pRelation->ReBox->BxYOutOfStruct &&
-		pRelation->ReBox->BxAbstractBox->AbVertPos.PosAbRef == pBox->BxAbstractBox &&
-		pRelation->ReBox->BxYToCompute)
-	      {
-		YMoveAllEnclosed (pRelation->ReBox, y, frame);
-		/* La boite distante est deja placee
-		pRelation->ReBox->BxYToCompute = FALSE;
-		AddBoxTranslations (pRelation->ReBox->BxAbstractBox, visibility, frame, FALSE, TRUE);
-		
-		SetYCompleteForOutOfStruct (pRelation->ReBox, visibility, frame);
-		***********/
-	      }
+	  pRel = &pPosRel->PosRTable[i];
+	  if (pRel->ReBox->BxAbstractBox &&
+	      /* Relation hors-struture sur l'origine de la boite */
+	      pRel->ReOp == OpVertDep &&
+	      pRel->ReBox->BxYOutOfStruct &&
+	      pRel->ReBox->BxAbstractBox->AbVertPos.PosAbRef == pBox->BxAbstractBox &&
+	      pRel->ReBox->BxYToCompute)
+	    YMoveAllEnclosed (pRel->ReBox, y, frame);
 	  i++;
 	  if (i < MAX_RELAT_POS)
 	    notEmpty = (pPosRel->PosRTable[i].ReBox != NULL);
@@ -87,7 +80,7 @@ static void SetYCompleteForOutOfStruct (PtrBox pBox, int visibility, int y, int
 static void SetXCompleteForOutOfStruct (PtrBox pBox, int visibility, int x, int frame)
 {
   PtrPosRelations     pPosRel;
-  BoxRelation        *pRelation;
+  BoxRelation        *pRel;
   ThotBool            notEmpty;
   int                 i;
 
@@ -98,21 +91,14 @@ static void SetXCompleteForOutOfStruct (PtrBox pBox, int visibility, int x, int
       notEmpty = (pPosRel->PosRTable[i].ReBox != NULL);
       while (i < MAX_RELAT_POS && notEmpty)
 	{
-	  pRelation = &pPosRel->PosRTable[i];
-	  if (pRelation->ReBox->BxAbstractBox != NULL)
-	    /* Relation hors-struture sur l'origine de la boite */
-	    if (pRelation->ReOp == OpHorizDep
-		&& pRelation->ReBox->BxXOutOfStruct
-		&& pRelation->ReBox->BxAbstractBox->AbHorizPos.PosAbRef == pBox->BxAbstractBox
-		&& pRelation->ReBox->BxXToCompute)
-	      {
-		XMoveAllEnclosed (pRelation->ReBox, x, frame);
-		/* La boite distante est deja placee
-		   pRelation->ReBox->BxXToCompute = FALSE;
-		   AddBoxTranslations (pRelation->ReBox->BxAbstractBox, visibility, frame, TRUE, FALSE);
-		   SetXCompleteForOutOfStruct (pRelation->ReBox, visibility, frame);
-		***********/
-	      }
+	  pRel = &pPosRel->PosRTable[i];
+	  if (pRel->ReBox->BxAbstractBox &&
+	      /* Relation hors-struture sur l'origine de la boite */
+	      pRel->ReOp == OpHorizDep &&
+	      pRel->ReBox->BxXOutOfStruct &&
+	      pRel->ReBox->BxAbstractBox->AbHorizPos.PosAbRef == pBox->BxAbstractBox &&
+	      pRel->ReBox->BxXToCompute)
+	    XMoveAllEnclosed (pRel->ReBox, x, frame);
 	  i++;
 	  if (i < MAX_RELAT_POS)
 	    notEmpty = (pPosRel->PosRTable[i].ReBox != NULL);
@@ -241,7 +227,7 @@ static void SetPageIndicators (PtrAbstractBox pAb, PtrAbstractBox table,
 	toContinue = TRUE;
       if (toContinue)
 	{
-	  if (pBox->BxType == BoSplit)
+	  if (pBox->BxType == BoSplit  || pBox->BxType == BoMulScript)
 	    {
 	      /* --- mis en lignes ------------------------- */
 	      pPreviousBox = pBox;
@@ -335,12 +321,12 @@ static void SetPageIndicators (PtrAbstractBox pAb, PtrAbstractBox table,
 		      pAb->AbOnPageBreak = TRUE;
 		      pAb->AbAfterPageBreak = FALSE;
 		    }
-		} else
-		  { /*if != BoGhost */
-		    pAb->AbOnPageBreak = FALSE;
-		    pAb->AbAfterPageBreak = FALSE;
-		  }
-	      
+		}
+	      else
+		{ /*if != BoGhost */
+		  pAb->AbOnPageBreak = FALSE;
+		  pAb->AbAfterPageBreak = FALSE;
+		}
 	      /* traite les paves fils */
 	      pChildAb = pAb->AbFirstEnclosed;
 	      if (pAb->AbBox->BxType == BoTable)
@@ -410,7 +396,7 @@ void AddBoxTranslations (PtrAbstractBox pAb, int visibility, int frame,
   PtrBox              box1;
   PtrBox              pChildBox;
   PtrBox              pBox;
-  BoxRelation        *pRelation;
+  BoxRelation        *pRel;
   PtrAbstractBox      pChildAb;
   PtrPosRelations     pPosRel;
   int                 width, height;
@@ -453,7 +439,7 @@ void AddBoxTranslations (PtrAbstractBox pAb, int visibility, int frame,
 	    
 	    /* Decale boites englobees dont l'origine depend de l'englobante */
 	    /* La boite est coupee, on decale les boites de coupure */
-	    if (pChildBox->BxType == BoSplit)
+	    if (pChildBox->BxType == BoSplit || pChildBox->BxType == BoMulScript)
 	      {
 		box1 = pChildBox->BxNexChild;
 		while (box1 != NULL)
@@ -472,7 +458,7 @@ void AddBoxTranslations (PtrAbstractBox pAb, int visibility, int frame,
 		if (pChildBox->BxType == BoBlock && pChildBox->BxHorizFlex)
 		  reengloby = TRUE;
 		/* Regarde si la boite est positionnee en X dans l'englobante */
-		box1 = GetHPosRelativePos (pChildBox, NULL);
+		box1 = GetHPosRelativeBox (pChildBox, NULL);
 		placeenX = pChildBox->BxXToCompute;
 		if (box1 == NULL)
 		  newX = TRUE;
@@ -563,7 +549,6 @@ void AddBoxTranslations (PtrAbstractBox pAb, int visibility, int frame,
 		      pChildBox->BxXOrg += x;
 		    
 		    pChildBox->BxXToCompute = FALSE;	/* La boite est placee */
-		    
 		    /* detecte les debordements de la boite englobante */
 		    if (pChildAb->AbHorizEnclosing
 			&& !eclate
@@ -587,29 +572,29 @@ void AddBoxTranslations (PtrAbstractBox pAb, int visibility, int frame,
 			if (!pChildBox->BxHorizFlex)
 			  while (i <= MAX_RELAT_POS && notEmpty)
 			    {
-			      pRelation = &pPosRel->PosRTable[i - 1];
-			      if (pRelation->ReBox->BxAbstractBox != NULL)
+			      pRel = &pPosRel->PosRTable[i - 1];
+			      if (pRel->ReBox->BxAbstractBox != NULL)
 				{
 				  /* Initialise la file des boites deplacees */
 				  pChildBox->BxMoved = NULL;
 				  /* Relation hors-struture sur l'origine de la boite */
-				  if (pRelation->ReOp == OpHorizDep
-				      && pRelation->ReBox->BxXOutOfStruct
-				      && pRelation->ReBox->BxAbstractBox->AbHorizPos.PosAbRef == pChildAb)
+				  if (pRel->ReOp == OpHorizDep
+				      && pRel->ReBox->BxXOutOfStruct
+				      && pRel->ReBox->BxAbstractBox->AbHorizPos.PosAbRef == pChildAb)
 				    {
-				      if (pRelation->ReBox->BxHorizFlex)
-					MoveBoxEdge (pRelation->ReBox, pChildBox, pRelation->ReOp, x, frame, TRUE);
+				      if (pRel->ReBox->BxHorizFlex)
+					MoveBoxEdge (pRel->ReBox, pChildBox, pRel->ReOp, x, frame, TRUE);
 				      else
-					XMove (pRelation->ReBox, pChildBox, x, frame);
+					XMove (pRel->ReBox, pChildBox, x, frame);
 				      /* La boite distante est placee */
-				      pRelation->ReBox->BxXToCompute = FALSE;
+				      pRel->ReBox->BxXToCompute = FALSE;
 				    }
 				  /* Relation sur la largeur elastique de la boite */
-				  else if (pRelation->ReOp == OpWidth)
+				  else if (pRel->ReOp == OpWidth)
 				    {
 				      /* Pas de deplacement du contenu des boites qui */
 				      /*  dependent de la boite elastique             */
-				      MoveBoxEdge (pRelation->ReBox, pChildBox, pRelation->ReOp, x, frame, TRUE);
+				      MoveBoxEdge (pRel->ReBox, pChildBox, pRel->ReOp, x, frame, TRUE);
 				    }
 				}
 			      i++;
@@ -677,29 +662,29 @@ void AddBoxTranslations (PtrAbstractBox pAb, int visibility, int frame,
 			if (!pChildBox->BxVertFlex)
 			  while (i <= MAX_RELAT_POS && notEmpty)
 			    {
-			      pRelation = &pPosRel->PosRTable[i - 1];
-			      if (pRelation->ReBox->BxAbstractBox != NULL)
+			      pRel = &pPosRel->PosRTable[i - 1];
+			      if (pRel->ReBox->BxAbstractBox != NULL)
 				{
 				  /* Initialise la file des boites deplacees */
 				  pChildBox->BxMoved = NULL;
 				  /* Relation hors-struture sur l'origine de la boite */
-				  if (pRelation->ReOp == OpVertDep
-				      && pRelation->ReBox->BxYOutOfStruct
-				      && pRelation->ReBox->BxAbstractBox->AbVertPos.PosAbRef == pChildAb)
+				  if (pRel->ReOp == OpVertDep
+				      && pRel->ReBox->BxYOutOfStruct
+				      && pRel->ReBox->BxAbstractBox->AbVertPos.PosAbRef == pChildAb)
 				    {
-				      if (pRelation->ReBox->BxVertFlex)
-					MoveBoxEdge (pRelation->ReBox, pChildBox, pRelation->ReOp, y, frame, FALSE);
+				      if (pRel->ReBox->BxVertFlex)
+					MoveBoxEdge (pRel->ReBox, pChildBox, pRel->ReOp, y, frame, FALSE);
 				      else
-					YMove (pRelation->ReBox, pChildBox, y, frame);
+					YMove (pRel->ReBox, pChildBox, y, frame);
 				      /* La boite distante est placee */
-				      pRelation->ReBox->BxYToCompute = FALSE;
+				      pRel->ReBox->BxYToCompute = FALSE;
 				    }
 				  /* Relation sur la hauteur elastique de la boite */
-				  else if (pRelation->ReOp == OpHeight)
+				  else if (pRel->ReOp == OpHeight)
 				    {
 				      /* Pas de deplacement du contenu des boites qui */
 				      /*  dependent de la boite elastique             */
-				      MoveBoxEdge (pRelation->ReBox, pChildBox, pRelation->ReOp, y, frame, FALSE);
+				      MoveBoxEdge (pRel->ReBox, pChildBox, pRel->ReOp, y, frame, FALSE);
 				    }
 				}
 			      i++;
@@ -751,117 +736,122 @@ void AddBoxTranslations (PtrAbstractBox pAb, int visibility, int frame,
   ----------------------------------------------------------------------*/
 void SetPageHeight (PtrAbstractBox pAb, int *ht, int *pos, int *nChars)
 {
-   PtrBox              box;
-   PtrBox              pChildBox;
-   PtrTextBuffer       adbuff;
-   int                 height;
-   int                 org, i;
-   ThotBool            still;
+  PtrBox              box;
+  PtrBox              pChildBox;
+  PtrTextBuffer       adbuff;
+  int                 height;
+  int                 org, i;
+  ThotBool            still;
 
-   *nChars = 0;
-   *pos = 0;
-   *ht = 0;
-   box = pAb->AbBox;
-   if (box != NULL)
-     {
-	if (box->BxType == BoSplit)
-	  {
-	    /* the box is split into different lines */
-	     pChildBox = box->BxNexChild;
-	     org = pChildBox->BxYOrg;
-	     /* look for the last piece */
-	     while (pChildBox->BxNexChild != NULL)
-		pChildBox = pChildBox->BxNexChild;
-
-	     height = pChildBox->BxYOrg + pChildBox->BxHeight - org;
-	     /* BxMoved gives the plit box */
-	     box = box->BxMoved;
-	     /* number of characters included in the page */
-	     if (pAb->AbOnPageBreak && box != NULL)
-	       {
-		  *nChars = box->BxIndChar;
-		  /* Il ne faut pas couper le dernier mot d'une page     */
-		  /* donc si la boite precedente est de type BtAvectrait */
-		  /* la limite de la page est deplacee sur le blanc qui  */
-		  /* precede ce mot */
-		  if (*nChars != 0 &&
-		      box->BxPrevious->BxType == BoDotted &&
-		      box->BxPrevious->BxNSpaces != 0)
-		    {
-		      /* get the previous space */
-		      adbuff = box->BxBuffer;
-		      i = box->BxFirstChar - 1;
-		      still = TRUE;
-		      while (still)
-			if (adbuff->BuContent[i] == SPACE)
+  *nChars = 0;
+  *pos = 0;
+  *ht = 0;
+  box = pAb->AbBox;
+  if (box)
+    {
+      if (box->BxType == BoSplit || box->BxType == BoMulScript)
+	{
+	  /* the box is split into different lines */
+	  pChildBox = box->BxNexChild;
+	  org = pChildBox->BxYOrg;
+	  /* look for the last piece */
+	  while (pChildBox->BxNexChild)
+	    pChildBox = pChildBox->BxNexChild;
+	  
+	  height = pChildBox->BxYOrg + pChildBox->BxHeight - org;
+	  /* BxMoved gives the plit box */
+	  box = box->BxMoved;
+	  /* number of characters included in the page */
+	  if (pAb->AbOnPageBreak && box)
+	    {
+	      *nChars = box->BxFirstChar - 1;
+	      /* Il ne faut pas couper le dernier mot d'une page     */
+	      /* donc si la boite precedente est de type BoDotted */
+	      /* la limite de la page est deplacee sur le blanc qui  */
+	      /* precede ce mot */
+	      if (*nChars >= 0 &&
+		  box->BxPrevious->BxType == BoDotted &&
+		  box->BxPrevious->BxNSpaces)
+		{
+		  /* get the previous space */
+		  adbuff = box->BxBuffer;
+		  i = box->BxIndChar;
+		  still = TRUE;
+		  while (still)
+		    if (adbuff->BuContent[i] == SPACE)
+		      {
+			/* found */
+			still = FALSE;
+			/* the next word starts with the next character */
+			(*nChars)++;
+		      }
+		    else
+		      {
+			(*nChars)--;
+			if (i == 0)
 			  {
-			    /* found */
-			    still = FALSE;
-			    /* the next word starts with the next character */
-			    (*nChars)++;
+			    if (adbuff->BuPrevious)
+			      {
+				adbuff = adbuff->BuPrevious;
+				i = adbuff->BuLength - 1;
+			      }
+			    else
+				/* stop */
+			      still = FALSE;
 			  }
 			else
-			  {
-			    (*nChars)--;
-			    if (i == 0)
-			      if (adbuff->BuPrevious != NULL)
-				{
-				  adbuff = adbuff->BuPrevious;
-				  i = adbuff->BuLength - 1;
-				}
-			      else
-				/* stop */
-				still = FALSE;
-			    else
-			      i--;
-			  }
-		    }
-	       }
-	  }
-	else if (box->BxType == BoGhost)
-	  {
-	     /* check enclosed boxes */
-	     while (box->BxType == BoGhost)
-		box = box->BxAbstractBox->AbFirstEnclosed->AbBox;
-
-	     if (box->BxType == BoSplit)
-		box = box->BxNexChild;
-	     /* get the position of the first child */
-	     org = box->BxYOrg;
-	     height = 0;
-	     while (box != NULL)
-	       {
-		  /* get the bottom limit */
-		  if (box->BxType == BoPiece)
-		     /* look for the last piece */
-		     while (box->BxNexChild != NULL)
-			box = box->BxNexChild;
-		  i = box->BxYOrg + box->BxHeight;
-		  if (i > height)
-		     height = i;
-		  if (box->BxAbstractBox->AbNext == NULL)
-		     box = NULL;
-		  else
-		     box = box->BxAbstractBox->AbNext->AbBox;
-	       }
-	     /* box height */
-	     height -= org;
-	  }
-	else
-	  {
-	     org = box->BxYOrg + box->BxTMargin + box->BxTBorder + box->BxTPadding;
-	     height = box->BxH;
-	  }
-
-	*pos = org;
-	*ht = height;
-     }
+			  i--;
+		      }
+		}
+	    }
+	}
+      else if (box->BxType == BoGhost)
+	{
+	  /* check enclosed boxes */
+	  while (box->BxType == BoGhost)
+	    box = box->BxAbstractBox->AbFirstEnclosed->AbBox;
+	  
+	  if (box->BxType == BoSplit || box->BxType == BoMulScript)
+	    box = box->BxNexChild;
+	  /* get the position of the first child */
+	  org = box->BxYOrg;
+	  height = 0;
+	  while (box != NULL)
+	    {
+	      /* get the bottom limit */
+	      if (box->BxType == BoSplit ||
+		  box->BxType == BoMulScript ||
+		  box->BxType == BoPiece ||
+		  box->BxType == BoScript ||
+		  box->BxType == BoDotted)
+		/* look for the last piece */
+		while (box->BxNexChild != NULL)
+		  box = box->BxNexChild;
+	      i = box->BxYOrg + box->BxHeight;
+	      if (i > height)
+		height = i;
+	      if (box->BxAbstractBox->AbNext == NULL)
+		box = NULL;
+	      else
+		box = box->BxAbstractBox->AbNext->AbBox;
+	    }
+	  /* box height */
+	  height -= org;
+	}
+      else
+	{
+	  org = box->BxYOrg + box->BxTMargin + box->BxTBorder + box->BxTPadding;
+	  height = box->BxH;
+	}
+      
+      *pos = org;
+      *ht = height;
+    }
 }
 
 
 /*----------------------------------------------------------------------
-   SetBoxToTranslate marque le sous-arbre des paves en cours de    
-   placement.                                              
+  SetBoxToTranslate marks boxes which are not already placed.
   ----------------------------------------------------------------------*/
 void SetBoxToTranslate (PtrAbstractBox pAb, ThotBool horizRef, ThotBool vertRef)
 {
@@ -876,10 +866,10 @@ void SetBoxToTranslate (PtrAbstractBox pAb, ThotBool horizRef, ThotBool vertRef)
    /* sont toujours placees en absolue           */
    if (pBox->BxHorizFlex || pBox->BxXOutOfStruct)
       horizRef = FALSE;
-   if (pBox->BxYToCompute && (pBox->BxVertFlex || pBox->BxYOutOfStruct))
+   if (pBox->BxVertFlex || pBox->BxYOutOfStruct)
       vertRef = FALSE;
 
-   if (pBox->BxType != BoSplit)
+   if (pBox->BxType != BoSplit && pBox->BxType != BoMulScript)
      {
 	pBox->BxXToCompute = horizRef;
 	pBox->BxYToCompute = vertRef;
