@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -12,7 +12,6 @@
  *
  */
 
-#include "ustring.h"
 #include "thot_sys.h"
 #include "thot_gui.h"
 #include "libmsg.h"
@@ -28,12 +27,13 @@
 #include "appli_f.h"
 #include "boxmoves_f.h"
 #include "boxrelations_f.h"
+#include "buildboxes_f.h"
 #include "font_f.h"
 #include "memory_f.h"
 
 /*----------------------------------------------------------------------
   ----------------------------------------------------------------------*/
-static void         ClearBoxMoved (PtrBox pBox)
+static void ClearBoxMoved (PtrBox pBox)
 {
   PtrBox           pNextBox;
   while (pBox != NULL)
@@ -97,19 +97,19 @@ static void InsertPosRelation (PtrBox pOrginBox, PtrBox pTargetBox,
   int                 i;
   ThotBool            loop;
   ThotBool            empty;
-  PtrPosRelations     pPreviousPosRel;
+  PtrPosRelations     pPreviousPosRel, pNext;
   PtrPosRelations     pPosRel;
   BoxRelation        *pRelation;
 
   i = 0;
-  /* PcFirst sens de la dependance dans les positionnements entre soeurs */
+  /* add a relation from origin to target */
   if (op != OpHorizRef && op != OpVertRef && op != OpWidth && op != OpHeight)
     {
-      /* On recherche une entree libre */
+      /* look for an empty entry */
       pPosRel = pOrginBox->BxPosRelations;
       pPreviousPosRel = NULL;
       loop = TRUE;
-      while (loop && pPosRel != NULL)
+      while (loop && pPosRel)
 	{
 	  i = 0;
 	  pPreviousPosRel = pPosRel;
@@ -126,14 +126,16 @@ static void InsertPosRelation (PtrBox pOrginBox, PtrBox pTargetBox,
 	      i--;
 	    }
 	  else
+	    /* next block */
 	    pPosRel = pPosRel->PosRNext;
-	  /* Bloc suivant */
 	}
 
-      /* Faut-il creer un nouveau bloc de relations ? */
       if (loop)
 	{
+	  /* add a new block */
+	  pNext = pPosRel;
 	  GetPosBlock (&pPosRel);
+	  pPosRel->PosRNext = pNext;
 	  if (pPreviousPosRel == NULL)
 	    pOrginBox->BxPosRelations = pPosRel;
 	  else
@@ -146,20 +148,20 @@ static void InsertPosRelation (PtrBox pOrginBox, PtrBox pTargetBox,
       pRelation->ReOp = op;
     }
 
-  /* Deuxieme sens de la dependance */
+  /* add a relation from target to origin */
   if (op != OpHorizInc && op != OpVertInc)
     {
-      /* On recherche une entree libre */
+      /* look for an empty entry */
       pPosRel = pTargetBox->BxPosRelations;
       pPreviousPosRel = NULL;
       loop = TRUE;
-      while (loop && pPosRel != NULL)
+      while (loop && pPosRel)
 	{
 	  i = 0;
 	  pPreviousPosRel = pPosRel;
 	  do
 	    {
-	      empty = pPosRel->PosRTable[i].ReBox == NULL;
+	      empty = (pPosRel->PosRTable[i].ReBox == NULL);
 	      i++;
 	    }
 	  while (i != MAX_RELAT_POS && !empty);
@@ -170,14 +172,16 @@ static void InsertPosRelation (PtrBox pOrginBox, PtrBox pTargetBox,
 	      i--;
 	    }
 	  else
-	    /* Bloc suivant */
+	    /* next block */
 	    pPosRel = pPosRel->PosRNext;
 	}
 
-      /* Faut-il creer un nouveau bloc de relations ? */
       if (loop)
 	{
+	  /* add a new block */
+	  pNext = pPosRel;
 	  GetPosBlock (&pPosRel);
+	  pPosRel->PosRNext = pNext;
 	  if (pPreviousPosRel == NULL)
 	    pTargetBox->BxPosRelations = pPosRel;
 	  else
@@ -200,7 +204,7 @@ static void InsertPosRelation (PtrBox pOrginBox, PtrBox pTargetBox,
 static void InsertDimRelation (PtrBox pOrginBox, PtrBox pTargetBox,
 			       ThotBool sameDimension, ThotBool horizRef)
 {
-  PtrDimRelations     pPreviousDimRel;
+  PtrDimRelations     pPreviousDimRel, pNext;
   PtrDimRelations     pDimRel;
   int                 i;
   ThotBool            loop;
@@ -210,7 +214,7 @@ static void InsertDimRelation (PtrBox pOrginBox, PtrBox pTargetBox,
     horizRef = !horizRef;
 
   i = 0;
-  /* On determine la dimension affectee */
+  /* add a relation from origin to target */
   if (horizRef)
     if (sameDimension)
       pDimRel = pOrginBox->BxWidthRelations;
@@ -221,7 +225,7 @@ static void InsertDimRelation (PtrBox pOrginBox, PtrBox pTargetBox,
   else
     pDimRel = pOrginBox->BxWidthRelations;
 
-  /* On recherche une entree libre */
+  /* look for an empty entry */
   pPreviousDimRel = NULL;
   loop = TRUE;
   while (loop && pDimRel != NULL)
@@ -230,7 +234,7 @@ static void InsertDimRelation (PtrBox pOrginBox, PtrBox pTargetBox,
       pPreviousDimRel = pDimRel;
       do
 	{
-	  empty = pDimRel->DimRTable[i] == NULL;
+	  empty = (pDimRel->DimRTable[i] == NULL);
 	  i++;
 	}
       while (i != MAX_RELAT_DIM && !empty);
@@ -241,18 +245,24 @@ static void InsertDimRelation (PtrBox pOrginBox, PtrBox pTargetBox,
 	  i--;
 	}
       else
+	/* next block */
 	pDimRel = pDimRel->DimRNext;
     }
 
   /* Faut-il creer un nouveau bloc de relations ? */
   if (loop)
     {
+      /* add a new block */
+      pNext = pDimRel;
       GetDimBlock (&pDimRel);
+      pDimRel->DimRNext = pNext;
       if (pPreviousDimRel == NULL)
-	if (horizRef)
-	  pOrginBox->BxWidthRelations = pDimRel;
-	else
-	  pOrginBox->BxHeightRelations = pDimRel;
+	{
+	  if (horizRef)
+	    pOrginBox->BxWidthRelations = pDimRel;
+	  else
+	    pOrginBox->BxHeightRelations = pDimRel;
+	}
       else
 	pPreviousDimRel->DimRNext = pDimRel;
       i = 0;
@@ -316,7 +326,8 @@ static PtrAbstractBox NextAbToCheck (PtrAbstractBox pAb, PtrAbstractBox pRefAb)
 /*----------------------------------------------------------------------
    PropagateXOutOfStruct propage l'indicateur hors-structure.        
   ----------------------------------------------------------------------*/
-static void  PropagateXOutOfStruct (PtrAbstractBox pCurrentAb, ThotBool status, ThotBool enclosed)
+static void PropagateXOutOfStruct (PtrAbstractBox pCurrentAb,
+				   ThotBool status, ThotBool enclosed)
 {
    PtrAbstractBox      pAb;
    PtrBox              pBox;
@@ -330,7 +341,7 @@ static void  PropagateXOutOfStruct (PtrAbstractBox pCurrentAb, ThotBool status,
    while (pAb != NULL)
      {
 	pBox = pAb->AbBox;
-	if (pAb == pCurrentAb || pBox == NULL || pAb->AbDead)
+	if (pAb == pCurrentAb || pBox == NULL || IsDead (pAb))
 	   ;			/* inutile de traiter ce pave */
 	else if (pBox->BxXOutOfStruct == status)
 	   ;			/* inutile de traiter ce pave */
@@ -345,8 +356,9 @@ static void  PropagateXOutOfStruct (PtrAbstractBox pCurrentAb, ThotBool status,
 		pAb->AbHorizEnclosing = enclosed;
 	     PropagateXOutOfStruct (pAb, status, pAb->AbHorizEnclosing);
 	  }
-	else if (pAb->AbWidth.DimIsPosition
-		 && pAb->AbWidth.DimPosition.PosAbRef == pCurrentAb && !pAb->AbWidthChange)
+	else if (pAb->AbWidth.DimIsPosition &&
+		 pAb->AbWidth.DimPosition.PosAbRef == pCurrentAb &&
+		 !pAb->AbWidthChange)
 	   /* Dependance de dimension */
 	   pBox->BxWOutOfStruct = status;
 
@@ -359,7 +371,8 @@ static void  PropagateXOutOfStruct (PtrAbstractBox pCurrentAb, ThotBool status,
 /*----------------------------------------------------------------------
    PropagateYOutOfStruct propage l'indicateur hors-structure.         
   ----------------------------------------------------------------------*/
-static void PropagateYOutOfStruct (PtrAbstractBox pCurrentAb, ThotBool status, ThotBool enclosed)
+static void PropagateYOutOfStruct (PtrAbstractBox pCurrentAb,
+				   ThotBool status, ThotBool enclosed)
 {
    PtrAbstractBox      pAb;
    PtrBox              pBox;
@@ -373,7 +386,7 @@ static void PropagateYOutOfStruct (PtrAbstractBox pCurrentAb, ThotBool status, T
    while (pAb != NULL)
      {
 	pBox = pAb->AbBox;
-	if (pAb == pCurrentAb || pBox == NULL || pAb->AbDead)
+	if (pAb == pCurrentAb || pBox == NULL || IsDead (pAb))
 	   ;			/* inutile de traiter ce pave */
 	else if (pBox->BxYOutOfStruct == status)
 	   ;			/* inutile de traiter ce pave */
@@ -402,7 +415,7 @@ static void PropagateYOutOfStruct (PtrAbstractBox pCurrentAb, ThotBool status, T
   updates the position value according to the current edge used.
   The initial value of val depends on the refered box.
   ----------------------------------------------------------------------*/
-void    SetPositionConstraint (BoxEdge localEdge, PtrBox pBox, int *val)
+void SetPositionConstraint (BoxEdge localEdge, PtrBox pBox, int *val)
 {
   AbPosition         *pPosAb;
 
@@ -496,20 +509,13 @@ void     ComputeRadius (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 void ComputeMBP (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 {
   PtrBox              pBox;
-  int                 dim, i;
+  int                 dim;
 
   if (horizRef)
     {
-      /* reference for computing percent rules */
-      if (pAb->AbEnclosing)
-	{
-	  pBox = pAb->AbEnclosing->AbBox;
-	  dim = pBox->BxW;
-	}
-      else
-	GetSizesFrame (frame, &dim, &i);
-
       pBox = pAb->AbBox;
+      /* reference for percent rules */
+      dim = pBox->BxW;
       /* left margin */
       if (pAb->AbLeftMarginUnit == UnPercent)
 	pBox->BxLMargin = PixelValue (pAb->AbLeftMargin, UnPercent, (PtrAbstractBox) dim, 0);
@@ -556,16 +562,9 @@ void ComputeMBP (PtrAbstractBox pAb, int frame, ThotBool horizRef)
     }
   else
     {
-      /* reference for computing percent rules */
-      if (pAb->AbEnclosing)
-	{
-	  pBox = pAb->AbEnclosing->AbBox;
-	  dim = pBox->BxH;
-	}
-      else
-	GetSizesFrame (frame, &i, &dim);
-
       pBox = pAb->AbBox;
+      /* reference for percent rules */
+      dim = pBox->BxH;
       /* top margin */
       if (pAb->AbTopMarginUnit == UnPercent)
 	pBox->BxTMargin = PixelValue (pAb->AbTopMargin, UnPercent, (PtrAbstractBox) dim, 0);
@@ -617,14 +616,13 @@ void ComputeMBP (PtrAbstractBox pAb, int frame, ThotBool horizRef)
   according to the parameter horizRef for the box pBox. 
   The box origin BxXOrg or BxYOrg is updated and dependencies between 
   boxes are registered.  
-
   Relation between values:
   ^
   BxXOrg
   <-LMargin-><-LBorder-><-LPadding-><-W-><-RPadding-><-RBorder-><-LRargin->
   <---------------------------------Width--------------------------------->
   ----------------------------------------------------------------------*/
-void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool horizRef)
+void ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool horizRef)
 {
   PtrAbstractBox      pRefAb;
   PtrAbstractBox      pAb, child;
@@ -642,7 +640,7 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
   pRefBox = NULL;
   pRefAb = rule.PosAbRef;
   pAb = pBox->BxAbstractBox;
-  if (pRefAb != NULL && pRefAb->AbDead)
+  if (pRefAb && IsDead (pRefAb))
     {
       fprintf (stderr, "Position refers a dead box");
       pRefAb = NULL;
@@ -676,7 +674,8 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
 	      if (rule.PosUnit == UnPercent)
 		dist = PixelValue (rule.PosDistance, UnPercent, (PtrAbstractBox) x, 0);
 	      else
-		dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
+		dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb,
+				   ViewFrameTable[frame - 1].FrMagnification);
 	    }
 	  else
 	    {
@@ -700,7 +699,8 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
 		  pRefBox = pRefAb->AbBox;
 		  if (rule.PosUnit == UnPercent)
 		    /* poucentage de la largeur de l'englobant */
-		    dist = PixelValue (rule.PosDistance, UnPercent, (PtrAbstractBox) pAb->AbEnclosing, 0);
+		    dist = PixelValue (rule.PosDistance, UnPercent,
+				       (PtrAbstractBox) pAb->AbEnclosing, 0);
 		  else
 		    dist = 0;
 		  refEdge = Left;
@@ -761,7 +761,8 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
 	      pAb->AbHorizPos.PosUnit = UnPixel;
 	    }
 	  else
-	    dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
+	    dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb,
+			       ViewFrameTable[frame - 1].FrMagnification);
 	}
     }
   else
@@ -792,13 +793,14 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
 	      if (rule.PosUnit == UnPercent)
 		dist = PixelValue (rule.PosDistance, UnPercent, (PtrAbstractBox) y, 0);
 	      else
-		dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
+		dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb,
+				   ViewFrameTable[frame - 1].FrMagnification);
 	    }
 	  else
 	    {
 	      /* there is an enclosing box */
 	      pRefAb = GetPosRelativeAb (pAb, horizRef);
-	      if (pRefAb != NULL)
+	      if (pRefAb)
 		{
 		  /* Align baselines */
 		  sibling = TRUE;
@@ -866,12 +868,13 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
 		  pBox->BxYOutOfStruct = TRUE;
 		  PropagateYOutOfStruct (pAb, TRUE, pAb->AbVertEnclosing);
 		}
-	      else if (pRefAb->AbBox != NULL)
+	      else if (pRefAb->AbBox)
 		{
 		  /* depend on a sibling box */
 		  sibling = TRUE;
 		  if (pRefAb->AbBox->BxYOutOfStruct
-		      || (pRefAb->AbBox->BxHorizFlex && pRefAb->AbLeafType == LtCompound && pRefAb->AbInLine && refEdge != Top)
+		      || (pRefAb->AbBox->BxHorizFlex && pRefAb->AbLeafType == LtCompound &&
+			  pRefAb->AbInLine && refEdge != Top)
 		      || (pRefAb->AbBox->BxHOutOfStruct && refEdge != Top))
 		    {
 		      /* The box inherits out from a structure position */
@@ -896,11 +899,12 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
 	      pAb->AbVertPos.PosUnit = UnPixel;
 	    }
 	  else
-	    dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
+	    dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb,
+			       ViewFrameTable[frame - 1].FrMagnification);
 	}
     }
 
-  if (pRefAb != NULL)
+  if (pRefAb)
     {
       /* Depend on another box */
       pRefBox = pRefAb->AbBox;
@@ -1035,7 +1039,7 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
 	  break;
 	}
     }
-
+ 
   /* Calcule l'origine de la boite et les points fixes */
   if (horizRef)
     {
@@ -1108,14 +1112,14 @@ void  ComputePosRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool hori
 
 
 /*----------------------------------------------------------------------
-   GetHPosRelativePos search the box which horizontally links   
+   GetHPosRelativeBox search the box which horizontally links   
    pBox to is enclosing:                                
    - If relations were not updated, that box is BxHorizInc
    - If the box has the relation OpHorizInc, it's itself
    - Else get the sibling box which gives the position.     
-   Return a box or NULL.                        
+   Return a box or NULL.
   ----------------------------------------------------------------------*/
-PtrBox              GetHPosRelativePos (PtrBox pBox, PtrBox pPreviousBox)
+PtrBox GetHPosRelativeBox (PtrBox pBox, PtrBox pPreviousBox)
 {
   PtrBox              pRelativeBox;
   PtrPosRelations     pPosRel;
@@ -1191,7 +1195,7 @@ PtrBox              GetHPosRelativePos (PtrBox pBox, PtrBox pPreviousBox)
 		  if (pRelation->ReBox->BxHorizFlex)
 		    pRelativeBox = pRelation->ReBox;
 		  else
-		    pRelativeBox = GetHPosRelativePos (pRelation->ReBox, pBox);
+		    pRelativeBox = GetHPosRelativeBox (pRelation->ReBox, pBox);
 		  
 		  /* Est-ce que l'on a trouve la boite qui donne la position ? */
 		  if (pRelativeBox != NULL)
@@ -1231,11 +1235,11 @@ PtrBox              GetHPosRelativePos (PtrBox pBox, PtrBox pPreviousBox)
    GetVPosRelativeBox search the box which vertically links 
    pBox to is enclosing:                                
    - If relations were not updated, that box is BxVertInc
-   - If the box has the relation OpHorizInc, it's itself
+   - If the box has the relation OpVertInc, it's itself
    - Else get the sibling box which gives the position.     
    Return a box or NULL.                        
   ----------------------------------------------------------------------*/
-PtrBox              GetVPosRelativeBox (PtrBox pBox, PtrBox pPreviousBox)
+PtrBox GetVPosRelativeBox (PtrBox pBox, PtrBox pPreviousBox)
 {
   PtrBox              pRelativeBox;
   PtrPosRelations     pPosRel;
@@ -1379,6 +1383,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
   AbDimension        *pDimAb;
   AbPosition         *pPosAb;
   int                 val, delta, i;
+  int                 dx, dy;
   ThotBool            inLine;
   ThotBool            defaultDim;
 
@@ -1429,7 +1434,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 	      pAb->AbWidth.DimUserSpecified = FALSE;
 	    }
 	  /* verifie que la dimension ne depend pas d'un pave mort */
-	  else if (pAb->AbHorizPos.PosAbRef->AbDead)
+	  else if (IsDead (pAb->AbHorizPos.PosAbRef))
 	    {
 	      fprintf (stderr, "Dimension refers a dead box");
 	      pAb->AbWidth.DimIsPosition = FALSE;
@@ -1478,7 +1483,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 	      pAb->AbHeight.DimUserSpecified = FALSE;
 	    }
 	  /* verifie que la dimension ne depend pas d'un pave mort */
-	  else if (pAb->AbVertPos.PosAbRef->AbDead)
+	  else if (IsDead (pAb->AbVertPos.PosAbRef))
 	    {
 	      fprintf (stderr, "Dimension refers a dead box");
 	      pAb->AbHeight.DimIsPosition = FALSE;
@@ -1505,9 +1510,9 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 	    }
 
 	  /* check if the relative box is not already dead */
-	  if (pDimAb->DimAbRef != NULL && pDimAb->DimAbRef->AbDead)
+	  if (pDimAb->DimAbRef && IsDead (pDimAb->DimAbRef))
 	    {
-	      fprintf (stderr, "Dimension refers a dead box");
+	      /* the dimension refers a dead box */
 	      pDimAb->DimAbRef = NULL;
 	      pDimAb->DimValue = -1;
 	    }
@@ -1533,7 +1538,9 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 	      else
 		pDimAb->DimUnit = UnPixel;
 	    }
-	  
+
+	  dx = pBox->BxLMargin + pBox->BxLBorder + pBox->BxLPadding + pBox->BxRMargin + pBox->BxRBorder + pBox->BxRPadding;
+	  dy = pBox->BxTMargin + pBox->BxTBorder + pBox->BxTPadding + pBox->BxBMargin + pBox->BxBBorder + pBox->BxBPadding;
 	  if (pParentAb == NULL)
 	    /* It's the root box */
 	    if (horizRef)
@@ -1553,7 +1560,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 		      val = PixelValue (pDimAb->DimValue, pDimAb->DimUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
 		    if (pDimAb->DimValue < 0 || pDimAb->DimUnit == UnPercent)
 		      /* the rule gives the outside value */
-		      val = val - pBox->BxLMargin - pBox->BxLBorder - pBox->BxLPadding - pBox->BxRMargin - pBox->BxRBorder - pBox->BxRPadding;
+		      val = val - dx;
 		    ResizeWidth (pBox, pBox, NULL, val - pBox->BxW, 0, 0, 0, frame);
 		  }
 	      }
@@ -1575,7 +1582,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 		      val = PixelValue (pDimAb->DimValue, pDimAb->DimUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
 		    if (pDimAb->DimValue < 0 || pDimAb->DimUnit == UnPercent)
 		      /* the rule gives the outside value */
-		      val = val - pBox->BxTMargin - pBox->BxTBorder - pBox->BxTPadding - pBox->BxBMargin - pBox->BxBBorder - pBox->BxBPadding;
+		      val = val - dy;
 		    ResizeHeight (pBox, pBox, NULL, val - pBox->BxH, 0, 0, frame);
 		  }
 	      }
@@ -1610,7 +1617,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 				/* inherited from the parent */
 			      val = PixelValue (pDimAb->DimValue, UnPercent, (PtrAbstractBox) pParentAb->AbBox->BxW, 0);
 				/* the rule gives the outside value */
-			      val = val - pBox->BxLMargin - pBox->BxLBorder - pBox->BxLPadding - pBox->BxRMargin - pBox->BxRBorder - pBox->BxRPadding;
+			      val = val - dx;
 			      InsertDimRelation (pParentAb->AbBox, pBox, pDimAb->DimSameDimension, horizRef);
 			    }
 			  else
@@ -1709,7 +1716,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 			      else
 				val += PixelValue (pDimAb->DimValue, pDimAb->DimUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
 			      /* the rule gives the outside value */
-			      val = val - pBox->BxLMargin - pBox->BxLBorder - pBox->BxLPadding - pBox->BxRMargin - pBox->BxRBorder - pBox->BxRPadding;
+			      val = val - dx;
 			      ResizeWidth (pBox, pBox, NULL, val - pBox->BxW, 0, 0, 0, frame);
 			      /* Marks out of structure relations */
 			      if (pDimAb->DimAbRef != pParentAb
@@ -1750,7 +1757,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 			      /* inherited from the parent */
 			      val = PixelValue (pDimAb->DimValue, UnPercent, (PtrAbstractBox) pParentAb->AbBox->BxH, 0);
 			      /* the rule gives the outside value */
-			      val = val - pBox->BxTMargin - pBox->BxTBorder - pBox->BxTPadding - pBox->BxBMargin - pBox->BxBBorder - pBox->BxBPadding;
+			      val = val - dy;
 			      InsertDimRelation (pParentAb->AbBox, pBox, pDimAb->DimSameDimension, horizRef);
 			    }
 			  else
@@ -1762,11 +1769,28 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 		  else if (inLine && pDimAb->DimAbRef == pParentAb
 			   && (pAb->AbLeafType == LtPicture || pAb->AbLeafType == LtCompound))
 		    {
-		      /* inherited from the contents */
-		      pBox->BxContentHeight = TRUE;
-		      pDimAb->DimAbRef = NULL;
-		      pDimAb->DimValue = -1;
-		      pDimAb->DimUnit = UnRelative;
+		      if (pDimAb->DimUnit == UnPercent)
+			{
+			  while (!pParentAb->AbHeight.DimIsPosition &&
+				 pParentAb->AbHeight.DimValue < 0 &&
+				 pParentAb->AbHeight.DimAbRef == NULL &&
+				 pParentAb->AbEnclosing)
+			    pParentAb = pParentAb->AbEnclosing;
+			  GetSizesFrame (frame, &val, &i);
+			  /* inherited from the parent */
+			  val = PixelValue (pDimAb->DimValue, UnPercent, (PtrAbstractBox) i, 0);
+			  /* the rule gives the outside value */
+			  val = val - dy;
+			  InsertDimRelation (pParentAb->AbBox, pBox, pDimAb->DimSameDimension, horizRef);
+			}
+		      else
+			{
+			  /* inherited from the contents */
+			  pBox->BxContentHeight = TRUE;
+			  pDimAb->DimAbRef = NULL;
+			  pDimAb->DimValue = -1;
+			  pDimAb->DimUnit = UnRelative;
+			}
 		    }
 		  else
 		    {
@@ -1845,7 +1869,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 				  else
 				    val += PixelValue (pDimAb->DimValue, pDimAb->DimUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
 				  /* the rule gives the outside value */
-				  val = val - pBox->BxTMargin - pBox->BxTBorder - pBox->BxTPadding - pBox->BxBMargin - pBox->BxBBorder - pBox->BxBPadding;
+				  val = val - dy;
 				  ResizeHeight (pBox, pBox, NULL, val - pBox->BxH, 0, 0, frame);
 				  /* Marks out of structure relations */
 				  if (pDimAb->DimAbRef != pParentAb
@@ -1926,7 +1950,7 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
 	  if (pRefBox == NULL)
 	    {
 	      /* On doit resoudre une reference en avant */
-	      if (!pPosAb->PosAbRef->AbDead)
+	      if (!IsDead (pPosAb->PosAbRef))
 		pRefBox = GetBox (pPosAb->PosAbRef);
 	      if (pRefBox != NULL)
 		pPosAb->PosAbRef->AbBox = pRefBox;
@@ -2112,150 +2136,154 @@ ThotBool  ComputeDimRelation (PtrAbstractBox pAb, int frame, ThotBool horizRef)
   ----------------------------------------------------------------------*/
 void ComputeAxisRelation (AbPosition rule, PtrBox pBox, int frame, ThotBool horizRef)
 {
-   int                 x, y, dist;
-   PtrBox              pRefBox;
-   PtrAbstractBox      pRefAb;
-   PtrAbstractBox      pAb;
-   BoxEdge             refEdge, localEdge;
+  PtrBox              pRefBox;
+  PtrAbstractBox      pRefAb;
+  PtrAbstractBox      pAb;
+  BoxEdge             refEdge, localEdge;
+  int                 x, y, dist;
 
-   /* Calcule la position de reference */
-   pRefAb = rule.PosAbRef;
-   pAb = pBox->BxAbstractBox;
-   /* Verifie que la position ne depend pas d'un pave mort */
-   if (pRefAb != NULL && pRefAb->AbDead)
-     {
-	fprintf (stderr, "Position refers a dead box");
-	pRefAb = NULL;
-     }
+  /* Calcule la position de reference */
+  pRefAb = rule.PosAbRef;
+  pAb = pBox->BxAbstractBox;
+  /* Verifie que la position ne depend pas d'un pave mort */
+  if (pRefAb && IsDead (pRefAb))
+    pRefAb = NULL;
 
-   if (pRefAb == NULL)
-     {
-       /* default rule */
-	pRefBox = pBox;
-	if (horizRef)
-	  {
-	     refEdge = Left;
-	     localEdge = VertRef;
-	     if (rule.PosUnit == UnPercent)
-		dist = PixelValue (rule.PosDistance, UnPercent, (PtrAbstractBox) pBox->BxW, 0);
-	     else
-		dist = 0;
-	  }
-	else
-	  {
-	     refEdge = Bottom;
-	     localEdge = HorizRef;
-	     if (rule.PosUnit == UnPercent)
-		dist = PixelValue (rule.PosDistance, UnPercent, (PtrAbstractBox) pBox->BxH, 0);
-	     else
-		dist = FontBase (pBox->BxFont) - pBox->BxH;
-	  }
-     }
-   else
-     {
-       /* explicit rule */
-	refEdge = rule.PosRefEdge;
-	localEdge = rule.PosEdge;
-	/* Convert the distance value */
-	if (rule.PosUnit == UnPercent)
-	  {
-	     if (horizRef)
-		dist = PixelValue (rule.PosDistance, UnPercent, (PtrAbstractBox) pAb->AbEnclosing->AbBox->BxW, 0);
-	     else
-		dist = PixelValue (rule.PosDistance, UnPercent, (PtrAbstractBox) pAb->AbEnclosing->AbBox->BxH, 0);
-	  }
-	else
-	   dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb, ViewFrameTable[frame - 1].FrMagnification);
+  if (pRefAb == NULL)
+    {
+      /* default rule */
+      pRefBox = pBox;
+      if (horizRef)
+	{
+	  refEdge = Left;
+	  localEdge = VertRef;
+	  if (rule.PosUnit == UnPercent)
+	    dist = PixelValue (rule.PosDistance, UnPercent,
+			       (PtrAbstractBox) pBox->BxW, 0);
+	  else
+	    dist = 0;
+	}
+      else
+	{
+	  refEdge = HorizRef;
+	  localEdge = HorizRef;
+	  if (rule.PosUnit == UnPercent)
+	    dist = PixelValue (rule.PosDistance, UnPercent,
+			       (PtrAbstractBox) pBox->BxH, 0);
+	  else
+	    dist = BoxFontBase (pBox->BxFont);
+	}
+    }
+  else
+    {
+      /* explicit rule */
+      refEdge = rule.PosRefEdge;
+      localEdge = rule.PosEdge;
+      /* Convert the distance value */
+      if (rule.PosUnit == UnPercent)
+	{
+	  if (horizRef)
+	    dist = PixelValue (rule.PosDistance, UnPercent,
+			       (PtrAbstractBox) pAb->AbEnclosing->AbBox->BxW, 0);
+	  else
+	    dist = PixelValue (rule.PosDistance, UnPercent,
+			       (PtrAbstractBox) pAb->AbEnclosing->AbBox->BxH, 0);
+	}
+      else
+	dist = PixelValue (rule.PosDistance, rule.PosUnit, pAb,
+			   ViewFrameTable[frame - 1].FrMagnification);
+      
+      pRefBox = pRefAb->AbBox;
+      if (pRefBox == NULL)
+	{
+	  /* On doit resoudre une reference en avant */
+	  pRefBox = GetBox (pRefAb);
+	  if (pRefBox)
+	    pRefAb->AbBox = pRefBox;
+	  else
+	    {
+	      if (horizRef)
+		pAb->AbVertRefChange = FALSE;
+	      else
+		pAb->AbHorizRefChange = FALSE;
+	      return;	/* plus de boite libre */
+	    }
+	}
+    }
 
-	pRefBox = pRefAb->AbBox;
-	if (pRefBox == NULL)
-	  {
-	     /* On doit resoudre une reference en avant */
-	     pRefBox = GetBox (pRefAb);
-	     if (pRefBox != NULL)
-		pRefAb->AbBox = pRefBox;
-	     else
-	       {
-		  if (horizRef)
-		     pAb->AbVertRefChange = FALSE;
-		  else
-		     pAb->AbHorizRefChange = FALSE;
-		  return;	/* plus de boite libre */
-	       }
-	  }
-     }
+  /* Deplacement par rapport a la boite distante */
+  /* L'axe est place par rapport a la boite elle-meme */
+  if (pRefBox == pBox
+      || pBox->BxType == BoGhost
+      || (pAb->AbInLine && pAb->AbLeafType == LtCompound))
+    {
+      x = pBox->BxLMargin + pBox->BxLBorder + pBox->BxLPadding;
+      y = pBox->BxTMargin + pBox->BxTBorder + pBox->BxTPadding;
+    }
+  /* L'axe est place par rapport a une incluse */
+  else if (Propagate != ToSiblings)
+    {
+      /* Il faut peut-etre envisager que pRefBox soit une boite coupee */
+      x = pRefBox->BxXOrg - pBox->BxXOrg;
+      y = pRefBox->BxYOrg - pBox->BxYOrg;
+    }
+  else
+    {
+      x = pRefBox->BxXOrg;
+      y = pRefBox->BxYOrg;
+    }
 
-   /* Deplacement par rapport a la boite distante */
-   /* L'axe est place par rapport a la boite elle-meme */
-   if (pRefBox == pBox
-       || pBox->BxType == BoGhost
-       || (pAb->AbInLine && pAb->AbLeafType == LtCompound))
-     {
-	x = pBox->BxLMargin + pBox->BxLBorder + pBox->BxLPadding;
-	y = pBox->BxTMargin + pBox->BxTBorder + pBox->BxTPadding;
-     }
-   /* L'axe est place par rapport a une incluse */
-   else if (Propagate != ToSiblings)
-     {
-	/* Il faut peut-etre envisager que pRefBox soit une boite coupee */
-	x = pRefBox->BxXOrg - pBox->BxXOrg;
-	y = pRefBox->BxYOrg - pBox->BxYOrg;
-     }
-   else
-     {
-	x = pRefBox->BxXOrg;
-	y = pRefBox->BxYOrg;
-     }
+  switch (refEdge)
+    {
+    case Left:
+    case Top:
+      break;
+    case Bottom:
+      y += pRefBox->BxH;
+      break;
+    case Right:
+      x += pRefBox->BxW;
+      break;
+    case HorizRef:
+      if (pRefBox != pBox)
+	y += pRefBox->BxHorizRef;
+      break;
+    case VertRef:
+      if (pRefBox != pBox)
+	x += pRefBox->BxVertRef;
+      break;
+    case HorizMiddle:
+      y += pRefBox->BxHeight / 2;
+      break;
+    case VertMiddle:
+      x += pRefBox->BxWidth / 2;
+      break;
+    case NoEdge:
+      break;
+    }
 
-   switch (refEdge)
-     {
-     case Left:
-     case Top:
-       break;
-     case Bottom:
-       y += pRefBox->BxHeight;
-       break;
-     case Right:
-       x += pRefBox->BxWidth;
-       break;
-     case HorizRef:
-       y += pRefBox->BxHorizRef;
-       break;
-     case VertRef:
-       x += pRefBox->BxVertRef;
-       break;
-     case HorizMiddle:
-       y += pRefBox->BxHeight / 2;
-       break;
-     case VertMiddle:
-       x += pRefBox->BxWidth / 2;
-       break;
-     case NoEdge:
-       break;
-     }
-
-   /* Met a jour l'axe de la boite */
-   if (horizRef)
-     {
-	x = x + dist + pBox->BxLMargin + pBox->BxLBorder + pBox->BxLPadding - pBox->BxVertRef;
-	MoveVertRef (pBox, NULL, x, frame);
-	/* la regle axe de reference est interpretee */
-	pAb->AbVertRefChange = FALSE;
-	if (pRefBox != NULL)
-	   InsertPosRelation (pBox, pRefBox, OpHorizRef, localEdge, refEdge);
-     }
-   else
-     {
-	y = y + dist + pBox->BxTMargin + pBox->BxTBorder + pBox->BxTPadding - pBox->BxHorizRef;
-	MoveHorizRef (pBox, NULL, y, frame);
-	/* la regle axe de reference est interpretee */
-	pAb->AbHorizRefChange = FALSE;
-	if (pRefBox != NULL)
-	   InsertPosRelation (pBox, pRefBox, OpVertRef, localEdge, refEdge);
-     }
-
-   /* break down the temporary link of moved boxes */
-   ClearBoxMoved (pBox);
+  /* Met a jour l'axe de la boite */
+  if (horizRef)
+    {
+      x = x + dist - pBox->BxVertRef;
+      MoveVertRef (pBox, NULL, x, frame);
+      /* la regle axe de reference est interpretee */
+      pAb->AbVertRefChange = FALSE;
+      if (pRefBox != NULL)
+	InsertPosRelation (pBox, pRefBox, OpHorizRef, localEdge, refEdge);
+    }
+  else
+    {
+      y = y + dist - pBox->BxHorizRef;
+      MoveHorizRef (pBox, NULL, y, frame);
+      /* la regle axe de reference est interpretee */
+      pAb->AbHorizRefChange = FALSE;
+      if (pRefBox != NULL)
+	InsertPosRelation (pBox, pRefBox, OpVertRef, localEdge, refEdge);
+    }
+  
+  /* break down the temporary link of moved boxes */
+  ClearBoxMoved (pBox);
 }
 
 
@@ -2299,39 +2327,7 @@ static ThotBool RemovePosRelation (PtrBox pOrginBox, PtrBox pTargetBox,
 	while (i <= MAX_RELAT_POS && notEmpty)
 	  {
 	    pRelation = &pPosRel->PosRTable[i - 1];
-	    if (pRelation->ReBox->BxAbstractBox == NULL)
-printf ("Relation not removed\n");
-#ifdef IV
-	      {
-		/* a dead relation, remove it */
-		j = i;
-		while (j < MAX_RELAT_POS)
-		  {
-		    k = j + 1;
-			  /* New patch LC 16.02.2001 */
-			  if (pPosRel->PosRTable[k - 1].ReBox == NULL)
-			    {
-			      pPosRel->PosRTable[j - 1].ReBox = NULL;
-			      j = MAX_RELAT_POS;
-			    }
-			  else
-			    {
-			      pPosRel->PosRTable[j - 1] = pPosRel->PosRTable[k - 1];
-			      j++;
-			      if (j == MAX_RELAT_POS)
-				pPosRel->PosRTable[j - 1].ReBox = NULL;
-			    }		  }
-		
-		if (i == MAX_RELAT_POS)
-		  /* the last entry */
-		  pRelation->ReBox = NULL;
-		else
-		  /* re-examine this entry */
-		  i--;
-	      }
-	    else
-#endif /* IV */
-            if (horizRef)
+	    if (horizRef)
 	      {
 		/* horizontal relation */
 		if (pRelation->ReBox == pTargetBox &&
@@ -2348,13 +2344,13 @@ printf ("Relation not removed\n");
 		else if (found == 0 && Pos && pRelation->ReOp == OpHorizDep &&
 			 pCurrentAb != NULL && pRelation->ReBox == pTargetBox)
 		  {
-		
+		    
 		    /* it's the position relation of pCurrentAb */
 		    pAb = pRelation->ReBox->BxAbstractBox;
 		    pAb = pAb->AbHorizPos.PosAbRef;
 		    
 		    if (pAb == NULL)
-		    /* a rule NIL, get the associated abstract box */
+		      /* a rule NIL, get the associated abstract box */
 		      pAb = GetPosRelativeAb (pRelation->ReBox->BxAbstractBox, horizRef);
 		    if (pAb != pCurrentAb)
 		      {
@@ -2632,7 +2628,7 @@ void ClearPosRelation (PtrBox pBox, ThotBool horizRef)
   if ((horizRef && pBox->BxXOutOfStruct) || (!horizRef && pBox->BxYOutOfStruct))
     {
       /* ClearOutOfStructRelation removes out of structure relations */
-      if (!pRefAb->AbDead)
+      if (!IsDead (pRefAb))
 	{
 	  /* On remonte a la racine depuis le pave pere */
 	  pCurrentAb = pRefAb;
@@ -2860,7 +2856,7 @@ void    ClearDimRelation (PtrBox pBox, ThotBool horizRef, int frame)
     {
       /* Si la boite est detruite la procedure ClearOutOfStructRelation
 	 detruit automatiquement cette relation */
-      if (!pRefAb->AbDead)
+      if (!IsDead (pRefAb))
 	{
 	  /* On remonte a la racine depuis le pave pere */
 	  if (pAb != NULL)
