@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -36,7 +36,7 @@
 /*----------------------------------------------------------------------
   DisplayPointSelection draw control points of the box.
   ----------------------------------------------------------------------*/
-void         DisplayPointSelection (int frame, PtrBox pBox, int pointselect)
+void DisplayPointSelection (int frame, PtrBox pBox, int pointselect)
 {
   ViewFrame          *pFrame;
   PtrAbstractBox      pAb;
@@ -60,12 +60,12 @@ void         DisplayPointSelection (int frame, PtrBox pBox, int pointselect)
       halfThick = thick / 2;
 
       /* selection points */
-      leftX = pBox->BxXOrg - pFrame->FrXOrg;
-      topY = pBox->BxYOrg - pFrame->FrYOrg;
-      bottomY = topY + pBox->BxHeight - thick;
-      rightX = leftX + pBox->BxWidth - thick;
-      middleX = leftX + (pBox->BxWidth / 2) - halfThick;
-      middleY = topY + (pBox->BxHeight / 2) - halfThick;
+      leftX = pBox->BxXOrg + pBox->BxLMargin + pBox->BxLBorder + pBox->BxLPadding - pFrame->FrXOrg;
+      topY = pBox->BxYOrg + pBox->BxTMargin + pBox->BxTBorder + pBox->BxTPadding - pFrame->FrYOrg;
+      bottomY = topY + pBox->BxH - thick;
+      rightX = leftX + pBox->BxW - thick;
+      middleX = leftX + (pBox->BxW / 2) - halfThick;
+      middleY = topY + (pBox->BxH / 2) - halfThick;
       if (pAb->AbLeafType == LtPicture)
 	{
 	  /* 8 control points */
@@ -400,7 +400,7 @@ void         DisplayPointSelection (int frame, PtrBox pBox, int pointselect)
   DisplayBgBoxSelection paints the box background with the selection
   color.
   ----------------------------------------------------------------------*/
-void         DisplayBgBoxSelection (int frame, PtrBox pBox)
+void DisplayBgBoxSelection (int frame, PtrBox pBox)
 {
   PtrBox              pChildBox;
   ViewFrame          *pFrame;
@@ -411,7 +411,7 @@ void         DisplayBgBoxSelection (int frame, PtrBox pBox)
     {
       pFrame = &ViewFrameTable[frame - 1];
       pAb = pBox->BxAbstractBox;
-      if (pBox->BxType == BoSplit)
+      if (pBox->BxType == BoSplit || pBox->BxType == BoMulScript)
 	{
 	  /* display the selection on pieces of the current box */
 	  pChildBox = pBox->BxNexChild;
@@ -443,7 +443,7 @@ void         DisplayBgBoxSelection (int frame, PtrBox pBox)
 /*----------------------------------------------------------------------
   DrawBoxSelection paints the box with the selection background.
   ----------------------------------------------------------------------*/
-void         DrawBoxSelection (int frame, PtrBox pBox)
+void DrawBoxSelection (int frame, PtrBox pBox)
 {
   PtrBox              pChildBox;
   PtrAbstractBox      pAb;
@@ -451,7 +451,7 @@ void         DrawBoxSelection (int frame, PtrBox pBox)
   if (pBox != NULL)
     {
       pAb = pBox->BxAbstractBox;      
-      if (pBox->BxType == BoSplit)
+      if (pBox->BxType == BoSplit || pBox->BxType == BoMulScript)
 	{
 	  /* display the selection on pieces of the current box */
 	  pChildBox = pBox->BxNexChild;
@@ -473,7 +473,7 @@ void         DrawBoxSelection (int frame, PtrBox pBox)
   SetNewSelectionStatus goes through the tree for switching the selection
   indicator.
   ----------------------------------------------------------------------*/
-void    SetNewSelectionStatus (int frame, PtrAbstractBox pAb, ThotBool status)
+void SetNewSelectionStatus (int frame, PtrAbstractBox pAb, ThotBool status)
 {
   PtrAbstractBox      pChildAb;
   ViewFrame          *pFrame;
@@ -511,16 +511,23 @@ void    SetNewSelectionStatus (int frame, PtrAbstractBox pAb, ThotBool status)
   DisplayStringSelection the selection on a substring of text
   between leftX and rightX.
   ----------------------------------------------------------------------*/
-void         DisplayStringSelection (int frame, int leftX, int rightX, PtrBox pBox)
+void DisplayStringSelection (int frame, int leftX, int rightX, PtrBox pBox)
 {
   PtrBox              pParentBox;
   ViewFrame          *pFrame;
   PtrAbstractBox      pAb;
   int                 width, height;
   int                 topY, h;
-  int                 col;
+  int                 col, l;
 
   pFrame = &ViewFrameTable[frame - 1];
+  if (leftX > rightX)
+    {
+      /* echange limits */
+      l = leftX;
+      leftX = rightX;
+      rightX = l;
+    }
   if (pBox->BxAbstractBox != NULL)
     {
       /* For holophrasted documents there is no enclosing */
@@ -543,20 +550,23 @@ void         DisplayStringSelection (int frame, int leftX, int rightX, PtrBox pB
       /* and the scrolling zone */
       width = FrameTable[frame].FrScrollOrg + FrameTable[frame].FrScrollWidth
 	      - pFrame->FrXOrg;
-
-      topY = pBox->BxYOrg - pFrame->FrYOrg;
-      h = pBox->BxHeight;
+      /* don't take into account margins and borders */
+      topY = pBox->BxYOrg + pBox->BxTMargin + pBox->BxTBorder +
+	     pBox->BxTPadding - pFrame->FrYOrg;
+      h = pBox->BxH;
       if (topY > height)
 	h = 0;
       else if (topY + h > height)
 	h = height - topY;
       
-      leftX = leftX + pBox->BxXOrg - pFrame->FrXOrg;
+      /* don't take into account margins and borders */
+      l = pBox->BxXOrg + pBox->BxLMargin + pBox->BxLBorder + pBox->BxLPadding;
+      leftX = leftX + l - pFrame->FrXOrg;
       if (leftX > width)
 	width = 0;
       else
 	{
-	  rightX = rightX + pBox->BxXOrg - pFrame->FrXOrg;
+	  rightX = rightX + l - pFrame->FrXOrg;
 	  if (rightX > width)
 	    width -= leftX;
 	  else
@@ -569,3 +579,6 @@ void         DisplayStringSelection (int frame, int leftX, int rightX, PtrBox pB
       DrawRectangle (frame, 0, 0, leftX, topY, width, h, 0, col, 2);
     }
 }
+
+
+
