@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996-2001
+ *  (c) COPYRIGHT INRIA, 1996-2002
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -42,10 +42,14 @@
 #include "displayselect_f.h"
 #include "exceptions_f.h"
 #include "font_f.h"
-#include "frame_f.h"
+#include "frame_f.h" 
 #include "picture_f.h"
 #include "xwindowdisplay_f.h"
 
+#ifdef _GL
+#include "glwindowdisplay.h"
+#endif /*_GL*/
+
 /*----------------------------------------------------------------------
    GetXYOrg : do a coordinate shift related to current frame.
   ----------------------------------------------------------------------*/
@@ -63,15 +67,40 @@ void GetXYOrg (int frame, int *XOrg, int *YOrg)
   ----------------------------------------------------------------------*/
 void DefClip (int frame, int xd, int yd, int xf, int yf)
 {
-   int                 width, height;
    ViewFrame          *pFrame;
+   int                 width, height;
+   int                xb, xe, yb, ye;
+
+   GetSizesFrame (frame, &width, &height);
 
    if ((xd == xf && xd == 0 && (yd != yf || yd != 0)) ||
        (yd == yf && yd == 0 && (xd != xf || xd != 0)))
      return;
+   else
+     {
+       /* exchange limits for rigth-to-left string */
+       if (xf < xd)
+	 {
+	   xe = xd;
+	   xd = xf;
+	   xf = xe;
+	 }
+       if (yf < yd)
+	 {
+	   ye = yd;
+	   yd = yf;
+	   yf = ye;
+	 }
+     }
    if (frame > 0 && frame <= MAX_FRAME)
      {
 	pFrame = &ViewFrameTable[frame - 1];
+	
+	xb = pFrame->FrClipXBegin - pFrame->FrXOrg;
+	xe = pFrame->FrClipXEnd - pFrame->FrXOrg - xb; 
+	yb = pFrame->FrClipYBegin - pFrame->FrYOrg;
+	ye = pFrame->FrClipYEnd - pFrame->FrYOrg - yb;	
+	
 	/* Should we take the whole width of the frame ? */
 	if (xd == xf && xd == -1)
 	  {
@@ -221,7 +250,7 @@ static void DrawFilledBox (PtrAbstractBox pAb, int frame, int xmin,
   int                 xd, yd;
   int                 width, height;
   int                 w, h;
- 
+
   pBox = pAb->AbBox;
   if (pBox->BxType == BoGhost)
     return;
@@ -351,8 +380,8 @@ static void DrawFilledBox (PtrAbstractBox pAb, int frame, int xmin,
 		  else if (!pAb->AbTruncatedHead)
 		    /* the clipping will work automatically */
 		    DrawPicture (pBox, imageDesc, frame,
-				 pBox->BxXOrg + pBox->BxLMargin + pBox->BxLBorder + pBox->BxLPadding - x,
-				 pBox->BxYOrg + pBox->BxTMargin + pBox->BxTBorder + pBox->BxTPadding + FrameTable[frame].FrTopMargin - y,
+				 pBox->BxXOrg - x,
+				 pBox->BxYOrg + FrameTable[frame].FrTopMargin - y,
 				 pBox->BxW, pBox->BxH);
 		}
 	    }
@@ -382,7 +411,7 @@ PtrBox DisplayAllBoxes (int frame, int xmin, int xmax, int ymin, int ymax,
   int                 winTop, winBottom;
   int                 bt, bb;
   int                 l, h;
-  ThotBool            userSpec;
+  ThotBool            userSpec = FALSE;
 
   pFrame = &ViewFrameTable[frame - 1];
   pAb = pFrame->FrAbstractBox;
@@ -419,11 +448,34 @@ PtrBox DisplayAllBoxes (int frame, int xmin, int xmax, int ymin, int ymax,
 		  if (pAb->AbVisibility >= pFrame->FrVisibility &&
 		      (pBox->BxDisplay || pAb->AbSelected))
 		    DrawFilledBox (pAb, frame, xmin, xmax, ymin, ymax);
+		  if (pBox->BxNew && pAb->AbFirstEnclosed == NULL)
+		    {
+		      /* this is a new box */
+		      pBox->BxNew = 0;
+		      specAb = pAb;
+		      while (!userSpec && specAb)
+			{
+			  if (specAb->AbWidth.DimIsPosition ||
+			      specAb->AbHeight.DimIsPosition)
+			    specAb = NULL;
+			  else if (specAb->AbHorizPos.PosUserSpecified ||
+				   specAb->AbVertPos.PosUserSpecified ||
+				   specAb->AbWidth.DimUserSpecified ||
+				   specAb->AbHeight.DimUserSpecified)
+			    {
+			      /* one paramater is given by the user */
+			      AddBoxToCreate (create, specAb->AbBox, frame);
+			      userSpec = TRUE;
+			    }
+			  else
+			    specAb = specAb->AbEnclosing;
+			}
+		    }
 		}
 	      else
 		{
 		  /* look for the box displayed at the top of the window */
-		  if (pBox->BxType == BoSplit)
+		  if (pBox->BxType == BoSplit || pBox->BxType == BoMulScript)
 		    {
 		      /* the box itself doen't give right positions */
 		      box = pBox->BxNexChild;
@@ -448,7 +500,7 @@ PtrBox DisplayAllBoxes (int frame, int xmin, int xmax, int ymin, int ymax,
 		    *bVol = *bVol + pBox->BxAbstractBox->AbVolume;
 		  else
 		    {
-		      if (pBox->BxType == BoSplit)
+		      if (pBox->BxType == BoSplit || pBox->BxType == BoMulScript)
 			/* the box itself doen't give right positions */
 			box = pBox->BxNexChild;
 		      else
@@ -484,7 +536,7 @@ PtrBox DisplayAllBoxes (int frame, int xmin, int xmax, int ymin, int ymax,
 			}
 		      if (!userSpec)
 			{
-			  if (pBox->BxType == BoSplit)
+			  if (pBox->BxType == BoSplit || pBox->BxType == BoMulScript)
 			    while (pBox->BxNexChild)
 			      {
 				pBox = pBox->BxNexChild;
@@ -557,7 +609,7 @@ PtrBox DisplayAllBoxes (int frame, int xmin, int xmax, int ymin, int ymax,
   ----------------------------------------------------------------------*/
 ThotBool            RedrawFrameTop (int frame, int scroll)
 {
-  PtrBox              topBox;
+  PtrBox              topBox = NULL;
   PtrBox              create;
   PtrBox              pRootBox;
   ViewFrame          *pFrame;
@@ -568,7 +620,7 @@ ThotBool            RedrawFrameTop (int frame, int scroll)
   int                 ymin;
   int                 ymax;
   int                 delta;
-  ThotBool            toadd;
+  ThotBool            toadd;  
 
   /* are new abstract boxes needed */
   toadd = FALSE;
@@ -593,112 +645,121 @@ ThotBool            RedrawFrameTop (int frame, int scroll)
       create = NULL;
       tVol = bVol = 0;
 #if defined(_WINDOWS) && !defined(_WIN_PRINT)
-  WIN_GetDeviceContext (frame);
+      WIN_GetDeviceContext (frame);
 #endif /* __WINDOWS && !_WINT_PRINT */
-      DefineClipping (frame, pFrame->FrXOrg, pFrame->FrYOrg,
-		      &xmin, &ymin, &xmax, &ymax, 1);
-
-      /* Is there a need to redisplay part of the frame ? */
-      if (xmin < xmax && ymin < ymax)
-	topBox = DisplayAllBoxes (frame, xmin, xmax, ymin, ymax, &create, &tVol, &bVol);
-	  
-      /* The updated area is redrawn */
-      DefClip (frame, 0, 0, 0, 0);
-      RemoveClipping (frame);
-#if defined(_WINDOWS) && !defined(_WIN_PRINT)
-  WIN_ReleaseDeviceContext ();
-#endif /*  _WINDOWS && !WIN_PRINT */
-
-      /* if needed complete the partial existing image */
-      pRootBox = pFrame->FrAbstractBox->AbBox;
-      if (!FrameUpdating && !TextInserting)
+#ifdef _GL      
+      if (GL_prepare((ThotWidget *) FrameTable[frame].WdFrame))
 	{
-	  /* The concrete image is being updated */
-	  FrameUpdating = TRUE;
-	  y = top - pRootBox->BxYOrg;
-	  if (pFrame->FrAbstractBox->AbTruncatedHead && y < 0)
-	    {
-	      /* it lacks some abstract image at the top of the frame */
-	      if (bVol > 0 && bVol < pFrame->FrAbstractBox->AbVolume)
-		{
-		  /* free abstract boxes at the bottom */
-		  DecreaseVolume (FALSE, bVol, frame);
-		  DefClip (frame, 0, 0, 0, 0);
-		}
-	      if (pFrame->FrAbstractBox == NULL)
-		{
-		  printf ("ERR: No more abstract boxes in %d\n", frame);
-		  bVol = -pFrame->FrVolume;
-		}
-	      else
-		bVol = pFrame->FrVolume - pFrame->FrAbstractBox->AbVolume;
-	      
-	      /* Volume to add */
-	      top = (h / 2 - y) * l;
-	      delta = 0;
-	      /* Volume of the area to recompute */
-	      if (topBox)
-		{
-		  /* register previous location */
-		  y = topBox->BxYOrg;
-		  delta = y + topBox->BxHeight;
-		}
-	      /* Adding abstract boxes at the beginning */
-	      IncreaseVolume (TRUE, GetCharsCapacity (top), frame);
-	      toadd = TRUE;
-	      /* Recompute the loaction of the frame in the abstract image */
-	      if (topBox)
-		{
-		  y = -y + topBox->BxYOrg;
-		  /* y equal the shift of previous first box */
-		  /* What's already displayed is related to this */
-		  /* previous first box location */
-		  pFrame->FrYOrg += y;
-		  /* delta equal the limit of redrawing after shifting */
-		  if (y > 0)
-		    delta = topBox->BxYOrg + topBox->BxHeight;
-		  /* new limit */
-		  pFrame->FrClipYEnd = delta;
-		}
-	      else
-		/* No previous box. The frame is drawn */
-		/* on top of the concrete image */
-		pFrame->FrYOrg = 0;
-	      /* Image should be complete */
-	      FrameUpdating = FALSE;
-	      RedrawFrameTop (frame, 0);
-	    }
-
-	  y = bottom - pRootBox->BxYOrg - pRootBox->BxHeight;
-	  if (pFrame->FrAbstractBox->AbTruncatedTail &&
-	      (y >= 0/* || (!pFrame->FrAbstractBox->AbHeight.DimIsPosition &&
-			 pFrame->FrAbstractBox->AbHeight.DimMinimum &&
-			 !pRootBox->BxContentHeight)*/))
-	    {
-	      /* it lacks some abstract image at the bottom of the frame */
-	      /* cleanup the bottom of the frame */
-	      Clear (frame, l, y, 0, pRootBox->BxYOrg + pRootBox->BxHeight);
-	      /* don't loop is volume didn't change */
-	      tVol = pFrame->FrAbstractBox->AbVolume;
-	      /* Volume to add */
-	      bottom = (y + h / 2) * l;
-	      IncreaseVolume (FALSE, GetCharsCapacity (bottom), frame);
-	      tVol -= pFrame->FrAbstractBox->AbVolume;
-	      /* Image should be completed */
-	      FrameUpdating = FALSE;
-	      if (tVol == 0)
-		printf ("ERR: Nothing to add\n");
-	      else
-		/* Maybe image is not complete yet */
-		RedrawFrameBottom (frame, 0, NULL);
-	    }
-	  else
-	    /* Volume computed is sufficient */
-	    pFrame->FrVolume = pFrame->FrAbstractBox->AbVolume;
+#endif /* _GL */
+	  DefineClipping (frame, pFrame->FrXOrg, pFrame->FrYOrg,
+			  &xmin, &ymin, &xmax, &ymax, 1);
 	  
-	  /* update of image is finished */
-	  FrameUpdating = FALSE;
+	  /* Is there a need to redisplay part of the frame ? */
+	  if (xmin < xmax && ymin < ymax)
+	    topBox = DisplayAllBoxes (frame, xmin, xmax, ymin, ymax,
+				      &create, &tVol, &bVol);
+	  
+	  /* The updated area is redrawn */
+	  DefClip (frame, 0, 0, 0, 0);
+	  RemoveClipping (frame);
+#if defined(_WINDOWS) && !defined(_WIN_PRINT)
+	  WIN_ReleaseDeviceContext ();
+#endif /*  _WINDOWS && !WIN_PRINT */
+	  if (!ShowOnePage)
+	    {
+	      /* if needed complete the partial existing image */
+	      pRootBox = pFrame->FrAbstractBox->AbBox;
+	      if (!FrameUpdating && !TextInserting)
+		{
+		  /* The concrete image is being updated */
+		  FrameUpdating = TRUE;
+		  y = top - pRootBox->BxYOrg;
+		  if (pFrame->FrAbstractBox->AbTruncatedHead && y < 0)
+		    {
+		      /* it lacks some abstract image at the top of the frame */
+		      if (bVol > 0 && bVol < pFrame->FrAbstractBox->AbVolume)
+			{
+			  /* free abstract boxes at the bottom */
+			  DecreaseVolume (FALSE, bVol, frame);
+			  DefClip (frame, 0, 0, 0, 0);
+			}
+		      if (pFrame->FrAbstractBox == NULL)
+			{
+			  printf ("ERR: No more abstract boxes in %d\n", frame);
+			  bVol = -pFrame->FrVolume;
+			}
+		      else
+			bVol = pFrame->FrVolume - pFrame->FrAbstractBox->AbVolume;
+		      
+		      /* Volume to add */
+		      top = (h / 2 - y) * l;
+		      delta = 0;
+		      /* Volume of the area to recompute */
+		      if (topBox)
+			{
+			  /* register previous location */
+			  y = topBox->BxYOrg;
+			  delta = y + topBox->BxHeight;
+			}
+		      /* Adding abstract boxes at the beginning */
+		      IncreaseVolume (TRUE, GetCharsCapacity (top), frame);
+		      toadd = TRUE;
+		      /* Recompute the loaction of the frame in the abstract image */
+		      if (topBox)
+			{
+			  y = -y + topBox->BxYOrg;
+			  /* y equal the shift of previous first box */
+			  /* What's already displayed is related to this */
+			  /* previous first box location */
+			  pFrame->FrYOrg += y;
+			  /* delta equal the limit of redrawing after shifting */
+			  if (y > 0)
+			    delta = topBox->BxYOrg + topBox->BxHeight;
+			  /* new limit */
+			  pFrame->FrClipYEnd = delta;
+			}
+		      else
+			/* No previous box. The frame is drawn */
+			/* on top of the concrete image */
+			pFrame->FrYOrg = 0;
+		      /* Image should be complete */
+		      FrameUpdating = FALSE;
+		      RedrawFrameTop (frame, 0);
+		    }
+		  y = bottom - pRootBox->BxYOrg - pRootBox->BxHeight;
+		  if (pFrame->FrAbstractBox->AbTruncatedTail &&
+		      (y >= 0/* || (!pFrame->FrAbstractBox->AbHeight.DimIsPosition &&
+				pFrame->FrAbstractBox->AbHeight.DimMinimum &&
+				!pRootBox->BxContentHeight)*/))
+		    {
+		      /* it lacks some abstract image at the bottom of the frame */
+		      /* cleanup the bottom of the frame */
+		      Clear (frame, l, y, 0, pRootBox->BxYOrg + pRootBox->BxHeight);
+		      /* don't loop is volume didn't change */
+		      tVol = pFrame->FrAbstractBox->AbVolume;
+		      /* Volume to add */
+		      bottom = (y + h / 2) * l;
+		      IncreaseVolume (FALSE, GetCharsCapacity (bottom), frame);
+		      tVol -= pFrame->FrAbstractBox->AbVolume;
+		      /* Image should be completed */
+		      FrameUpdating = FALSE;
+		      if (tVol == 0)
+			printf ("ERR: Nothing to add\n");
+		      else
+			/* Maybe image is not complete yet */
+			RedrawFrameBottom (frame, 0, NULL);
+		    }
+		  else
+		    /* Volume computed is sufficient */
+		    pFrame->FrVolume = pFrame->FrAbstractBox->AbVolume;
+		}
+	      /* update of image is finished */
+	      FrameUpdating = FALSE;
+	    }
+#ifdef _GL
+	  GL_realize ();
 	}
+#endif /* _GL */ 
     }
   else
     /* The modified area is not visible */
@@ -720,7 +781,7 @@ ThotBool            RedrawFrameTop (int frame, int scroll)
    Return non zero if new abstract boxes were added in order
    to build the corresponding abstract image.
   ----------------------------------------------------------------------*/
-ThotBool     RedrawFrameBottom (int frame, int scroll, PtrAbstractBox subtree)
+ThotBool RedrawFrameBottom (int frame, int scroll, PtrAbstractBox subtree)
 {
   PtrBox              topBox;
   PtrBox              pRootBox;
@@ -734,7 +795,6 @@ ThotBool     RedrawFrameBottom (int frame, int scroll, PtrAbstractBox subtree)
   int                 xmax;
   int                 ymin;
   int                 ymax;
-  ThotBool            stop;
   ThotBool            toadd;
 
   /* are new abstract boxes needed */
@@ -745,6 +805,8 @@ ThotBool     RedrawFrameBottom (int frame, int scroll, PtrAbstractBox subtree)
   xmax = pFrame->FrClipXEnd;
   ymin = pFrame->FrClipYBegin;
   ymax = pFrame->FrClipYEnd;
+  if (pFrame->FrAbstractBox && pFrame->FrAbstractBox->AbElement == NULL)
+    pFrame->FrAbstractBox = NULL;
   if (!pFrame->FrReady || pFrame->FrAbstractBox == NULL)
     return toadd;
   else if (xmin < xmax &&
@@ -759,148 +821,146 @@ ThotBool     RedrawFrameBottom (int frame, int scroll, PtrAbstractBox subtree)
       bottom = top + h;
       tVol = bVol = 0;
       delta = top - h / 2;
-      topBox = NULL;
+      pRootBox = topBox = NULL;
+      y = 0;
       /* used to store boxes created on the fly */
       create = NULL;
       /* Redraw from top to bottom all filled boxes */
 #if defined(_WINDOWS) && !defined(_WIN_PRINT)
-  WIN_GetDeviceContext (frame);
+      WIN_GetDeviceContext (frame);
 #endif /* __WINDOWS && !_WINT_PRINT */
-      DefineClipping (frame, pFrame->FrXOrg, pFrame->FrYOrg,
-		      &xmin, &ymin, &xmax, &ymax, 1);
       /* Is there a need to redisplay part of the frame ? */
       if (xmin < xmax && ymin < ymax)
-	{
-	  topBox = DisplayAllBoxes (frame, xmin, xmax, ymin, ymax, &create, &tVol, &bVol);
-	  
+	{ 
+#ifdef _GL
+	  if (GL_prepare((ThotWidget *) FrameTable[frame].WdFrame))
+	    {
+#endif /*_GL*/
+	      
+	      DefineClipping (frame, pFrame->FrXOrg, pFrame->FrYOrg,
+			      &xmin, &ymin, &xmax, &ymax, 1);
+
+	      topBox = DisplayAllBoxes (frame, xmin, xmax, ymin,
+					ymax, &create, &tVol, &bVol);
+	      /* The updated area is redrawn */
+	      DefClip (frame, 0, 0, 0, 0);
+	      RemoveClipping (frame);
+#if defined(_WINDOWS) && !defined(_WIN_PRINT)
+	      WIN_ReleaseDeviceContext ();
+#endif /* __WINDOWS && !_WINT_PRINT */
+	      if (!ShowOnePage)
+		{
+		  /* if needed complete the partial existing image */
+		  pRootBox = pFrame->FrAbstractBox->AbBox;
+		  if (!FrameUpdating && (!TextInserting || scroll > 0))
+		    {
+		      /* The concrete image is being updated */
+		      FrameUpdating = TRUE;
+		      y = top - pRootBox->BxYOrg;
+		      if (pFrame->FrAbstractBox->AbInLine)
+			FrameUpdating = FALSE;
+		      else if (pFrame->FrAbstractBox->AbTruncatedHead && y < 0)
+			{
+			  /* it lacks a piece of the concrete image at the frame top */
+			  /* filling on top will shift the whole concrete image */
+			  top = h / 2 - y;
+			  /* Height to add */
+			  top = top * l;
+			  /* Volume of the area to recompute */
+			  if (topBox != NULL)
+			    {
+			      /* register previous location */
+			      y = topBox->BxYOrg;
+			      delta = y + topBox->BxHeight;
+			    }
+			  /* Adding abstract boxes at the beginning */
+			  IncreaseVolume (TRUE, GetCharsCapacity (top), frame);
+			  toadd = TRUE;
+			  /* Recompute the location of the frame in the abstract image */
+			  if (topBox != NULL)
+			    {
+			      y = -y + topBox->BxYOrg;
+			      /* y equal the shift of previous first box */
+			      /* What's already displayed is related to this */
+			      /* previous first box location */
+			      pFrame->FrYOrg += y;
+			      /* delta equal the limit of redrawing after shifting */
+			      if (y > 0)
+				delta = topBox->BxYOrg + topBox->BxHeight;
+			      /* new limit */
+			      pFrame->FrClipYEnd = delta;
+			    }
+			  RedrawFrameTop (frame, 0);
+			}
+		      y = bottom - pRootBox->BxYOrg - pRootBox->BxHeight;
+		      if (pFrame->FrAbstractBox->AbTruncatedTail &&
+			  (y >= 0/* || (!pFrame->FrAbstractBox->AbHeight.DimIsPosition &&
+				    pFrame->FrAbstractBox->AbHeight.DimMinimum &&
+				    !pRootBox->BxContentHeight)*/))
+			{
+			  /* it lacks some abstract image at the bottom of the frame */
+			  if (tVol > 0 && tVol < pFrame->FrAbstractBox->AbVolume)
+			    {
+			      /* free abstract boxes on top of the frame */
+			      if (topBox)
+				y = topBox->BxYOrg;
+			      DecreaseVolume (TRUE, tVol, frame);
+			      DefClip (frame, 0, 0, 0, 0);
+			      /* check location of frame in concrete image */
+			      if (topBox)
+				pFrame->FrYOrg = pFrame->FrYOrg - y + topBox->BxYOrg;
+			    }
+			  if (pFrame->FrAbstractBox == NULL)
+			    {
+			      printf ("ERR: No more abstract box in %d\n", frame);
+			      tVol = -pFrame->FrVolume;
+			    }
+			  else
+			    tVol = pFrame->FrVolume - pFrame->FrAbstractBox->AbVolume;
+			  
+			  /* cleanup the bottom of the frame */
+			  Clear (frame, l, y, 0, pRootBox->BxYOrg + pRootBox->BxHeight);
+			  /* don't loop is volume didn't change */
+			  tVol = pFrame->FrAbstractBox->AbVolume;
+			  /* Volume to add */
+			  bottom = (y + h / 2) * l;
+			  IncreaseVolume (FALSE, GetCharsCapacity (bottom), frame);
+			  tVol -= pFrame->FrAbstractBox->AbVolume;
+			  /* Image should be completed */
+			  FrameUpdating = FALSE;
+			  if (tVol != 0)
+			    /* Maybe the image is not complete yet */
+			    RedrawFrameBottom (frame, 0, NULL);
+			}
+		      else
+			{
+			  /* Volume computed is sufficient */
+			  /* Is a cleanup of the bottom of frame needed ? */
+			  if (y > 0)
+			    Clear (frame, l, y, 0, pRootBox->BxYOrg + pRootBox->BxHeight);
+			  pFrame->FrVolume = pFrame->FrAbstractBox->AbVolume;
+			}
+		    }
+		  /* update of image is finished */
+		  FrameUpdating = FALSE;
+		}
+#ifdef _GL 
+	      GL_realize ();
+	    }
+#endif /* _GL */
 	  /* Interactive creation of boxes */
 	  if (create)
 	    {
 	      pAb = create->BxAbstractBox;
 	      DirectCreation (create, frame);
-	      /* Should son's boxes being handled too ? */
-	      while (pAb && pAb->AbFirstEnclosed)
-		{
-		  pAb = pAb->AbFirstEnclosed;
-		  stop = FALSE;
-		  while (!stop && pAb)
-		    {
-		      if (pAb->AbHorizPos.PosUserSpecified ||
-			  pAb->AbVertPos.PosUserSpecified ||
-			  pAb->AbWidth.DimUserSpecified ||
-			  pAb->AbHeight.DimUserSpecified)
-			DirectCreation (pAb->AbBox, frame);
-		      if (pAb->AbNext)
-			pAb = pAb->AbNext;
-		      else
-			stop = TRUE;
-		    }
-		}
-	      return (FALSE);
-	    }
-	}
-      /* The updated area is redrawn */
-      DefClip (frame, 0, 0, 0, 0);
-      RemoveClipping (frame);
-#if defined(_WINDOWS) && !defined(_WIN_PRINT)
-  WIN_ReleaseDeviceContext ();
-#endif /* __WINDOWS && !_WINT_PRINT */
-
-      /* if needed complete the partial existing image */
-      pRootBox = pFrame->FrAbstractBox->AbBox;
-      if (!FrameUpdating && (!TextInserting || scroll > 0))
-	{
-	  /* The concrete image is being updated */
-	  FrameUpdating = TRUE;
-	  y = top - pRootBox->BxYOrg;
-	  if (pFrame->FrAbstractBox->AbInLine)
-	    FrameUpdating = FALSE;
-	  else if (pFrame->FrAbstractBox->AbTruncatedHead && y < 0)
-	    {
-	      /* it lacks a piece of the concrete image at the frame top */
-	      /* filling on top will shift the whole concrete image */
-	      top = h / 2 - y;
-	      /* Height to add */
-	      top = top * l;
-	      /* Volume of the area to recompute */
-	      if (topBox != NULL)
-		{
-		  /* register previous location */
-		  y = topBox->BxYOrg;
-		  delta = y + topBox->BxHeight;
-		}
-	      /* Adding abstract boxes at the beginning */
-	      IncreaseVolume (TRUE, GetCharsCapacity (top), frame);
-	      toadd = TRUE;
-	      /* Recompute the location of the frame in the abstract image */
-	      if (topBox != NULL)
-		{
-		  y = -y + topBox->BxYOrg;
-		  /* y equal the shift of previous first box */
-		  /* What's already displayed is related to this */
-		  /* previous first box location */
-		  pFrame->FrYOrg += y;
-		  /* delta equal the limit of redrawing after shifting */
-		  if (y > 0)
-		    delta = topBox->BxYOrg + topBox->BxHeight;
-		  /* new limit */
-		  pFrame->FrClipYEnd = delta;
-		}
-	      RedrawFrameTop (frame, 0);
-	    }
-	  y = bottom - pRootBox->BxYOrg - pRootBox->BxHeight;
-	  if (pFrame->FrAbstractBox->AbTruncatedTail &&
-		   (y >= 0/* || (!pFrame->FrAbstractBox->AbHeight.DimIsPosition &&
-			      pFrame->FrAbstractBox->AbHeight.DimMinimum &&
-			      !pRootBox->BxContentHeight)*/))
-	    {
-	      /* it lacks some abstract image at the bottom of the frame */
-	      if (tVol > 0 && tVol < pFrame->FrAbstractBox->AbVolume)
-		{
-		  /* free abstract boxes on top of the frame */
-		  if (topBox)
-		    y = topBox->BxYOrg;
-		  DecreaseVolume (TRUE, tVol, frame);
-		  DefClip (frame, 0, 0, 0, 0);
-		  /* check location of frame in concrete image */
-		  if (topBox)
-		    pFrame->FrYOrg = pFrame->FrYOrg - y + topBox->BxYOrg;
-		}
-	      if (pFrame->FrAbstractBox == NULL)
-		{
-		  printf ("ERR: No more abstract box in %d\n", frame);
-		  tVol = -pFrame->FrVolume;
-		}
-	      else
-		tVol = pFrame->FrVolume - pFrame->FrAbstractBox->AbVolume;
-	      
-	      /* cleanup the bottom of the frame */
-	      Clear (frame, l, y, 0, pRootBox->BxYOrg + pRootBox->BxHeight);
-	      /* don't loop is volume didn't change */
-	      tVol = pFrame->FrAbstractBox->AbVolume;
-	      /* Volume to add */
-	      bottom = (y + h / 2) * l;
-	      IncreaseVolume (FALSE, GetCharsCapacity (bottom), frame);
-	      tVol -= pFrame->FrAbstractBox->AbVolume;
-	      /* Image should be completed */
-	      FrameUpdating = FALSE;
-	      if (tVol == 0)
-		printf ("ERR: Nothing to add\n");
-	      else
-		/* Maybe the image is not complete yet */
-		RedrawFrameBottom (frame, 0, NULL);
-	    }
-	  else
-	    {
 	      /* Volume computed is sufficient */
 	      /* Is a cleanup of the bottom of frame needed ? */
 	      if (y > 0)
 		Clear (frame, l, y, 0, pRootBox->BxYOrg + pRootBox->BxHeight);
 	      pFrame->FrVolume = pFrame->FrAbstractBox->AbVolume;
+	      /* update of image is finished */
+	      FrameUpdating = FALSE;
 	    }
-
-	  /* update of image is finished */
-	  FrameUpdating = FALSE;
 	}
     }
   else
@@ -931,14 +991,13 @@ void                DisplayFrame (int frame)
   int                 w, h;
 
   if (documentDisplayMode[frame - 1] == DisplayImmediately)
-    {
+    {     
       /* Check that the frame exists */
       pFrame = &ViewFrameTable[frame - 1];
       if (pFrame->FrAbstractBox != NULL)
 	{
 	  /* Drawing of the updated area */
-	  RedrawFrameBottom (frame, 0, NULL);
-	  
+	  RedrawFrameBottom (frame, 0, NULL);	  
 	  /* recompute scrolls */
 	  CheckScrollingWidth (frame);
 	  UpdateScrollbars (frame);
@@ -948,6 +1007,6 @@ void                DisplayFrame (int frame)
 	  /* clean the frame */
 	  GetSizesFrame (frame, &w, &h);
 	  Clear (frame, w, h, 0, 0);
-	}
+	} 
     }
 }
